@@ -254,6 +254,8 @@ static ULONG WINAPI filter_Release(IMediaStreamFilter *iface)
         free(filter->streams);
         if (filter->clock)
             IReferenceClock_Release(filter->clock);
+        if (filter->cs.DebugInfo)
+            filter->cs.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&filter->cs);
         free(filter);
     }
@@ -1100,7 +1102,7 @@ HRESULT filter_create(IUnknown *outer, void **out)
     object->refcount = 1;
     list_init(&object->free_events);
     list_init(&object->used_events);
-    InitializeCriticalSection(&object->cs);
+    InitializeCriticalSectionEx(&object->cs, 0, RTL_CRITICAL_SECTION_FLAG_FORCE_DEBUG_INFO);
     object->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": MediaStreamFilter.cs");
 
     TRACE("Created media stream filter %p.\n", object);
