@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <windows.h>
 #include <commctrl.h>
+#include <uxtheme.h>
 
 #include "wine/test.h"
 #include "v6util.h"
@@ -829,6 +830,7 @@ static void test_themed_background(void)
     HMODULE uxtheme;
     COLORREF color;
     WNDCLASSA cls;
+    HRESULT hr;
     HDC hdc;
     int i;
 
@@ -838,6 +840,7 @@ static void test_themed_background(void)
         DWORD style;
         const struct message *seq;
         BOOL todo;
+        const WCHAR *window_theme;
     }
     tests[] =
     {
@@ -886,6 +889,8 @@ static void test_themed_background(void)
         {TOOLTIPS_CLASSA, 0, empty_seq},
         {TRACKBAR_CLASSA, 0, wm_ctlcolorstatic_seq},
         {WC_TREEVIEWA, 0, treeview_seq},
+        {WC_TREEVIEWA, TVS_HASBUTTONS, treeview_seq},
+        {WC_TREEVIEWA, TVS_HASBUTTONS, treeview_seq, FALSE, L"explorer"},
         {UPDOWN_CLASSA, 0, empty_seq},
         {WC_SCROLLBARA, 0, scrollbar_seq},
         {WC_SCROLLBARA, SBS_SIZEBOX, empty_seq},
@@ -933,12 +938,29 @@ static void test_themed_background(void)
         child = CreateWindowA(tests[i].class_name, "    ", WS_CHILD | WS_VISIBLE | tests[i].style,
                               0, 0, 50, 100, parent, 0, 0, 0);
         ok(child != NULL, "CreateWindowA failed, error %lu.\n", GetLastError());
+
+        /* Extra preparations */
+        if (tests[i].window_theme)
+        {
+            hr = SetWindowTheme(child, tests[i].window_theme, NULL);
+            ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+        }
+
         flush_events();
         flush_sequences(sequences, NUM_MSG_SEQUENCES);
 
         RedrawWindow(child, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_UPDATENOW | RDW_ERASENOW | RDW_FRAME);
         ok_sequence(sequences, PARENT_SEQ_INDEX, tests[i].seq, "paint background", tests[i].todo);
 
+        /* Extra background tests */
+        if (!lstrcmpA(tests[i].class_name, WC_TREEVIEWA))
+        {
+            hdc = GetDC(child);
+            color = GetPixel(hdc, 40, 40);
+            ok(color == 0xffffff, "Expected color %#x, got %#lx.\n", 0xffffff, color);
+            ReleaseDC(child, hdc);
+        }
+
         /* For message sequences that contain both DrawThemeParentBackground() messages and
          * WM_CTLCOLOR*, do a color test to check which is really in effect for controls that can be
          * tested automatically. For WM_ERASEBKGND from DrawThemeParentBackground(), a red brush is
@@ -1160,6 +1182,95 @@ static void test_WM_STYLECHANGED(void)
     DestroyWindow(parent);
 }
 
+static void test_WM_SETFONT(void)
+{
+    HFONT hfont, hfont2;
+    HWND parent, hwnd;
+    LOGFONTA lf;
+    int i, ret;
+
+    static const struct
+    {
+        const CHAR *class_name;
+        BOOL use_system_font;
+    }
+    tests[] =
+    {
+        {ANIMATE_CLASSA, TRUE},
+        {WC_BUTTONA},
+        {WC_COMBOBOXA},
+        {WC_COMBOBOXEXA},
+        {DATETIMEPICK_CLASSA},
+        {WC_EDITA},
+        {WC_HEADERA},
+        {HOTKEY_CLASSA},
+        {WC_IPADDRESSA, TRUE},
+        {WC_LISTBOXA},
+        {WC_LISTVIEWA},
+        {MONTHCAL_CLASSA},
+        {WC_NATIVEFONTCTLA, TRUE},
+        {WC_PAGESCROLLERA, TRUE},
+        {PROGRESS_CLASSA},
+        {REBARCLASSNAMEA},
+        {WC_STATICA},
+        {STATUSCLASSNAMEA},
+        {"SysLink"},
+        {WC_TABCONTROLA},
+        {TOOLBARCLASSNAMEA},
+        {TOOLTIPS_CLASSA},
+        {TRACKBAR_CLASSA, TRUE},
+        {WC_TREEVIEWA},
+        {UPDOWN_CLASSA, TRUE},
+        {WC_SCROLLBARA, TRUE},
+    };
+
+    parent = CreateWindowA(WC_STATICA, "parent", WS_POPUP | WS_VISIBLE, 100, 100, 100, 100,
+                           0, 0, 0, 0);
+    ok(parent != NULL, "CreateWindowA failed, error %lu.\n", GetLastError());
+
+    for (i = 0; i < ARRAY_SIZE(tests); ++i)
+    {
+        winetest_push_context("%s", tests[i].class_name);
+
+        hwnd = CreateWindowA(tests[i].class_name, "test", WS_POPUP | WS_VISIBLE, 0, 0, 50, 50, parent,
+                             0, 0, 0);
+        /* SysLink is unavailable in comctl32 v5 */
+        if (!hwnd && !lstrcmpA(tests[i].class_name, "SysLink"))
+        {
+            winetest_pop_context();
+            continue;
+        }
+
+        ZeroMemory(&lf, sizeof(lf));
+        lf.lfWeight = FW_NORMAL;
+        lf.lfHeight = 20;
+        lstrcpyA(lf.lfFaceName, "Tahoma");
+        hfont = CreateFontIndirectA(&lf);
+        ok(hfont != NULL, "CreateFontIndirectA failed, error %lu.\n", GetLastError());
+
+        SendMessageA(hwnd, WM_SETFONT, (WPARAM)hfont, TRUE);
+        hfont2 = (HFONT)SendMessageA(hwnd, WM_GETFONT, 0, 0);
+        if (tests[i].use_system_font)
+            ok(hfont2 == NULL, "Got unexpected font %p.\n", hfont2);
+        else
+            ok(hfont2 == hfont, "Got unexpected font %p.\n", hfont2);
+        ret = GetObjectA(hfont, sizeof(lf), &lf);
+        ok(ret == sizeof(lf), "GetObjectA failed, error %lu.\n", GetLastError());
+
+        DestroyWindow(hwnd);
+
+        ret = GetObjectA(hfont, sizeof(lf), &lf);
+        if (!lstrcmpA(tests[i].class_name, WC_IPADDRESSA))
+            ok(ret == 0, "GetObjectA succeeded.\n");
+        else
+            ok(ret == sizeof(lf), "GetObjectA failed, error %lu.\n", GetLastError());
+
+        winetest_pop_context();
+    }
+
+    DestroyWindow(parent);
+}
+
 START_TEST(misc)
 {
     ULONG_PTR ctx_cookie;
@@ -1173,6 +1284,7 @@ START_TEST(misc)
     test_Alloc();
     test_comctl32_classes(FALSE);
     test_WM_STYLECHANGED();
+    test_WM_SETFONT();
 
     FreeLibrary(hComctl32);
 
@@ -1188,6 +1300,7 @@ START_TEST(misc)
     test_WM_THEMECHANGED();
     test_WM_SYSCOLORCHANGE();
     test_WM_STYLECHANGED();
+    test_WM_SETFONT();
 
     unload_v6_module(ctx_cookie, hCtx);
     FreeLibrary(hComctl32);
