@@ -302,6 +302,8 @@ static void test_disableowner(void)
     DestroyWindow(parenthwnd);
 }
 
+HANDLE active_page_changed;
+
 static INT_PTR CALLBACK nav_page_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
 {
     switch(msg){
@@ -312,6 +314,8 @@ static INT_PTR CALLBACK nav_page_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM
             case PSN_SETACTIVE:
                 active_page = /* PropSheet_HwndToIndex(hdr->hwndFrom, hwnd); */
                     (int)SendMessageA(hdr->hwndFrom, PSM_HWNDTOINDEX, (WPARAM)hwnd, 0);
+                if (active_page_changed)
+                    SetEvent(active_page_changed);
                 return TRUE;
             case PSN_KILLACTIVE:
                 /* prevent navigation away from the fourth page */
@@ -668,20 +672,20 @@ static const struct message property_sheet_seq[] = {
     { WM_NCCALCSIZE, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
     { DM_REPOSITION, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },*/
     { WM_WINDOWPOSCHANGING, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
-    { WM_WINDOWPOSCHANGING, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_WINDOWPOSCHANGING, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_WINDOWPOSCHANGED, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
-    { WM_ACTIVATEAPP, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_ACTIVATEAPP, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
     /*{ WM_NCACTIVATE, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_GETTEXT, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_GETICON, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_GETICON, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_GETICON, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_GETTEXT, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },*/
-    { WM_ACTIVATE, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_ACTIVATE, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
     /*{ WM_IME_SETCONTEXT, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_IME_NOTIFY, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },*/
-    { WM_SETFOCUS, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
-    { WM_KILLFOCUS, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_SETFOCUS, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_KILLFOCUS, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
     /*{ WM_IME_SETCONTEXT, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },*/
     { WM_PARENTNOTIFY, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_INITDIALOG, sent|id, 0, 0, RECEIVER_PAGE },
@@ -700,6 +704,11 @@ static const struct message property_sheet_seq[] = {
     { WM_GETICON, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },*/
     { WM_SETTEXT, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { WM_SHOWWINDOW, sent|id, 0, 0, RECEIVER_PAGE },
+    { WM_WINDOWPOSCHANGING, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_ACTIVATEAPP, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_ACTIVATEAPP, sent|id|optional, 0, 0, RECEIVER_PAGE },
+    { WM_ACTIVATE, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
+    { WM_ACTIVATE, sent|id|optional, 0, 0, RECEIVER_PAGE },
     /*{ 0x00000401, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },
     { 0x00000400, sent|id, 0, 0, RECEIVER_SHEET_WINPROC },*/
     { WM_CHANGEUISTATE, sent|id|optional, 0, 0, RECEIVER_SHEET_WINPROC },
@@ -1388,6 +1397,340 @@ static void test_invalid_hpropsheetpage(void)
     DestroyWindow(hdlg);
 }
 
+static int query_initial_focus = 0;
+static int edit_test_ID = IDC_PS_EDIT2;
+static LRESULT CALLBACK TestDlgProc(HWND hdlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
+{
+    LPNMHDR lpnmhdr;
+
+    switch (uMessage)
+    {
+    case WM_NOTIFY:
+        lpnmhdr = (NMHDR *)lParam;
+        switch (lpnmhdr->code)
+        {
+        case PSN_QUERYINITIALFOCUS:
+        {
+            query_initial_focus++;
+            if (edit_test_ID == IDC_PS_EDIT1)
+                return 0;
+            SetWindowLongPtrA(hdlg, DWLP_MSGRESULT, (LONG_PTR)GetDlgItem(hdlg, IDC_PS_EDIT2));
+            return 1;
+        }
+        }
+    }
+
+    return FALSE;
+}
+
+static void test_QueryInitialFocus(void)
+{
+    PROPSHEETPAGEA psp[2];
+    PROPSHEETHEADERA psh;
+    HWND tab_ctrl = NULL;
+    HWND hp;
+    RECT rc;
+    LPARAM lp;
+
+    memset(&psh, 0, sizeof(psh));
+    memset(psp, 0, sizeof(psp));
+    psp[0].dwSize = sizeof(PROPSHEETPAGEA);
+    psp[0].dwFlags = PSP_USETITLE;
+    psp[0].hInstance = GetModuleHandleA(NULL);
+    psp[0].pszTemplate = (const char *)MAKEINTRESOURCE(IDD_PROP_PAGE_EDIT);
+    psp[0].pszIcon = NULL;
+    psp[0].pfnDlgProc = (DLGPROC) TestDlgProc;
+    psp[0].pszTitle = "Page1";
+    psp[0].lParam = 0;
+
+    psp[1].dwSize = sizeof(PROPSHEETPAGEA);
+    psp[1].dwFlags = PSP_USETITLE;
+    psp[1].hInstance = GetModuleHandleA(NULL);
+    psp[1].pszTemplate = (const char*)MAKEINTRESOURCE(IDD_PROP_PAGE_EDIT);
+    psp[1].pszIcon = NULL;
+    psp[1].pfnDlgProc = (DLGPROC) TestDlgProc;
+    psp[1].pszTitle = "Page2";
+    psp[1].lParam = 0;
+
+    psh.dwSize = sizeof(PROPSHEETHEADERA);
+    psh.dwFlags = PSH_PROPSHEETPAGE | PSH_MODELESS;
+    psh.hwndParent = GetDesktopWindow();
+    psh.pszCaption = "Modeless Property Sheet";
+    psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGEA);
+    psh.ppsp = (LPCPROPSHEETPAGEA)&psp;
+
+    hp = (HWND)pPropertySheetA(&psh);
+    tab_ctrl = (HWND)SendMessageA(hp, PSM_GETTABCONTROL, 0, 0);
+
+    /* Test PSN_QUERYINITIALFOCUS gets sent on start */
+    ok(query_initial_focus == 1, "Did not recieve PSN_QUERYINITIALFOCUS on start.\n");
+    ok(GetDlgCtrlID(GetFocus()) == IDC_PS_EDIT2, "Focus not set to IDC_PS_EDIT2.\n");
+
+    /* Test changing of tabs */
+    edit_test_ID = IDC_PS_EDIT1;
+    query_initial_focus = 0;
+    ok(tab_ctrl != 0, "Could not test changing tabs. No tab control found.\n");
+    SendMessageA(tab_ctrl, TCM_GETITEMRECT, 1, (LPARAM) &rc);
+    lp = MAKELPARAM(rc.left + ((rc.right - rc.left) / 2), rc.top + ((rc.bottom - rc.top) / 2));
+    SendMessageA(tab_ctrl, WM_LBUTTONDOWN, MK_LBUTTON, lp);
+    ok(query_initial_focus == 1, "Did not recieve PSN_QUERYINITIALFOCUS from changing tabs.\n");
+    ok(GetDlgCtrlID(GetFocus()) == IDC_PS_EDIT1, "Focus not set to IDC_PS_EDIT1.\n");
+
+    /* Test Apply Button */
+    edit_test_ID = IDC_PS_EDIT2;
+    query_initial_focus = 0;
+    SendMessageA(hp, PSM_PRESSBUTTON, PSBTN_APPLYNOW, 0);
+    ok(query_initial_focus == 1, "Did not recieve PSN_QUERYINITIALFOCUS from apply button.\n");
+    ok(GetDlgCtrlID(GetFocus()) == IDC_PS_EDIT2, "Focus not set to IDC_PS_EDIT2.\n");
+
+    DestroyWindow(hp);
+
+    edit_test_ID = IDC_PS_EDIT1;
+    query_initial_focus = 0;
+
+    psh.dwFlags = PSH_PROPSHEETPAGE | PSH_USECALLBACK | PSH_WIZARD | PSH_MODELESS;
+    hp = (HWND)pPropertySheetA(&psh);
+
+    /* Test PSN_QUERYINITIALFOCUS gets sent on start */
+    ok(query_initial_focus == 1, "Did not recieve PSN_QUERYINITIALFOCUS on start.\n");
+    ok(GetDlgCtrlID(GetFocus()) == IDC_PS_EDIT1, "Focus not set to IDC_PS_EDIT1.\n");
+
+    /* Test Next Button */
+    edit_test_ID = IDC_PS_EDIT2;
+    query_initial_focus = 0;
+    SendMessageA(hp, PSM_PRESSBUTTON, PSBTN_NEXT, 0);
+    ok(query_initial_focus == 1, "Did not recieve PSN_QUERYINITIALFOCUS from next button.\n");
+    ok(GetDlgCtrlID(GetFocus()) == IDC_PS_EDIT2, "Focus not set to IDC_PS_EDIT2.\n");
+
+    DestroyWindow(hp);
+}
+
+static void test_init_page_creation(void)
+{
+    PROPSHEETPAGEA psp[3];
+    PROPSHEETHEADERA psh;
+    HWND hp;
+    HWND page;
+
+    memset(&psh, 0, sizeof(psh));
+    memset(psp, 0, sizeof(psp));
+
+    for (int p = 0; p < 3; p++)
+    {
+        psp[p].dwSize = sizeof(PROPSHEETPAGEA);
+        psp[p].dwFlags = PSP_USETITLE | (p != 2 ? PSP_PREMATURE : 0);
+        psp[p].hInstance = GetModuleHandleA(NULL);
+        psp[p].pszTemplate = (const char *)MAKEINTRESOURCE(IDD_PROP_PAGE_EDIT);
+        psp[p].pszIcon = NULL;
+        psp[p].pfnDlgProc = (DLGPROC) page_dlg_proc;
+        psp[p].pszTitle = "page title";
+        psp[p].lParam = 0;
+    }
+
+    psh.dwSize = sizeof(PROPSHEETHEADERA);
+    psh.dwFlags = PSH_PROPSHEETPAGE | PSH_MODELESS;
+    psh.hwndParent = GetDesktopWindow();
+    psh.pszCaption = "test caption";
+    psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGEA);
+    psh.ppsp = (LPCPROPSHEETPAGEA)&psp;
+
+    hp = (HWND)pPropertySheetA(&psh);
+
+    for (int p = 0; p < 3; p++)
+    {
+        page = (HWND)SendMessageA(hp, PSM_INDEXTOHWND, p, 0);
+        if(p == 2)
+            ok(page == NULL, "Page %d created.\n", p);
+        else
+            ok(page != NULL, "Page %d not created.\n", p);
+    }
+
+    DestroyWindow(hp);
+}
+
+#define WM_DIALOG_IDLE (WM_USER + 0x99)
+
+static WNDPROC old_hotkeys_dialog_proc;
+static HWND hotkey_dialog;
+static HANDLE hotkey_dialog_idle;
+
+static LRESULT CALLBACK new_hotkeys_dialog_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
+{
+    switch(msg)
+    {
+    case WM_INITDIALOG:
+        hotkey_dialog = hwnd;
+        PostMessageW(hwnd, WM_DIALOG_IDLE, 0, 0);
+        break;
+
+    case WM_DIALOG_IDLE:
+        SetEvent(hotkey_dialog_idle);
+        return TRUE;
+    }
+
+    return CallWindowProcW(old_hotkeys_dialog_proc, hwnd, msg, wparam, lparam);
+}
+
+
+static LRESULT CALLBACK hook_proc_hotkeys(int code, WPARAM wparam, LPARAM lparam)
+{
+    static BOOL done;
+    if (code == HCBT_CREATEWND)
+    {
+        CBT_CREATEWNDW *c = (CBT_CREATEWNDW *)lparam;
+
+        /* The first dialog created will be the parent dialog */
+        if (!done && c->lpcs->lpszClass == (LPWSTR)WC_DIALOG)
+        {
+            old_hotkeys_dialog_proc = (WNDPROC)SetWindowLongPtrW((HWND)wparam, GWLP_WNDPROC, (LONG_PTR)new_hotkeys_dialog_proc);
+            done = TRUE;
+        }
+    }
+
+    return CallNextHookEx(NULL, code, wparam, lparam);
+}
+
+static DWORD WINAPI hotkey_dialog_thread(void *arg)
+{
+    HPROPSHEETPAGE hpsp[2];
+    PROPSHEETPAGEA psp[2];
+    PROPSHEETHEADERA psh;
+    HHOOK hook;
+    int i;
+
+    /* set up a hook proc in order to subclass the main dialog early on */
+    hook = SetWindowsHookExW(WH_CBT, hook_proc_hotkeys, NULL, GetCurrentThreadId());
+
+    /* create the property sheet pages */
+    memset(psp, 0, sizeof(PROPSHEETPAGEA) * ARRAY_SIZE(psp));
+
+    for (i = 0; i < ARRAY_SIZE(psp); i++)
+    {
+        psp[i].dwSize = sizeof(PROPSHEETPAGEA);
+        psp[i].hInstance = GetModuleHandleA(NULL);
+        psp[i].pszTemplate = (LPCSTR)MAKEINTRESOURCE(IDD_PROP_PAGE_RADIO);
+        psp[i].pfnDlgProc = nav_page_proc;
+        hpsp[i] = pCreatePropertySheetPageA(&psp[i]);
+    }
+
+    /* set up the property sheet dialog */
+    memset(&psh, 0, sizeof(psh));
+    psh.dwSize = PROPSHEETHEADERA_V1_SIZE;
+    psh.dwFlags = PSH_WIZARD;
+    psh.pszCaption = "A Wizard";
+    psh.nPages = ARRAY_SIZE(psp);
+    psh.hwndParent = GetDesktopWindow();
+    psh.phpage = hpsp;
+    pPropertySheetA(&psh);
+
+    UnhookWindowsHookEx(hook);
+
+    return 0;
+}
+
+static void test_hotkey_navigation(void)
+{
+    HWND control;
+    HANDLE thread;
+    int i;
+    INPUT inputs[6] = {0};
+    BOOL success;
+
+    for (i = 0; i < ARRAY_SIZE(inputs); i++)
+    {
+        inputs[i].type = INPUT_KEYBOARD;
+    }
+
+    active_page_changed = CreateEventW(NULL, FALSE, FALSE, NULL);
+    ok(active_page_changed != NULL, "CreateEvent failed, error %lu\n", GetLastError());
+    hotkey_dialog_idle = CreateEventW(NULL, FALSE, FALSE, NULL);
+    ok(hotkey_dialog_idle != NULL, "CreateEvent failed, error %lu\n", GetLastError());
+
+    thread = CreateThread(NULL, 0, hotkey_dialog_thread, NULL, 0, NULL);
+
+    WaitForSingleObject(hotkey_dialog_idle, 10000);
+    ResetEvent(active_page_changed);
+    success = SetForegroundWindow(hotkey_dialog);
+    if (!success)
+    {
+        skip( "Window not in foreground\n" );
+        goto done;
+    }
+    SetFocus(hotkey_dialog);
+
+    control = (HWND)SendMessageA(hotkey_dialog, PSM_GETCURRENTPAGEHWND, 0, 0);
+    active_page = (LONG)SendMessageA(hotkey_dialog, PSM_HWNDTOINDEX, (WPARAM)control, 0);
+    ok(active_page == 0, "Got %ld\n", active_page);
+
+    inputs[0].ki.wVk = VK_CONTROL;
+    inputs[0].ki.dwFlags = 0;
+    inputs[1].ki.wVk = VK_NEXT;
+    inputs[1].ki.dwFlags = 0;
+    inputs[2].ki.wVk = VK_NEXT;
+    inputs[2].ki.dwFlags = KEYEVENTF_KEYUP;
+    inputs[3].ki.wVk = VK_CONTROL;
+    inputs[3].ki.dwFlags = KEYEVENTF_KEYUP;
+    SendInput(4, inputs, sizeof(INPUT));
+    WaitForSingleObject(active_page_changed, 1000);
+    ok(active_page == 1, "Got %ld\n", active_page);
+    PostMessageW(hotkey_dialog, WM_DIALOG_IDLE, 0, 0);
+    WaitForSingleObject(hotkey_dialog_idle, 10000);
+
+    inputs[0].ki.wVk = VK_CONTROL;
+    inputs[0].ki.dwFlags = 0;
+    inputs[1].ki.wVk = VK_PRIOR;
+    inputs[1].ki.dwFlags = 0;
+    inputs[2].ki.wVk = VK_PRIOR;
+    inputs[2].ki.dwFlags = KEYEVENTF_KEYUP;
+    inputs[3].ki.wVk = VK_CONTROL;
+    inputs[3].ki.dwFlags = KEYEVENTF_KEYUP;
+    SendInput(4, inputs, sizeof(INPUT));
+    WaitForSingleObject(active_page_changed, 1000);
+    ok(active_page == 0, "Got %ld\n", active_page);
+    PostMessageW(hotkey_dialog, WM_DIALOG_IDLE, 0, 0);
+    WaitForSingleObject(hotkey_dialog_idle, 10000);
+
+    inputs[0].ki.wVk = VK_CONTROL;
+    inputs[0].ki.dwFlags = 0;
+    inputs[1].ki.wVk = VK_TAB;
+    inputs[1].ki.dwFlags = 0;
+    inputs[2].ki.wVk = VK_TAB;
+    inputs[2].ki.dwFlags = KEYEVENTF_KEYUP;
+    inputs[3].ki.wVk = VK_CONTROL;
+    inputs[3].ki.dwFlags = KEYEVENTF_KEYUP;
+    SendInput(4, inputs, sizeof(INPUT));
+    WaitForSingleObject(active_page_changed, 1000);
+    ok(active_page == 1, "Got %ld\n", active_page);
+    PostMessageW(hotkey_dialog, WM_DIALOG_IDLE, 0, 0);
+    WaitForSingleObject(hotkey_dialog_idle, 10000);
+
+    inputs[0].ki.wVk = VK_CONTROL;
+    inputs[0].ki.dwFlags = 0;
+    inputs[1].ki.wVk = VK_SHIFT;
+    inputs[1].ki.dwFlags = 0;
+    inputs[2].ki.wVk = VK_TAB;
+    inputs[2].ki.dwFlags = 0;
+    inputs[3].ki.wVk = VK_TAB;
+    inputs[3].ki.dwFlags = KEYEVENTF_KEYUP;
+    inputs[4].ki.wVk = VK_SHIFT;
+    inputs[4].ki.dwFlags = KEYEVENTF_KEYUP;
+    inputs[5].ki.wVk = VK_CONTROL;
+    inputs[5].ki.dwFlags = KEYEVENTF_KEYUP;
+    SendInput(6, inputs, sizeof(INPUT));
+    WaitForSingleObject(active_page_changed, 1000);
+    ok(active_page == 0, "Got %ld\n", active_page);
+    PostMessageW(hotkey_dialog, WM_DIALOG_IDLE, 0, 0);
+    WaitForSingleObject(hotkey_dialog_idle, 10000);
+
+done:
+    PostMessageA(hotkey_dialog, WM_CLOSE, 0, 0);
+    WaitForSingleObject(thread, 10000);
+    CloseHandle(thread);
+    CloseHandle(hotkey_dialog_idle);
+    CloseHandle(active_page_changed);
+    active_page_changed = NULL;
+}
+
 static void init_comctl32_functions(void)
 {
     HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
@@ -1442,6 +1785,9 @@ START_TEST(propsheet)
     test_CreatePropertySheetPage();
     test_page_dialog_texture();
     test_invalid_hpropsheetpage();
+    test_init_page_creation();
+    test_QueryInitialFocus();
+    test_hotkey_navigation();
 
     if (!load_v6_module(&ctx_cookie, &ctx))
         return;
