@@ -1379,9 +1379,12 @@ static void test_TCM_SETITEMEXTRA(void)
 
 static void test_TCS_OWNERDRAWFIXED(void)
 {
-    LPARAM lparam;
-    ULONG_PTR itemdata, itemdata2;
-    TCITEMA item;
+    struct
+    {
+        TCITEMHEADERA h;
+        BYTE lparam[sizeof(LPARAM)+1];
+    } item;
+    ULONG_PTR itemdata;
     HWND hTab;
     BOOL ret;
 
@@ -1390,11 +1393,8 @@ static void test_TCS_OWNERDRAWFIXED(void)
 
     ok(GetParent(hTab) == NULL, "got %p, expected null parent\n", GetParent(hTab));
 
-    /* set some item data */
-    memset(&lparam, 0xde, sizeof(LPARAM));
-
-    item.mask = TCIF_PARAM;
-    item.lParam = lparam;
+    item.h.mask = TCIF_PARAM;
+    memset(item.lparam, 0xde, sizeof(LPARAM));
     ret = SendMessageA(hTab, TCM_SETITEMA, 0, (LPARAM)&item);
     ok(ret == TRUE, "got %d\n", ret);
 
@@ -1423,9 +1423,8 @@ static void test_TCS_OWNERDRAWFIXED(void)
     ok(ret == TRUE, "got %d\n", ret);
 
     /* set some item data */
-    memset(&lparam, 0xde, sizeof(LPARAM));
-    item.mask = TCIF_PARAM;
-    item.lParam = lparam;
+    item.h.mask = TCIF_PARAM;
+    memset(item.lparam, 0xde, sizeof(item.lparam));
 
     ret = SendMessageA(hTab, TCM_INSERTITEMA, 0, (LPARAM)&item);
     ok(ret == 0, "got %d\n", ret);
@@ -1453,9 +1452,8 @@ static void test_TCS_OWNERDRAWFIXED(void)
     ret = SendMessageA(hTab, TCM_SETITEMEXTRA, sizeof(LPARAM)-1, 0);
     ok(ret == TRUE, "got %d\n", ret);
 
-    memset(&lparam, 0xde, sizeof(lparam));
-    item.mask = TCIF_PARAM;
-    item.lParam = lparam;
+    item.h.mask = TCIF_PARAM;
+    memset(item.lparam, 0xde, sizeof(LPARAM));
 
     ret = SendMessageA(hTab, TCM_INSERTITEMA, 0, (LPARAM)&item);
     ok(ret == 0, "got %d\n", ret);
@@ -1465,11 +1463,12 @@ static void test_TCS_OWNERDRAWFIXED(void)
     ShowWindow(hTab, SW_SHOW);
     RedrawWindow(hTab, NULL, 0, RDW_UPDATENOW);
 
-    itemdata = itemdata2 = 0;
-    memset(&itemdata, 0xde, 4);
-    memset(&itemdata2, 0xde, sizeof(LPARAM)-1);
-    ok(g_drawitem.itemData == itemdata || broken(g_drawitem.itemData == itemdata2) /* win98 */,
-        "got 0x%Ix, expected 0x%Ix\n", g_drawitem.itemData, itemdata);
+    itemdata = 0;
+    memset(&itemdata, 0xde, min(4, sizeof(LPARAM)-1));
+    todo_wine_if(sizeof(void *) == 4)
+    ok(g_drawitem.itemData == itemdata ||
+       broken(sizeof(void *) == 4 && g_drawitem.itemData == 0xdededede), /* Win7-32-bit */
+       "got 0x%Ix, expected 0x%Ix\n", g_drawitem.itemData, itemdata);
 
     DestroyWindow(hTab);
 }
@@ -1621,6 +1620,20 @@ static void test_TCM_GETROWCOUNT(void)
     DestroyWindow(hTab);
 }
 
+static void test_WM_GETOBJECT(void)
+{
+    HWND hTab;
+    DWORD objid;
+
+    hTab = createFilledTabControl(parent_wnd, TCS_FIXEDWIDTH, TCIF_TEXT|TCIF_IMAGE, 2);
+    ok(hTab != NULL, "Failed to create tab control\n");
+
+    objid = SendMessageA(hTab, WM_GETOBJECT, 0, OBJID_QUERYCLASSNAMEIDX);
+    ok(objid == 0x1000f, "Unexpected objid %lu.\n", objid);
+
+    DestroyWindow(hTab);
+}
+
 START_TEST(tab)
 {
     LOGFONTA logfont;
@@ -1660,6 +1673,7 @@ START_TEST(tab)
     test_create();
     test_TCN_SELCHANGING();
     test_TCM_GETROWCOUNT();
+    test_WM_GETOBJECT();
 
     uninit_winevent_hook();
 
