@@ -19,7 +19,7 @@
 
 #include <assert.h>
 #include <stdarg.h>
-#define NONAMELESSUNION
+
 #include "windef.h"
 #include "winbase.h"
 #include "wincrypt.h"
@@ -299,7 +299,7 @@ static BOOL compare_crl_issued_by(PCCRL_CONTEXT pCrlContext, DWORD dwType,
                             ret = CertCompareCertificateName(
                              issuer->dwCertEncodingType,
                              &issuer->pCertInfo->Subject,
-                             &directoryName->u.DirectoryName);
+                             &directoryName->DirectoryName);
                             if (ret)
                                 ret = CertCompareIntegerBlob(
                                  &issuer->pCertInfo->SerialNumber,
@@ -692,31 +692,31 @@ static BOOL compare_dist_point_name(const CRL_DIST_POINT_NAME *name1,
         match = TRUE;
         if (name1->dwDistPointNameChoice == CRL_DIST_POINT_FULL_NAME)
         {
-            if (name1->u.FullName.cAltEntry == name2->u.FullName.cAltEntry)
+            if (name1->FullName.cAltEntry == name2->FullName.cAltEntry)
             {
                 DWORD i;
 
-                for (i = 0; match && i < name1->u.FullName.cAltEntry; i++)
+                for (i = 0; match && i < name1->FullName.cAltEntry; i++)
                 {
                     const CERT_ALT_NAME_ENTRY *entry1 =
-                     &name1->u.FullName.rgAltEntry[i];
+                     &name1->FullName.rgAltEntry[i];
                     const CERT_ALT_NAME_ENTRY *entry2 =
-                     &name2->u.FullName.rgAltEntry[i];
+                     &name2->FullName.rgAltEntry[i];
 
                     if (entry1->dwAltNameChoice == entry2->dwAltNameChoice)
                     {
                         switch (entry1->dwAltNameChoice)
                         {
                         case CERT_ALT_NAME_URL:
-                            match = !wcsicmp(entry1->u.pwszURL,
-                             entry2->u.pwszURL);
+                            match = !wcsicmp(entry1->pwszURL,
+                             entry2->pwszURL);
                             break;
                         case CERT_ALT_NAME_DIRECTORY_NAME:
-                            match = (entry1->u.DirectoryName.cbData ==
-                             entry2->u.DirectoryName.cbData) &&
-                             !memcmp(entry1->u.DirectoryName.pbData,
-                             entry2->u.DirectoryName.pbData,
-                             entry1->u.DirectoryName.cbData);
+                            match = (entry1->DirectoryName.cbData ==
+                             entry2->DirectoryName.cbData) &&
+                             !memcmp(entry1->DirectoryName.pbData,
+                             entry2->DirectoryName.pbData,
+                             entry1->DirectoryName.cbData);
                             break;
                         default:
                             FIXME("unimplemented for type %ld\n",
