@@ -69,6 +69,8 @@ static ULONG WINAPI d3d8_surface_AddRef(IDirect3DSurface8 *iface)
         if (surface->wined3d_rtv)
             wined3d_rendertarget_view_incref(surface->wined3d_rtv);
         wined3d_texture_incref(surface->wined3d_texture);
+        if (surface->swapchain)
+            wined3d_swapchain_incref(surface->swapchain);
     }
 
     return refcount;
@@ -102,6 +104,10 @@ static ULONG WINAPI d3d8_surface_Release(IDirect3DSurface8 *iface)
 
         if (surface->wined3d_rtv)
             wined3d_rendertarget_view_decref(surface->wined3d_rtv);
+        if (surface->swapchain)
+           wined3d_swapchain_decref(surface->swapchain);
+        /* Releasing the texture may free the d3d8 object, so do not access it
+         * after releasing the texture. */
         wined3d_texture_decref(surface->wined3d_texture);
 
         if (parent_device)
@@ -301,7 +307,7 @@ static void STDMETHODCALLTYPE surface_wined3d_object_destroyed(void *parent)
 {
     struct d3d8_surface *surface = parent;
     d3d8_resource_cleanup(&surface->resource);
-    heap_free(surface);
+    free(surface);
 }
 
 static const struct wined3d_parent_ops d3d8_surface_wined3d_parent_ops =
@@ -309,18 +315,23 @@ static const struct wined3d_parent_ops d3d8_surface_wined3d_parent_ops =
     surface_wined3d_object_destroyed,
 };
 
-void surface_init(struct d3d8_surface *surface, struct wined3d_texture *wined3d_texture, unsigned int sub_resource_idx,
-        const struct wined3d_parent_ops **parent_ops)
+struct d3d8_surface *d3d8_surface_create(struct wined3d_texture *wined3d_texture,
+        unsigned int sub_resource_idx, IUnknown *container)
 {
     IDirect3DBaseTexture8 *texture;
+    struct d3d8_surface *surface;
+
+    if (!(surface = calloc(1, sizeof(*surface))))
+        return NULL;
 
     surface->IDirect3DSurface8_iface.lpVtbl = &d3d8_surface_vtbl;
     d3d8_resource_init(&surface->resource);
     surface->resource.refcount = 0;
     list_init(&surface->rtv_entry);
-    surface->container = wined3d_texture_get_parent(wined3d_texture);
+    surface->container = container;
     surface->wined3d_texture = wined3d_texture;
     surface->sub_resource_idx = sub_resource_idx;
+    surface->swapchain = wined3d_texture_get_swapchain(wined3d_texture);
 
     if (surface->container && SUCCEEDED(IUnknown_QueryInterface(surface->container,
             &IID_IDirect3DBaseTexture8, (void **)&texture)))
@@ -329,7 +340,11 @@ void surface_init(struct d3d8_surface *surface, struct wined3d_texture *wined3d_
         IDirect3DBaseTexture8_Release(texture);
     }
 
-    *parent_ops = &d3d8_surface_wined3d_parent_ops;
+    wined3d_texture_set_sub_resource_parent(wined3d_texture, sub_resource_idx,
+            surface, &d3d8_surface_wined3d_parent_ops);
+
+    TRACE("Created surface %p.\n", surface);
+    return surface;
 }
 
 static void STDMETHODCALLTYPE view_wined3d_object_destroyed(void *parent)
@@ -357,7 +372,7 @@ static const struct wined3d_parent_ops d3d8_view_wined3d_parent_ops =
 struct d3d8_device *d3d8_surface_get_device(const struct d3d8_surface *surface)
 {
     IDirect3DDevice8 *device;
-    device = surface->texture ? surface->texture->parent_device : surface->parent_device;
+    device = surface->texture ? &surface->texture->parent_device->IDirect3DDevice8_iface : surface->parent_device;
     return impl_from_IDirect3DDevice8(device);
 }
 
