@@ -129,9 +129,9 @@ static ULONG WINAPI d3dx9_file_data_Release(ID3DXFileData *iface)
             ID3DXFileData *child = file_data->children[i];
             child->lpVtbl->Release(child);
         }
-        HeapFree(GetProcessHeap(), 0, file_data->children);
+        free(file_data->children);
         IDirectXFileData_Release(file_data->dxfile_data);
-        HeapFree(GetProcessHeap(), 0, file_data);
+        free(file_data);
     }
 
     return refcount;
@@ -303,7 +303,7 @@ static HRESULT d3dx9_file_data_create(IDirectXFileObject *dxfile_object, ID3DXFi
 
     *ret_iface = NULL;
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    object = calloc(1, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
@@ -322,7 +322,7 @@ static HRESULT d3dx9_file_data_create(IDirectXFileObject *dxfile_object, ID3DXFi
             IUnknown_Release(reference);
             if (FAILED(ret))
             {
-                HeapFree(GetProcessHeap(), 0, object);
+                free(object);
                 return E_FAIL;
             }
             object->reference = TRUE;
@@ -330,7 +330,7 @@ static HRESULT d3dx9_file_data_create(IDirectXFileObject *dxfile_object, ID3DXFi
         else
         {
             FIXME("Don't know what to do with binary object\n");
-            HeapFree(GetProcessHeap(), 0, object);
+            free(object);
             return E_FAIL;
         }
     }
@@ -341,17 +341,8 @@ static HRESULT d3dx9_file_data_create(IDirectXFileObject *dxfile_object, ID3DXFi
         {
             ID3DXFileData **new_children;
 
-            if (object->children)
-            {
-                children_array_size *= 2;
-                new_children = HeapReAlloc(GetProcessHeap(), 0, object->children,
-                        sizeof(*object->children) * children_array_size);
-            }
-            else
-            {
-                children_array_size = 4;
-                new_children = HeapAlloc(GetProcessHeap(), 0, sizeof(*object->children) * children_array_size);
-            }
+            children_array_size = object->children ? children_array_size * 2 : 4;
+            new_children = realloc(object->children, sizeof(*object->children) * children_array_size);
             if (!new_children)
             {
                 ret = E_OUTOFMEMORY;
@@ -374,8 +365,7 @@ static HRESULT d3dx9_file_data_create(IDirectXFileObject *dxfile_object, ID3DXFi
     {
         ID3DXFileData **new_children;
 
-        new_children = HeapReAlloc(GetProcessHeap(), 0, object->children,
-                sizeof(*object->children) * object->child_count);
+        new_children = realloc(object->children, sizeof(*object->children) * object->child_count);
         if (new_children)
             object->children = new_children;
     }
@@ -431,8 +421,8 @@ static ULONG WINAPI d3dx9_file_enum_object_Release(ID3DXFileEnumObject *iface)
             ID3DXFileData *child = file_enum->children[i];
             child->lpVtbl->Release(child);
         }
-        HeapFree(GetProcessHeap(), 0, file_enum->children);
-        HeapFree(GetProcessHeap(), 0, file_enum);
+        free(file_enum->children);
+        free(file_enum);
     }
 
     return refcount;
@@ -540,7 +530,7 @@ static ULONG WINAPI d3dx9_file_Release(ID3DXFile *iface)
     if (!refcount)
     {
         IDirectXFile_Release(file->dxfile);
-        HeapFree(GetProcessHeap(), 0, file);
+        free(file);
     }
 
     return refcount;
@@ -597,7 +587,7 @@ static HRESULT WINAPI d3dx9_file_CreateEnumObject(ID3DXFile *iface, const void *
         return E_NOTIMPL;
     }
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    object = calloc(1, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
@@ -608,7 +598,7 @@ static HRESULT WINAPI d3dx9_file_CreateEnumObject(ID3DXFile *iface, const void *
 
     if (ret != S_OK)
     {
-        HeapFree(GetProcessHeap(), 0, object);
+        free(object);
         return ret;
     }
 
@@ -619,17 +609,8 @@ static HRESULT WINAPI d3dx9_file_CreateEnumObject(ID3DXFile *iface, const void *
         {
             ID3DXFileData **new_children;
 
-            if (object->children)
-            {
-                children_array_size *= 2;
-                new_children = HeapReAlloc(GetProcessHeap(), 0, object->children,
-                        sizeof(*object->children) * children_array_size);
-            }
-            else
-            {
-                children_array_size = 4;
-                new_children = HeapAlloc(GetProcessHeap(), 0, sizeof(*object->children) * children_array_size);
-            }
+            children_array_size = object->children ? children_array_size * 2 : 4;
+            new_children = realloc(object->children, sizeof(*object->children) * children_array_size);
             if (!new_children)
             {
                 ret = E_OUTOFMEMORY;
@@ -648,8 +629,7 @@ static HRESULT WINAPI d3dx9_file_CreateEnumObject(ID3DXFile *iface, const void *
     {
         ID3DXFileData **new_children;
 
-        new_children = HeapReAlloc(GetProcessHeap(), 0, object->children,
-                sizeof(*object->children) * object->child_count);
+        new_children = realloc(object->children, sizeof(*object->children) * object->child_count);
         if (new_children)
             object->children = new_children;
     }
@@ -722,14 +702,14 @@ HRESULT WINAPI D3DXFileCreate(ID3DXFile **d3dxfile)
 
     *d3dxfile = NULL;
 
-    object = HeapAlloc(GetProcessHeap(), 0, sizeof(*object));
+    object = calloc(1, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
     ret = DirectXFileCreate(&object->dxfile);
     if (ret != S_OK)
     {
-        HeapFree(GetProcessHeap(), 0, object);
+        free(object);
         if (ret == E_OUTOFMEMORY)
             return ret;
         return E_FAIL;
