@@ -804,9 +804,11 @@ static const struct elf_sym *elf_lookup_symtab(const struct module* module,
         if (result)
         {
             FIXME("Already found symbol %s (%s) in symtab %s @%08x and %s @%08x\n",
-                  name, compiland_name,
-                  source_get(module, result->compiland->source), (unsigned int)result->sym.st_value,
-                  source_get(module, ste->compiland->source), (unsigned int)ste->sym.st_value);
+                  debugstr_a(name), debugstr_a(compiland_name),
+                  debugstr_a(source_get(module, result->compiland->source)),
+                  (unsigned int)result->sym.st_value,
+                  debugstr_a(source_get(module, ste->compiland->source)),
+                  (unsigned int)ste->sym.st_value);
         }
         else
         {
@@ -817,7 +819,7 @@ static const struct elf_sym *elf_lookup_symtab(const struct module* module,
     if (!result && !(result = weak_result))
     {
         FIXME("Couldn't find symbol %s!%s in symtab\n",
-              debugstr_w(module->modulename), name);
+              debugstr_w(module->modulename), debugstr_a(name));
         return NULL;
     }
     return &result->sym;
@@ -862,19 +864,19 @@ static void elf_finish_stabs_info(struct module* module, const struct hash_table
                 if (((struct symt_function*)sym)->ranges[0].low != elf_info->elf_addr &&
                     ((struct symt_function*)sym)->ranges[0].low != elf_info->elf_addr + symp->st_value)
                     FIXME("Changing address for %p/%s!%s from %I64x to %I64x\n",
-                          sym, debugstr_w(module->modulename), sym->hash_elt.name,
+                          sym, debugstr_w(module->modulename), debugstr_a(sym->hash_elt.name),
                           ((struct symt_function*)sym)->ranges[0].low,
                           elf_info->elf_addr + symp->st_value);
                 if (size && size != symp->st_size)
                     FIXME("Changing size for %p/%s!%s from %I64x to %I64x\n",
-                          sym, debugstr_w(module->modulename), sym->hash_elt.name,
+                          sym, debugstr_w(module->modulename), debugstr_a(sym->hash_elt.name),
                           size, symp->st_size);
 
                 ((struct symt_function*)sym)->ranges[0].low = elf_info->elf_addr + symp->st_value;
                 ((struct symt_function*)sym)->ranges[0].high = elf_info->elf_addr + symp->st_value + symp->st_size;
             } else
                 FIXME("Couldn't find %s!%s\n",
-                      debugstr_w(module->modulename), sym->hash_elt.name);
+                      debugstr_w(module->modulename), debugstr_a(sym->hash_elt.name));
             break;
         case SymTagData:
             switch (((struct symt_data*)sym)->kind)
@@ -891,7 +893,7 @@ static void elf_finish_stabs_info(struct module* module, const struct hash_table
                     if (((struct symt_data*)sym)->u.var.offset != elf_info->elf_addr &&
                         ((struct symt_data*)sym)->u.var.offset != elf_info->elf_addr + symp->st_value)
                         FIXME("Changing address for %p/%s!%s from %I64x to %I64x\n",
-                              sym, debugstr_w(module->modulename), sym->hash_elt.name,
+                              sym, debugstr_w(module->modulename), debugstr_a(sym->hash_elt.name),
                               ((struct symt_function*)sym)->ranges[0].low,
                               elf_info->elf_addr + symp->st_value);
                     ((struct symt_data*)sym)->u.var.offset = elf_info->elf_addr + symp->st_value;
@@ -899,7 +901,7 @@ static void elf_finish_stabs_info(struct module* module, const struct hash_table
                         DataIsFileStatic : DataIsGlobal;
                 } else
                     FIXME("Couldn't find %s!%s\n",
-                          debugstr_w(module->modulename), sym->hash_elt.name);
+                          debugstr_w(module->modulename), debugstr_a(sym->hash_elt.name));
                 break;
             default:;
             }
@@ -1783,7 +1785,6 @@ BOOL elf_read_wine_loader_dbg_info(struct process* pcs, ULONG_PTR addr)
     {
         ERR("Unable to access ELF libraries (outside 32bit limit)\n");
         module_remove(pcs, elf_info.module);
-        pcs->loader = &empty_loader_ops;
         return FALSE;
     }
     TRACE("Found ELF debug header %#I64x\n", elf_info.dbg_hdr_addr);
