@@ -29,32 +29,33 @@
 
 extern HINSTANCE DINPUT_instance;
 
-/* Implementation specification */
-typedef struct IDirectInputImpl IDirectInputImpl;
-struct IDirectInputImpl
+struct dinput
 {
-    IDirectInput7A              IDirectInput7A_iface;
-    IDirectInput7W              IDirectInput7W_iface;
-    IDirectInput8A              IDirectInput8A_iface;
-    IDirectInput8W              IDirectInput8W_iface;
-    IDirectInputJoyConfig8      IDirectInputJoyConfig8_iface;
+    IDirectInput7A IDirectInput7A_iface;
+    IDirectInput7W IDirectInput7W_iface;
+    IDirectInput8A IDirectInput8A_iface;
+    IDirectInput8W IDirectInput8W_iface;
+    IDirectInputJoyConfig8 IDirectInputJoyConfig8_iface;
+    LONG internal_ref;
+    LONG ref;
 
-    LONG                        ref;
-    BOOL                        initialized;
-    DWORD                       evsequence;     /* unique sequence number for events */
-    DWORD                       dwVersion;      /* direct input version number */
-    struct list                 device_players; /* device instance guid to player name */
+    DWORD dwVersion;            /* direct input version number */
+    DWORD evsequence;           /* unique sequence number for events */
+    struct list device_players; /* device instance guid to player name */
 };
 
-extern const IDirectInput7AVtbl dinput7_a_vtbl DECLSPEC_HIDDEN;
-extern const IDirectInput8AVtbl dinput8_a_vtbl DECLSPEC_HIDDEN;
+extern const IDirectInput7AVtbl dinput7_a_vtbl;
+extern const IDirectInput8AVtbl dinput8_a_vtbl;
+
+extern void dinput_internal_addref( struct dinput *dinput );
+extern void dinput_internal_release( struct dinput *dinput );
 
 extern HRESULT mouse_enum_device( DWORD type, DWORD flags, DIDEVICEINSTANCEW *instance, DWORD version );
-extern HRESULT mouse_create_device( IDirectInputImpl *dinput, const GUID *guid, IDirectInputDevice8W **out );
+extern HRESULT mouse_create_device( struct dinput *dinput, const GUID *guid, IDirectInputDevice8W **out );
 extern HRESULT keyboard_enum_device( DWORD type, DWORD flags, DIDEVICEINSTANCEW *instance, DWORD version );
-extern HRESULT keyboard_create_device( IDirectInputImpl *dinput, const GUID *guid, IDirectInputDevice8W **out );
+extern HRESULT keyboard_create_device( struct dinput *dinput, const GUID *guid, IDirectInputDevice8W **out );
 extern HRESULT hid_joystick_enum_device( DWORD type, DWORD flags, DIDEVICEINSTANCEW *instance, DWORD version, int index );
-extern HRESULT hid_joystick_create_device( IDirectInputImpl *dinput, const GUID *guid, IDirectInputDevice8W **out );
+extern HRESULT hid_joystick_create_device( struct dinput *dinput, const GUID *guid, IDirectInputDevice8W **out );
 
 struct DevicePlayer {
     GUID instance_guid;
@@ -62,20 +63,24 @@ struct DevicePlayer {
     struct list entry;
 };
 
+extern void input_thread_add_user(void);
+extern void input_thread_remove_user(void);
+
 extern void dinput_hooks_acquire_device( IDirectInputDevice8W *iface );
 extern void dinput_hooks_unacquire_device( IDirectInputDevice8W *iface );
 extern int dinput_mouse_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam );
 extern int dinput_keyboard_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam );
 extern void dinput_mouse_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam,
                                         RAWINPUT *raw );
+extern void dinput_keyboard_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam,
+                                           RAWINPUT *raw );
 
-extern void check_dinput_hooks( IDirectInputDevice8W *iface, BOOL acquired ) DECLSPEC_HIDDEN;
-extern void check_dinput_events(void) DECLSPEC_HIDDEN;
+extern void check_dinput_events(void);
 
-extern HRESULT _configure_devices(IDirectInput8W *iface, LPDICONFIGUREDEVICESCALLBACK lpdiCallback, LPDICONFIGUREDEVICESPARAMSW lpdiCDParams, DWORD dwFlags, LPVOID pvRefData) DECLSPEC_HIDDEN;
+extern HRESULT _configure_devices(IDirectInput8W *iface, LPDICONFIGUREDEVICESCALLBACK lpdiCallback, LPDICONFIGUREDEVICESPARAMSW lpdiCDParams, DWORD dwFlags, LPVOID pvRefData);
 
-extern WCHAR* get_mapping_path(const WCHAR *device, const WCHAR *username) DECLSPEC_HIDDEN;
-extern DWORD get_device_type(DWORD version, BOOL is_joystick) DECLSPEC_HIDDEN;
+extern WCHAR* get_mapping_path(const WCHAR *device, const WCHAR *username);
+extern DWORD get_device_type(DWORD version, BOOL is_joystick);
 
 #define IS_DIPROP(x)    (((ULONG_PTR)(x) >> 16) == 0)
 
