@@ -83,22 +83,13 @@ static BYTE map_dik_code(DWORD scanCode, DWORD vkCode, DWORD subType, DWORD vers
     return (BYTE)scanCode;
 }
 
-int dinput_keyboard_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam )
+static void keyboard_handle_event( struct keyboard *impl, DWORD vkey, DWORD scan_code, BOOL up )
 {
-    struct keyboard *impl = impl_from_IDirectInputDevice8W( iface );
     BYTE new_diks, subtype = GET_DIDEVICE_SUBTYPE( impl->base.instance.dwDevType );
-    int dik_code, ret = impl->base.dwCoopLevel & DISCL_EXCLUSIVE;
-    KBDLLHOOKSTRUCT *hook = (KBDLLHOOKSTRUCT *)lparam;
-    DWORD scan_code;
+    IDirectInputDevice8W *iface = &impl->base.IDirectInputDevice8W_iface;
+    int dik_code, index;
 
-    if (wparam != WM_KEYDOWN && wparam != WM_KEYUP &&
-        wparam != WM_SYSKEYDOWN && wparam != WM_SYSKEYUP)
-        return 0;
-
-    TRACE( "iface %p, wparam %#Ix, lparam %#Ix, vkCode %#lx, scanCode %#lx.\n", iface, wparam,
-           lparam, hook->vkCode, hook->scanCode );
-
-    switch (hook->vkCode)
+    switch (vkey)
     {
         /* R-Shift is special - it is an extended key with separate scan code */
         case VK_RSHIFT  : dik_code = DIK_RSHIFT; break;
@@ -106,25 +97,55 @@ int dinput_keyboard_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lpa
         case VK_NUMLOCK : dik_code = DIK_NUMLOCK; break;
         case VK_SUBTRACT: dik_code = DIK_SUBTRACT; break;
         default:
-            scan_code = hook->scanCode & 0xff;
-            if (hook->flags & LLKHF_EXTENDED) scan_code |= 0x100;
-            dik_code = map_dik_code( scan_code, hook->vkCode, subtype, impl->base.dinput->dwVersion );
+            dik_code = map_dik_code( scan_code, vkey, subtype, impl->base.dinput->dwVersion );
+            break;
     }
-    new_diks = hook->flags & LLKHF_UP ? 0 : 0x80;
+    new_diks = (up ? 0 : 0x80);
 
     /* returns now if key event already known */
-    if (new_diks == impl->base.device_state[dik_code]) return ret;
+    if (new_diks == impl->base.device_state[dik_code]) return;
 
     impl->base.device_state[dik_code] = new_diks;
-    TRACE( " setting key %02x to %02x\n", dik_code, impl->base.device_state[dik_code] );
+    TRACE( "setting key %02x to %02x\n", dik_code, impl->base.device_state[dik_code] );
 
     EnterCriticalSection( &impl->base.crit );
-    queue_event( iface, DIDFT_MAKEINSTANCE( dik_code ) | DIDFT_PSHBUTTON, new_diks,
-                 GetCurrentTime(), impl->base.dinput->evsequence++ );
+
+    if ((index = dinput_device_object_index_from_id( iface, DIDFT_PSHBUTTON | DIDFT_MAKEINSTANCE( dik_code ) )) >= 0)
+        queue_event( iface, index, new_diks, GetCurrentTime(), impl->base.dinput->evsequence++ );
+
     if (impl->base.hEvent) SetEvent( impl->base.hEvent );
     LeaveCriticalSection( &impl->base.crit );
+}
 
-    return ret;
+void dinput_keyboard_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam, RAWINPUT *ri )
+{
+    struct keyboard *impl = impl_from_IDirectInputDevice8W( iface );
+    DWORD scan_code;
+
+    TRACE("(%p) wparam %Ix, lparam %Ix\n", iface, wparam, lparam);
+
+    scan_code = ri->data.keyboard.MakeCode & 0xff;
+    if (ri->data.keyboard.Flags & RI_KEY_E0) scan_code |= 0x100;
+    keyboard_handle_event( impl, ri->data.keyboard.VKey, scan_code, ri->data.keyboard.Flags & RI_KEY_BREAK );
+}
+
+int dinput_keyboard_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam )
+{
+    struct keyboard *impl = impl_from_IDirectInputDevice8W( iface );
+    KBDLLHOOKSTRUCT *hook = (KBDLLHOOKSTRUCT *)lparam;
+    DWORD scan_code;
+
+    TRACE( "iface %p, wparam %#Ix, lparam %#Ix, vkCode %#lx, scanCode %#lx.\n", iface, wparam,
+           lparam, hook->vkCode, hook->scanCode );
+
+    if (wparam != WM_KEYDOWN && wparam != WM_KEYUP && wparam != WM_SYSKEYDOWN && wparam != WM_SYSKEYUP)
+        return 0;
+
+    scan_code = hook->scanCode & 0xff;
+    if (hook->flags & LLKHF_EXTENDED) scan_code |= 0x100;
+    keyboard_handle_event( impl, hook->vkCode, scan_code, hook->flags & LLKHF_UP );
+
+    return impl->base.dwCoopLevel & DISCL_EXCLUSIVE;
 }
 
 static DWORD get_keyboard_subtype(void)
@@ -166,7 +187,11 @@ HRESULT keyboard_enum_device( DWORD type, DWORD flags, DIDEVICEINSTANCEW *instan
 
 HRESULT keyboard_create_device( struct dinput *dinput, const GUID *guid, IDirectInputDevice8W **out )
 {
+    DIDEVICEOBJECTINSTANCEW instance;
     struct keyboard *impl;
+    DWORD i, index, dik;
+    BYTE subtype;
+    HRESULT hr;
 
     TRACE( "dinput %p, guid %s, out %p.\n", dinput, debugstr_guid( guid ), out );
 
@@ -181,9 +206,28 @@ HRESULT keyboard_create_device( struct dinput *dinput, const GUID *guid, IDirect
     impl->base.caps.dwDevType = impl->base.instance.dwDevType;
     impl->base.caps.dwFirmwareRevision = 100;
     impl->base.caps.dwHardwareRevision = 100;
+    if (dinput->dwVersion >= 0x0800) impl->base.use_raw_input = TRUE;
+    subtype = GET_DIDEVICE_SUBTYPE( impl->base.instance.dwDevType );
+
+    if (FAILED(hr = dinput_device_init_device_format( &impl->base.IDirectInputDevice8W_iface ))) goto failed;
+
+    for (i = 0, index = 0; i < 512; ++i)
+    {
+        if (!GetKeyNameTextW( i << 16, instance.tszName, ARRAY_SIZE(instance.tszName) )) continue;
+        if (!(dik = map_dik_code( i, 0, subtype, impl->base.dinput->dwVersion ))) continue;
+
+        if (dik == DIK_NUMLOCK) impl->base.object_properties[index++].scan_code = 0x451de1;
+        else if (dik == DIK_PAUSE) impl->base.object_properties[index++].scan_code = 0x45;
+        else if (dik < 0x80) impl->base.object_properties[index++].scan_code = dik;
+        else impl->base.object_properties[index++].scan_code = (dik - 0x80) << 8 | 0x00e0;
+    }
 
     *out = &impl->base.IDirectInputDevice8W_iface;
     return DI_OK;
+
+failed:
+    IDirectInputDevice_Release( &impl->base.IDirectInputDevice8W_iface );
+    return hr;
 }
 
 static HRESULT keyboard_poll( IDirectInputDevice8W *iface )
@@ -204,28 +248,28 @@ static HRESULT keyboard_unacquire( IDirectInputDevice8W *iface )
     return DI_OK;
 }
 
-static BOOL try_enum_object( const DIPROPHEADER *filter, DWORD flags, LPDIENUMDEVICEOBJECTSCALLBACKW callback,
-                             DIDEVICEOBJECTINSTANCEW *instance, void *data )
+static BOOL try_enum_object( struct dinput_device *impl, const DIPROPHEADER *filter, DWORD flags, enum_object_callback callback,
+                             UINT index, DIDEVICEOBJECTINSTANCEW *instance, void *data )
 {
     if (flags != DIDFT_ALL && !(flags & DIDFT_GETTYPE( instance->dwType ))) return DIENUM_CONTINUE;
 
     switch (filter->dwHow)
     {
     case DIPH_DEVICE:
-        return callback( instance, data );
+        return callback( impl, index, NULL, instance, data );
     case DIPH_BYOFFSET:
         if (filter->dwObj != instance->dwOfs) return DIENUM_CONTINUE;
-        return callback( instance, data );
+        return callback( impl, index, NULL, instance, data );
     case DIPH_BYID:
         if ((filter->dwObj & 0x00ffffff) != (instance->dwType & 0x00ffffff)) return DIENUM_CONTINUE;
-        return callback( instance, data );
+        return callback( impl, index, NULL, instance, data );
     }
 
     return DIENUM_CONTINUE;
 }
 
 static HRESULT keyboard_enum_objects( IDirectInputDevice8W *iface, const DIPROPHEADER *filter,
-                                      DWORD flags, LPDIENUMDEVICEOBJECTSCALLBACKW callback, void *context )
+                                      DWORD flags, enum_object_callback callback, void *context )
 {
     struct keyboard *impl = impl_from_IDirectInputDevice8W( iface );
     BYTE subtype = GET_DIDEVICE_SUBTYPE( impl->base.instance.dwDevType );
@@ -236,16 +280,16 @@ static HRESULT keyboard_enum_objects( IDirectInputDevice8W *iface, const DIPROPH
         .dwOfs = DIK_ESCAPE,
         .dwType = DIDFT_PSHBUTTON | DIDFT_MAKEINSTANCE( DIK_ESCAPE ),
     };
-    DWORD i, dik;
+    DWORD index, i, dik;
     BOOL ret;
 
-    for (i = 0; i < 512; ++i)
+    for (i = 0, index = 0; i < 512; ++i)
     {
         if (!GetKeyNameTextW( i << 16, instance.tszName, ARRAY_SIZE(instance.tszName) )) continue;
         if (!(dik = map_dik_code( i, 0, subtype, impl->base.dinput->dwVersion ))) continue;
         instance.dwOfs = dik;
         instance.dwType = DIDFT_PSHBUTTON | DIDFT_MAKEINSTANCE( dik );
-        ret = try_enum_object( filter, flags, callback, &instance, context );
+        ret = try_enum_object( &impl->base, filter, flags, callback, index++, &instance, context );
         if (ret != DIENUM_CONTINUE) return DIENUM_STOP;
     }
 
