@@ -21,11 +21,15 @@
 #include <stdarg.h>
 #include "windef.h"
 #include "winbase.h"
+#include "winternl.h"
 #include "winerror.h"
 #include "winnls.h"
 #include "windns.h"
 #include "nb30.h"
 #include "ws2def.h"
+#include "in6addr.h"
+#include "inaddr.h"
+#include "ip2string.h"
 
 #include "wine/debug.h"
 #include "dnsapi.h"
@@ -77,6 +81,7 @@ static DNS_STATUS do_query_netbios( PCSTR name, DNS_RECORDA **recp )
             if (!record->pName)
             {
                 status = ERROR_NOT_ENOUGH_MEMORY;
+                free( record );
                 goto exit;
             }
 
@@ -169,6 +174,7 @@ DNS_STATUS WINAPI DnsQuery_UTF8( const char *name, WORD type, DWORD options, voi
     unsigned char answer[4096];
     DWORD len = sizeof(answer);
     struct query_params query_params = { name, type, options, answer, &len };
+    const char *end;
 
     TRACE( "(%s, %s, %#lx, %p, %p, %p)\n", debugstr_a(name), debugstr_type( type ),
            options, servers, result, reserved );
@@ -176,6 +182,43 @@ DNS_STATUS WINAPI DnsQuery_UTF8( const char *name, WORD type, DWORD options, voi
     if (!name || !result)
         return ERROR_INVALID_PARAMETER;
 
+    if (type == DNS_TYPE_A)
+    {
+        struct in_addr addr;
+
+        if (!RtlIpv4StringToAddressA(name, TRUE, &end, &addr) && !*end)
+        {
+            DNS_RECORDA *r = calloc(1, sizeof(*r));
+            r->pName = strdup(name);
+            r->wType = DNS_TYPE_A;
+            r->wDataLength = sizeof(r->Data.A);
+            r->dwTtl = 604800;
+            r->Flags.S.Reserved = 0x20;
+            r->Flags.S.CharSet = DnsCharSetUtf8;
+            r->Data.A.IpAddress = addr.s_addr;
+            *result = r;
+            return ERROR_SUCCESS;
+        }
+    }
+    else if (type == DNS_TYPE_AAAA)
+    {
+        struct in6_addr addr;
+
+        if (!RtlIpv6StringToAddressA(name, &end, &addr) && !*end)
+        {
+            DNS_RECORDA *r = calloc(1, sizeof(*r));
+            r->pName = strdup(name);
+            r->wType = DNS_TYPE_AAAA;
+            r->wDataLength = sizeof(r->Data.AAAA);
+            r->dwTtl = 604800;
+            r->Flags.S.Reserved = 0x20;
+            r->Flags.S.CharSet = DnsCharSetUtf8;
+            memcpy(&r->Data.AAAA.Ip6Address, &addr, sizeof(r->Data.AAAA.Ip6Address));
+            *result = r;
+            return ERROR_SUCCESS;
+        }
+    }
+
     if ((ret = RESOLV_CALL( set_serverlist, servers ))) return ret;
 
     ret = RESOLV_CALL( query, &query_params );
@@ -286,9 +329,9 @@ static DNS_STATUS get_hostname_w( COMPUTER_NAME_FORMAT format, PWSTR buffer, PDW
 static DNS_STATUS get_dns_server_list( IP4_ARRAY *out, DWORD *len )
 {
     char buf[FIELD_OFFSET(DNS_ADDR_ARRAY, AddrArray[3])];
-    DNS_ADDR_ARRAY *servers = (DNS_ADDR_ARRAY *)buf;
     DWORD ret, needed, i, num, array_len = sizeof(buf);
-    struct get_serverlist_params params = { AF_INET, servers, &array_len };
+    struct get_serverlist_params params = { AF_INET, (DNS_ADDR_ARRAY *)buf, &array_len };
+
     for (;;)
     {
         ret = RESOLV_CALL( get_serverlist, &params );
@@ -303,9 +346,9 @@ static DNS_STATUS get_dns_server_list( IP4_ARRAY *out, DWORD *len )
         }
         if (!ret) break;
 
-        if ((char *)servers != buf) free( servers );
-        servers = malloc( array_len );
-        if (!servers)
+        if ((char *)params.addrs != buf) free( params.addrs );
+        params.addrs = malloc( array_len );
+        if (!params.addrs)
         {
             ret = ERROR_NOT_ENOUGH_MEMORY;
             goto err;
@@ -314,12 +357,12 @@ static DNS_STATUS get_dns_server_list( IP4_ARRAY *out, DWORD *len )
 
     out->AddrCount = num;
     for (i = 0; i < num; i++)
-        out->AddrArray[i] = ((SOCKADDR_IN *)servers->AddrArray[i].MaxSa)->sin_addr.s_addr;
+        out->AddrArray[i] = ((SOCKADDR_IN *)params.addrs->AddrArray[i].MaxSa)->sin_addr.s_addr;
     *len = needed;
     ret = ERROR_SUCCESS;
 
 err:
-    if ((char *)servers != buf) free( servers );
+    if ((char *)params.addrs != buf) free( params.addrs );
     return ret;
 }
 
