@@ -470,7 +470,7 @@ static HRESULT STDMETHODCALLTYPE dxgi_swapchain_factory_create_swapchain(IWineDX
 
     TRACE("Created swapchain %p.\n", object);
 
-    *swapchain = &object->IDXGISwapChain1_iface;
+    *swapchain = (IDXGISwapChain1 *)&object->IDXGISwapChain4_iface;
 
     return S_OK;
 }
