@@ -236,7 +236,7 @@ static ULONG STDMETHODCALLTYPE dxgi_output_Release(IDXGIOutput6 *iface)
     {
         wined3d_private_store_cleanup(&output->private_store);
         IWineDXGIAdapter_Release(&output->adapter->IWineDXGIAdapter_iface);
-        heap_free(output);
+        free(output);
     }
 
     return refcount;
@@ -725,7 +725,7 @@ static void dxgi_output_init(struct dxgi_output *output, unsigned int output_idx
 HRESULT dxgi_output_create(struct dxgi_adapter *adapter, unsigned int output_idx,
         struct dxgi_output **output)
 {
-    if (!(*output = heap_alloc_zero(sizeof(**output))))
+    if (!(*output = calloc(1, sizeof(**output))))
         return E_OUTOFMEMORY;
 
     dxgi_output_init(*output, output_idx, adapter);
