@@ -27,6 +27,7 @@ static const char *debug_feature_level(D3D_FEATURE_LEVEL feature_level)
 {
     switch (feature_level)
     {
+        WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_1_0_CORE);
         WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_9_1);
         WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_9_2);
         WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_9_3);
@@ -34,6 +35,9 @@ static const char *debug_feature_level(D3D_FEATURE_LEVEL feature_level)
         WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_10_1);
         WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_11_0);
         WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_11_1);
+        WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_12_0);
+        WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_12_1);
+        WINE_DXGI_TO_STR(D3D_FEATURE_LEVEL_12_2);
         default:
             FIXME("Unrecognized D3D_FEATURE_LEVEL %#x.\n", feature_level);
             return "unrecognized";
@@ -494,9 +498,6 @@ DXGI_USAGE dxgi_usage_from_wined3d_bind_flags(unsigned int wined3d_bind_flags)
     if (wined3d_bind_flags & WINED3D_BIND_UNORDERED_ACCESS)
         dxgi_usage |= DXGI_USAGE_UNORDERED_ACCESS;
 
-    wined3d_bind_flags &= ~(WINED3D_BIND_SHADER_RESOURCE | WINED3D_BIND_RENDER_TARGET | WINED3D_BIND_UNORDERED_ACCESS);
-    if (wined3d_bind_flags)
-        FIXME("Unhandled wined3d bind flags %#x.\n", wined3d_bind_flags);
     return dxgi_usage;
 }
 
@@ -575,7 +576,7 @@ HRESULT wined3d_swapchain_desc_from_dxgi(struct wined3d_swapchain_desc *wined3d_
 
     if (dxgi_desc->Scaling != DXGI_SCALING_STRETCH)
         FIXME("Ignoring scaling %#x.\n", dxgi_desc->Scaling);
-    if (dxgi_desc->AlphaMode != DXGI_ALPHA_MODE_IGNORE)
+    if (dxgi_desc->AlphaMode != DXGI_ALPHA_MODE_UNSPECIFIED && dxgi_desc->AlphaMode != DXGI_ALPHA_MODE_IGNORE)
         FIXME("Ignoring alpha mode %#x.\n", dxgi_desc->AlphaMode);
     if (dxgi_fullscreen_desc && dxgi_fullscreen_desc->ScanlineOrdering)
         FIXME("Unhandled scanline ordering %#x.\n", dxgi_fullscreen_desc->ScanlineOrdering);
