@@ -1732,6 +1732,7 @@ static void test_scans(void)
 static void test_getbounds(void)
 {
     GpRegion *region;
+    GpPath *path;
     GpGraphics *graphics;
     GpStatus status;
     GpRectF rectf;
@@ -1807,6 +1808,44 @@ static void test_getbounds(void)
     ok(rectf.Width  == 100.0, "Expected width = 0.0, got %.2f\n", rectf.Width);
     ok(rectf.Height == 100.0, "Expected height = 0.0, got %.2f\n", rectf.Height);
 
+    /* coordinates are not rounded */
+    status = GdipResetWorldTransform(graphics);
+    ok(status == Ok, "status %08x\n", status);
+    status = GdipResetPageTransform(graphics);
+    ok(status == Ok, "status %08x\n", status);
+    rectf.X = 0.125;
+    rectf.Y = 1.125;
+    rectf.Width = 2.125;
+    rectf.Height = 3.125;
+    status = GdipCombineRegionRect(region, &rectf, CombineModeReplace);
+    ok(status == Ok, "status %08x\n", status);
+    rectf.X = rectf.Y = 0.0;
+    rectf.Height = rectf.Width = 0.0;
+    status = GdipGetRegionBounds(region, graphics, &rectf);
+    ok(status == Ok, "status %08x\n", status);
+    ok(rectf.X == 0.125, "Expected X = 0.0, got %.2f\n", rectf.X);
+    ok(rectf.Y == 1.125, "Expected Y = 0.0, got %.2f\n", rectf.Y);
+    ok(rectf.Width  == 2.125, "Expected width = 0.0, got %.2f\n", rectf.Width);
+    ok(rectf.Height == 3.125, "Expected height = 0.0, got %.2f\n", rectf.Height);
+
+    /* test path */
+    status = GdipCreatePath(FillModeAlternate, &path);
+    ok(status == Ok, "status %08x\n", status);
+    status = GdipAddPathRectangle(path, 0.125, 1.125, 2.125, 3.125);
+    ok(status == Ok, "status %08x\n", status);
+    status = GdipCombineRegionPath(region, path, CombineModeReplace);
+    ok(status == Ok, "status %08x\n", status);
+    status = GdipDeletePath(path);
+    ok(status == Ok, "status %08x\n", status);
+    rectf.X = rectf.Y = 0.0;
+    rectf.Height = rectf.Width = 0.0;
+    status = GdipGetRegionBounds(region, graphics, &rectf);
+    ok(status == Ok, "status %08x\n", status);
+    ok(rectf.X == 0.125, "Expected X = 0.0, got %.2f\n", rectf.X);
+    ok(rectf.Y == 1.125, "Expected Y = 0.0, got %.2f\n", rectf.Y);
+    ok(rectf.Width  == 2.125, "Expected width = 0.0, got %.2f\n", rectf.Width);
+    ok(rectf.Height == 3.125, "Expected height = 0.0, got %.2f\n", rectf.Height);
+
     status = GdipDeleteRegion(region);
     ok(status == Ok, "status %08x\n", status);
     status = GdipDeleteGraphics(graphics);
@@ -2333,6 +2372,204 @@ static void test_GdipCreateRegionRgnData(void)
     ReleaseDC(0, hdc);
 }
 
+static void test_incombinedregion(void)
+{
+    struct testrgn
+    {
+        const char* desc;
+        BOOL origin_in_region;
+        GpRegion *region;
+    };
+
+    struct testrgn test_regions[] = {
+        { "infinite region", TRUE },
+        { "infinite region inverted", FALSE },
+        { "empty region", FALSE },
+        { "empty region inverted", TRUE },
+        { "inside rectangle", TRUE },
+        { "inside rectangle inverted", FALSE },
+        { "outside rectangle", FALSE },
+        { "outside rectangle inverted", TRUE },
+        { "inside path", TRUE },
+        { "inside path inverted", FALSE },
+        { "outside path but in bounding rect", FALSE },
+        { "outside path but in bounding rect inverted", TRUE },
+        { "outside path", FALSE },
+        { "outside path inverted", TRUE },
+    };
+
+    GpStatus stat;
+    GpRectF rect;
+    const GpPointF inside_path_points[] = { { -1, -2 }, { 2, 1 }, { -1, 1 } };
+    const GpPointF outside_path_bounding_points[] = { { -2, -1 }, { 1, 2 }, { -2, 2 } };
+    const GpPointF outside_path_points[] = { { 5, 5 }, { 5, 10 }, { 10, 10 } };
+    GpPath *path;
+    int i, j;
+    BOOL in_region;
+
+    /* Prepare test regions: */
+
+    /* infinite */
+    stat = GdipCreateRegion(&test_regions[0].region);
+    expect(Ok, stat);
+
+    /* empty */
+    stat = GdipCreateRegion(&test_regions[2].region);
+    expect(Ok, stat);
+    stat = GdipSetEmpty(test_regions[2].region);
+    expect(Ok, stat);
+
+    /* inside rectangle */
+    rect.X = -5;
+    rect.Y = -2;
+    rect.Width = 10;
+    rect.Height = 4;
+    stat = GdipCreateRegionRect(&rect, &test_regions[4].region);
+    expect(Ok, stat);
+
+    /* outside rectangle */
+    rect.X = -10;
+    rect.Y = -10;
+    rect.Width = 7;
+    rect.Height = 7;
+    stat = GdipCreateRegionRect(&rect, &test_regions[6].region);
+    expect(Ok, stat);
+
+    /* inside path */
+    stat = GdipCreatePath(FillModeAlternate, &path);
+    expect(Ok, stat);
+    stat = GdipAddPathPolygon(path, inside_path_points, ARRAY_SIZE(inside_path_points));
+    expect(Ok, stat);
+    stat = GdipCreateRegion(&test_regions[8].region);
+    expect(Ok, stat);
+    stat = GdipCombineRegionPath(test_regions[8].region, path, CombineModeReplace);
+    expect(Ok, stat);
+    stat = GdipDeletePath(path);
+    expect(Ok, stat);
+
+    /* outside path but in bounding rect */
+    stat = GdipCreatePath(FillModeAlternate, &path);
+    expect(Ok, stat);
+    stat = GdipAddPathPolygon(path, outside_path_bounding_points, ARRAY_SIZE(outside_path_bounding_points));
+    expect(Ok, stat);
+    stat = GdipCreateRegion(&test_regions[10].region);
+    expect(Ok, stat);
+    stat = GdipCombineRegionPath(test_regions[10].region, path, CombineModeReplace);
+    expect(Ok, stat);
+    stat = GdipDeletePath(path);
+    expect(Ok, stat);
+
+    /* outside path */
+    stat = GdipCreatePath(FillModeAlternate, &path);
+    expect(Ok, stat);
+    stat = GdipAddPathPolygon(path, outside_path_points, ARRAY_SIZE(outside_path_points));
+    expect(Ok, stat);
+    stat = GdipCreateRegion(&test_regions[12].region);
+    expect(Ok, stat);
+    stat = GdipCombineRegionPath(test_regions[12].region, path, CombineModeReplace);
+    expect(Ok, stat);
+    stat = GdipDeletePath(path);
+    expect(Ok, stat);
+
+    for (i = 1; i < ARRAY_SIZE(test_regions); i += 2)
+    {
+        winetest_push_context("%s", test_regions[i].desc);
+        stat = GdipCreateRegion(&test_regions[i].region);
+        expect(Ok, stat);
+        stat = GdipCombineRegionRegion(test_regions[i].region, test_regions[i-1].region, CombineModeExclude);
+        expect(Ok, stat);
+        winetest_pop_context();
+    }
+
+    /* Check regions individually */
+    for (i = 0; i < ARRAY_SIZE(test_regions); i++)
+    {
+        winetest_push_context("%s", test_regions[i].desc);
+        stat = GdipIsVisibleRegionPoint(test_regions[i].region, 0, 0, NULL, &in_region);
+        expect(Ok, stat);
+        expect(test_regions[i].origin_in_region, in_region);
+        winetest_pop_context();
+    }
+
+    /* Check combined regions */
+    for (i = 0; i < ARRAY_SIZE(test_regions); i++)
+    {
+        for (j = 0; j < ARRAY_SIZE(test_regions); j++)
+        {
+            GpRegion *region;
+            BOOL expected_result;
+
+            winetest_push_context("%s + %s", test_regions[i].desc, test_regions[j].desc);
+
+            stat = GdipCreateRegion(&region);
+            expect(Ok, stat);
+
+            /* CombineModeIntersect */
+            stat = GdipCombineRegionRegion(region, test_regions[i].region, CombineModeReplace);
+            expect(Ok, stat);
+            stat = GdipCombineRegionRegion(region, test_regions[j].region, CombineModeIntersect);
+            expect(Ok, stat);
+
+            expected_result = test_regions[i].origin_in_region & test_regions[j].origin_in_region;
+            stat = GdipIsVisibleRegionPoint(region, 0, 0, NULL, &in_region);
+            expect(Ok, stat);
+            ok(expected_result == in_region, "CombineModeIntersect: expected %i, got %i\n", expected_result, in_region);
+
+            /* CombineModeUnion */
+            stat = GdipCombineRegionRegion(region, test_regions[i].region, CombineModeReplace);
+            expect(Ok, stat);
+            stat = GdipCombineRegionRegion(region, test_regions[j].region, CombineModeUnion);
+            expect(Ok, stat);
+
+            expected_result = test_regions[i].origin_in_region | test_regions[j].origin_in_region;
+            stat = GdipIsVisibleRegionPoint(region, 0, 0, NULL, &in_region);
+            expect(Ok, stat);
+            ok(expected_result == in_region, "CombineModeUnion: expected %i, got %i\n", expected_result, in_region);
+
+            /* CombineModeXor */
+            stat = GdipCombineRegionRegion(region, test_regions[i].region, CombineModeReplace);
+            expect(Ok, stat);
+            stat = GdipCombineRegionRegion(region, test_regions[j].region, CombineModeXor);
+            expect(Ok, stat);
+
+            expected_result = test_regions[i].origin_in_region ^ test_regions[j].origin_in_region;
+            stat = GdipIsVisibleRegionPoint(region, 0, 0, NULL, &in_region);
+            expect(Ok, stat);
+            ok(expected_result == in_region, "CombineModeXor: expected %i, got %i\n", expected_result, in_region);
+
+            /* CombineModeExclude */
+            stat = GdipCombineRegionRegion(region, test_regions[i].region, CombineModeReplace);
+            expect(Ok, stat);
+            stat = GdipCombineRegionRegion(region, test_regions[j].region, CombineModeExclude);
+            expect(Ok, stat);
+
+            expected_result = test_regions[i].origin_in_region & !test_regions[j].origin_in_region;
+            stat = GdipIsVisibleRegionPoint(region, 0, 0, NULL, &in_region);
+            expect(Ok, stat);
+            ok(expected_result == in_region, "CombineModeExclude: expected %i, got %i\n", expected_result, in_region);
+
+            /* CombineModeComplement */
+            stat = GdipCombineRegionRegion(region, test_regions[i].region, CombineModeReplace);
+            expect(Ok, stat);
+            stat = GdipCombineRegionRegion(region, test_regions[j].region, CombineModeComplement);
+            expect(Ok, stat);
+
+            expected_result = (!test_regions[i].origin_in_region) & test_regions[j].origin_in_region;
+            stat = GdipIsVisibleRegionPoint(region, 0, 0, NULL, &in_region);
+            expect(Ok, stat);
+            ok(expected_result == in_region, "CombineModeComplement: expected %i, got %i\n", expected_result, in_region);
+
+            winetest_pop_context();
+        }
+    }
+
+    for (i = 0; i < ARRAY_SIZE(test_regions); i++)
+    {
+        stat = GdipDeleteRegion(test_regions[i].region);
+        expect(Ok, stat);
+    }
+}
+
 START_TEST(region)
 {
     struct GdiplusStartupInput gdiplusStartupInput;
@@ -2367,6 +2604,7 @@ START_TEST(region)
     test_isvisiblerect();
     test_excludeinfinite();
     test_GdipCreateRegionRgnData();
+    test_incombinedregion();
 
     GdiplusShutdown(gdiplusToken);
 }
