@@ -19,8 +19,6 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
-
 #include "initguid.h"
 #include "windows.h"
 #include "ole2.h"
@@ -204,7 +202,7 @@ static void test_CreateBody(void)
     ok(hr == S_OK, "ret %08lx\n", hr);
     off.QuadPart = 0;
     IStream_Seek(in, off, STREAM_SEEK_CUR, &pos);
-    ok(pos.u.LowPart == 359, "pos %lu\n", pos.u.LowPart);
+    ok(pos.LowPart == 359, "pos %lu\n", pos.LowPart);
 
     hr = IMimeBody_IsContentType(body, "multipart", "mixed");
     ok(hr == S_OK, "ret %08lx\n", hr);
@@ -347,7 +345,7 @@ static HRESULT WINAPI Stream_Seek(IStream *iface, LARGE_INTEGER dlibMove, DWORD
 
     if(dwOrigin == STREAM_SEEK_END) {
         CHECK_EXPECT(Stream_Seek_END);
-        ok(dlibMove.QuadPart == expect_seek_pos, "unexpected seek pos %lu\n", dlibMove.u.LowPart);
+        ok(dlibMove.QuadPart == expect_seek_pos, "unexpected seek pos %lu\n", dlibMove.LowPart);
         if(plibNewPosition)
             plibNewPosition->QuadPart = 10;
         return S_OK;
@@ -355,7 +353,7 @@ static HRESULT WINAPI Stream_Seek(IStream *iface, LARGE_INTEGER dlibMove, DWORD
 
     CHECK_EXPECT(Stream_Seek);
 
-    ok(dlibMove.QuadPart == expect_seek_pos, "unexpected seek pos %lu\n", dlibMove.u.LowPart);
+    ok(dlibMove.QuadPart == expect_seek_pos, "unexpected seek pos %lu\n", dlibMove.LowPart);
     ok(dwOrigin == STREAM_SEEK_SET, "dwOrigin = %ld\n", dwOrigin);
     This->pos = dlibMove.QuadPart;
     if(plibNewPosition)
