@@ -87,7 +87,6 @@ static void Enumerator_destructor(jsdisp_t *dispex)
     if(This->enumvar)
         IEnumVARIANT_Release(This->enumvar);
     jsval_release(This->item);
-    free(dispex);
 }
 
 static HRESULT Enumerator_gc_traverse(struct gc_ctx *gc_ctx, enum gc_traverse_op op, jsdisp_t *dispex)
@@ -182,25 +181,15 @@ static const builtin_prop_t Enumerator_props[] = {
 };
 
 static const builtin_info_t Enumerator_info = {
-    JSCLASS_ENUMERATOR,
-    NULL,
-    ARRAY_SIZE(Enumerator_props),
-    Enumerator_props,
-    NULL,
-    NULL
+    .class     = JSCLASS_ENUMERATOR,
+    .props_cnt = ARRAY_SIZE(Enumerator_props),
+    .props     = Enumerator_props,
 };
 
 static const builtin_info_t EnumeratorInst_info = {
-    JSCLASS_ENUMERATOR,
-    NULL,
-    0,
-    NULL,
-    Enumerator_destructor,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    Enumerator_gc_traverse
+    .class       = JSCLASS_ENUMERATOR,
+    .destructor  = Enumerator_destructor,
+    .gc_traverse = Enumerator_gc_traverse
 };
 
 static HRESULT alloc_enumerator(script_ctx_t *ctx, jsdisp_t *object_prototype, EnumeratorInstance **ret)
@@ -251,11 +240,11 @@ static HRESULT create_enumerator(script_ctx_t *ctx, jsval_t *argv, jsdisp_t **re
         /* Try to get a IEnumVARIANT by _NewEnum */
         VariantInit(&varresult);
         hres = IDispatch_Invoke(obj, DISPID_NEWENUM, &IID_NULL, LOCALE_NEUTRAL,
-                DISPATCH_METHOD, &dispparams, &varresult, NULL, NULL);
+                DISPATCH_METHOD | DISPATCH_PROPERTYGET, &dispparams, &varresult, NULL, NULL);
         if (FAILED(hres))
         {
             WARN("Enumerator: no DISPID_NEWENUM.\n");
-            return E_INVALIDARG;
+            return JS_E_OBJECT_NOT_COLLECTION;
         }
 
         if ((V_VT(&varresult) == VT_DISPATCH) || (V_VT(&varresult) == VT_UNKNOWN))
@@ -266,7 +255,7 @@ static HRESULT create_enumerator(script_ctx_t *ctx, jsval_t *argv, jsdisp_t **re
         else
         {
             FIXME("Enumerator: NewEnum unexpected type of varresult (%d).\n", V_VT(&varresult));
-            hres = E_INVALIDARG;
+            hres = JS_E_OBJECT_NOT_COLLECTION;
         }
         VariantClear(&varresult);
         if (FAILED(hres))
@@ -324,12 +313,8 @@ static HRESULT EnumeratorConstr_value(script_ctx_t *ctx, jsval_t vthis, WORD fla
 }
 
 static const builtin_info_t EnumeratorConstr_info = {
-    JSCLASS_FUNCTION,
-    Function_value,
-    0,
-    NULL,
-    NULL,
-    NULL
+    .class = JSCLASS_FUNCTION,
+    .call  = Function_value,
 };
 
 HRESULT create_enumerator_constr(script_ctx_t *ctx, jsdisp_t *object_prototype, jsdisp_t **ret)
