@@ -491,12 +491,8 @@ static const builtin_prop_t Math_props[] = {
 };
 
 static const builtin_info_t Math_info = {
-    JSCLASS_MATH,
-    NULL,
-    ARRAY_SIZE(Math_props),
-    Math_props,
-    NULL,
-    NULL
+    .class = JSCLASS_MATH,    .props_cnt = ARRAY_SIZE(Math_props),
+    .props = Math_props,
 };
 
 HRESULT create_math(script_ctx_t *ctx, jsdisp_t **ret)
