@@ -239,7 +239,7 @@ static HRESULT Number_toString(script_ctx_t *ctx, jsval_t vthis, WORD flags, uns
     if(FAILED(hres))
         return hres;
 
-    if(argc) {
+    if(argc && (ctx->version < SCRIPTLANGUAGEVERSION_ES5 || !is_undefined(argv[0]))) {
         hres = to_int32(ctx, argv[0], &radix);
         if(FAILED(hres))
             return hres;
@@ -525,7 +525,7 @@ static HRESULT Number_toPrecision(script_ctx_t *ctx, jsval_t vthis, WORD flags,
     if(FAILED(hres))
         return hres;
 
-    if(argc) {
+    if(argc && (ctx->version < 2 || !is_undefined(argv[0]))) {
         hres = to_int32(ctx, argv[0], &prec);
         if(FAILED(hres))
             return hres;
@@ -586,20 +586,13 @@ static const builtin_prop_t Number_props[] = {
 };
 
 static const builtin_info_t Number_info = {
-    JSCLASS_NUMBER,
-    NULL,
-    ARRAY_SIZE(Number_props),
-    Number_props,
-    NULL,
-    NULL
+    .class     = JSCLASS_NUMBER,
+    .props_cnt = ARRAY_SIZE(Number_props),
+    .props     = Number_props,
 };
 
 static const builtin_info_t NumberInst_info = {
-    JSCLASS_NUMBER,
-    NULL,
-    0, NULL,
-    NULL,
-    NULL
+    .class = JSCLASS_NUMBER,
 };
 
 static HRESULT NumberConstr_value(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned argc, jsval_t *argv,
