@@ -235,7 +235,6 @@ static void VBArray_destructor(jsdisp_t *dispex)
     VBArrayInstance *vbarray = vbarray_from_jsdisp(dispex);
 
     SafeArrayDestroy(vbarray->safearray);
-    free(vbarray);
 }
 
 static const builtin_prop_t VBArray_props[] = {
@@ -247,12 +246,11 @@ static const builtin_prop_t VBArray_props[] = {
 };
 
 static const builtin_info_t VBArray_info = {
-    JSCLASS_VBARRAY,
-    VBArray_value,
-    ARRAY_SIZE(VBArray_props),
-    VBArray_props,
-    VBArray_destructor,
-    NULL
+    .class      = JSCLASS_VBARRAY,
+    .call       = VBArray_value,
+    .props_cnt  = ARRAY_SIZE(VBArray_props),
+    .props      = VBArray_props,
+    .destructor = VBArray_destructor,
 };
 
 static HRESULT alloc_vbarray(script_ctx_t *ctx, jsdisp_t *object_prototype, VBArrayInstance **ret)
