@@ -42,10 +42,60 @@ WINE_DECLARE_DEBUG_CHANNEL(virtual);
 WINE_DECLARE_DEBUG_CHANNEL(globalmem);
 
 
+
+static CROSS_PROCESS_WORK_LIST *open_cross_process_connection( HANDLE process )
+{
+#ifdef __aarch64__
+    CROSS_PROCESS_WORK_LIST *list;
+    HANDLE section;
+
+    RtlOpenCrossProcessEmulatorWorkConnection( process, &section, (void **)&list );
+    if (section) NtClose( section );
+    return list;
+#else
+    return NULL;
+#endif
+}
+
+static void close_cross_process_connection( CROSS_PROCESS_WORK_LIST *list )
+{
+    if (list) NtUnmapViewOfSection( GetCurrentProcess(), list );
+}
+
+static void send_cross_process_notification( CROSS_PROCESS_WORK_LIST *list, UINT id,
+                                             const void *addr, SIZE_T size, int nb_args, ... )
+{
+#ifdef __aarch64__
+    CROSS_PROCESS_WORK_ENTRY *entry;
+    void *unused;
+    va_list args;
+    int i;
+
+    if (!list) return;
+    if ((entry = RtlWow64PopCrossProcessWorkFromFreeList( &list->free_list )))
+    {
+        entry->id = id;
+        entry->addr = (ULONG_PTR)addr;
+        entry->size = size;
+        if (nb_args)
+        {
+            va_start( args, nb_args );
+            for (i = 0; i < nb_args; i++) entry->args[i] = va_arg( args, int );
+            va_end( args );
+        }
+        RtlWow64PushCrossProcessWorkOntoWorkList( &list->work_list, entry, &unused );
+    }
+#endif
+}
+
+
 /***********************************************************************
  * Virtual memory functions
  ***********************************************************************/
 
+static const SIZE_T page_mask = 0xfff;
+#define ROUND_ADDR(addr) ((void *)((UINT_PTR)(addr) & ~page_mask))
+#define ROUND_SIZE(addr,size) (((SIZE_T)(size) + ((UINT_PTR)(addr) & page_mask) + page_mask) & ~page_mask)
 
 /***********************************************************************
  *             DiscardVirtualMemory   (kernelbase.@)
@@ -72,6 +122,22 @@ BOOL WINAPI DECLSPEC_HOTPATCH FlushViewOfFile( const void *base, SIZE_T size )
 }
 
 
+/****************************************************************************
+ *           FlushInstructionCache   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH FlushInstructionCache( HANDLE process, LPCVOID addr, SIZE_T size )
+{
+    CROSS_PROCESS_WORK_LIST *list;
+
+    if ((list = open_cross_process_connection( process )))
+    {
+        send_cross_process_notification( list, CrossProcessFlushCache, addr, size, 0 );
+        close_cross_process_connection( list );
+    }
+    return set_ntstatus( NtFlushInstructionCache( process, addr, size ));
+}
+
+
 /***********************************************************************
  *          GetLargePageMinimum   (kernelbase.@)
  */
@@ -537,7 +603,69 @@ BOOL WINAPI DECLSPEC_HOTPATCH VirtualUnlock( void *addr, SIZE_T size )
 BOOL WINAPI DECLSPEC_HOTPATCH WriteProcessMemory( HANDLE process, void *addr, const void *buffer,
                                                   SIZE_T size, SIZE_T *bytes_written )
 {
-    return set_ntstatus( NtWriteVirtualMemory( process, addr, buffer, size, bytes_written ));
+    CROSS_PROCESS_WORK_LIST *list = open_cross_process_connection( process );
+    DWORD old_prot, prot = PAGE_TARGETS_NO_UPDATE | PAGE_ENCLAVE_NO_CHANGE;
+    MEMORY_BASIC_INFORMATION info;
+    void *base_addr;
+    SIZE_T region_size;
+    NTSTATUS status, status2;
+
+    if (!VirtualQueryEx( process, addr, &info, sizeof(info) ))
+    {
+        close_cross_process_connection( list );
+        return FALSE;
+    }
+
+    switch (info.Protect & ~(PAGE_GUARD | PAGE_NOCACHE))
+    {
+    case PAGE_READWRITE:
+    case PAGE_WRITECOPY:
+    case PAGE_EXECUTE_READWRITE:
+    case PAGE_EXECUTE_WRITECOPY:
+        /* already writable */
+        if ((status = NtWriteVirtualMemory( process, addr, buffer, size, bytes_written ))) break;
+        send_cross_process_notification( list, CrossProcessFlushCache, addr, size, 0 );
+        NtFlushInstructionCache( process, addr, size );
+        break;
+
+    case PAGE_EXECUTE:
+    case PAGE_EXECUTE_READ:
+        /* make it writable */
+        base_addr = ROUND_ADDR( addr );
+        region_size = ROUND_SIZE( addr, size );
+        region_size = min( region_size,  (char *)info.BaseAddress + info.RegionSize - (char *)base_addr );
+        prot |= (info.Type == MEM_PRIVATE) ? PAGE_EXECUTE_READWRITE : PAGE_EXECUTE_WRITECOPY;
+
+        send_cross_process_notification( list, CrossProcessPreVirtualProtect,
+                                         base_addr, region_size, 1, prot );
+        status = NtProtectVirtualMemory( process, &base_addr, &region_size, prot, &old_prot );
+        send_cross_process_notification( list, CrossProcessPostVirtualProtect,
+                                         base_addr, region_size, 2, prot, status );
+        if (status) break;
+
+        status = NtWriteVirtualMemory( process, addr, buffer, size, bytes_written );
+        if (!status)
+        {
+            send_cross_process_notification( list, CrossProcessFlushCache, addr, size, 0 );
+            NtFlushInstructionCache( process, addr, size );
+        }
+
+        prot = PAGE_TARGETS_NO_UPDATE | PAGE_ENCLAVE_NO_CHANGE | old_prot;
+        send_cross_process_notification( list, CrossProcessPreVirtualProtect,
+                                         base_addr, region_size, 1, prot );
+        status2 = NtProtectVirtualMemory( process, &base_addr, &region_size, prot, &old_prot );
+        send_cross_process_notification( list, CrossProcessPostVirtualProtect,
+                                         base_addr, region_size, 2, prot, status2 );
+        break;
+
+    default:
+        /* not writable */
+        status = STATUS_ACCESS_VIOLATION;
+        break;
+    }
+
+    close_cross_process_connection( list );
+    return set_ntstatus( status );
 }
 
 
@@ -1657,26 +1785,12 @@ BOOL WINAPI GetXStateFeaturesMask( CONTEXT *context, DWORD64 *feature_mask )
  * Firmware functions
  ***********************************************************************/
 
-
-/***********************************************************************
- *             EnumSystemFirmwareTable   (kernelbase.@)
- */
-UINT WINAPI EnumSystemFirmwareTables( DWORD provider, void *buffer, DWORD size )
-{
-    FIXME( "(0x%08lx, %p, %ld)\n", provider, buffer, size );
-    return 0;
-}
-
-
-/***********************************************************************
- *             GetSystemFirmwareTable   (kernelbase.@)
- */
-UINT WINAPI GetSystemFirmwareTable( DWORD provider, DWORD id, void *buffer, DWORD size )
+static UINT get_firmware_table( DWORD provider, SYSTEM_FIRMWARE_TABLE_ACTION action, DWORD id,
+                                void *buffer, DWORD size )
 {
     SYSTEM_FIRMWARE_TABLE_INFORMATION *info;
     ULONG buffer_size = offsetof( SYSTEM_FIRMWARE_TABLE_INFORMATION, TableBuffer ) + size;
-
-    TRACE( "(0x%08lx, 0x%08lx, %p, %ld)\n", provider, id, buffer, size );
+    NTSTATUS status;
 
     if (!(info = RtlAllocateHeap( GetProcessHeap(), 0, buffer_size )))
     {
@@ -1685,14 +1799,34 @@ UINT WINAPI GetSystemFirmwareTable( DWORD provider, DWORD id, void *buffer, DWOR
     }
 
     info->ProviderSignature = provider;
-    info->Action = SystemFirmwareTable_Get;
+    info->Action = action;
     info->TableID = id;
 
-    set_ntstatus( NtQuerySystemInformation( SystemFirmwareTableInformation,
-                                            info, buffer_size, &buffer_size ));
+    status = NtQuerySystemInformation( SystemFirmwareTableInformation, info, buffer_size, &buffer_size );
+    set_ntstatus(status);
     buffer_size -= offsetof( SYSTEM_FIRMWARE_TABLE_INFORMATION, TableBuffer );
     if (buffer_size <= size) memcpy( buffer, info->TableBuffer, buffer_size );
 
     HeapFree( GetProcessHeap(), 0, info );
-    return buffer_size;
+    return NT_SUCCESS(status) || status == STATUS_BUFFER_TOO_SMALL ? buffer_size : 0;
+}
+
+/***********************************************************************
+ *             EnumSystemFirmwareTables   (kernelbase.@)
+ */
+UINT WINAPI EnumSystemFirmwareTables( DWORD provider, void *buffer, DWORD size )
+{
+    TRACE( "(0x%08lx, %p, %ld)\n", provider, buffer, size );
+
+    return get_firmware_table( provider, SystemFirmwareTable_Enumerate, 0, buffer, size );
+}
+
+/***********************************************************************
+ *             GetSystemFirmwareTable   (kernelbase.@)
+ */
+UINT WINAPI GetSystemFirmwareTable( DWORD provider, DWORD id, void *buffer, DWORD size )
+{
+    TRACE( "(0x%08lx, 0x%08lx, %p, %ld)\n", provider, id, buffer, size );
+
+    return get_firmware_table( provider, SystemFirmwareTable_Get, id, buffer, size );
 }
