@@ -199,8 +199,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH GetVolumeInformationW( LPCWSTR root, LPWSTR label,
     if (status)
     {
         TRACE( "cannot open device %s: %lx\n", debugstr_w(nt_name.Buffer), status );
-        if (status == STATUS_ACCESS_DENIED)
-            MESSAGE( "wine: Read access denied for device %s, FS volume label and serial are not available.\n", debugstr_w(nt_name.Buffer) );
         status = NtOpenFile( &handle, SYNCHRONIZE, &attr, &io, 0,
                                       FILE_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT );
     }
@@ -400,7 +398,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH DefineDosDeviceW( DWORD flags, const WCHAR *device
     InitializeObjectAttributes( &attr, &nt_name, OBJ_CASE_INSENSITIVE | OBJ_PERMANENT, 0, NULL );
     if (flags & DDD_REMOVE_DEFINITION)
     {
-        if (!set_ntstatus( NtOpenSymbolicLinkObject( &handle, 0, &attr ) ))
+        if (!set_ntstatus( NtOpenSymbolicLinkObject( &handle, DELETE, &attr ) ))
             return FALSE;
 
         status = NtMakeTemporaryObject( handle );
@@ -433,7 +431,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH DefineDosDeviceW( DWORD flags, const WCHAR *device
  */
 DWORD WINAPI QueryDosDeviceW( LPCWSTR devname, LPWSTR target, DWORD bufsize )
 {
-    UNICODE_STRING nt_name;
     NTSTATUS status;
 
     if (!bufsize)
@@ -471,12 +468,11 @@ DWORD WINAPI QueryDosDeviceW( LPCWSTR devname, LPWSTR target, DWORD bufsize )
     }
     else  /* return a list of all devices */
     {
+        UNICODE_STRING nt_name = RTL_CONSTANT_STRING( L"\\DosDevices" );
         OBJECT_ATTRIBUTES attr;
         HANDLE handle;
         WCHAR *p = target;
 
-        RtlInitUnicodeString( &nt_name, L"\\DosDevices\\" );
-        nt_name.Length -= sizeof(WCHAR);  /* without trailing slash */
         attr.Length = sizeof(attr);
         attr.RootDirectory = 0;
         attr.ObjectName = &nt_name;
@@ -517,12 +513,11 @@ DWORD WINAPI QueryDosDeviceW( LPCWSTR devname, LPWSTR target, DWORD bufsize )
 DWORD WINAPI DECLSPEC_HOTPATCH GetLogicalDrives(void)
 {
     OBJECT_ATTRIBUTES attr;
-    UNICODE_STRING nt_name;
+    UNICODE_STRING nt_name = RTL_CONSTANT_STRING( L"\\DosDevices\\" );
     DWORD bitmask = 0;
     NTSTATUS status;
     HANDLE handle;
 
-    RtlInitUnicodeString( &nt_name, L"\\DosDevices\\" );
     nt_name.Length -= sizeof(WCHAR);  /* without trailing slash */
     attr.Length = sizeof(attr);
     attr.RootDirectory = 0;
