@@ -65,6 +65,7 @@ static void appeared_callback( DADiskRef disk, void *context )
     CFDictionaryRef dict = DADiskCopyDescription( disk );
     const void *ref;
     char device[64];
+    CFURLRef volume_url;
     char mount_point[PATH_MAX];
     size_t model_len = 0;
     GUID guid, *guid_ptr = NULL;
@@ -87,10 +88,25 @@ static void appeared_callback( DADiskRef disk, void *context )
     strcpy( device, "/dev/r" );
     CFStringGetCString( ref, device + 6, sizeof(device) - 6, kCFStringEncodingASCII );
 
-    if ((ref = CFDictionaryGetValue( dict, CFSTR("DAVolumePath") )))
-        CFURLGetFileSystemRepresentation( ref, true, (UInt8 *)mount_point, sizeof(mount_point) );
+    if ((volume_url = CFDictionaryGetValue( dict, CFSTR("DAVolumePath") )))
+        CFURLGetFileSystemRepresentation( volume_url, true, (UInt8 *)mount_point, sizeof(mount_point) );
     else
-        mount_point[0] = 0;
+    {
+        TRACE( "ignoring volume %s, uuid %s: no macOS volume path\n", device, wine_dbgstr_guid(guid_ptr) );
+        goto done;
+    }
+
+    if (CFURLCopyResourcePropertyForKey( volume_url, kCFURLVolumeIsBrowsableKey, &ref, NULL ))
+    {
+        Boolean is_browsable = CFBooleanGetValue( ref );
+        CFRelease( ref );
+
+        if (!is_browsable)
+        {
+            TRACE( "ignoring volume %s, uuid %s: not browsable\n", device, wine_dbgstr_guid(guid_ptr) );
+            goto done;
+        }
+    }
 
     if ((ref = CFDictionaryGetValue( dict, CFSTR("DAMediaKind") )))
     {
@@ -164,9 +180,9 @@ static void appeared_callback( DADiskRef disk, void *context )
     }
 
     if (removable)
-        queue_device_op( ADD_DOS_DEVICE, device, device, mount_point, type, guid_ptr, NULL, &scsi_info );
+        queue_device_op( ADD_DOS_DEVICE, device, device, mount_point, type, guid_ptr, NULL, NULL, &scsi_info );
     else
-        if (guid_ptr) queue_device_op( ADD_VOLUME, device, device, mount_point, DEVICE_HARDDISK_VOL, guid_ptr, NULL, &scsi_info );
+        if (guid_ptr) queue_device_op( ADD_VOLUME, device, device, mount_point, DEVICE_HARDDISK_VOL, guid_ptr, NULL, NULL, &scsi_info );
 
 done:
     CFRelease( dict );
@@ -192,7 +208,7 @@ static void disappeared_callback( DADiskRef disk, void *context )
 
     TRACE( "got unmount notification for '%s'\n", device );
 
-    queue_device_op( REMOVE_DEVICE, device, NULL, NULL, 0, NULL, NULL, NULL );
+    queue_device_op( REMOVE_DEVICE, device, NULL, NULL, 0, NULL, NULL, NULL, NULL );
 
 done:
     CFRelease( dict );
