@@ -45,7 +45,7 @@ extern HRESULT assembly_get_runtime_version(ASSEMBLY *assembly, LPSTR *version);
 extern HRESULT assembly_get_vtable_fixups(ASSEMBLY *assembly, VTableFixup **fixups, DWORD *count);
 extern HRESULT assembly_get_native_entrypoint(ASSEMBLY *assembly, NativeEntryPointFunc *func);
 
-#define WINE_MONO_VERSION "8.1.0"
+#define WINE_MONO_VERSION "9.4.0"
 
 /* Mono embedding */
 typedef struct _MonoDomain MonoDomain;
@@ -153,6 +153,7 @@ typedef MonoAssembly* (CDECL *WineMonoAssemblyPreLoadFunc)(MonoAssemblyName *ana
 typedef void (CDECL *MonoProfileFunc)(MonoProfiler *prof);
 
 typedef void (CDECL *MonoPrintCallback) (const char *string, INT is_stdout);
+typedef void (*MonoLogCallback) (const char *log_domain, const char *log_level, const char *message, INT fatal, void *user_data);
 
 typedef enum {
     MONO_AOT_MODE_NONE,
@@ -227,5 +228,6 @@ extern void runtimehost_init(void);
 extern void runtimehost_uninit(void);
 
 extern void CDECL mono_print_handler_fn(const char *string, INT is_stdout);
+extern void CDECL mono_log_handler_fn(const char *log_domain, const char *log_level, const char *message, INT fatal, void *user_data);
 
 #endif   /* __MSCOREE_PRIVATE__ */
