@@ -24,6 +24,7 @@
 #include "winbase.h"
 #include "winuser.h"
 #include "ole2.h"
+#include "mshtmdid.h"
 
 #include "mshtml_private.h"
 #include "htmlstyle.h"
@@ -38,6 +39,7 @@ struct HTMLCurrentStyle {
     IHTMLCurrentStyle2 IHTMLCurrentStyle2_iface;
     IHTMLCurrentStyle3 IHTMLCurrentStyle3_iface;
     IHTMLCurrentStyle4 IHTMLCurrentStyle4_iface;
+    IWineCSSProperties IWineCSSProperties_iface;
 
     HTMLElement *elem;
 };
@@ -72,56 +74,8 @@ static inline HTMLCurrentStyle *impl_from_IHTMLCurrentStyle4(IHTMLCurrentStyle4
     return CONTAINING_RECORD(iface, HTMLCurrentStyle, IHTMLCurrentStyle4_iface);
 }
 
-static HRESULT WINAPI HTMLCurrentStyle_QueryInterface(IHTMLCurrentStyle *iface, REFIID riid, void **ppv)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-    return IHTMLCSSStyleDeclaration_QueryInterface(&This->css_style.IHTMLCSSStyleDeclaration_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLCurrentStyle_AddRef(IHTMLCurrentStyle *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    TRACE("(%p)\n", This);
-    return IHTMLCSSStyleDeclaration_AddRef(&This->css_style.IHTMLCSSStyleDeclaration_iface);
-}
-
-static ULONG WINAPI HTMLCurrentStyle_Release(IHTMLCurrentStyle *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    TRACE("(%p)\n", This);
-    return IHTMLCSSStyleDeclaration_Release(&This->css_style.IHTMLCSSStyleDeclaration_iface);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle_GetTypeInfoCount(IHTMLCurrentStyle *iface, UINT *pctinfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->css_style.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle_GetTypeInfo(IHTMLCurrentStyle *iface, UINT iTInfo,
-        LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    return IDispatchEx_GetTypeInfo(&This->css_style.dispex.IDispatchEx_iface, iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle_GetIDsOfNames(IHTMLCurrentStyle *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    return IDispatchEx_GetIDsOfNames(&This->css_style.dispex.IDispatchEx_iface, riid, rgszNames, cNames,
-            lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle_Invoke(IHTMLCurrentStyle *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle(iface);
-    return IDispatchEx_Invoke(&This->css_style.dispex.IDispatchEx_iface, dispIdMember, riid, lcid,
-            wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLCurrentStyle, IHTMLCurrentStyle,
+                      impl_from_IHTMLCurrentStyle(iface)->css_style.dispex)
 
 static HRESULT WINAPI HTMLCurrentStyle_get_position(IHTMLCurrentStyle *iface, BSTR *p)
 {
@@ -872,56 +826,8 @@ static const IHTMLCurrentStyleVtbl HTMLCurrentStyleVtbl = {
     HTMLCurrentStyle_get_textTransform
 };
 
-/* IHTMLCurrentStyle2 */
-static HRESULT WINAPI HTMLCurrentStyle2_QueryInterface(IHTMLCurrentStyle2 *iface, REFIID riid, void **ppv)
-{
-   HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-
-   return IHTMLCurrentStyle_QueryInterface(&This->IHTMLCurrentStyle_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLCurrentStyle2_AddRef(IHTMLCurrentStyle2 *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-
-    return IHTMLCurrentStyle_AddRef(&This->IHTMLCurrentStyle_iface);
-}
-
-static ULONG WINAPI HTMLCurrentStyle2_Release(IHTMLCurrentStyle2 *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-    return IHTMLCurrentStyle_Release(&This->IHTMLCurrentStyle_iface);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle2_GetTypeInfoCount(IHTMLCurrentStyle2 *iface, UINT *pctinfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->css_style.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle2_GetTypeInfo(IHTMLCurrentStyle2 *iface, UINT iTInfo,
-        LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-    return IDispatchEx_GetTypeInfo(&This->css_style.dispex.IDispatchEx_iface, iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle2_GetIDsOfNames(IHTMLCurrentStyle2 *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-    return IDispatchEx_GetIDsOfNames(&This->css_style.dispex.IDispatchEx_iface, riid, rgszNames, cNames,
-            lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle2_Invoke(IHTMLCurrentStyle2 *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle2(iface);
-    return IDispatchEx_Invoke(&This->css_style.dispex.IDispatchEx_iface, dispIdMember, riid, lcid,
-            wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLCurrentStyle2, IHTMLCurrentStyle2,
+                      impl_from_IHTMLCurrentStyle2(iface)->css_style.dispex)
 
 static HRESULT WINAPI HTMLCurrentStyle2_get_layoutFlow(IHTMLCurrentStyle2 *iface, BSTR *p)
 {
@@ -1090,56 +996,8 @@ static const IHTMLCurrentStyle2Vtbl HTMLCurrentStyle2Vtbl = {
     HTMLCurrentStyle2_get_isBlock
 };
 
-/* IHTMLCurrentStyle3 */
-static HRESULT WINAPI HTMLCurrentStyle3_QueryInterface(IHTMLCurrentStyle3 *iface, REFIID riid, void **ppv)
-{
-   HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-
-   return IHTMLCurrentStyle_QueryInterface(&This->IHTMLCurrentStyle_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLCurrentStyle3_AddRef(IHTMLCurrentStyle3 *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-
-    return IHTMLCurrentStyle_AddRef(&This->IHTMLCurrentStyle_iface);
-}
-
-static ULONG WINAPI HTMLCurrentStyle3_Release(IHTMLCurrentStyle3 *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-    return IHTMLCurrentStyle_Release(&This->IHTMLCurrentStyle_iface);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle3_GetTypeInfoCount(IHTMLCurrentStyle3 *iface, UINT *pctinfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->css_style.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle3_GetTypeInfo(IHTMLCurrentStyle3 *iface, UINT iTInfo,
-        LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-    return IDispatchEx_GetTypeInfo(&This->css_style.dispex.IDispatchEx_iface, iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle3_GetIDsOfNames(IHTMLCurrentStyle3 *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-    return IDispatchEx_GetIDsOfNames(&This->css_style.dispex.IDispatchEx_iface, riid, rgszNames, cNames,
-            lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle3_Invoke(IHTMLCurrentStyle3 *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle3(iface);
-    return IDispatchEx_Invoke(&This->css_style.dispex.IDispatchEx_iface, dispIdMember, riid, lcid,
-            wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLCurrentStyle3, IHTMLCurrentStyle3,
+                      impl_from_IHTMLCurrentStyle3(iface)->css_style.dispex)
 
 static HRESULT WINAPI HTMLCurrentStyle3_get_textOverflow(IHTMLCurrentStyle3 *iface, BSTR *p)
 {
@@ -1185,56 +1043,8 @@ static const IHTMLCurrentStyle3Vtbl HTMLCurrentStyle3Vtbl = {
     HTMLCurrentStyle3_get_whiteSpace
 };
 
-/* IHTMLCurrentStyle4 */
-static HRESULT WINAPI HTMLCurrentStyle4_QueryInterface(IHTMLCurrentStyle4 *iface, REFIID riid, void **ppv)
-{
-   HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-
-   return IHTMLCurrentStyle_QueryInterface(&This->IHTMLCurrentStyle_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLCurrentStyle4_AddRef(IHTMLCurrentStyle4 *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-
-    return IHTMLCurrentStyle_AddRef(&This->IHTMLCurrentStyle_iface);
-}
-
-static ULONG WINAPI HTMLCurrentStyle4_Release(IHTMLCurrentStyle4 *iface)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-    return IHTMLCurrentStyle_Release(&This->IHTMLCurrentStyle_iface);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle4_GetTypeInfoCount(IHTMLCurrentStyle4 *iface, UINT *pctinfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->css_style.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle4_GetTypeInfo(IHTMLCurrentStyle4 *iface, UINT iTInfo,
-        LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-    return IDispatchEx_GetTypeInfo(&This->css_style.dispex.IDispatchEx_iface, iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle4_GetIDsOfNames(IHTMLCurrentStyle4 *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-    return IDispatchEx_GetIDsOfNames(&This->css_style.dispex.IDispatchEx_iface, riid, rgszNames, cNames,
-            lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLCurrentStyle4_Invoke(IHTMLCurrentStyle4 *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLCurrentStyle *This = impl_from_IHTMLCurrentStyle4(iface);
-    return IDispatchEx_Invoke(&This->css_style.dispex.IDispatchEx_iface, dispIdMember, riid, lcid,
-            wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLCurrentStyle4, IHTMLCurrentStyle4,
+                      impl_from_IHTMLCurrentStyle4(iface)->css_style.dispex)
 
 static HRESULT WINAPI HTMLCurrentStyle4_msInterpolationMode(IHTMLCurrentStyle4 *iface, BSTR *p)
 {
@@ -1278,6 +1088,46 @@ static const IHTMLCurrentStyle4Vtbl HTMLCurrentStyle4Vtbl = {
     HTMLCurrentStyle4_get_maxWidth
 };
 
+static inline HTMLCurrentStyle *impl_from_IWineCSSProperties(IWineCSSProperties *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLCurrentStyle, IWineCSSProperties_iface);
+}
+
+DISPEX_IDISPATCH_IMPL(HTMLCurrentStyle_CSSProperties, IWineCSSProperties, impl_from_IWineCSSProperties(iface)->css_style.dispex)
+
+static HRESULT WINAPI HTMLCurrentStyle_CSSProperties_getAttribute(IWineCSSProperties *iface, BSTR name, LONG flags, VARIANT *p)
+{
+    HTMLCurrentStyle *This = impl_from_IWineCSSProperties(iface);
+    return HTMLCurrentStyle_getAttribute(&This->IHTMLCurrentStyle_iface, name, flags, p);
+}
+
+static HRESULT WINAPI HTMLCurrentStyle_CSSProperties_setAttribute(IWineCSSProperties *iface, BSTR name, VARIANT value, LONG flags)
+{
+    HTMLCurrentStyle *This = impl_from_IWineCSSProperties(iface);
+    FIXME("(%p)->(%s %s %08lx)\n", This, debugstr_w(name), debugstr_variant(&value), flags);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLCurrentStyle_CSSProperties_removeAttribute(IWineCSSProperties *iface, BSTR name, LONG flags, VARIANT_BOOL *p)
+{
+    HTMLCurrentStyle *This = impl_from_IWineCSSProperties(iface);
+    FIXME("(%p)->(%s %08lx %p)\n", This, debugstr_w(name), flags, p);
+    return E_NOTIMPL;
+}
+
+static const IWineCSSPropertiesVtbl HTMLCurrentStyle_CSSPropertiesVtbl = {
+    HTMLCurrentStyle_CSSProperties_QueryInterface,
+    HTMLCurrentStyle_CSSProperties_AddRef,
+    HTMLCurrentStyle_CSSProperties_Release,
+    HTMLCurrentStyle_CSSProperties_GetTypeInfoCount,
+    HTMLCurrentStyle_CSSProperties_GetTypeInfo,
+    HTMLCurrentStyle_CSSProperties_GetIDsOfNames,
+    HTMLCurrentStyle_CSSProperties_Invoke,
+    HTMLCurrentStyle_CSSProperties_setAttribute,
+    HTMLCurrentStyle_CSSProperties_getAttribute,
+    HTMLCurrentStyle_CSSProperties_removeAttribute
+};
+
 static inline HTMLCurrentStyle *impl_from_DispatchEx(DispatchEx *dispex)
 {
     return CONTAINING_RECORD(dispex, HTMLCurrentStyle, css_style.dispex);
@@ -1295,6 +1145,8 @@ static void *HTMLCurrentStyle_query_interface(DispatchEx *dispex, REFIID riid)
         return &This->IHTMLCurrentStyle3_iface;
     if(IsEqualGUID(&IID_IHTMLCurrentStyle4, riid))
         return &This->IHTMLCurrentStyle4_iface;
+    if(IsEqualGUID(&IID_IWineCSSProperties, riid))
+        return &This->IWineCSSProperties_iface;
     return CSSStyle_query_interface(&This->css_style.dispex, riid);
 }
 
@@ -1319,26 +1171,36 @@ static void HTMLCurrentStyle_unlink(DispatchEx *dispex)
     }
 }
 
-static const dispex_static_data_vtbl_t HTMLCurrentStyle_dispex_vtbl = {
+static void MSCurrentStyleCSSProperties_init_dispex_info(dispex_data_t *info, compat_mode_t mode)
+{
+    static const dispex_hook_t currentstyle_ie11_hooks[] = {
+        {DISPID_IHTMLCURRENTSTYLE_BEHAVIOR},
+        {DISPID_UNKNOWN}
+    };
+    MSCSSProperties_init_dispex_info(info, mode);
+    dispex_info_add_interface(info, IHTMLCurrentStyle_tid, mode >= COMPAT_MODE_IE11 ? currentstyle_ie11_hooks : NULL);
+}
+
+static const dispex_static_data_vtbl_t MSCurrentStyleCSSProperties_dispex_vtbl = {
     CSSSTYLE_DISPEX_VTBL_ENTRIES,
     .query_interface   = HTMLCurrentStyle_query_interface,
     .traverse          = HTMLCurrentStyle_traverse,
     .unlink            = HTMLCurrentStyle_unlink
 };
 
-static const tid_t HTMLCurrentStyle_iface_tids[] = {
-    IHTMLCurrentStyle_tid,
+static const tid_t MSCurrentStyleCSSProperties_iface_tids[] = {
     IHTMLCurrentStyle2_tid,
     IHTMLCurrentStyle3_tid,
     IHTMLCurrentStyle4_tid,
     0
 };
-static dispex_static_data_t HTMLCurrentStyle_dispex = {
-    "MSCurrentStyleCSSProperties",
-    &HTMLCurrentStyle_dispex_vtbl,
-    DispHTMLCurrentStyle_tid,
-    HTMLCurrentStyle_iface_tids,
-    CSSStyle_init_dispex_info
+dispex_static_data_t MSCurrentStyleCSSProperties_dispex = {
+    .id           = PROT_MSCurrentStyleCSSProperties,
+    .prototype_id = PROT_MSCSSProperties,
+    .vtbl         = &MSCurrentStyleCSSProperties_dispex_vtbl,
+    .disp_tid     = DispHTMLCurrentStyle_tid,
+    .iface_tids   = MSCurrentStyleCSSProperties_iface_tids,
+    .init_info    = MSCurrentStyleCSSProperties_init_dispex_info,
 };
 
 HRESULT HTMLCurrentStyle_Create(HTMLElement *elem, IHTMLCurrentStyle **p)
@@ -1389,8 +1251,9 @@ HRESULT HTMLCurrentStyle_Create(HTMLElement *elem, IHTMLCurrentStyle **p)
     ret->IHTMLCurrentStyle2_iface.lpVtbl = &HTMLCurrentStyle2Vtbl;
     ret->IHTMLCurrentStyle3_iface.lpVtbl = &HTMLCurrentStyle3Vtbl;
     ret->IHTMLCurrentStyle4_iface.lpVtbl = &HTMLCurrentStyle4Vtbl;
+    ret->IWineCSSProperties_iface.lpVtbl = &HTMLCurrentStyle_CSSPropertiesVtbl;
 
-    init_css_style(&ret->css_style, nsstyle, &HTMLCurrentStyle_dispex, dispex_compat_mode(&elem->node.event_target.dispex));
+    init_css_style(&ret->css_style, nsstyle, &MSCurrentStyleCSSProperties_dispex, &elem->node.event_target.dispex);
     nsIDOMCSSStyleDeclaration_Release(nsstyle);
 
     IHTMLElement_AddRef(&elem->IHTMLElement_iface);
