@@ -44,57 +44,8 @@ static inline HTMLGenericElement *impl_from_IHTMLGenericElement(IHTMLGenericElem
     return CONTAINING_RECORD(iface, HTMLGenericElement, IHTMLGenericElement_iface);
 }
 
-static HRESULT WINAPI HTMLGenericElement_QueryInterface(IHTMLGenericElement *iface, REFIID riid, void **ppv)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLGenericElement_AddRef(IHTMLGenericElement *iface)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLGenericElement_Release(IHTMLGenericElement *iface)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-
-    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLGenericElement_GetTypeInfoCount(IHTMLGenericElement *iface, UINT *pctinfo)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLGenericElement_GetTypeInfo(IHTMLGenericElement *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
-            ppTInfo);
-}
-
-static HRESULT WINAPI HTMLGenericElement_GetIDsOfNames(IHTMLGenericElement *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
-            cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLGenericElement_Invoke(IHTMLGenericElement *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLGenericElement *This = impl_from_IHTMLGenericElement(iface);
-    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
-            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLGenericElement, IHTMLGenericElement,
+                      impl_from_IHTMLGenericElement(iface)->element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLGenericElement_get_recordset(IHTMLGenericElement *iface, IDispatch **p)
 {
@@ -157,18 +108,18 @@ static const event_target_vtbl_t HTMLGenericElement_event_target_vtbl = {
     .handle_event       = HTMLElement_handle_event
 };
 
-static const tid_t HTMLGenericElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
+static const tid_t HTMLUnknownElement_iface_tids[] = {
     IHTMLGenericElement_tid,
     0
 };
 
-static dispex_static_data_t HTMLGenericElement_dispex = {
-    "HTMLUnknownElement",
-    &HTMLGenericElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLGenericElement_tid,
-    HTMLGenericElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLUnknownElement_dispex = {
+    .id           = PROT_HTMLUnknownElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLGenericElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLGenericElement_tid,
+    .iface_tids   = HTMLUnknownElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLGenericElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
@@ -182,7 +133,7 @@ HRESULT HTMLGenericElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem,
     ret->IHTMLGenericElement_iface.lpVtbl = &HTMLGenericElementVtbl;
     ret->element.node.vtbl = &HTMLGenericElementImplVtbl;
 
-    HTMLElement_Init(&ret->element, doc, nselem, &HTMLGenericElement_dispex);
+    HTMLElement_Init(&ret->element, doc, nselem, &HTMLUnknownElement_dispex);
 
     *elem = &ret->element;
     return S_OK;
