@@ -48,61 +48,8 @@ static inline HTMLObjectElement *impl_from_IHTMLObjectElement(IHTMLObjectElement
     return CONTAINING_RECORD(iface, HTMLObjectElement, IHTMLObjectElement_iface);
 }
 
-static HRESULT WINAPI HTMLObjectElement_QueryInterface(IHTMLObjectElement *iface,
-        REFIID riid, void **ppv)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->plugin_container.element.node.IHTMLDOMNode_iface,
-            riid, ppv);
-}
-
-static ULONG WINAPI HTMLObjectElement_AddRef(IHTMLObjectElement *iface)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->plugin_container.element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLObjectElement_Release(IHTMLObjectElement *iface)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-
-    return IHTMLDOMNode_Release(&This->plugin_container.element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLObjectElement_GetTypeInfoCount(IHTMLObjectElement *iface, UINT *pctinfo)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            pctinfo);
-}
-
-static HRESULT WINAPI HTMLObjectElement_GetTypeInfo(IHTMLObjectElement *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLObjectElement_GetIDsOfNames(IHTMLObjectElement *iface, REFIID riid,
-                                                LPOLESTR *rgszNames, UINT cNames,
-                                                LCID lcid, DISPID *rgDispId)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            riid, rgszNames, cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLObjectElement_Invoke(IHTMLObjectElement *iface, DISPID dispIdMember,
-                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_Invoke(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            dispIdMember, riid, lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLObjectElement, IHTMLObjectElement,
+                      impl_from_IHTMLObjectElement(iface)->plugin_container.element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLObjectElement_get_object(IHTMLObjectElement *iface, IDispatch **p)
 {
@@ -507,60 +454,8 @@ static inline HTMLObjectElement *impl_from_IHTMLObjectElement2(IHTMLObjectElemen
     return CONTAINING_RECORD(iface, HTMLObjectElement, IHTMLObjectElement2_iface);
 }
 
-static HRESULT WINAPI HTMLObjectElement2_QueryInterface(IHTMLObjectElement2 *iface,
-        REFIID riid, void **ppv)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->plugin_container.element.node.IHTMLDOMNode_iface,
-            riid, ppv);
-}
-
-static ULONG WINAPI HTMLObjectElement2_AddRef(IHTMLObjectElement2 *iface)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-
-    return IHTMLDOMNode_AddRef(&This->plugin_container.element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLObjectElement2_Release(IHTMLObjectElement2 *iface)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-
-    return IHTMLDOMNode_Release(&This->plugin_container.element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLObjectElement2_GetTypeInfoCount(IHTMLObjectElement2 *iface, UINT *pctinfo)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            pctinfo);
-}
-
-static HRESULT WINAPI HTMLObjectElement2_GetTypeInfo(IHTMLObjectElement2 *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLObjectElement2_GetIDsOfNames(IHTMLObjectElement2 *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            riid, rgszNames, cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLObjectElement2_Invoke(IHTMLObjectElement2 *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult,
-        EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_Invoke(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
-            dispIdMember, riid, lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLObjectElement2, IHTMLObjectElement2,
+                      impl_from_IHTMLObjectElement2(iface)->plugin_container.element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLObjectElement2_namedRecordset(IHTMLObjectElement2 *iface, BSTR dataMember,
         VARIANT *hierarchy, IDispatch **ppRecordset)
@@ -632,6 +527,58 @@ static const IHTMLObjectElement2Vtbl HTMLObjectElement2Vtbl = {
     HTMLObjectElement2_get_data
 };
 
+/*
+ * This object wraps any unrecognized interface overriding its IUnknown methods, allowing
+ * us to return external interface from our QI implementation preserving COM rules.
+ * This can't be done right and it seems to be broken by design.
+ */
+typedef struct {
+    IUnknown IUnknown_iface;
+    IUnknown *iface;
+    IUnknown *ref_unk;
+    LONG ref;
+} iface_wrapper_t;
+
+static inline iface_wrapper_t *impl_from_IUnknown(IUnknown *iface)
+{
+    return CONTAINING_RECORD(iface, iface_wrapper_t, IUnknown_iface);
+}
+
+HRESULT WINAPI wrapper_QueryInterface(IUnknown *iface, REFIID riid, void **ppv)
+{
+    iface_wrapper_t *This = impl_from_IUnknown(iface);
+
+    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+
+    return IUnknown_QueryInterface(This->ref_unk, riid, ppv);
+}
+
+ULONG WINAPI wrapper_AddRef(IUnknown *iface)
+{
+    iface_wrapper_t *This = impl_from_IUnknown(iface);
+    LONG ref = InterlockedIncrement(&This->ref);
+
+    TRACE("(%p) ref=%ld\n", This, ref);
+
+    return ref;
+}
+
+ULONG WINAPI wrapper_Release(IUnknown *iface)
+{
+    iface_wrapper_t *This = impl_from_IUnknown(iface);
+    LONG ref = InterlockedDecrement(&This->ref);
+
+    TRACE("(%p) ref=%ld\n", This, ref);
+
+    if(!ref) {
+        IUnknown_Release(This->iface);
+        IUnknown_Release(This->ref_unk);
+        free(This);
+    }
+
+    return ref;
+}
+
 static inline HTMLObjectElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
 {
     return CONTAINING_RECORD(iface, HTMLObjectElement, plugin_container.element.node);
@@ -665,19 +612,25 @@ static void *HTMLObjectElement_query_interface(DispatchEx *dispex, REFIID riid)
 
     elem_iface = HTMLElement_query_interface(&This->plugin_container.element.node.event_target.dispex, riid);
     if(!elem_iface && This->plugin_container.plugin_host && This->plugin_container.plugin_host->plugin_unk) {
-        IUnknown *plugin_iface, *ret;
+        IUnknown *plugin_iface;
         HRESULT hres = IUnknown_QueryInterface(This->plugin_container.plugin_host->plugin_unk, riid, (void**)&plugin_iface);
 
         if(hres == S_OK) {
-            hres = wrap_iface(plugin_iface, (IUnknown*)&This->IHTMLObjectElement_iface, &ret);
-            IUnknown_Release(plugin_iface);
-            if(FAILED(hres)) {
-                ERR("wrap_iface failed: %08lx\n", hres);
+            iface_wrapper_t *wrapper = malloc(sizeof(*wrapper));
+            if(!wrapper) {
+                IUnknown_Release(plugin_iface);
                 return NULL;
             }
 
-            TRACE("returning plugin iface %p wrapped to %p\n", plugin_iface, ret);
-            return ret;
+            wrapper->IUnknown_iface.lpVtbl = (const IUnknownVtbl *)iface_wrapper_vtbl;
+            wrapper->ref = 1;
+            wrapper->iface = plugin_iface;
+            IUnknown_AddRef(wrapper->iface);
+            wrapper->ref_unk = (IUnknown*)&This->IHTMLObjectElement_iface;
+
+            IUnknown_Release(plugin_iface);
+            TRACE("returning plugin iface %p wrapped to %p\n", plugin_iface, wrapper);
+            return &wrapper->IUnknown_iface;
         }
     }
 
@@ -700,44 +653,6 @@ static void HTMLObjectElement_unlink(DispatchEx *dispex)
     unlink_ref(&This->nsobject);
 }
 
-static void HTMLObjectElement_destructor(DispatchEx *dispex)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    if(This->plugin_container.plugin_host)
-        detach_plugin_host(This->plugin_container.plugin_host);
-
-    HTMLElement_destructor(&This->plugin_container.element.node.event_target.dispex);
-}
-
-static HRESULT HTMLObjectElement_get_dispid(DispatchEx *dispex, BSTR name, DWORD grfdex, DISPID *dispid)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    TRACE("(%p)->(%s %lx %p)\n", This, debugstr_w(name), grfdex, dispid);
-
-    return get_plugin_dispid(&This->plugin_container, name, dispid);
-}
-
-static HRESULT HTMLObjectElement_dispex_get_name(DispatchEx *dispex, DISPID id, BSTR *name)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    FIXME("(%p)->(%lx %p)\n", This, id, name);
-
-    return E_NOTIMPL;
-}
-
-static HRESULT HTMLObjectElement_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
-        VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    TRACE("(%p)->(%ld)\n", This, id);
-
-    return invoke_plugin_prop(&This->plugin_container, id, lcid, flags, params, res, ei);
-}
-
 static const NodeImplVtbl HTMLObjectElementImplVtbl = {
     .clsid                 = &CLSID_HTMLObjectElement,
     .cpc_entries           = HTMLElement_cpc,
@@ -750,12 +665,12 @@ static const event_target_vtbl_t HTMLObjectElement_event_target_vtbl = {
     {
         HTMLELEMENT_DISPEX_VTBL_ENTRIES,
         .query_interface= HTMLObjectElement_query_interface,
-        .destructor     = HTMLObjectElement_destructor,
+        .destructor     = HTMLPluginContainer_destructor,
         .traverse       = HTMLObjectElement_traverse,
         .unlink         = HTMLObjectElement_unlink,
-        .get_dispid     = HTMLObjectElement_get_dispid,
-        .get_name       = HTMLObjectElement_dispex_get_name,
-        .invoke         = HTMLObjectElement_invoke
+        .get_dispid     = HTMLPluginContainer_get_dispid,
+        .get_prop_desc  = HTMLPluginContainer_get_prop_desc,
+        .invoke         = HTMLPluginContainer_invoke
     },
     HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
     .handle_event       = HTMLElement_handle_event
@@ -764,15 +679,15 @@ static const event_target_vtbl_t HTMLObjectElement_event_target_vtbl = {
 static const tid_t HTMLObjectElement_iface_tids[] = {
     IHTMLObjectElement2_tid,
     IHTMLObjectElement_tid,
-    HTMLELEMENT_TIDS,
     0
 };
-static dispex_static_data_t HTMLObjectElement_dispex = {
-    "HTMLObjectElement",
-    &HTMLObjectElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLObjectElement_tid,
-    HTMLObjectElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLObjectElement_dispex = {
+    .id           = PROT_HTMLObjectElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLObjectElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLObjectElement_tid,
+    .iface_tids   = HTMLObjectElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLObjectElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
@@ -808,58 +723,8 @@ static inline HTMLEmbed *impl_from_IHTMLEmbedElement(IHTMLEmbedElement *iface)
     return CONTAINING_RECORD(iface, HTMLEmbed, IHTMLEmbedElement_iface);
 }
 
-static HRESULT WINAPI HTMLEmbedElement_QueryInterface(IHTMLEmbedElement *iface,
-        REFIID riid, void **ppv)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLEmbedElement_AddRef(IHTMLEmbedElement *iface)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLEmbedElement_Release(IHTMLEmbedElement *iface)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-
-    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLEmbedElement_GetTypeInfoCount(IHTMLEmbedElement *iface, UINT *pctinfo)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLEmbedElement_GetTypeInfo(IHTMLEmbedElement *iface, UINT iTInfo,
-        LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
-            ppTInfo);
-}
-
-static HRESULT WINAPI HTMLEmbedElement_GetIDsOfNames(IHTMLEmbedElement *iface, REFIID riid,
-        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
-            cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLEmbedElement_Invoke(IHTMLEmbedElement *iface, DISPID dispIdMember,
-        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-        VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLEmbed *This = impl_from_IHTMLEmbedElement(iface);
-    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
-            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLEmbedElement, IHTMLEmbedElement,
+                      impl_from_IHTMLEmbedElement(iface)->element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLEmbedElement_put_hidden(IHTMLEmbedElement *iface, BSTR v)
 {
@@ -1018,16 +883,16 @@ static const event_target_vtbl_t HTMLEmbedElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLEmbedElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLEmbedElement_tid,
     0
 };
-static dispex_static_data_t HTMLEmbedElement_dispex = {
-    "HTMLEmbedElement",
-    &HTMLEmbedElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLEmbed_tid,
-    HTMLEmbedElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLEmbedElement_dispex = {
+    .id           = PROT_HTMLEmbedElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLEmbedElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLEmbed_tid,
+    .iface_tids   = HTMLEmbedElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLEmbedElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
