@@ -45,59 +45,8 @@ static inline HTMLOptionElement *impl_from_IHTMLOptionElement(IHTMLOptionElement
     return CONTAINING_RECORD(iface, HTMLOptionElement, IHTMLOptionElement_iface);
 }
 
-static HRESULT WINAPI HTMLOptionElement_QueryInterface(IHTMLOptionElement *iface,
-        REFIID riid, void **ppv)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLOptionElement_AddRef(IHTMLOptionElement *iface)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLOptionElement_Release(IHTMLOptionElement *iface)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-
-    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLOptionElement_GetTypeInfoCount(IHTMLOptionElement *iface, UINT *pctinfo)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLOptionElement_GetTypeInfo(IHTMLOptionElement *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
-            ppTInfo);
-}
-
-static HRESULT WINAPI HTMLOptionElement_GetIDsOfNames(IHTMLOptionElement *iface, REFIID riid,
-                                                LPOLESTR *rgszNames, UINT cNames,
-                                                LCID lcid, DISPID *rgDispId)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
-            cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLOptionElement_Invoke(IHTMLOptionElement *iface, DISPID dispIdMember,
-                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
-    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
-            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLOptionElement, IHTMLOptionElement,
+                      impl_from_IHTMLOptionElement(iface)->element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLOptionElement_put_selected(IHTMLOptionElement *iface, VARIANT_BOOL v)
 {
@@ -392,16 +341,16 @@ static const event_target_vtbl_t HTMLOptionElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLOptionElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLOptionElement_tid,
     0
 };
-static dispex_static_data_t HTMLOptionElement_dispex = {
-    "HTMLOptionElement",
-    &HTMLOptionElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLOptionElement_tid,
-    HTMLOptionElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLOptionElement_dispex = {
+    .id           = PROT_HTMLOptionElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLOptionElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLOptionElement_tid,
+    .iface_tids   = HTMLOptionElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLOptionElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
@@ -430,54 +379,8 @@ static inline HTMLOptionElementFactory *impl_from_IHTMLOptionElementFactory(IHTM
     return CONTAINING_RECORD(iface, HTMLOptionElementFactory, IHTMLOptionElementFactory_iface);
 }
 
-static HRESULT WINAPI HTMLOptionElementFactory_QueryInterface(IHTMLOptionElementFactory *iface,
-                                                              REFIID riid, void **ppv)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_QueryInterface(&This->dispex.IDispatchEx_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLOptionElementFactory_AddRef(IHTMLOptionElementFactory *iface)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_AddRef(&This->dispex.IDispatchEx_iface);
-}
-
-static ULONG WINAPI HTMLOptionElementFactory_Release(IHTMLOptionElementFactory *iface)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_Release(&This->dispex.IDispatchEx_iface);
-}
-
-static HRESULT WINAPI HTMLOptionElementFactory_GetTypeInfoCount(IHTMLOptionElementFactory *iface, UINT *pctinfo)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLOptionElementFactory_GetTypeInfo(IHTMLOptionElementFactory *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_GetTypeInfo(&This->dispex.IDispatchEx_iface, iTInfo, lcid, ppTInfo);
-}
-
-static HRESULT WINAPI HTMLOptionElementFactory_GetIDsOfNames(IHTMLOptionElementFactory *iface, REFIID riid,
-                                                LPOLESTR *rgszNames, UINT cNames,
-                                                LCID lcid, DISPID *rgDispId)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_GetIDsOfNames(&This->dispex.IDispatchEx_iface, riid, rgszNames, cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLOptionElementFactory_Invoke(IHTMLOptionElementFactory *iface, DISPID dispIdMember,
-                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLOptionElementFactory *This = impl_from_IHTMLOptionElementFactory(iface);
-    return IDispatchEx_Invoke(&This->dispex.IDispatchEx_iface, dispIdMember, riid, lcid, wFlags, pDispParams,
-            pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLOptionElementFactory, IHTMLOptionElementFactory,
+                      impl_from_IHTMLOptionElementFactory(iface)->dispex)
 
 static HRESULT WINAPI HTMLOptionElementFactory_create(IHTMLOptionElementFactory *iface,
         VARIANT text, VARIANT value, VARIANT defaultselected, VARIANT selected,
@@ -621,10 +524,11 @@ static const dispex_static_data_vtbl_t HTMLOptionElementFactory_dispex_vtbl = {
 };
 
 static dispex_static_data_t HTMLOptionElementFactory_dispex = {
-    "Function",
-    &HTMLOptionElementFactory_dispex_vtbl,
-    IHTMLOptionElementFactory_tid,
-    HTMLOptionElementFactory_iface_tids,
+    .name           = "Function",
+    .constructor_id = PROT_HTMLOptionElement,
+    .vtbl           = &HTMLOptionElementFactory_dispex_vtbl,
+    .disp_tid       = IHTMLOptionElementFactory_tid,
+    .iface_tids     = HTMLOptionElementFactory_iface_tids,
 };
 
 HRESULT HTMLOptionElementFactory_Create(HTMLInnerWindow *window, HTMLOptionElementFactory **ret_ptr)
@@ -639,7 +543,8 @@ HRESULT HTMLOptionElementFactory_Create(HTMLInnerWindow *window, HTMLOptionEleme
     ret->window = window;
     IHTMLWindow2_AddRef(&window->base.IHTMLWindow2_iface);
 
-    init_dispatch(&ret->dispex, &HTMLOptionElementFactory_dispex, dispex_compat_mode(&window->event_target.dispex));
+    init_dispatch(&ret->dispex, &HTMLOptionElementFactory_dispex, window,
+                  dispex_compat_mode(&window->event_target.dispex));
 
     *ret_ptr = ret;
     return S_OK;
@@ -831,63 +736,8 @@ static const IEnumVARIANTVtbl HTMLSelectElementEnumVtbl = {
     HTMLSelectElementEnum_Clone
 };
 
-static HRESULT WINAPI HTMLSelectElement_QueryInterface(IHTMLSelectElement *iface,
-                                                         REFIID riid, void **ppv)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLSelectElement_AddRef(IHTMLSelectElement *iface)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLSelectElement_Release(IHTMLSelectElement *iface)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLSelectElement_GetTypeInfoCount(IHTMLSelectElement *iface, UINT *pctinfo)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLSelectElement_GetTypeInfo(IHTMLSelectElement *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
-            ppTInfo);
-}
-
-static HRESULT WINAPI HTMLSelectElement_GetIDsOfNames(IHTMLSelectElement *iface, REFIID riid,
-                                                LPOLESTR *rgszNames, UINT cNames,
-                                                LCID lcid, DISPID *rgDispId)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
-            cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLSelectElement_Invoke(IHTMLSelectElement *iface, DISPID dispIdMember,
-                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLSelectElement *This = impl_from_IHTMLSelectElement(iface);
-
-    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
-            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLSelectElement, IHTMLSelectElement,
+                      impl_from_IHTMLSelectElement(iface)->element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLSelectElement_put_size(IHTMLSelectElement *iface, LONG v)
 {
@@ -1366,7 +1216,7 @@ static void HTMLSelectElement_unlink(DispatchEx *dispex)
 
 #define DISPID_OPTIONCOL_0 MSHTML_DISPID_CUSTOM_MIN
 
-static HRESULT HTMLSelectElement_get_dispid(DispatchEx *dispex, BSTR name, DWORD flags, DISPID *dispid)
+static HRESULT HTMLSelectElement_get_dispid(DispatchEx *dispex, const WCHAR *name, DWORD flags, DISPID *dispid)
 {
     const WCHAR *ptr;
     DWORD idx = 0;
@@ -1385,19 +1235,6 @@ static HRESULT HTMLSelectElement_get_dispid(DispatchEx *dispex, BSTR name, DWORD
     return S_OK;
 }
 
-static HRESULT HTMLSelectElement_dispex_get_name(DispatchEx *dispex, DISPID id, BSTR *name)
-{
-    DWORD idx = id - DISPID_OPTIONCOL_0;
-    WCHAR buf[11];
-    UINT len;
-
-    if(idx > MSHTML_CUSTOM_DISPID_CNT)
-        return DISP_E_MEMBERNOTFOUND;
-
-    len = swprintf(buf, ARRAY_SIZE(buf), L"%u", idx);
-    return (*name = SysAllocStringLen(buf, len)) ? S_OK : E_OUTOFMEMORY;
-}
-
 static HRESULT HTMLSelectElement_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
         VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
@@ -1448,7 +1285,7 @@ static const event_target_vtbl_t HTMLSelectElement_event_target_vtbl = {
         .traverse       = HTMLSelectElement_traverse,
         .unlink         = HTMLSelectElement_unlink,
         .get_dispid     = HTMLSelectElement_get_dispid,
-        .get_name       = HTMLSelectElement_dispex_get_name,
+        .get_prop_desc  = dispex_index_prop_desc,
         .invoke         = HTMLSelectElement_invoke
     },
     HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
@@ -1456,17 +1293,17 @@ static const event_target_vtbl_t HTMLSelectElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLSelectElement_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLSelectElement_tid,
     0
 };
 
-static dispex_static_data_t HTMLSelectElement_dispex = {
-    "HTMLSelectElement",
-    &HTMLSelectElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLSelectElement_tid,
-    HTMLSelectElement_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLSelectElement_dispex = {
+    .id           = PROT_HTMLSelectElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLSelectElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLSelectElement_tid,
+    .iface_tids   = HTMLSelectElement_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLSelectElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
