@@ -45,59 +45,8 @@ static inline HTMLTextAreaElement *impl_from_IHTMLTextAreaElement(IHTMLTextAreaE
     return CONTAINING_RECORD(iface, HTMLTextAreaElement, IHTMLTextAreaElement_iface);
 }
 
-static HRESULT WINAPI HTMLTextAreaElement_QueryInterface(IHTMLTextAreaElement *iface,
-                                                         REFIID riid, void **ppv)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-
-    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
-}
-
-static ULONG WINAPI HTMLTextAreaElement_AddRef(IHTMLTextAreaElement *iface)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-
-    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static ULONG WINAPI HTMLTextAreaElement_Release(IHTMLTextAreaElement *iface)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-
-    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
-}
-
-static HRESULT WINAPI HTMLTextAreaElement_GetTypeInfoCount(IHTMLTextAreaElement *iface, UINT *pctinfo)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
-}
-
-static HRESULT WINAPI HTMLTextAreaElement_GetTypeInfo(IHTMLTextAreaElement *iface, UINT iTInfo,
-                                              LCID lcid, ITypeInfo **ppTInfo)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
-            ppTInfo);
-}
-
-static HRESULT WINAPI HTMLTextAreaElement_GetIDsOfNames(IHTMLTextAreaElement *iface, REFIID riid,
-                                                LPOLESTR *rgszNames, UINT cNames,
-                                                LCID lcid, DISPID *rgDispId)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
-            cNames, lcid, rgDispId);
-}
-
-static HRESULT WINAPI HTMLTextAreaElement_Invoke(IHTMLTextAreaElement *iface, DISPID dispIdMember,
-                            REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
-                            VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
-{
-    HTMLTextAreaElement *This = impl_from_IHTMLTextAreaElement(iface);
-    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
-            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-}
+DISPEX_IDISPATCH_IMPL(HTMLTextAreaElement, IHTMLTextAreaElement,
+                      impl_from_IHTMLTextAreaElement(iface)->element.node.event_target.dispex)
 
 static HRESULT WINAPI HTMLTextAreaElement_get_type(IHTMLTextAreaElement *iface, BSTR *p)
 {
@@ -461,17 +410,17 @@ static const event_target_vtbl_t HTMLTextAreaElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLTextAreaElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLTextAreaElement_tid,
     0
 };
 
-static dispex_static_data_t HTMLTextAreaElement_dispex = {
-    "HTMLTextAreaElement",
-    &HTMLTextAreaElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLTextAreaElement_tid,
-    HTMLTextAreaElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLTextAreaElement_dispex = {
+    .id           = PROT_HTMLTextAreaElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLTextAreaElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLTextAreaElement_tid,
+    .iface_tids   = HTMLTextAreaElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLTextAreaElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
