@@ -347,7 +347,7 @@ end:
     return r;
 }
 
-static LPWSTR msi_read_text_archive(LPCWSTR path, DWORD *len)
+static WCHAR *read_text_archive(const WCHAR *path, DWORD *len)
 {
     HANDLE file;
     LPSTR data = NULL;
@@ -377,7 +377,7 @@ done:
     return wdata;
 }
 
-static void msi_parse_line(LPWSTR *line, LPWSTR **entries, DWORD *num_entries, DWORD *len)
+static UINT parse_line(WCHAR **line, WCHAR ***entries, DWORD *num_entries, DWORD *len)
 {
     LPWSTR ptr = *line, save;
     DWORD i, count = 1, chars_left = *len;
@@ -395,9 +395,16 @@ static void msi_parse_line(LPWSTR *line, LPWSTR **entries, DWORD *num_entries, D
         chars_left--;
     }
 
+    /*
+     * make sure this line has the same number of entries as there are columns
+     * which are indicated by the first line.
+     */
+    if (*num_entries && *num_entries != count)
+        return ERROR_FUNCTION_FAILED;
+
     *entries = malloc(count * sizeof(WCHAR *));
     if (!*entries)
-        return;
+        return ERROR_OUTOFMEMORY;
 
     /* store pointers into the data */
     chars_left = *len;
@@ -442,11 +449,13 @@ static void msi_parse_line(LPWSTR *line, LPWSTR **entries, DWORD *num_entries, D
     /* move to the next line if there's more, else EOF */
     *line = ptr;
     *len = chars_left;
-    if (num_entries)
+    if (!*num_entries)
         *num_entries = count;
+
+    return ERROR_SUCCESS;
 }
 
-static LPWSTR msi_build_createsql_prelude(LPWSTR table)
+static WCHAR *build_createsql_prelude(const WCHAR *table)
 {
     LPWSTR prelude;
     DWORD size;
@@ -460,7 +469,7 @@ static LPWSTR msi_build_createsql_prelude(LPWSTR table)
     return prelude;
 }
 
-static LPWSTR msi_build_createsql_columns(LPWSTR *columns_data, LPWSTR *types, DWORD num_columns)
+static WCHAR *build_createsql_columns(WCHAR **columns_data, WCHAR **types, DWORD num_columns)
 {
     LPWSTR columns, p;
     LPCWSTR type;
@@ -547,7 +556,7 @@ static LPWSTR msi_build_createsql_columns(LPWSTR *columns_data, LPWSTR *types, D
     return columns;
 }
 
-static LPWSTR msi_build_createsql_postlude(LPWSTR *primary_keys, DWORD num_keys)
+static WCHAR *build_createsql_postlude(WCHAR **primary_keys, DWORD num_keys)
 {
     LPWSTR postlude, keys, ptr;
     DWORD size, i;
@@ -579,7 +588,8 @@ done:
     return postlude;
 }
 
-static UINT msi_add_table_to_db(MSIDATABASE *db, LPWSTR *columns, LPWSTR *types, LPWSTR *labels, DWORD num_labels, DWORD num_columns)
+static UINT add_table_to_db(MSIDATABASE *db, WCHAR **columns, WCHAR **types, WCHAR **labels, DWORD num_labels,
+                            DWORD num_columns)
 {
     UINT r = ERROR_OUTOFMEMORY;
     DWORD size;
@@ -587,9 +597,9 @@ static UINT msi_add_table_to_db(MSIDATABASE *db, LPWSTR *columns, LPWSTR *types,
     LPWSTR create_sql = NULL;
     LPWSTR prelude, columns_sql, postlude;
 
-    prelude = msi_build_createsql_prelude(labels[0]);
-    columns_sql = msi_build_createsql_columns(columns, types, num_columns);
-    postlude = msi_build_createsql_postlude(labels + 1, num_labels - 1); /* skip over table name */
+    prelude = build_createsql_prelude(labels[0]);
+    columns_sql = build_createsql_columns(columns, types, num_columns);
+    postlude = build_createsql_postlude(labels + 1, num_labels - 1); /* skip over table name */
 
     if (!prelude || !columns_sql || !postlude)
         goto done;
@@ -619,7 +629,7 @@ done:
     return r;
 }
 
-static LPWSTR msi_import_stream_filename(LPCWSTR path, LPCWSTR name)
+static WCHAR *import_stream_filename(const WCHAR *path, const WCHAR *name)
 {
     DWORD len;
     LPWSTR fullname, ptr;
@@ -667,7 +677,7 @@ static UINT construct_record(DWORD num_columns, LPWSTR *types,
                 if (*data[i])
                 {
                     UINT r;
-                    LPWSTR file = msi_import_stream_filename(path, data[i]);
+                    WCHAR *file = import_stream_filename(path, data[i]);
                     if (!file)
                         return ERROR_FUNCTION_FAILED;
 
@@ -687,10 +697,8 @@ static UINT construct_record(DWORD num_columns, LPWSTR *types,
     return ERROR_SUCCESS;
 }
 
-static UINT msi_add_records_to_table(MSIDATABASE *db, LPWSTR *columns, LPWSTR *types,
-                                     LPWSTR *labels, LPWSTR **records,
-                                     int num_columns, int num_records,
-                                     LPWSTR path)
+static UINT add_records_to_table(MSIDATABASE *db, WCHAR **columns, WCHAR **types, WCHAR **labels, WCHAR ***records,
+                                 int num_columns, int num_records, WCHAR *path)
 {
     UINT r;
     int i;
@@ -733,7 +741,7 @@ done:
 static UINT MSI_DatabaseImport(MSIDATABASE *db, LPCWSTR folder, LPCWSTR file)
 {
     UINT r;
-    DWORD len, i, num_labels, num_types, num_columns, num_records = 0;
+    DWORD len, i, num_labels = 0, num_types = 0, num_columns = 0, num_records = 0;
     WCHAR **columns, **types, **labels, *path, *ptr, *data, ***records = NULL, ***temp_records;
 
     TRACE("%p %s %s\n", db, debugstr_w(folder), debugstr_w(file) );
@@ -750,7 +758,7 @@ static UINT MSI_DatabaseImport(MSIDATABASE *db, LPCWSTR folder, LPCWSTR file)
     lstrcatW( path, L"\\" );
     lstrcatW( path, file );
 
-    data = msi_read_text_archive( path, &len );
+    data = read_text_archive( path, &len );
     if (!data)
     {
         free(path);
@@ -758,9 +766,9 @@ static UINT MSI_DatabaseImport(MSIDATABASE *db, LPCWSTR folder, LPCWSTR file)
     }
 
     ptr = data;
-    msi_parse_line( &ptr, &columns, &num_columns, &len );
-    msi_parse_line( &ptr, &types, &num_types, &len );
-    msi_parse_line( &ptr, &labels, &num_labels, &len );
+    parse_line( &ptr, &columns, &num_columns, &len );
+    parse_line( &ptr, &types, &num_types, &len );
+    parse_line( &ptr, &labels, &num_labels, &len );
 
     if (num_columns == 1 && !columns[0][0] && num_labels == 1 && !labels[0][0] &&
         num_types == 2 && !wcscmp( types[1], L"_ForceCodepage" ))
@@ -785,7 +793,9 @@ static UINT MSI_DatabaseImport(MSIDATABASE *db, LPCWSTR folder, LPCWSTR file)
     /* read in the table records */
     while (len)
     {
-        msi_parse_line( &ptr, &records[num_records], NULL, &len );
+        r = parse_line( &ptr, &records[num_records], &num_columns, &len );
+        if (r != ERROR_SUCCESS)
+            goto done;
 
         num_records++;
         temp_records = realloc(records, (num_records + 1) * sizeof(WCHAR **));
@@ -810,7 +820,7 @@ static UINT MSI_DatabaseImport(MSIDATABASE *db, LPCWSTR folder, LPCWSTR file)
     {
         if (!TABLE_Exists(db, labels[0]))
         {
-            r = msi_add_table_to_db( db, columns, types, labels, num_labels, num_columns );
+            r = add_table_to_db( db, columns, types, labels, num_labels, num_columns );
             if (r != ERROR_SUCCESS)
             {
                 r = ERROR_FUNCTION_FAILED;
@@ -818,7 +828,7 @@ static UINT MSI_DatabaseImport(MSIDATABASE *db, LPCWSTR folder, LPCWSTR file)
             }
         }
 
-        r = msi_add_records_to_table( db, columns, types, labels, records, num_columns, num_records, path );
+        r = add_records_to_table( db, columns, types, labels, records, num_columns, num_records, path );
     }
 
 done:
@@ -880,9 +890,9 @@ end:
     return r;
 }
 
-static UINT msi_export_field( HANDLE handle, MSIRECORD *row, UINT field )
+static UINT export_field( HANDLE handle, MSIRECORD *row, UINT field )
 {
-    char *buffer;
+    char *buffer, *ptr;
     BOOL ret;
     DWORD sz = 0x100;
     UINT r;
@@ -918,12 +928,28 @@ static UINT msi_export_field( HANDLE handle, MSIRECORD *row, UINT field )
         return r;
     }
 
+    ptr = buffer;
+    while( *ptr )
+    {
+         if (*ptr == '\r' && *( ptr + 1 ) == '\n')
+         {
+             *ptr++ = '\x11';
+             *ptr++ = '\x19';
+             continue;
+         }
+
+         if (*ptr == '\n')
+             *ptr = '\x19';
+
+         ptr++;
+    }
+
     ret = WriteFile( handle, buffer, sz, &sz, NULL );
     free( buffer );
     return ret ? ERROR_SUCCESS : ERROR_FUNCTION_FAILED;
 }
 
-static UINT msi_export_stream( const WCHAR *folder, const WCHAR *table, MSIRECORD *row, UINT field, UINT start )
+static UINT export_stream( const WCHAR *folder, const WCHAR *table, MSIRECORD *row, UINT field, UINT start )
 {
     WCHAR stream[MAX_STREAM_NAME_LEN + 1], *path;
     DWORD sz, read_size, write_size;
@@ -981,7 +1007,7 @@ struct row_export_info
     const WCHAR *table;
 };
 
-static UINT msi_export_record( struct row_export_info *row_export_info, MSIRECORD *row, UINT start )
+static UINT export_record( struct row_export_info *row_export_info, MSIRECORD *row, UINT start )
 {
     HANDLE handle = row_export_info->handle;
     UINT i, count, r = ERROR_SUCCESS;
@@ -991,15 +1017,15 @@ static UINT msi_export_record( struct row_export_info *row_export_info, MSIRECOR
     count = MSI_RecordGetFieldCount( row );
     for (i = start; i <= count; i++)
     {
-        r = msi_export_field( handle, row, i );
+        r = export_field( handle, row, i );
         if (r == ERROR_INVALID_PARAMETER)
         {
-            r = msi_export_stream( row_export_info->folder, row_export_info->table, row, i, start );
+            r = export_stream( row_export_info->folder, row_export_info->table, row, i, start );
             if (r != ERROR_SUCCESS)
                 return r;
 
             /* exporting a binary stream, repeat the "Name" field */
-            r = msi_export_field( handle, row, start );
+            r = export_field( handle, row, start );
             if (r != ERROR_SUCCESS)
                 return r;
         }
@@ -1013,12 +1039,12 @@ static UINT msi_export_record( struct row_export_info *row_export_info, MSIRECOR
     return r;
 }
 
-static UINT msi_export_row( MSIRECORD *row, void *arg )
+static UINT export_row( MSIRECORD *row, void *arg )
 {
-    return msi_export_record( arg, row, 1 );
+    return export_record( arg, row, 1 );
 }
 
-static UINT msi_export_forcecodepage( HANDLE handle, UINT codepage )
+static UINT export_forcecodepage( HANDLE handle, UINT codepage )
 {
     static const char fmt[] = "\r\n\r\n%u\t_ForceCodepage\r\n";
     char data[sizeof(fmt) + 10];
@@ -1030,7 +1056,7 @@ static UINT msi_export_forcecodepage( HANDLE handle, UINT codepage )
     return ERROR_SUCCESS;
 }
 
-static UINT msi_export_summaryinformation( MSIDATABASE *db, HANDLE handle )
+static UINT export_summaryinformation( MSIDATABASE *db, HANDLE handle )
 {
     static const char header[] = "PropertyId\tValue\r\n"
                                  "i2\tl255\r\n"
@@ -1075,13 +1101,13 @@ static UINT MSI_DatabaseExport( MSIDATABASE *db, LPCWSTR table, LPCWSTR folder,
     if (!wcscmp( table, L"_ForceCodepage" ))
     {
         UINT codepage = msi_get_string_table_codepage( db->strings );
-        r = msi_export_forcecodepage( handle, codepage );
+        r = export_forcecodepage( handle, codepage );
         goto done;
     }
 
     if (!wcscmp( table, L"_SummaryInformation" ))
     {
-        r = msi_export_summaryinformation( db, handle );
+        r = export_summaryinformation( db, handle );
         goto done;
     }
 
@@ -1094,7 +1120,7 @@ static UINT MSI_DatabaseExport( MSIDATABASE *db, LPCWSTR table, LPCWSTR folder,
         r = MSI_ViewGetColumnInfo(view, MSICOLINFO_NAMES, &rec);
         if (r == ERROR_SUCCESS)
         {
-            msi_export_record( &row_export_info, rec, 1 );
+            export_record( &row_export_info, rec, 1 );
             msiobj_release( &rec->hdr );
         }
 
@@ -1102,7 +1128,7 @@ static UINT MSI_DatabaseExport( MSIDATABASE *db, LPCWSTR table, LPCWSTR folder,
         r = MSI_ViewGetColumnInfo(view, MSICOLINFO_TYPES, &rec);
         if (r == ERROR_SUCCESS)
         {
-            msi_export_record( &row_export_info, rec, 1 );
+            export_record( &row_export_info, rec, 1 );
             msiobj_release( &rec->hdr );
         }
 
@@ -1111,12 +1137,12 @@ static UINT MSI_DatabaseExport( MSIDATABASE *db, LPCWSTR table, LPCWSTR folder,
         if (r == ERROR_SUCCESS)
         {
             MSI_RecordSetStringW( rec, 0, table );
-            msi_export_record( &row_export_info, rec, 0 );
+            export_record( &row_export_info, rec, 0 );
             msiobj_release( &rec->hdr );
         }
 
         /* write out row 4 onwards, the data */
-        r = MSI_IterateRecords( view, 0, msi_export_row, &row_export_info );
+        r = MSI_IterateRecords( view, 0, export_row, &row_export_info );
         msiobj_release( &view->hdr );
     }
 
@@ -1207,7 +1233,7 @@ UINT WINAPI MsiDatabaseMergeA( MSIHANDLE hDatabase, MSIHANDLE hDatabaseMerge, co
     return r;
 }
 
-typedef struct _tagMERGETABLE
+struct merge_table
 {
     struct list entry;
     struct list rows;
@@ -1219,22 +1245,22 @@ typedef struct _tagMERGETABLE
     DWORD numtypes;
     LPWSTR *labels;
     DWORD numlabels;
-} MERGETABLE;
+};
 
-typedef struct _tagMERGEROW
+struct merge_row
 {
     struct list entry;
     MSIRECORD *data;
-} MERGEROW;
+};
 
-typedef struct _tagMERGEDATA
+struct merge_data
 {
     MSIDATABASE *db;
     MSIDATABASE *merge;
-    MERGETABLE *curtable;
+    struct merge_table *curtable;
     MSIQUERY *curview;
     struct list *tabledata;
-} MERGEDATA;
+};
 
 static BOOL merge_type_match(LPCWSTR type1, LPCWSTR type2)
 {
@@ -1412,7 +1438,7 @@ static LPWSTR get_key_value(MSIQUERY *view, LPCWSTR key, MSIRECORD *rec)
 static LPWSTR create_diff_row_query(MSIDATABASE *merge, MSIQUERY *view,
                                     LPWSTR table, MSIRECORD *rec)
 {
-    LPWSTR query = NULL, clause = NULL, val;
+    WCHAR *query = NULL, *clause = NULL, *new_clause, *val;
     LPCWSTR setptr, key;
     DWORD size, oldsize;
     MSIRECORD *keys;
@@ -1422,10 +1448,6 @@ static LPWSTR create_diff_row_query(MSIDATABASE *merge, MSIQUERY *view,
     if (r != ERROR_SUCCESS)
         return NULL;
 
-    clause = calloc(1, sizeof(WCHAR));
-    if (!clause)
-        goto done;
-
     size = 1;
     count = MSI_RecordGetFieldCount(keys);
     for (i = 1; i <= count; i++)
@@ -1440,12 +1462,13 @@ static LPWSTR create_diff_row_query(MSIDATABASE *merge, MSIQUERY *view,
 
         oldsize = size;
         size += lstrlenW(setptr) + lstrlenW(key) + lstrlenW(val) - 4;
-        clause = realloc(clause, size * sizeof(WCHAR));
-        if (!clause)
+        new_clause = realloc(clause, size * sizeof(WCHAR));
+        if (!new_clause)
         {
             free(val);
             goto done;
         }
+        clause = new_clause;
 
         swprintf(clause + oldsize - 1, size - (oldsize - 1), setptr, key, val);
         free(val);
@@ -1466,9 +1489,9 @@ done:
 
 static UINT merge_diff_row(MSIRECORD *rec, LPVOID param)
 {
-    MERGEDATA *data = param;
-    MERGETABLE *table = data->curtable;
-    MERGEROW *mergerow;
+    struct merge_data *data = param;
+    struct merge_table *table = data->curtable;
+    struct merge_row *mergerow;
     MSIQUERY *dbview = NULL;
     MSIRECORD *row = NULL;
     LPWSTR query = NULL;
@@ -1500,7 +1523,7 @@ static UINT merge_diff_row(MSIRECORD *rec, LPVOID param)
         r = ERROR_SUCCESS;
     }
 
-    mergerow = malloc(sizeof(MERGEROW));
+    mergerow = malloc(sizeof(*mergerow));
     if (!mergerow)
     {
         r = ERROR_OUTOFMEMORY;
@@ -1524,7 +1547,7 @@ done:
     return r;
 }
 
-static UINT msi_get_table_labels(MSIDATABASE *db, LPCWSTR table, LPWSTR **labels, DWORD *numlabels)
+static UINT get_table_labels(MSIDATABASE *db, const WCHAR *table, WCHAR ***labels, DWORD *numlabels)
 {
     UINT r, i, count;
     MSIRECORD *prec = NULL;
@@ -1553,7 +1576,7 @@ end:
     return r;
 }
 
-static UINT msi_get_query_columns(MSIQUERY *query, LPWSTR **columns, DWORD *numcolumns)
+static UINT get_query_columns(MSIQUERY *query, WCHAR ***columns, DWORD *numcolumns)
 {
     UINT r, i, count;
     MSIRECORD *prec = NULL;
@@ -1582,7 +1605,7 @@ end:
     return r;
 }
 
-static UINT msi_get_query_types(MSIQUERY *query, LPWSTR **types, DWORD *numtypes)
+static UINT get_query_types(MSIQUERY *query, WCHAR ***types, DWORD *numtypes)
 {
     UINT r, i, count;
     MSIRECORD *prec = NULL;
@@ -1610,13 +1633,13 @@ end:
     return r;
 }
 
-static void merge_free_rows(MERGETABLE *table)
+static void merge_free_rows(struct merge_table *table)
 {
     struct list *item, *cursor;
 
     LIST_FOR_EACH_SAFE(item, cursor, &table->rows)
     {
-        MERGEROW *row = LIST_ENTRY(item, MERGEROW, entry);
+        struct merge_row *row = LIST_ENTRY(item, struct merge_row, entry);
 
         list_remove(&row->entry);
         msiobj_release(&row->data->hdr);
@@ -1624,7 +1647,7 @@ static void merge_free_rows(MERGETABLE *table)
     }
 }
 
-static void free_merge_table(MERGETABLE *table)
+static void free_merge_table(struct merge_table *table)
 {
     UINT i;
 
@@ -1658,20 +1681,20 @@ static void free_merge_table(MERGETABLE *table)
     free(table);
 }
 
-static UINT msi_get_merge_table (MSIDATABASE *db, LPCWSTR name, MERGETABLE **ptable)
+static UINT get_merge_table(MSIDATABASE *db, const WCHAR *name, struct merge_table **ptable)
 {
     UINT r;
-    MERGETABLE *table;
+    struct merge_table *table;
     MSIQUERY *mergeview = NULL;
 
-    table = calloc(1, sizeof(MERGETABLE));
+    table = calloc(1, sizeof(*table));
     if (!table)
     {
        *ptable = NULL;
        return ERROR_OUTOFMEMORY;
     }
 
-    r = msi_get_table_labels(db, name, &table->labels, &table->numlabels);
+    r = get_table_labels(db, name, &table->labels, &table->numlabels);
     if (r != ERROR_SUCCESS)
         goto err;
 
@@ -1679,11 +1702,11 @@ static UINT msi_get_merge_table (MSIDATABASE *db, LPCWSTR name, MERGETABLE **pta
     if (r != ERROR_SUCCESS)
         goto err;
 
-    r = msi_get_query_columns(mergeview, &table->columns, &table->numcolumns);
+    r = get_query_columns(mergeview, &table->columns, &table->numcolumns);
     if (r != ERROR_SUCCESS)
         goto err;
 
-    r = msi_get_query_types(mergeview, &table->types, &table->numtypes);
+    r = get_query_types(mergeview, &table->types, &table->numtypes);
     if (r != ERROR_SUCCESS)
         goto err;
 
@@ -1705,8 +1728,8 @@ err:
 
 static UINT merge_diff_tables(MSIRECORD *rec, LPVOID param)
 {
-    MERGEDATA *data = param;
-    MERGETABLE *table;
+    struct merge_data *data = param;
+    struct merge_table *table;
     MSIQUERY *dbview = NULL;
     MSIQUERY *mergeview = NULL;
     LPCWSTR name;
@@ -1733,7 +1756,7 @@ static UINT merge_diff_tables(MSIRECORD *rec, LPVOID param)
             goto done;
     }
 
-    r = msi_get_merge_table(data->merge, name, &table);
+    r = get_merge_table(data->merge, name, &table);
     if (r != ERROR_SUCCESS)
         goto done;
 
@@ -1758,7 +1781,7 @@ static UINT gather_merge_data(MSIDATABASE *db, MSIDATABASE *merge,
                               struct list *tabledata)
 {
     MSIQUERY *view;
-    MERGEDATA data;
+    struct merge_data data;
     UINT r;
 
     r = MSI_DatabaseOpenViewW(merge, L"SELECT * FROM `_Tables`", &view);
@@ -1773,21 +1796,20 @@ static UINT gather_merge_data(MSIDATABASE *db, MSIDATABASE *merge,
     return r;
 }
 
-static UINT merge_table(MSIDATABASE *db, MERGETABLE *table)
+static UINT merge_table(MSIDATABASE *db, struct merge_table *table)
 {
     UINT r;
-    MERGEROW *row;
+    struct merge_row *row;
     MSIVIEW *tv;
 
     if (!TABLE_Exists(db, table->name))
     {
-        r = msi_add_table_to_db(db, table->columns, table->types,
-               table->labels, table->numlabels, table->numcolumns);
+        r = add_table_to_db(db, table->columns, table->types, table->labels, table->numlabels, table->numcolumns);
         if (r != ERROR_SUCCESS)
            return ERROR_FUNCTION_FAILED;
     }
 
-    LIST_FOR_EACH_ENTRY(row, &table->rows, MERGEROW, entry)
+    LIST_FOR_EACH_ENTRY(row, &table->rows, struct merge_row, entry)
     {
         r = TABLE_CreateView(db, table->name, &tv);
         if (r != ERROR_SUCCESS)
@@ -1837,7 +1859,7 @@ UINT WINAPI MsiDatabaseMergeW( MSIHANDLE hDatabase, MSIHANDLE hDatabaseMerge, co
     struct list tabledata = LIST_INIT(tabledata);
     struct list *item, *cursor;
     MSIDATABASE *db, *merge;
-    MERGETABLE *table;
+    struct merge_table *table;
     BOOL conflicts;
     UINT r;
 
@@ -1859,7 +1881,7 @@ UINT WINAPI MsiDatabaseMergeW( MSIHANDLE hDatabase, MSIHANDLE hDatabaseMerge, co
         goto done;
 
     conflicts = FALSE;
-    LIST_FOR_EACH_ENTRY(table, &tabledata, MERGETABLE, entry)
+    LIST_FOR_EACH_ENTRY(table, &tabledata, struct merge_table, entry)
     {
         if (table->numconflicts)
         {
@@ -1880,7 +1902,7 @@ UINT WINAPI MsiDatabaseMergeW( MSIHANDLE hDatabase, MSIHANDLE hDatabaseMerge, co
 
     LIST_FOR_EACH_SAFE(item, cursor, &tabledata)
     {
-        MERGETABLE *table = LIST_ENTRY(item, MERGETABLE, entry);
+        struct merge_table *table = LIST_ENTRY(item, struct merge_table, entry);
         list_remove(&table->entry);
         free_merge_table(table);
     }
