@@ -38,20 +38,20 @@
 WINE_DEFAULT_DEBUG_CHANNEL(msidb);
 
 /* below is the query interface to a table */
-typedef struct tagMSIROWENTRY
+struct row_entry
 {
     struct tagMSIWHEREVIEW *wv; /* used during sorting */
     UINT values[1];
-} MSIROWENTRY;
+};
 
-typedef struct tagJOINTABLE
+struct join_table
 {
-    struct tagJOINTABLE *next;
+    struct join_table *next;
     MSIVIEW *view;
     UINT col_count;
     UINT row_count;
     UINT table_index;
-} JOINTABLE;
+};
 
 typedef struct tagMSIORDERINFO
 {
@@ -64,11 +64,11 @@ typedef struct tagMSIWHEREVIEW
 {
     MSIVIEW        view;
     MSIDATABASE   *db;
-    JOINTABLE     *tables;
+    struct join_table *tables;
     UINT           row_count;
     UINT           col_count;
     UINT           table_count;
-    MSIROWENTRY  **reorder;
+    struct row_entry **reorder;
     UINT           reorder_size; /* number of entries available in reorder */
     struct expr   *cond;
     UINT           rec_index;
@@ -100,7 +100,7 @@ static void free_reorder(MSIWHEREVIEW *wv)
 
 static UINT init_reorder(MSIWHEREVIEW *wv)
 {
-    MSIROWENTRY **new = calloc(INITIAL_REORDER_SIZE, sizeof(MSIROWENTRY *));
+    struct row_entry **new = calloc(INITIAL_REORDER_SIZE, sizeof(*new));
     if (!new)
         return ERROR_OUTOFMEMORY;
 
@@ -124,11 +124,11 @@ static inline UINT find_row(MSIWHEREVIEW *wv, UINT row, UINT *(values[]))
 
 static UINT add_row(MSIWHEREVIEW *wv, UINT vals[])
 {
-    MSIROWENTRY *new;
+    struct row_entry *new;
 
     if (wv->reorder_size <= wv->row_count)
     {
-        MSIROWENTRY **new_reorder;
+        struct row_entry **new_reorder;
         UINT newsize = wv->reorder_size * 2;
 
         new_reorder = realloc(wv->reorder, newsize * sizeof(*new_reorder));
@@ -140,7 +140,7 @@ static UINT add_row(MSIWHEREVIEW *wv, UINT vals[])
         wv->reorder_size = newsize;
     }
 
-    new = malloc(offsetof(MSIROWENTRY, values[wv->table_count]));
+    new = malloc(offsetof(struct row_entry, values[wv->table_count]));
 
     if (!new)
         return ERROR_OUTOFMEMORY;
@@ -153,9 +153,9 @@ static UINT add_row(MSIWHEREVIEW *wv, UINT vals[])
     return ERROR_SUCCESS;
 }
 
-static JOINTABLE *find_table(MSIWHEREVIEW *wv, UINT col, UINT *table_col)
+static struct join_table *find_table(MSIWHEREVIEW *wv, UINT col, UINT *table_col)
 {
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
 
     if(col == 0 || col > wv->col_count)
          return NULL;
@@ -174,7 +174,7 @@ static JOINTABLE *find_table(MSIWHEREVIEW *wv, UINT col, UINT *table_col)
 static UINT parse_column(MSIWHEREVIEW *wv, union ext_column *column,
                          UINT *column_type)
 {
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
     UINT i, r;
 
     do
@@ -216,7 +216,7 @@ static UINT parse_column(MSIWHEREVIEW *wv, union ext_column *column,
 static UINT WHERE_fetch_int( struct tagMSIVIEW *view, UINT row, UINT col, UINT *val )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table;
+    struct join_table *table;
     UINT *rows;
     UINT r;
 
@@ -239,7 +239,7 @@ static UINT WHERE_fetch_int( struct tagMSIVIEW *view, UINT row, UINT col, UINT *
 static UINT WHERE_fetch_stream( struct tagMSIVIEW *view, UINT row, UINT col, IStream **stm )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table;
+    struct join_table *table;
     UINT *rows;
     UINT r;
 
@@ -262,7 +262,7 @@ static UINT WHERE_fetch_stream( struct tagMSIVIEW *view, UINT row, UINT col, ISt
 static UINT WHERE_set_int(struct tagMSIVIEW *view, UINT row, UINT col, int val)
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table;
+    struct join_table *table;
     UINT *rows;
     UINT r;
 
@@ -282,7 +282,7 @@ static UINT WHERE_set_int(struct tagMSIVIEW *view, UINT row, UINT col, int val)
 static UINT WHERE_set_string(struct tagMSIVIEW *view, UINT row, UINT col, const WCHAR *val, int len)
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table;
+    struct join_table *table;
     UINT *rows;
     UINT r;
 
@@ -302,7 +302,7 @@ static UINT WHERE_set_string(struct tagMSIVIEW *view, UINT row, UINT col, const
 static UINT WHERE_set_stream(MSIVIEW *view, UINT row, UINT col, IStream *stream)
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table;
+    struct join_table *table;
     UINT *rows;
     UINT r;
 
@@ -323,7 +323,7 @@ static UINT WHERE_set_row( struct tagMSIVIEW *view, UINT row, MSIRECORD *rec, UI
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
     UINT i, r, offset = 0;
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
     UINT *rows;
     UINT mask_copy = mask;
 
@@ -493,7 +493,7 @@ static INT INT_evaluate_binary( MSIWHEREVIEW *wv, const UINT rows[],
 
 static inline UINT expr_fetch_value(const union ext_column *expr, const UINT rows[], UINT *val)
 {
-    JOINTABLE *table = expr->parsed.table;
+    struct join_table *table = expr->parsed.table;
 
     if( rows[table->table_index] == INVALID_ROW_INDEX )
     {
@@ -647,7 +647,7 @@ static UINT WHERE_evaluate( MSIWHEREVIEW *wv, const UINT rows[],
     return ERROR_SUCCESS;
 }
 
-static UINT check_condition( MSIWHEREVIEW *wv, MSIRECORD *record, JOINTABLE **tables,
+static UINT check_condition( MSIWHEREVIEW *wv, MSIRECORD *record, struct join_table **tables,
                              UINT table_rows[] )
 {
     UINT r = ERROR_FUNCTION_FAILED;
@@ -684,8 +684,8 @@ static UINT check_condition( MSIWHEREVIEW *wv, MSIRECORD *record, JOINTABLE **ta
 
 static int __cdecl compare_entry( const void *left, const void *right )
 {
-    const MSIROWENTRY *le = *(const MSIROWENTRY**)left;
-    const MSIROWENTRY *re = *(const MSIROWENTRY**)right;
+    const struct row_entry *le = *(const struct row_entry **)left;
+    const struct row_entry *re = *(const struct row_entry **)right;
     const MSIWHEREVIEW *wv = le->wv;
     MSIORDERINFO *order = wv->order_info;
     UINT i, j, r, l_val, r_val;
@@ -729,7 +729,7 @@ static int __cdecl compare_entry( const void *left, const void *right )
     return 0;
 }
 
-static void add_to_array( JOINTABLE **array, JOINTABLE *elem )
+static void add_to_array( struct join_table **array, struct join_table *elem )
 {
     while (*array && *array != elem)
         array++;
@@ -737,7 +737,7 @@ static void add_to_array( JOINTABLE **array, JOINTABLE *elem )
         *array = elem;
 }
 
-static BOOL in_array( JOINTABLE **array, JOINTABLE *elem )
+static BOOL in_array( struct join_table **array, struct join_table *elem )
 {
     while (*array && *array != elem)
         array++;
@@ -748,8 +748,8 @@ static BOOL in_array( JOINTABLE **array, JOINTABLE *elem )
 #define JOIN_TO_CONST_EXPR 0x10000 /* comparison to a table involved with
                                       a CONST_EXPR comaprison */
 
-static UINT reorder_check( const struct expr *expr, JOINTABLE **ordered_tables,
-                           BOOL process_joins, JOINTABLE **lastused )
+static UINT reorder_check( const struct expr *expr, struct join_table **ordered_tables,
+                           BOOL process_joins, struct join_table **lastused )
 {
     UINT res = 0;
 
@@ -787,10 +787,9 @@ static UINT reorder_check( const struct expr *expr, JOINTABLE **ordered_tables,
 }
 
 /* reorders the tablelist in a way to evaluate the condition as fast as possible */
-static JOINTABLE **ordertables( MSIWHEREVIEW *wv )
+static struct join_table **ordertables( MSIWHEREVIEW *wv )
 {
-    JOINTABLE *table;
-    JOINTABLE **tables;
+    struct join_table *table, **tables;
 
     tables = calloc(wv->table_count + 1, sizeof(*tables));
 
@@ -815,9 +814,9 @@ static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
     UINT r;
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
     UINT *rows;
-    JOINTABLE **ordered_tables;
+    struct join_table **ordered_tables;
     UINT i = 0;
 
     TRACE("%p %p\n", wv, record);
@@ -857,7 +856,7 @@ static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
     if (wv->order_info)
         wv->order_info->error = ERROR_SUCCESS;
 
-    qsort(wv->reorder, wv->row_count, sizeof(MSIROWENTRY *), compare_entry);
+    qsort(wv->reorder, wv->row_count, sizeof(struct row_entry *), compare_entry);
 
     if (wv->order_info)
         r = wv->order_info->error;
@@ -870,7 +869,7 @@ static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
 static UINT WHERE_close( struct tagMSIVIEW *view )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
 
     TRACE("%p\n", wv );
 
@@ -910,7 +909,7 @@ static UINT WHERE_get_column_info( struct tagMSIVIEW *view, UINT n, LPCWSTR *nam
                                    UINT *type, BOOL *temporary, LPCWSTR *table_name )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table;
+    struct join_table *table;
 
     TRACE("%p %d %p %p %p %p\n", wv, n, name, type, temporary, table_name );
 
@@ -980,7 +979,7 @@ static UINT WHERE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
                           MSIRECORD *rec, UINT row )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
     UINT r;
 
     TRACE("%p %d %p\n", wv, eModifyMode, rec);
@@ -1035,13 +1034,13 @@ static UINT WHERE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
 static UINT WHERE_delete( struct tagMSIVIEW *view )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
 
     TRACE("%p\n", wv );
 
     while(table)
     {
-        JOINTABLE *next;
+        struct join_table *next;
 
         table->view->ops->delete(table->view);
         table->view = NULL;
@@ -1066,7 +1065,7 @@ static UINT WHERE_delete( struct tagMSIVIEW *view )
 static UINT WHERE_sort(struct tagMSIVIEW *view, column_info *columns)
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW *)view;
-    JOINTABLE *table = wv->tables;
+    struct join_table *table = wv->tables;
     column_info *column = columns;
     MSIORDERINFO *orderinfo;
     UINT r, count = 0;
@@ -1245,12 +1244,12 @@ UINT WHERE_CreateView( MSIDATABASE *db, MSIVIEW **view, LPWSTR tables,
 
     while (*tables)
     {
-        JOINTABLE *table;
+        struct join_table *table;
 
         if ((ptr = wcschr(tables, ' ')))
             *ptr = '\0';
 
-        table = malloc(sizeof(JOINTABLE));
+        table = malloc(sizeof(*table));
         if (!table)
         {
             r = ERROR_OUTOFMEMORY;
