@@ -24,7 +24,6 @@
 #include "rpcproxy.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msident);
 
@@ -76,7 +75,7 @@ static ULONG WINAPI EnumUserIdentity_Release(IEnumUserIdentity *iface)
     TRACE("(%p) ref=%ld\n", This, ref);
 
     if(!ref)
-        heap_free(This);
+        free(This);
 
     return ref;
 }
@@ -166,7 +165,7 @@ static HRESULT WINAPI UserIdentityManager_EnumIdentities(IUserIdentityManager *i
 
     TRACE("(%p)\n", ppEnumUser);
 
-    ret = heap_alloc(sizeof(*ret));
+    ret = malloc(sizeof(*ret));
     if(!ret)
         return E_OUTOFMEMORY;
 
