@@ -194,7 +194,8 @@ static NTSTATUS NTAPI ntlm_SpAcquireCredentialsHandle( UNICODE_STRING *principal
     TRACE( "%s, %#lx, %p, %p, %p, %p, %p, %p\n", debugstr_us(principal), cred_use, logon_id, auth_data,
            get_key_fn, get_key_arg, cred, expiry );
 
-    switch (cred_use & ~SECPKG_CRED_RESERVED)
+    cred_use &= ~SECPKG_CRED_RESERVED;
+    switch (cred_use)
     {
     case SECPKG_CRED_INBOUND:
         if (!(cred = malloc( sizeof(*cred) ))) return SEC_E_INSUFFICIENT_MEMORY;
@@ -209,10 +210,12 @@ static NTSTATUS NTAPI ntlm_SpAcquireCredentialsHandle( UNICODE_STRING *principal
         status = SEC_E_OK;
         break;
 
+    case SECPKG_CRED_BOTH:
+        /* fall through */
     case SECPKG_CRED_OUTBOUND:
         if (!(cred = malloc( sizeof(*cred) ))) return SEC_E_INSUFFICIENT_MEMORY;
 
-        cred->mode         = MODE_CLIENT;
+        cred->mode         = cred_use == SECPKG_CRED_OUTBOUND ? MODE_CLIENT : MODE_BOTH;
         cred->username_arg = NULL;
         cred->domain_arg   = NULL;
         cred->password     = NULL;
@@ -272,11 +275,6 @@ static NTSTATUS NTAPI ntlm_SpAcquireCredentialsHandle( UNICODE_STRING *principal
         status = SEC_E_OK;
         break;
 
-    case SECPKG_CRED_BOTH:
-        FIXME( "SECPKG_CRED_BOTH not supported\n" );
-        status = SEC_E_UNSUPPORTED_FUNCTION;
-        break;
-
     default:
         status = SEC_E_UNKNOWN_CREDENTIALS;
         break;
@@ -601,7 +599,7 @@ static NTSTATUS NTAPI ntlm_SpInitLsaModeContext( LSA_SEC_HANDLE cred_handle, LSA
      *
      * The squid cache size is 2010 chars and that's what ntlm_auth uses */
 
-    if (!(buf = malloc( NTLM_MAX_BUF ))) return SEC_E_INSUFFICIENT_MEMORY;
+    if (!(buf = malloc( NTLM_MAX_BUF * 3 + 64 ))) return SEC_E_INSUFFICIENT_MEMORY;
     if (!(bin = malloc( NTLM_MAX_BUF ))) goto done;
 
     if (!ctx_handle && !input)
@@ -610,7 +608,7 @@ static NTSTATUS NTAPI ntlm_SpInitLsaModeContext( LSA_SEC_HANDLE cred_handle, LSA
         int password_len = 0;
         struct ntlm_cred *cred = (struct ntlm_cred *)cred_handle;
 
-        if (!cred || cred->mode != MODE_CLIENT)
+        if (!cred || !(cred->mode & MODE_CLIENT))
         {
             status = SEC_E_INVALID_HANDLE;
             goto done;
@@ -893,7 +891,7 @@ static NTSTATUS NTAPI ntlm_SpAcceptLsaModeContext( LSA_SEC_HANDLE cred_handle, L
            new_ctx_handle, output, ctx_attr, expiry, mapped_ctx, ctx_data );
     if (ctx_req) FIXME( "ignoring flags %#lx\n", ctx_req );
 
-    if (!(buf = malloc( NTLM_MAX_BUF ))) return SEC_E_INSUFFICIENT_MEMORY;
+    if (!(buf = malloc( NTLM_MAX_BUF * 3 + 64 ))) return SEC_E_INSUFFICIENT_MEMORY;
     if (!(bin = malloc( NTLM_MAX_BUF ))) goto done;
 
     if (!ctx_handle)
@@ -901,7 +899,7 @@ static NTSTATUS NTAPI ntlm_SpAcceptLsaModeContext( LSA_SEC_HANDLE cred_handle, L
         struct ntlm_cred *cred = (struct ntlm_cred *)cred_handle;
         char *argv[3];
 
-        if (!cred || cred->mode != MODE_SERVER)
+        if (!cred || !(cred->mode & MODE_SERVER))
         {
             status = SEC_E_INVALID_HANDLE;
             goto done;
