@@ -1118,7 +1118,7 @@ __ASM_BLOCK_END
 
 void init_concurrency_details(void *base)
 {
-#ifdef __x86_64__
+#ifdef RTTI_USE_RVA
     init__Concurrent_queue_base_v4_rtti(base);
     init__Runtime_object_rtti(base);
 #endif
