@@ -57,6 +57,8 @@ thread_data_t *CDECL msvcrt_get_thread_data(void)
         ptr->random_seed = 1;
         ptr->locinfo = MSVCRT_locale->locinfo;
         ptr->mbcinfo = MSVCRT_locale->mbcinfo;
+        ptr->cached_locale[0] = 'C';
+        ptr->cached_locale[1] = 0;
 #if _MSVCR_VER >= 140
         ptr->module = NULL;
 #endif
