@@ -25,7 +25,6 @@
 #include "dispex.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 #include "msxml_dispex.h"
 
@@ -200,9 +199,9 @@ void release_typelib(void)
         for(i=0; i < iter->func_cnt; i++)
             SysFreeString(iter->funcs[i].name);
 
-        heap_free(iter->funcs);
-        heap_free(iter->name_table);
-        heap_free(iter);
+        free(iter->funcs);
+        free(iter->name_table);
+        free(iter);
     }
 
     for(i=0; i < ARRAY_SIZE(typeinfos); i++)
@@ -224,7 +223,7 @@ static void add_func_info(dispex_data_t *data, DWORD *size, tid_t tid, DISPID id
         return;
 
     if(data->func_cnt == *size)
-        data->funcs = heap_realloc(data->funcs, (*size <<= 1)*sizeof(func_info_t));
+        data->funcs = realloc(data->funcs, (*size <<= 1) * sizeof(func_info_t));
 
     hres = ITypeInfo_GetDocumentation(dti, id, &data->funcs[data->func_cnt].name, NULL, NULL, NULL);
     if(FAILED(hres))
@@ -263,9 +262,9 @@ static dispex_data_t *preprocess_dispex_data(DispatchEx *This)
         return NULL;
     }
 
-    data = heap_alloc(sizeof(dispex_data_t));
+    data = malloc(sizeof(dispex_data_t));
     data->func_cnt = 0;
-    data->funcs = heap_alloc(size*sizeof(func_info_t));
+    data->funcs = malloc(size * sizeof(func_info_t));
     list_add_tail(&dispex_data_list, &data->entry);
 
     while(*tid) {
@@ -288,16 +287,16 @@ static dispex_data_t *preprocess_dispex_data(DispatchEx *This)
     }
 
     if(!data->func_cnt) {
-        heap_free(data->funcs);
+        free(data->funcs);
         data->funcs = NULL;
     }else if(data->func_cnt != size) {
-        data->funcs = heap_realloc(data->funcs, data->func_cnt * sizeof(func_info_t));
+        data->funcs = realloc(data->funcs, data->func_cnt * sizeof(func_info_t));
     }
 
     if(data->funcs) {
         qsort(data->funcs, data->func_cnt, sizeof(func_info_t), dispid_cmp);
 
-        data->name_table = heap_alloc(data->func_cnt * sizeof(func_info_t*));
+        data->name_table = malloc(data->func_cnt * sizeof(func_info_t*));
         for(i=0; i < data->func_cnt; i++)
             data->name_table[i] = data->funcs+i;
         qsort(data->name_table, data->func_cnt, sizeof(func_info_t*), func_name_cmp);
