@@ -16,12 +16,31 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include <bcrypt.h>
+
+enum algid
+{
+    /* symmetric */
+    AES,
+    /* asymmetric */
+    RSA,
+    DSA,
+    ECDSA,
+};
+
+struct key
+{
+    enum algid algid;
+    BCRYPT_KEY_HANDLE bcrypt_key;
+};
+
 struct storage_provider
 {
 };
 
 enum object_type
 {
+    KEY,
     STORAGE_PROVIDER,
 };
 
@@ -39,6 +58,7 @@ struct object
     struct object_property *properties;
     union
     {
+        struct key key;
         struct storage_provider storage_provider;
     };
 };
