@@ -1036,7 +1036,7 @@ static UCHAR NetBTCall(void *adapt, PNCB ncb, void **sess)
                     if (session)
                     {
                         session->fd = fd;
-                        InitializeCriticalSection(&session->cs);
+                        InitializeCriticalSectionEx(&session->cs, 0, RTL_CRITICAL_SECTION_FLAG_FORCE_DEBUG_INFO);
                         session->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": NetBTSession.cs");
                         *sess = session;
                     }
