@@ -36,6 +36,7 @@
 #include "ntdll_misc.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(seh);
+WINE_DECLARE_DEBUG_CHANNEL(threadname);
 
 typedef struct
 {
@@ -58,7 +59,7 @@ static RTL_CRITICAL_SECTION vectored_handlers_section = { &critsect_debug, -1, 0
 
 static PRTL_EXCEPTION_FILTER unhandled_exception_filter;
 
-const char *debugstr_exception_code( DWORD code )
+static const char *debugstr_exception_code( DWORD code )
 {
     switch (code)
     {
@@ -143,7 +144,7 @@ static ULONG remove_vectored_handler( struct list *handler_list, VECTORED_HANDLE
  *
  * Call the vectored handlers chain.
  */
-LONG call_vectored_handlers( EXCEPTION_RECORD *rec, CONTEXT *context )
+static LONG call_vectored_handlers( EXCEPTION_RECORD *rec, CONTEXT *context )
 {
     struct list *ptr;
     LONG ret = EXCEPTION_CONTINUE_SEARCH;
@@ -185,6 +186,133 @@ LONG call_vectored_handlers( EXCEPTION_RECORD *rec, CONTEXT *context )
 }
 
 
+/*******************************************************************
+ *		dispatch_exception
+ */
+NTSTATUS WINAPI dispatch_exception( EXCEPTION_RECORD *rec, CONTEXT *context )
+{
+    NTSTATUS status;
+    DWORD i;
+
+    switch (rec->ExceptionCode)
+    {
+    case EXCEPTION_WINE_STUB:
+        if (rec->ExceptionInformation[1] >> 16)
+            MESSAGE( "wine: Call from %p to unimplemented function %s.%s, aborting\n",
+                     rec->ExceptionAddress,
+                     (char *)rec->ExceptionInformation[0], (char *)rec->ExceptionInformation[1] );
+        else
+            MESSAGE( "wine: Call from %p to unimplemented function %s.%u, aborting\n",
+                     rec->ExceptionAddress,
+                     (char *)rec->ExceptionInformation[0], (USHORT)rec->ExceptionInformation[1] );
+        break;
+
+    case EXCEPTION_WINE_NAME_THREAD:
+        if (rec->ExceptionInformation[0] == 0x1000)
+        {
+            const char *name = (char *)rec->ExceptionInformation[1];
+            DWORD tid = (DWORD)rec->ExceptionInformation[2];
+
+            if (tid == -1 || tid == GetCurrentThreadId())
+                WARN_(threadname)( "Thread renamed to %s\n", debugstr_a(name) );
+            else
+                WARN_(threadname)( "Thread ID %04lx renamed to %s\n", tid, debugstr_a(name) );
+            set_native_thread_name( tid, name );
+        }
+        break;
+
+    case DBG_PRINTEXCEPTION_C:
+        WARN( "%s\n", debugstr_an((char *)rec->ExceptionInformation[1], rec->ExceptionInformation[0] - 1) );
+        break;
+
+    case DBG_PRINTEXCEPTION_WIDE_C:
+        WARN( "%s\n", debugstr_wn((WCHAR *)rec->ExceptionInformation[1], rec->ExceptionInformation[0] - 1) );
+        break;
+
+    case STATUS_ASSERTION_FAILURE:
+        ERR( "assertion failure exception\n" );
+        break;
+
+    default:
+        if (!TRACE_ON(seh)) WARN( "%s exception (code=%lx) raised\n",
+                                  debugstr_exception_code(rec->ExceptionCode), rec->ExceptionCode );
+        break;
+    }
+
+    TRACE( "code=%lx (%s) flags=%lx addr=%p\n",
+           rec->ExceptionCode, debugstr_exception_code(rec->ExceptionCode),
+           rec->ExceptionFlags, rec->ExceptionAddress );
+    for (i = 0; i < min( EXCEPTION_MAXIMUM_PARAMETERS, rec->NumberParameters ); i++)
+        TRACE( " info[%ld]=%p\n", i, (void *)rec->ExceptionInformation[i] );
+    TRACE_CONTEXT( context );
+
+    if (call_vectored_handlers( rec, context ) == EXCEPTION_CONTINUE_EXECUTION)
+        NtContinue( context, FALSE );
+
+    if ((status = call_seh_handlers( rec, context )) == STATUS_SUCCESS)
+        NtContinue( context, FALSE );
+
+    if (status != STATUS_UNHANDLED_EXCEPTION) RtlRaiseStatus( status );
+    return NtRaiseException( rec, context, FALSE );
+}
+
+
+#if defined(__WINE_PE_BUILD) && !defined(__i386__)
+
+/*******************************************************************
+ *		user_callback_handler
+ *
+ * Exception handler for KiUserCallbackDispatcher.
+ */
+EXCEPTION_DISPOSITION WINAPI user_callback_handler( EXCEPTION_RECORD *record, void *frame,
+                                                    CONTEXT *context, void *dispatch )
+{
+    if (!(record->ExceptionFlags & (EXCEPTION_UNWINDING | EXCEPTION_EXIT_UNWIND)))
+    {
+        ERR( "ignoring exception %lx\n", record->ExceptionCode );
+        RtlUnwind( frame, KiUserCallbackDispatcherReturn, record, ULongToPtr(record->ExceptionCode) );
+    }
+    return ExceptionContinueSearch;
+}
+
+#else
+
+/*******************************************************************
+ *		dispatch_user_callback
+ *
+ * Implementation of KiUserCallbackDispatcher.
+ */
+NTSTATUS WINAPI dispatch_user_callback( void *args, ULONG len, ULONG id )
+{
+    NTSTATUS status;
+
+    __TRY
+    {
+        KERNEL_CALLBACK_PROC func = NtCurrentTeb()->Peb->KernelCallbackTable[id];
+        status = func( args, len );
+    }
+    __EXCEPT_ALL
+    {
+        status = GetExceptionCode();
+        ERR( "ignoring exception %lx\n", status );
+    }
+    __ENDTRY
+    return status;
+}
+
+#endif
+
+/*******************************************************************
+ *         nested_exception_handler
+ */
+EXCEPTION_DISPOSITION WINAPI nested_exception_handler( EXCEPTION_RECORD *rec, void *frame,
+                                                       CONTEXT *context, void *dispatch )
+{
+    if (rec->ExceptionFlags & (EXCEPTION_UNWINDING | EXCEPTION_EXIT_UNWIND)) return ExceptionContinueSearch;
+    return ExceptionNestedException;
+}
+
+
 /*******************************************************************
  *		raise_status
  *
@@ -195,7 +323,7 @@ void DECLSPEC_NORETURN raise_status( NTSTATUS status, EXCEPTION_RECORD *rec )
     EXCEPTION_RECORD ExceptionRec;
 
     ExceptionRec.ExceptionCode    = status;
-    ExceptionRec.ExceptionFlags   = EH_NONCONTINUABLE;
+    ExceptionRec.ExceptionFlags   = EXCEPTION_NONCONTINUABLE;
     ExceptionRec.ExceptionRecord  = rec;
     ExceptionRec.NumberParameters = 0;
     for (;;) RtlRaiseException( &ExceptionRec );  /* never returns */
@@ -301,345 +429,6 @@ EXCEPTION_DISPOSITION WINAPI call_unhandled_exception_handler( EXCEPTION_RECORD
 }
 
 
-#if defined(__x86_64__) || defined(__arm__) || defined(__aarch64__)
-
-struct dynamic_unwind_entry
-{
-    struct list       entry;
-    ULONG_PTR         base;
-    ULONG_PTR         end;
-    RUNTIME_FUNCTION *table;
-    DWORD             count;
-    DWORD             max_count;
-    PGET_RUNTIME_FUNCTION_CALLBACK callback;
-    PVOID             context;
-};
-
-static struct list dynamic_unwind_list = LIST_INIT(dynamic_unwind_list);
-
-static RTL_CRITICAL_SECTION dynamic_unwind_section;
-static RTL_CRITICAL_SECTION_DEBUG dynamic_unwind_debug =
-{
-    0, 0, &dynamic_unwind_section,
-    { &dynamic_unwind_debug.ProcessLocksList, &dynamic_unwind_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": dynamic_unwind_section") }
-};
-static RTL_CRITICAL_SECTION dynamic_unwind_section = { &dynamic_unwind_debug, -1, 0, 0, 0, 0 };
-
-static ULONG_PTR get_runtime_function_end( RUNTIME_FUNCTION *func, ULONG_PTR addr )
-{
-#ifdef __x86_64__
-    return func->EndAddress;
-#elif defined(__arm__)
-    if (func->Flag) return func->BeginAddress + func->FunctionLength * 2;
-    else
-    {
-        struct unwind_info
-        {
-            DWORD function_length : 18;
-            DWORD version : 2;
-            DWORD x : 1;
-            DWORD e : 1;
-            DWORD f : 1;
-            DWORD count : 5;
-            DWORD words : 4;
-        } *info = (struct unwind_info *)(addr + func->UnwindData);
-        return func->BeginAddress + info->function_length * 2;
-    }
-#else  /* __aarch64__ */
-    if (func->Flag) return func->BeginAddress + func->FunctionLength * 4;
-    else
-    {
-        struct unwind_info
-        {
-            DWORD function_length : 18;
-            DWORD version : 2;
-            DWORD x : 1;
-            DWORD e : 1;
-            DWORD epilog : 5;
-            DWORD codes : 5;
-        } *info = (struct unwind_info *)(addr + func->UnwindData);
-        return func->BeginAddress + info->function_length * 4;
-    }
-#endif
-}
-
-/**********************************************************************
- *              RtlAddFunctionTable   (NTDLL.@)
- */
-BOOLEAN CDECL RtlAddFunctionTable( RUNTIME_FUNCTION *table, DWORD count, ULONG_PTR addr )
-{
-    struct dynamic_unwind_entry *entry;
-
-    TRACE( "%p %lu %Ix\n", table, count, addr );
-
-    /* NOTE: Windows doesn't check if table is aligned or a NULL pointer */
-
-    entry = RtlAllocateHeap( GetProcessHeap(), 0, sizeof(*entry) );
-    if (!entry)
-        return FALSE;
-
-    entry->base      = addr;
-    entry->end       = addr + (count ? get_runtime_function_end( &table[count - 1], addr ) : 0);
-    entry->table     = table;
-    entry->count     = count;
-    entry->max_count = 0;
-    entry->callback  = NULL;
-    entry->context   = NULL;
-
-    RtlEnterCriticalSection( &dynamic_unwind_section );
-    list_add_tail( &dynamic_unwind_list, &entry->entry );
-    RtlLeaveCriticalSection( &dynamic_unwind_section );
-    return TRUE;
-}
-
-
-/**********************************************************************
- *              RtlInstallFunctionTableCallback   (NTDLL.@)
- */
-BOOLEAN CDECL RtlInstallFunctionTableCallback( ULONG_PTR table, ULONG_PTR base, DWORD length,
-                                               PGET_RUNTIME_FUNCTION_CALLBACK callback, PVOID context,
-                                               PCWSTR dll )
-{
-    struct dynamic_unwind_entry *entry;
-
-    TRACE( "%Ix %Ix %ld %p %p %s\n", table, base, length, callback, context, wine_dbgstr_w(dll) );
-
-    /* NOTE: Windows doesn't check if the provided callback is a NULL pointer */
-
-    /* both low-order bits must be set */
-    if ((table & 0x3) != 0x3)
-        return FALSE;
-
-    entry = RtlAllocateHeap( GetProcessHeap(), 0, sizeof(*entry) );
-    if (!entry)
-        return FALSE;
-
-    entry->base      = base;
-    entry->end       = base + length;
-    entry->table     = (RUNTIME_FUNCTION *)table;
-    entry->count     = 0;
-    entry->max_count = 0;
-    entry->callback  = callback;
-    entry->context   = context;
-
-    RtlEnterCriticalSection( &dynamic_unwind_section );
-    list_add_tail( &dynamic_unwind_list, &entry->entry );
-    RtlLeaveCriticalSection( &dynamic_unwind_section );
-
-    return TRUE;
-}
-
-
-/*************************************************************************
- *              RtlAddGrowableFunctionTable   (NTDLL.@)
- */
-DWORD WINAPI RtlAddGrowableFunctionTable( void **table, RUNTIME_FUNCTION *functions, DWORD count,
-                                          DWORD max_count, ULONG_PTR base, ULONG_PTR end )
-{
-    struct dynamic_unwind_entry *entry;
-
-    TRACE( "%p, %p, %lu, %lu, %Ix, %Ix\n", table, functions, count, max_count, base, end );
-
-    entry = RtlAllocateHeap( GetProcessHeap(), 0, sizeof(*entry) );
-    if (!entry)
-        return STATUS_NO_MEMORY;
-
-    entry->base      = base;
-    entry->end       = end;
-    entry->table     = functions;
-    entry->count     = count;
-    entry->max_count = max_count;
-    entry->callback  = NULL;
-    entry->context   = NULL;
-
-    RtlEnterCriticalSection( &dynamic_unwind_section );
-    list_add_tail( &dynamic_unwind_list, &entry->entry );
-    RtlLeaveCriticalSection( &dynamic_unwind_section );
-
-    *table = entry;
-
-    return STATUS_SUCCESS;
-}
-
-
-/*************************************************************************
- *              RtlGrowFunctionTable   (NTDLL.@)
- */
-void WINAPI RtlGrowFunctionTable( void *table, DWORD count )
-{
-    struct dynamic_unwind_entry *entry;
-
-    TRACE( "%p, %lu\n", table, count );
-
-    RtlEnterCriticalSection( &dynamic_unwind_section );
-    LIST_FOR_EACH_ENTRY( entry, &dynamic_unwind_list, struct dynamic_unwind_entry, entry )
-    {
-        if (entry == table)
-        {
-            if (count > entry->count && count <= entry->max_count)
-                entry->count = count;
-            break;
-        }
-    }
-    RtlLeaveCriticalSection( &dynamic_unwind_section );
-}
-
-
-/*************************************************************************
- *              RtlDeleteGrowableFunctionTable   (NTDLL.@)
- */
-void WINAPI RtlDeleteGrowableFunctionTable( void *table )
-{
-    struct dynamic_unwind_entry *entry, *to_free = NULL;
-
-    TRACE( "%p\n", table );
-
-    RtlEnterCriticalSection( &dynamic_unwind_section );
-    LIST_FOR_EACH_ENTRY( entry, &dynamic_unwind_list, struct dynamic_unwind_entry, entry )
-    {
-        if (entry == table)
-        {
-            to_free = entry;
-            list_remove( &entry->entry );
-            break;
-        }
-    }
-    RtlLeaveCriticalSection( &dynamic_unwind_section );
-
-    RtlFreeHeap( GetProcessHeap(), 0, to_free );
-}
-
-
-/**********************************************************************
- *              RtlDeleteFunctionTable   (NTDLL.@)
- */
-BOOLEAN CDECL RtlDeleteFunctionTable( RUNTIME_FUNCTION *table )
-{
-    struct dynamic_unwind_entry *entry, *to_free = NULL;
-
-    TRACE( "%p\n", table );
-
-    RtlEnterCriticalSection( &dynamic_unwind_section );
-    LIST_FOR_EACH_ENTRY( entry, &dynamic_unwind_list, struct dynamic_unwind_entry, entry )
-    {
-        if (entry->table == table)
-        {
-            to_free = entry;
-            list_remove( &entry->entry );
-            break;
-        }
-    }
-    RtlLeaveCriticalSection( &dynamic_unwind_section );
-
-    if (!to_free) return FALSE;
-
-    RtlFreeHeap( GetProcessHeap(), 0, to_free );
-    return TRUE;
-}
-
-
-/* helper for lookup_function_info() */
-static RUNTIME_FUNCTION *find_function_info( ULONG_PTR pc, ULONG_PTR base,
-                                             RUNTIME_FUNCTION *func, ULONG size )
-{
-    int min = 0;
-    int max = size - 1;
-
-    while (min <= max)
-    {
-#ifdef __x86_64__
-        int pos = (min + max) / 2;
-        if (pc < base + func[pos].BeginAddress) max = pos - 1;
-        else if (pc >= base + func[pos].EndAddress) min = pos + 1;
-        else
-        {
-            func += pos;
-            while (func->UnwindData & 1)  /* follow chained entry */
-                func = (RUNTIME_FUNCTION *)(base + (func->UnwindData & ~1));
-            return func;
-        }
-#elif defined(__arm__)
-        int pos = (min + max) / 2;
-        if (pc < base + (func[pos].BeginAddress & ~1)) max = pos - 1;
-        else if (pc >= base + get_runtime_function_end( &func[pos], base )) min = pos + 1;
-        else return func + pos;
-#else  /* __aarch64__ */
-        int pos = (min + max) / 2;
-        if (pc < base + func[pos].BeginAddress) max = pos - 1;
-        else if (pc >= base + get_runtime_function_end( &func[pos], base )) min = pos + 1;
-        else return func + pos;
-#endif
-    }
-    return NULL;
-}
-
-/**********************************************************************
- *           lookup_function_info
- */
-RUNTIME_FUNCTION *lookup_function_info( ULONG_PTR pc, ULONG_PTR *base, LDR_DATA_TABLE_ENTRY **module )
-{
-    RUNTIME_FUNCTION *func = NULL;
-    struct dynamic_unwind_entry *entry;
-    ULONG size;
-
-    /* PE module or wine module */
-    if (!LdrFindEntryForAddress( (void *)pc, module ))
-    {
-        *base = (ULONG_PTR)(*module)->DllBase;
-        if ((func = RtlImageDirectoryEntryToData( (*module)->DllBase, TRUE,
-                                                  IMAGE_DIRECTORY_ENTRY_EXCEPTION, &size )))
-        {
-            /* lookup in function table */
-            func = find_function_info( pc, (ULONG_PTR)(*module)->DllBase, func, size/sizeof(*func) );
-        }
-    }
-    else
-    {
-        *module = NULL;
-
-        RtlEnterCriticalSection( &dynamic_unwind_section );
-        LIST_FOR_EACH_ENTRY( entry, &dynamic_unwind_list, struct dynamic_unwind_entry, entry )
-        {
-            if (pc >= entry->base && pc < entry->end)
-            {
-                *base = entry->base;
-                /* use callback or lookup in function table */
-                if (entry->callback)
-                    func = entry->callback( pc, entry->context );
-                else
-                    func = find_function_info( pc, entry->base, entry->table, entry->count );
-                break;
-            }
-        }
-        RtlLeaveCriticalSection( &dynamic_unwind_section );
-    }
-
-    return func;
-}
-
-/**********************************************************************
- *              RtlLookupFunctionEntry   (NTDLL.@)
- */
-PRUNTIME_FUNCTION WINAPI RtlLookupFunctionEntry( ULONG_PTR pc, ULONG_PTR *base,
-                                                 UNWIND_HISTORY_TABLE *table )
-{
-    LDR_DATA_TABLE_ENTRY *module;
-    RUNTIME_FUNCTION *func;
-
-    /* FIXME: should use the history table to make things faster */
-
-    if (!(func = lookup_function_info( pc, base, &module )))
-    {
-        *base = 0;
-        WARN( "no exception table found for %Ix\n", pc );
-    }
-    return func;
-}
-
-#endif  /* __x86_64__ || __arm__ || __aarch64__ */
-
-
 /*************************************************************
  *            _assert
  */
@@ -661,7 +450,7 @@ void __cdecl __wine_spec_unimplemented_stub( const char *module, const char *fun
     EXCEPTION_RECORD record;
 
     record.ExceptionCode    = EXCEPTION_WINE_STUB;
-    record.ExceptionFlags   = EH_NONCONTINUABLE;
+    record.ExceptionFlags   = EXCEPTION_NONCONTINUABLE;
     record.ExceptionRecord  = NULL;
     record.ExceptionAddress = __wine_spec_unimplemented_stub;
     record.NumberParameters = 2;
@@ -721,6 +510,41 @@ __ASM_GLOBAL_IMPORT(IsBadStringPtrA)
 __ASM_GLOBAL_IMPORT(IsBadStringPtrW)
 #endif
 
+
+/*************************************************************************
+ *		RtlCaptureStackBackTrace (NTDLL.@)
+ */
+USHORT WINAPI RtlCaptureStackBackTrace( ULONG skip, ULONG count, void **buffer, ULONG *hash_ret )
+{
+    ULONG i, ret, hash;
+
+    TRACE( "(%lu, %lu, %p, %p)\n", skip, count, buffer, hash_ret );
+
+    skip++;  /* skip our own frame */
+    ret = RtlWalkFrameChain( buffer, count + skip, skip << 8 );
+    if (hash_ret)
+    {
+        for (i = hash = 0; i < ret; i++) hash += (ULONG_PTR)buffer[i];
+        *hash_ret = hash;
+    }
+    return ret;
+}
+
+
+/*************************************************************************
+ *		RtlGetCallersAddress (NTDLL.@)
+ */
+void WINAPI RtlGetCallersAddress( void **caller, void **parent )
+{
+    void *buffer[2];
+    ULONG count = ARRAY_SIZE(buffer), skip = 2;  /* skip our frame and the parent */
+
+    count = RtlWalkFrameChain( buffer, count + skip, skip << 8 );
+    *caller = count > 0 ? buffer[0] : NULL;
+    *parent = count > 1 ? buffer[1] : NULL;
+}
+
+
 /**********************************************************************
  *              RtlGetEnabledExtendedFeatures   (NTDLL.@)
  */
@@ -797,6 +621,52 @@ static const struct context_parameters *context_get_parameters( ULONG context_fl
     return NULL;
 }
 
+/* offset is from the start of XSAVE_AREA_HEADER. */
+static int next_compacted_xstate_offset( int off, UINT64 compaction_mask, int feature_idx )
+{
+    const UINT64 feature_mask = (UINT64)1 << feature_idx;
+
+    if (compaction_mask & feature_mask) off += user_shared_data->XState.Features[feature_idx].Size;
+    if (user_shared_data->XState.AlignedFeatures & (feature_mask << 1))
+        off = (off + 63) & ~63;
+    return off;
+}
+
+/* size includes XSAVE_AREA_HEADER but not XSAVE_FORMAT (legacy save area). */
+static int xstate_get_compacted_size( UINT64 mask )
+{
+    UINT64 compaction_mask;
+    unsigned int i;
+    int off;
+
+    compaction_mask = ((UINT64)1 << 63) | mask;
+    mask >>= 2;
+    off = sizeof(XSAVE_AREA_HEADER);
+    i = 2;
+    while (mask)
+    {
+        if (mask == 1) return off + user_shared_data->XState.Features[i].Size;
+        off = next_compacted_xstate_offset( off, compaction_mask, i );
+        mask >>= 1;
+        ++i;
+    }
+    return off;
+}
+
+static int xstate_get_size( UINT64 mask )
+{
+    unsigned int i;
+
+    mask >>= 2;
+    if (!mask) return sizeof(XSAVE_AREA_HEADER);
+    i = 2;
+    while (mask != 1)
+    {
+        mask >>= 1;
+        ++i;
+    }
+    return user_shared_data->XState.Features[i].Offset + user_shared_data->XState.Features[i].Size - sizeof(XSAVE_FORMAT);
+}
 
 /**********************************************************************
  *              RtlGetExtendedContextLength2    (NTDLL.@)
@@ -822,12 +692,12 @@ NTSTATUS WINAPI RtlGetExtendedContextLength2( ULONG context_flags, ULONG *length
     if (!(supported_mask = RtlGetEnabledExtendedFeatures( ~(ULONG64)0) ))
         return STATUS_NOT_SUPPORTED;
 
-    compaction_mask &= supported_mask;
+    size = p->context_size + p->context_ex_size + 63;
 
-    size = p->context_size + p->context_ex_size + offsetof(XSTATE, YmmContext) + 63;
-
-    if (compaction_mask & supported_mask & (1 << XSTATE_AVX))
-        size += sizeof(YMMCONTEXT);
+    compaction_mask &= supported_mask & ~(ULONG64)3;
+    if (user_shared_data->XState.CompactionEnabled) size += xstate_get_compacted_size( compaction_mask );
+    else if (compaction_mask)                       size += xstate_get_size( compaction_mask );
+    else                                            size += sizeof(XSAVE_AREA_HEADER);
 
     *length = size;
     return STATUS_SUCCESS;
@@ -878,11 +748,11 @@ NTSTATUS WINAPI RtlInitializeExtendedContext2( void *context, ULONG context_flag
         xs = (XSTATE *)(((ULONG_PTR)c_ex + p->context_ex_size + 63) & ~(ULONG_PTR)63);
 
         c_ex->XState.Offset = (ULONG_PTR)xs - (ULONG_PTR)c_ex;
-        c_ex->XState.Length = offsetof(XSTATE, YmmContext);
         compaction_mask &= supported_mask;
 
-        if (compaction_mask & (1 << XSTATE_AVX))
-            c_ex->XState.Length += sizeof(YMMCONTEXT);
+        if (user_shared_data->XState.CompactionEnabled) c_ex->XState.Length = xstate_get_compacted_size( compaction_mask );
+        else if (compaction_mask & ~(ULONG64)3)         c_ex->XState.Length = xstate_get_size( compaction_mask );
+        else                                            c_ex->XState.Length = sizeof(XSAVE_AREA_HEADER);
 
         memset( xs, 0, c_ex->XState.Length );
         if (user_shared_data->XState.CompactionEnabled)
@@ -916,6 +786,10 @@ NTSTATUS WINAPI RtlInitializeExtendedContext( void *context, ULONG context_flags
 void * WINAPI RtlLocateExtendedFeature2( CONTEXT_EX *context_ex, ULONG feature_id,
         XSTATE_CONFIGURATION *xstate_config, ULONG *length )
 {
+    UINT64 feature_mask = (ULONG64)1 << feature_id;
+    XSAVE_AREA_HEADER *xs;
+    unsigned int offset, i;
+
     TRACE( "context_ex %p, feature_id %lu, xstate_config %p, length %p.\n",
             context_ex, feature_id, xstate_config, length );
 
@@ -931,16 +805,31 @@ void * WINAPI RtlLocateExtendedFeature2( CONTEXT_EX *context_ex, ULONG feature_i
         return NULL;
     }
 
-    if (feature_id != XSTATE_AVX)
+    if (feature_id < 2 || feature_id >= 64)
         return NULL;
 
+    xs = (XSAVE_AREA_HEADER *)((BYTE *)context_ex + context_ex->XState.Offset);
+
     if (length)
-        *length = sizeof(YMMCONTEXT);
+        *length = xstate_config->Features[feature_id].Size;
 
-    if (context_ex->XState.Length < sizeof(XSTATE))
+    if (xstate_config->CompactionEnabled)
+    {
+        if (!(xs->CompactionMask & feature_mask)) return NULL;
+        offset = sizeof(XSAVE_AREA_HEADER);
+        for (i = 2; i < feature_id; ++i)
+            offset = next_compacted_xstate_offset( offset, xs->CompactionMask, i );
+    }
+    else
+    {
+        if (!(feature_mask & xstate_config->EnabledFeatures)) return NULL;
+        offset = xstate_config->Features[feature_id].Offset - sizeof(XSAVE_FORMAT);
+    }
+
+    if (context_ex->XState.Length < offset + xstate_config->Features[feature_id].Size)
         return NULL;
 
-    return (BYTE *)context_ex + context_ex->XState.Offset + offsetof(XSTATE, YmmContext);
+    return (BYTE *)xs + offset;
 }
 
 
@@ -1070,8 +959,9 @@ NTSTATUS WINAPI RtlCopyContext( CONTEXT *dst, DWORD context_flags, CONTEXT *src
 NTSTATUS WINAPI RtlCopyExtendedContext( CONTEXT_EX *dst, ULONG context_flags, CONTEXT_EX *src )
 {
     const struct context_parameters *p;
-    XSTATE *dst_xs, *src_xs;
+    XSAVE_AREA_HEADER *dst_xs, *src_xs;
     ULONG64 feature_mask;
+    unsigned int i, off, size;
 
     TRACE( "dst %p, context_flags %#lx, src %p.\n", dst, context_flags, src );
 
@@ -1086,18 +976,35 @@ NTSTATUS WINAPI RtlCopyExtendedContext( CONTEXT_EX *dst, ULONG context_flags, CO
     if (!(context_flags & 0x40))
         return STATUS_SUCCESS;
 
-    if (dst->XState.Length < offsetof(XSTATE, YmmContext))
+    if (dst->XState.Length < sizeof(XSAVE_AREA_HEADER))
         return STATUS_BUFFER_OVERFLOW;
 
-    dst_xs = (XSTATE *)((BYTE *)dst + dst->XState.Offset);
-    src_xs = (XSTATE *)((BYTE *)src + src->XState.Offset);
+    dst_xs = (XSAVE_AREA_HEADER *)((BYTE *)dst + dst->XState.Offset);
+    src_xs = (XSAVE_AREA_HEADER *)((BYTE *)src + src->XState.Offset);
 
-    memset(dst_xs, 0, offsetof(XSTATE, YmmContext));
+    memset(dst_xs, 0, sizeof(XSAVE_AREA_HEADER));
     dst_xs->Mask = (src_xs->Mask & ~(ULONG64)3) & feature_mask;
     dst_xs->CompactionMask = user_shared_data->XState.CompactionEnabled
             ? ((ULONG64)1 << 63) | (src_xs->CompactionMask & feature_mask) : 0;
 
-    if (dst_xs->Mask & 4 && src->XState.Length >= sizeof(XSTATE) && dst->XState.Length >= sizeof(XSTATE))
-        memcpy( &dst_xs->YmmContext, &src_xs->YmmContext, sizeof(dst_xs->YmmContext) );
+
+    if (dst_xs->CompactionMask) feature_mask &= dst_xs->CompactionMask;
+    feature_mask = dst_xs->Mask >> 2;
+
+    i = 2;
+    off = sizeof(XSAVE_AREA_HEADER);
+    while (1)
+    {
+        if (feature_mask & 1)
+        {
+            if (!dst_xs->CompactionMask) off = user_shared_data->XState.Features[i].Offset - sizeof(XSAVE_FORMAT);
+            size = user_shared_data->XState.Features[i].Size;
+            if (src->XState.Length < off + size || dst->XState.Length < off + size) break;
+            memcpy( (BYTE *)dst_xs + off, (BYTE *)src_xs + off, size );
+        }
+        if (!(feature_mask >>= 1)) break;
+        if (dst_xs->CompactionMask) off = next_compacted_xstate_offset( off, dst_xs->CompactionMask, i);
+        ++i;
+    }
     return STATUS_SUCCESS;
 }
