@@ -21,10 +21,12 @@
  */
 
 #include <limits.h>
+#include <errno.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdint.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -140,17 +142,185 @@ void * __cdecl memmove( void *dst, const void *src, size_t n )
 }
 
 
+/*********************************************************************
+ *		memcpy_s (MSVCRT.@)
+ */
+errno_t __cdecl memcpy_s( void *dst, size_t len, const void *src, size_t count )
+{
+    if (!count) return 0;
+    if (!dst) return EINVAL;
+    if (!src)
+    {
+        memset( dst, 0, len );
+        return EINVAL;
+    }
+    if (count > len)
+    {
+        memset( dst, 0, len );
+        return ERANGE;
+    }
+    memmove( dst, src, count );
+    return 0;
+}
+
+
+/*********************************************************************
+ *		memmove_s (MSVCRT.@)
+ */
+errno_t __cdecl memmove_s( void *dst, size_t len, const void *src, size_t count )
+{
+    if (!count) return 0;
+    if (!dst) return EINVAL;
+    if (!src) return EINVAL;
+    if (count > len) return ERANGE;
+    memmove( dst, src, count );
+    return 0;
+}
+
+
+static inline void memset_aligned_32( unsigned char *d, uint64_t v, size_t n )
+{
+    unsigned char *end = d + n;
+    while (d < end)
+    {
+        *(uint64_t *)(d + 0) = v;
+        *(uint64_t *)(d + 8) = v;
+        *(uint64_t *)(d + 16) = v;
+        *(uint64_t *)(d + 24) = v;
+        d += 32;
+    }
+}
+
 /*********************************************************************
  *                  memset   (NTDLL.@)
  */
-void * __cdecl memset( void *dst, int c, size_t n )
+void *__cdecl memset( void *dst, int c, size_t n )
 {
-    volatile unsigned char *d = dst;  /* avoid gcc optimizations */
-    while (n--) *d++ = c;
+    typedef uint64_t DECLSPEC_ALIGN(1) unaligned_ui64;
+    typedef uint32_t DECLSPEC_ALIGN(1) unaligned_ui32;
+    typedef uint16_t DECLSPEC_ALIGN(1) unaligned_ui16;
+
+    uint64_t v = 0x101010101010101ull * (unsigned char)c;
+    unsigned char *d = (unsigned char *)dst;
+    size_t a = 0x20 - ((uintptr_t)d & 0x1f);
+
+    if (n >= 16)
+    {
+        *(unaligned_ui64 *)(d + 0) = v;
+        *(unaligned_ui64 *)(d + 8) = v;
+        *(unaligned_ui64 *)(d + n - 16) = v;
+        *(unaligned_ui64 *)(d + n - 8) = v;
+        if (n <= 32) return dst;
+        *(unaligned_ui64 *)(d + 16) = v;
+        *(unaligned_ui64 *)(d + 24) = v;
+        *(unaligned_ui64 *)(d + n - 32) = v;
+        *(unaligned_ui64 *)(d + n - 24) = v;
+        if (n <= 64) return dst;
+
+        n = (n - a) & ~0x1f;
+        memset_aligned_32( d + a, v, n );
+        return dst;
+    }
+    if (n >= 8)
+    {
+        *(unaligned_ui64 *)d = v;
+        *(unaligned_ui64 *)(d + n - 8) = v;
+        return dst;
+    }
+    if (n >= 4)
+    {
+        *(unaligned_ui32 *)d = v;
+        *(unaligned_ui32 *)(d + n - 4) = v;
+        return dst;
+    }
+    if (n >= 2)
+    {
+        *(unaligned_ui16 *)d = v;
+        *(unaligned_ui16 *)(d + n - 2) = v;
+        return dst;
+    }
+    if (n >= 1)
+    {
+        *(uint8_t *)d = v;
+        return dst;
+    }
     return dst;
 }
 
 
+/******************************************************************************
+ *                  RtlCopyMemory   (NTDLL.@)
+ */
+#undef RtlCopyMemory
+void WINAPI RtlCopyMemory( void *dest, const void *src, SIZE_T len )
+{
+    memcpy( dest, src, len );
+}
+
+
+/******************************************************************************
+ *                  RtlMoveMemory   (NTDLL.@)
+ */
+#undef RtlMoveMemory
+void WINAPI RtlMoveMemory( void *dest, const void *src, SIZE_T len )
+{
+    memmove( dest, src, len );
+}
+
+
+/******************************************************************************
+ *                  RtlFillMemory   (NTDLL.@)
+ */
+#undef RtlFillMemory
+void WINAPI RtlFillMemory( VOID *dest, SIZE_T len, BYTE fill )
+{
+    memset( dest, fill, len );
+}
+
+
+/******************************************************************************
+ *                  RtlZeroMemory   (NTDLL.@)
+ */
+#undef RtlZeroMemory
+void WINAPI RtlZeroMemory( VOID *dest, SIZE_T len )
+{
+    memset( dest, 0, len );
+}
+
+
+/******************************************************************************
+ *                  RtlCompareMemory   (NTDLL.@)
+ */
+SIZE_T WINAPI RtlCompareMemory( const void *src1, const void *src2, SIZE_T len )
+{
+    SIZE_T i = 0;
+    while (i < len && ((const BYTE *)src1)[i] == ((const BYTE *)src2)[i]) i++;
+    return i;
+}
+
+
+/******************************************************************************
+ *                  RtlCompareMemoryUlong   (NTDLL.@)
+ */
+SIZE_T WINAPI RtlCompareMemoryUlong( void *src, SIZE_T len, ULONG val )
+{
+    SIZE_T i = 0;
+    len /= sizeof(ULONG);
+    while (i < len && ((ULONG *)src)[i] == val) i++;
+    return i * sizeof(ULONG);
+}
+
+
+/*************************************************************************
+ *                  RtlFillMemoryUlong   (NTDLL.@)
+ */
+void WINAPI RtlFillMemoryUlong( ULONG *dest, ULONG len, ULONG val )
+{
+    len /= sizeof(ULONG);
+    while (len--) *dest++ = val;
+}
+
+
 /*********************************************************************
  *                  strcat   (NTDLL.@)
  */
@@ -163,6 +333,26 @@ char * __cdecl strcat( char *dst, const char *src )
 }
 
 
+/*********************************************************************
+ *                  strcat_s   (NTDLL.@)
+ */
+errno_t __cdecl strcat_s( char *dst, size_t len, const char *src )
+{
+    size_t i, j;
+
+    if (!dst || !len) return EINVAL;
+    if (!src)
+    {
+        *dst = 0;
+        return EINVAL;
+    }
+    for (i = 0; i < len; i++) if (!dst[i]) break;
+    for (j = 0; (j + i) < len; j++) if (!(dst[j + i] = src[j])) return 0;
+    *dst = 0;
+    return ERANGE;
+}
+
+
 /*********************************************************************
  *                  strchr   (NTDLL.@)
  */
@@ -196,6 +386,26 @@ char * __cdecl strcpy( char *dst, const char *src )
 }
 
 
+/*********************************************************************
+ *                  strcpy_s   (NTDLL.@)
+ */
+errno_t __cdecl strcpy_s( char *dst, size_t len, const char *src )
+{
+    size_t i;
+
+    if (!dst || !len) return EINVAL;
+    if (!src)
+    {
+        *dst = 0;
+        return EINVAL;
+    }
+
+    for (i = 0; i < len; i++) if (!(dst[i] = src[i])) return 0;
+    *dst = 0;
+    return ERANGE;
+}
+
+
 /*********************************************************************
  *                  strcspn   (NTDLL.@)
  */
@@ -231,6 +441,47 @@ char * __cdecl strncat( char *dst, const char *src, size_t len )
 }
 
 
+/*********************************************************************
+ *                  strncat_s   (NTDLL.@)
+ */
+errno_t __cdecl strncat_s( char *dst, size_t len, const char *src, size_t count )
+{
+    size_t i, j;
+
+    if (!dst || !len) return EINVAL;
+    if (!count) return 0;
+    if (!src)
+    {
+        *dst = 0;
+        return EINVAL;
+    }
+
+    for (i = 0; i < len; i++) if (!dst[i]) break;
+
+    if (i == len)
+    {
+        *dst = 0;
+        return EINVAL;
+    }
+
+    for (j = 0; (j + i) < len; j++)
+    {
+        if (count == _TRUNCATE && j + i == len - 1)
+        {
+            dst[j + i] = 0;
+            return STRUNCATE;
+        }
+        if (j == count || !(dst[j + i] = src[j]))
+        {
+            dst[j + i] = 0;
+            return 0;
+        }
+    }
+    *dst = 0;
+    return ERANGE;
+}
+
+
 /*********************************************************************
  *                  strncmp   (NTDLL.@)
  */
@@ -255,12 +506,54 @@ char * __cdecl strncpy( char *dst, const char *src, size_t len )
 }
 
 
+/*********************************************************************
+ *                  strncpy_s   (NTDLL.@)
+ */
+errno_t __cdecl strncpy_s( char *dst, size_t len, const char *src, size_t count )
+{
+    size_t i, end;
+
+    if (!count)
+    {
+        if (dst && len) *dst = 0;
+        return 0;
+    }
+    if (!dst || !len) return EINVAL;
+    if (!src)
+    {
+        *dst = 0;
+        return EINVAL;
+    }
+
+    if (count != _TRUNCATE && count < len)
+        end = count;
+    else
+        end = len - 1;
+
+    for (i = 0; i < end; i++)
+        if (!(dst[i] = src[i])) return 0;
+
+    if (count == _TRUNCATE)
+    {
+        dst[i] = 0;
+        return STRUNCATE;
+    }
+    if (end == count)
+    {
+        dst[i] = 0;
+        return 0;
+    }
+    dst[0] = 0;
+    return ERANGE;
+}
+
+
 /*********************************************************************
  *                  strnlen   (NTDLL.@)
  */
 size_t __cdecl strnlen( const char *str, size_t len )
 {
-    const char *s = str;
+    const char *s;
     for (s = str; len && *s; s++, len--) ;
     return s - str;
 }
@@ -416,11 +709,29 @@ int __cdecl _stricmp( LPCSTR str1, LPCSTR str2 )
 LPSTR __cdecl _strupr( LPSTR str )
 {
     LPSTR ret = str;
-    for ( ; *str; str++) *str = RtlUpperChar(*str);
+    for ( ; *str; str++) if (*str >= 'a' && *str <= 'z') *str += 'A' + 'a';
     return ret;
 }
 
 
+/*********************************************************************
+ *                  _strupr_s   (NTDLL.@)
+ */
+errno_t __cdecl _strupr_s( char *str, size_t len )
+{
+    if (!str) return EINVAL;
+
+    if (strnlen( str, len ) == len)
+    {
+        *str = 0;
+        return EINVAL;
+    }
+
+    _strupr( str );
+    return 0;
+}
+
+
 /*********************************************************************
  *                  _strlwr   (NTDLL.@)
  *
@@ -436,25 +747,40 @@ LPSTR __cdecl _strupr( LPSTR str )
 LPSTR __cdecl _strlwr( LPSTR str )
 {
     LPSTR ret = str;
-    for ( ; *str; str++) *str = tolower(*str);
+    for ( ; *str; str++) if (*str >= 'A' && *str <= 'Z') *str += 'a' - 'A';
     return ret;
 }
 
 
+/*********************************************************************
+ *                  _strlwr_s   (NTDLL.@)
+ */
+errno_t __cdecl _strlwr_s( char *str, size_t len )
+{
+    if (!str) return EINVAL;
+
+    if (strnlen( str, len ) == len)
+    {
+        *str = 0;
+        return EINVAL;
+    }
+    _strlwr( str );
+    return 0;
+}
+
+
 /*********************************************************************
  *                  toupper   (NTDLL.@)
  */
 int __cdecl toupper( int c )
 {
-    char str[2], *p = str;
+    char str[4], *p = str;
     WCHAR wc;
     DWORD len;
 
-    str[0] = c;
-    str[1] = c >> 8;
+    memcpy( str, &c, sizeof(c) );
     wc = RtlAnsiCharToUnicodeChar( &p );
-    wc = RtlUpcaseUnicodeChar( wc );
-    RtlUnicodeToMultiByteN( str, sizeof(str), &len, &wc, sizeof(wc) );
+    if (RtlUpcaseUnicodeToMultiByteN( str, 2, &len, &wc, sizeof(wc) )) return c;
     if (len == 2) return ((unsigned char)str[0] << 8) + (unsigned char)str[1];
     return (unsigned char)str[0];
 }
@@ -613,6 +939,33 @@ int CDECL _tolower(int c)
 }
 
 
+/*********************************************************************
+ *                  strtok_s   (NTDLL.@)
+ */
+char * __cdecl strtok_s( char *str, const char *delim, char **ctx )
+{
+    char *next;
+
+    if (!delim || !ctx) return NULL;
+    if (!str)
+    {
+        str = *ctx;
+        if (!str) return NULL;
+    }
+    while (*str && strchr( delim, *str )) str++;
+    if (!*str)
+    {
+        *ctx = str;
+        return NULL;
+    }
+    next = str + 1;
+    while (*next && !strchr( delim, *next )) next++;
+    if (*next) *next++ = 0;
+    *ctx = next;
+    return str;
+}
+
+
 static int char_to_int( char c )
 {
     if ('0' <= c && c <= '9') return c - '0';
@@ -933,6 +1286,125 @@ char * __cdecl _i64toa(
 }
 
 
+/*********************************************************************
+ *      _ui64toa_s  (NTDLL.@)
+ */
+errno_t __cdecl _ui64toa_s( unsigned __int64 value, char *str, size_t size, int radix )
+{
+    char buffer[65], *pos;
+
+    if (!str || !size) return EINVAL;
+    if (radix < 2 || radix > 36)
+    {
+        str[0] = 0;
+        return EINVAL;
+    }
+
+    pos = buffer + 64;
+    *pos = 0;
+
+    do {
+	int digit = value % radix;
+	value = value / radix;
+	if (digit < 10)
+	    *--pos = '0' + digit;
+	else
+	    *--pos = 'a' + digit - 10;
+    } while (value != 0);
+
+    if (buffer - pos + 65 > size)
+    {
+        str[0] = 0;
+        return ERANGE;
+    }
+    memcpy( str, pos, buffer - pos + 65 );
+    return 0;
+}
+
+
+/*********************************************************************
+ *      _ultoa_s  (NTDLL.@)
+ */
+errno_t __cdecl _ultoa_s( __msvcrt_ulong value, char *str, size_t size, int radix )
+{
+    return _ui64toa_s( value, str, size, radix );
+}
+
+
+/*********************************************************************
+ *      _i64toa_s  (NTDLL.@)
+ */
+errno_t __cdecl _i64toa_s( __int64 value, char *str, size_t size, int radix )
+{
+    unsigned __int64 val;
+    BOOL is_negative;
+    char buffer[65], *pos;
+
+    if (!str || !size) return EINVAL;
+    if (radix < 2 || radix > 36)
+    {
+        str[0] = 0;
+        return EINVAL;
+    }
+
+    if (value < 0 && radix == 10)
+    {
+        is_negative = TRUE;
+        val = -value;
+    }
+    else
+    {
+        is_negative = FALSE;
+        val = value;
+    }
+
+    pos = buffer + 64;
+    *pos = 0;
+
+    do
+    {
+        unsigned int digit = val % radix;
+        val /= radix;
+
+        if (digit < 10)
+            *--pos = '0' + digit;
+        else
+            *--pos = 'a' + digit - 10;
+    }
+    while (val != 0);
+
+    if (is_negative) *--pos = '-';
+
+    if (buffer - pos + 65 > size)
+    {
+        str[0] = 0;
+        return ERANGE;
+    }
+    memcpy( str, pos, buffer - pos + 65 );
+    return 0;
+}
+
+
+/*********************************************************************
+ *      _ltoa_s  (NTDLL.@)
+ */
+errno_t __cdecl _ltoa_s( __msvcrt_long value, char *str, size_t size, int radix )
+{
+    if (value < 0 && radix == 10) return _i64toa_s( value, str, size, radix );
+    return _ui64toa_s( (__msvcrt_ulong)value, str, size, radix );
+}
+
+
+/*********************************************************************
+ *      _itoa_s  (NTDLL.@)
+ */
+errno_t __cdecl _itoa_s( int value, char *str, size_t size, int radix )
+{
+    if (value < 0 && radix == 10) return _i64toa_s( value, str, size, radix );
+    return _ui64toa_s( (unsigned int)value, str, size, radix );
+}
+
+
 /*********************************************************************
  *      _atoi64   (NTDLL.@)
  *
@@ -1007,7 +1479,7 @@ static int char2digit( char c, int base )
 }
 
 
-static int vsscanf( const char *str, const char *format, __ms_va_list ap)
+static int vsscanf( const char *str, const char *format, va_list ap)
 {
     int rd = 0, consumed = 0;
     int nch;
@@ -1445,14 +1917,90 @@ static int vsscanf( const char *str, const char *format, __ms_va_list ap)
 int WINAPIV sscanf( const char *str, const char *format, ... )
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start( valist, format );
+    va_list valist;
+    va_start( valist, format );
     ret = vsscanf( str, format, valist );
-    __ms_va_end( valist );
+    va_end( valist );
     return ret;
 }
 
 
+/******************************************************************
+ *      _splitpath_s   (NTDLL.@)
+ */
+errno_t __cdecl _splitpath_s( const char *inpath, char *drive, size_t sz_drive,
+                              char *dir, size_t sz_dir, char *fname, size_t sz_fname,
+                              char *ext, size_t sz_ext )
+{
+    const char *p, *end;
+
+    if (!inpath || (!drive && sz_drive) ||
+        (drive && !sz_drive) ||
+        (!dir && sz_dir) ||
+        (dir && !sz_dir) ||
+        (!fname && sz_fname) ||
+        (fname && !sz_fname) ||
+        (!ext && sz_ext) ||
+        (ext && !sz_ext))
+        return EINVAL;
+
+    if (inpath[0] && inpath[1] == ':')
+    {
+        if (drive)
+        {
+            if (sz_drive <= 2) goto error;
+            drive[0] = inpath[0];
+            drive[1] = inpath[1];
+            drive[2] = 0;
+        }
+        inpath += 2;
+    }
+    else if (drive) drive[0] = '\0';
+
+    /* look for end of directory part */
+    end = NULL;
+    for (p = inpath; *p; p++) if (*p == '/' || *p == '\\') end = p + 1;
+
+    if (end)  /* got a directory */
+    {
+        if (dir)
+        {
+            if (sz_dir <= end - inpath) goto error;
+            memcpy( dir, inpath, end - inpath );
+            dir[end - inpath] = 0;
+        }
+        inpath = end;
+    }
+    else if (dir) dir[0] = 0;
+
+    /* look for extension: what's after the last dot */
+    end = NULL;
+    for (p = inpath; *p; p++) if (*p == '.') end = p;
+
+    if (!end) end = p; /* there's no extension */
+
+    if (fname)
+    {
+        if (sz_fname <= end - inpath) goto error;
+        memcpy( fname, inpath, end - inpath );
+        fname[end - inpath] = 0;
+    }
+    if (ext)
+    {
+        if (sz_ext <= strlen(end)) goto error;
+        strcpy( ext, end );
+    }
+    return 0;
+
+error:
+    if (drive) drive[0] = 0;
+    if (dir) dir[0] = 0;
+    if (fname) fname[0]= 0;
+    if (ext) ext[0]= 0;
+    return ERANGE;
+}
+
+
 /*********************************************************************
  *		_splitpath (NTDLL.@)
  *
@@ -1471,45 +2019,83 @@ int WINAPIV sscanf( const char *str, const char *format, ... )
 void __cdecl _splitpath(const char* inpath, char * drv, char * dir,
                         char* fname, char * ext )
 {
-    const char *p, *end;
+    _splitpath_s( inpath, drv, drv ? _MAX_DRIVE : 0, dir, dir ? _MAX_DIR : 0,
+                  fname, fname ? _MAX_FNAME : 0, ext, ext ? _MAX_EXT : 0 );
+}
 
-    if (inpath[0] && inpath[1] == ':')
+
+/*********************************************************************
+ *      _makepath_s   (NTDLL.@)
+ */
+errno_t __cdecl _makepath_s( char *path, size_t size, const char *drive,
+                             const char *directory, const char *filename,
+                             const char *extension )
+{
+    char *p = path;
+
+    if (!path || !size) return EINVAL;
+
+    if (drive && drive[0])
     {
-        if (drv)
+        if (size <= 2) goto range;
+        *p++ = drive[0];
+        *p++ = ':';
+        size -= 2;
+    }
+
+    if (directory && directory[0])
+    {
+        unsigned int len = strlen(directory);
+        unsigned int needs_separator = directory[len - 1] != '/' && directory[len - 1] != '\\';
+        unsigned int copylen = min(size - 1, len);
+
+        if (size < 2) goto range;
+        memmove(p, directory, copylen);
+        if (size <= len) goto range;
+        p += copylen;
+        size -= copylen;
+        if (needs_separator)
         {
-            drv[0] = inpath[0];
-            drv[1] = inpath[1];
-            drv[2] = 0;
+            if (size < 2) goto range;
+            *p++ = '\\';
+            size -= 1;
         }
-        inpath += 2;
     }
-    else if (drv) drv[0] = 0;
 
-    /* look for end of directory part */
-    end = NULL;
-    for (p = inpath; *p; p++) if (*p == '/' || *p == '\\') end = p + 1;
+    if (filename && filename[0])
+    {
+        unsigned int len = strlen(filename);
+        unsigned int copylen = min(size - 1, len);
+
+        if (size < 2) goto range;
+        memmove(p, filename, copylen);
+        if (size <= len) goto range;
+        p += len;
+        size -= len;
+    }
 
-    if (end)  /* got a directory */
+    if (extension && extension[0])
     {
-        if (dir)
+        unsigned int len = strlen(extension);
+        unsigned int needs_period = extension[0] != '.';
+        unsigned int copylen;
+
+        if (size < 2) goto range;
+        if (needs_period)
         {
-            memcpy( dir, inpath, end - inpath );
-            dir[end - inpath] = 0;
+            *p++ = '.';
+            size -= 1;
         }
-        inpath = end;
+        copylen = min(size - 1, len);
+        memcpy(p, extension, copylen);
+        if (size <= len) goto range;
+        p += copylen;
     }
-    else if (dir) dir[0] = 0;
 
-    /* look for extension: what's after the last dot */
-    end = NULL;
-    for (p = inpath; *p; p++) if (*p == '.') end = p;
+    *p = 0;
+    return 0;
 
-    if (!end) end = p; /* there's no extension */
-
-    if (fname)
-    {
-        memcpy( fname, inpath, end - inpath );
-        fname[end - inpath] = 0;
-    }
-    if (ext) strcpy( ext, end );
+range:
+    path[0] = 0;
+    return ERANGE;
 }
