@@ -51,10 +51,6 @@
 #ifdef HAVE_SYS_UCONTEXT_H
 # include <sys/ucontext.h>
 #endif
-#ifdef HAVE_LIBUNWIND
-# define UNW_LOCAL_ONLY
-# include <libunwind.h>
-#endif
 #ifdef HAVE_LINK_H
 # include <link.h>
 #endif
@@ -260,529 +256,19 @@ static BOOL is_inside_syscall( ucontext_t *sigcontext )
             (char *)SP_sig(sigcontext) <= (char *)arm_thread_data()->syscall_frame);
 }
 
-extern void raise_func_trampoline( EXCEPTION_RECORD *rec, CONTEXT *context, void *dispatcher );
 
-struct exidx_entry
+void set_process_instrumentation_callback( void *callback )
 {
-    uint32_t addr;
-    uint32_t data;
-};
-
-static uint32_t prel31_to_abs(const uint32_t *ptr)
-{
-    uint32_t prel31 = *ptr;
-    uint32_t rel = prel31 | ((prel31 << 1) & 0x80000000);
-    return (uintptr_t)ptr + rel;
+    if (callback) FIXME( "Not supported.\n" );
 }
 
-static uint8_t get_byte(const uint32_t *ptr, int offset, int bytes)
-{
-    int word = offset >> 2;
-    int byte = offset & 0x3;
-    if (offset >= bytes)
-        return 0xb0; /* finish opcode */
-    return (ptr[word] >> (24 - 8*byte)) & 0xff;
-}
-
-static uint32_t get_uleb128(const uint32_t *ptr, int *offset, int bytes)
-{
-    int shift = 0;
-    uint32_t val = 0;
-    while (1)
-    {
-        uint8_t byte = get_byte(ptr, (*offset)++, bytes);
-        val |= (byte & 0x7f) << shift;
-        if ((byte & 0x80) == 0)
-            break;
-        shift += 7;
-    }
-    return val;
-}
-
-static void pop_regs(CONTEXT *context, uint32_t regs)
-{
-    int i;
-    DWORD new_sp = 0;
-    for (i = 0; i < 16; i++)
-    {
-        if (regs & (1U << i))
-        {
-            DWORD val = *(DWORD *)context->Sp;
-            if (i != 13)
-                (&context->R0)[i] = val;
-            else
-                new_sp = val;
-            context->Sp += 4;
-        }
-    }
-    if (regs & (1 << 13))
-        context->Sp = new_sp;
-}
-
-static void pop_vfp(CONTEXT *context, int first, int last)
-{
-    int i;
-    for (i = first; i <= last; i++)
-    {
-        context->D[i] = *(ULONGLONG *)context->Sp;
-        context->Sp += 8;
-    }
-}
-
-static uint32_t regmask(int first_bit, int n_bits)
-{
-    return ((1U << (n_bits + 1)) - 1) << first_bit;
-}
-
-/***********************************************************************
- *           ehabi_virtual_unwind
- */
-static NTSTATUS ehabi_virtual_unwind( UINT ip, DWORD *frame, CONTEXT *context,
-                                      const struct exidx_entry *entry,
-                                      PEXCEPTION_ROUTINE *handler, void **handler_data )
-{
-    const uint32_t *ptr;
-    const void *lsda = NULL;
-    int compact_inline = 0;
-    int offset = 0;
-    int bytes = 0;
-    int personality;
-    int extra_words;
-    int finish = 0;
-    int set_pc = 0;
-    UINT func_begin = prel31_to_abs(&entry->addr);
-
-    *frame = context->Sp;
-
-    TRACE( "ip %#x function %#x\n", ip, func_begin );
-
-    if (entry->data == 1)
-    {
-        ERR("EXIDX_CANTUNWIND\n");
-        return STATUS_UNSUCCESSFUL;
-    }
-    else if (entry->data & 0x80000000)
-    {
-        if ((entry->data & 0x7f000000) != 0)
-        {
-            ERR("compact inline EXIDX must have personality 0\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-        ptr = &entry->data;
-        compact_inline = 1;
-    }
-    else
-    {
-        ptr = (uint32_t *)prel31_to_abs(&entry->data);
-    }
-
-    if ((*ptr & 0x80000000) == 0)
-    {
-        /* Generic */
-        void *personality_func = (void *)prel31_to_abs(ptr);
-        int words = (ptr[1] >> 24) & 0xff;
-        lsda = ptr + 1 + words + 1;
-
-        ERR("generic EHABI unwinding not supported\n");
-        (void)personality_func;
-        return STATUS_UNSUCCESSFUL;
-    }
-
-    /* Compact */
-
-    personality = (*ptr >> 24) & 0x0f;
-    switch (personality)
-    {
-    case 0:
-        if (!compact_inline)
-            lsda = ptr + 1;
-        extra_words = 0;
-        offset = 1;
-        break;
-    case 1:
-        extra_words = (*ptr >> 16) & 0xff;
-        lsda = ptr + extra_words + 1;
-        offset = 2;
-        break;
-    case 2:
-        extra_words = (*ptr >> 16) & 0xff;
-        lsda = ptr + extra_words + 1;
-        offset = 2;
-        break;
-    default:
-        ERR("unsupported compact EXIDX personality %d\n", personality);
-        return STATUS_UNSUCCESSFUL;
-    }
-
-    /* Not inspecting the descriptors */
-    (void)lsda;
-
-    bytes = 4 + 4*extra_words;
-    while (offset < bytes && !finish)
-    {
-        uint8_t byte = get_byte(ptr, offset++, bytes);
-        if ((byte & 0xc0) == 0x00)
-        {
-            /* Increment Sp */
-            context->Sp += (byte & 0x3f) * 4 + 4;
-        }
-        else if ((byte & 0xc0) == 0x40)
-        {
-            /* Decrement Sp */
-            context->Sp -= (byte & 0x3f) * 4 + 4;
-        }
-        else if ((byte & 0xf0) == 0x80)
-        {
-            /* Pop {r4-r15} based on register mask */
-            int regs = ((byte & 0x0f) << 8) | get_byte(ptr, offset++, bytes);
-            if (!regs)
-            {
-                ERR("refuse to unwind\n");
-                return STATUS_UNSUCCESSFUL;
-            }
-            regs <<= 4;
-            pop_regs(context, regs);
-            if (regs & (1 << 15))
-                set_pc = 1;
-        }
-        else if ((byte & 0xf0) == 0x90)
-        {
-            /* Restore Sp from other register */
-            int reg = byte & 0x0f;
-            if (reg == 13 || reg == 15)
-            {
-                ERR("reserved opcode\n");
-                return STATUS_UNSUCCESSFUL;
-            }
-            context->Sp = (&context->R0)[reg];
-        }
-        else if ((byte & 0xf0) == 0xa0)
-        {
-            /* Pop r4-r(4+n) (+lr) */
-            int n = byte & 0x07;
-            int regs = regmask(4, n);
-            if (byte & 0x08)
-                regs |= 1 << 14;
-            pop_regs(context, regs);
-        }
-        else if (byte == 0xb0)
-        {
-            finish = 1;
-        }
-        else if (byte == 0xb1)
-        {
-            /* Pop {r0-r3} based on register mask */
-            int regs = get_byte(ptr, offset++, bytes);
-            if (regs == 0 || (regs & 0xf0) != 0)
-            {
-                ERR("spare opcode\n");
-                return STATUS_UNSUCCESSFUL;
-            }
-            pop_regs(context, regs);
-        }
-        else if (byte == 0xb2)
-        {
-            /* Increment Sp by a larger amount */
-            int imm = get_uleb128(ptr, &offset, bytes);
-            context->Sp += 0x204 + imm * 4;
-        }
-        else if (byte == 0xb3)
-        {
-            /* Pop VFP registers as if saved by FSTMFDX; this opcode
-             * is deprecated. */
-            ERR("FSTMFDX unsupported\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-        else if ((byte & 0xfc) == 0xb4)
-        {
-            ERR("spare opcode\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-        else if ((byte & 0xf8) == 0xb8)
-        {
-            /* Pop VFP registers as if saved by FSTMFDX; this opcode
-             * is deprecated. */
-            ERR("FSTMFDX unsupported\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-        else if ((byte & 0xf8) == 0xc0)
-        {
-            ERR("spare opcode / iWMMX\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-        else if ((byte & 0xfe) == 0xc8)
-        {
-            /* Pop VFP registers d(16+ssss)-d(16+ssss+cccc), or
-             * d(0+ssss)-d(0+ssss+cccc) as if saved by VPUSH */
-            int first, last;
-            if ((byte & 0x01) == 0)
-                first = 16;
-            else
-                first = 0;
-            byte = get_byte(ptr, offset++, bytes);
-            first += (byte & 0xf0) >> 4;
-            last = first + (byte & 0x0f);
-            if (last >= 32)
-            {
-                ERR("reserved opcode\n");
-                return STATUS_UNSUCCESSFUL;
-            }
-            pop_vfp(context, first, last);
-        }
-        else if ((byte & 0xf8) == 0xc8)
-        {
-            ERR("spare opcode\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-        else if ((byte & 0xf8) == 0xd0)
-        {
-            /* Pop VFP registers d8-d(8+n) as if saved by VPUSH */
-            int n = byte & 0x07;
-            pop_vfp(context, 8, 8 + n);
-        }
-        else
-        {
-            ERR("spare opcode\n");
-            return STATUS_UNSUCCESSFUL;
-        }
-    }
-    if (offset > bytes)
-    {
-        ERR("truncated opcodes\n");
-        return STATUS_UNSUCCESSFUL;
-    }
-
-    *handler      = NULL; /* personality */
-    *handler_data = NULL; /* lsda */
-
-    context->ContextFlags |= CONTEXT_UNWOUND_TO_CALL;
-    if (!set_pc)
-        context->Pc = context->Lr;
-
-    /* There's no need to check for raise_func_trampoline and manually restore
-     * Lr separately from Pc like with libunwind; the EHABI unwind info
-     * describes how both of them are restored separately, and as long as
-     * the unwind info restored Pc, it doesn't have to be set from Lr. */
-
-    TRACE( "next function pc=%08lx\n", context->Pc );
-    TRACE("  r0=%08lx  r1=%08lx  r2=%08lx  r3=%08lx\n",
-          context->R0, context->R1, context->R2, context->R3 );
-    TRACE("  r4=%08lx  r5=%08lx  r6=%08lx  r7=%08lx\n",
-          context->R4, context->R5, context->R6, context->R7 );
-    TRACE("  r8=%08lx  r9=%08lx r10=%08lx r11=%08lx\n",
-          context->R8, context->R9, context->R10, context->R11 );
-    TRACE(" r12=%08lx  sp=%08lx  lr=%08lx  pc=%08lx\n",
-          context->R12, context->Sp, context->Lr, context->Pc );
-
-    return STATUS_SUCCESS;
-}
-
-#ifdef linux
-struct iterate_data
-{
-    ULONG_PTR ip;
-    int failed;
-    struct exidx_entry *entry;
-};
-
-static int contains_addr(struct dl_phdr_info *info, const ElfW(Phdr) *phdr, struct iterate_data *data)
-{
-    if (phdr->p_type != PT_LOAD)
-        return 0;
-    return data->ip >= info->dlpi_addr + phdr->p_vaddr && data->ip < info->dlpi_addr + phdr->p_vaddr + phdr->p_memsz;
-}
-
-static int check_exidx(struct dl_phdr_info *info, size_t info_size, void *arg)
-{
-    struct iterate_data *data = arg;
-    int i;
-    int found_addr;
-    const ElfW(Phdr) *exidx = NULL;
-    struct exidx_entry *begin, *end;
-
-    if (info->dlpi_phnum == 0 || data->ip < info->dlpi_addr || data->failed)
-        return 0;
-
-    found_addr = 0;
-    for (i = 0; i < info->dlpi_phnum; i++)
-    {
-        const ElfW(Phdr) *phdr = &info->dlpi_phdr[i];
-        if (contains_addr(info, phdr, data))
-            found_addr = 1;
-        if (phdr->p_type == PT_ARM_EXIDX)
-            exidx = phdr;
-    }
-
-    if (!found_addr || !exidx)
-    {
-        if (found_addr)
-        {
-            TRACE("found matching address in %s, but no EXIDX\n", info->dlpi_name);
-            data->failed = 1;
-        }
-        return 0;
-    }
-
-    begin = (struct exidx_entry *)(info->dlpi_addr + exidx->p_vaddr);
-    end = (struct exidx_entry *)(info->dlpi_addr + exidx->p_vaddr + exidx->p_memsz);
-    if (data->ip < prel31_to_abs(&begin->addr))
-    {
-        TRACE("%lx before EXIDX start at %x\n", data->ip, prel31_to_abs(&begin->addr));
-        data->failed = 1;
-        return 0;
-    }
-
-    while (begin + 1 < end)
-    {
-        struct exidx_entry *mid = begin + (end - begin)/2;
-        uint32_t abs_addr = prel31_to_abs(&mid->addr);
-        if (abs_addr > data->ip)
-        {
-            end = mid;
-        }
-        else if (abs_addr < data->ip)
-        {
-            begin = mid;
-        }
-        else
-        {
-            begin = mid;
-            end = mid + 1;
-        }
-    }
-
-    data->entry = begin;
-    TRACE("found %lx in %s, base %x, entry %p with addr %x (rel %x) data %x\n",
-          data->ip, info->dlpi_name, info->dlpi_addr, begin,
-          prel31_to_abs(&begin->addr),
-          prel31_to_abs(&begin->addr) - info->dlpi_addr, begin->data);
-    return 1;
-}
-
-static const struct exidx_entry *find_exidx_entry( void *ip )
-{
-    struct iterate_data data = {};
-
-    data.ip = (ULONG_PTR)ip;
-    data.failed = 0;
-    data.entry = NULL;
-    dl_iterate_phdr(check_exidx, &data);
-
-    return data.entry;
-}
-#endif
-
-#ifdef HAVE_LIBUNWIND
-static NTSTATUS libunwind_virtual_unwind( DWORD ip, DWORD *frame, CONTEXT *context,
-                                          PEXCEPTION_ROUTINE *handler, void **handler_data )
-{
-    unw_context_t unw_context;
-    unw_cursor_t cursor;
-    unw_proc_info_t info;
-    int rc, i;
-
-    for (i = 0; i <= 12; i++)
-        unw_context.regs[i] = (&context->R0)[i];
-    unw_context.regs[13] = context->Sp;
-    unw_context.regs[14] = context->Lr;
-    unw_context.regs[15] = context->Pc;
-    rc = unw_init_local( &cursor, &unw_context );
-
-    if (rc != UNW_ESUCCESS)
-    {
-        WARN( "setup failed: %d\n", rc );
-        return STATUS_INVALID_DISPOSITION;
-    }
-    rc = unw_get_proc_info( &cursor, &info );
-    if (UNW_ENOINFO < 0) rc = -rc;  /* LLVM libunwind has negative error codes */
-    if (rc != UNW_ESUCCESS && rc != -UNW_ENOINFO)
-    {
-        WARN( "failed to get info: %d\n", rc );
-        return STATUS_INVALID_DISPOSITION;
-    }
-    if (rc == -UNW_ENOINFO || ip < info.start_ip || ip > info.end_ip)
-    {
-        NTSTATUS status = context->Pc != context->Lr ?
-                          STATUS_SUCCESS : STATUS_INVALID_DISPOSITION;
-        TRACE( "no info found for %x ip %x-%x, %s\n",
-               ip, info.start_ip, info.end_ip, status == STATUS_SUCCESS ?
-               "assuming leaf function" : "error, stuck" );
-        *handler = NULL;
-        *frame = context->Sp;
-        context->Pc = context->Lr;
-        context->ContextFlags |= CONTEXT_UNWOUND_TO_CALL;
-        return status;
-    }
-
-    TRACE( "ip %#x function %#lx-%#lx personality %#lx lsda %#lx fde %#lx\n",
-           ip, (unsigned long)info.start_ip, (unsigned long)info.end_ip, (unsigned long)info.handler,
-           (unsigned long)info.lsda, (unsigned long)info.unwind_info );
-
-    rc = unw_step( &cursor );
-    if (rc < 0)
-    {
-        WARN( "failed to unwind: %d %d\n", rc, UNW_ENOINFO );
-        return STATUS_INVALID_DISPOSITION;
-    }
-
-    *handler      = (void *)info.handler;
-    *handler_data = (void *)info.lsda;
-    *frame        = context->Sp;
-
-    for (i = 0; i <= 12; i++)
-        unw_get_reg( &cursor, UNW_ARM_R0 + i, (unw_word_t *)&(&context->R0)[i] );
-    unw_get_reg( &cursor, UNW_ARM_R13, (unw_word_t *)&context->Sp );
-    unw_get_reg( &cursor, UNW_ARM_R14, (unw_word_t *)&context->Lr );
-    unw_get_reg( &cursor, UNW_REG_IP,  (unw_word_t *)&context->Pc );
-    context->ContextFlags |= CONTEXT_UNWOUND_TO_CALL;
-
-    if ((info.start_ip & ~(unw_word_t)1) ==
-        ((unw_word_t)raise_func_trampoline & ~(unw_word_t)1)) {
-        /* raise_func_trampoline stores the original Lr at the bottom of the
-         * stack. The unwinder normally can't restore both Pc and Lr to
-         * individual values, thus do that manually here.
-         * (The function we unwind to might be a leaf function that hasn't
-         * backed up its own original Lr value on the stack.) */
-        const DWORD *orig_lr = (const DWORD *) *frame;
-        context->Lr = *orig_lr;
-    }
-
-    TRACE( "next function pc=%08lx%s\n", context->Pc, rc ? "" : " (last frame)" );
-    TRACE("  r0=%08lx  r1=%08lx  r2=%08lx  r3=%08lx\n",
-          context->R0, context->R1, context->R2, context->R3 );
-    TRACE("  r4=%08lx  r5=%08lx  r6=%08lx  r7=%08lx\n",
-          context->R4, context->R5, context->R6, context->R7 );
-    TRACE("  r8=%08lx  r9=%08lx r10=%08lx r11=%08lx\n",
-          context->R8, context->R9, context->R10, context->R11 );
-    TRACE(" r12=%08lx  sp=%08lx  lr=%08lx  pc=%08lx\n",
-          context->R12, context->Sp, context->Lr, context->Pc );
-    return STATUS_SUCCESS;
-}
-#endif
 
 /***********************************************************************
  *           unwind_builtin_dll
  */
 NTSTATUS unwind_builtin_dll( void *args )
 {
-    struct unwind_builtin_dll_params *params = args;
-    DISPATCHER_CONTEXT *dispatch = params->dispatch;
-    CONTEXT *context = params->context;
-    DWORD ip = context->Pc - (dispatch->ControlPcIsUnwound ? 2 : 0);
-#ifdef linux
-    const struct exidx_entry *entry = find_exidx_entry( (void *)ip );
-
-    if (entry)
-        return ehabi_virtual_unwind( ip, &dispatch->EstablisherFrame, context, entry,
-                                     &dispatch->LanguageHandler, &dispatch->HandlerData );
-#endif
-#ifdef HAVE_LIBUNWIND
-    return libunwind_virtual_unwind( ip, &dispatch->EstablisherFrame, context,
-                                     &dispatch->LanguageHandler, &dispatch->HandlerData );
-#else
-    ERR("libunwind not available, unable to unwind\n");
-    return STATUS_INVALID_DISPOSITION;
-#endif
+    return STATUS_UNSUCCESSFUL;
 }
 
 
@@ -1033,6 +519,7 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
         memcpy( context->D, frame->d, sizeof(frame->d) );
         context->ContextFlags |= CONTEXT_FLOATING_POINT;
     }
+    set_context_exception_reporting_flags( &context->ContextFlags, CONTEXT_SERVICE_ACTIVE );
     return STATUS_SUCCESS;
 }
 
@@ -1056,33 +543,27 @@ NTSTATUS get_thread_wow64_context( HANDLE handle, void *ctx, ULONG size )
 
 
 /***********************************************************************
- *           setup_exception
- *
- * Modify the signal context to call the exception raise function.
+ *           setup_raise_exception
  */
-static void setup_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec )
+static void setup_raise_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec, CONTEXT *context )
 {
     struct exc_stack_layout *stack;
     void *stack_ptr = (void *)(SP_sig(sigcontext) & ~7);
-    CONTEXT context;
     NTSTATUS status;
 
-    rec->ExceptionAddress = (void *)PC_sig(sigcontext);
-    save_context( &context, sigcontext );
-
-    status = send_debug_event( rec, &context, TRUE );
+    status = send_debug_event( rec, context, TRUE, TRUE );
     if (status == DBG_CONTINUE || status == DBG_EXCEPTION_HANDLED)
     {
-        restore_context( &context, sigcontext );
+        restore_context( context, sigcontext );
         return;
     }
 
     /* fix up instruction pointer in context for EXCEPTION_BREAKPOINT */
-    if (rec->ExceptionCode == EXCEPTION_BREAKPOINT) context.Pc -= 2;
+    if (rec->ExceptionCode == EXCEPTION_BREAKPOINT) context->Pc -= 2;
 
     stack = virtual_setup_exception( stack_ptr, sizeof(*stack), rec );
     stack->rec = *rec;
-    stack->context = context;
+    stack->context = *context;
 
     /* now modify the sigcontext to return to the raise function */
     SP_sig(sigcontext) = (DWORD)stack;
@@ -1092,6 +573,21 @@ static void setup_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec )
 }
 
 
+/***********************************************************************
+ *           setup_exception
+ *
+ * Modify the signal context to call the exception raise function.
+ */
+static void setup_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec )
+{
+    CONTEXT context;
+
+    rec->ExceptionAddress = (void *)PC_sig(sigcontext);
+    save_context( &context, sigcontext );
+    setup_raise_exception( sigcontext, rec, &context );
+}
+
+
 /***********************************************************************
  *           call_user_apc_dispatcher
  */
@@ -1115,10 +611,11 @@ NTSTATUS call_user_apc_dispatcher( CONTEXT *context, ULONG_PTR arg1, ULONG_PTR a
         NtGetContextThread( GetCurrentThread(), &stack->context );
         stack->context.R0 = status;
     }
-    stack->func    = func;
-    stack->args[0] = arg1;
-    stack->args[1] = arg2;
-    stack->args[2] = arg3;
+    stack->func      = func;
+    stack->args[0]   = arg1;
+    stack->args[1]   = arg2;
+    stack->args[2]   = arg3;
+    stack->alertable = TRUE;
 
     frame->sp = (DWORD)stack;
     frame->pc = (DWORD)pKiUserApcDispatcher;
@@ -1132,7 +629,10 @@ NTSTATUS call_user_apc_dispatcher( CONTEXT *context, ULONG_PTR arg1, ULONG_PTR a
  */
 void call_raise_user_exception_dispatcher(void)
 {
-    arm_thread_data()->syscall_frame->pc = (DWORD)pKiRaiseUserExceptionDispatcher;
+    struct syscall_frame *frame = arm_thread_data()->syscall_frame;
+
+    frame->sp += 16;
+    frame->pc = (DWORD)pKiRaiseUserExceptionDispatcher;
 }
 
 
@@ -1178,13 +678,11 @@ __ASM_GLOBAL_FUNC( call_user_mode_callback,
                    "ldr r4, [sp, #0x28]\n\t"  /* teb */
                    "ldr r5, [r4]\n\t"         /* teb->Tib.ExceptionList */
                    "push {r1,r2,r4,r5}\n\t"   /* ret_ptr, ret_len, teb, exception_list */
-#ifndef __SOFTFP__
                    "sub sp, sp, #0x90\n\t"
                    "mov r5, sp\n\t"
                    "vmrs r6, fpscr\n\t"
                    "vstm r5, {d8-d15}\n\t"
                    "str r6, [r5, #0x80]\n\t"
-#endif
                    "sub sp, sp, #0x160\n\t"   /* sizeof(struct syscall_frame) + registers */
                    "ldr r5, [r4, #0x1d8]\n\t" /* arm_thread_data()->syscall_frame */
                    "str r5, [sp, #0x4c]\n\t"  /* frame->prev_frame */
@@ -1205,12 +703,10 @@ __ASM_GLOBAL_FUNC( user_mode_callback_return,
                    "ldr r5, [r4, #0x4c]\n\t"  /* frame->prev_frame */
                    "str r5, [r3, #0x1d8]\n\t" /* arm_thread_data()->syscall_frame */
                    "add r5, r4, #0x160\n\t"
-#ifndef __SOFTFP__
                    "vldm r5, {d8-d15}\n\t"
                    "ldr r6, [r5, #0x80]\n\t"
                    "vmsr fpscr, r6\n\t"
                    "add r5, r5, #0x90\n\t"
-#endif
                    "mov sp, r5\n\t"
                    "pop {r4-r7}\n\t"          /* ret_ptr, ret_len, teb, exception_list */
                    "str r7, [r3]\n\t"         /* teb->Tib.ExceptionList */
@@ -1309,7 +805,7 @@ static BOOL handle_syscall_fault( ucontext_t *context, EXCEPTION_RECORD *rec )
         TRACE( "returning to handler\n" );
         REGn_sig(0, context) = (DWORD)ntdll_get_thread_data()->jmp_buf;
         REGn_sig(1, context) = 1;
-        PC_sig(context)      = (DWORD)__wine_longjmp;
+        PC_sig(context)      = (DWORD)longjmp;
         ntdll_get_thread_data()->jmp_buf = NULL;
     }
     else
@@ -1332,6 +828,10 @@ static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
     EXCEPTION_RECORD rec = { 0 };
     ucontext_t *context = sigcontext;
+    CONTEXT ctx;
+
+    rec.ExceptionAddress = (void *)PC_sig(context);
+    save_context( &ctx, sigcontext );
 
     switch (get_trap_code(signal, context))
     {
@@ -1340,18 +840,15 @@ static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
         {
         case 0xfb:  /* __fastfail */
         {
-            CONTEXT ctx;
-            save_context( &ctx, sigcontext );
             rec.ExceptionCode = STATUS_STACK_BUFFER_OVERRUN;
-            rec.ExceptionAddress = (void *)ctx.Pc;
-            rec.ExceptionFlags = EH_NONCONTINUABLE;
+            rec.ExceptionFlags = EXCEPTION_NONCONTINUABLE;
             rec.NumberParameters = 1;
             rec.ExceptionInformation[0] = ctx.R0;
             NtRaiseException( &rec, &ctx, FALSE );
             return;
         }
         case 0xfe:  /* breakpoint */
-            PC_sig(context) += 2;  /* skip the instruction */
+            ctx.Pc += 2;  /* skip the breakpoint instruction */
             rec.ExceptionCode = EXCEPTION_BREAKPOINT;
             rec.NumberParameters = 1;
             break;
@@ -1364,9 +861,7 @@ static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
         rec.NumberParameters = 2;
         rec.ExceptionInformation[0] = (get_error_code(context) & 0x800) != 0;
         rec.ExceptionInformation[1] = (ULONG_PTR)siginfo->si_addr;
-        rec.ExceptionCode = virtual_handle_fault( siginfo->si_addr, rec.ExceptionInformation[0],
-                                                  (void *)SP_sig(context) );
-        if (!rec.ExceptionCode) return;
+        if (!virtual_handle_fault( &rec, (void *)SP_sig(context) )) return;
         break;
     case TRAP_ARM_ALIGNFLT:  /* Alignment check exception */
         rec.ExceptionCode = EXCEPTION_DATATYPE_MISALIGNMENT;
@@ -1383,7 +878,7 @@ static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
         break;
     }
     if (handle_syscall_fault( context, &rec )) return;
-    setup_exception( context, &rec );
+    setup_raise_exception( context, &rec, &ctx );
 }
 
 
@@ -1395,6 +890,11 @@ static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 static void trap_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
     EXCEPTION_RECORD rec = { 0 };
+    ucontext_t *context = sigcontext;
+    CONTEXT ctx;
+
+    rec.ExceptionAddress = (void *)PC_sig(context);
+    save_context( &ctx, sigcontext );
 
     switch (siginfo->si_code)
     {
@@ -1403,11 +903,12 @@ static void trap_handler( int signal, siginfo_t *siginfo, void *sigcontext )
         break;
     case TRAP_BRKPT:
     default:
+        ctx.Pc += 2;  /* skip the breakpoint instruction */
         rec.ExceptionCode = EXCEPTION_BREAKPOINT;
         rec.NumberParameters = 1;
         break;
     }
-    setup_exception( sigcontext, &rec );
+    setup_raise_exception( sigcontext, &rec, &ctx );
 }
 
 
@@ -1491,7 +992,7 @@ static void int_handler( int signal, siginfo_t *siginfo, void *sigcontext )
  */
 static void abrt_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    EXCEPTION_RECORD rec = { EXCEPTION_WINE_ASSERTION, EH_NONCONTINUABLE };
+    EXCEPTION_RECORD rec = { EXCEPTION_WINE_ASSERTION, EXCEPTION_NONCONTINUABLE };
 
     setup_exception( sigcontext, &rec );
 }
@@ -1520,7 +1021,7 @@ static void usr1_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 
     if (is_inside_syscall( sigcontext ))
     {
-        context.ContextFlags = CONTEXT_FULL;
+        context.ContextFlags = CONTEXT_FULL | CONTEXT_EXCEPTION_REQUEST;
         NtGetContextThread( GetCurrentThread(), &context );
         wait_suspend( &context );
         NtSetContextThread( GetCurrentThread(), &context );
@@ -1528,6 +1029,7 @@ static void usr1_handler( int signal, siginfo_t *siginfo, void *sigcontext )
     else
     {
         save_context( &context, sigcontext );
+        context.ContextFlags |= CONTEXT_EXCEPTION_REPORTING;
         wait_suspend( &context );
         restore_context( &context, sigcontext );
     }
@@ -1634,7 +1136,11 @@ void call_init_thunk( LPTHREAD_START_ROUTINE entry, void *arg, BOOL suspend, TEB
     if (context.Pc & 1) context.Cpsr |= 0x20; /* thumb mode */
     if ((ctx = get_cpu_area( IMAGE_FILE_MACHINE_ARMNT ))) *ctx = context;
 
-    if (suspend) wait_suspend( &context );
+    if (suspend)
+    {
+        context.ContextFlags |= CONTEXT_EXCEPTION_REPORTING | CONTEXT_EXCEPTION_ACTIVE;
+        wait_suspend( &context );
+    }
 
     ctx = (CONTEXT *)((ULONG_PTR)context.Sp & ~15) - 1;
     *ctx = context;
@@ -1681,20 +1187,17 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    "ldr r1, [r2, #0x1d8]\n\t"       /* arm_thread_data()->syscall_frame */
                    "add r0, r1, #0x10\n\t"
                    "stm r0, {r4-r12,lr}\n\t"
-                   "add r0, sp, #0x10\n\t"
-                   "str r0, [r1, #0x38]\n\t"
+                   "str sp, [r1, #0x38]\n\t"
                    "str r3, [r1, #0x3c]\n\t"
                    "mrs r0, CPSR\n\t"
                    "bfi r0, lr, #5, #1\n\t"         /* set thumb bit */
                    "str r0, [r1, #0x40]\n\t"
                    "mov r0, #0\n\t"
                    "str r0, [r1, #0x44]\n\t"        /* frame->restore_flags */
-#ifndef __SOFTFP__
                    "vmrs r0, fpscr\n\t"
                    "str r0, [r1, #0x48]\n\t"
                    "add r0, r1, #0x60\n\t"
                    "vstm r0, {d0-d15}\n\t"
-#endif
                    "mov r6, sp\n\t"
                    "mov r8, r1\n\t"
                    /* switch to kernel stack */
@@ -1735,7 +1238,6 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    "blx ip\n"
                    __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") ":\n\t"
                    "ldr ip, [r8, #0x44]\n\t"    /* frame->restore_flags */
-#ifndef __SOFTFP__
                    "tst ip, #4\n\t"                 /* CONTEXT_FLOATING_POINT */
                    "beq 3f\n\t"
                    "ldr r4, [r8, #0x48]\n\t"
@@ -1743,7 +1245,6 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    "add r4, r8, #0x60\n\t"
                    "vldm r4, {d0-d15}\n"
                    "3:\n\t"
-#endif
                    "tst ip, #2\n\t"                 /* CONTEXT_INTEGER */
                    "it ne\n\t"
                    "ldmne r8, {r0-r3}\n\t"
@@ -1755,10 +1256,9 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
 
                    "5:\tmovw r0, #0x000d\n\t" /* STATUS_INVALID_PARAMETER */
                    "movt r0, #0xc000\n\t"
-                   "add sp, sp, #0x10\n\t"
-                   "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") "\n\t"
-                   ".globl " __ASM_NAME("__wine_syscall_dispatcher_return") "\n"
-                   __ASM_NAME("__wine_syscall_dispatcher_return") ":\n\t"
+                   "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") )
+
+__ASM_GLOBAL_FUNC( __wine_syscall_dispatcher_return,
                    "mov r8, r0\n\t"
                    "mov r0, r1\n\t"
                    "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") )
@@ -1780,12 +1280,10 @@ __ASM_GLOBAL_FUNC( __wine_unix_call_dispatcher,
                    "str r4, [r1, #0x40]\n\t"
                    "mov r4, #0\n\t"
                    "str r4, [r1, #0x44]\n\t"        /* frame->restore_flags */
-#ifndef __SOFTFP__
                    "vmrs r4, fpscr\n\t"
                    "str r4, [r1, #0x48]\n\t"
                    "add r4, r1, #0x60\n\t"
                    "vstm r4, {d0-d15}\n\t"
-#endif
                    "ldr ip, [r0, r2, lsl #2]\n\t"
                    "mov r8, r1\n\t"
                    /* switch to kernel stack */
@@ -1811,40 +1309,4 @@ __ASM_GLOBAL_FUNC( __wine_unix_call_dispatcher,
                    "ldm r8, {r4-r12,pc}\n\t"
                    "1:\tb " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") )
 
-
-/***********************************************************************
- *           __wine_setjmpex
- */
-__ASM_GLOBAL_FUNC( __wine_setjmpex,
-                   __ASM_EHABI(".cantunwind\n\t")
-                   "stm r0, {r1,r4-r11}\n"         /* jmp_buf->Frame,R4..R11 */
-                   "str sp, [r0, #0x24]\n\t"       /* jmp_buf->Sp */
-                   "str lr, [r0, #0x28]\n\t"       /* jmp_buf->Pc */
-#ifndef __SOFTFP__
-                   "vmrs r2, fpscr\n\t"
-                   "str r2, [r0, #0x2c]\n\t"       /* jmp_buf->Fpscr */
-                   "add r0, r0, #0x30\n\t"
-                   "vstm r0, {d8-d15}\n\t"         /* jmp_buf->D[0..7] */
-#endif
-                   "mov r0, #0\n\t"
-                   "bx lr" )
-
-
-/***********************************************************************
- *           __wine_longjmp
- */
-__ASM_GLOBAL_FUNC( __wine_longjmp,
-                   __ASM_EHABI(".cantunwind\n\t")
-                   "ldm r0, {r3-r11}\n\t"          /* jmp_buf->Frame,R4..R11 */
-                   "ldr sp, [r0, #0x24]\n\t"       /* jmp_buf->Sp */
-                   "ldr r2, [r0, #0x28]\n\t"       /* jmp_buf->Pc */
-#ifndef __SOFTFP__
-                   "ldr r3, [r0, #0x2c]\n\t"       /* jmp_buf->Fpscr */
-                   "vmsr fpscr, r3\n\t"
-                   "add r0, r0, #0x30\n\t"
-                   "vldm r0, {d8-d15}\n\t"         /* jmp_buf->D[0..7] */
-#endif
-                   "mov r0, r1\n\t"                /* retval */
-                   "bx r2" )
-
 #endif  /* __arm__ */
