@@ -523,14 +523,14 @@ NTSTATUS tape_DeviceIoControl( HANDLE device, HANDLE event, PIO_APC_ROUTINE apc,
 {
     DWORD sz = 0;
     NTSTATUS status = STATUS_INVALID_PARAMETER;
+    unsigned int options;
     int fd, needs_close;
 
     TRACE( "%p %s %p %d %p %d %p\n", device, io2str(code),
            in_buffer, in_size, out_buffer, out_size, io );
 
-    io->Information = 0;
-
-    if ((status = server_get_unix_fd( device, 0, &fd, &needs_close, NULL, NULL ))) goto error;
+    if ((status = server_get_unix_fd( device, 0, &fd, &needs_close, NULL, &options )))
+        return status;
 
     switch (code)
     {
@@ -580,9 +580,7 @@ NTSTATUS tape_DeviceIoControl( HANDLE device, HANDLE event, PIO_APC_ROUTINE apc,
 
     if (needs_close) close( fd );
 
-error:
-    io->Status = status;
-    io->Information = sz;
-    if (event) NtSetEvent( event, NULL );
+    if (!NT_ERROR(status))
+        file_complete_async( device, options, event, apc, apc_user, io, status, sz );
     return status;
 }
