@@ -31,19 +31,11 @@
 #include "winnls.h"
 #include "sqlext.h"
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 #include "odbcinst.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(odbc);
 
-/* Registry key names */
-static const WCHAR drivers_key[] = {'S','o','f','t','w','a','r','e','\\','O','D','B','C','\\','O','D','B','C','I','N','S','T','.','I','N','I','\\','O','D','B','C',' ','D','r','i','v','e','r','s',0};
-static const WCHAR odbcW[] = {'S','o','f','t','w','a','r','e','\\','O','D','B','C',0};
-static const WCHAR odbcini[] = {'S','o','f','t','w','a','r','e','\\','O','D','B','C','\\','O','D','B','C','I','N','S','T','.','I','N','I','\\',0};
-static const WCHAR odbcdrivers[] = {'O','D','B','C',' ','D','r','i','v','e','r','s',0};
-static const WCHAR odbctranslators[] = {'O','D','B','C',' ','T','r','a','n','s','l','a','t','o','r','s',0};
-
 /* This config mode is known to be process-wide.
  * MSDN documentation suggests that the value is hidden somewhere in the registry but I haven't found it yet.
  * Although both the registry and the ODBC.ini files appear to be maintained together they are not maintained automatically through the registry's IniFileMapping.
@@ -57,16 +49,6 @@ static UWORD config_mode = ODBC_BOTH_DSN;
 static int num_errors;
 static int error_code[8];
 static const WCHAR *error_msg[8];
-static const WCHAR odbc_error_general_err[] = {'G','e','n','e','r','a','l',' ','e','r','r','o','r',0};
-static const WCHAR odbc_error_invalid_buff_len[] = {'I','n','v','a','l','i','d',' ','b','u','f','f','e','r',' ','l','e','n','g','t','h',0};
-static const WCHAR odbc_error_component_not_found[] = {'C','o','m','p','o','n','e','n','t',' ','n','o','t',' ','f','o','u','n','d',0};
-static const WCHAR odbc_error_out_of_mem[] = {'O','u','t',' ','o','f',' ','m','e','m','o','r','y',0};
-static const WCHAR odbc_error_invalid_param_sequence[] = {'I','n','v','a','l','i','d',' ','p','a','r','a','m','e','t','e','r',' ','s','e','q','u','e','n','c','e',0};
-static const WCHAR odbc_error_invalid_param_string[] = {'I','n','v','a','l','i','d',' ','p','a','r','a','m','e','t','e','r',' ','s','t','r','i','n','g',0};
-static const WCHAR odbc_error_invalid_dsn[] = {'I','n','v','a','l','i','d',' ','D','S','N',0};
-static const WCHAR odbc_error_load_lib_failed[] = {'L','o','a','d',' ','L','i','b','r','a','r','y',' ','F','a','i','l','e','d',0};
-static const WCHAR odbc_error_request_failed[] = {'R','e','q','u','e','s','t',' ','F','a','i','l','e','d',0};
-static const WCHAR odbc_error_invalid_keyword[] = {'I','n','v','a','l','i','d',' ','k','e','y','w','o','r','d',' ','v','a','l','u','e',0};
 
 static BOOL (WINAPI *pConfigDSN)(HWND hwnd, WORD request, const char *driver, const char *attr);
 static BOOL (WINAPI *pConfigDSNW)(HWND hwnd, WORD request, const WCHAR *driver, const WCHAR *attr);
@@ -88,7 +70,7 @@ static void clear_errors(void)
     num_errors = 0;
 }
 
-static inline WCHAR *heap_strdupAtoW(const char *str)
+static inline WCHAR *strdupAtoW(const char *str)
 {
     LPWSTR ret = NULL;
 
@@ -96,7 +78,7 @@ static inline WCHAR *heap_strdupAtoW(const char *str)
         DWORD len;
 
         len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
-        ret = heap_alloc(len*sizeof(WCHAR));
+        ret = malloc(len * sizeof(WCHAR));
         if(ret)
             MultiByteToWideChar(CP_ACP, 0, str, -1, ret, len);
     }
@@ -125,13 +107,45 @@ static LPWSTR SQLInstall_strdup_multi(LPCSTR str)
         ;
 
     len = MultiByteToWideChar(CP_ACP, 0, str, p - str, NULL, 0 );
-    ret = HeapAlloc(GetProcessHeap(), 0, (len+1)*sizeof(WCHAR));
+    ret = malloc((len + 1) * sizeof(WCHAR));
     MultiByteToWideChar(CP_ACP, 0, str, p - str, ret, len );
     ret[len] = 0;
 
     return ret;
 }
 
+static LPSTR SQLInstall_strdup_multiWtoA(LPCWSTR str)
+{
+    LPCWSTR p;
+    LPSTR ret = NULL;
+    DWORD len;
+
+    if (!str)
+        return ret;
+
+    for (p = str; *p; p += lstrlenW(p) + 1)
+        ;
+
+    len = WideCharToMultiByte(CP_ACP, 0, str,   p - str, NULL, 0, NULL, NULL );
+    ret = malloc((len + 1));
+    WideCharToMultiByte(CP_ACP, 0, str, p - str, ret, len, NULL, NULL );
+    ret[len] = 0;
+
+    return ret;
+}
+
+static inline char *strdupWtoA( const WCHAR *str )
+{
+    char *ret = NULL;
+    if (str)
+    {
+        DWORD len = WideCharToMultiByte( CP_ACP, 0, str, -1, NULL, 0, NULL, NULL );
+        if ((ret = malloc( len )))
+            WideCharToMultiByte( CP_ACP, 0, str, -1, ret, len, NULL, NULL );
+    }
+    return ret;
+}
+
 static LPWSTR SQLInstall_strdup(LPCSTR str)
 {
     DWORD len;
@@ -141,7 +155,7 @@ static LPWSTR SQLInstall_strdup(LPCSTR str)
         return ret;
 
     len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0 );
-    ret = HeapAlloc(GetProcessHeap(), 0, len*sizeof(WCHAR));
+    ret = malloc(len * sizeof(WCHAR));
     MultiByteToWideChar(CP_ACP, 0, str, -1, ret, len );
 
     return ret;
@@ -182,7 +196,7 @@ static BOOL SQLInstall_narrow(int mode, LPSTR buffer, LPCWSTR str, WORD str_leng
     {
         if (len > buffer_length)
         {
-            pbuf = HeapAlloc(GetProcessHeap(), 0, len);
+            pbuf = malloc(len);
         }
         else
         {
@@ -212,7 +226,7 @@ static BOOL SQLInstall_narrow(int mode, LPSTR buffer, LPCWSTR str, WORD str_leng
         }
         if (pbuf != buffer)
         {
-            HeapFree(GetProcessHeap(), 0, pbuf);
+            free(pbuf);
         }
     }
     else
@@ -224,39 +238,38 @@ static BOOL SQLInstall_narrow(int mode, LPSTR buffer, LPCWSTR str, WORD str_leng
 
 static HMODULE load_config_driver(const WCHAR *driver)
 {
-    static WCHAR reg_driver[] = {'d','r','i','v','e','r',0};
     long ret;
     HMODULE hmod;
     WCHAR *filename = NULL;
     DWORD size = 0, type;
     HKEY hkey;
 
-    if ((ret = RegOpenKeyW(HKEY_LOCAL_MACHINE, odbcini, &hkey)) == ERROR_SUCCESS)
+    if ((ret = RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\", &hkey)) == ERROR_SUCCESS)
     {
         HKEY hkeydriver;
 
         if ((ret = RegOpenKeyW(hkey, driver, &hkeydriver)) == ERROR_SUCCESS)
         {
-            ret = RegGetValueW(hkeydriver, NULL, reg_driver, RRF_RT_REG_SZ, &type, NULL, &size);
+            ret = RegGetValueW(hkeydriver, NULL, L"Setup", RRF_RT_REG_SZ, &type, NULL, &size);
             if(ret != ERROR_SUCCESS || type != REG_SZ)
             {
                 RegCloseKey(hkeydriver);
                 RegCloseKey(hkey);
-                push_error(ODBC_ERROR_INVALID_DSN, odbc_error_invalid_dsn);
+                push_error(ODBC_ERROR_INVALID_DSN, L"Invalid DSN");
 
                 return NULL;
             }
 
-            filename = HeapAlloc(GetProcessHeap(), 0, size);
+            filename = malloc(size);
             if(!filename)
             {
                 RegCloseKey(hkeydriver);
                 RegCloseKey(hkey);
-                push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+                push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
 
                 return NULL;
             }
-            ret = RegGetValueW(hkeydriver, NULL, reg_driver, RRF_RT_REG_SZ, &type, filename, &size);
+            ret = RegGetValueW(hkeydriver, NULL, L"Setup", RRF_RT_REG_SZ, &type, filename, &size);
 
             RegCloseKey(hkeydriver);
         }
@@ -266,16 +279,16 @@ static HMODULE load_config_driver(const WCHAR *driver)
 
     if(ret != ERROR_SUCCESS)
     {
-        HeapFree(GetProcessHeap(), 0, filename);
-        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, odbc_error_component_not_found);
+        free(filename);
+        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, L"Component not found");
         return NULL;
     }
 
-    hmod = LoadLibraryW(filename);
-    HeapFree(GetProcessHeap(), 0, filename);
+    hmod = LoadLibraryExW(filename, NULL, LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR | LOAD_LIBRARY_SEARCH_DEFAULT_DIRS);
+    free(filename);
 
     if(!hmod)
-        push_error(ODBC_ERROR_LOAD_LIB_FAILED, odbc_error_load_lib_failed);
+        push_error(ODBC_ERROR_LOAD_LIB_FAILED, L"Load Library Failed");
 
     return hmod;
 }
@@ -289,16 +302,16 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
     if(!args)
         return FALSE;
 
-    if((ret = RegOpenKeyW(HKEY_LOCAL_MACHINE, odbcini, &hkey)) == ERROR_SUCCESS)
+    if((ret = RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\", &hkey)) == ERROR_SUCCESS)
     {
         if((ret = RegOpenKeyW(hkey, driver, &hkeydriver)) == ERROR_SUCCESS)
         {
             WCHAR *divider, *value;
 
-            name = heap_alloc( (lstrlenW(args) + 1) * sizeof(WCHAR));
+            name = malloc((wcslen(args) + 1) * sizeof(WCHAR));
             if(!name)
             {
-                push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+                push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
                 goto fail;
             }
             lstrcpyW(name, args);
@@ -306,7 +319,7 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
             divider = wcschr(name,'=');
             if(!divider)
             {
-                push_error(ODBC_ERROR_INVALID_KEYWORD_VALUE, odbc_error_invalid_keyword);
+                push_error(ODBC_ERROR_INVALID_KEYWORD_VALUE, L"Invalid keyword value");
                 goto fail;
             }
 
@@ -317,7 +330,7 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
             if(RegSetValueExW(hkeydriver, name, 0, REG_SZ, (BYTE*)value,
                                (lstrlenW(value)+1) * sizeof(WCHAR)) != ERROR_SUCCESS)
                 ERR("Failed to write registry installed key\n");
-            heap_free(name);
+            free(name);
 
             RegCloseKey(hkeydriver);
         }
@@ -326,14 +339,14 @@ static BOOL write_config_value(const WCHAR *driver, const WCHAR *args)
     }
 
     if(ret != ERROR_SUCCESS)
-        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, odbc_error_component_not_found);
+        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, L"Component not found");
 
     return ret == ERROR_SUCCESS;
 
 fail:
     RegCloseKey(hkeydriver);
     RegCloseKey(hkey);
-    heap_free(name);
+    free(name);
 
     return FALSE;
 }
@@ -374,7 +387,7 @@ BOOL WINAPI SQLConfigDataSourceW(HWND hwnd, WORD request, LPCWSTR driver, LPCWST
     WORD mapped_request;
 
     TRACE("%p, %d, %s, %s\n", hwnd, request, debugstr_w(driver), debugstr_w(attributes));
-    if (TRACE_ON(odbc))
+    if (TRACE_ON(odbc) && attributes)
     {
         const WCHAR *p;
         for (p = attributes; *p; p += lstrlenW(p) + 1)
@@ -397,12 +410,26 @@ BOOL WINAPI SQLConfigDataSourceW(HWND hwnd, WORD request, LPCWSTR driver, LPCWST
     if(pConfigDSNW)
         ret = pConfigDSNW(hwnd, mapped_request, driver, attributes);
     else
-        ERR("Failed to find ConfigDSNW\n");
+    {
+        pConfigDSN = (void*)GetProcAddress(mod, "ConfigDSN");
+        if (pConfigDSN)
+        {
+            LPSTR attr = SQLInstall_strdup_multiWtoA(attributes);
+            char *driverA = strdupWtoA(driver);
+            TRACE("Calling ConfigDSN\n");
+
+            ret = pConfigDSN(hwnd, mapped_request, driverA, attr);
+            free(attr);
+            free(driverA);
+        }
+        else
+            ERR("Failed to find ConfigDSN/W\n");
+    }
 
     config_mode = config_mode_prev;
 
     if (!ret)
-        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+        push_error(ODBC_ERROR_REQUEST_FAILED, L"Request Failed");
 
     FreeLibrary(mod);
 
@@ -419,7 +446,7 @@ BOOL WINAPI SQLConfigDataSource(HWND hwnd, WORD request, LPCSTR driver, LPCSTR a
 
     TRACE("%p, %d, %s, %s\n", hwnd, request, debugstr_a(driver), debugstr_a(attributes));
 
-    if (TRACE_ON(odbc))
+    if (TRACE_ON(odbc) && attributes)
     {
         const char *p;
         for (p = attributes; *p; p += lstrlenA(p) + 1)
@@ -432,17 +459,17 @@ BOOL WINAPI SQLConfigDataSource(HWND hwnd, WORD request, LPCSTR driver, LPCSTR a
     if (!mapped_request)
         return FALSE;
 
-    driverW = heap_strdupAtoW(driver);
+    driverW = strdupAtoW(driver);
     if (!driverW)
     {
-        push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+        push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
         return FALSE;
     }
 
     mod = load_config_driver(driverW);
     if (!mod)
     {
-        heap_free(driverW);
+        free(driverW);
         return FALSE;
     }
 
@@ -465,16 +492,16 @@ BOOL WINAPI SQLConfigDataSource(HWND hwnd, WORD request, LPCSTR driver, LPCSTR a
             attr = SQLInstall_strdup_multi(attributes);
             if(attr)
                 ret = pConfigDSNW(hwnd, mapped_request, driverW, attr);
-            heap_free(attr);
+            free(attr);
         }
     }
 
     config_mode = config_mode_prev;
 
     if (!ret)
-        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+        push_error(ODBC_ERROR_REQUEST_FAILED, L"Request Failed");
 
-    heap_free(driverW);
+    free(driverW);
     FreeLibrary(mod);
 
     return ret;
@@ -505,7 +532,7 @@ BOOL WINAPI SQLConfigDriverW(HWND hwnd, WORD request, LPCWSTR driver,
         funcret = pConfigDriverW(hwnd, request, driver, args, msg, msgmax, msgout);
 
     if(!funcret)
-        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+        push_error(ODBC_ERROR_REQUEST_FAILED, L"Request Failed");
 
     FreeLibrary(hmod);
 
@@ -524,33 +551,33 @@ BOOL WINAPI SQLConfigDriver(HWND hwnd, WORD request, LPCSTR driver,
     TRACE("(%p %d %s %s %p %d %p)\n", hwnd, request, debugstr_a(driver),
           debugstr_a(args), msg, msgmax, msgout);
 
-    driverW = heap_strdupAtoW(driver);
+    driverW = strdupAtoW(driver);
     if(!driverW)
     {
-        push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+        push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
         return FALSE;
     }
     if(request == ODBC_CONFIG_DRIVER)
     {
         BOOL ret = FALSE;
-        WCHAR *argsW = heap_strdupAtoW(args);
+        WCHAR *argsW = strdupAtoW(args);
         if(argsW)
         {
             ret = write_config_value(driverW, argsW);
-            HeapFree(GetProcessHeap(), 0, argsW);
+            free(argsW);
         }
         else
         {
-            push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+            push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
         }
 
-        HeapFree(GetProcessHeap(), 0, driverW);
+        free(driverW);
 
         return ret;
     }
 
     hmod = load_config_driver(driverW);
-    HeapFree(GetProcessHeap(), 0, driverW);
+    free(driverW);
     if(!hmod)
         return FALSE;
 
@@ -559,7 +586,7 @@ BOOL WINAPI SQLConfigDriver(HWND hwnd, WORD request, LPCSTR driver,
         funcret = pConfigDriverA(hwnd, request, driver, args, msg, msgmax, msgout);
 
     if(!funcret)
-        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+        push_error(ODBC_ERROR_REQUEST_FAILED, L"Request Failed");
 
     FreeLibrary(hmod);
 
@@ -626,19 +653,20 @@ BOOL WINAPI SQLGetInstalledDriversW(WCHAR *buf, WORD size, WORD *sizeout)
 
     if (!buf || !size)
     {
-        push_error(ODBC_ERROR_INVALID_BUFF_LEN, odbc_error_invalid_buff_len);
+        push_error(ODBC_ERROR_INVALID_BUFF_LEN, L"Invalid buffer length");
         return FALSE;
     }
 
-    res = RegOpenKeyExW(HKEY_LOCAL_MACHINE, drivers_key, 0, KEY_QUERY_VALUE, &drivers);
+    res = RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\ODBC Drivers", 0,
+                        KEY_QUERY_VALUE, &drivers);
     if (res)
     {
-        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, odbc_error_component_not_found);
+        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, L"Component not found");
         return FALSE;
     }
 
     valuelen = 256;
-    value = heap_alloc(valuelen * sizeof(WCHAR));
+    value = malloc(valuelen * sizeof(WCHAR));
 
     size--;
 
@@ -648,7 +676,7 @@ BOOL WINAPI SQLGetInstalledDriversW(WCHAR *buf, WORD size, WORD *sizeout)
         res = RegEnumValueW(drivers, index, value, &len, NULL, NULL, NULL, NULL);
         while (res == ERROR_MORE_DATA)
         {
-            value = heap_realloc(value, ++len * sizeof(WCHAR));
+            value = realloc(value, ++len * sizeof(WCHAR));
             res = RegEnumValueW(drivers, index, value, &len, NULL, NULL, NULL, NULL);
         }
         if (res == ERROR_SUCCESS)
@@ -660,7 +688,7 @@ BOOL WINAPI SQLGetInstalledDriversW(WCHAR *buf, WORD size, WORD *sizeout)
             break;
         else
         {
-            push_error(ODBC_ERROR_GENERAL_ERR, odbc_error_general_err);
+            push_error(ODBC_ERROR_GENERAL_ERR, L"General error");
             ret = FALSE;
             break;
         }
@@ -669,7 +697,7 @@ BOOL WINAPI SQLGetInstalledDriversW(WCHAR *buf, WORD size, WORD *sizeout)
 
     buf[written++] = 0;
 
-    heap_free(value);
+    free(value);
     RegCloseKey(drivers);
     if (sizeout)
         *sizeout = written;
@@ -686,21 +714,21 @@ BOOL WINAPI SQLGetInstalledDrivers(char *buf, WORD size, WORD *sizeout)
 
     if (!buf || !size)
     {
-        push_error(ODBC_ERROR_INVALID_BUFF_LEN, odbc_error_invalid_buff_len);
+        push_error(ODBC_ERROR_INVALID_BUFF_LEN, L"Invalid buffer length");
         return FALSE;
     }
 
-    wbuf = heap_alloc(size * sizeof(WCHAR));
+    wbuf = malloc(size * sizeof(WCHAR));
     if (!wbuf)
     {
-        push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+        push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
         return FALSE;
     }
 
     ret = SQLGetInstalledDriversW(wbuf, size, &written);
     if (!ret)
     {
-        heap_free(wbuf);
+        free(wbuf);
         return FALSE;
     }
 
@@ -708,16 +736,16 @@ BOOL WINAPI SQLGetInstalledDrivers(char *buf, WORD size, WORD *sizeout)
         *sizeout = WideCharToMultiByte(CP_ACP, 0, wbuf, written, NULL, 0, NULL, NULL);
     WideCharToMultiByte(CP_ACP, 0, wbuf, written, buf, size, NULL, NULL);
 
-    heap_free(wbuf);
+    free(wbuf);
     return TRUE;
 }
 
-static HKEY get_privateprofile_sectionkey(LPCWSTR section, LPCWSTR filename)
+static HKEY get_privateprofile_sectionkey(HKEY root, const WCHAR *section, const WCHAR *filename)
 {
     HKEY hkey, hkeyfilename, hkeysection;
     LONG ret;
 
-    if (RegOpenKeyW(HKEY_CURRENT_USER, odbcW, &hkey))
+    if (RegOpenKeyW(root, L"Software\\ODBC", &hkey))
         return NULL;
 
     ret = RegOpenKeyW(hkey, filename, &hkeyfilename);
@@ -752,7 +780,17 @@ int WINAPI SQLGetPrivateProfileStringW(LPCWSTR section, LPCWSTR entry,
     if (!defvalue || !buff)
         return 0;
 
-    sectionkey = get_privateprofile_sectionkey(section, filename);
+    /* odbcinit.ini is only for drivers, so default to local Machine */
+    if (!wcsicmp(filename, L"ODBCINST.INI") || config_mode == ODBC_SYSTEM_DSN)
+        sectionkey = get_privateprofile_sectionkey(HKEY_LOCAL_MACHINE, section, filename);
+    else if (config_mode == ODBC_USER_DSN)
+        sectionkey = get_privateprofile_sectionkey(HKEY_CURRENT_USER, section, filename);
+    else
+    {
+        sectionkey = get_privateprofile_sectionkey(HKEY_CURRENT_USER, section, filename);
+        if (!sectionkey) sectionkey = get_privateprofile_sectionkey(HKEY_LOCAL_MACHINE, section, filename);
+    }
+
     if (sectionkey)
     {
         DWORD type, size;
@@ -825,13 +863,25 @@ int WINAPI SQLGetPrivateProfileString(LPCSTR section, LPCSTR entry,
     if (!section || !defvalue || !buff)
         return 0;
 
-    sectionW = heap_strdupAtoW(section);
-    filenameW = heap_strdupAtoW(filename);
+    if (!(sectionW = strdupAtoW(section))) return 0;
+    if (!(filenameW = strdupAtoW(filename)))
+    {
+        free(sectionW);
+        return 0;
+    }
 
-    sectionkey = get_privateprofile_sectionkey(sectionW, filenameW);
+    if (config_mode == ODBC_USER_DSN)
+        sectionkey = get_privateprofile_sectionkey(HKEY_CURRENT_USER, sectionW, filenameW);
+    else if (config_mode == ODBC_SYSTEM_DSN)
+        sectionkey = get_privateprofile_sectionkey(HKEY_LOCAL_MACHINE, sectionW, filenameW);
+    else
+    {
+        sectionkey = get_privateprofile_sectionkey(HKEY_CURRENT_USER, sectionW, filenameW);
+        if (!sectionkey) sectionkey = get_privateprofile_sectionkey(HKEY_LOCAL_MACHINE, sectionW, filenameW);
+    }
 
-    heap_free(sectionW);
-    heap_free(filenameW);
+    free(sectionW);
+    free(filenameW);
 
     if (sectionkey)
     {
@@ -933,7 +983,7 @@ BOOL WINAPI SQLInstallDriver(LPCSTR lpszInfFile, LPCSTR lpszDriver,
 
     if (lpszInfFile)
         return FALSE;
-   
+
     return SQLInstallDriverEx(lpszDriver, NULL, lpszPath, cbPathMax,
                               pcbPathOut, ODBC_INSTALL_COMPLETE, &usage);
 }
@@ -941,18 +991,13 @@ BOOL WINAPI SQLInstallDriver(LPCSTR lpszInfFile, LPCSTR lpszDriver,
 static void write_registry_values(const WCHAR *regkey, const WCHAR *driver, const  WCHAR *path_in, WCHAR *path,
                                   DWORD *usage_count)
 {
-    static const WCHAR installed[] = {'I','n','s','t','a','l','l','e','d',0};
-    static const WCHAR slash[] = {'\\', 0};
-    static const WCHAR driverW[] = {'D','r','i','v','e','r',0};
-    static const WCHAR setupW[] = {'S','e','t','u','p',0};
-    static const WCHAR translator[] = {'T','r','a','n','s','l','a','t','o','r',0};
     HKEY hkey, hkeydriver;
 
-    if (RegCreateKeyW(HKEY_LOCAL_MACHINE, odbcini, &hkey) == ERROR_SUCCESS)
+    if (RegCreateKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\", &hkey) == ERROR_SUCCESS)
     {
         if (RegCreateKeyW(hkey, regkey, &hkeydriver) == ERROR_SUCCESS)
         {
-            if(RegSetValueExW(hkeydriver, driver, 0, REG_SZ, (BYTE*)installed, sizeof(installed)) != ERROR_SUCCESS)
+            if(RegSetValueExW(hkeydriver, driver, 0, REG_SZ, (BYTE*)L"Installed", sizeof(L"Installed")) != ERROR_SUCCESS)
                 ERR("Failed to write registry installed key\n");
 
             RegCloseKey(hkeydriver);
@@ -995,32 +1040,48 @@ static void write_registry_values(const WCHAR *regkey, const WCHAR *driver, cons
                     TRACE("Writing pair %s,%s\n", debugstr_w(entry), debugstr_w(divider));
 
                     /* Driver, Setup, Translator entries use the system path unless a path is specified. */
-                    if(lstrcmpiW(driverW, entry) == 0 || lstrcmpiW(setupW, entry) == 0 ||
-                       lstrcmpiW(translator, entry) == 0)
+                    if(lstrcmpiW(L"Driver", entry) == 0 || lstrcmpiW(L"Setup", entry) == 0 ||
+                       lstrcmpiW(L"Translator", entry) == 0)
                     {
-                        len = lstrlenW(path) + lstrlenW(slash) + lstrlenW(divider) + 1;
-                        value = heap_alloc(len * sizeof(WCHAR));
-                        if(!value)
+                        if(GetFileAttributesW(divider) == INVALID_FILE_ATTRIBUTES)
                         {
-                            ERR("Out of memory\n");
-                            return;
-                        }
+                            int pathlen = lstrlenW(path);
+                            len = pathlen + 1 + lstrlenW(divider) + 1;
+                            value = malloc(len * sizeof(WCHAR));
+                            if(!value)
+                            {
+                                RegCloseKey(hkeydriver);
+                                ERR("Out of memory\n");
+                                return;
+                            }
 
-                        lstrcpyW(value, path);
-                        lstrcatW(value, slash);
+                            lstrcpyW(value, path);
+                            if (pathlen && path[pathlen - 1] != '\\')
+                                lstrcatW(value, L"\\");
+                        }
+                        else
+                        {
+                            value = calloc(1, (lstrlenW(divider)+1) * sizeof(WCHAR));
+                            if(!value)
+                            {
+                                RegCloseKey(hkeydriver);
+                                ERR("Out of memory\n");
+                                return;
+                            }
+                        }
                         lstrcatW(value, divider);
                     }
                     else
                     {
                         len = lstrlenW(divider) + 1;
-                        value = heap_alloc(len * sizeof(WCHAR));
+                        value = malloc(len * sizeof(WCHAR));
                         lstrcpyW(value, divider);
                     }
 
                     if (RegSetValueExW(hkeydriver, entry, 0, REG_SZ, (BYTE*)value,
                                     (lstrlenW(value)+1)*sizeof(WCHAR)) != ERROR_SUCCESS)
                         ERR("Failed to write registry data %s %s\n", debugstr_w(entry), debugstr_w(value));
-                    heap_free(value);
+                    free(value);
                 }
                 else
                 {
@@ -1056,7 +1117,7 @@ BOOL WINAPI SQLInstallDriverExW(LPCWSTR lpszDriver, LPCWSTR lpszPathIn,
           debugstr_w(lpszPathIn), lpszPathOut, cbPathOutMax, pcbPathOut,
           fRequest, lpdwUsageCount);
 
-    write_registry_values(odbcdrivers, lpszDriver, lpszPathIn, path, lpdwUsageCount);
+    write_registry_values(L"ODBC Drivers", lpszDriver, lpszPathIn, path, lpdwUsageCount);
 
     len = lstrlenW(path);
 
@@ -1110,8 +1171,8 @@ BOOL WINAPI SQLInstallDriverEx(LPCSTR lpszDriver, LPCSTR lpszPathIn,
     }
 
 out:
-    HeapFree(GetProcessHeap(), 0, driver);
-    HeapFree(GetProcessHeap(), 0, pathin);
+    free(driver);
+    free(pathin);
     return ret;
 }
 
@@ -1266,7 +1327,7 @@ SQLRETURN WINAPI SQLInstallerError(WORD iError, DWORD *pfErrorCode,
     wbuf = 0;
     if (lpszErrorMsg && cbErrorMsgMax)
     {
-        wbuf = HeapAlloc(GetProcessHeap(), 0, cbErrorMsgMax*sizeof(WCHAR));
+        wbuf = malloc(cbErrorMsgMax * sizeof(WCHAR));
         if (!wbuf)
             return SQL_ERROR;
     }
@@ -1275,7 +1336,7 @@ SQLRETURN WINAPI SQLInstallerError(WORD iError, DWORD *pfErrorCode,
     {
         WORD cbBuf = 0;
         SQLInstall_narrow(1, lpszErrorMsg, wbuf, cbwbuf+1, cbErrorMsgMax, &cbBuf);
-        HeapFree(GetProcessHeap(), 0, wbuf);
+        free(wbuf);
         if (pcbErrorMsg)
             *pcbErrorMsg = cbBuf-1;
     }
@@ -1294,7 +1355,7 @@ BOOL WINAPI SQLInstallTranslatorExW(LPCWSTR lpszTranslator, LPCWSTR lpszPathIn,
           debugstr_w(lpszPathIn), lpszPathOut, cbPathOutMax, pcbPathOut,
           fRequest, lpdwUsageCount);
 
-    write_registry_values(odbctranslators, lpszTranslator, lpszPathIn, path, lpdwUsageCount);
+    write_registry_values(L"ODBC Translators", lpszTranslator, lpszPathIn, path, lpdwUsageCount);
 
     len = lstrlenW(path);
 
@@ -1352,8 +1413,8 @@ BOOL WINAPI SQLInstallTranslatorEx(LPCSTR lpszTranslator, LPCSTR lpszPathIn,
     }
 
 out:
-    HeapFree(GetProcessHeap(), 0, translator);
-    HeapFree(GetProcessHeap(), 0, pathin);
+    free(translator);
+    free(pathin);
     return ret;
 }
 
@@ -1449,7 +1510,7 @@ BOOL WINAPI SQLRemoveDriverW(LPCWSTR drivername, BOOL remove_dsn, LPDWORD usage_
     clear_errors();
     TRACE("%s %d %p\n", debugstr_w(drivername), remove_dsn, usage_count);
 
-    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, odbcini, &hkey) == ERROR_SUCCESS)
+    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\", &hkey) == ERROR_SUCCESS)
     {
         HKEY hkeydriver;
 
@@ -1479,7 +1540,7 @@ BOOL WINAPI SQLRemoveDriverW(LPCWSTR drivername, BOOL remove_dsn, LPDWORD usage_
             if (RegDeleteKeyW(hkey, drivername) != ERROR_SUCCESS)
                 ERR("Failed to delete registry key: %s\n", debugstr_w(drivername));
 
-            if (RegOpenKeyW(hkey, odbcdrivers, &hkeydriver) == ERROR_SUCCESS)
+            if (RegOpenKeyW(hkey, L"ODBC Drivers", &hkeydriver) == ERROR_SUCCESS)
             {
                 if(RegDeleteValueW(hkeydriver, drivername) != ERROR_SUCCESS)
                     ERR("Failed to delete registry value: %s\n", debugstr_w(drivername));
@@ -1509,7 +1570,7 @@ BOOL WINAPI SQLRemoveDriver(LPCSTR lpszDriver, BOOL fRemoveDSN,
 
     ret =  SQLRemoveDriverW(driver, fRemoveDSN, lpdwUsageCount);
 
-    HeapFree(GetProcessHeap(), 0, driver);
+    free(driver);
     return ret;
 }
 
@@ -1523,19 +1584,48 @@ BOOL WINAPI SQLRemoveDriverManager(LPDWORD pdwUsageCount)
 
 BOOL WINAPI SQLRemoveDSNFromIniW(LPCWSTR lpszDSN)
 {
-    HKEY hkey;
+    HKEY hkey, hkeyroot = HKEY_CURRENT_USER;
 
     TRACE("%s\n", debugstr_w(lpszDSN));
 
+    if (!SQLValidDSNW(lpszDSN))
+    {
+        push_error(ODBC_ERROR_INVALID_DSN, L"Invalid DSN");
+        return FALSE;
+    }
+
     clear_errors();
 
-    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBC.INI\\ODBC Data Sources", &hkey) == ERROR_SUCCESS)
+    if (config_mode == ODBC_SYSTEM_DSN)
+        hkeyroot = HKEY_LOCAL_MACHINE;
+    else if (config_mode == ODBC_BOTH_DSN)
+    {
+        WCHAR *regpath = malloc( (wcslen(L"Software\\ODBC\\ODBC.INI\\") + wcslen(lpszDSN) + 1) * sizeof(WCHAR) );
+        if (!regpath)
+        {
+            push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
+            return FALSE;
+        }
+        wcscpy(regpath, L"Software\\ODBC\\ODBC.INI\\");
+        wcscat(regpath, lpszDSN);
+
+        /* ONLY removes one DSN, USER or SYSTEM */
+        if (RegOpenKeyW(HKEY_CURRENT_USER, regpath, &hkey) == ERROR_SUCCESS)
+            hkeyroot = HKEY_CURRENT_USER;
+        else
+            hkeyroot = HKEY_LOCAL_MACHINE;
+
+        RegCloseKey(hkey);
+        free(regpath);
+    }
+
+    if (RegOpenKeyW(hkeyroot, L"Software\\ODBC\\ODBC.INI\\ODBC Data Sources", &hkey) == ERROR_SUCCESS)
     {
         RegDeleteValueW(hkey, lpszDSN);
         RegCloseKey(hkey);
     }
 
-    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBC.INI", &hkey) == ERROR_SUCCESS)
+    if (RegOpenKeyW(hkeyroot, L"Software\\ODBC\\ODBC.INI", &hkey) == ERROR_SUCCESS)
     {
         RegDeleteTreeW(hkey, lpszDSN);
         RegCloseKey(hkey);
@@ -1557,9 +1647,9 @@ BOOL WINAPI SQLRemoveDSNFromIni(LPCSTR lpszDSN)
     if (dsn)
         ret = SQLRemoveDSNFromIniW(dsn);
     else
-        push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+        push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
 
-    heap_free(dsn);
+    free(dsn);
 
     return ret;
 }
@@ -1573,7 +1663,7 @@ BOOL WINAPI SQLRemoveTranslatorW(const WCHAR *translator, DWORD *usage_count)
     clear_errors();
     TRACE("%s %p\n", debugstr_w(translator), usage_count);
 
-    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, odbcini, &hkey) == ERROR_SUCCESS)
+    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\", &hkey) == ERROR_SUCCESS)
     {
         HKEY hkeydriver;
 
@@ -1602,16 +1692,16 @@ BOOL WINAPI SQLRemoveTranslatorW(const WCHAR *translator, DWORD *usage_count)
         {
             if(RegDeleteKeyW(hkey, translator) != ERROR_SUCCESS)
             {
-                push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, odbc_error_component_not_found);
+                push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, L"Component not found");
                 WARN("Failed to delete registry key: %s\n", debugstr_w(translator));
                 ret = FALSE;
             }
 
-            if (ret && RegOpenKeyW(hkey, odbctranslators, &hkeydriver) == ERROR_SUCCESS)
+            if (ret && RegOpenKeyW(hkey, L"ODBC Translators", &hkeydriver) == ERROR_SUCCESS)
             {
                 if(RegDeleteValueW(hkeydriver, translator) != ERROR_SUCCESS)
                 {
-                    push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, odbc_error_component_not_found);
+                    push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, L"Component not found");
                     WARN("Failed to delete registry key: %s\n", debugstr_w(translator));
                     ret = FALSE;
                 }
@@ -1640,7 +1730,7 @@ BOOL WINAPI SQLRemoveTranslator(LPCSTR lpszTranslator, LPDWORD lpdwUsageCount)
     translator = SQLInstall_strdup(lpszTranslator);
     ret =  SQLRemoveTranslatorW(translator, lpdwUsageCount);
 
-    HeapFree(GetProcessHeap(), 0, translator);
+    free(translator);
     return ret;
 }
 
@@ -1651,7 +1741,7 @@ BOOL WINAPI SQLSetConfigMode(UWORD wConfigMode)
 
     if (wConfigMode > ODBC_SYSTEM_DSN)
     {
-        push_error(ODBC_ERROR_INVALID_PARAM_SEQUENCE, odbc_error_invalid_param_sequence);
+        push_error(ODBC_ERROR_INVALID_PARAM_SEQUENCE, L"Invalid parameter sequence");
         return FALSE;
     }
     else
@@ -1663,11 +1753,10 @@ BOOL WINAPI SQLSetConfigMode(UWORD wConfigMode)
 
 BOOL WINAPI SQLValidDSNW(LPCWSTR lpszDSN)
 {
-    static const WCHAR invalid[] = {'[',']','{','}','(',')',',',';','?','*','=','!','@','\\',0};
     clear_errors();
     TRACE("%s\n", debugstr_w(lpszDSN));
 
-    if(lstrlenW(lpszDSN) > SQL_MAX_DSN_LENGTH || wcspbrk(lpszDSN, invalid) != NULL)
+    if (!lpszDSN || !*lpszDSN || lstrlenW(lpszDSN) > SQL_MAX_DSN_LENGTH || wcspbrk(lpszDSN, L"[]{}(),;?*=!@\\"))
     {
         return FALSE;
     }
@@ -1681,7 +1770,7 @@ BOOL WINAPI SQLValidDSN(LPCSTR lpszDSN)
     clear_errors();
     TRACE("%s\n", debugstr_a(lpszDSN));
 
-    if(strlen(lpszDSN) > SQL_MAX_DSN_LENGTH || strpbrk(lpszDSN, invalid) != NULL)
+    if (!lpszDSN || !*lpszDSN || strlen(lpszDSN) > SQL_MAX_DSN_LENGTH || strpbrk(lpszDSN, invalid))
     {
         return FALSE;
     }
@@ -1692,7 +1781,7 @@ BOOL WINAPI SQLValidDSN(LPCSTR lpszDSN)
 BOOL WINAPI SQLWriteDSNToIniW(LPCWSTR lpszDSN, LPCWSTR lpszDriver)
 {
     DWORD ret;
-    HKEY hkey, hkeydriver;
+    HKEY hkey, hkeydriver, hkeyroot = HKEY_CURRENT_USER;
     WCHAR filename[MAX_PATH];
 
     TRACE("%s %s\n", debugstr_w(lpszDSN), debugstr_w(lpszDriver));
@@ -1701,13 +1790,13 @@ BOOL WINAPI SQLWriteDSNToIniW(LPCWSTR lpszDSN, LPCWSTR lpszDriver)
 
     if (!SQLValidDSNW(lpszDSN))
     {
-        push_error(ODBC_ERROR_INVALID_DSN, odbc_error_invalid_dsn);
+        push_error(ODBC_ERROR_INVALID_DSN, L"Invalid DSN");
         return FALSE;
     }
 
     /* It doesn't matter if we cannot find the driver, windows just writes a blank value. */
     filename[0] = 0;
-    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, odbcini, &hkey) == ERROR_SUCCESS)
+    if (RegOpenKeyW(HKEY_LOCAL_MACHINE, L"Software\\ODBC\\ODBCINST.INI\\", &hkey) == ERROR_SUCCESS)
     {
         HKEY hkeydriver;
 
@@ -1720,7 +1809,30 @@ BOOL WINAPI SQLWriteDSNToIniW(LPCWSTR lpszDSN, LPCWSTR lpszDriver)
         RegCloseKey(hkey);
     }
 
-    if ((ret = RegCreateKeyW(HKEY_LOCAL_MACHINE, L"SOFTWARE\\ODBC\\ODBC.INI", &hkey)) == ERROR_SUCCESS)
+    if (config_mode == ODBC_SYSTEM_DSN)
+        hkeyroot = HKEY_LOCAL_MACHINE;
+    else if (config_mode == ODBC_BOTH_DSN)
+    {
+        WCHAR *regpath = malloc( (wcslen(L"Software\\ODBC\\ODBC.INI\\") + wcslen(lpszDSN) + 1) * sizeof(WCHAR) );
+        if (!regpath)
+        {
+            push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
+            return FALSE;
+        }
+        wcscpy(regpath, L"Software\\ODBC\\ODBC.INI\\");
+        wcscat(regpath, lpszDSN);
+
+        /* Check for existing entry */
+        if (RegOpenKeyW(HKEY_LOCAL_MACHINE, regpath, &hkey) == ERROR_SUCCESS)
+            hkeyroot = HKEY_LOCAL_MACHINE;
+        else
+            hkeyroot = HKEY_CURRENT_USER;
+
+        RegCloseKey(hkey);
+        free(regpath);
+    }
+
+    if ((ret = RegCreateKeyW(hkeyroot, L"SOFTWARE\\ODBC\\ODBC.INI", &hkey)) == ERROR_SUCCESS)
     {
         HKEY sources;
 
@@ -1740,7 +1852,7 @@ BOOL WINAPI SQLWriteDSNToIniW(LPCWSTR lpszDSN, LPCWSTR lpszDriver)
     }
 
     if (ret != ERROR_SUCCESS)
-        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+        push_error(ODBC_ERROR_REQUEST_FAILED, L"Request Failed");
 
     return ret == ERROR_SUCCESS;
 }
@@ -1757,10 +1869,10 @@ BOOL WINAPI SQLWriteDSNToIni(LPCSTR lpszDSN, LPCSTR lpszDriver)
     if (dsn && driver)
         ret = SQLWriteDSNToIniW(dsn, driver);
     else
-        push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+        push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
 
-    heap_free(dsn);
-    heap_free(driver);
+    free(dsn);
+    free(driver);
 
     return ret;
 }
@@ -1788,9 +1900,9 @@ BOOL WINAPI SQLWriteFileDSN(LPCSTR lpszFileName, LPCSTR lpszAppName,
 BOOL WINAPI SQLWritePrivateProfileStringW(LPCWSTR lpszSection, LPCWSTR lpszEntry,
                LPCWSTR lpszString, LPCWSTR lpszFilename)
 {
-    static const WCHAR empty[] = {0};
     LONG ret;
     HKEY hkey;
+    WCHAR *regpath;
 
     clear_errors();
     TRACE("%s %s %s %s\n", debugstr_w(lpszSection), debugstr_w(lpszEntry),
@@ -1798,30 +1910,45 @@ BOOL WINAPI SQLWritePrivateProfileStringW(LPCWSTR lpszSection, LPCWSTR lpszEntry
 
     if(!lpszFilename || !*lpszFilename)
     {
-        push_error(ODBC_ERROR_INVALID_STR, odbc_error_invalid_param_string);
+        push_error(ODBC_ERROR_INVALID_STR, L"Invalid parameter string");
         return FALSE;
     }
 
-    if ((ret = RegCreateKeyW(HKEY_CURRENT_USER, odbcW, &hkey)) == ERROR_SUCCESS)
+    regpath = malloc ( (wcslen(L"Software\\ODBC\\") + wcslen(lpszFilename) + wcslen(L"\\")
+                            + wcslen(lpszSection) + 1) * sizeof(WCHAR));
+    if (!regpath)
     {
-         HKEY hkeyfilename;
+        push_error(ODBC_ERROR_OUT_OF_MEM, L"Out of memory");
+        return FALSE;
+    }
+    wcscpy(regpath, L"Software\\ODBC\\");
+    wcscat(regpath, lpszFilename);
+    wcscat(regpath, L"\\");
+    wcscat(regpath, lpszSection);
 
-         if ((ret = RegCreateKeyW(hkey, lpszFilename, &hkeyfilename)) == ERROR_SUCCESS)
-         {
-              HKEY hkey_section;
+    /* odbcinit.ini is only for drivers, so default to local Machine */
+    if (!wcsicmp(lpszFilename, L"ODBCINST.INI") || config_mode == ODBC_SYSTEM_DSN)
+        ret = RegCreateKeyW(HKEY_LOCAL_MACHINE, regpath, &hkey);
+    else if (config_mode == ODBC_USER_DSN)
+        ret = RegCreateKeyW(HKEY_CURRENT_USER, regpath, &hkey);
+    else
+    {
+        /* Check existing keys first */
+        if ((ret = RegOpenKeyW(HKEY_CURRENT_USER, regpath, &hkey)) != ERROR_SUCCESS)
+            ret = RegOpenKeyW(HKEY_LOCAL_MACHINE, regpath, &hkey);
 
-              if ((ret = RegCreateKeyW(hkeyfilename, lpszSection, &hkey_section)) == ERROR_SUCCESS)
-              {
-                  if(lpszString)
-                      ret = RegSetValueExW(hkey_section, lpszEntry, 0, REG_SZ, (BYTE*)lpszString, (lstrlenW(lpszString)+1)*sizeof(WCHAR));
-                  else
-                      ret = RegSetValueExW(hkey_section, lpszEntry, 0, REG_SZ, (BYTE*)empty, sizeof(empty));
-                  RegCloseKey(hkey_section);
-              }
+        if (ret != ERROR_SUCCESS)
+            ret = RegCreateKeyW(HKEY_CURRENT_USER, regpath, &hkey);
+    }
 
-              RegCloseKey(hkeyfilename);
-         }
+    free(regpath);
 
+    if (ret == ERROR_SUCCESS)
+    {
+        if(lpszString)
+            ret = RegSetValueExW(hkey, lpszEntry, 0, REG_SZ, (BYTE*)lpszString, (lstrlenW(lpszString)+1)*sizeof(WCHAR));
+        else
+            ret = RegSetValueExW(hkey, lpszEntry, 0, REG_SZ, (BYTE*)L"", sizeof(L""));
          RegCloseKey(hkey);
     }
 
@@ -1836,17 +1963,17 @@ BOOL WINAPI SQLWritePrivateProfileString(LPCSTR lpszSection, LPCSTR lpszEntry,
     clear_errors();
     TRACE("%s %s %s %s\n", lpszSection, lpszEntry, lpszString, lpszFilename);
 
-    sect = heap_strdupAtoW(lpszSection);
-    entry = heap_strdupAtoW(lpszEntry);
-    string = heap_strdupAtoW(lpszString);
-    file = heap_strdupAtoW(lpszFilename);
+    sect = strdupAtoW(lpszSection);
+    entry = strdupAtoW(lpszEntry);
+    string = strdupAtoW(lpszString);
+    file = strdupAtoW(lpszFilename);
 
     ret = SQLWritePrivateProfileStringW(sect, entry, string, file);
 
-    heap_free(sect);
-    heap_free(entry);
-    heap_free(string);
-    heap_free(file);
+    free(sect);
+    free(entry);
+    free(string);
+    free(file);
 
     return ret;
 }
