@@ -205,40 +205,39 @@ BOOL actctx_get_miscstatus(const CLSID *clsid, DWORD aspect, DWORD *status)
 }
 
 /* wrapper for NtCreateKey that creates the key recursively if necessary */
-static NTSTATUS create_key( HKEY *retkey, ACCESS_MASK access, OBJECT_ATTRIBUTES *attr )
+static NTSTATUS create_key( HKEY *retkey, ACCESS_MASK access, const OBJECT_ATTRIBUTES *attr )
 {
     NTSTATUS status = NtCreateKey( (HANDLE *)retkey, access, attr, 0, NULL, 0, NULL );
 
     if (status == STATUS_OBJECT_NAME_NOT_FOUND)
     {
-        HANDLE subkey, root = attr->RootDirectory;
+        HANDLE subkey;
         WCHAR *buffer = attr->ObjectName->Buffer;
-        DWORD attrs, pos = 0, i = 0, len = attr->ObjectName->Length / sizeof(WCHAR);
+        DWORD pos = 0, i = 0, len = attr->ObjectName->Length / sizeof(WCHAR);
         UNICODE_STRING str;
+        OBJECT_ATTRIBUTES attr2 = *attr;
 
         while (i < len && buffer[i] != '\\') i++;
         if (i == len) return status;
 
-        attrs = attr->Attributes;
-        attr->ObjectName = &str;
+        attr2.ObjectName = &str;
 
         while (i < len)
         {
             str.Buffer = buffer + pos;
             str.Length = (i - pos) * sizeof(WCHAR);
-            status = NtCreateKey( &subkey, access, attr, 0, NULL, 0, NULL );
-            if (attr->RootDirectory != root) NtClose( attr->RootDirectory );
+            status = NtCreateKey( &subkey, access, &attr2, 0, NULL, 0, NULL );
+            if (attr2.RootDirectory != attr->RootDirectory) NtClose( attr2.RootDirectory );
             if (status) return status;
-            attr->RootDirectory = subkey;
+            attr2.RootDirectory = subkey;
             while (i < len && buffer[i] == '\\') i++;
             pos = i;
             while (i < len && buffer[i] != '\\') i++;
         }
         str.Buffer = buffer + pos;
         str.Length = (i - pos) * sizeof(WCHAR);
-        attr->Attributes = attrs;
-        status = NtCreateKey( (PHANDLE)retkey, access, attr, 0, NULL, 0, NULL );
-        if (attr->RootDirectory != root) NtClose( attr->RootDirectory );
+        status = NtCreateKey( (PHANDLE)retkey, access, &attr2, 0, NULL, 0, NULL );
+        if (attr2.RootDirectory != attr->RootDirectory) NtClose( attr2.RootDirectory );
     }
     return status;
 }
