@@ -31,8 +31,6 @@
 #include <string.h>
 
 #define COBJMACROS
-#define NONAMELESSUNION
-
 #include "windef.h"
 #include "winbase.h"
 #include "winerror.h"
@@ -253,9 +251,10 @@ void WINAPI DECLSPEC_HOTPATCH OleUninitialize(void)
 /******************************************************************************
  *		OleInitializeWOW	[OLE32.@]
  */
-HRESULT WINAPI OleInitializeWOW(DWORD x, DWORD y) {
-        FIXME("(0x%08x, 0x%08x),stub!\n",x, y);
-        return 0;
+HRESULT WINAPI OleInitializeWOW(DWORD x, DWORD y)
+{
+    FIXME("%#lx, %#lx stub!\n", x, y);
+    return 0;
 }
 
 /*************************************************************
@@ -664,7 +663,7 @@ HRESULT WINAPI OleRegGetUserType(REFCLSID clsid, DWORD form, LPOLESTR *usertype)
   HRESULT hres;
   LONG    ret;
 
-  TRACE("(%s, %u, %p)\n", debugstr_guid(clsid), form, usertype);
+  TRACE("%s, %lu, %p.\n", debugstr_guid(clsid), form, usertype);
 
   if (!usertype)
     return E_INVALIDARG;
@@ -735,7 +734,7 @@ HRESULT WINAPI DoDragDrop (
   HWND            hwndTrackWindow;
   MSG             msg;
 
-  TRACE("(%p, %p, %08x, %p)\n", pDataObject, pDropSource, dwOKEffect, pdwEffect);
+  TRACE("%p, %p, %#lx, %p.\n", pDataObject, pDropSource, dwOKEffect, pdwEffect);
 
   if (!pDataObject || !pDropSource || !pdwEffect)
       return E_INVALIDARG;
@@ -844,7 +843,7 @@ HRESULT WINAPI OleRegGetMiscStatus(
   LONG    result;
   HRESULT hr;
 
-  TRACE("(%s, %d, %p)\n", debugstr_guid(clsid), dwAspect, pdwStatus);
+  TRACE("%s, %ld, %p.\n", debugstr_guid(clsid), dwAspect, pdwStatus);
 
   if (!pdwStatus) return E_INVALIDARG;
 
@@ -934,7 +933,7 @@ static HRESULT WINAPI EnumOLEVERB_Next(
     EnumOLEVERB *This = impl_from_IEnumOLEVERB(iface);
     HRESULT hr = S_OK;
 
-    TRACE("(%d, %p, %p)\n", celt, rgelt, pceltFetched);
+    TRACE("%p, %lu, %p, %p.\n", iface, celt, rgelt, pceltFetched);
 
     if (pceltFetched)
         *pceltFetched = 0;
@@ -954,14 +953,14 @@ static HRESULT WINAPI EnumOLEVERB_Next(
         }
         else if (res != ERROR_SUCCESS)
         {
-            ERR("RegEnumKeyW failed with error %d\n", res);
+            ERR("RegEnumKeyW failed with error %ld\n", res);
             hr = REGDB_E_READREGDB;
             break;
         }
         res = RegQueryValueW(This->hkeyVerb, wszSubKey, NULL, &cbData);
         if (res != ERROR_SUCCESS)
         {
-            ERR("RegQueryValueW failed with error %d\n", res);
+            ERR("RegQueryValueW failed with error %ld\n", res);
             hr = REGDB_E_READREGDB;
             break;
         }
@@ -974,7 +973,7 @@ static HRESULT WINAPI EnumOLEVERB_Next(
         res = RegQueryValueW(This->hkeyVerb, wszSubKey, pwszOLEVERB, &cbData);
         if (res != ERROR_SUCCESS)
         {
-            ERR("RegQueryValueW failed with error %d\n", res);
+            ERR("RegQueryValueW failed with error %ld\n", res);
             hr = REGDB_E_READREGDB;
             CoTaskMemFree(pwszOLEVERB);
             break;
@@ -1020,7 +1019,7 @@ static HRESULT WINAPI EnumOLEVERB_Skip(
 {
     EnumOLEVERB *This = impl_from_IEnumOLEVERB(iface);
 
-    TRACE("(%d)\n", celt);
+    TRACE("%p, %lu.\n", iface, celt);
 
     This->index += celt;
     return S_OK;
@@ -1108,7 +1107,7 @@ HRESULT WINAPI OleRegEnumVerbs (REFCLSID clsid, LPENUMOLEVERB* ppenum)
         else if (res == REGDB_E_KEYMISSING)
             ERR("no Verbs key for class %s\n", debugstr_guid(clsid));
         else
-            ERR("failed to open Verbs key for CLSID %s with error %d\n",
+            ERR("failed to open Verbs key for CLSID %s with error %ld\n",
                 debugstr_guid(clsid), res);
         return res;
     }
@@ -1117,7 +1116,7 @@ HRESULT WINAPI OleRegEnumVerbs (REFCLSID clsid, LPENUMOLEVERB* ppenum)
                           NULL, NULL, NULL, NULL, NULL, NULL);
     if (res != ERROR_SUCCESS)
     {
-        ERR("failed to get subkey count with error %d\n", GetLastError());
+        ERR("failed to get subkey count with error %ld\n", GetLastError());
         return REGDB_E_READREGDB;
     }
 
@@ -1602,7 +1601,7 @@ static LRESULT CALLBACK OLEMenu_CallWndProc(INT code, WPARAM wParam, LPARAM lPar
   OleMenuHookItem *pHookItem = NULL;
   WORD fuFlags;
 
-  TRACE("%i, %04lx, %08lx\n", code, wParam, lParam );
+  TRACE("%i, %#Ix, %#Ix.\n", code, wParam, lParam );
 
   /* Check if we're being asked to process the message */
   if ( HC_ACTION != code )
@@ -1707,7 +1706,7 @@ static LRESULT CALLBACK OLEMenu_GetMsgProc(INT code, WPARAM wParam, LPARAM lPara
   OleMenuHookItem *pHookItem = NULL;
   WORD wCode;
 
-  TRACE("%i, %04lx, %08lx\n", code, wParam, lParam );
+  TRACE("%i, %#Ix, %#Ix.\n", code, wParam, lParam );
 
   /* Check if we're being asked to process a  messages */
   if ( HC_ACTION != code )
@@ -1943,7 +1942,7 @@ BOOL WINAPI IsAccelerator(HACCEL hAccel, int cAccelEntries, LPMSG lpMsg, WORD* l
     }
 
     TRACE_(accel)("hAccel=%p, cAccelEntries=%d,"
-		"msg->hwnd=%p, msg->message=%04x, wParam=%08lx, lParam=%08lx\n",
+		"msg->hwnd=%p, msg->message=%04x, wParam=%#Ix, lParam=%#Ix\n",
 		hAccel, cAccelEntries,
 		lpMsg->hwnd, lpMsg->message, lpMsg->wParam, lpMsg->lParam);
     for(i = 0; i < cAccelEntries; i++)
@@ -1964,7 +1963,7 @@ BOOL WINAPI IsAccelerator(HACCEL hAccel, int cAccelEntries, LPMSG lpMsg, WORD* l
 	    if(lpAccelTbl[i].fVirt & FVIRTKEY)
 	    {
 		INT mask = 0;
-		TRACE_(accel)("found accel for virt_key %04lx (scan %04x)\n",
+		TRACE_(accel)("found accel for virt_key %Ix (scan %04x)\n",
 				lpMsg->wParam, HIWORD(lpMsg->lParam) & 0xff);
 		if(GetKeyState(VK_SHIFT) & 0x8000) mask |= FSHIFT;
 		if(GetKeyState(VK_CONTROL) & 0x8000) mask |= FCONTROL;
@@ -2009,64 +2008,64 @@ void WINAPI ReleaseStgMedium(
     case TYMED_HGLOBAL:
     {
       if ( (pmedium->pUnkForRelease==0) &&
-	   (pmedium->u.hGlobal!=0) )
-	GlobalFree(pmedium->u.hGlobal);
+	   (pmedium->hGlobal!=0) )
+	GlobalFree(pmedium->hGlobal);
       break;
     }
     case TYMED_FILE:
     {
-      if (pmedium->u.lpszFileName!=0)
+      if (pmedium->lpszFileName!=0)
       {
 	if (pmedium->pUnkForRelease==0)
 	{
-	  DeleteFileW(pmedium->u.lpszFileName);
+	  DeleteFileW(pmedium->lpszFileName);
 	}
 
-	CoTaskMemFree(pmedium->u.lpszFileName);
+	CoTaskMemFree(pmedium->lpszFileName);
       }
       break;
     }
     case TYMED_ISTREAM:
     {
-      if (pmedium->u.pstm!=0)
+      if (pmedium->pstm!=0)
       {
-	IStream_Release(pmedium->u.pstm);
+	IStream_Release(pmedium->pstm);
       }
       break;
     }
     case TYMED_ISTORAGE:
     {
-      if (pmedium->u.pstg!=0)
+      if (pmedium->pstg!=0)
       {
-	IStorage_Release(pmedium->u.pstg);
+	IStorage_Release(pmedium->pstg);
       }
       break;
     }
     case TYMED_GDI:
     {
       if ( (pmedium->pUnkForRelease==0) &&
-	   (pmedium->u.hBitmap!=0) )
-	DeleteObject(pmedium->u.hBitmap);
+	   (pmedium->hBitmap!=0) )
+	DeleteObject(pmedium->hBitmap);
       break;
     }
     case TYMED_MFPICT:
     {
       if ( (pmedium->pUnkForRelease==0) &&
-	   (pmedium->u.hMetaFilePict!=0) )
+	   (pmedium->hMetaFilePict!=0) )
       {
-	LPMETAFILEPICT pMP = GlobalLock(pmedium->u.hMetaFilePict);
+	LPMETAFILEPICT pMP = GlobalLock(pmedium->hMetaFilePict);
 	DeleteMetaFile(pMP->hMF);
-	GlobalUnlock(pmedium->u.hMetaFilePict);
-	GlobalFree(pmedium->u.hMetaFilePict);
+	GlobalUnlock(pmedium->hMetaFilePict);
+	GlobalFree(pmedium->hMetaFilePict);
       }
       break;
     }
     case TYMED_ENHMF:
     {
       if ( (pmedium->pUnkForRelease==0) &&
-	   (pmedium->u.hEnhMetaFile!=0) )
+	   (pmedium->hEnhMetaFile!=0) )
       {
-	DeleteEnhMetaFile(pmedium->u.hEnhMetaFile);
+	DeleteEnhMetaFile(pmedium->hEnhMetaFile);
       }
       break;
     }
@@ -2463,7 +2462,7 @@ HRESULT WINAPI OleCreate(
     IUnknown * pUnk = NULL;
     IOleObject *pOleObject = NULL;
 
-    TRACE("(%s, %s, %d, %p, %p, %p, %p)\n", debugstr_guid(rclsid),
+    TRACE("%s, %s, %ld, %p, %p, %p, %p.\n", debugstr_guid(rclsid),
         debugstr_guid(riid), renderopt, pFormatEtc, pClientSite, pStg, ppvObj);
 
     hres = CoCreateInstance(rclsid, 0, CLSCTX_INPROC_SERVER|CLSCTX_INPROC_HANDLER, riid, (LPVOID*)&pUnk);
@@ -2488,7 +2487,7 @@ HRESULT WINAPI OleCreate(
         {
             TRACE("trying to set stg %p\n", pStg);
             hres = IPersistStorage_InitNew(pPS, pStg);
-            TRACE("-- result 0x%08x\n", hres);
+            TRACE("-- result %#lx\n", hres);
             IPersistStorage_Release(pPS);
         }
     }
@@ -2497,7 +2496,7 @@ HRESULT WINAPI OleCreate(
     {
         TRACE("trying to set clientsite %p\n", pClientSite);
         hres = IOleObject_SetClientSite(pOleObject, pClientSite);
-        TRACE("-- result 0x%08x\n", hres);
+        TRACE("-- result %#lx\n", hres);
     }
 
     if (pOleObject)
@@ -2781,7 +2780,7 @@ static HRESULT get_storage(IDataObject *data, IStorage *stg, UINT *src_cf, BOOL
     /* CF_EMBEDEDOBJECT */
     init_fmtetc(&fmt, embedded_object_clipboard_format, TYMED_ISTORAGE);
     med.tymed = TYMED_ISTORAGE;
-    med.u.pstg = stg;
+    med.pstg = stg;
     med.pUnkForRelease = NULL;
     hr = IDataObject_GetDataHere(data, &fmt, &med);
     if(SUCCEEDED(hr))
@@ -2793,7 +2792,7 @@ static HRESULT get_storage(IDataObject *data, IStorage *stg, UINT *src_cf, BOOL
     /* CF_EMBEDSOURCE */
     init_fmtetc(&fmt, embed_source_clipboard_format, TYMED_ISTORAGE);
     med.tymed = TYMED_ISTORAGE;
-    med.u.pstg = stg;
+    med.pstg = stg;
     med.pUnkForRelease = NULL;
     hr = IDataObject_GetDataHere(data, &fmt, &med);
     if(SUCCEEDED(hr))
@@ -2851,7 +2850,7 @@ HRESULT WINAPI OleCreateFromDataEx(IDataObject *data, REFIID iid, DWORD flags,
     HRESULT hr;
     UINT src_cf;
 
-    FIXME("(%p, %s, %08x, %08x, %d, %p, %p, %p, %p, %p, %p, %p): stub\n",
+    FIXME("%p, %s, %#lx, %#lx, %ld, %p, %p, %p, %p, %p, %p, %p: stub\n",
           data, debugstr_guid(iid), flags, renderopt, num_cache_fmts, adv_flags, cache_fmts,
           sink, conns, client_site, stg, obj);
 
@@ -2884,7 +2883,7 @@ HRESULT WINAPI OleCreateFromData(IDataObject *data, REFIID iid, DWORD renderopt,
 HRESULT WINAPI OleCreateLinkFromData(IDataObject *data, REFIID iid, DWORD renderopt, FORMATETC *fmt,
         IOleClientSite *client_site, IStorage *stg, void **obj)
 {
-    FIXME("%p,%s,%08x,%p,%p,%p,%p: semi-stub\n",
+    FIXME("%p, %s, %#lx, %p, %p, %p, %p: semi-stub\n",
           data, debugstr_guid(iid), renderopt, fmt, client_site, stg, obj);
     return OleCreateFromData(data, iid, renderopt, fmt, client_site, stg, obj);
 }
@@ -2904,8 +2903,7 @@ HRESULT WINAPI OleCreateStaticFromData(IDataObject *data, REFIID iid, DWORD rend
     STGMEDIUM stgmedium;
     LPOLESTR ole_typename;
 
-    TRACE("(%p, %s, 0x%08x, %p, %p, %p, %p)\n",
-          data, debugstr_guid(iid), renderopt, fmt, client_site, stg, obj);
+    TRACE("%p, %s, %#lx, %p, %p, %p, %p.\n", data, debugstr_guid(iid), renderopt, fmt, client_site, stg, obj);
 
     if (!obj || !stg)
         return E_INVALIDARG;
@@ -3007,11 +3005,11 @@ HRESULT WINAPI OleCreateFromFileEx(REFCLSID clsid, const OLECHAR *filename, REFI
     IOleCache *cache = NULL;
     ULONG i;
 
-    TRACE("cls %s, %s, iid %s, flags %d, render opts %d, num fmts %d, adv flags %p, fmts %p\n", debugstr_guid(clsid),
+    TRACE("cls %s, %s, iid %s, flags %ld, render opts %ld, num fmts %ld, adv flags %p, fmts %p\n", debugstr_guid(clsid),
           debugstr_w(filename), debugstr_guid(iid), flags, renderopt, num_fmts, adv_flags, fmts);
     TRACE("sink %p, conns %p, client site %p, storage %p, obj %p\n", sink, conns, client_site, stg, obj);
     for (i = 0; i < num_fmts; i++)
-        TRACE("\t%d: fmt %s adv flags %d\n", i, debugstr_formatetc(fmts + i), adv_flags[i]);
+        TRACE("\t%ld: fmt %s adv flags %ld\n", i, debugstr_formatetc(fmts + i), adv_flags[i]);
 
     hr = CreateFileMoniker( filename, &mon );
     if (FAILED(hr)) return hr;
