@@ -24,8 +24,6 @@
 #include <string.h>
 
 #define COBJMACROS
-#define NONAMELESSUNION
-
 #include "windef.h"
 #include "winbase.h"
 #include "winuser.h"
@@ -767,7 +765,7 @@ static HRESULT WINAPI DataAdviseHolder_SendOnDataChange(IDataAdviseHolder *iface
         {
             STGMEDIUM stg;
             stg.tymed = TYMED_NULL;
-            stg.u.pstg = NULL;
+            stg.pstg = NULL;
             stg.pUnkForRelease = NULL;
 
             if(!(statdata.advf & ADVF_NODATA))
