@@ -90,16 +90,16 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
 
     /* test setting one that I can't set */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_DICTIONARY;
+    spec.propid = PID_DICTIONARY;
     var.vt = VT_I4;
-    U(var).lVal = 1;
+    var.lVal = 1;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == STG_E_INVALIDPARAMETER,
      "Expected STG_E_INVALIDPARAMETER, got 0x%08lx\n", hr);
 
     /* test setting one by name with an invalid propidNameFirst */
     spec.ulKind = PRSPEC_LPWSTR;
-    U(spec).lpwstr = propName;
+    spec.lpwstr = propName;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var,
      PID_DICTIONARY);
     ok(hr == STG_E_INVALIDPARAMETER,
@@ -107,43 +107,43 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
 
     /* test setting behavior (case-sensitive) */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_BEHAVIOR;
-    U(var).lVal = 1;
+    spec.propid = PID_BEHAVIOR;
+    var.lVal = 1;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == STG_E_INVALIDPARAMETER,
      "Expected STG_E_INVALIDPARAMETER, got 0x%08lx\n", hr);
 
     /* set one by value.. */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
-    U(var).lVal = 1;
+    spec.propid = PID_FIRST_USABLE;
+    var.lVal = 1;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
 
     /* set one by name */
     spec.ulKind = PRSPEC_LPWSTR;
-    U(spec).lpwstr = propName;
-    U(var).lVal = 2;
+    spec.lpwstr = propName;
+    var.lVal = 2;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var,
      PID_FIRST_USABLE);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
 
     /* set a string value */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PIDSI_AUTHOR;
+    spec.propid = PIDSI_AUTHOR;
     var.vt = VT_LPSTR;
-    U(var).pszVal = val;
+    var.pszVal = val;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
 
     /* set a clipboard value */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PIDSI_THUMBNAIL;
+    spec.propid = PIDSI_THUMBNAIL;
     var.vt = VT_CF;
     clipdata.cbSize = sizeof clipcontent + sizeof (ULONG);
     clipdata.ulClipFmt = CF_ENHMETAFILE;
     clipdata.pClipData = clipcontent;
-    U(var).pclipdata = &clipdata;
+    var.pclipdata = &clipdata;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
 
@@ -155,41 +155,41 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
     ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got 0x%08lx\n", hr);
     /* read by propid */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I4 && U(var).lVal == 1,
+    ok(var.vt == VT_I4 && var.lVal == 1,
      "Didn't get expected type or value for property (got type %d, value %ld)\n",
-     var.vt, U(var).lVal);
+     var.vt, var.lVal);
     /* read by name */
     spec.ulKind = PRSPEC_LPWSTR;
-    U(spec).lpwstr = propName;
+    spec.lpwstr = propName;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I4 && U(var).lVal == 2,
+    ok(var.vt == VT_I4 && var.lVal == 2,
      "Didn't get expected type or value for property (got type %d, value %ld)\n",
-     var.vt, U(var).lVal);
+     var.vt, var.lVal);
     /* read string value */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PIDSI_AUTHOR;
+    spec.propid = PIDSI_AUTHOR;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_LPSTR && !lstrcmpA(U(var).pszVal, val),
+    ok(var.vt == VT_LPSTR && !lstrcmpA(var.pszVal, val),
      "Didn't get expected type or value for property (got type %d, value %s)\n",
-     var.vt, U(var).pszVal);
+     var.vt, var.pszVal);
     PropVariantClear(&var);
 
     /* read clipboard format */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PIDSI_THUMBNAIL;
+    spec.propid = PIDSI_THUMBNAIL;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
     ok(var.vt == VT_CF, "variant type wrong\n");
-    ok(U(var).pclipdata->ulClipFmt == CF_ENHMETAFILE,
+    ok(var.pclipdata->ulClipFmt == CF_ENHMETAFILE,
         "clipboard type wrong\n");
-    ok(U(var).pclipdata->cbSize == sizeof clipcontent + sizeof (ULONG),
+    ok(var.pclipdata->cbSize == sizeof clipcontent + sizeof (ULONG),
         "clipboard size wrong\n");
-    ok(!memcmp(U(var).pclipdata->pClipData, clipcontent, sizeof clipcontent),
+    ok(!memcmp(var.pclipdata->pClipData, clipcontent, sizeof clipcontent),
         "clipboard contents wrong\n");
     ok(S_OK == PropVariantClear(&var), "failed to clear variant\n");
 
@@ -200,12 +200,12 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
     ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got 0x%08lx\n", hr);
     /* contrary to what the docs say, you can't delete the dictionary */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_DICTIONARY;
+    spec.propid = PID_DICTIONARY;
     hr = IPropertyStorage_DeleteMultiple(propertyStorage, 1, &spec);
     ok(hr == STG_E_INVALIDPARAMETER,
      "Expected STG_E_INVALIDPARAMETER, got 0x%08lx\n", hr);
     /* now delete the first value.. */
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     hr = IPropertyStorage_DeleteMultiple(propertyStorage, 1, &spec);
     ok(hr == S_OK, "DeleteMultiple failed: 0x%08lx\n", hr);
     /* and check that it's no longer readable */
@@ -217,7 +217,7 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
 
     /* check reverting */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     hr = IPropertyStorage_Revert(propertyStorage);
@@ -227,9 +227,9 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
     ok(hr == S_FALSE, "Expected S_FALSE, got 0x%08lx\n", hr);
     /* set an integer value again */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     var.vt = VT_I4;
-    U(var).lVal = 1;
+    var.lVal = 1;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* commit it */
@@ -237,7 +237,7 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
     ok(hr == S_OK, "Commit failed: 0x%08lx\n", hr);
     /* set it to a string value */
     var.vt = VT_LPSTR;
-    U(var).pszVal = val;
+    var.pszVal = val;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* revert it */
@@ -280,19 +280,19 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
 
     /* check properties again */
     spec.ulKind = PRSPEC_LPWSTR;
-    U(spec).lpwstr = propName;
+    spec.lpwstr = propName;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I4 && U(var).lVal == 2,
+    ok(var.vt == VT_I4 && var.lVal == 2,
      "Didn't get expected type or value for property (got type %d, value %ld)\n",
-     var.vt, U(var).lVal);
+     var.vt, var.lVal);
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PIDSI_AUTHOR;
+    spec.propid = PIDSI_AUTHOR;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_LPSTR && !lstrcmpA(U(var).pszVal, val),
+    ok(var.vt == VT_LPSTR && !lstrcmpA(var.pszVal, val),
      "Didn't get expected type or value for property (got type %d, value %s)\n",
-     var.vt, U(var).pszVal);
+     var.vt, var.pszVal);
     PropVariantClear(&var);
 
     IPropertyStorage_Release(propertyStorage);
@@ -330,9 +330,9 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
     }
 
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     var.vt = VT_I4;
-    U(var).lVal = 1;
+    var.lVal = 1;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
 
@@ -370,13 +370,13 @@ static void testPropsHelper(IPropertySetStorage **propSetStorage)
     }
 
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
 
-    ok(var.vt == VT_I4 && U(var).lVal == 1,
+    ok(var.vt == VT_I4 && var.lVal == 1,
      "Didn't get expected type or value for property (got type %d, value %ld)\n",
-     var.vt, U(var).lVal);
+     var.vt, var.lVal);
 
     IPropertyStorage_Release(propertyStorage);
     if(propSetStorage) IPropertySetStorage_Release(*propSetStorage);
@@ -426,41 +426,41 @@ static void testCodepage(void)
 
     PropVariantInit(&var);
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_CODEPAGE;
+    spec.propid = PID_CODEPAGE;
     /* check code page before it's been explicitly set */
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I2 && U(var).iVal == 1200,
+    ok(var.vt == VT_I2 && var.iVal == 1200,
      "Didn't get expected type or value for property\n");
     /* Set the code page to ascii */
     var.vt = VT_I2;
-    U(var).iVal = 1252;
+    var.iVal = 1252;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* check code page */
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I2 && U(var).iVal == 1252,
+    ok(var.vt == VT_I2 && var.iVal == 1252,
      "Didn't get expected type or value for property\n");
     /* Set code page to Unicode */
-    U(var).iVal = 1200;
+    var.iVal = 1200;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* check code page */
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I2 && U(var).iVal == 1200,
+    ok(var.vt == VT_I2 && var.iVal == 1200,
      "Didn't get expected type or value for property\n");
     /* Set a string value */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     var.vt = VT_LPSTR;
-    U(var).pszVal = aval;
+    var.pszVal = aval;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_LPSTR && !strcmp(U(var).pszVal, "hi"),
+    ok(var.vt == VT_LPSTR && !strcmp(var.pszVal, "hi"),
      "Didn't get expected type or value for property\n");
     PropVariantClear(&var);
     /* This seemingly non-sensical test is to show that the string is indeed
@@ -469,20 +469,20 @@ static void testCodepage(void)
      * string would be maintained.  As it is, only the first character is.)
      */
     var.vt = VT_LPSTR;
-    U(var).pszVal = (LPSTR)wval;
+    var.pszVal = (LPSTR)wval;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_LPSTR && !strcmp(U(var).pszVal, "h"),
+    ok(var.vt == VT_LPSTR && !strcmp(var.pszVal, "h"),
      "Didn't get expected type or value for property\n");
     PropVariantClear(&var);
 
     /* now that a property's been set, you can't change the code page */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_CODEPAGE;
+    spec.propid = PID_CODEPAGE;
     var.vt = VT_I2;
-    U(var).iVal = 1200;
+    var.iVal = 1200;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == STG_E_INVALIDPARAMETER,
      "Expected STG_E_INVALIDPARAMETER, got 0x%08lx\n", hr);
@@ -512,13 +512,13 @@ static void testCodepage(void)
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
     ok(var.vt == VT_I2, "Didn't get expected type for property (%u)\n", var.vt);
     /* Set code page to Unicode */
-    U(var).iVal = 1200;
+    var.iVal = 1200;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* check code page */
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_I2 && U(var).iVal == 1200,
+    ok(var.vt == VT_I2 && var.iVal == 1200,
      "Didn't get expected type or value for property\n");
     /* This test is commented out for documentation.  It fails under Wine,
      * and I expect it would under Windows as well, yet it succeeds.  There's
@@ -527,22 +527,22 @@ static void testCodepage(void)
     if(0) {
     static unsigned char strVal[] = { 0x81, 0xff, 0x04, 0 };
     /* Set code page to 950 (Traditional Chinese) */
-    U(var).iVal = 950;
+    var.iVal = 950;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* Try writing an invalid string: lead byte 0x81 is unused in Traditional
      * Chinese.
      */
     spec.ulKind = PRSPEC_PROPID;
-    U(spec).propid = PID_FIRST_USABLE;
+    spec.propid = PID_FIRST_USABLE;
     var.vt = VT_LPSTR;
-    U(var).pszVal = (LPSTR)strVal;
+    var.pszVal = (LPSTR)strVal;
     hr = IPropertyStorage_WriteMultiple(propertyStorage, 1, &spec, &var, 0);
     ok(hr == S_OK, "WriteMultiple failed: 0x%08lx\n", hr);
     /* Check returned string */
     hr = IPropertyStorage_ReadMultiple(propertyStorage, 1, &spec, &var);
     ok(hr == S_OK, "ReadMultiple failed: 0x%08lx\n", hr);
-    ok(var.vt == VT_LPSTR && !strcmp(U(var).pszVal, (LPCSTR)strVal),
+    ok(var.vt == VT_LPSTR && !strcmp(var.pszVal, (LPCSTR)strVal),
      "Didn't get expected type or value for property\n");
     }
 
@@ -623,16 +623,96 @@ static void testFmtId(void)
      "Got unexpected FMTID, expected IID_IPropertySetStorage\n");
 }
 
+typedef struct
+{
+    WORD byte_order;
+    WORD format;
+    DWORD os_ver;
+    CLSID clsid;
+    DWORD reserved;
+} PROPERTYSETHEADER;
+
+typedef struct
+{
+    FMTID fmtid;
+    DWORD offset;
+} FORMATIDOFFSET;
+
+typedef struct
+{
+    DWORD size;
+    DWORD properties;
+} PROPERTYSECTIONHEADER;
+
+typedef struct
+{
+    DWORD propid;
+    DWORD offset;
+} PROPERTYIDOFFSET;
+
+typedef struct
+{
+    DWORD type;
+    DWORD data;
+} PROPVARIANT_DWORD;
+
+struct test_prop_data
+{
+    PROPERTYSETHEADER header;
+    FORMATIDOFFSET doc_summary;
+    FORMATIDOFFSET user_def_props;
+    PROPERTYSECTIONHEADER doc_summary_header;
+    PROPERTYIDOFFSET prop1;
+    PROPVARIANT_DWORD prop1_val;
+    PROPERTYSECTIONHEADER user_def_props_header;
+    PROPERTYIDOFFSET prop2;
+    PROPVARIANT_DWORD prop2_val;
+} test_prop_data = {
+    {
+        0xfffe, 0, 0x2000a00,
+        /* IID_IUnknown */
+        {0x00000000, 0x0000, 0x0000, {0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46}},
+        2
+    },
+    {
+        /* FMTID_DocSummaryInformation */
+        {0xd5cdd502, 0x2e9c, 0x101b, {0x93, 0x97, 0x08, 0x00, 0x2b, 0x2c, 0xf9, 0xae}},
+        FIELD_OFFSET(struct test_prop_data, doc_summary_header)
+    },
+    {
+        /* FMTID_UserDefinedProperties */
+        {0xd5cdd505, 0x2e9c, 0x101b, {0x93, 0x97, 0x08, 0x00, 0x2b, 0x2c, 0xf9, 0xae}},
+        FIELD_OFFSET(struct test_prop_data, user_def_props_header)
+    },
+    {
+        sizeof(PROPERTYSECTIONHEADER) + sizeof(PROPERTYIDOFFSET) + sizeof(PROPVARIANT_DWORD),
+        1
+    },
+    { 0x10, sizeof(PROPERTYSECTIONHEADER) + sizeof(PROPERTYIDOFFSET) },
+    { VT_UI4, 11 },
+    {
+        sizeof(PROPERTYSECTIONHEADER) + sizeof(PROPERTYIDOFFSET) + sizeof(PROPVARIANT_DWORD),
+        1
+    },
+    { 0x10, sizeof(PROPERTYSECTIONHEADER) + sizeof(PROPERTYIDOFFSET) },
+    { VT_I4, 10 }
+};
+
 static void test_propertyset_storage_enum(void)
 {
     IPropertyStorage *prop_storage, *prop_storage2;
     IPropertySetStorage *ps_storage;
     IEnumSTATPROPSETSTG *ps_enum;
+    IEnumSTATPROPSTG *prop_enum;
     WCHAR filename[MAX_PATH];
     STATPROPSETSTG psstg;
+    STATPROPSTG pstg;
     DWORD ret, fetched;
     IStorage *storage;
+    IStream *stream;
     FILETIME ftime;
+    PROPVARIANT pv;
+    PROPSPEC ps;
     HRESULT hr;
 
     ret = GetTempFileNameW(L".", L"stg", 0, filename);
@@ -733,6 +813,120 @@ todo_wine {
     IPropertySetStorage_Release(ps_storage);
     IStorage_Release(storage);
 
+    /* test FMTID_UserDefinedProperties */
+    hr = StgCreateDocfile(filename, STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE, 0, &storage);
+    ok(hr == S_OK, "Failed to crate storage, hr %#lx.\n", hr);
+
+    hr = StgCreatePropSetStg(storage, 0, &ps_storage);
+    ok(hr == S_OK, "Failed to create property set storage, hr %#lx.\n", hr);
+
+    hr = IPropertySetStorage_Create(ps_storage, &FMTID_UserDefinedProperties, &IID_IUnknown,
+            PROPSETFLAG_ANSI, STGM_READWRITE | STGM_CREATE | STGM_SHARE_EXCLUSIVE, &prop_storage);
+    ok(hr == S_OK, "Failed to create property storage, hr %#lx.\n", hr);
+    ps.ulKind = PRSPEC_PROPID;
+    ps.propid = 0x10;
+    pv.vt = VT_I4;
+    pv.lVal = 10;
+    hr = IPropertyStorage_WriteMultiple(prop_storage, 1, &ps, &pv, 0x10);
+    ok(hr == S_OK, "Failed to add property, hr %#lx.\n", hr);
+    IPropertyStorage_Release(prop_storage);
+
+    hr = IPropertySetStorage_Open(ps_storage, &FMTID_DocSummaryInformation,
+            STGM_READWRITE | STGM_SHARE_EXCLUSIVE, &prop_storage);
+    ok(hr == S_OK, "Failed to open FMTID_DocSummaryInformation, hr %#lx.\n", hr);
+    pv.vt = VT_UI4;
+    pv.lVal = 11;
+    hr = IPropertyStorage_WriteMultiple(prop_storage, 1, &ps, &pv, 0x10);
+    ok(hr == S_OK, "Failed to add property, hr %#lx.\n", hr);
+    IPropertyStorage_Release(prop_storage);
+
+    hr = IPropertySetStorage_Enum(ps_storage, &ps_enum);
+    ok(hr == S_OK, "Failed to get enum object, hr %#lx.\n", hr);
+    memset(&psstg, 0, sizeof(psstg));
+    hr = IEnumSTATPROPSETSTG_Next(ps_enum, 1, &psstg, &fetched);
+    ok(hr == S_OK, "Failed to get enum item, hr %#lx.\n", hr);
+    ok(fetched == 1, "Unexpected fetched count.\n");
+    ok(IsEqualCLSID(&psstg.fmtid, &FMTID_DocSummaryInformation), "Unexpected fmtid %s.\n",
+            wine_dbgstr_guid(&psstg.fmtid));
+    memset(&psstg, 0, sizeof(psstg));
+    hr = IEnumSTATPROPSETSTG_Next(ps_enum, 1, &psstg, &fetched);
+    ok(hr == S_FALSE, "Unexpected hr %#lx.\n", hr);
+    IEnumSTATPROPSETSTG_Release(ps_enum);
+
+    hr = IPropertySetStorage_Open(ps_storage, &FMTID_DocSummaryInformation,
+            STGM_READ | STGM_SHARE_EXCLUSIVE, &prop_storage);
+    ok(hr == S_OK, "Failed to open FMTID_DocSummaryInformation, hr %#lx.\n", hr);
+    hr = IPropertyStorage_Enum(prop_storage, &prop_enum);
+    ok(hr == S_OK, "IPropertyStorage_Enum failed, hr %#lx.\n", hr);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_OK, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    ok(pstg.propid == 0x10, "pstg.propid = %lx\n", pstg.propid);
+    ok(pstg.vt == VT_UI4, "pstg.vt = %d\n", pstg.vt);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_FALSE, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    IEnumSTATPROPSTG_Release(prop_enum);
+    IPropertyStorage_Release(prop_storage);
+
+    hr = IPropertySetStorage_Open(ps_storage, &FMTID_UserDefinedProperties,
+            STGM_READ | STGM_SHARE_EXCLUSIVE, &prop_storage);
+    ok(hr == S_OK, "Failed to open FMTID_DocSummaryInformation, hr %#lx.\n", hr);
+    hr = IPropertyStorage_Enum(prop_storage, &prop_enum);
+    ok(hr == S_OK, "IPropertyStorage_Enum failed, hr %#lx.\n", hr);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_OK, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    ok(pstg.propid == 0x10, "pstg.propid = %lx\n", pstg.propid);
+    todo_wine ok(pstg.vt == VT_I4, "pstg.vt = %d\n", pstg.vt);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_FALSE, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    IEnumSTATPROPSTG_Release(prop_enum);
+    IPropertyStorage_Release(prop_storage);
+
+    hr = IStorage_OpenStream(storage, L"\5DocumentSummaryInformation", NULL,
+            STGM_READWRITE | STGM_SHARE_EXCLUSIVE, 0, &stream);
+    ok(hr == S_OK, "IStorage_CreateStream failed, hr %#lx.\n", hr);
+    hr = IStream_Write(stream, &test_prop_data, sizeof(test_prop_data), NULL);
+    ok(hr == S_OK, "IStream_Write failed, hr %#lx.\n", hr);
+    IStream_Release(stream);
+
+    hr = IPropertySetStorage_Open(ps_storage, &FMTID_DocSummaryInformation,
+            STGM_READ | STGM_SHARE_EXCLUSIVE, &prop_storage);
+    ok(hr == S_OK, "Failed to open FMTID_DocSummaryInformation, hr %#lx.\n", hr);
+    hr = IPropertyStorage_Enum(prop_storage, &prop_enum);
+    ok(hr == S_OK, "IPropertyStorage_Enum failed, hr %#lx.\n", hr);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_OK, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    ok(pstg.propid == 0x10, "pstg.propid = %lx\n", pstg.propid);
+    ok(pstg.vt == VT_UI4, "pstg.vt = %d\n", pstg.vt);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_FALSE, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    IEnumSTATPROPSTG_Release(prop_enum);
+    IPropertyStorage_Release(prop_storage);
+
+    hr = IPropertySetStorage_Open(ps_storage, &FMTID_UserDefinedProperties,
+            STGM_READ | STGM_SHARE_EXCLUSIVE, &prop_storage);
+    ok(hr == S_OK, "Failed to open FMTID_DocSummaryInformation, hr %#lx.\n", hr);
+    hr = IPropertyStorage_Enum(prop_storage, &prop_enum);
+    ok(hr == S_OK, "IPropertyStorage_Enum failed, hr %#lx.\n", hr);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_OK, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    ok(pstg.propid == 0x10, "pstg.propid = %lx\n", pstg.propid);
+    todo_wine ok(pstg.vt == VT_I4, "pstg.vt = %d\n", pstg.vt);
+    memset(&pstg, 0, sizeof(pstg));
+    hr = IEnumSTATPROPSTG_Next(prop_enum, 1, &pstg, &fetched);
+    ok(hr == S_FALSE, "IEnumSTATPROPSTG_Next failed, hr %#lx.\n", hr);
+    IEnumSTATPROPSTG_Release(prop_enum);
+    IPropertyStorage_Release(prop_storage);
+
+    IPropertySetStorage_Release(ps_storage);
+    IStorage_Release(storage);
+
     ret = DeleteFileW(filename);
     ok(ret, "Failed to delete storage file.\n");
 }
