@@ -55,14 +55,14 @@ struct opc_uri
     struct opc_uri *source_uri;
 };
 
-extern HRESULT opc_package_create(IOpcFactory *factory, IOpcPackage **package) DECLSPEC_HIDDEN;
-extern HRESULT opc_part_uri_create(IUri *uri, struct opc_uri *source_uri, IOpcPartUri **part_uri) DECLSPEC_HIDDEN;
-extern HRESULT opc_root_uri_create(IOpcUri **opc_uri) DECLSPEC_HIDDEN;
+extern HRESULT opc_package_create(IOpcFactory *factory, IOpcPackage **package);
+extern HRESULT opc_part_uri_create(IUri *uri, struct opc_uri *source_uri, IOpcPartUri **part_uri);
+extern HRESULT opc_root_uri_create(IOpcUri **opc_uri);
 
-extern HRESULT opc_package_write(IOpcPackage *package, OPC_WRITE_FLAGS flags, IStream *stream) DECLSPEC_HIDDEN;
+extern HRESULT opc_package_write(IOpcPackage *package, OPC_WRITE_FLAGS flags, IStream *stream);
 
 struct zip_archive;
-extern HRESULT compress_create_archive(IStream *output, struct zip_archive **archive) DECLSPEC_HIDDEN;
+extern HRESULT compress_create_archive(IStream *output, struct zip_archive **archive);
 extern HRESULT compress_add_file(struct zip_archive *archive, const WCHAR *path, IStream *content,
-        OPC_COMPRESSION_OPTIONS options) DECLSPEC_HIDDEN;
-extern void compress_finalize_archive(struct zip_archive *archive) DECLSPEC_HIDDEN;
+        OPC_COMPRESSION_OPTIONS options);
+extern void compress_finalize_archive(struct zip_archive *archive);
