@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include <assert.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <math.h>
@@ -35,12 +36,18 @@
 
 #include "wine/glu.h"
 #include "wine/debug.h"
+#include "wine/wgl_driver.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(opengl);
 WINE_DECLARE_DEBUG_CHANNEL(fps);
 
 static const MAT2 identity = { {0,1},{0,0},{0,0},{0,1} };
 
+#define WINE_GL_RESERVED_FORMATS_HDC      2
+#define WINE_GL_RESERVED_FORMATS_PTR      3
+#define WINE_GL_RESERVED_FORMATS_NUM      4
+#define WINE_GL_RESERVED_FORMATS_ONSCREEN 5
+
 #ifndef _WIN64
 
 static char **wow64_strings;
@@ -278,7 +285,14 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
                 continue;
             }
         }
-        if (ppfd->dwFlags & PFD_DEPTH_DONTCARE && format.cDepthBits < best.cDepthBits)
+        if (ppfd->dwFlags & PFD_DEPTH_DONTCARE)
+        {
+            if (format.cDepthBits < best.cDepthBits)
+                goto found;
+            continue;
+        }
+
+        if (!ppfd->cDepthBits && format.cDepthBits > best.cDepthBits)
             goto found;
 
         continue;
@@ -294,6 +308,587 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
     return best_format;
 }
 
+static struct wgl_pixel_format *get_pixel_formats( HDC hdc, UINT *num_formats,
+                                                   UINT *num_onscreen_formats )
+{
+    struct get_pixel_formats_params args = { .teb = NtCurrentTeb(), .hdc = hdc };
+    PVOID *glReserved = NtCurrentTeb()->glReserved1;
+    NTSTATUS status;
+
+    if (glReserved[WINE_GL_RESERVED_FORMATS_HDC] == hdc)
+    {
+        *num_formats = PtrToUlong( glReserved[WINE_GL_RESERVED_FORMATS_NUM] );
+        *num_onscreen_formats = PtrToUlong( glReserved[WINE_GL_RESERVED_FORMATS_ONSCREEN] );
+        return glReserved[WINE_GL_RESERVED_FORMATS_PTR];
+    }
+
+    if ((status = UNIX_CALL( get_pixel_formats, &args ))) goto error;
+    /* Clear formats memory since not all drivers deal with all wgl_pixel_format
+     * fields at the moment. */
+    if (!(args.formats = calloc( args.num_formats, sizeof(*args.formats) ))) goto error;
+    args.max_formats = args.num_formats;
+    if ((status = UNIX_CALL( get_pixel_formats, &args ))) goto error;
+
+    *num_formats = args.num_formats;
+    *num_onscreen_formats = args.num_onscreen_formats;
+
+    free( glReserved[WINE_GL_RESERVED_FORMATS_PTR] );
+    glReserved[WINE_GL_RESERVED_FORMATS_HDC] = hdc;
+    glReserved[WINE_GL_RESERVED_FORMATS_PTR] = args.formats;
+    glReserved[WINE_GL_RESERVED_FORMATS_NUM] = ULongToPtr( args.num_formats );
+    glReserved[WINE_GL_RESERVED_FORMATS_ONSCREEN] = ULongToPtr( args.num_onscreen_formats );
+
+    return args.formats;
+
+error:
+    *num_formats = *num_onscreen_formats = 0;
+    free( args.formats );
+    return NULL;
+}
+
+static BOOL wgl_attrib_uses_layer( int attrib )
+{
+    switch (attrib)
+    {
+    case WGL_ACCELERATION_ARB:
+    case WGL_TRANSPARENT_ARB:
+    case WGL_SHARE_DEPTH_ARB:
+    case WGL_SHARE_STENCIL_ARB:
+    case WGL_SHARE_ACCUM_ARB:
+    case WGL_TRANSPARENT_RED_VALUE_ARB:
+    case WGL_TRANSPARENT_GREEN_VALUE_ARB:
+    case WGL_TRANSPARENT_BLUE_VALUE_ARB:
+    case WGL_TRANSPARENT_ALPHA_VALUE_ARB:
+    case WGL_TRANSPARENT_INDEX_VALUE_ARB:
+    case WGL_SUPPORT_GDI_ARB:
+    case WGL_SUPPORT_OPENGL_ARB:
+    case WGL_DOUBLE_BUFFER_ARB:
+    case WGL_STEREO_ARB:
+    case WGL_PIXEL_TYPE_ARB:
+    case WGL_COLOR_BITS_ARB:
+    case WGL_RED_BITS_ARB:
+    case WGL_RED_SHIFT_ARB:
+    case WGL_GREEN_BITS_ARB:
+    case WGL_GREEN_SHIFT_ARB:
+    case WGL_BLUE_BITS_ARB:
+    case WGL_BLUE_SHIFT_ARB:
+    case WGL_ALPHA_BITS_ARB:
+    case WGL_ALPHA_SHIFT_ARB:
+    case WGL_ACCUM_BITS_ARB:
+    case WGL_ACCUM_RED_BITS_ARB:
+    case WGL_ACCUM_GREEN_BITS_ARB:
+    case WGL_ACCUM_BLUE_BITS_ARB:
+    case WGL_ACCUM_ALPHA_BITS_ARB:
+    case WGL_DEPTH_BITS_ARB:
+    case WGL_STENCIL_BITS_ARB:
+    case WGL_AUX_BUFFERS_ARB:
+    case WGL_SAMPLE_BUFFERS_ARB:
+    case WGL_SAMPLES_ARB:
+    case WGL_FRAMEBUFFER_SRGB_CAPABLE_ARB:
+    case WGL_FLOAT_COMPONENTS_NV:
+        return TRUE;
+    default:
+        return FALSE;
+    }
+}
+
+static BOOL wgl_pixel_format_get_attrib( const struct wgl_pixel_format *fmt, int attrib, int *value )
+{
+    int val = 0;
+    int valid = -1;
+
+    switch (attrib)
+    {
+    case WGL_DRAW_TO_WINDOW_ARB: val = !!(fmt->pfd.dwFlags & PFD_DRAW_TO_WINDOW); break;
+    case WGL_DRAW_TO_BITMAP_ARB: val = !!(fmt->pfd.dwFlags & PFD_DRAW_TO_BITMAP); break;
+    case WGL_ACCELERATION_ARB:
+        if (fmt->pfd.dwFlags & PFD_GENERIC_ACCELERATED)
+            val = WGL_GENERIC_ACCELERATION_ARB;
+        else if (fmt->pfd.dwFlags & PFD_GENERIC_FORMAT)
+            val = WGL_NO_ACCELERATION_ARB;
+        else
+            val = WGL_FULL_ACCELERATION_ARB;
+        break;
+    case WGL_NEED_PALETTE_ARB: val = !!(fmt->pfd.dwFlags & PFD_NEED_PALETTE); break;
+    case WGL_NEED_SYSTEM_PALETTE_ARB: val = !!(fmt->pfd.dwFlags & PFD_NEED_SYSTEM_PALETTE); break;
+    case WGL_SWAP_LAYER_BUFFERS_ARB: val = !!(fmt->pfd.dwFlags & PFD_SWAP_LAYER_BUFFERS); break;
+    case WGL_SWAP_METHOD_ARB: val = fmt->swap_method; break;
+    case WGL_NUMBER_OVERLAYS_ARB:
+    case WGL_NUMBER_UNDERLAYS_ARB:
+        /* We don't support any overlays/underlays. */
+        val = 0;
+        break;
+    case WGL_TRANSPARENT_ARB: val = fmt->transparent; break;
+    case WGL_SHARE_DEPTH_ARB:
+    case WGL_SHARE_STENCIL_ARB:
+    case WGL_SHARE_ACCUM_ARB:
+        /* We support only a main plane at the moment which by definition
+         * shares the depth/stencil/accum buffers with itself. */
+        val = GL_TRUE;
+        break;
+    case WGL_SUPPORT_GDI_ARB: val = !!(fmt->pfd.dwFlags & PFD_SUPPORT_GDI); break;
+    case WGL_SUPPORT_OPENGL_ARB: val = !!(fmt->pfd.dwFlags & PFD_SUPPORT_OPENGL); break;
+    case WGL_DOUBLE_BUFFER_ARB: val = !!(fmt->pfd.dwFlags & PFD_DOUBLEBUFFER); break;
+    case WGL_STEREO_ARB: val = !!(fmt->pfd.dwFlags & PFD_STEREO); break;
+    case WGL_PIXEL_TYPE_ARB: val = fmt->pixel_type; break;
+    case WGL_COLOR_BITS_ARB: val = fmt->pfd.cColorBits; break;
+    case WGL_RED_BITS_ARB: val = fmt->pfd.cRedBits; break;
+    case WGL_RED_SHIFT_ARB: val = fmt->pfd.cRedShift; break;
+    case WGL_GREEN_BITS_ARB: val = fmt->pfd.cGreenBits; break;
+    case WGL_GREEN_SHIFT_ARB: val = fmt->pfd.cGreenShift; break;
+    case WGL_BLUE_BITS_ARB: val = fmt->pfd.cBlueBits; break;
+    case WGL_BLUE_SHIFT_ARB: val = fmt->pfd.cBlueShift; break;
+    case WGL_ALPHA_BITS_ARB: val = fmt->pfd.cAlphaBits; break;
+    case WGL_ALPHA_SHIFT_ARB: val = fmt->pfd.cAlphaShift; break;
+    case WGL_ACCUM_BITS_ARB: val = fmt->pfd.cAccumBits; break;
+    case WGL_ACCUM_RED_BITS_ARB: val = fmt->pfd.cAccumRedBits; break;
+    case WGL_ACCUM_GREEN_BITS_ARB: val = fmt->pfd.cAccumGreenBits; break;
+    case WGL_ACCUM_BLUE_BITS_ARB: val = fmt->pfd.cAccumBlueBits; break;
+    case WGL_ACCUM_ALPHA_BITS_ARB: val = fmt->pfd.cAccumAlphaBits; break;
+    case WGL_DEPTH_BITS_ARB: val = fmt->pfd.cDepthBits; break;
+    case WGL_STENCIL_BITS_ARB: val = fmt->pfd.cStencilBits; break;
+    case WGL_AUX_BUFFERS_ARB: val = fmt->pfd.cAuxBuffers; break;
+    case WGL_DRAW_TO_PBUFFER_ARB: val = fmt->draw_to_pbuffer; break;
+    case WGL_MAX_PBUFFER_PIXELS_ARB: val = fmt->max_pbuffer_pixels; break;
+    case WGL_MAX_PBUFFER_WIDTH_ARB: val = fmt->max_pbuffer_width; break;
+    case WGL_MAX_PBUFFER_HEIGHT_ARB: val = fmt->max_pbuffer_height; break;
+    case WGL_TRANSPARENT_RED_VALUE_ARB:
+        val = fmt->transparent_red_value;
+        valid = !!fmt->transparent_red_value_valid;
+        break;
+    case WGL_TRANSPARENT_GREEN_VALUE_ARB:
+        val = fmt->transparent_green_value;
+        valid = !!fmt->transparent_green_value_valid;
+        break;
+    case WGL_TRANSPARENT_BLUE_VALUE_ARB:
+        val = fmt->transparent_blue_value;
+        valid = !!fmt->transparent_blue_value_valid;
+        break;
+    case WGL_TRANSPARENT_ALPHA_VALUE_ARB:
+        val = fmt->transparent_alpha_value;
+        valid = !!fmt->transparent_alpha_value_valid;
+        break;
+    case WGL_TRANSPARENT_INDEX_VALUE_ARB:
+        val = fmt->transparent_index_value;
+        valid = !!fmt->transparent_index_value_valid;
+        break;
+    case WGL_SAMPLE_BUFFERS_ARB: val = fmt->sample_buffers; break;
+    case WGL_SAMPLES_ARB: val = fmt->samples; break;
+    case WGL_BIND_TO_TEXTURE_RGB_ARB: val = fmt->bind_to_texture_rgb; break;
+    case WGL_BIND_TO_TEXTURE_RGBA_ARB: val = fmt->bind_to_texture_rgba; break;
+    case WGL_BIND_TO_TEXTURE_RECTANGLE_RGB_NV: val = fmt->bind_to_texture_rectangle_rgb; break;
+    case WGL_BIND_TO_TEXTURE_RECTANGLE_RGBA_NV: val = fmt->bind_to_texture_rectangle_rgba; break;
+    case WGL_FRAMEBUFFER_SRGB_CAPABLE_ARB: val = fmt->framebuffer_srgb_capable; break;
+    case WGL_FLOAT_COMPONENTS_NV: val = fmt->float_components; break;
+    default:
+        FIXME( "unsupported 0x%x WGL attribute\n", attrib );
+        valid = 0;
+        break;
+    }
+
+    /* If we haven't already determined validity, use the default check */
+    if (valid == -1) valid = val != -1;
+    if (valid) *value = val;
+
+    return valid;
+}
+
+enum attrib_match
+{
+    ATTRIB_MATCH_INVALID = -1,
+    ATTRIB_MATCH_IGNORE,
+    ATTRIB_MATCH_EXACT,
+    ATTRIB_MATCH_MINIMUM,
+};
+
+static enum attrib_match wgl_attrib_match_criteria( int attrib )
+{
+    switch (attrib)
+    {
+    case WGL_DRAW_TO_WINDOW_ARB:
+    case WGL_DRAW_TO_BITMAP_ARB:
+    case WGL_ACCELERATION_ARB:
+    case WGL_NEED_PALETTE_ARB:
+    case WGL_NEED_SYSTEM_PALETTE_ARB:
+    case WGL_SWAP_LAYER_BUFFERS_ARB:
+    case WGL_SWAP_METHOD_ARB:
+    case WGL_SHARE_DEPTH_ARB:
+    case WGL_SHARE_STENCIL_ARB:
+    case WGL_SHARE_ACCUM_ARB:
+    case WGL_SUPPORT_GDI_ARB:
+    case WGL_SUPPORT_OPENGL_ARB:
+    case WGL_DOUBLE_BUFFER_ARB:
+    case WGL_STEREO_ARB:
+    case WGL_PIXEL_TYPE_ARB:
+    case WGL_DRAW_TO_PBUFFER_ARB:
+    case WGL_BIND_TO_TEXTURE_RGB_ARB:
+    case WGL_BIND_TO_TEXTURE_RGBA_ARB:
+    case WGL_BIND_TO_TEXTURE_RECTANGLE_RGB_NV:
+    case WGL_BIND_TO_TEXTURE_RECTANGLE_RGBA_NV:
+    case WGL_FRAMEBUFFER_SRGB_CAPABLE_ARB:
+    case WGL_FLOAT_COMPONENTS_NV:
+        return ATTRIB_MATCH_EXACT;
+    case WGL_NUMBER_OVERLAYS_ARB:
+    case WGL_NUMBER_UNDERLAYS_ARB:
+    case WGL_COLOR_BITS_ARB:
+    case WGL_RED_BITS_ARB:
+    case WGL_GREEN_BITS_ARB:
+    case WGL_BLUE_BITS_ARB:
+    case WGL_ALPHA_BITS_ARB:
+    case WGL_ACCUM_BITS_ARB:
+    case WGL_ACCUM_RED_BITS_ARB:
+    case WGL_ACCUM_GREEN_BITS_ARB:
+    case WGL_ACCUM_BLUE_BITS_ARB:
+    case WGL_ACCUM_ALPHA_BITS_ARB:
+    case WGL_DEPTH_BITS_ARB:
+    case WGL_STENCIL_BITS_ARB:
+    case WGL_AUX_BUFFERS_ARB:
+    case WGL_SAMPLE_BUFFERS_ARB:
+    case WGL_SAMPLES_ARB:
+        return ATTRIB_MATCH_MINIMUM;
+    case WGL_NUMBER_PIXEL_FORMATS_ARB:
+    case WGL_RED_SHIFT_ARB:
+    case WGL_GREEN_SHIFT_ARB:
+    case WGL_BLUE_SHIFT_ARB:
+    case WGL_ALPHA_SHIFT_ARB:
+    case WGL_TRANSPARENT_ARB:
+    case WGL_TRANSPARENT_RED_VALUE_ARB:
+    case WGL_TRANSPARENT_GREEN_VALUE_ARB:
+    case WGL_TRANSPARENT_BLUE_VALUE_ARB:
+    case WGL_TRANSPARENT_ALPHA_VALUE_ARB:
+    case WGL_TRANSPARENT_INDEX_VALUE_ARB:
+        return ATTRIB_MATCH_IGNORE;
+    default:
+        return ATTRIB_MATCH_INVALID;
+    }
+}
+
+static void filter_format_array( const struct wgl_pixel_format **array,
+                                 UINT num_formats, int attrib, int value )
+{
+    enum attrib_match match = wgl_attrib_match_criteria( attrib );
+    int fmt_value;
+    UINT i;
+
+    assert(match != ATTRIB_MATCH_INVALID);
+
+    if (match == ATTRIB_MATCH_IGNORE) return;
+
+    for (i = 0; i < num_formats; ++i)
+    {
+        if (!array[i]) continue;
+        if (!wgl_pixel_format_get_attrib( array[i], attrib, &fmt_value ) ||
+            (match == ATTRIB_MATCH_EXACT && fmt_value != value) ||
+            (match == ATTRIB_MATCH_MINIMUM && fmt_value < value))
+        {
+            array[i] = NULL;
+        }
+    }
+}
+
+static int wgl_attrib_sort_priority( int attrib )
+{
+    switch (attrib)
+    {
+    case WGL_DRAW_TO_WINDOW_ARB: return 1;
+    case WGL_DRAW_TO_BITMAP_ARB: return 2;
+    case WGL_ACCELERATION_ARB: return 3;
+    case WGL_COLOR_BITS_ARB: return 4;
+    case WGL_ACCUM_BITS_ARB: return 5;
+    case WGL_PIXEL_TYPE_ARB: return 6;
+    case WGL_ALPHA_BITS_ARB: return 7;
+    case WGL_AUX_BUFFERS_ARB: return 8;
+    case WGL_DEPTH_BITS_ARB: return 9;
+    case WGL_STENCIL_BITS_ARB: return 10;
+    case WGL_DOUBLE_BUFFER_ARB: return 11;
+    case WGL_SWAP_METHOD_ARB: return 12;
+    default: return 100;
+    }
+}
+
+static int compare_attribs( const void *a, const void *b )
+{
+    return wgl_attrib_sort_priority( *(int *)a ) - wgl_attrib_sort_priority( *(int *)b );
+}
+
+static int wgl_attrib_value_priority( int value )
+{
+    switch (value)
+    {
+    case WGL_SWAP_UNDEFINED_ARB: return 1;
+    case WGL_SWAP_EXCHANGE_ARB: return 2;
+    case WGL_SWAP_COPY_ARB: return 3;
+
+    case WGL_FULL_ACCELERATION_ARB: return 1;
+    case WGL_GENERIC_ACCELERATION_ARB: return 2;
+    case WGL_NO_ACCELERATION_ARB: return 3;
+
+    case WGL_TYPE_RGBA_ARB: return 1;
+    case WGL_TYPE_RGBA_FLOAT_ATI: return 2;
+    case WGL_TYPE_RGBA_UNSIGNED_FLOAT_EXT: return 3;
+    case WGL_TYPE_COLORINDEX_ARB: return 4;
+
+    default: return 100;
+    }
+}
+
+struct compare_formats_ctx
+{
+    int attribs[256];
+    UINT num_attribs;
+};
+
+static int compare_formats( void *arg, const void *a, const void *b )
+{
+    const struct wgl_pixel_format *fmt_a = *(void **)a, *fmt_b = *(void **)b;
+    struct compare_formats_ctx *ctx = arg;
+    int attrib, val_a, val_b;
+    UINT i;
+
+    if (!fmt_a) return 1;
+    if (!fmt_b) return -1;
+
+    for (i = 0; i < ctx->num_attribs; ++i)
+    {
+        attrib = ctx->attribs[2 * i];
+        if (wgl_pixel_format_get_attrib( fmt_a, attrib, &val_a ) &&
+            wgl_pixel_format_get_attrib( fmt_b, attrib, &val_b ) &&
+            val_a != val_b)
+        {
+            switch (attrib)
+            {
+            case WGL_ACCELERATION_ARB:
+            case WGL_SWAP_METHOD_ARB:
+            case WGL_PIXEL_TYPE_ARB:
+                return wgl_attrib_value_priority( val_a ) -
+                       wgl_attrib_value_priority( val_b );
+            case WGL_COLOR_BITS_ARB:
+                /* Prefer 32bpp over other values */
+                if (val_a >= 32 && val_b >= 32) return val_a - val_b;
+                else return val_b - val_a;
+            default:
+                /* Smaller values first */
+                return val_a - val_b;
+            }
+        }
+    }
+
+    /* Maintain pixel format id order */
+    return fmt_a - fmt_b;
+}
+
+static void compare_formats_ctx_set_attrib( struct compare_formats_ctx *ctx,
+                                            int attrib, int value )
+{
+    UINT i;
+
+    /* Overwrite attribute if it exists already */
+    for (i = 0; i < ctx->num_attribs; ++i)
+        if (ctx->attribs[2 * i] == attrib) break;
+
+    assert(i < ARRAY_SIZE(ctx->attribs) / 2);
+
+    ctx->attribs[2 * i] = attrib;
+    ctx->attribs[2 * i + 1] = value;
+    if (i == ctx->num_attribs) ++ctx->num_attribs;
+}
+
+/***********************************************************************
+ *		wglChoosePixelFormatARB (OPENGL32.@)
+ */
+BOOL WINAPI wglChoosePixelFormatARB( HDC hdc, const int *attribs_int, const FLOAT *attribs_float,
+                                     UINT max_formats, int *formats, UINT *num_formats )
+{
+    struct wgl_pixel_format *wgl_formats;
+    UINT i, num_wgl_formats, num_wgl_onscreen_formats;
+    const struct wgl_pixel_format **format_array;
+    struct compare_formats_ctx ctx = { 0 };
+
+    TRACE( "hdc %p, attribs_int %p, attribs_float %p, max_formats %u, formats %p, num_formats %p\n",
+           hdc, attribs_int, attribs_float, max_formats, formats, num_formats );
+
+    wgl_formats = get_pixel_formats( hdc, &num_wgl_formats, &num_wgl_onscreen_formats );
+
+    /* If the driver doesn't yet provide ARB attrib information in
+     * wgl_pixel_format, fall back to an explicit call. */
+    if (num_wgl_formats && !wgl_formats[0].pixel_type)
+    {
+        struct wglChoosePixelFormatARB_params args =
+        {
+            .teb = NtCurrentTeb(),
+            .hdc = hdc,
+            .piAttribIList = attribs_int,
+            .pfAttribFList = attribs_float,
+            .nMaxFormats = max_formats,
+            .piFormats = formats,
+            .nNumFormats = num_formats
+        };
+        NTSTATUS status;
+
+        if ((status = UNIX_CALL( wglChoosePixelFormatARB, &args )))
+            WARN( "wglChoosePixelFormatARB returned %#lx\n", status );
+
+        return args.ret;
+    }
+
+    /* Gather, validate and deduplicate all attributes */
+    for (i = 0; attribs_int && attribs_int[i]; i += 2)
+    {
+        if (wgl_attrib_match_criteria( attribs_int[i] ) == ATTRIB_MATCH_INVALID) return FALSE;
+        compare_formats_ctx_set_attrib( &ctx, attribs_int[i], attribs_int[i + 1] );
+    }
+    for (i = 0; attribs_float && attribs_float[i]; i += 2)
+    {
+        if (wgl_attrib_match_criteria( attribs_float[i] ) == ATTRIB_MATCH_INVALID) return FALSE;
+        compare_formats_ctx_set_attrib( &ctx, attribs_float[i], attribs_float[i + 1] );
+    }
+
+    /* Initialize the format_array with (pointers to) all wgl formats */
+    format_array = malloc( num_wgl_formats * sizeof(*format_array) );
+    if (!format_array) return FALSE;
+    for (i = 0; i < num_wgl_formats; ++i) format_array[i] = &wgl_formats[i];
+
+    /* Remove formats that are not acceptable */
+    for (i = 0; i < ctx.num_attribs; ++i)
+        filter_format_array( format_array, num_wgl_formats, ctx.attribs[2 * i],
+                             ctx.attribs[2 * i + 1] );
+
+    /* Some attributes we always want to sort by (values don't matter for sorting) */
+    compare_formats_ctx_set_attrib( &ctx, WGL_ACCELERATION_ARB, 0 );
+    compare_formats_ctx_set_attrib( &ctx, WGL_COLOR_BITS_ARB, 0 );
+    compare_formats_ctx_set_attrib( &ctx, WGL_ACCUM_BITS_ARB, 0 );
+
+    /* Arrange attributes in the order which we want to check them */
+    qsort( ctx.attribs, ctx.num_attribs, 2 * sizeof(*ctx.attribs), compare_attribs );
+
+    /* Sort pixel formats based on the specified attributes */
+    qsort_s( format_array, num_wgl_formats, sizeof(*format_array), compare_formats, &ctx );
+
+    /* Return the best max_formats format ids */
+    *num_formats = 0;
+    for (i = 0; i < num_wgl_formats && i < max_formats && format_array[i]; ++i)
+    {
+        ++*num_formats;
+        formats[i] = format_array[i] - wgl_formats + 1;
+    }
+
+    free( format_array );
+    return TRUE;
+}
+
+INT WINAPI wglDescribePixelFormat( HDC hdc, int index, UINT size, PIXELFORMATDESCRIPTOR *ppfd )
+{
+    struct wgl_pixel_format *formats;
+    UINT num_formats, num_onscreen_formats;
+
+    TRACE( "hdc %p, index %d, size %u, ppfd %p\n", hdc, index, index, ppfd );
+
+    if (!(formats = get_pixel_formats( hdc, &num_formats, &num_onscreen_formats ))) return 0;
+    if (!ppfd) return num_onscreen_formats;
+    if (size < sizeof(*ppfd)) return 0;
+    if (index <= 0 || index > num_onscreen_formats) return 0;
+
+    *ppfd = formats[index - 1].pfd;
+
+    return num_onscreen_formats;
+}
+
+/***********************************************************************
+ *		wglGetPixelFormatAttribivARB (OPENGL32.@)
+ */
+BOOL WINAPI wglGetPixelFormatAttribivARB( HDC hdc, int index, int plane, UINT count,
+                                          const int *attributes, int *values )
+{
+    static const DWORD invalid_data_error = 0xC007000D;
+    struct wgl_pixel_format *formats;
+    UINT i, num_formats, num_onscreen_formats;
+
+    TRACE( "hdc %p, index %d, plane %d, count %u, attributes %p, values %p\n",
+           hdc, index, plane, count, attributes, values );
+
+    formats = get_pixel_formats( hdc, &num_formats, &num_onscreen_formats );
+
+    /* If the driver doesn't yet provide ARB attrib information in
+     * wgl_pixel_format, fall back to an explicit call. */
+    if (num_formats && !formats[0].pixel_type)
+    {
+        struct wglGetPixelFormatAttribivARB_params args =
+        {
+            .teb = NtCurrentTeb(),
+            .hdc = hdc,
+            .iPixelFormat = index,
+            .iLayerPlane = plane,
+            .nAttributes = count,
+            .piAttributes = attributes,
+            .piValues = values
+        };
+        NTSTATUS status;
+
+        if ((status = UNIX_CALL( wglGetPixelFormatAttribivARB, &args )))
+            WARN( "wglGetPixelFormatAttribivARB returned %#lx\n", status );
+
+        return args.ret;
+    }
+
+    if (!count) return TRUE;
+    if (count == 1 && attributes[0] == WGL_NUMBER_PIXEL_FORMATS_ARB)
+    {
+        values[0] = num_formats;
+        return TRUE;
+    }
+    if (index <= 0 || index > num_formats)
+    {
+        SetLastError( invalid_data_error );
+        return FALSE;
+    }
+
+    for (i = 0; i < count; ++i)
+    {
+        int attrib = attributes[i];
+
+        if (attrib == WGL_NUMBER_PIXEL_FORMATS_ARB)
+        {
+            values[i] = num_formats;
+        }
+        else if ((plane != 0 && wgl_attrib_uses_layer( attrib )) ||
+                 !wgl_pixel_format_get_attrib( &formats[index - 1], attrib, &values[i] ))
+        {
+            SetLastError( invalid_data_error );
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
+/***********************************************************************
+ *		wglGetPixelFormatAttribfvARB (OPENGL32.@)
+ */
+BOOL WINAPI wglGetPixelFormatAttribfvARB( HDC hdc, int index, int plane, UINT count,
+                                          const int *attributes, FLOAT *values )
+{
+    int *ivalues;
+    BOOL ret;
+    UINT i;
+
+    TRACE( "hdc %p, index %d, plane %d, count %u, attributes %p, values %p\n",
+           hdc, index, plane, count, attributes, values );
+
+    if (!(ivalues = malloc( count * sizeof(int) ))) return FALSE;
+
+    /* For now we can piggy-back on wglGetPixelFormatAttribivARB, since we don't support
+     * any non-integer attributes. */
+    ret = wglGetPixelFormatAttribivARB( hdc, index, plane, count, attributes, ivalues );
+    if (ret)
+    {
+        for (i = 0; i < count; i++)
+            values[i] = ivalues[i];
+    }
+
+    free( ivalues );
+    return ret;
+}
+
 /***********************************************************************
  *		wglGetPixelFormat (OPENGL32.@)
  */
@@ -373,14 +968,10 @@ BOOL WINAPI wglDescribeLayerPlane(HDC hdc,
 /***********************************************************************
  *		wglGetLayerPaletteEntries (OPENGL32.@)
  */
-int WINAPI wglGetLayerPaletteEntries(HDC hdc,
-				     int iLayerPlane,
-				     int iStart,
-				     int cEntries,
-				     const COLORREF *pcr) {
-  FIXME("(): stub!\n");
-
-  return 0;
+int WINAPI wglGetLayerPaletteEntries( HDC hdc, int plane, int start, int count, COLORREF *colors )
+{
+    FIXME( "hdc %p, plane %d, start %d, count %d, colors %p, stub!\n", hdc, plane, start, count, colors );
+    return 0;
 }
 
 /***********************************************************************
@@ -438,8 +1029,8 @@ PROC WINAPI wglGetDefaultProcAddress( LPCSTR name )
 /***********************************************************************
  *		wglSwapLayerBuffers (OPENGL32.@)
  */
-BOOL WINAPI wglSwapLayerBuffers(HDC hdc,
-				UINT fuPlanes) {
+BOOL WINAPI DECLSPEC_HOTPATCH wglSwapLayerBuffers(HDC hdc, UINT fuPlanes)
+{
   TRACE("(%p, %08x)\n", hdc, fuPlanes);
 
   if (fuPlanes & WGL_SWAP_MAIN_PLANE) {
@@ -585,7 +1176,7 @@ typedef struct _bezier_vector {
     GLdouble y;
 } bezier_vector;
 
-static double bezier_deviation_squared(const bezier_vector *p)
+static BOOL bezier_fits_deviation(const bezier_vector *p, FLOAT max_deviation)
 {
     bezier_vector deviation;
     bezier_vector vertex;
@@ -593,25 +1184,36 @@ static double bezier_deviation_squared(const bezier_vector *p)
     double base_length;
     double dot;
 
+    max_deviation *= max_deviation;
+
     vertex.x = (p[0].x + p[1].x*2 + p[2].x)/4 - p[0].x;
     vertex.y = (p[0].y + p[1].y*2 + p[2].y)/4 - p[0].y;
 
     base.x = p[2].x - p[0].x;
     base.y = p[2].y - p[0].y;
 
-    base_length = sqrt(base.x*base.x + base.y*base.y);
-    base.x /= base_length;
-    base.y /= base_length;
+    base_length = base.x * base.x + base.y * base.y;
+    if (base_length <= max_deviation)
+    {
+        base.x = 0.0;
+        base.y = 0.0;
+    }
+    else
+    {
+        base_length = sqrt(base_length);
+        base.x /= base_length;
+        base.y /= base_length;
 
-    dot = base.x*vertex.x + base.y*vertex.y;
-    dot = min(max(dot, 0.0), base_length);
-    base.x *= dot;
-    base.y *= dot;
+        dot = base.x*vertex.x + base.y*vertex.y;
+        dot = min(max(dot, 0.0), base_length);
+        base.x *= dot;
+        base.y *= dot;
+    }
 
     deviation.x = vertex.x-base.x;
     deviation.y = vertex.y-base.y;
 
-    return deviation.x*deviation.x + deviation.y*deviation.y;
+    return deviation.x*deviation.x + deviation.y*deviation.y <= max_deviation;
 }
 
 static int bezier_approximate(const bezier_vector *p, bezier_vector *points, FLOAT deviation)
@@ -621,7 +1223,7 @@ static int bezier_approximate(const bezier_vector *p, bezier_vector *points, FLO
     bezier_vector vertex;
     int total_vertices;
 
-    if(bezier_deviation_squared(p) <= deviation*deviation)
+    if (bezier_fits_deviation(p, deviation))
     {
         if(points)
             *points = p[2];
@@ -1285,11 +1887,16 @@ GLboolean WINAPI glUnmapNamedBufferEXT( GLuint buffer )
     return gl_unmap_named_buffer( unix_glUnmapNamedBufferEXT, buffer );
 }
 
-static BOOL WINAPI call_opengl_debug_message_callback( struct wine_gl_debug_message_params *params, ULONG size )
+typedef void (WINAPI *gl_debug_message)(GLenum, GLenum, GLuint, GLenum, GLsizei, const GLchar *, const void *);
+
+static NTSTATUS WINAPI call_gl_debug_message_callback( void *args, ULONG size )
 {
-    params->user_callback( params->source, params->type, params->id, params->severity,
-                           params->length, params->message, params->user_data );
-    return TRUE;
+    struct gl_debug_message_callback_params *params = args;
+    gl_debug_message callback = (void *)(UINT_PTR)params->debug_callback;
+    const void *user = (void *)(UINT_PTR)params->debug_user;
+    callback( params->source, params->type, params->id, params->severity,
+              params->length, params->message, user );
+    return STATUS_SUCCESS;
 }
 
 /***********************************************************************
@@ -1297,20 +1904,22 @@ static BOOL WINAPI call_opengl_debug_message_callback( struct wine_gl_debug_mess
  */
 BOOL WINAPI DllMain( HINSTANCE hinst, DWORD reason, LPVOID reserved )
 {
-    void **kernel_callback_table;
+    struct process_attach_params params =
+    {
+        .call_gl_debug_message_callback = (UINT_PTR)call_gl_debug_message_callback,
+    };
     NTSTATUS status;
 
     switch(reason)
     {
     case DLL_PROCESS_ATTACH:
-        if ((status = __wine_init_unix_call()))
+        if ((status = __wine_init_unix_call()) ||
+            (status = UNIX_CALL( process_attach, &params )))
         {
             ERR( "Failed to load unixlib, status %#lx\n", status );
             return FALSE;
         }
 
-        kernel_callback_table = NtCurrentTeb()->Peb->KernelCallbackTable;
-        kernel_callback_table[NtUserCallOpenGLDebugMessageCallback] = call_opengl_debug_message_callback;
         /* fallthrough */
     case DLL_THREAD_ATTACH:
         if ((status = UNIX_CALL( thread_attach, NtCurrentTeb() )))
@@ -1326,6 +1935,9 @@ BOOL WINAPI DllMain( HINSTANCE hinst, DWORD reason, LPVOID reserved )
 #ifndef _WIN64
         cleanup_wow64_strings();
 #endif
+        /* fallthrough */
+    case DLL_THREAD_DETACH:
+        free( NtCurrentTeb()->glReserved1[WINE_GL_RESERVED_FORMATS_PTR] );
         return TRUE;
     }
     return TRUE;
