@@ -38,6 +38,27 @@
 #include <oleauto.h>
 #include <limits.h>
 
+#define DEFINE_EXPECT(func) \
+    static UINT called_count_ ## func = 0
+
+#define INCREASE_CALL_COUNTER(func) \
+    do { \
+        called_count_ ## func++;     \
+    }while(0)
+
+#define CHECK_CALLED(func) \
+    do { \
+        ok(called_count_ ## func, "expected " #func "\n"); \
+    }while(0)
+
+#define CHECK_NOT_CALLED(func) \
+    do { \
+        ok(!called_count_ ## func, "unexpected " #func "\n"); \
+    }while(0)
+
+#define CLEAR_COUNTER(func) \
+    called_count_ ## func = 0
+
 static HMODULE hmoduleRichEdit;
 static IID *pIID_ITextServices;
 static IID *pIID_ITextHost;
@@ -81,6 +102,10 @@ static ITextServicesVtbl itextServicesStdcallVtbl;
 /************************************************************************/
 /* ITextHost implementation for conformance testing. */
 
+DEFINE_EXPECT(ITextHostImpl_TxViewChange);
+DEFINE_EXPECT(ITextHostImpl_TxScrollWindowEx);
+DEFINE_EXPECT(ITextHostImpl_TxGetClientRect);
+
 typedef struct ITextHostTestImpl
 {
     ITextHost ITextHost_iface;
@@ -198,6 +223,7 @@ static void __thiscall ITextHostImpl_TxViewChange(ITextHost *iface, BOOL fUpdate
     ITextHostTestImpl *This = impl_from_ITextHost(iface);
     TRACECALL("Call to TxViewChange(%p, fUpdate=%d)\n",
                This, fUpdate);
+    INCREASE_CALL_COUNTER(ITextHostImpl_TxViewChange);
 }
 
 static BOOL __thiscall ITextHostImpl_TxCreateCaret(ITextHost *iface, HBITMAP hbmp, INT xWidth, INT yHeight)
@@ -244,6 +270,7 @@ static void __thiscall ITextHostImpl_TxScrollWindowEx(ITextHost *iface, INT dx,
     ITextHostTestImpl *This = impl_from_ITextHost(iface);
     TRACECALL("Call to TxScrollWindowEx(%p, %d, %d, %p, %p, %p, %p, %d)\n",
               This, dx, dy, lprcScroll, lprcClip, hRgnUpdate, lprcUpdate, fuScroll);
+    INCREASE_CALL_COUNTER(ITextHostImpl_TxScrollWindowEx);
 }
 
 static void __thiscall ITextHostImpl_TxSetCapture(ITextHost *iface, BOOL fCapture)
@@ -297,6 +324,7 @@ static HRESULT __thiscall ITextHostImpl_TxGetClientRect(ITextHost *iface, LPRECT
 {
     ITextHostTestImpl *This = impl_from_ITextHost(iface);
     TRACECALL("Call to TxGetClientRect(%p, prc=%p)\n", This, prc);
+    INCREASE_CALL_COUNTER(ITextHostImpl_TxGetClientRect);
     *prc = This->client_rect;
     return S_OK;
 }
@@ -400,6 +428,8 @@ static HRESULT __thiscall ITextHostImpl_TxGetPropertyBits(ITextHost *iface, DWOR
 
 static int en_vscroll_sent;
 static int en_update_sent;
+static int en_change_sent;
+static int en_selchange_sent;
 static HRESULT __thiscall ITextHostImpl_TxNotify( ITextHost *iface, DWORD code, void *data )
 {
     ITextHostTestImpl *This = impl_from_ITextHost(iface);
@@ -414,6 +444,15 @@ static HRESULT __thiscall ITextHostImpl_TxNotify( ITextHost *iface, DWORD code,
         en_update_sent++;
         ok( !data, "got %p\n", data );
         break;
+    case EN_CHANGE:
+        en_change_sent++;
+        todo_wine
+        ok( data != NULL, "got %p\n", data );
+        break;
+    case EN_SELCHANGE:
+        en_selchange_sent++;
+        ok( data != NULL, "got %p\n", data );
+        break;
     }
     return S_OK;
 }
@@ -620,7 +659,7 @@ static BOOL init_texthost(ITextServices **txtserv, ITextHost **ret)
     memset(&dummyTextHost->char_format, 0, sizeof(dummyTextHost->char_format));
     dummyTextHost->char_format.cbSize = sizeof(dummyTextHost->char_format);
     dummyTextHost->char_format.dwMask = CFM_ALL2;
-    dummyTextHost->scrollbars = 0;
+    dummyTextHost->scrollbars = ES_AUTOVSCROLL;
     dummyTextHost->props = 0;
     hf = GetStockObject(DEFAULT_GUI_FONT);
     hf_to_cf(hf, &dummyTextHost->char_format);
@@ -753,7 +792,7 @@ static void _check_txgetnaturalsize(HRESULT res, LONG width, LONG height, HDC hd
     RECT expected_rect = rect;
     LONG expected_width, expected_height;
 
-    DrawTextW(hdc, string, -1, &expected_rect, DT_LEFT | DT_CALCRECT | DT_NOCLIP | DT_EDITCONTROL | DT_WORDBREAK);
+    DrawTextW(hdc, string, -1, &expected_rect, DT_LEFT | DT_CALCRECT | DT_NOCLIP | DT_EDITCONTROL);
     expected_width = expected_rect.right - expected_rect.left;
     expected_height = expected_rect.bottom - expected_rect.top;
     ok_(__FILE__,line)(res == S_OK, "ITextServices_TxGetNaturalSize failed: 0x%08lx.\n", res);
@@ -815,15 +854,20 @@ static void test_TxGetNaturalSize(void)
 static void test_TxDraw(void)
 {
     ITextServices *txtserv;
+    ITextDocument *txtdoc;
     ITextHost *host;
     HRESULT hr;
     RECT client = {0, 0, 100, 100};
     ITextHostTestImpl *host_impl;
+    LONG freeze_count;
     HDC hdc;
 
     if (!init_texthost(&txtserv, &host))
         return;
 
+    hr = ITextServices_QueryInterface( txtserv, &IID_ITextDocument, (void **)&txtdoc );
+    ok( hr == S_OK, "ITextServices_QueryInterface (ITextDocument) returned %#lx\n", hr );
+
     host_impl = impl_from_ITextHost( host );
     host_impl->window = CreateWindowExA( 0, "static", NULL, WS_POPUP | WS_VISIBLE,
                                          0, 0, 400, 400, 0, 0, 0, NULL );
@@ -856,9 +900,27 @@ static void test_TxDraw(void)
                                NULL, NULL, 0, TXTVIEW_INACTIVE );
     ok( hr == S_OK, "got %08lx\n", hr );
 
+    freeze_count = 0xdeadbeef;
+    hr = ITextDocument_Freeze( txtdoc, &freeze_count );
+    ok( hr == S_OK, "ITextDocument_Freeze returned %#lx\n", hr );
+    ok( freeze_count == 1, "expected count to be %d, got %ld\n", 1, freeze_count );
+
+    hr = ITextServices_TxDraw( txtserv, DVASPECT_CONTENT, 0, NULL, NULL, hdc, NULL, (RECTL *)&client, NULL,
+                               NULL, NULL, 0, TXTVIEW_INACTIVE );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    hr = ITextServices_TxDraw( txtserv, DVASPECT_CONTENT, 0, NULL, NULL, hdc, NULL, (RECTL *)&client, NULL,
+                               NULL, NULL, 0, TXTVIEW_ACTIVE );
+    ok( hr == E_UNEXPECTED, "got %08lx\n", hr );
+
+    freeze_count = 0xdeadbeef;
+    hr = ITextDocument_Unfreeze( txtdoc, &freeze_count );
+    ok( hr == S_OK, "ITextDocument_Unfreeze returned %#lx\n", hr );
+    ok( freeze_count == 0, "expected count to be %d, got %ld\n", 0, freeze_count );
+
     hr = ITextServices_OnTxInPlaceDeactivate( txtserv );
 
     ReleaseDC( host_impl->window, hdc );
+    ITextDocument_Release(txtdoc);
     ITextServices_Release(txtserv);
     DestroyWindow( host_impl->window );
     ITextHost_Release(host);
@@ -1217,6 +1279,181 @@ static void test_notifications( void )
     ITextHost_Release( host );
 }
 
+static void test_set_selection_message( void )
+{
+    ITextServices *txtserv;
+    ITextHost *host;
+    CHARRANGE range;
+    LRESULT result;
+    HRESULT hr;
+
+    if (!init_texthost(&txtserv, &host))
+        return;
+
+    ITextServices_TxSetText(txtserv, L"");
+
+    if (winetest_debug > 1) trace("TxSendMessage EM_SETEVENTMASK");
+
+    hr = ITextServices_TxSendMessage( txtserv, EM_SETEVENTMASK, 0, ENM_CHANGE | ENM_SELCHANGE, &result );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    if (winetest_debug > 1) trace("TxSendMessage EM_SETSEL 0 20");
+
+    CLEAR_COUNTER(ITextHostImpl_TxViewChange);
+    en_change_sent = 0;
+    en_selchange_sent = 0;
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_SETSEL, 0, 20, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    CHECK_CALLED(ITextHostImpl_TxViewChange);
+    ok(en_change_sent == 0, "got %d\n", en_change_sent);
+    todo_wine
+    ok(en_selchange_sent == 0, "got %d\n", en_selchange_sent);
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXGETSEL, 0, (LPARAM)&range, &result); ok( hr == S_OK, "got %08lx\n", hr ); trace("getsel: %lu, %lu\n", range.cpMin, range.cpMax);
+
+    if (winetest_debug > 1) trace("TxSendMessage EM_SETSELEX 0 20");
+
+    CLEAR_COUNTER(ITextHostImpl_TxViewChange);
+    en_change_sent = 0;
+    en_selchange_sent = 0;
+
+    range.cpMin = 0;
+    range.cpMax = 20;
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXSETSEL, 0, (LPARAM)&range, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    CHECK_CALLED(ITextHostImpl_TxViewChange);
+    ok(en_change_sent == 0, "got %d\n", en_change_sent);
+    ok(en_selchange_sent == 0, "got %d\n", en_selchange_sent);
+
+    if (winetest_debug > 1) trace("TxSetText");
+
+    CLEAR_COUNTER(ITextHostImpl_TxViewChange);
+    en_change_sent = 0;
+    en_selchange_sent = 0;
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXGETSEL, 0, (LPARAM)&range, &result); ok( hr == S_OK, "got %08lx\n", hr ); trace("getsel: %lu, %lu\n", range.cpMin, range.cpMax);
+
+    ITextServices_TxSetText(txtserv, lorem);
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXGETSEL, 0, (LPARAM)&range, &result); ok( hr == S_OK, "got %08lx\n", hr ); trace("getsel: %lu, %lu\n", range.cpMin, range.cpMax);
+
+    CHECK_CALLED(ITextHostImpl_TxViewChange);
+    ok(en_change_sent == 1, "got %d\n", en_change_sent);
+    todo_wine
+    ok(en_selchange_sent == 0, "got %d\n", en_selchange_sent);
+
+    if (winetest_debug > 1) trace("TxSendMessage EM_SETSEL 0 20");
+
+    CLEAR_COUNTER(ITextHostImpl_TxViewChange);
+    en_change_sent = 0;
+    en_selchange_sent = 0;
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_SETSEL, 0, 20, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    CHECK_CALLED(ITextHostImpl_TxViewChange);
+    ok(en_change_sent == 0, "got %d\n", en_change_sent);
+    ok(en_selchange_sent == 1, "got %d\n", en_selchange_sent);
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXGETSEL, 0, (LPARAM)&range, &result); ok( hr == S_OK, "got %08lx\n", hr ); trace("getsel: %lu, %lu\n", range.cpMin, range.cpMax);
+
+    if (winetest_debug > 1) trace("TxSendMessage EM_SETSELEX 0 20");
+
+    CLEAR_COUNTER(ITextHostImpl_TxViewChange);
+    en_change_sent = 0;
+    en_selchange_sent = 0;
+
+    range.cpMin = 0;
+    range.cpMax = 20;
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXSETSEL, 0, (LPARAM)&range, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    CHECK_CALLED(ITextHostImpl_TxViewChange);
+    ok(en_change_sent == 0, "got %d\n", en_change_sent);
+    ok(en_selchange_sent == 0, "got %d\n", en_selchange_sent);
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_EXGETSEL, 0, (LPARAM)&range, &result); ok( hr == S_OK, "got %08lx\n", hr ); trace("getsel: %lu, %lu\n", range.cpMin, range.cpMax);
+
+    if (winetest_debug > 1) trace("release interfaces");
+
+    ITextServices_Release( txtserv );
+    ITextHost_Release( host );
+}
+
+static void test_scrollcaret( void )
+{
+    ITextServices *txtserv;
+    ITextHost *host;
+    LRESULT result;
+    HRESULT hr;
+
+    if (!init_texthost(&txtserv, &host))
+        return;
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_SCROLLCARET, 0, 0, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    ITextServices_Release( txtserv );
+    ITextHost_Release( host );
+}
+
+static void test_inplace_active( BOOL active )
+{
+    ITextServices *txtserv;
+    ITextHost *host;
+    LRESULT result;
+    HRESULT hr;
+
+    ITextHostTestImpl *host_impl;
+    if (!init_texthost(&txtserv, &host))
+        return;
+
+    host_impl = impl_from_ITextHost( host );
+    host_impl->window = CreateWindowExA( 0, "static", NULL, WS_POPUP|WS_VISIBLE, 0, 0, 100, 100, 0, 0, 0, NULL );
+    host_impl->client_rect = (RECT) { 0, 0, 150, 150 };
+    host_impl->scrollbars = WS_VSCROLL | ES_AUTOVSCROLL;
+    host_impl->props = TXTBIT_MULTILINE | TXTBIT_RICHTEXT | TXTBIT_WORDWRAP;
+    ITextServices_OnTxPropertyBitsChange( txtserv, TXTBIT_SCROLLBARCHANGE | TXTBIT_MULTILINE | TXTBIT_RICHTEXT | TXTBIT_WORDWRAP, host_impl->props );
+
+    CLEAR_COUNTER(ITextHostImpl_TxScrollWindowEx);
+    CLEAR_COUNTER(ITextHostImpl_TxGetClientRect);
+
+    if (active) {
+        hr = ITextServices_OnTxInPlaceActivate( txtserv, &host_impl->client_rect );
+        ok( hr == S_OK, "got %08lx\n", hr );
+    }
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_SETEVENTMASK, 0, ENM_REQUESTRESIZE, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ITextServices_TxDraw( txtserv, DVASPECT_CONTENT, 0, NULL, NULL, GetDC(host_impl->window), NULL, (RECTL *)&host_impl->client_rect, NULL, NULL, NULL, 0, TXTVIEW_INACTIVE );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_SETSEL, 0, -1, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ITextServices_TxSendMessage(txtserv, EM_REPLACESEL, TRUE, (LPARAM) lorem, &result);
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ITextServices_TxDraw( txtserv, DVASPECT_CONTENT, 0, NULL, NULL, GetDC(host_impl->window), NULL, (RECTL *)&host_impl->client_rect, NULL, NULL, NULL, 0, TXTVIEW_INACTIVE );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    if (active) {
+        CHECK_CALLED(ITextHostImpl_TxScrollWindowEx);
+        CHECK_CALLED(ITextHostImpl_TxGetClientRect);
+    } else {
+        CHECK_NOT_CALLED(ITextHostImpl_TxScrollWindowEx);
+        CHECK_NOT_CALLED(ITextHostImpl_TxGetClientRect);
+    }
+
+    DestroyWindow( host_impl->window );
+    ITextServices_Release( txtserv );
+    ITextHost_Release( host );
+}
+
 START_TEST( txtsrv )
 {
     ITextServices *txtserv;
@@ -1250,6 +1487,10 @@ START_TEST( txtsrv )
         test_default_format();
         test_TxGetScroll();
         test_notifications();
+        test_set_selection_message();
+        test_scrollcaret();
+        test_inplace_active(TRUE);
+        test_inplace_active(FALSE);
     }
     if (wrapperCodeMem) VirtualFree(wrapperCodeMem, 0, MEM_RELEASE);
 }
