@@ -32,14 +32,13 @@
 #include "rpcproxy.h"
 
 #include "wine/debug.h"
+#include "wine/asm.h"
 #include "wine/exception.h"
 
 #include "cpsf.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(ole);
 
-#define STUB_HEADER(This) (((const CInterfaceStubHeader*)((This)->lpVtbl))[-1])
-
 static LONG WINAPI stub_filter(EXCEPTION_POINTERS *eptr)
 {
     if (eptr->ExceptionRecord->ExceptionFlags & EXCEPTION_NONCONTINUABLE)
@@ -47,9 +46,21 @@ static LONG WINAPI stub_filter(EXCEPTION_POINTERS *eptr)
     return EXCEPTION_EXECUTE_HANDLER;
 }
 
+static CStdStubBuffer *impl_from_IRpcStubBuffer(IRpcStubBuffer *iface)
+{
+    return CONTAINING_RECORD(&iface->lpVtbl, CStdStubBuffer, lpVtbl);
+}
+
 static inline cstdstubbuffer_delegating_t *impl_from_delegating( IRpcStubBuffer *iface )
 {
-    return CONTAINING_RECORD(iface, cstdstubbuffer_delegating_t, stub_buffer);
+    return CONTAINING_RECORD(impl_from_IRpcStubBuffer(iface), cstdstubbuffer_delegating_t, stub_buffer);
+}
+
+static const CInterfaceStubHeader *get_stub_header(const CStdStubBuffer *stub)
+{
+    const CInterfaceStubVtbl *vtbl = CONTAINING_RECORD(stub->lpVtbl, CInterfaceStubVtbl, Vtbl);
+
+    return &vtbl->header;
 }
 
 HRESULT CStdStubBuffer_Construct(REFIID riid,
@@ -91,184 +102,13 @@ HRESULT CStdStubBuffer_Construct(REFIID riid,
   return S_OK;
 }
 
-static CRITICAL_SECTION delegating_vtbl_section;
-static CRITICAL_SECTION_DEBUG critsect_debug =
-{
-    0, 0, &delegating_vtbl_section,
-    { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": delegating_vtbl_section") }
-};
-static CRITICAL_SECTION delegating_vtbl_section = { &critsect_debug, -1, 0, 0, 0, 0 };
-
-typedef struct
-{
-    DWORD ref;
-    DWORD size;
-    IUnknownVtbl vtbl;
-    /* remaining entries in vtbl */
-} ref_counted_vtbl;
-
-static ref_counted_vtbl *current_vtbl;
-
-
-static HRESULT WINAPI delegating_QueryInterface(IUnknown *pUnk, REFIID iid, void **ppv)
-{
-    *ppv = pUnk;
-    return S_OK;
-}
-
-static ULONG WINAPI delegating_AddRef(IUnknown *pUnk)
-{
-    return 1;
-}
-
-static ULONG WINAPI delegating_Release(IUnknown *pUnk)
-{
-    return 1;
-}
-
-/* The idea here is to replace the first param on the stack
-   ie. This (which will point to cstdstubbuffer_delegating_t)
-   with This->stub_buffer.pvServerObject and then jump to the
-   relevant offset in This->stub_buffer.pvServerObject's vtbl.
-*/
-#ifdef __i386__
-
-#include "pshpack1.h"
-typedef struct {
-    BYTE mov1[4];    /* mov 0x4(%esp),%eax     8b 44 24 04 */
-    BYTE mov2[3];    /* mov 0x10(%eax),%eax    8b 40 10 */
-    BYTE mov3[4];    /* mov %eax,0x4(%esp)     89 44 24 04 */
-    BYTE mov4[2];    /* mov (%eax),%eax        8b 00 */
-    BYTE mov5[2];    /* jmp *offset(%eax)      ff a0 offset */
-    DWORD offset;
-    BYTE pad[1];     /* nop                    90 */
-} vtbl_method_t;
-#include "poppack.h"
-
-static const BYTE opcodes[20] = { 0x8b, 0x44, 0x24, 0x04, 0x8b, 0x40, 0x10, 0x89, 0x44, 0x24, 0x04,
-                                  0x8b, 0x00, 0xff, 0xa0, 0, 0, 0, 0, 0x90 };
-
-#elif defined(__x86_64__)
-
-#include "pshpack1.h"
-typedef struct
-{
-    BYTE mov1[4];    /* movq 0x20(%rcx),%rcx   48 8b 49 20 */
-    BYTE mov2[3];    /* movq (%rcx),%rax       48 8b 01 */
-    BYTE jmp[2];     /* jmp *offset(%rax)      ff a0 offset */
-    DWORD offset;
-    BYTE pad[3];     /* lea 0x0(%rsi),%rsi     48 8d 36 */
-} vtbl_method_t;
-#include "poppack.h"
-
-static const BYTE opcodes[16] = { 0x48, 0x8b, 0x49, 0x20, 0x48, 0x8b, 0x01,
-                                  0xff, 0xa0, 0, 0, 0, 0, 0x48, 0x8d, 0x36 };
-#elif defined(__arm__)
-
-static const DWORD opcodes[] =
-{
-    0xe52d4004,    /* push {r4} */
-    0xe5900010,    /* ldr r0, [r0, #16] */
-    0xe5904000,    /* ldr r4, [r0] */
-    0xe59fc008,    /* ldr ip, [pc, #8] */
-    0xe08cc004,    /* add ip, ip, r4 */
-    0xe49d4004,    /* pop {r4} */
-    0xe59cf000     /* ldr pc, [ip] */
-};
-
-typedef struct
-{
-    DWORD opcodes[ARRAY_SIZE(opcodes)];
-    DWORD offset;
-} vtbl_method_t;
-
-#elif defined(__aarch64__)
-
-static const DWORD opcodes[] =
-{
-    0xf9401000,   /* ldr x0, [x0,#32] */
-    0xf9400010,   /* ldr x16, [x0] */
-    0x18000071,   /* ldr w17, offset */
-    0xf8716a10,   /* ldr x16, [x16,x17] */
-    0xd61f0200    /* br x16 */
-};
-
-typedef struct
-{
-    DWORD opcodes[ARRAY_SIZE(opcodes)];
-    DWORD offset;
-} vtbl_method_t;
-
-#else
-
-#warning You must implement delegated proxies/stubs for your CPU
-typedef struct
-{
-    DWORD offset;
-} vtbl_method_t;
-static const BYTE opcodes[1];
-
-#endif
-
-#define BLOCK_SIZE 1024
-#define MAX_BLOCKS 64  /* 64k methods should be enough for anybody */
-
-static const vtbl_method_t *method_blocks[MAX_BLOCKS];
-
-static const vtbl_method_t *allocate_block( unsigned int num )
-{
-    unsigned int i;
-    vtbl_method_t *prev, *block;
-    DWORD oldprot;
-
-    block = VirtualAlloc( NULL, BLOCK_SIZE * sizeof(*block),
-                          MEM_COMMIT | MEM_RESERVE, PAGE_EXECUTE_READWRITE );
-    if (!block) return NULL;
-
-    for (i = 0; i < BLOCK_SIZE; i++)
-    {
-        memcpy( &block[i], opcodes, sizeof(opcodes) );
-        block[i].offset = (BLOCK_SIZE * num + i + 3) * sizeof(void *);
-    }
-    VirtualProtect( block, BLOCK_SIZE * sizeof(*block), PAGE_EXECUTE_READ, &oldprot );
-    prev = InterlockedCompareExchangePointer( (void **)&method_blocks[num], block, NULL );
-    if (prev) /* someone beat us to it */
-    {
-        VirtualFree( block, 0, MEM_RELEASE );
-        block = prev;
-    }
-    return block;
-}
-
-static BOOL fill_delegated_stub_table(IUnknownVtbl *vtbl, DWORD num)
-{
-    const void **entry = (const void **)(vtbl + 1);
-    DWORD i, j;
-
-    if (num - 3 > BLOCK_SIZE * MAX_BLOCKS)
-    {
-        FIXME( "%lu methods not supported\n", num );
-        return FALSE;
-    }
-    vtbl->QueryInterface = delegating_QueryInterface;
-    vtbl->AddRef = delegating_AddRef;
-    vtbl->Release = delegating_Release;
-    for (i = 0; i < (num - 3 + BLOCK_SIZE - 1) / BLOCK_SIZE; i++)
-    {
-        const vtbl_method_t *block = method_blocks[i];
-        if (!block && !(block = allocate_block( i ))) return FALSE;
-        for (j = 0; j < BLOCK_SIZE && j < num - 3 - i * BLOCK_SIZE; j++) *entry++ = &block[j];
-    }
-    return TRUE;
-}
 
 BOOL fill_delegated_proxy_table(IUnknownVtbl *vtbl, DWORD num)
 {
     const void **entry = (const void **)(vtbl + 1);
-    DWORD i, j;
+    DWORD i;
 
-    if (num - 3 > BLOCK_SIZE * MAX_BLOCKS)
+    if (num > NB_THUNK_ENTRIES)
     {
         FIXME( "%lu methods not supported\n", num );
         return FALSE;
@@ -276,64 +116,19 @@ BOOL fill_delegated_proxy_table(IUnknownVtbl *vtbl, DWORD num)
     vtbl->QueryInterface = IUnknown_QueryInterface_Proxy;
     vtbl->AddRef = IUnknown_AddRef_Proxy;
     vtbl->Release = IUnknown_Release_Proxy;
-    for (i = 0; i < (num - 3 + BLOCK_SIZE - 1) / BLOCK_SIZE; i++)
-    {
-        const vtbl_method_t *block = method_blocks[i];
-        if (!block && !(block = allocate_block( i ))) return FALSE;
-        for (j = 0; j < BLOCK_SIZE && j < num - 3 - i * BLOCK_SIZE; j++, entry++)
-            if (!*entry) *entry = &block[j];
-    }
+    for (i = 0; i < num - 3; i++)
+        if (!entry[i]) entry[i] = delegating_vtbl.methods[i];
     return TRUE;
 }
 
-IUnknownVtbl *get_delegating_vtbl(DWORD num_methods)
+const IUnknownVtbl *get_delegating_vtbl(DWORD num)
 {
-    IUnknownVtbl *ret;
-
-    if (num_methods < 256) num_methods = 256;  /* avoid frequent reallocations */
-
-    EnterCriticalSection(&delegating_vtbl_section);
-
-    if(!current_vtbl || num_methods > current_vtbl->size)
+    if (num > NB_THUNK_ENTRIES)
     {
-        ref_counted_vtbl *table = malloc(FIELD_OFFSET(ref_counted_vtbl, vtbl) + num_methods * sizeof(void *));
-        if (!table)
-        {
-            LeaveCriticalSection(&delegating_vtbl_section);
-            return NULL;
-        }
-
-        table->ref = 0;
-        table->size = num_methods;
-        fill_delegated_stub_table(&table->vtbl, num_methods);
-
-        if (current_vtbl && current_vtbl->ref == 0)
-        {
-            TRACE("freeing old table\n");
-            free(current_vtbl);
-        }
-        current_vtbl = table;
+        FIXME( "%lu methods not supported\n", num );
+        return NULL;
     }
-
-    current_vtbl->ref++;
-    ret = &current_vtbl->vtbl;
-    LeaveCriticalSection(&delegating_vtbl_section);
-    return ret;
-}
-
-void release_delegating_vtbl(IUnknownVtbl *vtbl)
-{
-    ref_counted_vtbl *table = (ref_counted_vtbl*)((DWORD *)vtbl - 1);
-
-    EnterCriticalSection(&delegating_vtbl_section);
-    table->ref--;
-    TRACE("ref now %ld\n", table->ref);
-    if(table->ref == 0 && table != current_vtbl)
-    {
-        TRACE("... and we're not current so free'ing\n");
-        free(table);
-    }
-    LeaveCriticalSection(&delegating_vtbl_section);
+    return &delegating_vtbl.vtbl;
 }
 
 HRESULT CStdStubBuffer_Delegating_Construct(REFIID riid,
@@ -368,11 +163,10 @@ HRESULT CStdStubBuffer_Delegating_Construct(REFIID riid,
         return E_OUTOFMEMORY;
     }
 
-    This->base_obj = get_delegating_vtbl( vtbl->header.DispatchTableCount );
-    r = create_stub(delegating_iid, (IUnknown*)&This->base_obj, &This->base_stub);
+    This->base_obj.lpVtbl = get_delegating_vtbl( vtbl->header.DispatchTableCount );
+    r = create_stub(delegating_iid, &This->base_obj, &This->base_stub);
     if(FAILED(r))
     {
-        release_delegating_vtbl(This->base_obj);
         free(This);
         IUnknown_Release(pvServer);
         return r;
@@ -392,7 +186,7 @@ HRESULT WINAPI CStdStubBuffer_QueryInterface(LPRPCSTUBBUFFER iface,
                                             REFIID riid,
                                             LPVOID *obj)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   TRACE("(%p)->QueryInterface(%s,%p)\n",This,debugstr_guid(riid),obj);
 
   if (IsEqualIID(&IID_IUnknown, riid) ||
@@ -408,7 +202,7 @@ HRESULT WINAPI CStdStubBuffer_QueryInterface(LPRPCSTUBBUFFER iface,
 
 ULONG WINAPI CStdStubBuffer_AddRef(LPRPCSTUBBUFFER iface)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   TRACE("(%p)->AddRef()\n",This);
   return InterlockedIncrement(&This->RefCount);
 }
@@ -416,7 +210,7 @@ ULONG WINAPI CStdStubBuffer_AddRef(LPRPCSTUBBUFFER iface)
 ULONG WINAPI NdrCStdStubBuffer_Release(LPRPCSTUBBUFFER iface,
                                       LPPSFACTORYBUFFER pPSF)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   ULONG refs;
 
   TRACE("(%p)->Release()\n",This);
@@ -450,8 +244,6 @@ ULONG WINAPI NdrCStdStubBuffer2_Release(LPRPCSTUBBUFFER iface,
         IRpcStubBuffer_Disconnect((IRpcStubBuffer *)&This->stub_buffer);
 
         IRpcStubBuffer_Release(This->base_stub);
-        release_delegating_vtbl(This->base_obj);
-
         IPSFactoryBuffer_Release(pPSF);
         free(This);
     }
@@ -462,13 +254,13 @@ ULONG WINAPI NdrCStdStubBuffer2_Release(LPRPCSTUBBUFFER iface,
 HRESULT WINAPI CStdStubBuffer_Connect(LPRPCSTUBBUFFER iface,
                                      LPUNKNOWN lpUnkServer)
 {
-    CStdStubBuffer *This = (CStdStubBuffer *)iface;
+    CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
     HRESULT r;
     IUnknown *new = NULL;
 
     TRACE("(%p)->Connect(%p)\n",This,lpUnkServer);
 
-    r = IUnknown_QueryInterface(lpUnkServer, STUB_HEADER(This).piid, (void**)&new);
+    r = IUnknown_QueryInterface(lpUnkServer, get_stub_header(This)->piid, (void**)&new);
     new = InterlockedExchangePointer((void**)&This->pvServerObject, new);
     if(new)
         IUnknown_Release(new);
@@ -477,7 +269,7 @@ HRESULT WINAPI CStdStubBuffer_Connect(LPRPCSTUBBUFFER iface,
 
 void WINAPI CStdStubBuffer_Disconnect(LPRPCSTUBBUFFER iface)
 {
-    CStdStubBuffer *This = (CStdStubBuffer *)iface;
+    CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
     IUnknown *old;
     TRACE("(%p)->Disconnect()\n",This);
 
@@ -491,7 +283,8 @@ HRESULT WINAPI CStdStubBuffer_Invoke(LPRPCSTUBBUFFER iface,
                                     PRPCOLEMESSAGE pMsg,
                                     LPRPCCHANNELBUFFER pChannel)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
+  const CInterfaceStubHeader *header = get_stub_header(This);
   DWORD dwPhase = STUB_UNMARSHAL;
   HRESULT hr = S_OK;
 
@@ -499,8 +292,8 @@ HRESULT WINAPI CStdStubBuffer_Invoke(LPRPCSTUBBUFFER iface,
 
   __TRY
   {
-    if (STUB_HEADER(This).pDispatchTable)
-      STUB_HEADER(This).pDispatchTable[pMsg->iMethod](iface, pChannel, (PRPC_MESSAGE)pMsg, &dwPhase);
+    if (header->pDispatchTable)
+      header->pDispatchTable[pMsg->iMethod](iface, pChannel, (PRPC_MESSAGE)pMsg, &dwPhase);
     else /* pure interpreted */
       NdrStubCall2(iface, pChannel, (PRPC_MESSAGE)pMsg, &dwPhase);
   }
@@ -521,14 +314,18 @@ HRESULT WINAPI CStdStubBuffer_Invoke(LPRPCSTUBBUFFER iface,
 LPRPCSTUBBUFFER WINAPI CStdStubBuffer_IsIIDSupported(LPRPCSTUBBUFFER iface,
                                                     REFIID riid)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
-  TRACE("(%p)->IsIIDSupported(%s)\n",This,debugstr_guid(riid));
-  return IsEqualGUID(STUB_HEADER(This).piid, riid) ? iface : NULL;
+    CStdStubBuffer *stub = impl_from_IRpcStubBuffer(iface);
+
+    TRACE("(%p)->IsIIDSupported(%s)\n", stub, debugstr_guid(riid));
+
+    if (IsEqualGUID(get_stub_header(stub)->piid, riid))
+        return iface;
+    return NULL;
 }
 
 ULONG WINAPI CStdStubBuffer_CountRefs(LPRPCSTUBBUFFER iface)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   TRACE("(%p)->CountRefs()\n",This);
   return This->RefCount;
 }
@@ -536,7 +333,7 @@ ULONG WINAPI CStdStubBuffer_CountRefs(LPRPCSTUBBUFFER iface)
 HRESULT WINAPI CStdStubBuffer_DebugServerQueryInterface(LPRPCSTUBBUFFER iface,
                                                        LPVOID *ppv)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   TRACE("(%p)->DebugServerQueryInterface(%p)\n",This,ppv);
   return S_OK;
 }
@@ -544,7 +341,7 @@ HRESULT WINAPI CStdStubBuffer_DebugServerQueryInterface(LPRPCSTUBBUFFER iface,
 void WINAPI CStdStubBuffer_DebugServerRelease(LPRPCSTUBBUFFER iface,
                                              LPVOID pv)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   TRACE("(%p)->DebugServerRelease(%p)\n",This,pv);
 }
 
@@ -613,8 +410,9 @@ const IRpcStubBufferVtbl CStdStubBuffer_Delegating_Vtbl =
 
 const MIDL_SERVER_INFO *CStdStubBuffer_GetServerInfo(IRpcStubBuffer *iface)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
-  return STUB_HEADER(This).pServerInfo;
+    CStdStubBuffer *stub = impl_from_IRpcStubBuffer(iface);
+
+    return get_stub_header(stub)->pServerInfo;
 }
 
 /************************************************************************
@@ -654,14 +452,14 @@ void WINAPI NdrStubGetBuffer(LPRPCSTUBBUFFER iface,
                             LPRPCCHANNELBUFFER pRpcChannelBuffer,
                             PMIDL_STUB_MESSAGE pStubMsg)
 {
-  CStdStubBuffer *This = (CStdStubBuffer *)iface;
+  CStdStubBuffer *This = impl_from_IRpcStubBuffer(iface);
   HRESULT hr;
 
   TRACE("(%p, %p, %p)\n", This, pRpcChannelBuffer, pStubMsg);
 
   pStubMsg->RpcMsg->BufferLength = pStubMsg->BufferLength;
   hr = IRpcChannelBuffer_GetBuffer(pRpcChannelBuffer,
-    (RPCOLEMESSAGE *)pStubMsg->RpcMsg, STUB_HEADER(This).piid);
+    (RPCOLEMESSAGE *)pStubMsg->RpcMsg, get_stub_header(This)->piid);
   if (FAILED(hr))
   {
     RpcRaiseException(hr);
