@@ -26,6 +26,8 @@
 
 #include "wine/test.h"
 
+DEFINE_GUID(GUID_NULL,0,0,0,0,0,0,0,0,0,0,0);
+
 static void test_data_key(void)
 {
     ISpRegDataKey *data_key;
@@ -203,6 +205,16 @@ static void test_token_category(void)
 
     IEnumSpObjectTokens_Release( enum_tokens );
 
+    hr = ISpObjectTokenCategory_EnumTokens( cat, L"", NULL, &enum_tokens );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    count = 0xdeadbeef;
+    hr = IEnumSpObjectTokens_GetCount( enum_tokens, &count );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( count == 5, "got %lu\n", count );
+
+    IEnumSpObjectTokens_Release( enum_tokens );
+
     hr = ISpObjectTokenCategory_EnumTokens( cat, L"Language=409", NULL, &enum_tokens );
     ok( hr == S_OK, "got %08lx\n", hr );
 
@@ -246,12 +258,46 @@ static void test_token_enum(void)
 {
     ISpObjectTokenEnumBuilder *token_enum;
     HRESULT hr;
+    IDispatch *disp;
+    ISpeechObjectTokens *speech_tokens;
+    IUnknown *unk;
+    IEnumVARIANT *enumvar;
     ISpObjectToken *tokens[5];
     ISpObjectToken *out_tokens[5];
     WCHAR token_id[MAX_PATH];
     ULONG count;
+    VARIANT vars[3], ret;
+    DISPPARAMS params;
     int i;
 
+    hr = CoCreateInstance( &CLSID_SpObjectTokenEnum, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_ISpObjectTokenEnumBuilder, (void **)&token_enum );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ISpObjectTokenEnumBuilder_QueryInterface( token_enum,
+                                                   &IID_IDispatch, (void **)&disp );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    IDispatch_Release( disp );
+
+    hr = ISpObjectTokenEnumBuilder_QueryInterface( token_enum,
+                                                   &IID_ISpeechObjectTokens,
+                                                   (void **)&speech_tokens );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ISpeechObjectTokens_Release( speech_tokens );
+
+    ISpObjectTokenEnumBuilder_Release( token_enum );
+
+    hr = CoCreateInstance( &CLSID_SpObjectTokenEnum, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_IDispatch, (void **)&disp );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    IDispatch_Release( disp );
+
+    hr = CoCreateInstance( &CLSID_SpObjectTokenEnum, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_ISpeechObjectTokens, (void **)&speech_tokens );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ISpeechObjectTokens_Release( speech_tokens );
+
+
     hr = CoCreateInstance( &CLSID_SpObjectTokenEnum, NULL, CLSCTX_INPROC_SERVER,
                            &IID_ISpObjectTokenEnumBuilder, (void **)&token_enum );
     ok( hr == S_OK, "got %08lx\n", hr );
@@ -318,6 +364,70 @@ static void test_token_enum(void)
                            &IID_ISpObjectTokenEnumBuilder, (void **)&token_enum );
     ok( hr == S_OK, "got %08lx\n", hr );
 
+    hr = ISpObjectTokenEnumBuilder_QueryInterface( token_enum,
+                                                   &IID_ISpeechObjectTokens,
+                                                   (void **)&speech_tokens );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ISpObjectTokenEnumBuilder_SetAttribs( token_enum, NULL, NULL );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    hr = ISpObjectTokenEnumBuilder_AddTokens( token_enum, 3, tokens );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    count = 0xdeadbeef;
+    hr = ISpeechObjectTokens_get_Count( speech_tokens, (LONG *)&count );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( count == 3, "got %lu\n", count );
+
+    hr = ISpeechObjectTokens_get__NewEnum( speech_tokens, &unk );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    hr = IUnknown_QueryInterface( unk, &IID_IEnumVARIANT, (void **)&enumvar );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    IUnknown_Release( unk );
+
+    V_VT( &vars[0] ) = VT_ILLEGAL;
+    V_DISPATCH( &vars[0] ) = (IDispatch *)0xdeadbeef;
+    hr = IEnumVARIANT_Next( enumvar, 1, vars, NULL );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( V_VT( &vars[0] ) == VT_DISPATCH, "got %#x\n", V_VT( &vars[0] ) );
+    ok( V_DISPATCH( &vars[0] ) != (IDispatch *)0xdeadbeef && V_DISPATCH( &vars[0] ) != NULL,
+        "got %p\n", V_DISPATCH( &vars[0] ) );
+    VariantClear( &vars[0] );
+
+    for ( i = 0; i < 3; i++ ) {
+        V_VT( &vars[i] ) = VT_ILLEGAL;
+        V_DISPATCH( &vars[i] ) = (IDispatch *)0xdeadbeef;
+    }
+    count = 0xdeadbeef;
+
+    hr = IEnumVARIANT_Next( enumvar, 3, vars, &count );
+    ok( hr == S_FALSE, "got %08lx\n", hr );
+    ok( count == 2, "got %lu\n", count );
+    for ( i = 0; i < 2; i++ ) {
+        ok( V_VT( &vars[i] ) == VT_DISPATCH, "got %#x\n", V_VT( &vars[i] ) );
+        ok( V_DISPATCH( &vars[i] ) != (IDispatch *)0xdeadbeef && V_DISPATCH( &vars[i] ) != NULL,
+            "got %p\n", V_DISPATCH( &vars[i] ) );
+        VariantClear( &vars[i] );
+    }
+    ok( V_VT( &vars[2] ) == VT_ILLEGAL, "got %#x\n", V_VT( &vars[2] ) );
+
+    IEnumVARIANT_Release( enumvar );
+
+    memset( &params, 0, sizeof(params) );
+    VariantInit( &ret );
+    hr = ISpeechObjectTokens_Invoke( speech_tokens, DISPID_SOTsCount, &IID_NULL, 0,
+                                     DISPATCH_PROPERTYGET, &params, &ret, NULL, NULL );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( V_VT( &ret ) == VT_I4, "got %#x\n", V_VT( &ret ) );
+    ok( V_I4( &ret ) == 3, "got %ld\n", V_I4( &ret ) );
+
+    ISpeechObjectTokens_Release( speech_tokens );
+    ISpObjectTokenEnumBuilder_Release( token_enum );
+
+    hr = CoCreateInstance( &CLSID_SpObjectTokenEnum, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_ISpObjectTokenEnumBuilder, (void **)&token_enum );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
     /* Vendor attribute must exist */
     hr = ISpObjectTokenEnumBuilder_SetAttribs( token_enum, L"Vendor", NULL );
     ok( hr == S_OK, "got %08lx\n", hr );
@@ -601,15 +711,47 @@ static IClassFactory test_class_cf = { &ClassFactoryVtbl };
 
 static void test_object_token(void)
 {
-    static const WCHAR test_token_id[] = L"HKEY_LOCAL_MACHINE\\Software\\Wine\\Winetest\\sapi\\TestToken";
+    static const WCHAR test_token_id[] = L"HKEY_LOCAL_MACHINE\\Software\\Winetest\\sapi\\TestToken";
+    static const WCHAR *get_description = L"GetDescription";
 
     ISpObjectToken *token;
+    IDispatch *disp;
+    ISpeechObjectToken *speech_token;
     ISpDataKey *sub_key;
     HRESULT hr;
     LPWSTR tempW, token_id;
+    BSTR tempB;
     ISpObjectTokenCategory *cat;
     DWORD regid;
     IUnknown *obj;
+    DISPID dispid;
+    DISPPARAMS params;
+    VARIANT arg, ret;
+
+    hr = CoCreateInstance( &CLSID_SpObjectToken, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_ISpObjectToken, (void **)&token );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ISpObjectToken_QueryInterface( token, &IID_IDispatch, (void **)&disp );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    IDispatch_Release( disp );
+
+    hr = ISpObjectToken_QueryInterface( token, &IID_ISpeechObjectToken, (void **)&speech_token );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ISpeechObjectToken_Release( speech_token );
+
+    ISpObjectToken_Release( token );
+
+    hr = CoCreateInstance( &CLSID_SpObjectToken, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_IDispatch, (void **)&disp );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    IDispatch_Release( disp );
+
+    hr = CoCreateInstance( &CLSID_SpObjectToken, NULL, CLSCTX_INPROC_SERVER,
+                           &IID_ISpeechObjectToken, (void **)&speech_token );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ISpeechObjectToken_Release( speech_token );
+
 
     hr = CoCreateInstance( &CLSID_SpObjectToken, NULL, CLSCTX_INPROC_SERVER,
                            &IID_ISpObjectToken, (void **)&token );
@@ -780,11 +922,85 @@ static void test_object_token(void)
         CoTaskMemFree( tempW );
     }
 
+    hr = ISpObjectToken_SetStringValue( token, L"409", L"409 - TestToken" );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    hr = ISpObjectToken_SetStringValue( token, L"407", L"407 - TestToken" );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    hr = ISpObjectToken_SetStringValue( token, L"E40C", L"E40C - TestToken" );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ISpObjectToken_QueryInterface( token, &IID_ISpeechObjectToken, (void **)&speech_token );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0x409, NULL );
+    ok( hr == E_POINTER, "got %08lx\n", hr );
+
+    tempB = NULL;
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0x409, &tempB );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( tempB && !wcscmp( tempB, L"409 - TestToken" ), "got %s\n", wine_dbgstr_w( tempB ) );
+    SysFreeString( tempB );
+
+    tempB = NULL;
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0x10407, &tempB );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( tempB && !wcscmp( tempB, L"407 - TestToken" ), "got %s\n", wine_dbgstr_w( tempB ) );
+    SysFreeString( tempB );
+
+    tempB = NULL;
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0xE40C, &tempB );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( tempB && !wcscmp( tempB, L"E40C - TestToken" ), "got %s\n", wine_dbgstr_w( tempB ) );
+    SysFreeString( tempB );
+
+    tempB = (BSTR)0xdeadbeef;
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0x406, &tempB );
+    ok( hr == SPERR_NOT_FOUND, "got %08lx\n", hr );
+    ok( tempB == (BSTR)0xdeadbeef || broken(tempB == NULL) /* < win7 */, "got %p\n", tempB );
+
+    hr = ISpObjectToken_SetStringValue( token, NULL, L"TestToken" );
+    ok( hr == S_OK, "got %08lx\n", hr );
+
+    tempB = NULL;
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0x406, &tempB );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( tempB && !wcscmp( tempB, L"TestToken" ), "got %s\n", wine_dbgstr_w( tempB ) );
+    SysFreeString( tempB );
+
+    tempB = NULL;
+    hr = ISpeechObjectToken_GetDescription( speech_token, 0x0, &tempB );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( tempB && !wcscmp( tempB, L"TestToken" ), "got %s\n", wine_dbgstr_w( tempB ) );
+    SysFreeString( tempB );
+
+    dispid = 0xdeadbeef;
+    hr = ISpeechObjectToken_GetIDsOfNames( speech_token, &IID_NULL, (WCHAR **)&get_description, 1, 0x409, &dispid );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( dispid == DISPID_SOTGetDescription, "got %08lx\n", dispid );
+
+    memset( &params, 0, sizeof(params) );
+    params.cArgs = 1;
+    params.cNamedArgs = 0;
+    params.rgvarg = &arg;
+    VariantInit( &arg );
+    V_VT( &arg ) = VT_I4;
+    V_I4( &arg ) = 0x409;
+    VariantInit( &ret );
+    hr = ISpeechObjectToken_Invoke( speech_token, DISPID_SOTGetDescription, &IID_NULL,
+                                    0, DISPATCH_METHOD, &params, &ret, NULL, NULL );
+    ok( hr == S_OK, "got %08lx\n", hr );
+    ok( V_VT( &ret ) == VT_BSTR, "got %#x\n", V_VT( &ret ) );
+    ok( V_BSTR( &ret ) && !wcscmp( V_BSTR( &ret ), L"409 - TestToken" ),
+        "got %s\n", wine_dbgstr_w( V_BSTR( &ret ) ) );
+    VariantClear( &ret );
+
+    ISpeechObjectToken_Release( speech_token );
+
     ISpObjectToken_Release( test_class_token );
     IUnknown_Release( obj );
     ISpObjectToken_Release( token );
 
-    RegDeleteTreeA( HKEY_LOCAL_MACHINE, "Software\\Wine\\Winetest\\sapi" );
+    RegDeleteTreeA( HKEY_LOCAL_MACHINE, "Software\\Winetest\\sapi" );
 }
 
 START_TEST(token)
