@@ -32,6 +32,7 @@
 #include "ntsecapi.h"
 #include "ntsecpkg.h"
 #include "winternl.h"
+#include "ddk/ntddk.h"
 #include "rpc.h"
 
 #include "wine/debug.h"
@@ -71,6 +72,24 @@ static const char *debugstr_as(const LSA_STRING *str)
     return debugstr_an(str->Buffer, str->Length);
 }
 
+SECPKG_FUNCTION_TABLE *lsa_find_package(const char *name, SECPKG_USER_FUNCTION_TABLE **user_api)
+{
+    LSA_STRING package_name;
+    ULONG i;
+
+    RtlInitString(&package_name, name);
+
+    for (i = 0; i < loaded_packages_count; i++)
+    {
+        if (!RtlCompareString(loaded_packages[i].name, &package_name, FALSE))
+        {
+            *user_api = loaded_packages[i].user_api;
+            return loaded_packages[i].lsa_api;
+        }
+    }
+    return NULL;
+}
+
 NTSTATUS WINAPI LsaCallAuthenticationPackage(HANDLE lsa_handle, ULONG package_id,
         PVOID in_buffer, ULONG in_buffer_length,
         PVOID *out_buffer, PULONG out_buffer_length, PNTSTATUS status)
@@ -80,6 +99,10 @@ NTSTATUS WINAPI LsaCallAuthenticationPackage(HANDLE lsa_handle, ULONG package_id
     TRACE("%p,%lu,%p,%lu,%p,%p,%p\n", lsa_handle, package_id, in_buffer,
         in_buffer_length, out_buffer, out_buffer_length, status);
 
+    if (out_buffer) *out_buffer = NULL;
+    if (out_buffer_length) *out_buffer_length = 0;
+    if (status) *status = STATUS_SUCCESS;
+
     for (i = 0; i < loaded_packages_count; i++)
     {
         if (loaded_packages[i].package_id == package_id)
@@ -92,7 +115,7 @@ NTSTATUS WINAPI LsaCallAuthenticationPackage(HANDLE lsa_handle, ULONG package_id
         }
     }
 
-    return STATUS_INVALID_PARAMETER;
+    return STATUS_NO_SUCH_PACKAGE;
 }
 
 static struct lsa_handle *alloc_lsa_handle(ULONG magic)
@@ -133,7 +156,8 @@ NTSTATUS WINAPI LsaDeregisterLogonProcess(HANDLE LsaHandle)
     TRACE("%p\n", LsaHandle);
 
     if (!lsa_conn || lsa_conn->magic != LSA_MAGIC_CONNECTION) return STATUS_INVALID_HANDLE;
-    lsa_conn->magic = 0;
+    /* Ensure compiler doesn't optimize out the assignment with 0. */
+    SecureZeroMemory(&lsa_conn->magic, sizeof(lsa_conn->magic));
     free(lsa_conn);
 
     return STATUS_SUCCESS;
@@ -442,7 +466,8 @@ static SECURITY_STATUS WINAPI lsa_FreeCredentialsHandle(CredHandle *credential)
 
     status = lsa_cred->package->lsa_api->FreeCredentialsHandle(lsa_cred->handle);
 
-    lsa_cred->magic = 0;
+    /* Ensure compiler doesn't optimize out the assignment with 0. */
+    SecureZeroMemory(&lsa_cred->magic, sizeof(lsa_cred->magic));
     free(lsa_cred);
     return status;
 }
@@ -663,6 +688,7 @@ static SECURITY_STATUS WINAPI lsa_QueryContextAttributesA(CtxtHandle *context, U
     switch (attribute)
     {
     case SECPKG_ATTR_SIZES:
+    case SECPKG_ATTR_SESSION_KEY:
         return lsa_QueryContextAttributesW( context, attribute, buffer );
 
     case SECPKG_ATTR_NEGOTIATION_INFO:
@@ -699,7 +725,6 @@ static SECURITY_STATUS WINAPI lsa_QueryContextAttributesA(CtxtHandle *context, U
     X(SECPKG_ATTR_NATIVE_NAMES);
     X(SECPKG_ATTR_PACKAGE_INFO);
     X(SECPKG_ATTR_PASSWORD_EXPIRY);
-    X(SECPKG_ATTR_SESSION_KEY);
     X(SECPKG_ATTR_STREAM_SIZES);
     X(SECPKG_ATTR_TARGET_INFORMATION);
 #undef X
@@ -860,6 +885,41 @@ static void add_package(struct lsa_package *package)
     }
 }
 
+static BOOL initialize_package(struct lsa_package *package,
+                               NTSTATUS (NTAPI *pSpLsaModeInitialize)(ULONG, PULONG, PSECPKG_FUNCTION_TABLE *, PULONG),
+                               NTSTATUS (NTAPI *pSpUserModeInitialize)(ULONG, PULONG, PSECPKG_USER_FUNCTION_TABLE *, PULONG))
+{
+    NTSTATUS status;
+
+    if (!pSpLsaModeInitialize || !pSpUserModeInitialize)
+        return FALSE;
+
+    status = pSpLsaModeInitialize(SECPKG_INTERFACE_VERSION, &package->lsa_api_version, &package->lsa_api, &package->lsa_table_count);
+    if (status == STATUS_SUCCESS)
+    {
+        status = package->lsa_api->InitializePackage(package->package_id, &lsa_dispatch, NULL, NULL, &package->name);
+        if (status == STATUS_SUCCESS)
+        {
+            TRACE("name %s, version %#lx, api table %p, table count %lu\n",
+                  debugstr_an(package->name->Buffer, package->name->Length),
+                  package->lsa_api_version, package->lsa_api, package->lsa_table_count);
+
+            status = package->lsa_api->Initialize(package->package_id, NULL /* FIXME: params */, NULL);
+            if (status == STATUS_SUCCESS)
+            {
+                status = pSpUserModeInitialize(SECPKG_INTERFACE_VERSION, &package->user_api_version, &package->user_api, &package->user_table_count);
+                if (status == STATUS_SUCCESS)
+                {
+                    package->user_api->InstanceInit(SECPKG_INTERFACE_VERSION, &lsa_dll_dispatch, NULL);
+                    return TRUE;
+                }
+            }
+        }
+    }
+
+    return FALSE;
+}
+
 static BOOL load_package(const WCHAR *name, struct lsa_package *package, ULONG package_id)
 {
     NTSTATUS (NTAPI *pSpLsaModeInitialize)(ULONG, PULONG, PSECPKG_FUNCTION_TABLE *, PULONG);
@@ -867,40 +927,15 @@ static BOOL load_package(const WCHAR *name, struct lsa_package *package, ULONG p
 
     memset(package, 0, sizeof(*package));
 
+    package->package_id = package_id;
     package->mod = LoadLibraryW(name);
     if (!package->mod) return FALSE;
 
     pSpLsaModeInitialize = (void *)GetProcAddress(package->mod, "SpLsaModeInitialize");
-    if (pSpLsaModeInitialize)
-    {
-        NTSTATUS status;
+    pSpUserModeInitialize = (void *)GetProcAddress(package->mod, "SpUserModeInitialize");
 
-        status = pSpLsaModeInitialize(SECPKG_INTERFACE_VERSION, &package->lsa_api_version, &package->lsa_api, &package->lsa_table_count);
-        if (status == STATUS_SUCCESS)
-        {
-            status = package->lsa_api->InitializePackage(package_id, &lsa_dispatch, NULL, NULL, &package->name);
-            if (status == STATUS_SUCCESS)
-            {
-                TRACE("%s => %p, name %s, version %#lx, api table %p, table count %lu\n",
-                    debugstr_w(name), package->mod, debugstr_an(package->name->Buffer, package->name->Length),
-                    package->lsa_api_version, package->lsa_api, package->lsa_table_count);
-                package->package_id = package_id;
-
-                status = package->lsa_api->Initialize(package_id, NULL /* FIXME: params */, NULL);
-                if (status == STATUS_SUCCESS)
-                {
-                    pSpUserModeInitialize = (void *)GetProcAddress(package->mod, "SpUserModeInitialize");
-                    if (pSpUserModeInitialize)
-                    {
-                        status = pSpUserModeInitialize(SECPKG_INTERFACE_VERSION, &package->user_api_version, &package->user_api, &package->user_table_count);
-                        if (status == STATUS_SUCCESS)
-                            package->user_api->InstanceInit(SECPKG_INTERFACE_VERSION, &lsa_dll_dispatch, NULL);
-                    }
-                }
-                return TRUE;
-            }
-        }
-    }
+    if (initialize_package(package, pSpLsaModeInitialize, pSpUserModeInitialize))
+        return TRUE;
 
     FreeLibrary(package->mod);
     return FALSE;
@@ -913,6 +948,14 @@ void load_auth_packages(void)
     DWORD err, i;
     HKEY root;
     SecureProvider *provider;
+    struct lsa_package package;
+
+    memset(&package, 0, sizeof(package));
+
+    /* "Negotiate" has package id 0, .Net depends on this. */
+    package.package_id = 0;
+    if (initialize_package(&package, nego_SpLsaModeInitialize, nego_SpUserModeInitialize))
+        add_package(&package);
 
     err = RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"System\\CurrentControlSet\\Control\\Lsa", 0, KEY_READ, &root);
     if (err != ERROR_SUCCESS) return;
@@ -921,19 +964,18 @@ void load_auth_packages(void)
     for (;;)
     {
         WCHAR name[MAX_SERVICE_NAME];
-        struct lsa_package package;
 
-        err = RegEnumKeyW(root, i++, name, MAX_SERVICE_NAME);
+        err = RegEnumKeyW(root, i, name, MAX_SERVICE_NAME);
         if (err == ERROR_NO_MORE_ITEMS)
             break;
 
         if (err != ERROR_SUCCESS)
             continue;
 
-        if (!load_package(name, &package, i))
-            continue;
+        if (load_package(name, &package, i + 1))
+            add_package(&package);
 
-        add_package(&package);
+        i++;
     }
 
     RegCloseKey(root);
