@@ -55,8 +55,7 @@ static BOOL SHLWAPI_ParamAToW(LPCSTR lpszParam, LPWSTR lpszBuff, DWORD dwLen,
     else
     {
       /* Create a new buffer big enough for the string */
-      *lpszOut = HeapAlloc(GetProcessHeap(), 0,
-                                   dwStrLen * sizeof(WCHAR));
+      *lpszOut = malloc(dwStrLen * sizeof(WCHAR));
       if (!*lpszOut)
         return FALSE;
     }
@@ -197,10 +196,10 @@ HRESULT WINAPI AssocQueryKeyA(ASSOCF cfFlags, ASSOCKEY assockey, LPCSTR pszAssoc
   }
 
   if (lpszAssocW != szAssocW)
-    HeapFree(GetProcessHeap(), 0, lpszAssocW);
+    free(lpszAssocW);
 
   if (lpszExtraW != szExtraW)
-    HeapFree(GetProcessHeap(), 0, lpszExtraW);
+    free(lpszExtraW);
 
   return hRet;
 }
@@ -272,8 +271,7 @@ HRESULT WINAPI AssocQueryStringA(ASSOCF cfFlags, ASSOCSTR str, LPCSTR pszAssoc,
     DWORD dwLenOut = *pcchOut;
 
     if (dwLenOut >= MAX_PATH)
-      lpszReturnW = HeapAlloc(GetProcessHeap(), 0,
-                                      (dwLenOut + 1) * sizeof(WCHAR));
+      lpszReturnW = malloc((dwLenOut + 1) * sizeof(WCHAR));
     else
       dwLenOut = ARRAY_SIZE(szReturnW);
 
@@ -290,14 +288,14 @@ HRESULT WINAPI AssocQueryStringA(ASSOCF cfFlags, ASSOCSTR str, LPCSTR pszAssoc,
 
       *pcchOut = dwLenOut;
       if (lpszReturnW != szReturnW)
-        HeapFree(GetProcessHeap(), 0, lpszReturnW);
+        free(lpszReturnW);
     }
   }
 
   if (lpszAssocW != szAssocW)
-    HeapFree(GetProcessHeap(), 0, lpszAssocW);
+    free(lpszAssocW);
   if (lpszExtraW != szExtraW)
-    HeapFree(GetProcessHeap(), 0, lpszExtraW);
+    free(lpszExtraW);
   return hRet;
 }
 
@@ -364,8 +362,7 @@ HRESULT WINAPI AssocQueryStringByKeyA(ASSOCF cfFlags, ASSOCSTR str, HKEY hkAssoc
   {
     DWORD dwLenOut = *pcchOut;
     if (dwLenOut >= MAX_PATH)
-      lpszReturnW = HeapAlloc(GetProcessHeap(), 0,
-                                      (dwLenOut + 1) * sizeof(WCHAR));
+      lpszReturnW = malloc((dwLenOut + 1) * sizeof(WCHAR));
 
     if (lpszReturnW)
     {
@@ -377,12 +374,12 @@ HRESULT WINAPI AssocQueryStringByKeyA(ASSOCF cfFlags, ASSOCSTR str, HKEY hkAssoc
       *pcchOut = dwLenOut;
 
       if (lpszReturnW != szReturnW)
-        HeapFree(GetProcessHeap(), 0, lpszReturnW);
+        free(lpszReturnW);
     }
   }
 
   if (lpszExtraW != szExtraW)
-    HeapFree(GetProcessHeap(), 0, lpszExtraW);
+    free(lpszExtraW);
   return hRet;
 }
 
