@@ -49,7 +49,7 @@ static const char szCardsKey[] =
  */
 DWORD WINAPI lineAccept(HCALL hCall, LPCSTR lpsUserUserInfo, DWORD dwSize)
 {
-    FIXME("(%p, %s, %d): stub.\n", hCall, lpsUserUserInfo, dwSize);
+    FIXME("(%p, %s, %ld): stub.\n", hCall, lpsUserUserInfo, dwSize);
     return 1;
 }
 
@@ -85,7 +85,7 @@ DWORD WINAPI lineAddToConference(HCALL hConfCall, HCALL hConsultCall)
  */
 DWORD WINAPI lineAnswer(HCALL hCall, LPCSTR lpsUserUserInfo, DWORD dwSize)
 {
-    FIXME("(%p, %s, %d): stub.\n", hCall, lpsUserUserInfo, dwSize);
+    FIXME("(%p, %s, %ld): stub.\n", hCall, lpsUserUserInfo, dwSize);
     return 1;
 }
 
@@ -94,7 +94,7 @@ DWORD WINAPI lineAnswer(HCALL hCall, LPCSTR lpsUserUserInfo, DWORD dwSize)
  */
 DWORD WINAPI lineBlindTransferA(HCALL hCall, LPCSTR lpszDestAddress, DWORD dwCountryCode)
 {
-    FIXME("(%p, %s, %08x): stub.\n", hCall, lpszDestAddress, dwCountryCode);
+    FIXME("(%p, %s, %08lx): stub.\n", hCall, lpszDestAddress, dwCountryCode);
     return 1;
 }
 
@@ -112,7 +112,7 @@ DWORD WINAPI lineClose(HLINE hLine)
  */
 DWORD WINAPI lineCompleteCall(HCALL hCall, LPDWORD lpdwCompletionID, DWORD dwCompletionMode, DWORD dwMessageID)
 {
-    FIXME("(%p, %p, %08x, %08x): stub.\n", hCall, lpdwCompletionID, dwCompletionMode, dwMessageID);
+    FIXME("(%p, %p, %08lx, %08lx): stub.\n", hCall, lpdwCompletionID, dwCompletionMode, dwMessageID);
     return 1;
 }
 
@@ -121,7 +121,7 @@ DWORD WINAPI lineCompleteCall(HCALL hCall, LPDWORD lpdwCompletionID, DWORD dwCom
  */
 DWORD WINAPI lineCompleteTransfer(HCALL hCall, HCALL hConsultCall, LPHCALL lphConfCall, DWORD dwTransferMode)
 {
-    FIXME("(%p, %p, %p, %08x): stub.\n", hCall, hConsultCall, lphConfCall, dwTransferMode);
+    FIXME("(%p, %p, %p, %08lx): stub.\n", hCall, hConsultCall, lphConfCall, dwTransferMode);
     return 1;
 }
 
@@ -130,7 +130,7 @@ DWORD WINAPI lineCompleteTransfer(HCALL hCall, HCALL hConsultCall, LPHCALL lphCo
  */
 DWORD WINAPI lineConfigDialogA(DWORD dwDeviceID, HWND hwndOwner, LPCSTR lpszDeviceClass)
 {
-    FIXME("(%08x, %p, %s): stub.\n", dwDeviceID, hwndOwner, lpszDeviceClass);
+    FIXME("(%08lx, %p, %s): stub.\n", dwDeviceID, hwndOwner, lpszDeviceClass);
     return 0;
 }
 
@@ -139,7 +139,7 @@ DWORD WINAPI lineConfigDialogA(DWORD dwDeviceID, HWND hwndOwner, LPCSTR lpszDevi
  */
 DWORD WINAPI lineConfigDialogW(DWORD dwDeviceID, HWND hwndOwner, LPCWSTR lpszDeviceClass)
 {
-    FIXME("(%08x, %p, %s): stub.\n", dwDeviceID, hwndOwner, debugstr_w(lpszDeviceClass));
+    FIXME("(%08lx, %p, %s): stub.\n", dwDeviceID, hwndOwner, debugstr_w(lpszDeviceClass));
     return 0;
 }
 
@@ -157,7 +157,7 @@ DWORD WINAPI lineConfigDialogEditA(DWORD dwDeviceID, HWND hwndOwner, LPCSTR lpsz
  */
 DWORD WINAPI lineConfigProvider(HWND hwndOwner, DWORD dwPermanentProviderID)
 {
-    FIXME("(%p, %08x): stub.\n", hwndOwner, dwPermanentProviderID);
+    FIXME("(%p, %08lx): stub.\n", hwndOwner, dwPermanentProviderID);
     return 0;
 }
 
@@ -175,7 +175,7 @@ DWORD WINAPI lineDeallocateCall(HCALL hCall)
  */
 DWORD WINAPI lineDevSpecific(HLINE hLine, DWORD dwAddressId, HCALL hCall, LPVOID lpParams, DWORD dwSize)
 {
-    FIXME("(%p, %08x, %p, %p, %d): stub.\n", hLine, dwAddressId, hCall, lpParams, dwSize);
+    FIXME("(%p, %08lx, %p, %p, %ld): stub.\n", hLine, dwAddressId, hCall, lpParams, dwSize);
     return 1;
 }
 
@@ -184,7 +184,7 @@ DWORD WINAPI lineDevSpecific(HLINE hLine, DWORD dwAddressId, HCALL hCall, LPVOID
  */
 DWORD WINAPI lineDevSpecificFeature(HLINE hLine, DWORD dwFeature, LPVOID lpParams, DWORD dwSize)
 {
-    FIXME("(%p, %08x, %p, %d): stub.\n", hLine, dwFeature, lpParams, dwSize);
+    FIXME("(%p, %08lx, %p, %ld): stub.\n", hLine, dwFeature, lpParams, dwSize);
     return 1;
 }
 
@@ -193,7 +193,7 @@ DWORD WINAPI lineDevSpecificFeature(HLINE hLine, DWORD dwFeature, LPVOID lpParam
  */
 DWORD WINAPI lineDialA(HCALL hCall, LPCSTR lpszDestAddress, DWORD dwCountryCode)
 {
-    FIXME("(%p, %s, %08x): stub.\n", hCall, lpszDestAddress, dwCountryCode);
+    FIXME("(%p, %s, %08lx): stub.\n", hCall, lpszDestAddress, dwCountryCode);
     return 1;
 }
 
@@ -202,7 +202,7 @@ DWORD WINAPI lineDialA(HCALL hCall, LPCSTR lpszDestAddress, DWORD dwCountryCode)
  */
 DWORD WINAPI lineDialW(HCALL hCall, LPCWSTR lpszDestAddress, DWORD dwCountryCode)
 {
-    FIXME("(%p, %s, %08x): stub.\n", hCall, debugstr_w(lpszDestAddress), dwCountryCode);
+    FIXME("(%p, %s, %08lx): stub.\n", hCall, debugstr_w(lpszDestAddress), dwCountryCode);
     return 1;
 }
 
@@ -211,7 +211,7 @@ DWORD WINAPI lineDialW(HCALL hCall, LPCWSTR lpszDestAddress, DWORD dwCountryCode
  */
 DWORD WINAPI lineDrop(HCALL hCall, LPCSTR lpsUserUserInfo, DWORD dwSize)
 {
-    FIXME("(%p, %s, %08x): stub.\n", hCall, lpsUserUserInfo, dwSize);
+    FIXME("(%p, %s, %08lx): stub.\n", hCall, lpsUserUserInfo, dwSize);
     return 1;
 }
 
@@ -238,7 +238,7 @@ DWORD WINAPI lineGatherDigitsA(HCALL hCall, DWORD dwDigitModes, LPSTR lpsDigits,
  */
 DWORD WINAPI lineGenerateDigitsA(HCALL hCall, DWORD dwDigitModes, LPCSTR lpszDigits, DWORD dwDuration)
 {
-    FIXME("(%p, %08x, %s, %d): stub.\n", hCall, dwDigitModes, lpszDigits, dwDuration);
+    FIXME("(%p, %08lx, %s, %ld): stub.\n", hCall, dwDigitModes, lpszDigits, dwDuration);
     return 0;
 }
 
@@ -247,7 +247,7 @@ DWORD WINAPI lineGenerateDigitsA(HCALL hCall, DWORD dwDigitModes, LPCSTR lpszDig
  */
 DWORD WINAPI lineGenerateTone(HCALL hCall, DWORD dwToneMode, DWORD dwDuration, DWORD dwNumTones, LPLINEGENERATETONE lpTones)
 {
-    FIXME("(%p, %08x, %d, %d, %p): stub.\n", hCall, dwToneMode, dwDuration, dwNumTones, lpTones);
+    FIXME("(%p, %08lx, %ld, %ld, %p): stub.\n", hCall, dwToneMode, dwDuration, dwNumTones, lpTones);
     return 0;
 }
 
@@ -256,7 +256,7 @@ DWORD WINAPI lineGenerateTone(HCALL hCall, DWORD dwToneMode, DWORD dwDuration, D
  */
 DWORD WINAPI lineGetAddressCapsA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAddressID, DWORD dwAPIVersion, DWORD dwExtVersion, LPLINEADDRESSCAPS lpAddressCaps)
 {
-    FIXME("(%p, %08x, %08x, %08x, %08x, %p): stub.\n", hLineApp, dwDeviceID, dwAddressID, dwAPIVersion, dwExtVersion, lpAddressCaps);
+    FIXME("(%p, %08lx, %08lx, %08lx, %08lx, %p): stub.\n", hLineApp, dwDeviceID, dwAddressID, dwAPIVersion, dwExtVersion, lpAddressCaps);
     return 0;
 }
 
@@ -265,7 +265,7 @@ DWORD WINAPI lineGetAddressCapsA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAd
  */
 DWORD WINAPI lineGetAddressIDA(HLINE hLine, LPDWORD lpdwAddressID, DWORD dwAddressMode, LPCSTR lpsAddress, DWORD dwSize)
 {
-    FIXME("%p, %p, %08x, %s, %d): stub.\n", hLine, lpdwAddressID, dwAddressMode, lpsAddress, dwSize);
+    FIXME("%p, %p, %08lx, %s, %ld): stub.\n", hLine, lpdwAddressID, dwAddressMode, lpsAddress, dwSize);
     return 0;
 }
 
@@ -274,7 +274,7 @@ DWORD WINAPI lineGetAddressIDA(HLINE hLine, LPDWORD lpdwAddressID, DWORD dwAddre
  */
 DWORD WINAPI lineGetAddressStatusA(HLINE hLine, DWORD dwAddressID, LPLINEADDRESSSTATUS lpAddressStatus)
 {
-    FIXME("(%p, %08x, %p): stub.\n", hLine, dwAddressID, lpAddressStatus);
+    FIXME("(%p, %08lx, %p): stub.\n", hLine, dwAddressID, lpAddressStatus);
     return 0;
 }
 
@@ -283,7 +283,7 @@ DWORD WINAPI lineGetAddressStatusA(HLINE hLine, DWORD dwAddressID, LPLINEADDRESS
  */
 DWORD WINAPI lineGetAppPriorityA(LPCSTR lpszAppFilename, DWORD dwMediaMode, LPLINEEXTENSIONID const lpExtensionID, DWORD dwRequestMode, LPVARSTRING lpExtensionName, LPDWORD lpdwPriority)
 {
-    FIXME("(%s, %08x, %p, %08x, %p, %p): stub.\n", lpszAppFilename, dwMediaMode, lpExtensionID, dwRequestMode, lpExtensionName, lpdwPriority);
+    FIXME("(%s, %08lx, %p, %08lx, %p, %p): stub.\n", lpszAppFilename, dwMediaMode, lpExtensionID, dwRequestMode, lpExtensionName, lpdwPriority);
     return 0;
 }
 
@@ -296,6 +296,15 @@ DWORD WINAPI lineGetCallInfoA(HCALL hCall, LPLINECALLINFO lpCallInfo)
     return 0;
 }
 
+/***********************************************************************
+ *		lineGetCallInfoW (TAPI32.@)
+ */
+DWORD WINAPI lineGetCallInfoW(HCALL hCall, LPLINECALLINFO lpCallInfo)
+{
+    FIXME("(%p, %p): stub.\n", hCall, lpCallInfo);
+    return 0;
+}
+
 /***********************************************************************
  *		lineGetCallStatus (TAPI32.@)
  */
@@ -325,12 +334,12 @@ DWORD WINAPI lineGetCountryA(DWORD dwCountryID, DWORD dwAPIVersion, LPLINECOUNTR
     char *subkey_name;
 
     if(!lpLineCountryList) {
-	TRACE("(%08x, %08x, %p): stub. Returning LINEERR_INVALPOINTER\n",
+	TRACE("(%08lx, %08lx, %p): stub. Returning LINEERR_INVALPOINTER\n",
 	      dwCountryID, dwAPIVersion, lpLineCountryList);
         return LINEERR_INVALPOINTER;
     }
 
-    TRACE("(%08x, %08x, %p(%d)): stub.\n",
+    TRACE("(%08lx, %08lx, %p(%ld)): stub.\n",
 	  dwCountryID, dwAPIVersion, lpLineCountryList,
 	  lpLineCountryList->dwTotalSize);
 
@@ -466,7 +475,7 @@ DWORD WINAPI lineGetCountryA(DWORD dwCountryID, DWORD dwAPIVersion, LPLINECOUNTR
 
     lpLineCountryList->dwNeededSize = dwOffset;
 
-    TRACE("%d available %d required\n", dwAvailSize, dwOffset);
+    TRACE("%ld available %ld required\n", dwAvailSize, dwOffset);
 
     HeapFree(GetProcessHeap(), 0, subkey_name);
     RegCloseKey(hkey);
@@ -499,7 +508,7 @@ DWORD WINAPI lineGetCountryW(DWORD id, DWORD version, LPLINECOUNTRYLIST list)
     HKEY hkey;
 
     if (!list) return LINEERR_INVALPOINTER;
-    TRACE("(%08x, %08x, %p(%d))\n", id, version, list, list->dwTotalSize);
+    TRACE("(%08lx, %08lx, %p(%ld))\n", id, version, list, list->dwTotalSize);
 
     if (RegOpenKeyW(HKEY_LOCAL_MACHINE, country_listW, &hkey) != ERROR_SUCCESS)
         return LINEERR_INIFILECORRUPT;
@@ -603,7 +612,7 @@ DWORD WINAPI lineGetCountryW(DWORD id, DWORD version, LPLINECOUNTRYLIST list)
     }
     list->dwNeededSize = offset;
 
-    TRACE("%d available %d required\n", total_size, offset);
+    TRACE("%ld available %ld required\n", total_size, offset);
 
     HeapFree(GetProcessHeap(), 0, subkey_name);
     RegCloseKey(hkey);
@@ -619,7 +628,7 @@ DWORD WINAPI lineGetDevCapsW(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAPIVer
     static int warn_once;
 
     if(!warn_once++)
-        FIXME("(%p, %08x, %08x, %08x, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
+        FIXME("(%p, %08lx, %08lx, %08lx, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
                                                  dwExtVersion, lpLineDevCaps);
     return LINEERR_OPERATIONFAILED;
 }
@@ -633,7 +642,7 @@ DWORD WINAPI lineGetDevCapsA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAPIVer
     static int warn_once;
 
     if(!warn_once++)
-        FIXME("(%p, %08x, %08x, %08x, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
+        FIXME("(%p, %08lx, %08lx, %08lx, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
                                                  dwExtVersion, lpLineDevCaps);
     return LINEERR_OPERATIONFAILED;
 }
@@ -643,7 +652,16 @@ DWORD WINAPI lineGetDevCapsA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAPIVer
  */
 DWORD WINAPI lineGetDevConfigA(DWORD dwDeviceID, LPVARSTRING lpDeviceConfig, LPCSTR lpszDeviceClass)
 {
-    FIXME("(%08x, %p, %s): stub.\n", dwDeviceID, lpDeviceConfig, lpszDeviceClass);
+    FIXME("(%08lx, %p, %s): stub.\n", dwDeviceID, lpDeviceConfig, lpszDeviceClass);
+    return 0;
+}
+
+/***********************************************************************
+ *		lineGetDevConfigW (TAPI32.@)
+ */
+DWORD WINAPI lineGetDevConfigW(DWORD dwDeviceID, LPVARSTRING lpDeviceConfig, LPCWSTR lpszDeviceClass)
+{
+    FIXME("(%08lx, %p, %s): stub.\n", dwDeviceID, lpDeviceConfig, debugstr_w(lpszDeviceClass));
     return 0;
 }
 
@@ -653,7 +671,7 @@ DWORD WINAPI lineGetDevConfigA(DWORD dwDeviceID, LPVARSTRING lpDeviceConfig, LPC
 DWORD WINAPI lineGetIDW(HLINE hLine, DWORD dwAddressID, HCALL hCall, DWORD dwSelect,
                         LPVARSTRING lpDeviceID, LPCWSTR lpszDeviceClass)
 {
-    FIXME("(%p, %08x, %p, %08x, %p, %s): stub.\n", hLine, dwAddressID, hCall,
+    FIXME("(%p, %08lx, %p, %08lx, %p, %s): stub.\n", hLine, dwAddressID, hCall,
                                                    dwSelect, lpDeviceID,
                                                    debugstr_w(lpszDeviceClass));
     return LINEERR_OPERATIONFAILED;
@@ -665,7 +683,7 @@ DWORD WINAPI lineGetIDW(HLINE hLine, DWORD dwAddressID, HCALL hCall, DWORD dwSel
 DWORD WINAPI lineGetIDA(HLINE hLine, DWORD dwAddressID, HCALL hCall, DWORD dwSelect,
                         LPVARSTRING lpDeviceID, LPCSTR lpszDeviceClass)
 {
-    FIXME("(%p, %08x, %p, %08x, %p, %s): stub.\n", hLine, dwAddressID, hCall,
+    FIXME("(%p, %08lx, %p, %08lx, %p, %s): stub.\n", hLine, dwAddressID, hCall,
                                                    dwSelect, lpDeviceID, lpszDeviceClass);
     return LINEERR_OPERATIONFAILED;
 }
@@ -675,7 +693,16 @@ DWORD WINAPI lineGetIDA(HLINE hLine, DWORD dwAddressID, HCALL hCall, DWORD dwSel
  */
 DWORD WINAPI lineGetIconA(DWORD dwDeviceID, LPCSTR lpszDeviceClass, HICON *lphIcon)
 {
-    FIXME("(%08x, %s, %p): stub.\n", dwDeviceID, lpszDeviceClass, lphIcon);
+    FIXME("(%08lx, %s, %p): stub.\n", dwDeviceID, lpszDeviceClass, lphIcon);
+    return 0;
+}
+
+/***********************************************************************
+ *		lineGetIconW (TAPI32.@)
+ */
+DWORD WINAPI lineGetIconW(DWORD dwDeviceID, LPCWSTR lpszDeviceClass, HICON *lphIcon)
+{
+    FIXME("(%08lx, %s, %p): stub.\n", dwDeviceID, debugstr_w(lpszDeviceClass), lphIcon);
     return 0;
 }
 
@@ -693,7 +720,7 @@ DWORD WINAPI lineGetLineDevStatusA(HLINE hLine, LPLINEDEVSTATUS lpLineDevStatus)
  */
 DWORD WINAPI lineGetMessage(HLINEAPP hLineApp, LPLINEMESSAGE lpMessage, DWORD dwTimeout)
 {
-    FIXME("(%p, %p, %08x): stub.\n", hLineApp, lpMessage, dwTimeout);
+    FIXME("(%p, %p, %08lx): stub.\n", hLineApp, lpMessage, dwTimeout);
     return 0;
 }
 
@@ -702,7 +729,7 @@ DWORD WINAPI lineGetMessage(HLINEAPP hLineApp, LPLINEMESSAGE lpMessage, DWORD dw
  */
 DWORD WINAPI lineGetNewCalls(HLINE hLine, DWORD dwAddressID, DWORD dwSelect, LPLINECALLLIST lpCallList)
 {
-    FIXME("(%p, %08x, %08x, %p): stub.\n", hLine, dwAddressID, dwSelect, lpCallList);
+    FIXME("(%p, %08lx, %08lx, %p): stub.\n", hLine, dwAddressID, dwSelect, lpCallList);
     return 0;
 }
 
@@ -711,7 +738,7 @@ DWORD WINAPI lineGetNewCalls(HLINE hLine, DWORD dwAddressID, DWORD dwSelect, LPL
  */
 DWORD WINAPI lineGetNumRings(HLINE hLine, DWORD dwAddressID, LPDWORD lpdwNumRings)
 {
-    FIXME("(%p, %08x, %p): stub.\n", hLine, dwAddressID, lpdwNumRings);
+    FIXME("(%p, %08lx, %p): stub.\n", hLine, dwAddressID, lpdwNumRings);
     return 0;
 }
 
@@ -720,7 +747,7 @@ DWORD WINAPI lineGetNumRings(HLINE hLine, DWORD dwAddressID, LPDWORD lpdwNumRing
  */
 DWORD WINAPI lineGetProviderListA(DWORD dwAPIVersion, LPLINEPROVIDERLIST lpProviderList)
 {
-    FIXME("(%08x, %p): stub.\n", dwAPIVersion, lpProviderList);
+    FIXME("(%08lx, %p): stub.\n", dwAPIVersion, lpProviderList);
     return LINEERR_OPERATIONFAILED;
 }
 
@@ -729,7 +756,7 @@ DWORD WINAPI lineGetProviderListA(DWORD dwAPIVersion, LPLINEPROVIDERLIST lpProvi
  */
 DWORD WINAPI lineGetProviderListW(DWORD dwAPIVersion, LPLINEPROVIDERLIST lpProviderList)
 {
-    FIXME("(%08x, %p): stub.\n", dwAPIVersion, lpProviderList);
+    FIXME("(%08lx, %p): stub.\n", dwAPIVersion, lpProviderList);
     return LINEERR_OPERATIONFAILED;
 }
 
@@ -738,7 +765,7 @@ DWORD WINAPI lineGetProviderListW(DWORD dwAPIVersion, LPLINEPROVIDERLIST lpProvi
  */
 DWORD WINAPI lineGetRequestA(HLINEAPP hLineApp, DWORD dwRequestMode, LPVOID lpRequestBuffer)
 {
-    FIXME("%p, %08x, %p): stub.\n", hLineApp, dwRequestMode, lpRequestBuffer);
+    FIXME("%p, %08lx, %p): stub.\n", hLineApp, dwRequestMode, lpRequestBuffer);
     return 0;
 }
 
@@ -756,7 +783,7 @@ DWORD WINAPI lineGetStatusMessages(HLINE hLine, LPDWORD lpdwLineStatus, LPDWORD
  */
 DWORD WINAPI lineGetTranslateCapsW(HLINEAPP line_app, DWORD api_version, LINETRANSLATECAPS *caps)
 {
-    FIXME("(%p, %08x, %p): stub.\n", line_app, api_version, caps);
+    FIXME("(%p, %08lx, %p): stub.\n", line_app, api_version, caps);
     return LINEERR_OPERATIONFAILED;
 }
 
@@ -823,7 +850,7 @@ DWORD WINAPI lineGetTranslateCapsA(HLINEAPP hLineApp, DWORD dwAPIVersion,
     LPLINELOCATIONENTRY pLocEntry;
     LPLINECARDENTRY pCardEntry;
     
-    TRACE("(%p, %08x, %p (tot. size %d)\n", hLineApp, dwAPIVersion,
+    TRACE("(%p, %08lx, %p (tot. size %ld)\n", hLineApp, dwAPIVersion,
             lpTranslateCaps, lpTranslateCaps->dwTotalSize );
     if( lpTranslateCaps->dwTotalSize < sizeof(LINETRANSLATECAPS))
         return LINEERR_STRUCTURETOOSMALL;
@@ -960,7 +987,7 @@ DWORD WINAPI lineGetTranslateCapsA(HLINEAPP hLineApp, DWORD dwAPIVersion,
         HeapFree(GetProcessHeap(), 0, loc_key_name);
         HeapFree(GetProcessHeap(), 0, card_key_name);
         lpTranslateCaps->dwUsedSize = sizeof(LINETRANSLATECAPS);
-        TRACE("Insufficient space: total %d needed %d used %d\n",
+        TRACE("Insufficient space: total %ld needed %ld used %ld\n",
                 lpTranslateCaps->dwTotalSize,
                 lpTranslateCaps->dwNeededSize,
                 lpTranslateCaps->dwUsedSize);
@@ -1063,9 +1090,9 @@ DWORD WINAPI lineGetTranslateCapsA(HLINEAPP hLineApp, DWORD dwAPIVersion,
         if(pLocEntry->dwPermanentLocationID == currentid )
             lpTranslateCaps->dwCurrentPreferredCardID =
                     pLocEntry->dwPreferredCardID;
-        TRACE("added: ID %d %s CountryCode %d CityCode %s CardID %d "
-                "LocalAccess: %s LongDistanceAccess: %s CountryID %d "
-                "Options %d CancelCallWait %s\n",
+        TRACE("added: ID %ld %s CountryCode %ld CityCode %s CardID %ld "
+                "LocalAccess: %s LongDistanceAccess: %s CountryID %ld "
+                "Options %ld CancelCallWait %s\n",
                 pLocEntry->dwPermanentLocationID,
                 debugstr_a( (char*)lpTranslateCaps + pLocEntry->dwLocationNameOffset),
                 pLocEntry->dwCountryCode,
@@ -1120,7 +1147,7 @@ DWORD WINAPI lineGetTranslateCapsA(HLINEAPP hLineApp, DWORD dwAPIVersion,
             size_val=sizeof(DWORD);
             RegQueryValueExA(hsubkey, "Flags",NULL, NULL,
                     (LPBYTE) &(pCardEntry->dwOptions), &size_val); 
-            TRACE( "added card: ID %d name %s SameArea %s LongDistance %s International %s Options 0x%x\n", 
+            TRACE( "added card: ID %ld name %s SameArea %s LongDistance %s International %s Options 0x%lx\n",
                     pCardEntry->dwPermanentCardID,
                     debugstr_a( (char*)lpTranslateCaps + pCardEntry->dwCardNameOffset),
                     debugstr_a( (char*)lpTranslateCaps + pCardEntry->dwSameAreaRuleOffset),
@@ -1137,7 +1164,7 @@ DWORD WINAPI lineGetTranslateCapsA(HLINEAPP hLineApp, DWORD dwAPIVersion,
     if(hkCardLocations) RegCloseKey(hkCardLocations);
     HeapFree(GetProcessHeap(), 0, loc_key_name);
     HeapFree(GetProcessHeap(), 0, card_key_name);
-    TRACE(" returning success tot %d needed %d used %d\n",
+    TRACE(" returning success tot %ld needed %ld used %ld\n",
             lpTranslateCaps->dwTotalSize,
             lpTranslateCaps->dwNeededSize,
             lpTranslateCaps->dwUsedSize );
@@ -1149,7 +1176,7 @@ DWORD WINAPI lineGetTranslateCapsA(HLINEAPP hLineApp, DWORD dwAPIVersion,
  */
 DWORD WINAPI lineHandoffA(HCALL hCall, LPCSTR lpszFileName, DWORD dwMediaMode)
 {
-    FIXME("(%p, %s, %08x): stub.\n", hCall, lpszFileName, dwMediaMode);
+    FIXME("(%p, %s, %08lx): stub.\n", hCall, lpszFileName, dwMediaMode);
     return 0;
 }
 
@@ -1203,7 +1230,7 @@ LONG WINAPI lineInitializeExW(LPHLINEAPP lphLineApp, HINSTANCE hInstance, LINECA
 DWORD WINAPI lineMakeCallW(HLINE hLine, LPHCALL lphCall, LPCWSTR lpszDestAddress,
                            DWORD dwCountryCode, LPLINECALLPARAMS lpCallParams)
 {
-    FIXME("(%p, %p, %s, %08x, %p): stub.\n", hLine, lphCall, debugstr_w(lpszDestAddress),
+    FIXME("(%p, %p, %s, %08lx, %p): stub.\n", hLine, lphCall, debugstr_w(lpszDestAddress),
                                              dwCountryCode, lpCallParams);
     return LINEERR_OPERATIONFAILED;
 }
@@ -1214,7 +1241,7 @@ DWORD WINAPI lineMakeCallW(HLINE hLine, LPHCALL lphCall, LPCWSTR lpszDestAddress
 DWORD WINAPI lineMakeCallA(HLINE hLine, LPHCALL lphCall, LPCSTR lpszDestAddress,
                            DWORD dwCountryCode, LPLINECALLPARAMS lpCallParams)
 {
-    FIXME("(%p, %p, %s, %08x, %p): stub.\n", hLine, lphCall, lpszDestAddress,
+    FIXME("(%p, %p, %s, %08lx, %p): stub.\n", hLine, lphCall, lpszDestAddress,
                                              dwCountryCode, lpCallParams);
     return LINEERR_OPERATIONFAILED;
 }
@@ -1224,7 +1251,7 @@ DWORD WINAPI lineMakeCallA(HLINE hLine, LPHCALL lphCall, LPCSTR lpszDestAddress,
  */
 DWORD WINAPI lineMonitorDigits(HCALL hCall, DWORD dwDigitModes)
 {
-    FIXME("(%p, %08x): stub.\n", hCall, dwDigitModes);
+    FIXME("(%p, %08lx): stub.\n", hCall, dwDigitModes);
     return 0;
 }
 
@@ -1233,7 +1260,7 @@ DWORD WINAPI lineMonitorDigits(HCALL hCall, DWORD dwDigitModes)
  */
 DWORD WINAPI lineMonitorMedia(HCALL hCall, DWORD dwMediaModes)
 {
-    FIXME("(%p, %08x): stub.\n", hCall, dwMediaModes);
+    FIXME("(%p, %08lx): stub.\n", hCall, dwMediaModes);
     return 0;
 }
 
@@ -1242,7 +1269,7 @@ DWORD WINAPI lineMonitorMedia(HCALL hCall, DWORD dwMediaModes)
  */
 DWORD WINAPI lineMonitorTones(HCALL hCall, LPLINEMONITORTONE lpToneList, DWORD dwNumEntries)
 {
-    FIXME("(%p, %p, %08x): stub.\n", hCall, lpToneList, dwNumEntries);
+    FIXME("(%p, %p, %08lx): stub.\n", hCall, lpToneList, dwNumEntries);
     return 0;
 }
 
@@ -1261,7 +1288,7 @@ DWORD WINAPI lineNegotiateAPIVersion(
     static int warn_once;
 
     if(!warn_once++)
-        FIXME("(%p, %d, %d, %d, %p, %p): stub.\n", hLineApp, dwDeviceID,
+        FIXME("(%p, %ld, %ld, %ld, %p, %p): stub.\n", hLineApp, dwDeviceID,
               dwAPILowVersion, dwAPIHighVersion, lpdwAPIVersion, lpExtensionID);
     *lpdwAPIVersion = dwAPIHighVersion;
     return 0;
@@ -1299,7 +1326,7 @@ DWORD WINAPI lineOpenA(HLINEAPP hLineApp, DWORD dwDeviceID, LPHLINE lphLine, DWO
  */
 DWORD WINAPI lineParkA(HCALL hCall, DWORD dwParkMode, LPCSTR lpszDirAddress, LPVARSTRING lpNonDirAddress)
 {
-    FIXME("(%p, %08x, %s, %p): stub.\n", hCall, dwParkMode, lpszDirAddress, lpNonDirAddress);
+    FIXME("(%p, %08lx, %s, %p): stub.\n", hCall, dwParkMode, lpszDirAddress, lpNonDirAddress);
     return 1;
 }
 
@@ -1308,7 +1335,7 @@ DWORD WINAPI lineParkA(HCALL hCall, DWORD dwParkMode, LPCSTR lpszDirAddress, LPV
  */
 DWORD WINAPI linePickupA(HLINE hLine, DWORD dwAddressID, LPHCALL lphCall, LPCSTR lpszDestAddress, LPCSTR lpszGroupID)
 {
-    FIXME("(%p, %08x, %p, %s, %s): stub.\n", hLine, dwAddressID, lphCall, lpszDestAddress, lpszGroupID);
+    FIXME("(%p, %08lx, %p, %s, %s): stub.\n", hLine, dwAddressID, lphCall, lpszDestAddress, lpszGroupID);
     return 1;
 }
 
@@ -1338,7 +1365,7 @@ DWORD WINAPI lineRedirectA(
  */
 DWORD WINAPI lineRegisterRequestRecipient(HLINEAPP hLineApp, DWORD dwRegistrationInstance, DWORD dwRequestMode, DWORD dwEnable)
 {
-    FIXME("(%p, %08x, %08x, %08x): stub.\n", hLineApp, dwRegistrationInstance, dwRequestMode, dwEnable);
+    FIXME("(%p, %08lx, %08lx, %08lx): stub.\n", hLineApp, dwRegistrationInstance, dwRequestMode, dwEnable);
     return 1;
 }
 
@@ -1365,7 +1392,7 @@ DWORD WINAPI lineRemoveFromConference(HCALL hCall)
  */
 DWORD WINAPI lineRemoveProvider(DWORD dwPermanentProviderID, HWND hwndOwner)
 {
-    FIXME("(%08x, %p): stub.\n", dwPermanentProviderID, hwndOwner);
+    FIXME("(%08lx, %p): stub.\n", dwPermanentProviderID, hwndOwner);
     return 1;
 }
 
@@ -1383,7 +1410,7 @@ DWORD WINAPI lineSecureCall(HCALL hCall)
  */
 DWORD WINAPI lineSendUserUserInfo(HCALL hCall, LPCSTR lpsUserUserInfo, DWORD dwSize)
 {
-    FIXME("(%p, %s, %08x): stub.\n", hCall, lpsUserUserInfo, dwSize);
+    FIXME("(%p, %s, %08lx): stub.\n", hCall, lpsUserUserInfo, dwSize);
     return 1;
 }
 
@@ -1392,7 +1419,7 @@ DWORD WINAPI lineSendUserUserInfo(HCALL hCall, LPCSTR lpsUserUserInfo, DWORD dwS
  */
 DWORD WINAPI lineSetAppPriorityA(LPCSTR lpszAppFilename, DWORD dwMediaMode, LPLINEEXTENSIONID const lpExtensionID, DWORD dwRequestMode, LPCSTR lpszExtensionName, DWORD dwPriority)
 {
-    FIXME("(%s, %08x, %p, %08x, %s, %08x): stub.\n", lpszAppFilename, dwMediaMode, lpExtensionID, dwRequestMode, lpszExtensionName, dwPriority);
+    FIXME("(%s, %08lx, %p, %08lx, %s, %08lx): stub.\n", lpszAppFilename, dwMediaMode, lpExtensionID, dwRequestMode, lpszExtensionName, dwPriority);
     return 0;
 }
 
@@ -1401,7 +1428,7 @@ DWORD WINAPI lineSetAppPriorityA(LPCSTR lpszAppFilename, DWORD dwMediaMode, LPLI
  */
 DWORD WINAPI lineSetAppSpecific(HCALL hCall, DWORD dwAppSpecific)
 {
-    FIXME("(%p, %08x): stub.\n", hCall, dwAppSpecific);
+    FIXME("(%p, %08lx): stub.\n", hCall, dwAppSpecific);
     return 0;
 }
 
@@ -1410,7 +1437,7 @@ DWORD WINAPI lineSetAppSpecific(HCALL hCall, DWORD dwAppSpecific)
  */
 DWORD WINAPI lineSetCallParams(HCALL hCall, DWORD dwBearerMode, DWORD dwMinRate, DWORD dwMaxRate, LPLINEDIALPARAMS lpDialParams)
 {
-    FIXME("(%p, %08x, %08x, %08x, %p): stub.\n", hCall, dwBearerMode, dwMinRate, dwMaxRate, lpDialParams);
+    FIXME("(%p, %08lx, %08lx, %08lx, %p): stub.\n", hCall, dwBearerMode, dwMinRate, dwMaxRate, lpDialParams);
     return 1;
 }
 
@@ -1419,7 +1446,7 @@ DWORD WINAPI lineSetCallParams(HCALL hCall, DWORD dwBearerMode, DWORD dwMinRate,
  */
 DWORD WINAPI lineSetCallPrivilege(HCALL hCall, DWORD dwCallPrivilege)
 {
-    FIXME("(%p, %08x): stub.\n", hCall, dwCallPrivilege);
+    FIXME("(%p, %08lx): stub.\n", hCall, dwCallPrivilege);
     return 0;
 }
 
@@ -1428,7 +1455,7 @@ DWORD WINAPI lineSetCallPrivilege(HCALL hCall, DWORD dwCallPrivilege)
  */
 DWORD WINAPI lineSetCurrentLocation(HLINEAPP hLineApp, DWORD dwLocation)
 {
-    FIXME("(%p, %08x): stub.\n", hLineApp, dwLocation);
+    FIXME("(%p, %08lx): stub.\n", hLineApp, dwLocation);
     return 0;
 }
 
@@ -1437,7 +1464,16 @@ DWORD WINAPI lineSetCurrentLocation(HLINEAPP hLineApp, DWORD dwLocation)
  */
 DWORD WINAPI lineSetDevConfigA(DWORD dwDeviceID, LPVOID lpDeviceConfig, DWORD dwSize, LPCSTR lpszDeviceClass)
 {
-    FIXME("(%08x, %p, %08x, %s): stub.\n", dwDeviceID, lpDeviceConfig, dwSize, lpszDeviceClass);
+    FIXME("(%08lx, %p, %08lx, %s): stub.\n", dwDeviceID, lpDeviceConfig, dwSize, lpszDeviceClass);
+    return 0;
+}
+
+/***********************************************************************
+ *		lineSetDevConfigW (TAPI32.@)
+ */
+DWORD WINAPI lineSetDevConfigW(DWORD dwDeviceID, LPVOID lpDeviceConfig, DWORD dwSize, LPCWSTR lpszDeviceClass)
+{
+    FIXME("(%08lx, %p, %08lx, %s): stub.\n", dwDeviceID, lpDeviceConfig, dwSize, debugstr_w(lpszDeviceClass));
     return 0;
 }
 
@@ -1467,7 +1503,7 @@ DWORD dwCallStateNumEntries)
  */
 DWORD WINAPI lineSetMediaMode(HCALL hCall, DWORD dwMediaModes)
 {
-    FIXME("(%p, %08x): stub.\n", hCall, dwMediaModes);
+    FIXME("(%p, %08lx): stub.\n", hCall, dwMediaModes);
     return 0;
 }
 
@@ -1476,7 +1512,7 @@ DWORD WINAPI lineSetMediaMode(HCALL hCall, DWORD dwMediaModes)
  */
 DWORD WINAPI lineSetNumRings(HLINE hLine, DWORD dwAddressID, DWORD dwNumRings)
 {
-    FIXME("(%p, %08x, %08x): stub.\n", hLine, dwAddressID, dwNumRings);
+    FIXME("(%p, %08lx, %08lx): stub.\n", hLine, dwAddressID, dwNumRings);
     return 0;
 }
 
@@ -1485,7 +1521,7 @@ DWORD WINAPI lineSetNumRings(HLINE hLine, DWORD dwAddressID, DWORD dwNumRings)
  */
 DWORD WINAPI lineSetStatusMessages(HLINE hLine, DWORD dwLineStates, DWORD dwAddressStates)
 {
-    FIXME("(%p, %08x, %08x): stub.\n", hLine, dwLineStates, dwAddressStates);
+    FIXME("(%p, %08lx, %08lx): stub.\n", hLine, dwLineStates, dwAddressStates);
     return 0;
 }
 
@@ -1494,7 +1530,7 @@ DWORD WINAPI lineSetStatusMessages(HLINE hLine, DWORD dwLineStates, DWORD dwAddr
  */
 DWORD WINAPI lineSetTerminal(HLINE hLine, DWORD dwAddressID, HCALL hCall, DWORD dwSelect, DWORD dwTerminalModes, DWORD dwTerminalID, DWORD bEnable)
 {
-    FIXME("(%p, %08x, %p, %08x, %08x, %08x, %08x): stub.\n", hLine, dwAddressID, hCall, dwSelect, dwTerminalModes, dwTerminalID, bEnable);
+    FIXME("(%p, %08lx, %p, %08lx, %08lx, %08lx, %08lx): stub.\n", hLine, dwAddressID, hCall, dwSelect, dwTerminalModes, dwTerminalID, bEnable);
     return 1;
 }
 
@@ -1503,7 +1539,7 @@ DWORD WINAPI lineSetTerminal(HLINE hLine, DWORD dwAddressID, HCALL hCall, DWORD
  */
 DWORD WINAPI lineSetTollListA(HLINEAPP hLineApp, DWORD dwDeviceID, LPCSTR lpszAddressIn, DWORD dwTollListOption)
 {
-    FIXME("(%p, %08x, %s, %08x): stub.\n", hLineApp, dwDeviceID, lpszAddressIn, dwTollListOption);
+    FIXME("(%p, %08lx, %s, %08lx): stub.\n", hLineApp, dwDeviceID, lpszAddressIn, dwTollListOption);
     return 0;
 }
 
@@ -1512,7 +1548,7 @@ DWORD WINAPI lineSetTollListA(HLINEAPP hLineApp, DWORD dwDeviceID, LPCSTR lpszAd
  */
 DWORD WINAPI lineSetupConferenceA(HCALL hCall, HLINE hLine, LPHCALL lphConfCall, LPHCALL lphConsultCall, DWORD dwNumParties, LPLINECALLPARAMS lpCallParams)
 {
-    FIXME("(%p, %p, %p, %p, %08x, %p): stub.\n", hCall, hLine, lphConfCall, lphConsultCall, dwNumParties, lpCallParams);
+    FIXME("(%p, %p, %p, %p, %08lx, %p): stub.\n", hCall, hLine, lphConfCall, lphConsultCall, dwNumParties, lpCallParams);
     return 1;
 }
 
@@ -1548,7 +1584,7 @@ DWORD WINAPI lineSwapHold(HCALL hActiveCall, HCALL hHeldCall)
  */
 DWORD WINAPI lineTranslateAddressA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAPIVersion, LPCSTR lpszAddressIn, DWORD dwCard, DWORD dwTranslateOptions, LPLINETRANSLATEOUTPUT lpTranslateOutput)
 {
-    FIXME("(%p, %08x, %08x, %s, %08x, %08x, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion, lpszAddressIn, dwCard, dwTranslateOptions, lpTranslateOutput);
+    FIXME("(%p, %08lx, %08lx, %s, %08lx, %08lx, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion, lpszAddressIn, dwCard, dwTranslateOptions, lpTranslateOutput);
     return 0;
 }
 
@@ -1559,7 +1595,7 @@ DWORD WINAPI lineTranslateAddressW(HLINEAPP hLineApp, DWORD dwDeviceID,
         DWORD dwAPIVersion, LPCWSTR lpszAddressIn, DWORD dwCard,
         DWORD dwTranslateOptions, LPLINETRANSLATEOUTPUT lpTranslateOutput)
 {
-    FIXME("(%p, %08x, %08x, %s, %08x, %08x, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
+    FIXME("(%p, %08lx, %08lx, %s, %08lx, %08lx, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
             debugstr_w(lpszAddressIn), dwCard, dwTranslateOptions, lpTranslateOutput);
     return 0;
 }
@@ -1569,7 +1605,7 @@ DWORD WINAPI lineTranslateAddressW(HLINEAPP hLineApp, DWORD dwDeviceID,
  */
 DWORD WINAPI lineTranslateDialogA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwAPIVersion, HWND hwndOwner, LPCSTR lpszAddressIn)
 {
-    FIXME("(%p, %08x, %08x, %p, %s): stub.\n", hLineApp, dwDeviceID, dwAPIVersion, hwndOwner, lpszAddressIn);
+    FIXME("(%p, %08lx, %08lx, %p, %s): stub.\n", hLineApp, dwDeviceID, dwAPIVersion, hwndOwner, lpszAddressIn);
     return 0;
 }
 
@@ -1579,7 +1615,7 @@ DWORD WINAPI lineTranslateDialogA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwA
 DWORD WINAPI lineTranslateDialogW(HLINEAPP hLineApp, DWORD dwDeviceID,
         DWORD dwAPIVersion, HWND hwndOwner, LPCWSTR lpszAddressIn)
 {
-    FIXME("(%p, %08x, %08x, %p, %s): stub.\n", hLineApp, dwDeviceID,
+    FIXME("(%p, %08lx, %08lx, %p, %s): stub.\n", hLineApp, dwDeviceID,
             dwAPIVersion, hwndOwner, debugstr_w(lpszAddressIn));
     return 0;
 }
@@ -1589,7 +1625,7 @@ DWORD WINAPI lineTranslateDialogW(HLINEAPP hLineApp, DWORD dwDeviceID,
  */
 DWORD WINAPI lineUncompleteCall(HLINE hLine, DWORD dwCompletionID)
 {
-    FIXME("(%p, %08x): stub.\n", hLine, dwCompletionID);
+    FIXME("(%p, %08lx): stub.\n", hLine, dwCompletionID);
     return 1;
 }
 
@@ -1607,6 +1643,6 @@ DWORD WINAPI lineUnhold(HCALL hCall)
  */
 DWORD WINAPI lineUnparkA(HLINE hLine, DWORD dwAddressID, LPHCALL lphCall, LPCSTR lpszDestAddress)
 {
-    FIXME("(%p, %08x, %p, %s): stub.\n", hLine, dwAddressID, lphCall, lpszDestAddress);
+    FIXME("(%p, %08lx, %p, %s): stub.\n", hLine, dwAddressID, lphCall, lpszDestAddress);
     return 1;
 }
