@@ -60,10 +60,13 @@ static BOOL check_win_version(int min_major, int min_minor)
             rtlver.dwMinorVersion >= min_minor);
 }
 #define is_win8_plus() check_win_version(6, 2)
+#define is_win10_plus() check_win_version(10, 0)
 
 static void test_Connect(void)
 {
     WCHAR comp_name[MAX_COMPUTERNAME_LENGTH + 1];
+    WCHAR user_name[256];
+    WCHAR domain_name[256];
     DWORD len;
     HRESULT hr;
     BSTR bstr;
@@ -93,6 +96,18 @@ static void test_Connect(void)
     hr = ITaskService_get_TargetServer(service, &bstr);
     ok(hr == HRESULT_FROM_WIN32(ERROR_ONLY_IF_CONNECTED), "expected ERROR_ONLY_IF_CONNECTED, got %#lx\n", hr);
 
+    hr = ITaskService_get_ConnectedUser(service, NULL);
+    ok(hr == E_POINTER, "expected E_POINTER, got %#lx\n", hr);
+
+    hr = ITaskService_get_ConnectedUser(service, &bstr);
+    ok(hr == HRESULT_FROM_WIN32(ERROR_ONLY_IF_CONNECTED), "expected ERROR_ONLY_IF_CONNECTED, got %#lx\n", hr);
+
+    hr = ITaskService_get_ConnectedDomain(service, NULL);
+    ok(hr == E_POINTER, "expected E_POINTER, got %#lx\n", hr);
+
+    hr = ITaskService_get_ConnectedDomain(service, &bstr);
+    ok(hr == HRESULT_FROM_WIN32(ERROR_ONLY_IF_CONNECTED), "expected ERROR_ONLY_IF_CONNECTED, got %#lx\n", hr);
+
     /* Win7 doesn't support UNC \\ prefix, but according to a user
      * comment on MSDN Win8 supports both ways.
      */
@@ -144,6 +159,27 @@ static void test_Connect(void)
     ok(!lstrcmpW(comp_name, bstr), "compname %s != server name %s\n", wine_dbgstr_w(comp_name), wine_dbgstr_w(bstr));
     SysFreeString(bstr);
 
+    len = ARRAY_SIZE(user_name);
+    GetUserNameW(user_name, &len);
+
+    hr = ITaskService_get_ConnectedUser(service, &bstr);
+    ok(hr == S_OK, "get_ConnectedUser error %#lx\n", hr);
+    ok(!lstrcmpW(user_name, bstr), "username %s != user name %s\n", wine_dbgstr_w(user_name), wine_dbgstr_w(bstr));
+    SysFreeString(bstr);
+
+    len = ARRAY_SIZE(domain_name);
+    if (!GetEnvironmentVariableW(L"USERDOMAIN", domain_name, len))
+    {
+         GetComputerNameExW(ComputerNameDnsHostname, domain_name, &len);
+         if (is_win10_plus())
+             wcsupr(domain_name);
+    }
+
+    hr = ITaskService_get_ConnectedDomain(service, &bstr);
+    ok(hr == S_OK, "get_ConnectedDomain error %#lx\n", hr);
+    ok(!lstrcmpW(domain_name, bstr), "domainname %s != domain name %s\n", wine_dbgstr_w(domain_name), wine_dbgstr_w(bstr));
+    SysFreeString(bstr);
+
     ITaskService_Release(service);
 }
 
@@ -1283,19 +1319,20 @@ static void test_daily_trigger(ITrigger *trigger)
     static const struct
     {
         const WCHAR *str;
+        const WCHAR *end;
         HRESULT      hr;
     }
     start_test[] =
     {
-        {L"2004-01-01T00:00:00", S_OK},
-        {L"2004-01-01T00:00:00Z", S_OK},
-        {L"2004-01-01T00:00:00+01:00", S_OK},
-        {L"2004.01.01T00.00.00", S_OK},
-        {L"9999-99-99T99:99:99", S_OK},
-        {L"invalid", S_OK},
+        {L"2004-01-01T00:00:00", L"2004-01-02T00:00:00", S_OK},
+        {L"2004-01-01T00:00:00Z", L"2004-01-02T00:00:00Z", S_OK},
+        {L"2004-01-01T00:00:00+01:00", L"2004-01-02T00:00:00+01:00", S_OK},
+        {L"2004.01.01T00.00.00", L"2004.01.02T00.00.00", S_OK},
+        {L"9999-99-99T99:99:99", L"9999-99-99T99:99:99", S_OK},
+        {L"invalid", L"invalid", S_OK},
     };
     IDailyTrigger *daily_trigger;
-    BSTR start_boundary;
+    BSTR start_boundary, end_boundary;
     VARIANT_BOOL enabled;
     short interval;
     HRESULT hr;
@@ -1330,11 +1367,19 @@ static void test_daily_trigger(ITrigger *trigger)
     hr = IDailyTrigger_get_StartBoundary(daily_trigger, NULL);
     ok(hr == E_POINTER, "get_StartBoundary failed: %08lx\n", hr);
 
+    hr = IDailyTrigger_get_EndBoundary(daily_trigger, NULL);
+    ok(hr == E_POINTER, "get_EndBoundary failed: %08lx\n", hr);
+
     start_boundary = (BSTR)0xdeadbeef;
     hr = IDailyTrigger_get_StartBoundary(daily_trigger, &start_boundary);
     ok(hr == S_OK, "get_StartBoundary failed: %08lx\n", hr);
     ok(start_boundary == NULL, "start_boundary not set\n");
 
+    end_boundary = (BSTR)0xdeadbeef;
+    hr = IDailyTrigger_get_EndBoundary(daily_trigger, &end_boundary);
+    ok(hr == S_OK, "get_EndBoundary failed: %08lx\n", hr);
+    ok(end_boundary == NULL, "end_boundary not set\n");
+
     for (i = 0; i < ARRAY_SIZE(start_test); i++)
     {
         winetest_push_context("%lu", i);
@@ -1351,12 +1396,29 @@ static void test_daily_trigger(ITrigger *trigger)
             ok(!lstrcmpW(start_boundary, start_test[i].str), "got %s\n", wine_dbgstr_w(start_boundary));
             SysFreeString(start_boundary);
         }
+
+        end_boundary = SysAllocString(start_test[i].end);
+        hr = IDailyTrigger_put_EndBoundary(daily_trigger, end_boundary);
+        ok(hr == start_test[i].hr, "got %08lx expected %08lx\n", hr, start_test[i].hr);
+        SysFreeString(end_boundary);
+        if (hr == S_OK)
+        {
+            end_boundary = NULL;
+            hr = IDailyTrigger_get_EndBoundary(daily_trigger, &end_boundary);
+            ok(hr == S_OK, "got %08lx\n", hr);
+            ok(end_boundary != NULL, "end_boundary not set\n");
+            ok(!lstrcmpW(end_boundary, start_test[i].end), "got %s\n", wine_dbgstr_w(end_boundary));
+            SysFreeString(end_boundary);
+        }
         winetest_pop_context();
     }
 
     hr = IDailyTrigger_put_StartBoundary(daily_trigger, NULL);
     ok(hr == S_OK, "put_StartBoundary failed: %08lx\n", hr);
 
+    hr = IDailyTrigger_put_EndBoundary(daily_trigger, NULL);
+    ok(hr == S_OK, "put_EndBoundary failed: %08lx\n", hr);
+
     hr = IDailyTrigger_get_Enabled(daily_trigger, NULL);
     ok(hr == E_POINTER, "get_Enabled failed: %08lx\n", hr);
 
@@ -1376,6 +1438,31 @@ static void test_daily_trigger(ITrigger *trigger)
     IDailyTrigger_Release(daily_trigger);
 }
 
+static void test_registration_trigger(ITrigger *trigger)
+{
+    IRegistrationTrigger *reg_trigger;
+    VARIANT_BOOL enabled;
+    HRESULT hr;
+
+    hr = ITrigger_QueryInterface(trigger, &IID_IRegistrationTrigger, (void**)&reg_trigger);
+    ok(hr == S_OK, "Could not get IRegistrationTrigger iface: %08lx\n", hr);
+
+    enabled = VARIANT_FALSE;
+    hr = IRegistrationTrigger_get_Enabled(reg_trigger, &enabled);
+    ok(hr == S_OK, "get_Enabled failed: %08lx\n", hr);
+    ok(enabled == VARIANT_TRUE, "got %d\n", enabled);
+
+    hr = IRegistrationTrigger_put_Enabled(reg_trigger, VARIANT_FALSE);
+    ok(hr == S_OK, "put_Enabled failed: %08lx\n", hr);
+
+    enabled = VARIANT_TRUE;
+    hr = IRegistrationTrigger_get_Enabled(reg_trigger, &enabled);
+    ok(hr == S_OK, "get_Enabled failed: %08lx\n", hr);
+    ok(enabled == VARIANT_FALSE, "got %d\n", enabled);
+
+    IRegistrationTrigger_Release(reg_trigger);
+}
+
 static void create_action(ITaskDefinition *taskdef)
 {
     static WCHAR task1_exe[] = L"task1.exe";
@@ -1774,6 +1861,12 @@ static void test_TaskDefinition(void)
     ok(trigger != NULL, "trigger = NULL\n");
     test_daily_trigger(trigger);
     ITrigger_Release(trigger);
+
+    hr = ITriggerCollection_Create(trigger_col, TASK_TRIGGER_REGISTRATION, &trigger);
+    ok(hr == S_OK, "Create failed: %08lx\n", hr);
+    ok(trigger != NULL, "trigger = NULL\n");
+    test_registration_trigger(trigger);
+    ITrigger_Release(trigger);
     ITriggerCollection_Release(trigger_col);
 
     hr = ITaskDefinition_get_Triggers(taskdef, &trigger_col2);
