@@ -37,15 +37,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(threadpool);
 
-static const char *debugstr_hstring(HSTRING hstr)
-{
-    const WCHAR *str;
-    UINT32 len;
-    if (hstr && !((ULONG_PTR)hstr >> 16)) return "(invalid)";
-    str = WindowsGetStringRawBuffer(hstr, &len);
-    return wine_dbgstr_wn(str, len);
-}
-
 struct threadpool_factory
 {
     IActivationFactory IActivationFactory_iface;
@@ -100,6 +91,7 @@ static HRESULT STDMETHODCALLTYPE async_action_QueryInterface(IAsyncAction *iface
     {
         *out = NULL;
         WARN("Unsupported interface %s.\n", debugstr_guid(iid));
+        return E_NOINTERFACE;
     }
 
     IUnknown_AddRef((IUnknown *)*out);
@@ -202,7 +194,7 @@ static ULONG STDMETHODCALLTYPE async_info_AddRef(IAsyncInfo *iface)
 static ULONG STDMETHODCALLTYPE async_info_Release(IAsyncInfo *iface)
 {
     struct async_action *action = impl_from_IAsyncInfo(iface);
-    return IAsyncAction_AddRef(&action->IAsyncAction_iface);
+    return IAsyncAction_Release(&action->IAsyncAction_iface);
 }
 
 static HRESULT STDMETHODCALLTYPE async_info_GetIids(IAsyncInfo *iface, ULONG *iid_count, IID **iids)
@@ -343,7 +335,7 @@ static DWORD WINAPI sliced_thread_proc(void *arg)
 struct thread_pool
 {
     INIT_ONCE init_once;
-    TP_POOL *pool;
+    TP_CALLBACK_ENVIRON environment;
 };
 
 static struct thread_pool pools[3];
@@ -352,9 +344,12 @@ static BOOL CALLBACK pool_init_once(INIT_ONCE *init_once, void *param, void **co
 {
     struct thread_pool *pool = param;
 
-    if (!(pool->pool = CreateThreadpool(NULL))) return FALSE;
+    memset(&pool->environment, 0, sizeof(pool->environment));
+    pool->environment.Version = 1;
 
-    SetThreadpoolThreadMaximum(pool->pool, 10);
+    if (!(pool->environment.Pool = CreateThreadpool(NULL))) return FALSE;
+
+    SetThreadpoolThreadMaximum(pool->environment.Pool, 10);
 
     return TRUE;
 }
@@ -379,7 +374,7 @@ static HRESULT submit_threadpool_work(struct work_item *item, WorkItemPriority p
     if (!InitOnceExecuteOnce(&pool->init_once, pool_init_once, pool, NULL))
         return E_FAIL;
 
-    if (!(work = CreateThreadpoolWork(pool_work_callback, item, NULL)))
+    if (!(work = CreateThreadpoolWork(pool_work_callback, item, &pool->environment)))
         return E_FAIL;
 
     IAsyncAction_AddRef((*action = item->action));
