@@ -74,17 +74,17 @@ static void __cdecl test_invalid_parameter_handler(const wchar_t *expression,
     ok(function == NULL, "function is not NULL\n");
     ok(file == NULL, "file is not NULL\n");
     ok(line == 0, "line = %u\n", line);
-    ok(arg == 0, "arg = %lx\n", (UINT_PTR)arg);
+    ok(arg == 0, "arg = %Ix\n", arg);
 }
 
 static int WINAPIV vsprintf_wrapper(unsigned __int64 options, char *str,
                                     size_t len, const char *format, ...)
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start(valist, format);
+    va_list valist;
+    va_start(valist, format);
     ret = __stdio_common_vsprintf(options, str, len, format, NULL, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     return ret;
 }
 
@@ -165,10 +165,10 @@ static int WINAPIV vswprintf_wrapper(unsigned __int64 options, wchar_t *str,
                                      size_t len, const wchar_t *format, ...)
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start(valist, format);
+    va_list valist;
+    va_start(valist, format);
     ret = __stdio_common_vswprintf(options, str, len, format, NULL, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     return ret;
 }
 
@@ -259,10 +259,10 @@ static int WINAPIV vfprintf_wrapper(FILE *file,
                                     const char *format, ...)
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start(valist, format);
+    va_list valist;
+    va_start(valist, format);
     ret = __stdio_common_vfprintf(0, file, format, NULL, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     return ret;
 }
 
@@ -332,10 +332,10 @@ static int WINAPIV vfwprintf_wrapper(FILE *file,
                                      const wchar_t *format, ...)
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start(valist, format);
+    va_list valist;
+    va_start(valist, format);
     ret = __stdio_common_vfwprintf(0, file, format, NULL, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     return ret;
 }
 
@@ -429,10 +429,10 @@ static int WINAPIV _vsnprintf_s_wrapper(char *str, size_t sizeOfBuffer,
                                         size_t count, const char *format, ...)
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start(valist, format);
+    va_list valist;
+    va_start(valist, format);
     ret = __stdio_common_vsnprintf_s(0, str, sizeOfBuffer, count, format, NULL, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     return ret;
 }
 
@@ -481,10 +481,10 @@ static int WINAPIV _vsnwprintf_s_wrapper(WCHAR *str, size_t sizeOfBuffer,
                                         size_t count, const WCHAR *format, ...)
 {
     int ret;
-    __ms_va_list valist;
-    __ms_va_start(valist, format);
+    va_list valist;
+    va_start(valist, format);
     ret = __stdio_common_vsnwprintf_s(0, str, sizeOfBuffer, count, format, NULL, valist);
-    __ms_va_end(valist);
+    va_end(valist);
     return ret;
 }
 
@@ -815,11 +815,30 @@ static void test_printf_fp(void)
         { "%#.20g", -765, { "-765.00000000000000000" }},
         { "%.30f", 1.0/3.0, { "0.333333333333333314829616256247" }},
         { "%.30lf", sqrt(2), { "1.414213562373095145474621858739" }},
+        { "%f", 3.141592653590000, { "3.141593" }},
+        { "%.10f", 3.141592653590000, { "3.1415926536" }},
+        { "%.11f", 3.141592653590000, { "3.14159265359" }},
+        { "%.15f", 3.141592653590000, { "3.141592653590000" }},
+        { "%.15f", M_PI, { "3.141592653589793" }},
+        { "%.13f", 37.866261574537077, { "37.8662615745371" }},
+        { "%.14f", 37.866261574537077, { "37.86626157453708" }},
+        { "%.15f", 37.866261574537077, { "37.866261574537077" }},
         { "%.0g", 9.8949714229143402e-05, { "0.0001" }},
         { "%.0f", 0.5, { "1", NULL, NULL, NULL, "0" }, {NULL, NULL, NULL, NULL, "1" }},
         { "%.0f", 1.5, { "2" }},
         { "%.0f", 2.5, { "3", NULL, NULL, NULL, "2" }, {NULL, NULL, NULL, NULL, "3" }},
         { "%g", 9.999999999999999e-5, { "0.0001" }},
+
+        { "%g", 0.0005, { "0.0005" }},
+        { "%g", 0.00005, { "5e-05", NULL, "5e-005" }},
+        { "%g", 0.000005, { "5e-06", NULL, "5e-006" }},
+        { "%g", 999999999999999.0, { "1e+15", NULL, "1e+015" }},
+        { "%g", 1000000000000000.0, { "1e+15", NULL, "1e+015" }},
+        { "%.15g", 0.0005, { "0.0005" }},
+        { "%.15g", 0.00005, { "5e-05", NULL, "5e-005" }},
+        { "%.15g", 0.000005, { "5e-06", NULL, "5e-006" }},
+        { "%.15g", 999999999999999.0, { "999999999999999" }},
+        { "%.15g", 1000000000000000.0, { "1e+15", NULL, "1e+015" }},
     };
 
     const char *res = NULL;
@@ -838,13 +857,31 @@ static void test_printf_fp(void)
 
             r = vsprintf_wrapper(flags[j], buf, sizeof(buf), tests[i].fmt, tests[i].d);
             ok(r == strlen(res) || broken(broken_res && r == strlen(broken_res)),
-                    "%d,%d) r = %d, expected %d\n", i, j, r, strlen(res));
+                    "%d,%d) r = %d, expected %Id\n", i, j, r, strlen(res));
             ok(!strcmp(buf, res) || broken(broken_res && !strcmp(buf, broken_res)),
                     "%d,%d) buf = %s, expected %s\n", i, j, buf, res);
         }
     }
 }
 
+static void test_printf_width_specification(void)
+{
+    int r;
+    char buffer[20];
+
+    r = vsprintf_wrapper(0, buffer, sizeof(buffer), "%0*2d", 1, 3);
+    ok(r == 2, "r = %d\n", r);
+    ok(!strcmp(buffer, "03"), "buffer wrong, got=%s\n", buffer);
+
+    r = vsprintf_wrapper(0, buffer, sizeof(buffer), "%*0d", 1, 2);
+    ok(r == 1, "r = %d\n", r);
+    ok(!strcmp(buffer, "2"), "buffer wrong, got=%s\n", buffer);
+
+    r = vsprintf_wrapper(0, buffer, sizeof(buffer), "% *2d", 0, 7);
+    ok(r == 2, "r = %d\n", r);
+    ok(!strcmp(buffer, " 7"), "buffer wrong, got=%s\n", buffer);
+}
+
 START_TEST(printf)
 {
     ok(_set_invalid_parameter_handler(test_invalid_parameter_handler) == NULL,
@@ -862,4 +899,5 @@ START_TEST(printf)
     test_printf_c99();
     test_printf_natural_string();
     test_printf_fp();
+    test_printf_width_specification();
 }
