@@ -22,6 +22,8 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "wine/winuser16.h"
 #include "wownt32.h"
 #include "winerror.h"
@@ -2581,28 +2583,18 @@ HWND create_window16( CREATESTRUCTW *cs, LPCWSTR className, HINSTANCE instance,
 }
 
 
-static void WINAPI User16CallFreeIcon( ULONG *param, ULONG size )
+static NTSTATUS WINAPI thunk_lock_callback( void *args, ULONG size )
 {
-    GlobalFree16( LOWORD(*param) );
-}
-
-
-static DWORD WINAPI User16ThunkLock( DWORD *param, ULONG size )
-{
-    if (size != sizeof(DWORD))
-    {
-        DWORD lock;
-        ReleaseThunkLock( &lock );
-        return lock;
-    }
-    RestoreThunkLock( *param );
-    return 0;
+    const struct thunk_lock_params *params = args;
+    DWORD locks = params->locks;
+    if (params->restore) RestoreThunkLock( locks );
+    else ReleaseThunkLock( &locks );
+    return NtCallbackReturn( &locks, sizeof(locks), STATUS_SUCCESS );
 }
 
 
 void register_wow_handlers(void)
 {
-    void **callback_table = NtCurrentTeb()->Peb->KernelCallbackTable;
     static const struct wow_handlers16 handlers16 =
     {
         button_proc16,
@@ -2617,10 +2609,7 @@ void register_wow_handlers(void)
         call_dialog_proc_Ato16,
     };
 
-    callback_table[NtUserCallFreeIcon] = User16CallFreeIcon;
-    callback_table[NtUserThunkLock]    = User16ThunkLock;
-
-    NtUserEnableThunkLock( TRUE );
+    NtUserEnableThunkLock( thunk_lock_callback );
 
     UserRegisterWowHandlers( &handlers16, &wow_handlers32 );
 }
