@@ -24,7 +24,6 @@
 #include "user_private.h"
 #include "controls.h"
 #include "dde.h"
-#include "wine/server.h"
 #include "wine/debug.h"
 #include "wine/exception.h"
 
@@ -333,10 +332,9 @@ static HGLOBAL dde_get_pair(HGLOBAL shm)
  *
  * Post a DDE message
  */
-BOOL post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD dest_tid, DWORD type )
+NTSTATUS post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD dest_tid )
 {
-    void*       ptr = NULL;
-    int         size = 0;
+    struct post_dde_message_call_params params = { .dest_tid = dest_tid };
     UINT_PTR    uiLo, uiHi;
     LPARAM      lp;
     HGLOBAL     hunlock = 0;
@@ -344,7 +342,7 @@ BOOL post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD
     ULONGLONG   hpack;
 
     if (!UnpackDDElParam( msg, lparam, &uiLo, &uiHi ))
-        return FALSE;
+        return STATUS_INVALID_PARAMETER;
 
     lp = lparam;
     switch (msg)
@@ -364,8 +362,8 @@ BOOL post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD
             {
                 hpack = pack_ptr( h );
                 /* send back the value of h on the other side */
-                ptr = &hpack;
-                size = sizeof(hpack);
+                params.ptr = &hpack;
+                params.size = sizeof(hpack);
                 lp = uiLo;
                 TRACE( "send dde-ack %Ix %08Ix => %p\n", uiLo, uiHi, h );
             }
@@ -382,34 +380,34 @@ BOOL post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD
     case WM_DDE_POKE:
         if (uiLo)
         {
-            size = GlobalSize( (HGLOBAL)uiLo ) ;
-            if ((msg == WM_DDE_ADVISE && size < sizeof(DDEADVISE)) ||
-                (msg == WM_DDE_DATA   && size < FIELD_OFFSET(DDEDATA, Value)) ||
-                (msg == WM_DDE_POKE   && size < FIELD_OFFSET(DDEPOKE, Value)))
-                return FALSE;
+            params.size = GlobalSize( (HGLOBAL)uiLo ) ;
+            if ((msg == WM_DDE_ADVISE && params.size < sizeof(DDEADVISE)) ||
+                (msg == WM_DDE_DATA   && params.size < FIELD_OFFSET(DDEDATA, Value)) ||
+                (msg == WM_DDE_POKE   && params.size < FIELD_OFFSET(DDEPOKE, Value)))
+                return STATUS_INVALID_PARAMETER;
         }
-        else if (msg != WM_DDE_DATA) return FALSE;
+        else if (msg != WM_DDE_DATA) return STATUS_INVALID_PARAMETER;
 
         lp = uiHi;
         if (uiLo)
         {
-            if ((ptr = GlobalLock( (HGLOBAL)uiLo) ))
+            if ((params.ptr = GlobalLock( (HGLOBAL)uiLo) ))
             {
-                DDEDATA *dde_data = ptr;
+                DDEDATA *dde_data = params.ptr;
                 TRACE("unused %d, fResponse %d, fRelease %d, fDeferUpd %d, fAckReq %d, cfFormat %d\n",
                        dde_data->unused, dde_data->fResponse, dde_data->fRelease,
                        dde_data->reserved, dde_data->fAckReq, dde_data->cfFormat);
                 hunlock = (HGLOBAL)uiLo;
             }
         }
-        TRACE( "send ddepack %u %Ix\n", size, uiHi );
+        TRACE( "send ddepack %u %Ix\n", params.size, uiHi );
         break;
     case WM_DDE_EXECUTE:
         if (lparam)
         {
-            if ((ptr = GlobalLock( (HGLOBAL)lparam) ))
+            if ((params.ptr = GlobalLock( (HGLOBAL)lparam) ))
             {
-                size = GlobalSize( (HGLOBAL)lparam );
+                params.size = GlobalSize( (HGLOBAL)lparam );
                 /* so that the other side can send it back on ACK */
                 lp = lparam;
                 hunlock = (HGLOBAL)lparam;
@@ -417,32 +415,12 @@ BOOL post_dde_message( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, DWORD
         }
         break;
     }
-    SERVER_START_REQ( send_message )
-    {
-        req->id      = dest_tid;
-        req->type    = type;
-        req->flags   = 0;
-        req->win     = wine_server_user_handle( hwnd );
-        req->msg     = msg;
-        req->wparam  = wparam;
-        req->lparam  = lp;
-        req->timeout = TIMEOUT_INFINITE;
-        if (size) wine_server_add_data( req, ptr, size );
-        if ((res = wine_server_call( req )))
-        {
-            if (res == STATUS_INVALID_PARAMETER)
-                /* FIXME: find a STATUS_ value for this one */
-                SetLastError( ERROR_INVALID_THREAD_ID );
-            else
-                SetLastError( RtlNtStatusToDosError(res) );
-        }
-        else
-            FreeDDElParam( msg, lparam );
-    }
-    SERVER_END_REQ;
+
+    res = NtUserMessageCall( hwnd, msg, wparam, lp, &params, NtUserPostDdeCall, FALSE );
+    if (!res) FreeDDElParam( msg, lparam );
     if (hunlock) GlobalUnlock(hunlock);
 
-    return !res;
+    return res;
 }
 
 /***********************************************************************
@@ -755,12 +733,7 @@ BOOL WINAPI PostThreadMessageA( DWORD thread, UINT msg, WPARAM wparam, LPARAM lp
  */
 void WINAPI PostQuitMessage( INT exit_code )
 {
-    SERVER_START_REQ( post_quit_message )
-    {
-        req->exit_code = exit_code;
-        wine_server_call( req );
-    }
-    SERVER_END_REQ;
+    NtUserPostQuitMessage( exit_code );
 }
 
 /***********************************************************************
@@ -1314,15 +1287,7 @@ BOOL WINAPI IsGUIThread( BOOL convert )
  */
 BOOL WINAPI IsHungAppWindow( HWND hWnd )
 {
-    BOOL ret;
-
-    SERVER_START_REQ( is_window_hung )
-    {
-        req->win = wine_server_user_handle( hWnd );
-        ret = !wine_server_call_err( req ) && reply->is_hung;
-    }
-    SERVER_END_REQ;
-    return ret;
+    return HandleToUlong( NtUserQueryWindow( hWnd, WindowIsHung ));
 }
 
 /******************************************************************
