@@ -384,9 +384,9 @@ static void TEXT_WordBreak (HDC hdc, WCHAR *str, unsigned int max_str,
     }
     else
     {
-        while (i > 0 && !sla[(--i)+1].fSoftBreak) p--;
+        while (i > 0 && !sla[(--i)+1].fWhiteSpace) p--;
         p--;
-        word_fits = (i != 0 || sla[i+1].fSoftBreak );
+        word_fits = (i != 0 || sla[i+1].fWhiteSpace );
     }
 
     /* If there was one. */
@@ -874,8 +874,7 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
     int len, lh, count=i_count;
     TEXTMETRICW tm;
     int lmargin = 0, rmargin = 0;
-    int x = rect->left, y = rect->top;
-    int width = rect->right - rect->left;
+    int x, y, width;
     int max_width = 0;
     int last_line;
     int tabwidth /* to keep gcc happy */ = 0;
@@ -897,7 +896,13 @@ INT WINAPI DrawTextExW( HDC hdc, LPWSTR str, INT i_count,
     if (flags & DT_SINGLELINE)
         flags &= ~DT_WORDBREAK;
 
-    GetTextMetricsW(hdc, &tm);
+    if (!GetTextMetricsW(hdc, &tm))
+        return 0;
+
+    x = rect->left;
+    y = rect->top;
+    width = rect->right - rect->left;
+
     if (flags & DT_EXTERNALLEADING)
 	lh = tm.tmHeight + tm.tmExternalLeading;
     else
@@ -1086,18 +1091,24 @@ INT WINAPI DrawTextExA( HDC hdc, LPSTR str, INT count,
    DWORD wmax;
    DWORD amax;
    UINT cp;
+   TEXTMETRICA tm;
+
+   if (!GetTextMetricsA(hdc, &tm))
+   {
+       SetLastError(ERROR_INVALID_HANDLE);
+       return 0;
+   }
 
    if (!count) return 0;
    if (!str && count > 0) return 0;
    if( !str || ((count == -1) && !(count = strlen(str))))
    {
         int lh;
-        TEXTMETRICA tm;
 
         if (dtp && dtp->cbSize != sizeof(DRAWTEXTPARAMS))
             return 0;
 
-        GetTextMetricsA(hdc, &tm);
+
         if (flags & DT_EXTERNALLEADING)
             lh = tm.tmHeight + tm.tmExternalLeading;
         else
