@@ -50,8 +50,6 @@ static GDI_HANDLE_ENTRY *next_unused;
 static LONG debug_count;
 SYSTEM_BASIC_INFORMATION system_info;
 
-const struct user_callbacks *user_callbacks = NULL;
-
 static inline HGDIOBJ entry_to_handle( GDI_HANDLE_ENTRY *entry )
 {
     unsigned int idx = entry - gdi_shared->Handles;
@@ -464,40 +462,22 @@ void make_gdi_object_system( HGDIOBJ handle, BOOL set)
 /******************************************************************************
  *      get_default_fonts
  */
-static const struct DefaultFontInfo* get_default_fonts(UINT charset)
+static const struct DefaultFontInfo* get_default_fonts(void)
 {
-        unsigned int n;
+    unsigned int n;
+    CHARSETINFO csi;
 
-        for(n = 0; n < ARRAY_SIZE( default_fonts ); n++)
-        {
-                if ( default_fonts[n].charset == charset )
-                        return &default_fonts[n];
-        }
+    if (ansi_cp.CodePage == CP_UTF8) return &default_fonts[0];
 
-        FIXME( "unhandled charset 0x%08x - use ANSI_CHARSET for default stock objects\n", charset );
-        return &default_fonts[0];
-}
-
-
-/******************************************************************************
- *      get_default_charset    (internal)
- *
- * get the language-dependent charset that can handle CP_ACP correctly.
- */
-static UINT get_default_charset( void )
-{
-    CHARSETINFO     csi;
-    UINT    uACP;
-
-    uACP = get_acp();
     csi.ciCharset = ANSI_CHARSET;
-    if ( !translate_charset_info( ULongToPtr(uACP), &csi, TCI_SRCCODEPAGE ) )
-    {
-        FIXME( "unhandled codepage %u - use ANSI_CHARSET for default stock objects\n", uACP );
-        return ANSI_CHARSET;
-    }
+    translate_charset_info( ULongToPtr(ansi_cp.CodePage), &csi, TCI_SRCCODEPAGE );
 
-    return csi.ciCharset;
+    for(n = 0; n < ARRAY_SIZE( default_fonts ); n++)
+        if ( default_fonts[n].charset == csi.ciCharset )
+            return &default_fonts[n];
+
+    FIXME( "unhandled charset 0x%08x - use ANSI_CHARSET for default stock objects\n", csi.ciCharset );
+    return &default_fonts[0];
 }
 
 
@@ -586,8 +566,8 @@ static void init_gdi_shared(void)
 {
     SIZE_T size = sizeof(*gdi_shared);
 
-    if (NtAllocateVirtualMemory( GetCurrentProcess(), (void **)&gdi_shared, 0, &size,
-                                 MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE ))
+    if (NtAllocateVirtualMemory( GetCurrentProcess(), (void **)&gdi_shared, zero_bits,
+                                 &size, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE ))
         return;
     next_unused = gdi_shared->Handles + FIRST_GDI_HANDLE;
 
@@ -604,7 +584,10 @@ static void init_gdi_shared(void)
     NtCurrentTeb()->Peb->GdiSharedHandleTable = gdi_shared;
 }
 
-HGDIOBJ get_stock_object( INT obj )
+/***********************************************************************
+ *           GetStockObject    (win32u.so)
+ */
+HGDIOBJ WINAPI GetStockObject( INT obj )
 {
     assert( obj >= 0 && obj <= STOCK_LAST + 1 && obj != 9 );
 
@@ -656,7 +639,7 @@ static void init_stock_objects( unsigned int dpi )
     create_font( &AnsiVarFont );
 
     /* language-dependent stock fonts */
-    deffonts = get_default_fonts(get_default_charset());
+    deffonts = get_default_fonts();
     create_font( &deffonts->SystemFont );
     create_font( &deffonts->DeviceDefaultFont );
 
@@ -760,11 +743,11 @@ HGDIOBJ alloc_gdi_handle( struct gdi_obj_header *obj, DWORD type, const struct g
     entry->Object  = (UINT_PTR)obj;
     entry->ExtType = type >> NTGDI_HANDLE_TYPE_SHIFT;
     entry->Type    = entry->ExtType & 0x1f;
-    if (++entry->Generation == 0xff) entry->Generation = 1;
+    if (++entry->Generation == 0x80) entry->Generation = 1;
     ret = entry_to_handle( entry );
     pthread_mutex_unlock( &gdi_lock );
     TRACE( "allocated %s %p %u/%u\n", gdi_obj_type(type), ret,
-           InterlockedIncrement( &debug_count ), GDI_MAX_HANDLE_COUNT );
+           (int)InterlockedIncrement( &debug_count ), GDI_MAX_HANDLE_COUNT );
     return ret;
 }
 
@@ -783,7 +766,7 @@ void *free_gdi_handle( HGDIOBJ handle )
     if ((entry = handle_entry( handle )))
     {
         TRACE( "freed %s %p %u/%u\n", gdi_obj_type( entry->ExtType << NTGDI_HANDLE_TYPE_SHIFT ),
-               handle, InterlockedDecrement( &debug_count ) + 1, GDI_MAX_HANDLE_COUNT );
+               handle, (int)InterlockedDecrement( &debug_count ) + 1, GDI_MAX_HANDLE_COUNT );
         object = entry_obj( entry );
         entry->Type = 0;
         entry->Object = (UINT_PTR)next_free;
@@ -960,7 +943,7 @@ INT WINAPI NtGdiExtGetObjectW( HGDIOBJ handle, INT count, void *buffer )
     if (funcs && funcs->pGetObjectW)
     {
         if (buffer && ((ULONG_PTR)buffer >> 16) == 0) /* catch apps getting argument order wrong */
-            SetLastError( ERROR_NOACCESS );
+            RtlSetLastWin32Error( ERROR_NOACCESS );
         else
             result = funcs->pGetObjectW( handle, count, buffer );
     }
@@ -986,7 +969,10 @@ HANDLE WINAPI NtGdiGetDCObject( HDC hdc, UINT type )
     case NTGDI_OBJ_BRUSH:  ret = dc->hBrush; break;
     case NTGDI_OBJ_PAL:    ret = dc->hPalette; break;
     case NTGDI_OBJ_FONT:   ret = dc->hFont; break;
-    case NTGDI_OBJ_SURF:   ret = dc->hBitmap; break;
+    case NTGDI_OBJ_SURF:
+        /* Update bitmap for display device contexts */
+        if (dc->is_display) dc->hBitmap = get_display_bitmap();
+        ret = dc->hBitmap; break;
     default:
         FIXME( "(%p, %d): unknown type.\n", hdc, type );
         break;
@@ -1044,150 +1030,7 @@ BOOL WINAPI NtGdiSetColorAdjustment( HDC hdc, const COLORADJUSTMENT *ca )
     return FALSE;
 }
 
-
-static struct unix_funcs unix_funcs =
-{
-    NtGdiAbortDoc,
-    NtGdiAbortPath,
-    NtGdiAlphaBlend,
-    NtGdiAngleArc,
-    NtGdiArcInternal,
-    NtGdiBeginPath,
-    NtGdiBitBlt,
-    NtGdiCloseFigure,
-    NtGdiComputeXformCoefficients,
-    NtGdiCreateCompatibleBitmap,
-    NtGdiCreateCompatibleDC,
-    NtGdiCreateDIBitmapInternal,
-    NtGdiCreateMetafileDC,
-    NtGdiDdDDICheckVidPnExclusiveOwnership,
-    NtGdiDdDDICreateDCFromMemory,
-    NtGdiDdDDIDestroyDCFromMemory,
-    NtGdiDdDDIDestroyDevice,
-    NtGdiDdDDIEscape,
-    NtGdiDdDDISetVidPnSourceOwner,
-    NtGdiDeleteObjectApp,
-    NtGdiDoPalette,
-    NtGdiEllipse,
-    NtGdiEndDoc,
-    NtGdiEndPath,
-    NtGdiEndPage,
-    NtGdiEnumFonts,
-    NtGdiExcludeClipRect,
-    NtGdiExtEscape,
-    NtGdiExtFloodFill,
-    NtGdiExtTextOutW,
-    NtGdiExtSelectClipRgn,
-    NtGdiFillPath,
-    NtGdiFillRgn,
-    NtGdiFontIsLinked,
-    NtGdiFrameRgn,
-    NtGdiGetAndSetDCDword,
-    NtGdiGetAppClipBox,
-    NtGdiGetBoundsRect,
-    NtGdiGetCharABCWidthsW,
-    NtGdiGetCharWidthW,
-    NtGdiGetCharWidthInfo,
-    NtGdiGetDIBitsInternal,
-    NtGdiGetDeviceCaps,
-    NtGdiGetDeviceGammaRamp,
-    NtGdiGetFontData,
-    NtGdiGetFontUnicodeRanges,
-    NtGdiGetGlyphIndicesW,
-    NtGdiGetGlyphOutline,
-    NtGdiGetKerningPairs,
-    NtGdiGetNearestColor,
-    NtGdiGetOutlineTextMetricsInternalW,
-    NtGdiGetPixel,
-    NtGdiGetRandomRgn,
-    NtGdiGetRasterizerCaps,
-    NtGdiGetRealizationInfo,
-    NtGdiGetTextCharsetInfo,
-    NtGdiGetTextExtentExW,
-    NtGdiGetTextFaceW,
-    NtGdiGetTextMetricsW,
-    NtGdiGradientFill,
-    NtGdiIntersectClipRect,
-    NtGdiInvertRgn,
-    NtGdiLineTo,
-    NtGdiMaskBlt,
-    NtGdiModifyWorldTransform,
-    NtGdiMoveTo,
-    NtGdiOffsetClipRgn,
-    NtGdiOpenDCW,
-    NtGdiPatBlt,
-    NtGdiPlgBlt,
-    NtGdiPolyDraw,
-    NtGdiPolyPolyDraw,
-    NtGdiPtVisible,
-    NtGdiRectVisible,
-    NtGdiRectangle,
-    NtGdiResetDC,
-    NtGdiResizePalette,
-    NtGdiRestoreDC,
-    NtGdiRoundRect,
-    NtGdiScaleViewportExtEx,
-    NtGdiScaleWindowExtEx,
-    NtGdiSelectBitmap,
-    NtGdiSelectBrush,
-    NtGdiSelectClipPath,
-    NtGdiSelectFont,
-    NtGdiSelectPen,
-    NtGdiSetBoundsRect,
-    NtGdiSetDIBitsToDeviceInternal,
-    NtGdiSetDeviceGammaRamp,
-    NtGdiSetLayout,
-    NtGdiSetPixel,
-    NtGdiSetSystemPaletteUse,
-    NtGdiStartDoc,
-    NtGdiStartPage,
-    NtGdiStretchBlt,
-    NtGdiStretchDIBitsInternal,
-    NtGdiStrokeAndFillPath,
-    NtGdiStrokePath,
-    NtGdiTransparentBlt,
-    NtGdiUnrealizeObject,
-    NtGdiUpdateColors,
-    NtGdiWidenPath,
-    NtUserActivateKeyboardLayout,
-    NtUserCallOneParam,
-    NtUserCallTwoParam,
-    NtUserChangeDisplaySettings,
-    NtUserCountClipboardFormats,
-    NtUserEnumDisplayDevices,
-    NtUserEnumDisplayMonitors,
-    NtUserEnumDisplaySettings,
-    NtUserGetDisplayConfigBufferSizes,
-    NtUserGetKeyNameText,
-    NtUserGetKeyboardLayoutList,
-    NtUserGetPriorityClipboardFormat,
-    NtUserGetUpdatedClipboardFormats,
-    NtUserIsClipboardFormatAvailable,
-    NtUserMapVirtualKeyEx,
-    NtUserScrollDC,
-    NtUserSelectPalette,
-    NtUserSetSysColors,
-    NtUserShowCursor,
-    NtUserSystemParametersInfo,
-    NtUserSystemParametersInfoForDpi,
-    NtUserToUnicodeEx,
-    NtUserUnregisterHotKey,
-    NtUserVkKeyScanEx,
-
-    GetDCHook,
-    SetDCHook,
-    SetDIBits,
-    SetHookFlags,
-    __wine_get_brush_bitmap_info,
-    __wine_get_file_outline_text_metric,
-    __wine_get_icm_profile,
-    __wine_get_vulkan_driver,
-    __wine_get_wgl_driver,
-    __wine_set_display_driver,
-    __wine_set_visible_region,
-};
-
-NTSTATUS gdi_init(void)
+void gdi_init(void)
 {
     pthread_mutexattr_t attr;
     unsigned int dpi;
@@ -1199,16 +1042,8 @@ NTSTATUS gdi_init(void)
 
     NtQuerySystemInformation( SystemBasicInformation, &system_info, sizeof(system_info), NULL );
     init_gdi_shared();
-    if (!gdi_shared) return STATUS_NO_MEMORY;
+    if (!gdi_shared) return;
 
     dpi = font_init();
     init_stock_objects( dpi );
-    return 0;
-}
-
-NTSTATUS callbacks_init( void *args )
-{
-    user_callbacks = *(const struct user_callbacks **)args;
-    *(const struct unix_funcs **)args = &unix_funcs;
-    return 0;
 }
