@@ -1017,7 +1017,7 @@ BOOL WINAPI NtUserScrollDC( HDC hdc, INT dx, INT dy, const RECT *scroll, const R
     if (ret && update_rect)
     {
         NtGdiGetRgnBox( update_rgn, update_rect );
-        NtGdiTransformPoints( hdc, (POINT *)&update_rect, (POINT *)&update_rect, 2, NtGdiDPtoLP );
+        NtGdiTransformPoints( hdc, (POINT *)update_rect, (POINT *)update_rect, 2, NtGdiDPtoLP );
         TRACE( "returning update_rect %s\n", wine_dbgstr_rect(update_rect) );
     }
     if (!ret_update_rgn) NtGdiDeleteObjectApp( update_rgn );
