@@ -250,7 +250,7 @@ HRESULT BitmapClipper_Create(IWICBitmapClipper **clipper)
     This->IWICBitmapClipper_iface.lpVtbl = &BitmapClipper_Vtbl;
     This->ref = 1;
     This->source = NULL;
-    InitializeCriticalSection(&This->lock);
+    InitializeCriticalSectionEx(&This->lock, 0, RTL_CRITICAL_SECTION_FLAG_FORCE_DEBUG_INFO);
     This->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": BitmapClipper.lock");
 
     *clipper = &This->IWICBitmapClipper_iface;
