@@ -294,14 +294,14 @@ static HRESULT create_decoder(const void *image_data, UINT image_size, IWICBitma
     if (hr == S_OK)
     {
         hr = IWICBitmapDecoder_GetContainerFormat(*decoder, &format);
-        ok(hr == S_OK, "GetContainerFormat error %#x\n", hr);
+        ok(hr == S_OK, "GetContainerFormat error %#lx\n", hr);
         ok(IsEqualGUID(&format, &GUID_ContainerFormatPng),
            "wrong container format %s\n", wine_dbgstr_guid(&format));
 
         zero.QuadPart = 0;
         IStream_Seek (stream, zero, STREAM_SEEK_CUR, &pos);
-        ok(pos.QuadPart < image_size, "seek beyond the end of stream: %x%08x >= %x\n",
-           (UINT)(pos.QuadPart >> 32), (UINT)pos.QuadPart, image_size);
+        ok(pos.QuadPart < image_size, "seek beyond the end of stream: %I64x >= %x\n",
+           pos.QuadPart, image_size);
 
         refcount = IStream_Release(stream);
         ok(refcount > 0, "expected stream refcount > 0\n");
@@ -346,182 +346,182 @@ static void test_color_contexts(void)
     BOOL ret;
 
     hr = create_decoder(png_no_color_profile, sizeof(png_no_color_profile), &decoder);
-    ok(hr == S_OK, "Failed to load PNG image data %#x\n", hr);
+    ok(hr == S_OK, "Failed to load PNG image data %#lx\n", hr);
     if (hr != S_OK) return;
 
     /* global color context */
     hr = IWICBitmapDecoder_GetColorContexts(decoder, 0, NULL, NULL);
-    ok(hr == WINCODEC_ERR_UNSUPPORTEDOPERATION, "GetColorContexts error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_UNSUPPORTEDOPERATION, "GetColorContexts error %#lx\n", hr);
 
     count = 0xdeadbeef;
     hr = IWICBitmapDecoder_GetColorContexts(decoder, 0, NULL, &count);
-    ok(hr == WINCODEC_ERR_UNSUPPORTEDOPERATION, "GetColorContexts error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_UNSUPPORTEDOPERATION, "GetColorContexts error %#lx\n", hr);
     ok(count == 0xdeadbeef, "unexpected count %u\n", count);
 
     /* frame color context */
     hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-    ok(hr == S_OK, "GetFrame error %#x\n", hr);
+    ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
     hr = IWICBitmapFrameDecode_GetColorContexts(frame, 0, NULL, NULL);
-    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#x\n", hr);
+    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#lx\n", hr);
 
     count = 0xdeadbeef;
     hr = IWICBitmapFrameDecode_GetColorContexts(frame, 0, NULL, &count);
-    ok(hr == S_OK, "GetColorContexts error %#x\n", hr);
+    ok(hr == S_OK, "GetColorContexts error %#lx\n", hr);
     ok(!count, "unexpected count %u\n", count);
 
     IWICBitmapFrameDecode_Release(frame);
     IWICBitmapDecoder_Release(decoder);
 
     hr = create_decoder(png_color_profile, sizeof(png_color_profile), &decoder);
-    ok(hr == S_OK, "Failed to load PNG image data %#x\n", hr);
+    ok(hr == S_OK, "Failed to load PNG image data %#lx\n", hr);
     if (hr != S_OK) return;
 
     /* global color context */
     count = 0xdeadbeef;
     hr = IWICBitmapDecoder_GetColorContexts(decoder, 0, NULL, &count);
-    ok(hr == WINCODEC_ERR_UNSUPPORTEDOPERATION, "GetColorContexts error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_UNSUPPORTEDOPERATION, "GetColorContexts error %#lx\n", hr);
     ok(count == 0xdeadbeef, "unexpected count %u\n", count);
 
     /* frame color context */
     hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-    ok(hr == S_OK, "GetFrame error %#x\n", hr);
+    ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
     count = 0xdeadbeef;
     hr = IWICBitmapFrameDecode_GetColorContexts(frame, 0, NULL, &count);
-    ok(hr == S_OK, "GetColorContexts error %#x\n", hr);
+    ok(hr == S_OK, "GetColorContexts error %#lx\n", hr);
     ok(count == 1, "unexpected count %u\n", count);
 
     hr = IWICImagingFactory_CreateColorContext(factory, NULL);
-    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#x\n", hr);
+    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#lx\n", hr);
 
     hr = IWICImagingFactory_CreateColorContext(factory, &context);
-    ok(hr == S_OK, "CreateColorContext error %#x\n", hr);
+    ok(hr == S_OK, "CreateColorContext error %#lx\n", hr);
 
     hr = IWICColorContext_GetType(context, NULL);
-    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#x\n", hr);
+    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#lx\n", hr);
 
     type = 0xdeadbeef;
     hr = IWICColorContext_GetType(context, &type);
-    ok(hr == S_OK, "GetType error %#x\n", hr);
+    ok(hr == S_OK, "GetType error %#lx\n", hr);
     ok(type == WICColorContextUninitialized, "unexpected type %u\n", type);
 
     hr = IWICColorContext_GetProfileBytes(context, 0, NULL, NULL);
-    ok(hr == WINCODEC_ERR_NOTINITIALIZED, "GetProfileBytes error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_NOTINITIALIZED, "GetProfileBytes error %#lx\n", hr);
 
     size = 0;
     hr = IWICColorContext_GetProfileBytes(context, 0, NULL, &size);
-    ok(hr == WINCODEC_ERR_NOTINITIALIZED, "GetProfileBytes error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_NOTINITIALIZED, "GetProfileBytes error %#lx\n", hr);
     ok(!size, "unexpected size %u\n", size);
 
     hr = IWICColorContext_GetExifColorSpace(context, NULL);
-    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#x\n", hr);
+    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#lx\n", hr);
 
     colorspace = 0xdeadbeef;
     hr = IWICColorContext_GetExifColorSpace(context, &colorspace);
-    ok(hr == S_OK, "GetExifColorSpace error %#x\n", hr);
+    ok(hr == S_OK, "GetExifColorSpace error %#lx\n", hr);
     ok(colorspace == 0xffffffff, "unexpected color space %u\n", colorspace);
 
     hr = IWICColorContext_InitializeFromExifColorSpace(context, 0);
-    ok(hr == S_OK, "InitializeFromExifColorSpace error %#x\n", hr);
+    ok(hr == S_OK, "InitializeFromExifColorSpace error %#lx\n", hr);
 
     hr = IWICColorContext_InitializeFromExifColorSpace(context, 1);
-    ok(hr == S_OK, "InitializeFromExifColorSpace error %#x\n", hr);
+    ok(hr == S_OK, "InitializeFromExifColorSpace error %#lx\n", hr);
 
     hr = IWICColorContext_InitializeFromExifColorSpace(context, 2);
-    ok(hr == S_OK, "InitializeFromExifColorSpace error %#x\n", hr);
+    ok(hr == S_OK, "InitializeFromExifColorSpace error %#lx\n", hr);
 
     colorspace = 0xdeadbeef;
     hr = IWICColorContext_GetExifColorSpace(context, &colorspace);
-    ok(hr == S_OK, "GetExifColorSpace error %#x\n", hr);
+    ok(hr == S_OK, "GetExifColorSpace error %#lx\n", hr);
     ok(colorspace == 2, "unexpected color space %u\n", colorspace);
 
     size = 0;
     hr = IWICColorContext_GetProfileBytes(context, 0, NULL, &size);
-    ok(hr == WINCODEC_ERR_NOTINITIALIZED, "GetProfileBytes error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_NOTINITIALIZED, "GetProfileBytes error %#lx\n", hr);
     ok(!size, "unexpected size %u\n", size);
 
     type = 0xdeadbeef;
     hr = IWICColorContext_GetType(context, &type);
-    ok(hr == S_OK, "GetType error %#x\n", hr);
+    ok(hr == S_OK, "GetType error %#lx\n", hr);
     ok(type == WICColorContextExifColorSpace, "unexpected type %u\n", type);
 
     hr = IWICBitmapFrameDecode_GetColorContexts(frame, count, &context, &count);
-    ok(hr == WINCODEC_ERR_WRONGSTATE, "GetColorContexts error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_WRONGSTATE, "GetColorContexts error %#lx\n", hr);
 
     IWICColorContext_Release(context);
     IWICBitmapFrameDecode_Release(frame);
 
     hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-    ok(hr == S_OK, "GetFrame error %#x\n", hr);
+    ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
     hr = IWICImagingFactory_CreateColorContext(factory, &context);
-    ok(hr == S_OK, "CreateColorContext error %#x\n", hr);
+    ok(hr == S_OK, "CreateColorContext error %#lx\n", hr);
 
     count = 1;
     hr = IWICBitmapFrameDecode_GetColorContexts(frame, count, &context, &count);
-    ok(hr == S_OK, "GetColorContexts error %#x\n", hr);
+    ok(hr == S_OK, "GetColorContexts error %#lx\n", hr);
 
     hr = IWICColorContext_GetProfileBytes(context, 0, NULL, NULL);
-    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#x\n", hr);
+    ok(hr == E_INVALIDARG, "expected E_INVALIDARG, got %#lx\n", hr);
 
     size = 0;
     hr = IWICColorContext_GetProfileBytes(context, 0, NULL, &size);
-    ok(hr == S_OK, "GetProfileBytes error %#x\n", hr);
+    ok(hr == S_OK, "GetProfileBytes error %#lx\n", hr);
     ok(size, "unexpected size %u\n", size);
 
     buffer = HeapAlloc(GetProcessHeap(), 0, size);
     hr = IWICColorContext_GetProfileBytes(context, size, buffer, &size);
-    ok(hr == S_OK, "GetProfileBytes error %#x\n", hr);
+    ok(hr == S_OK, "GetProfileBytes error %#lx\n", hr);
 
     tmpfile = save_profile( buffer, size );
     HeapFree(GetProcessHeap(), 0, buffer);
 
     type = 0xdeadbeef;
     hr = IWICColorContext_GetType(context, &type);
-    ok(hr == S_OK, "GetType error %#x\n", hr);
+    ok(hr == S_OK, "GetType error %#lx\n", hr);
     ok(type == WICColorContextProfile, "unexpected type %u\n", type);
 
     colorspace = 0xdeadbeef;
     hr = IWICColorContext_GetExifColorSpace(context, &colorspace);
-    ok(hr == S_OK, "GetExifColorSpace error %#x\n", hr);
+    ok(hr == S_OK, "GetExifColorSpace error %#lx\n", hr);
     ok(colorspace == 0xffffffff, "unexpected color space %u\n", colorspace);
 
     hr = IWICColorContext_InitializeFromExifColorSpace(context, 1);
-    ok(hr == WINCODEC_ERR_WRONGSTATE, "InitializeFromExifColorSpace error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_WRONGSTATE, "InitializeFromExifColorSpace error %#lx\n", hr);
 
     if (tmpfile)
     {
         hr = IWICColorContext_InitializeFromFilename(context, NULL);
-        ok(hr == E_INVALIDARG, "InitializeFromFilename error %#x\n", hr);
+        ok(hr == E_INVALIDARG, "InitializeFromFilename error %#lx\n", hr);
 
         hr = IWICColorContext_InitializeFromFilename(context, tmpfile);
-        ok(hr == S_OK, "InitializeFromFilename error %#x\n", hr);
+        ok(hr == S_OK, "InitializeFromFilename error %#lx\n", hr);
 
         ret = DeleteFileW(tmpfile);
-        ok(ret, "DeleteFileW failed %u\n", GetLastError());
+        ok(ret, "DeleteFileW failed %lu\n", GetLastError());
 
         type = 0xdeadbeef;
         hr = IWICColorContext_GetType(context, &type);
-        ok(hr == S_OK, "GetType error %#x\n", hr);
+        ok(hr == S_OK, "GetType error %#lx\n", hr);
         ok(type == WICColorContextProfile, "unexpected type %u\n", type);
 
         colorspace = 0xdeadbeef;
         hr = IWICColorContext_GetExifColorSpace(context, &colorspace);
-        ok(hr == S_OK, "GetExifColorSpace error %#x\n", hr);
+        ok(hr == S_OK, "GetExifColorSpace error %#lx\n", hr);
         ok(colorspace == 0xffffffff, "unexpected color space %u\n", colorspace);
 
         hr = IWICColorContext_InitializeFromExifColorSpace(context, 1);
-        ok(hr == WINCODEC_ERR_WRONGSTATE, "InitializeFromExifColorSpace error %#x\n", hr);
+        ok(hr == WINCODEC_ERR_WRONGSTATE, "InitializeFromExifColorSpace error %#lx\n", hr);
 
         size = 0;
         hr = IWICColorContext_GetProfileBytes(context, 0, NULL, &size);
-        ok(hr == S_OK, "GetProfileBytes error %#x\n", hr);
+        ok(hr == S_OK, "GetProfileBytes error %#lx\n", hr);
         ok(size, "unexpected size %u\n", size);
 
         buffer = HeapAlloc(GetProcessHeap(), 0, size);
         hr = IWICColorContext_GetProfileBytes(context, size, buffer, &size);
-        ok(hr == S_OK, "GetProfileBytes error %#x\n", hr);
+        ok(hr == S_OK, "GetProfileBytes error %#lx\n", hr);
 
         HeapFree(GetProcessHeap(), 0, buffer);
         HeapFree(GetProcessHeap(), 0, tmpfile);
@@ -562,28 +562,28 @@ static void test_png_palette(void)
     char *buf;
 
     hr = create_decoder(png_PLTE_tRNS, sizeof(png_PLTE_tRNS), &decoder);
-    ok(hr == S_OK, "Failed to load PNG image data %#x\n", hr);
+    ok(hr == S_OK, "Failed to load PNG image data %#lx\n", hr);
     if (hr != S_OK) return;
 
     hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-    ok(hr == S_OK, "GetFrame error %#x\n", hr);
+    ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
     hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-    ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
+    ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
     ok(IsEqualGUID(&format, &GUID_WICPixelFormat1bppIndexed),
        "got wrong format %s\n", wine_dbgstr_guid(&format));
 
     hr = IWICImagingFactory_CreatePalette(factory, &palette);
-    ok(hr == S_OK, "CreatePalette error %#x\n", hr);
+    ok(hr == S_OK, "CreatePalette error %#lx\n", hr);
     hr = IWICBitmapFrameDecode_CopyPalette(frame, palette);
-    ok(hr == S_OK, "CopyPalette error %#x\n", hr);
+    ok(hr == S_OK, "CopyPalette error %#lx\n", hr);
 
     hr = IWICPalette_GetColorCount(palette, &count);
-    ok(hr == S_OK, "GetColorCount error %#x\n", hr);
+    ok(hr == S_OK, "GetColorCount error %#lx\n", hr);
     ok(count == 2, "expected 2, got %u\n", count);
 
     hr = IWICPalette_GetColors(palette, 256, color, &ret);
-    ok(hr == S_OK, "GetColors error %#x\n", hr);
+    ok(hr == S_OK, "GetColors error %#lx\n", hr);
     ok(ret == count, "expected %u, got %u\n", count, ret);
     ok(color[0] == 0xff010203, "expected 0xff010203, got %#x\n", color[0]);
     ok(color[1] == 0x00040506, "expected 0x00040506, got %#x\n", color[1]);
@@ -593,21 +593,21 @@ static void test_png_palette(void)
     IWICBitmapDecoder_Release(decoder);
 
     hr = create_decoder(png_gray_tRNS, sizeof(png_gray_tRNS), &decoder);
-    ok(hr == S_OK, "Failed to load PNG image data %#x\n", hr);
+    ok(hr == S_OK, "Failed to load PNG image data %#lx\n", hr);
     if (hr != S_OK) return;
 
     hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-    ok(hr == S_OK, "GetFrame error %#x\n", hr);
+    ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
     hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-    ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
+    ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
     ok(IsEqualGUID(&format, &GUID_WICPixelFormat64bppRGBA),
        "got wrong format %s\n", wine_dbgstr_guid(&format));
 
     hr = IWICImagingFactory_CreatePalette(factory, &palette);
-    ok(hr == S_OK, "CreatePalette error %#x\n", hr);
+    ok(hr == S_OK, "CreatePalette error %#lx\n", hr);
     hr = IWICBitmapFrameDecode_CopyPalette(frame, palette);
-    ok(hr == WINCODEC_ERR_PALETTEUNAVAILABLE, "CopyPalette error %#x\n", hr);
+    ok(hr == WINCODEC_ERR_PALETTEUNAVAILABLE, "CopyPalette error %#lx\n", hr);
 
     IWICPalette_Release(palette);
     IWICBitmapFrameDecode_Release(frame);
@@ -619,28 +619,28 @@ static void test_png_palette(void)
     buf[24] = 8; /* override bit depth */
 
     hr = create_decoder(buf, sizeof(png_gray_tRNS), &decoder);
-    ok(hr == S_OK, "Failed to load PNG image data %#x\n", hr);
+    ok(hr == S_OK, "Failed to load PNG image data %#lx\n", hr);
     if (hr != S_OK) return;
 
     hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-    ok(hr == S_OK, "GetFrame error %#x\n", hr);
+    ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
     hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-    ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
+    ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
     ok(IsEqualGUID(&format, &GUID_WICPixelFormat8bppIndexed),
        "got wrong format %s\n", wine_dbgstr_guid(&format));
 
     hr = IWICImagingFactory_CreatePalette(factory, &palette);
-    ok(hr == S_OK, "CreatePalette error %#x\n", hr);
+    ok(hr == S_OK, "CreatePalette error %#lx\n", hr);
     hr = IWICBitmapFrameDecode_CopyPalette(frame, palette);
-    ok(hr == S_OK, "CopyPalette error %#x\n", hr);
+    ok(hr == S_OK, "CopyPalette error %#lx\n", hr);
 
     hr = IWICPalette_GetColorCount(palette, &count);
-    ok(hr == S_OK, "GetColorCount error %#x\n", hr);
+    ok(hr == S_OK, "GetColorCount error %#lx\n", hr);
     ok(count == 256, "expected 256, got %u\n", count);
 
     hr = IWICPalette_GetColors(palette, 256, color, &ret);
-    ok(hr == S_OK, "GetColors error %#x\n", hr);
+    ok(hr == S_OK, "GetColors error %#lx\n", hr);
     ok(ret == count, "expected %u, got %u\n", count, ret);
     ok(color[0] == 0x00000000, "expected 0x00000000, got %#x\n", color[0]);
     ok(color[1] == 0xff010101, "expected 0xff010101, got %#x\n", color[1]);
@@ -813,18 +813,18 @@ static void test_color_formats(void)
 
         hr = create_decoder(buf, sizeof(buf), &decoder);
         if (!is_valid_png_type_depth(td[i].color_type, td[i].bit_depth, TRUE))
-            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#x\n", i, hr);
+            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#lx\n", i, hr);
         else
-todo_wine_if(td[i].todo_load)
-            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#x\n", i, td[i].color_type, td[i].bit_depth, hr);
+            todo_wine_if(td[i].todo_load)
+            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#lx\n", i, td[i].color_type, td[i].bit_depth, hr);
         if (hr != S_OK) goto next_1;
 
         hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-        ok(hr == S_OK, "GetFrame error %#x\n", hr);
+        ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
         hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-        ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
-todo_wine_if(td[i].todo)
+        ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
+        todo_wine_if(td[i].todo)
         ok(IsEqualGUID(&format, td[i].format_PLTE_tRNS),
            "PLTE+tRNS: expected %s, got %s (type %d, bpp %d)\n",
             wine_dbgstr_guid(td[i].format_PLTE_tRNS), wine_dbgstr_guid(&format), td[i].color_type, td[i].bit_depth);
@@ -841,17 +841,17 @@ next_1:
 
         hr = create_decoder(buf, sizeof(buf), &decoder);
         if (!is_valid_png_type_depth(td[i].color_type, td[i].bit_depth, TRUE))
-            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#x\n", i, hr);
+            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#lx\n", i, hr);
         else
-todo_wine_if(td[i].todo_load)
-            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#x\n", i, td[i].color_type, td[i].bit_depth, hr);
+            todo_wine_if(td[i].todo_load)
+            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#lx\n", i, td[i].color_type, td[i].bit_depth, hr);
         if (hr != S_OK) goto next_2;
 
         hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-        ok(hr == S_OK, "GetFrame error %#x\n", hr);
+        ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
         hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-        ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
+        ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
         ok(IsEqualGUID(&format, td[i].format_PLTE),
            "PLTE: expected %s, got %s (type %d, bpp %d)\n",
             wine_dbgstr_guid(td[i].format_PLTE), wine_dbgstr_guid(&format), td[i].color_type, td[i].bit_depth);
@@ -869,17 +869,17 @@ next_2:
 
         hr = create_decoder(buf, sizeof(buf), &decoder);
         if (!is_valid_png_type_depth(td[i].color_type, td[i].bit_depth, FALSE))
-            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#x\n", i, hr);
+            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#lx\n", i, hr);
         else
-todo_wine_if(td[i].todo_load)
-            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#x\n", i, td[i].color_type, td[i].bit_depth, hr);
+            todo_wine_if(td[i].todo_load)
+            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#lx\n", i, td[i].color_type, td[i].bit_depth, hr);
         if (hr != S_OK) goto next_3;
 
         hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-        ok(hr == S_OK, "GetFrame error %#x\n", hr);
+        ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
         hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-        ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
+        ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
         ok(IsEqualGUID(&format, td[i].format),
            "expected %s, got %s (type %d, bpp %d)\n",
             wine_dbgstr_guid(td[i].format), wine_dbgstr_guid(&format), td[i].color_type, td[i].bit_depth);
@@ -896,18 +896,18 @@ next_3:
 
         hr = create_decoder(buf, sizeof(buf), &decoder);
         if (!is_valid_png_type_depth(td[i].color_type, td[i].bit_depth, FALSE))
-            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#x\n", i, hr);
+            ok(hr == WINCODEC_ERR_UNKNOWNIMAGEFORMAT, "%d: wrong error %#lx\n", i, hr);
         else
-todo_wine_if(td[i].todo_load)
-            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#x\n", i, td[i].color_type, td[i].bit_depth, hr);
+            todo_wine_if(td[i].todo_load)
+            ok(hr == S_OK, "%d: Failed to load PNG image data (type %d, bpp %d) %#lx\n", i, td[i].color_type, td[i].bit_depth, hr);
         if (hr != S_OK) continue;
 
         hr = IWICBitmapDecoder_GetFrame(decoder, 0, &frame);
-        ok(hr == S_OK, "GetFrame error %#x\n", hr);
+        ok(hr == S_OK, "GetFrame error %#lx\n", hr);
 
         hr = IWICBitmapFrameDecode_GetPixelFormat(frame, &format);
-        ok(hr == S_OK, "GetPixelFormat error %#x\n", hr);
-todo_wine_if(td[i].todo)
+        ok(hr == S_OK, "GetPixelFormat error %#lx\n", hr);
+        todo_wine_if(td[i].todo)
         ok(IsEqualGUID(&format, td[i].format_PLTE_tRNS),
            "tRNS: expected %s, got %s (type %d, bpp %d)\n",
             wine_dbgstr_guid(td[i].format_PLTE_tRNS), wine_dbgstr_guid(&format), td[i].color_type, td[i].bit_depth);
@@ -944,7 +944,7 @@ static void test_chunk_size(void)
     memcpy(png_8M_tEXt + sizeof(png_8M_tEXt) - sizeof(png_8M_tEXt_end), png_8M_tEXt_end, sizeof(png_8M_tEXt_end));
 
     hr = create_decoder(png_8M_tEXt, sizeof(png_8M_tEXt), &decoder);
-    ok(hr == S_OK, "Failed to load PNG image data %#x\n", hr);
+    ok(hr == S_OK, "Failed to load PNG image data %#lx\n", hr);
     if (hr != S_OK) return;
 
     IWICBitmapDecoder_Release(decoder);
@@ -957,7 +957,7 @@ START_TEST(pngformat)
     CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
     hr = CoCreateInstance(&CLSID_WICImagingFactory, NULL, CLSCTX_INPROC_SERVER,
                           &IID_IWICImagingFactory, (void **)&factory);
-    ok(hr == S_OK, "CoCreateInstance error %#x\n", hr);
+    ok(hr == S_OK, "CoCreateInstance error %#lx\n", hr);
     if (FAILED(hr)) return;
 
     test_color_contexts();
