@@ -953,7 +953,7 @@ HRESULT TgaDecoder_CreateInstance(REFIID iid, void** ppv)
     This->initialized = FALSE;
     This->stream = NULL;
     This->imagebits = NULL;
-    InitializeCriticalSection(&This->lock);
+    InitializeCriticalSectionEx(&This->lock, 0, RTL_CRITICAL_SECTION_FLAG_FORCE_DEBUG_INFO);
     This->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": TgaDecoder.lock");
 
     ret = IWICBitmapDecoder_QueryInterface(&This->IWICBitmapDecoder_iface, iid, ppv);
