@@ -201,7 +201,7 @@ void reverse_bgr8(UINT bytesperpixel, LPBYTE bits, UINT width, UINT height, INT
 
     for (y=0; y<height; y++)
     {
-        pixel = bits + stride * y;
+        pixel = bits + stride * (INT)y;
 
         for (x=0; x<width; x++)
         {
