@@ -68,7 +68,7 @@ DECL_FUNCPTR( eglSwapInterval );
 
 static const int egl_client_version = 2;
 
-struct wgl_pixel_format
+struct egl_pixel_format
 {
     EGLConfig config;
 };
@@ -96,7 +96,7 @@ struct gl_drawable
 
 static void *egl_handle;
 static void *opengl_handle;
-static struct wgl_pixel_format *pixel_formats;
+static struct egl_pixel_format *pixel_formats;
 static int nb_pixel_formats, nb_onscreen_formats;
 static EGLDisplay display;
 static int swap_interval;
@@ -208,11 +208,10 @@ void update_gl_drawable( HWND hwnd )
     }
 }
 
-static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
+static BOOL set_pixel_format( HDC hdc, int format, BOOL internal )
 {
     struct gl_drawable *gl;
     HWND hwnd = NtUserWindowFromDC( hdc );
-    int prev = 0;
 
     if (!hwnd || hwnd == NtUserGetDesktopWindow())
     {
@@ -226,10 +225,18 @@ static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
     }
     TRACE( "%p/%p format %d\n", hdc, hwnd, format );
 
+    if (!internal)
+    {
+        /* cannot change it if already set */
+        int prev = win32u_get_window_pixel_format( hwnd );
+
+        if (prev)
+            return prev == format;
+    }
+
     if ((gl = get_gl_drawable( hwnd, 0 )))
     {
-        prev = gl->format;
-        if (allow_change)
+        if (internal)
         {
             EGLint pf;
             p_eglGetConfigAttrib( display, pixel_formats[format - 1].config, EGL_NATIVE_VISUAL_ID, &pf );
@@ -241,8 +248,7 @@ static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
 
     release_gl_drawable( gl );
 
-    if (prev && prev != format && !allow_change) return FALSE;
-    if (NtUserSetWindowPixelFormat( hwnd, format )) return TRUE;
+    if (win32u_set_window_pixel_format( hwnd, format, internal )) return TRUE;
     destroy_gl_drawable( hwnd );
     return FALSE;
 }
@@ -267,6 +273,39 @@ static struct wgl_context *create_context( HDC hdc, struct wgl_context *share, c
     return ctx;
 }
 
+static void describe_pixel_format( struct egl_pixel_format *fmt, PIXELFORMATDESCRIPTOR *pfd )
+{
+    EGLint val;
+    EGLConfig config = fmt->config;
+
+    memset( pfd, 0, sizeof(*pfd) );
+    pfd->nSize = sizeof(*pfd);
+    pfd->nVersion = 1;
+    pfd->dwFlags = PFD_SUPPORT_OPENGL | PFD_DRAW_TO_WINDOW | PFD_DOUBLEBUFFER | PFD_SUPPORT_COMPOSITION;
+    pfd->iPixelType = PFD_TYPE_RGBA;
+    pfd->iLayerType = PFD_MAIN_PLANE;
+
+    p_eglGetConfigAttrib( display, config, EGL_BUFFER_SIZE, &val );
+    pfd->cColorBits = val;
+    p_eglGetConfigAttrib( display, config, EGL_RED_SIZE, &val );
+    pfd->cRedBits = val;
+    p_eglGetConfigAttrib( display, config, EGL_GREEN_SIZE, &val );
+    pfd->cGreenBits = val;
+    p_eglGetConfigAttrib( display, config, EGL_BLUE_SIZE, &val );
+    pfd->cBlueBits = val;
+    p_eglGetConfigAttrib( display, config, EGL_ALPHA_SIZE, &val );
+    pfd->cAlphaBits = val;
+    p_eglGetConfigAttrib( display, config, EGL_DEPTH_SIZE, &val );
+    pfd->cDepthBits = val;
+    p_eglGetConfigAttrib( display, config, EGL_STENCIL_SIZE, &val );
+    pfd->cStencilBits = val;
+
+    pfd->cAlphaShift = 0;
+    pfd->cBlueShift = pfd->cAlphaShift + pfd->cAlphaBits;
+    pfd->cGreenShift = pfd->cBlueShift + pfd->cBlueBits;
+    pfd->cRedShift = pfd->cGreenShift + pfd->cGreenBits;
+}
+
 /***********************************************************************
  *		android_wglGetExtensionsStringARB
  */
@@ -444,52 +483,6 @@ static BOOL android_wglDeleteContext( struct wgl_context *ctx )
     return TRUE;
 }
 
-/***********************************************************************
- *		android_wglDescribePixelFormat
- */
-static int android_wglDescribePixelFormat( HDC hdc, int fmt, UINT size, PIXELFORMATDESCRIPTOR *pfd )
-{
-    EGLint val;
-    EGLConfig config;
-
-    if (!pfd) return nb_onscreen_formats;
-    if (!is_onscreen_pixel_format( fmt )) return 0;
-    if (size < sizeof(*pfd)) return 0;
-    config = pixel_formats[fmt - 1].config;
-
-    memset( pfd, 0, sizeof(*pfd) );
-    pfd->nSize = sizeof(*pfd);
-    pfd->nVersion = 1;
-    pfd->dwFlags = PFD_SUPPORT_OPENGL | PFD_DRAW_TO_WINDOW | PFD_DOUBLEBUFFER;
-    pfd->iPixelType = PFD_TYPE_RGBA;
-    pfd->iLayerType = PFD_MAIN_PLANE;
-
-    p_eglGetConfigAttrib( display, config, EGL_BUFFER_SIZE, &val );
-    pfd->cColorBits = val;
-    p_eglGetConfigAttrib( display, config, EGL_RED_SIZE, &val );
-    pfd->cRedBits = val;
-    p_eglGetConfigAttrib( display, config, EGL_GREEN_SIZE, &val );
-    pfd->cGreenBits = val;
-    p_eglGetConfigAttrib( display, config, EGL_BLUE_SIZE, &val );
-    pfd->cBlueBits = val;
-    p_eglGetConfigAttrib( display, config, EGL_ALPHA_SIZE, &val );
-    pfd->cAlphaBits = val;
-    p_eglGetConfigAttrib( display, config, EGL_DEPTH_SIZE, &val );
-    pfd->cDepthBits = val;
-    p_eglGetConfigAttrib( display, config, EGL_STENCIL_SIZE, &val );
-    pfd->cStencilBits = val;
-
-    pfd->cAlphaShift = 0;
-    pfd->cBlueShift = pfd->cAlphaShift + pfd->cAlphaBits;
-    pfd->cGreenShift = pfd->cBlueShift + pfd->cBlueBits;
-    pfd->cRedShift = pfd->cGreenShift + pfd->cGreenBits;
-
-    TRACE( "fmt %u color %u %u/%u/%u/%u depth %u stencil %u\n",
-           fmt, pfd->cColorBits, pfd->cRedBits, pfd->cGreenBits, pfd->cBlueBits,
-           pfd->cAlphaBits, pfd->cDepthBits, pfd->cStencilBits );
-    return nb_onscreen_formats;
-}
-
 /***********************************************************************
  *		android_wglGetPixelFormat
  */
@@ -497,8 +490,14 @@ static int android_wglGetPixelFormat( HDC hdc )
 {
     struct gl_drawable *gl;
     int ret = 0;
+    HWND hwnd;
 
-    if ((gl = get_gl_drawable( NtUserWindowFromDC( hdc ), hdc )))
+    if ((hwnd = NtUserWindowFromDC( hdc )))
+        return win32u_get_window_pixel_format( hwnd );
+
+    /* This code is currently dead, but will be necessary if WGL_ARB_pbuffer
+     * support is introduced. */
+    if ((gl = get_gl_drawable( NULL, hdc )))
     {
         ret = gl->format;
         /* offscreen formats can't be used with traditional WGL calls */
@@ -593,6 +592,24 @@ static BOOL android_wglSwapBuffers( HDC hdc )
     return TRUE;
 }
 
+/**********************************************************************
+ *              android_get_pixel_formats
+ */
+static void android_get_pixel_formats( struct wgl_pixel_format *formats,
+                                       UINT max_formats, UINT *num_formats,
+                                       UINT *num_onscreen_formats )
+{
+    UINT i;
+
+    if (formats)
+    {
+        for (i = 0; i < min( max_formats, nb_pixel_formats ); ++i)
+            describe_pixel_format( &pixel_formats[i], &formats[i].pfd );
+    }
+    *num_formats = nb_pixel_formats;
+    *num_onscreen_formats = nb_onscreen_formats;
+}
+
 static void wglFinish(void)
 {
     struct wgl_context *ctx = NtCurrentTeb()->glContext;
@@ -1049,13 +1066,13 @@ static struct opengl_funcs egl_funcs =
         android_wglCopyContext,
         android_wglCreateContext,
         android_wglDeleteContext,
-        android_wglDescribePixelFormat,
         android_wglGetPixelFormat,
         android_wglGetProcAddress,
         android_wglMakeCurrent,
         android_wglSetPixelFormat,
         android_wglShareLists,
         android_wglSwapBuffers,
+        android_get_pixel_formats,
     },
 #define USE_GL_FUNC(name) (void *)glstub_##name,
     { ALL_WGL_FUNCS }
