@@ -25,6 +25,7 @@
 #include <winbase.h>
 #include <winternl.h>
 #include <ddk/hidclass.h>
+#include <ddk/hidpi.h>
 #include <hidusage.h>
 
 #include "wine/debug.h"
@@ -38,6 +39,8 @@ struct device_desc
     UINT input;
     UINT uid;
     BOOL is_gamepad;
+    BOOL is_hidraw;
+    BOOL is_bluetooth;
 
     WCHAR manufacturer[MAX_PATH];
     WCHAR product[MAX_PATH];
@@ -147,8 +150,49 @@ enum unix_funcs
 static inline const char *debugstr_device_desc(struct device_desc *desc)
 {
     if (!desc) return "(null)";
-    return wine_dbg_sprintf("{vid %04x, pid %04x, version %04x, input %d, uid %08x, is_gamepad %u}",
-                            desc->vid, desc->pid, desc->version, desc->input, desc->uid, desc->is_gamepad);
+    return wine_dbg_sprintf("{vid %04x, pid %04x, version %04x, input %d, uid %08x, is_gamepad %u, is_hidraw %u, is_bluetooth %u}",
+                            desc->vid, desc->pid, desc->version, desc->input, desc->uid,
+                            desc->is_gamepad, desc->is_hidraw, desc->is_bluetooth);
+}
+
+static inline BOOL is_xbox_gamepad(WORD vid, WORD pid)
+{
+    if (vid != 0x045e) return FALSE;
+    if (pid == 0x0202) return TRUE; /* Xbox Controller */
+    if (pid == 0x0285) return TRUE; /* Xbox Controller S */
+    if (pid == 0x0289) return TRUE; /* Xbox Controller S */
+    if (pid == 0x028e) return TRUE; /* Xbox360 Controller */
+    if (pid == 0x028f) return TRUE; /* Xbox360 Wireless Controller */
+    if (pid == 0x02d1) return TRUE; /* Xbox One Controller */
+    if (pid == 0x02dd) return TRUE; /* Xbox One Controller (Covert Forces/Firmware 2015) */
+    if (pid == 0x02e0) return TRUE; /* Xbox One X Controller */
+    if (pid == 0x02e3) return TRUE; /* Xbox One Elite Controller */
+    if (pid == 0x02e6) return TRUE; /* Wireless XBox Controller Dongle */
+    if (pid == 0x02ea) return TRUE; /* Xbox One S Controller */
+    if (pid == 0x02fd) return TRUE; /* Xbox One S Controller (Firmware 2017) */
+    if (pid == 0x0b00) return TRUE; /* Xbox Elite 2 */
+    if (pid == 0x0b05) return TRUE; /* Xbox Elite 2 Wireless */
+    if (pid == 0x0b12) return TRUE; /* Xbox Series */
+    if (pid == 0x0b13) return TRUE; /* Xbox Series Wireless */
+    if (pid == 0x0719) return TRUE; /* Xbox 360 Wireless Adapter */
+    return FALSE;
+}
+
+static inline BOOL is_dualshock4_gamepad(WORD vid, WORD pid)
+{
+    if (vid != 0x054c) return FALSE;
+    if (pid == 0x05c4) return TRUE; /* DualShock 4 [CUH-ZCT1x] */
+    if (pid == 0x09cc) return TRUE; /* DualShock 4 [CUH-ZCT2x] */
+    if (pid == 0x0ba0) return TRUE; /* Dualshock 4 Wireless Adaptor */
+    return FALSE;
+}
+
+static inline BOOL is_dualsense_gamepad(WORD vid, WORD pid)
+{
+    if (vid != 0x054c) return FALSE;
+    if (pid == 0x0ce6) return TRUE; /* DualSense */
+    if (pid == 0x0df2) return TRUE; /* DualSense Edge */
+    return FALSE;
 }
 
 #endif /* __WINEBUS_UNIXLIB_H */
