@@ -25,6 +25,7 @@
 
 #include <stdbool.h>
 #include <gst/gst.h>
+#include <gst/audio/audio.h>
 
 /* unixlib.c */
 
@@ -45,18 +46,21 @@ extern bool link_src_to_sink(GstPad *src_pad, GstPad *sink_pad);
 extern bool link_src_to_element(GstPad *src_pad, GstElement *element);
 extern bool link_element_to_sink(GstElement *element, GstPad *sink_pad);
 extern bool push_event(GstPad *pad, GstEvent *event);
+extern void set_max_threads(GstElement *element);
 
 /* wg_format.c */
 
 extern void wg_format_from_caps(struct wg_format *format, const GstCaps *caps);
 extern bool wg_format_compare(const struct wg_format *a, const struct wg_format *b);
 extern GstCaps *wg_format_to_caps(const struct wg_format *format);
+extern uint32_t wg_channel_mask_from_gst(const GstAudioInfo *info);
 
 /* wg_transform.c */
 
 extern NTSTATUS wg_transform_create(void *args);
 extern NTSTATUS wg_transform_destroy(void *args);
-extern NTSTATUS wg_transform_set_output_format(void *args);
+extern NTSTATUS wg_transform_get_output_type(void *args);
+extern NTSTATUS wg_transform_set_output_type(void *args);
 extern NTSTATUS wg_transform_push_data(void *args);
 extern NTSTATUS wg_transform_read_data(void *args);
 extern NTSTATUS wg_transform_get_status(void *args);
@@ -64,6 +68,17 @@ extern NTSTATUS wg_transform_drain(void *args);
 extern NTSTATUS wg_transform_flush(void *args);
 extern NTSTATUS wg_transform_notify_qos(void *args);
 
+/* wg_media_type.c */
+
+static inline BOOL is_mf_video_area_empty(const MFVideoArea *area)
+{
+    return !area->OffsetX.value && !area->OffsetY.value && !area->Area.cx && !area->Area.cy;
+}
+
+extern GstCaps *caps_from_media_type(const struct wg_media_type *media_type);
+extern NTSTATUS caps_to_media_type(GstCaps *caps, struct wg_media_type *media_type,
+        UINT32 video_plane_align);
+
 /* wg_muxer.c */
 
 extern NTSTATUS wg_muxer_create(void *args);
