@@ -96,7 +96,7 @@ static uint32_t wg_channel_position_from_gst(GstAudioChannelPosition position)
     return 0;
 }
 
-static uint32_t wg_channel_mask_from_gst(const GstAudioInfo *info)
+uint32_t wg_channel_mask_from_gst(const GstAudioInfo *info)
 {
     uint32_t mask = 0, position;
     unsigned int i;
@@ -145,6 +145,8 @@ static enum wg_video_format wg_video_format_from_gst(GstVideoFormat format)
             return WG_VIDEO_FORMAT_RGB15;
         case GST_VIDEO_FORMAT_RGB16:
             return WG_VIDEO_FORMAT_RGB16;
+        case GST_VIDEO_FORMAT_RGBA:
+            return WG_VIDEO_FORMAT_RGBA;
         case GST_VIDEO_FORMAT_AYUV:
             return WG_VIDEO_FORMAT_AYUV;
         case GST_VIDEO_FORMAT_I420:
@@ -196,9 +198,70 @@ static void wg_format_from_caps_audio_mpeg1(struct wg_format *format, const GstC
     }
 
     format->major_type = WG_MAJOR_TYPE_AUDIO_MPEG1;
-    format->u.audio_mpeg1.layer = layer;
-    format->u.audio_mpeg1.channels = channels;
-    format->u.audio_mpeg1.rate = rate;
+    format->u.audio.layer = layer;
+    format->u.audio.channels = channels;
+    format->u.audio.rate = rate;
+}
+
+static void wg_format_from_caps_audio_mpeg4(struct wg_format *format, const GstCaps *caps)
+{
+    static const size_t waveinfo_size = sizeof(HEAACWAVEINFO) - sizeof(WAVEFORMATEX);
+    const GstStructure *structure = gst_caps_get_structure(caps, 0);
+    const GValue *codec_data_value;
+    const gchar *stream_format;
+    GstBuffer *codec_data;
+    gint channels, rate;
+    GstMapInfo map;
+
+    if (!gst_structure_get_int(structure, "channels", &channels))
+    {
+        GST_WARNING("Missing \"channels\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+    if (!gst_structure_get_int(structure, "rate", &rate))
+    {
+        GST_WARNING("Missing \"rate\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+    if (!(codec_data_value = gst_structure_get_value(structure, "codec_data"))
+            || !(codec_data = gst_value_get_buffer(codec_data_value)))
+    {
+        GST_WARNING("Missing \"codec_data\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+    if (!(stream_format = gst_structure_get_string(structure, "stream-format")))
+    {
+        GST_WARNING("Missing \"stream-format\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+
+    format->major_type = WG_MAJOR_TYPE_AUDIO_MPEG4;
+    format->u.audio.channels = channels;
+    format->u.audio.rate = rate;
+    if (!strcmp(stream_format, "raw"))
+        format->u.audio.payload_type = 0;
+    else if (!strcmp(stream_format, "adts"))
+        format->u.audio.payload_type = 1;
+    else if (!strcmp(stream_format, "adif"))
+        format->u.audio.payload_type = 2;
+    else if (!strcmp(stream_format, "loas"))
+        format->u.audio.payload_type = 3;
+
+    gst_buffer_map(codec_data, &map, GST_MAP_READ);
+    if ((map.size + waveinfo_size) <= sizeof(format->u.audio.codec_data))
+    {
+        HEAACWAVEINFO wave_info = {0};
+
+        wave_info.wPayloadType = format->u.audio.payload_type;
+        /* Native apparently sets wAudioProfileLevelIndication to zero. */
+
+        format->u.audio.codec_data_len = map.size + waveinfo_size;
+        memcpy(format->u.audio.codec_data, &wave_info.wPayloadType, waveinfo_size);
+        memcpy(format->u.audio.codec_data + waveinfo_size, map.data, map.size);
+    }
+    else
+        GST_WARNING("Too big codec_data value (%u) in %" GST_PTR_FORMAT ".", (UINT)map.size, caps);
+    gst_buffer_unmap(codec_data, &map);
 }
 
 static void wg_format_from_caps_audio_wma(struct wg_format *format, const GstCaps *caps)
@@ -247,18 +310,18 @@ static void wg_format_from_caps_audio_wma(struct wg_format *format, const GstCap
     }
 
     format->major_type = WG_MAJOR_TYPE_AUDIO_WMA;
-    format->u.audio_wma.version = version;
-    format->u.audio_wma.bitrate = bitrate;
-    format->u.audio_wma.rate = rate;
-    format->u.audio_wma.depth = depth;
-    format->u.audio_wma.channels = channels;
-    format->u.audio_wma.block_align = block_align;
+    format->u.audio.version = version;
+    format->u.audio.bitrate = bitrate;
+    format->u.audio.rate = rate;
+    format->u.audio.depth = depth;
+    format->u.audio.channels = channels;
+    format->u.audio.block_align = block_align;
 
     gst_buffer_map(codec_data, &map, GST_MAP_READ);
-    if (map.size <= sizeof(format->u.audio_wma.codec_data))
+    if (map.size <= sizeof(format->u.audio.codec_data))
     {
-        format->u.audio_wma.codec_data_len = map.size;
-        memcpy(format->u.audio_wma.codec_data, map.data, map.size);
+        format->u.audio.codec_data_len = map.size;
+        memcpy(format->u.audio.codec_data, map.data, map.size);
     }
     else
         GST_WARNING("Too big codec_data value (%u) in %" GST_PTR_FORMAT ".", (UINT)map.size, caps);
@@ -287,10 +350,10 @@ static void wg_format_from_caps_video_cinepak(struct wg_format *format, const Gs
     }
 
     format->major_type = WG_MAJOR_TYPE_VIDEO_CINEPAK;
-    format->u.video_cinepak.width = width;
-    format->u.video_cinepak.height = height;
-    format->u.video_cinepak.fps_n = fps_n;
-    format->u.video_cinepak.fps_d = fps_d;
+    format->u.video.width = width;
+    format->u.video.height = height;
+    format->u.video.fps_n = fps_n;
+    format->u.video.fps_d = fps_d;
 }
 
 static void wg_format_from_caps_video_wmv(struct wg_format *format, const GstCaps *caps)
@@ -298,8 +361,8 @@ static void wg_format_from_caps_video_wmv(struct wg_format *format, const GstCap
     const GstStructure *structure = gst_caps_get_structure(caps, 0);
     gint width, height, fps_n, fps_d, wmv_version = 0;
     gchar format_buffer[5] = {'W','M','V','0',0};
-    enum wg_wmv_video_format wmv_format;
     const gchar *wmv_format_str = NULL;
+    enum wg_video_format video_format;
     const GValue *codec_data_value;
     GstBuffer *codec_data;
     GstMapInfo map;
@@ -323,17 +386,17 @@ static void wg_format_from_caps_video_wmv(struct wg_format *format, const GstCap
         wmv_format_str = format_buffer;
     }
     if (!strcmp(wmv_format_str, "WMV1"))
-        wmv_format = WG_WMV_VIDEO_FORMAT_WMV1;
+        video_format = WG_VIDEO_FORMAT_WMV1;
     else if (!strcmp(wmv_format_str, "WMV2"))
-        wmv_format = WG_WMV_VIDEO_FORMAT_WMV2;
+        video_format = WG_VIDEO_FORMAT_WMV2;
     else if (!strcmp(wmv_format_str, "WMV3"))
-        wmv_format = WG_WMV_VIDEO_FORMAT_WMV3;
+        video_format = WG_VIDEO_FORMAT_WMV3;
     else if (!strcmp(wmv_format_str, "WMVA"))
-        wmv_format = WG_WMV_VIDEO_FORMAT_WMVA;
+        video_format = WG_VIDEO_FORMAT_WMVA;
     else if (!strcmp(wmv_format_str, "WVC1"))
-        wmv_format = WG_WMV_VIDEO_FORMAT_WVC1;
+        video_format = WG_VIDEO_FORMAT_WVC1;
     else
-        wmv_format = WG_WMV_VIDEO_FORMAT_UNKNOWN;
+        video_format = WG_VIDEO_FORMAT_UNKNOWN;
 
     if (!gst_structure_get_fraction(structure, "framerate", &fps_n, &fps_d))
     {
@@ -342,19 +405,19 @@ static void wg_format_from_caps_video_wmv(struct wg_format *format, const GstCap
     }
 
     format->major_type = WG_MAJOR_TYPE_VIDEO_WMV;
-    format->u.video_wmv.width = width;
-    format->u.video_wmv.height = height;
-    format->u.video_wmv.format = wmv_format;
-    format->u.video_wmv.fps_n = fps_n;
-    format->u.video_wmv.fps_d = fps_d;
+    format->u.video.width = width;
+    format->u.video.height = height;
+    format->u.video.format = video_format;
+    format->u.video.fps_n = fps_n;
+    format->u.video.fps_d = fps_d;
 
     if ((codec_data_value = gst_structure_get_value(structure, "codec_data")) && (codec_data = gst_value_get_buffer(codec_data_value)))
     {
         gst_buffer_map(codec_data, &map, GST_MAP_READ);
-        if (map.size <= sizeof(format->u.video_wmv.codec_data))
+        if (map.size <= sizeof(format->u.video.codec_data))
         {
-            format->u.video_wmv.codec_data_len = map.size;
-            memcpy(format->u.video_wmv.codec_data, map.data, map.size);
+            format->u.video.codec_data_len = map.size;
+            memcpy(format->u.video.codec_data, map.data, map.size);
         }
         else
             GST_WARNING("Too big codec_data value (%u) in %" GST_PTR_FORMAT ".", (UINT)map.size, caps);
@@ -384,10 +447,127 @@ static void wg_format_from_caps_video_mpeg1(struct wg_format *format, const GstC
     }
 
     format->major_type = WG_MAJOR_TYPE_VIDEO_MPEG1;
-    format->u.video_mpeg1.width = width;
-    format->u.video_mpeg1.height = height;
-    format->u.video_mpeg1.fps_n = fps_n;
-    format->u.video_mpeg1.fps_d = fps_d;
+    format->u.video.width = width;
+    format->u.video.height = height;
+    format->u.video.fps_n = fps_n;
+    format->u.video.fps_d = fps_d;
+}
+
+static const struct
+{
+    enum eAVEncH264VLevel level;
+    const char *string;
+}
+h264_levels[] =
+{
+    {eAVEncH264VLevel1,   "1"},
+    {eAVEncH264VLevel1_1, "1.1"},
+    {eAVEncH264VLevel1_2, "1.2"},
+    {eAVEncH264VLevel1_3, "1.3"},
+    {eAVEncH264VLevel2,   "2"},
+    {eAVEncH264VLevel2_1, "2.1"},
+    {eAVEncH264VLevel2_2, "2.2"},
+    {eAVEncH264VLevel3,   "3"},
+    {eAVEncH264VLevel3_1, "3.1"},
+    {eAVEncH264VLevel3_2, "3.2"},
+    {eAVEncH264VLevel4,   "4"},
+    {eAVEncH264VLevel4_1, "4.1"},
+    {eAVEncH264VLevel4_2, "4.2"},
+    {eAVEncH264VLevel5,   "5"},
+    {eAVEncH264VLevel5_1, "5.1"},
+    {eAVEncH264VLevel5_2, "5.2"},
+};
+
+static enum eAVEncH264VLevel level_from_string(const char *string)
+{
+    for (unsigned int i = 0; i < ARRAY_SIZE(h264_levels); ++i)
+    {
+        if (!strcmp(h264_levels[i].string, string))
+            return h264_levels[i].level;
+    }
+
+    GST_FIXME("Unrecognized level %s.\n", string);
+    return eAVEncH264VLevel1;
+}
+
+static void wg_format_from_caps_video_h264(struct wg_format *format, const GstCaps *caps)
+{
+    const GstStructure *structure = gst_caps_get_structure(caps, 0);
+    const gchar *stream_format, *profile, *level;
+    gint width, height, fps_n, fps_d;
+    const GValue *codec_data_value;
+    GstBuffer *codec_data;
+    GstMapInfo map;
+
+    if (!(stream_format = gst_structure_get_string(structure, "stream-format")))
+    {
+        GST_WARNING("Missing \"stream-format\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+
+    /* Windows requires byte-stream format. */
+    if (strcmp(stream_format, "byte-stream"))
+    {
+        GST_DEBUG("Rejecting stream format %s.\n", stream_format);
+        return;
+    }
+
+    if (!gst_structure_get_int(structure, "width", &width))
+    {
+        GST_WARNING("Missing \"width\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+    if (!gst_structure_get_int(structure, "height", &height))
+    {
+        GST_WARNING("Missing \"height\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+    if (!gst_structure_get_fraction(structure, "framerate", &fps_n, &fps_d))
+    {
+        fps_n = 0;
+        fps_d = 1;
+    }
+    if (!(profile = gst_structure_get_string(structure, "profile")))
+    {
+        GST_WARNING("Missing \"profile\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+    if (!(level = gst_structure_get_string(structure, "level")))
+    {
+        GST_WARNING("Missing \"level\" value in %" GST_PTR_FORMAT ".", caps);
+        return;
+    }
+
+    format->major_type = WG_MAJOR_TYPE_VIDEO_H264;
+    format->u.video.width = width;
+    format->u.video.height = height;
+    format->u.video.fps_n = fps_n;
+    format->u.video.fps_d = fps_d;
+
+    if (!strcmp(profile, "high"))
+        format->u.video.profile = eAVEncH264VProfile_High;
+    else if (!strcmp(profile, "high-4:4:4"))
+        format->u.video.profile = eAVEncH264VProfile_444;
+    else if (!strcmp(profile, "main"))
+        format->u.video.profile = eAVEncH264VProfile_Main;
+    else
+        GST_FIXME("Unhandled profile %s.\n", profile);
+
+    format->u.video.level = level_from_string(level);
+
+    if ((codec_data_value = gst_structure_get_value(structure, "streamheader"))
+            && (codec_data = gst_value_get_buffer(codec_data_value)))
+    {
+        gst_buffer_map(codec_data, &map, GST_MAP_READ);
+        if (map.size <= sizeof(format->u.video.codec_data))
+        {
+            format->u.video.codec_data_len = map.size;
+            memcpy(format->u.video.codec_data, map.data, map.size);
+        }
+        else
+            GST_WARNING("Too big codec_data value (%u) in %" GST_PTR_FORMAT ".", (UINT)map.size, caps);
+        gst_buffer_unmap(codec_data, &map);
+    }
 }
 
 void wg_format_from_caps(struct wg_format *format, const GstCaps *caps)
@@ -395,6 +575,7 @@ void wg_format_from_caps(struct wg_format *format, const GstCaps *caps)
     const GstStructure *structure = gst_caps_get_structure(caps, 0);
     const char *name = gst_structure_get_name(structure);
     gboolean parsed;
+    gint version;
 
     memset(format, 0, sizeof(*format));
 
@@ -416,6 +597,12 @@ void wg_format_from_caps(struct wg_format *format, const GstCaps *caps)
     {
         wg_format_from_caps_audio_mpeg1(format, caps);
     }
+    else if (!strcmp(name, "audio/mpeg")
+            && gst_structure_get_int(structure, "mpegversion", &version) && version == 4
+            && gst_structure_get_boolean(structure, "framed", &parsed) && parsed)
+    {
+        wg_format_from_caps_audio_mpeg4(format, caps);
+    }
     else if (!strcmp(name, "audio/x-wma"))
     {
         wg_format_from_caps_audio_wma(format, caps);
@@ -428,10 +615,16 @@ void wg_format_from_caps(struct wg_format *format, const GstCaps *caps)
     {
         wg_format_from_caps_video_wmv(format, caps);
     }
-    else if (!strcmp(name, "video/mpeg") && gst_structure_get_boolean(structure, "parsed", &parsed) && parsed)
+    else if (!strcmp(name, "video/mpeg")
+            && gst_structure_get_int(structure, "mpegversion", &version) && (version == 1 || version == 2)
+            && gst_structure_get_boolean(structure, "parsed", &parsed) && parsed)
     {
         wg_format_from_caps_video_mpeg1(format, caps);
     }
+    else if (!strcmp(name, "video/x-h264"))
+    {
+        wg_format_from_caps_video_h264(format, caps);
+    }
     else
     {
         GST_FIXME("Unhandled caps %" GST_PTR_FORMAT ".", caps);
@@ -506,9 +699,9 @@ static GstCaps *wg_format_to_caps_audio_mpeg1(const struct wg_format *format)
         return NULL;
 
     gst_caps_set_simple(caps, "mpegversion", G_TYPE_INT, 1, NULL);
-    gst_caps_set_simple(caps, "layer", G_TYPE_INT, format->u.audio_mpeg1.layer, NULL);
-    gst_caps_set_simple(caps, "rate", G_TYPE_INT, format->u.audio_mpeg1.rate, NULL);
-    gst_caps_set_simple(caps, "channels", G_TYPE_INT, format->u.audio_mpeg1.channels, NULL);
+    gst_caps_set_simple(caps, "layer", G_TYPE_INT, format->u.audio.layer, NULL);
+    gst_caps_set_simple(caps, "rate", G_TYPE_INT, format->u.audio.rate, NULL);
+    gst_caps_set_simple(caps, "channels", G_TYPE_INT, format->u.audio.channels, NULL);
     gst_caps_set_simple(caps, "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
 
     return caps;
@@ -516,6 +709,7 @@ static GstCaps *wg_format_to_caps_audio_mpeg1(const struct wg_format *format)
 
 static GstCaps *wg_format_to_caps_audio_mpeg4(const struct wg_format *format)
 {
+    static const size_t waveinfo_size = sizeof(HEAACWAVEINFO) - sizeof(WAVEFORMATEX);
     GstBuffer *buffer;
     GstCaps *caps;
 
@@ -524,7 +718,7 @@ static GstCaps *wg_format_to_caps_audio_mpeg4(const struct wg_format *format)
 
     gst_caps_set_simple(caps, "mpegversion", G_TYPE_INT, 4, NULL);
 
-    switch (format->u.audio_mpeg4.payload_type)
+    switch (format->u.audio.payload_type)
     {
         case 0: gst_caps_set_simple(caps, "stream-format", G_TYPE_STRING, "raw", NULL); break;
         case 1: gst_caps_set_simple(caps, "stream-format", G_TYPE_STRING, "adts", NULL); break;
@@ -534,10 +728,11 @@ static GstCaps *wg_format_to_caps_audio_mpeg4(const struct wg_format *format)
 
     /* FIXME: Use gst_codec_utils_aac_caps_set_level_and_profile from GStreamer pbutils library */
 
-    if (format->u.audio_mpeg4.codec_data_len)
+    if (format->u.audio.codec_data_len > waveinfo_size)
     {
-        buffer = gst_buffer_new_and_alloc(format->u.audio_mpeg4.codec_data_len);
-        gst_buffer_fill(buffer, 0, format->u.audio_mpeg4.codec_data, format->u.audio_mpeg4.codec_data_len);
+        buffer = gst_buffer_new_and_alloc(format->u.audio.codec_data_len - waveinfo_size);
+        gst_buffer_fill(buffer, 0, format->u.audio.codec_data + waveinfo_size,
+                format->u.audio.codec_data_len - waveinfo_size);
         gst_caps_set_simple(caps, "codec_data", GST_TYPE_BUFFER, buffer, NULL);
         gst_buffer_unref(buffer);
     }
@@ -568,6 +763,7 @@ static GstVideoFormat wg_video_format_to_gst(enum wg_video_format format)
         case WG_VIDEO_FORMAT_BGR:   return GST_VIDEO_FORMAT_BGR;
         case WG_VIDEO_FORMAT_RGB15: return GST_VIDEO_FORMAT_RGB15;
         case WG_VIDEO_FORMAT_RGB16: return GST_VIDEO_FORMAT_RGB16;
+        case WG_VIDEO_FORMAT_RGBA:  return GST_VIDEO_FORMAT_RGBA;
         case WG_VIDEO_FORMAT_AYUV:  return GST_VIDEO_FORMAT_AYUV;
         case WG_VIDEO_FORMAT_I420:  return GST_VIDEO_FORMAT_I420;
         case WG_VIDEO_FORMAT_NV12:  return GST_VIDEO_FORMAT_NV12;
@@ -622,12 +818,12 @@ static GstCaps *wg_format_to_caps_video_cinepak(const struct wg_format *format)
     if (!(caps = gst_caps_new_empty_simple("video/x-cinepak")))
         return NULL;
 
-    if (format->u.video_cinepak.width)
-        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video_cinepak.width, NULL);
-    if (format->u.video_cinepak.height)
-        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video_cinepak.height, NULL);
-    if (format->u.video_cinepak.fps_d || format->u.video_cinepak.fps_n)
-        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video_cinepak.fps_n, format->u.video_cinepak.fps_d, NULL);
+    if (format->u.video.width)
+        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video.width, NULL);
+    if (format->u.video.height)
+        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video.height, NULL);
+    if (format->u.video.fps_d || format->u.video.fps_n)
+        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video.fps_n, format->u.video.fps_d, NULL);
 
     return caps;
 }
@@ -639,29 +835,29 @@ static GstCaps *wg_format_to_caps_audio_wma(const struct wg_format *format)
 
     if (!(caps = gst_caps_new_empty_simple("audio/x-wma")))
         return NULL;
-    if (format->u.audio_wma.version)
-        gst_caps_set_simple(caps, "wmaversion", G_TYPE_INT, format->u.audio_wma.version, NULL);
+    if (format->u.audio.version)
+        gst_caps_set_simple(caps, "wmaversion", G_TYPE_INT, format->u.audio.version, NULL);
 
-    if (format->u.audio_wma.bitrate)
-        gst_caps_set_simple(caps, "bitrate", G_TYPE_INT, format->u.audio_wma.bitrate, NULL);
-    if (format->u.audio_wma.rate)
-        gst_caps_set_simple(caps, "rate", G_TYPE_INT, format->u.audio_wma.rate, NULL);
-    if (format->u.audio_wma.depth)
-        gst_caps_set_simple(caps, "depth", G_TYPE_INT, format->u.audio_wma.depth, NULL);
-    if (format->u.audio_wma.channels)
-        gst_caps_set_simple(caps, "channels", G_TYPE_INT, format->u.audio_wma.channels, NULL);
-    if (format->u.audio_wma.block_align)
-        gst_caps_set_simple(caps, "block_align", G_TYPE_INT, format->u.audio_wma.block_align, NULL);
+    if (format->u.audio.bitrate)
+        gst_caps_set_simple(caps, "bitrate", G_TYPE_INT, format->u.audio.bitrate, NULL);
+    if (format->u.audio.rate)
+        gst_caps_set_simple(caps, "rate", G_TYPE_INT, format->u.audio.rate, NULL);
+    if (format->u.audio.depth)
+        gst_caps_set_simple(caps, "depth", G_TYPE_INT, format->u.audio.depth, NULL);
+    if (format->u.audio.channels)
+        gst_caps_set_simple(caps, "channels", G_TYPE_INT, format->u.audio.channels, NULL);
+    if (format->u.audio.block_align)
+        gst_caps_set_simple(caps, "block_align", G_TYPE_INT, format->u.audio.block_align, NULL);
 
-    if (format->u.audio_wma.codec_data_len)
+    if (format->u.audio.codec_data_len)
     {
-        if (!(buffer = gst_buffer_new_and_alloc(format->u.audio_wma.codec_data_len)))
+        if (!(buffer = gst_buffer_new_and_alloc(format->u.audio.codec_data_len)))
         {
             gst_caps_unref(caps);
             return NULL;
         }
 
-        gst_buffer_fill(buffer, 0, format->u.audio_wma.codec_data, format->u.audio_wma.codec_data_len);
+        gst_buffer_fill(buffer, 0, format->u.audio.codec_data, format->u.audio.codec_data_len);
         gst_caps_set_simple(caps, "codec_data", GST_TYPE_BUFFER, buffer, NULL);
         gst_buffer_unref(buffer);
     }
@@ -680,20 +876,20 @@ static GstCaps *wg_format_to_caps_video_h264(const struct wg_format *format)
     gst_caps_set_simple(caps, "stream-format", G_TYPE_STRING, "byte-stream", NULL);
     gst_caps_set_simple(caps, "alignment", G_TYPE_STRING, "au", NULL);
 
-    if (format->u.video_h264.width)
-        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video_h264.width, NULL);
-    if (format->u.video_h264.height)
-        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video_h264.height, NULL);
-    if (format->u.video_h264.fps_n || format->u.video_h264.fps_d)
-        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video_h264.fps_n, format->u.video_h264.fps_d, NULL);
+    if (format->u.video.width)
+        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video.width, NULL);
+    if (format->u.video.height)
+        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video.height, NULL);
+    if (format->u.video.fps_n || format->u.video.fps_d)
+        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video.fps_n, format->u.video.fps_d, NULL);
 
-    switch (format->u.video_h264.profile)
+    switch (format->u.video.profile)
     {
         case eAVEncH264VProfile_Main: profile = "main"; break;
         case eAVEncH264VProfile_High: profile = "high"; break;
         case eAVEncH264VProfile_444:  profile = "high-4:4:4"; break;
         default:
-            GST_FIXME("H264 profile attribute %u not implemented.", format->u.video_h264.profile);
+            GST_FIXME("H264 profile attribute %u not implemented.", format->u.video.profile);
             /* fallthrough */
         case eAVEncH264VProfile_unknown:
             profile = "baseline";
@@ -701,7 +897,7 @@ static GstCaps *wg_format_to_caps_video_h264(const struct wg_format *format)
     }
     gst_caps_set_simple(caps, "profile", G_TYPE_STRING, profile, NULL);
 
-    switch (format->u.video_h264.level)
+    switch (format->u.video.level)
     {
         case eAVEncH264VLevel1:   level = "1";   break;
         case eAVEncH264VLevel1_1: level = "1.1"; break;
@@ -720,7 +916,7 @@ static GstCaps *wg_format_to_caps_video_h264(const struct wg_format *format)
         case eAVEncH264VLevel5_1: level = "5.1"; break;
         case eAVEncH264VLevel5_2: level = "5.2"; break;
         default:
-            GST_FIXME("H264 level attribute %u not implemented.", format->u.video_h264.level);
+            GST_FIXME("H264 level attribute %u not implemented.", format->u.video.level);
             /* fallthrough */
         case 0:
             level = NULL;
@@ -729,9 +925,9 @@ static GstCaps *wg_format_to_caps_video_h264(const struct wg_format *format)
     if (level)
         gst_caps_set_simple(caps, "level", G_TYPE_STRING, level, NULL);
 
-    if (format->u.video_h264.codec_data_len)
+    if (format->u.video.codec_data_len)
     {
-        if (!(buffer = gst_buffer_new_and_alloc(format->u.video_h264.codec_data_len)))
+        if (!(buffer = gst_buffer_new_and_alloc(format->u.video.codec_data_len)))
         {
             gst_caps_unref(caps);
             return NULL;
@@ -739,7 +935,7 @@ static GstCaps *wg_format_to_caps_video_h264(const struct wg_format *format)
 
         GST_BUFFER_PTS(buffer) = 0;
         GST_BUFFER_DTS(buffer) = 0;
-        gst_buffer_fill(buffer, 0, format->u.video_h264.codec_data, format->u.video_h264.codec_data_len);
+        gst_buffer_fill(buffer, 0, format->u.video.codec_data, format->u.video.codec_data_len);
         gst_caps_set_simple(caps, "streamheader", GST_TYPE_BUFFER, buffer, NULL);
         gst_buffer_unref(buffer);
     }
@@ -757,32 +953,32 @@ static GstCaps *wg_format_to_caps_video_wmv(const struct wg_format *format)
     if (!(caps = gst_caps_new_empty_simple("video/x-wmv")))
         return NULL;
 
-    switch (format->u.video_wmv.format)
+    switch (format->u.video.format)
     {
-        case WG_WMV_VIDEO_FORMAT_WMV1:
+        case WG_VIDEO_FORMAT_WMV1:
             wmv_format = "WMV1";
             wmv_version = 1;
             break;
-        case WG_WMV_VIDEO_FORMAT_WMV2:
+        case WG_VIDEO_FORMAT_WMV2:
             wmv_format = "WMV2";
             wmv_version = 2;
             break;
-        case WG_WMV_VIDEO_FORMAT_WMV3:
+        case WG_VIDEO_FORMAT_WMV3:
             wmv_format = "WMV3";
             wmv_version = 3;
             break;
-        case WG_WMV_VIDEO_FORMAT_WMVA:
+        case WG_VIDEO_FORMAT_WMVA:
             wmv_format = "WMVA";
             wmv_version = 3;
             break;
-        case WG_WMV_VIDEO_FORMAT_WVC1:
+        case WG_VIDEO_FORMAT_WVC1:
             wmv_format = "WVC1";
             wmv_version = 3;
             break;
         default:
-            GST_WARNING("Unknown WMV format %u.", format->u.video_wmv.format);
+            GST_WARNING("Unknown WMV format %u.", format->u.video.format);
             /* fallthrough */
-        case WG_WMV_VIDEO_FORMAT_UNKNOWN:
+        case WG_VIDEO_FORMAT_UNKNOWN:
             wmv_format = NULL;
             wmv_version = 0;
             break;
@@ -792,22 +988,22 @@ static GstCaps *wg_format_to_caps_video_wmv(const struct wg_format *format)
         gst_caps_set_simple(caps, "format", G_TYPE_STRING, wmv_format, NULL);
     if (wmv_version)
         gst_caps_set_simple(caps, "wmvversion", G_TYPE_INT, wmv_version, NULL);
-    if (format->u.video_wmv.width)
-        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video_wmv.width, NULL);
-    if (format->u.video_wmv.height)
-        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video_wmv.height, NULL);
-    if (format->u.video_wmv.fps_d || format->u.video_wmv.fps_n)
-        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video_wmv.fps_n, format->u.video_wmv.fps_d, NULL);
+    if (format->u.video.width)
+        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video.width, NULL);
+    if (format->u.video.height)
+        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video.height, NULL);
+    if (format->u.video.fps_d || format->u.video.fps_n)
+        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video.fps_n, format->u.video.fps_d, NULL);
 
-    if (format->u.video_wmv.codec_data_len)
+    if (format->u.video.codec_data_len)
     {
-        if (!(buffer = gst_buffer_new_and_alloc(format->u.video_wmv.codec_data_len)))
+        if (!(buffer = gst_buffer_new_and_alloc(format->u.video.codec_data_len)))
         {
             gst_caps_unref(caps);
             return NULL;
         }
 
-        gst_buffer_fill(buffer, 0, format->u.video_wmv.codec_data, format->u.video_wmv.codec_data_len);
+        gst_buffer_fill(buffer, 0, format->u.video.codec_data, format->u.video.codec_data_len);
         gst_caps_set_simple(caps, "codec_data", GST_TYPE_BUFFER, buffer, NULL);
         gst_buffer_unref(buffer);
     }
@@ -822,14 +1018,14 @@ static GstCaps *wg_format_to_caps_video_indeo(const struct wg_format *format)
     if (!(caps = gst_caps_new_empty_simple("video/x-indeo")))
         return NULL;
 
-    if (format->u.video_indeo.width)
-        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video_indeo.width, NULL);
-    if (format->u.video_indeo.height)
-        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video_indeo.height, NULL);
-    if (format->u.video_indeo.fps_d || format->u.video_indeo.fps_n)
-        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video_indeo.fps_n, format->u.video_indeo.fps_d, NULL);
-    if (format->u.video_indeo.version)
-        gst_caps_set_simple(caps, "indeoversion", G_TYPE_INT, format->u.video_indeo.version, NULL);
+    if (format->u.video.width)
+        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video.width, NULL);
+    if (format->u.video.height)
+        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video.height, NULL);
+    if (format->u.video.fps_d || format->u.video.fps_n)
+        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video.fps_n, format->u.video.fps_d, NULL);
+    if (format->u.video.version)
+        gst_caps_set_simple(caps, "indeoversion", G_TYPE_INT, format->u.video.version, NULL);
 
     return caps;
 }
@@ -844,12 +1040,12 @@ static GstCaps *wg_format_to_caps_video_mpeg1(const struct wg_format *format)
     gst_caps_set_simple(caps, "mpegversion", G_TYPE_INT, 1, NULL);
     gst_caps_set_simple(caps, "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
     gst_caps_set_simple(caps, "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
-    if (format->u.video_mpeg1.width)
-        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video_mpeg1.width, NULL);
-    if (format->u.video_mpeg1.height)
-        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video_mpeg1.height, NULL);
-    if (format->u.video_mpeg1.fps_d || format->u.video_cinepak.fps_n)
-        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video_mpeg1.fps_n, format->u.video_mpeg1.fps_d, NULL);
+    if (format->u.video.width)
+        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.video.width, NULL);
+    if (format->u.video.height)
+        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.video.height, NULL);
+    if (format->u.video.fps_d || format->u.video.fps_n)
+        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.video.fps_n, format->u.video.fps_d, NULL);
     return caps;
 }
 
@@ -916,14 +1112,14 @@ bool wg_format_compare(const struct wg_format *a, const struct wg_format *b)
 
         case WG_MAJOR_TYPE_VIDEO_CINEPAK:
             /* Do not compare FPS. */
-            return a->u.video_cinepak.width == b->u.video_cinepak.width
-                    && a->u.video_cinepak.height == b->u.video_cinepak.height;
+            return a->u.video.width == b->u.video.width
+                    && a->u.video.height == b->u.video.height;
 
         case WG_MAJOR_TYPE_VIDEO_WMV:
             /* Do not compare FPS. */
-            return a->u.video_wmv.format == b->u.video_wmv.format
-                    && a->u.video_wmv.width == b->u.video_wmv.width
-                    && a->u.video_wmv.height == b->u.video_wmv.height;
+            return a->u.video.format == b->u.video.format
+                    && a->u.video.width == b->u.video.width
+                    && a->u.video.height == b->u.video.height;
     }
 
     assert(0);
