@@ -44,9 +44,6 @@
     void* hwnd;
     WineEventQueue* queue;
 
-    void* surface;
-    pthread_mutex_t* surface_mutex;
-
     CGDirectDisplayID _lastDisplayID;
     NSTimeInterval _lastDisplayTime;
 
@@ -55,9 +52,6 @@
 
     BOOL shapeChangedSinceLastDraw;
 
-    BOOL colorKeyed;
-    CGFloat colorKeyRed, colorKeyGreen, colorKeyBlue;
-
     BOOL usePerPixelAlpha;
 
     NSUInteger lastModifierFlags;
@@ -98,6 +92,11 @@
 @property (readonly, getter=isFakingClose, nonatomic) BOOL fakingClose;
 @property (readonly, nonatomic) NSRect wine_fractionalFrame;
 
+/* Whether this window, when ordered in and not miniaturized, would appear to
+   the user on-screen. That means it has a non-zero size and is not empty-
+   shaped, or has a child window that meets those criteria. */
+@property (readonly, nonatomic) BOOL presentsVisibleContent;
+
     - (NSInteger) minimumLevelForActive:(BOOL)active;
     - (void) updateFullscreen;
 
