@@ -26,12 +26,16 @@
 
 #include "config.h"
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "macdrv.h"
 #include "oleidl.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(event);
 WINE_DECLARE_DEBUG_CHANNEL(imm);
 
+static pthread_mutex_t ime_mutex = PTHREAD_MUTEX_INITIALIZER;
+static RECT ime_composition_rect;
 
 /* return the name of an Mac event */
 static const char *dbgstr_event(int type)
@@ -60,6 +64,7 @@ static const char *dbgstr_event(int type)
         "STATUS_ITEM_MOUSE_MOVE",
         "WINDOW_BROUGHT_FORWARD",
         "WINDOW_CLOSE_REQUESTED",
+        "WINDOW_DID_MINIMIZE",
         "WINDOW_DID_UNMINIMIZE",
         "WINDOW_DRAG_BEGIN",
         "WINDOW_DRAG_END",
@@ -71,6 +76,7 @@ static const char *dbgstr_event(int type)
         "WINDOW_RESIZE_ENDED",
         "WINDOW_RESTORE_REQUESTED",
     };
+    C_ASSERT(ARRAYSIZE(event_names) == NUM_EVENT_TYPES);
 
     if (0 <= type && type < NUM_EVENT_TYPES) return event_names[type];
     return wine_dbg_sprintf("Unknown event %d", type);
@@ -144,6 +150,11 @@ static macdrv_event_mask get_event_mask(DWORD mask)
     return event_mask;
 }
 
+static void post_ime_update( HWND hwnd, UINT cursor_pos, WCHAR *comp_str, WCHAR *result_str )
+{
+    NtUserMessageCall( hwnd, WINE_IME_POST_UPDATE, cursor_pos, (LPARAM)comp_str,
+                       result_str, NtUserImeDriverCall, FALSE );
+}
 
 /***********************************************************************
  *              macdrv_im_set_text
@@ -151,26 +162,23 @@ static macdrv_event_mask get_event_mask(DWORD mask)
 static void macdrv_im_set_text(const macdrv_event *event)
 {
     HWND hwnd = macdrv_get_window_hwnd(event->window);
-    struct ime_set_text_params *params;
-    CFIndex length = 0, size;
+    WCHAR *text = NULL;
 
-    TRACE_(imm)("win %p/%p himc %p text %s complete %u\n", hwnd, event->window, event->im_set_text.data,
+    TRACE_(imm)("win %p/%p himc %p text %s complete %u\n", hwnd, event->window, event->im_set_text.himc,
                 debugstr_cf(event->im_set_text.text), event->im_set_text.complete);
 
     if (event->im_set_text.text)
-        length = CFStringGetLength(event->im_set_text.text);
+    {
+        CFIndex length = CFStringGetLength(event->im_set_text.text);
+        if (!(text = malloc((length + 1) * sizeof(WCHAR)))) return;
+        if (length) CFStringGetCharacters(event->im_set_text.text, CFRangeMake(0, length), text);
+        text[length] = 0;
+    }
 
-    size = offsetof(struct ime_set_text_params, text[length]);
-    if (!(params = malloc(size))) return;
-    params->hwnd = HandleToUlong(hwnd);
-    params->data = (UINT_PTR)event->im_set_text.data;
-    params->cursor_pos = event->im_set_text.cursor_pos;
-    params->complete = event->im_set_text.complete;
+    if (event->im_set_text.complete) post_ime_update(hwnd, -1, NULL, text);
+    else post_ime_update(hwnd, event->im_set_text.cursor_pos, text, NULL);
 
-    if (length)
-        CFStringGetCharacters(event->im_set_text.text, CFRangeMake(0, length), params->text);
-
-    macdrv_client_func(client_func_ime_set_text, params, size);
+    free(text);
 }
 
 /***********************************************************************
@@ -215,49 +223,60 @@ static uint32_t dropeffect_to_drag_operation(DWORD effect, uint32_t ops)
 
 
 /**************************************************************************
- *              query_drag_drop
+ *              query_drag_drop_drop
  */
-static BOOL query_drag_drop(macdrv_query *query)
+static BOOL query_drag_drop_drop(macdrv_query *query)
 {
     HWND hwnd = macdrv_get_window_hwnd(query->window);
-    struct macdrv_win_data *data = get_win_data(hwnd);
-    struct dnd_query_drop_params params;
+    struct macdrv_win_data *data;
 
-    if (!data)
+    if (!(data = get_win_data(hwnd)))
     {
         WARN("no win_data for win %p/%p\n", hwnd, query->window);
         return FALSE;
     }
 
-    params.hwnd = HandleToUlong(hwnd);
-    params.effect = drag_operations_to_dropeffects(query->drag_drop.op);
-    params.x = query->drag_drop.x + data->whole_rect.left;
-    params.y = query->drag_drop.y + data->whole_rect.top;
-    params.handle = (UINT_PTR)query->drag_drop.pasteboard;
     release_win_data(data);
-    return macdrv_client_func(client_func_dnd_query_drop, &params, sizeof(params));
+
+    NtUserMessageCall(hwnd, WINE_DRAG_DROP_DROP, 0, 0, NULL, NtUserDragDropCall, FALSE);
+    return TRUE;
 }
 
 /**************************************************************************
- *              query_drag_exited
+ *              query_drag_drop_enter
  */
-static BOOL query_drag_exited(macdrv_query *query)
+static BOOL query_drag_drop_enter(macdrv_query *query)
 {
-    struct dnd_query_exited_params params;
-    params.hwnd = HandleToUlong(macdrv_get_window_hwnd(query->window));
-    return macdrv_client_func(client_func_dnd_query_exited, &params, sizeof(params));
+    CFTypeRef pasteboard = query->drag_drop.pasteboard;
+    struct format_entry *entries;
+    UINT entries_size;
+
+    if (!(entries = get_format_entries(pasteboard, &entries_size))) return FALSE;
+    NtUserMessageCall(0, WINE_DRAG_DROP_ENTER, entries_size, (LPARAM)entries, NULL, NtUserDragDropCall, FALSE);
+    free(entries);
+
+    return TRUE;
+}
+
+/**************************************************************************
+ *              query_drag_drop_leave
+ */
+static BOOL query_drag_drop_leave(macdrv_query *query)
+{
+    NtUserMessageCall(0, WINE_DRAG_DROP_LEAVE, 0, 0, NULL, NtUserDragDropCall, FALSE);
+    return TRUE;
 }
 
 
 /**************************************************************************
- *              query_drag_operation
+ *              query_drag_drop_drag
  */
-static BOOL query_drag_operation(macdrv_query *query)
+static BOOL query_drag_drop_drag(macdrv_query *query)
 {
-    struct dnd_query_drag_params params;
     HWND hwnd = macdrv_get_window_hwnd(query->window);
     struct macdrv_win_data *data = get_win_data(hwnd);
     DWORD effect;
+    POINT point;
 
     if (!data)
     {
@@ -265,18 +284,15 @@ static BOOL query_drag_operation(macdrv_query *query)
         return FALSE;
     }
 
-    params.hwnd = HandleToUlong(hwnd);
-    params.effect = drag_operations_to_dropeffects(query->drag_operation.offered_ops);
-    params.x = query->drag_operation.x + data->whole_rect.left;
-    params.y = query->drag_operation.y + data->whole_rect.top;
-    params.handle = (UINT_PTR)query->drag_operation.pasteboard;
+    effect = drag_operations_to_dropeffects(query->drag_drop.ops);
+    point.x = query->drag_drop.x + data->rects.visible.left;
+    point.y = query->drag_drop.y + data->rects.visible.top;
     release_win_data(data);
 
-    effect = macdrv_client_func(client_func_dnd_query_drag, &params, sizeof(params));
+    effect = NtUserMessageCall(hwnd, WINE_DRAG_DROP_DRAG, MAKELONG(point.x, point.y), effect, NULL, NtUserDragDropCall, FALSE);
     if (!effect) return FALSE;
 
-    query->drag_operation.accepted_op = dropeffect_to_drag_operation(effect,
-                                                                     query->drag_operation.offered_ops);
+    query->drag_drop.ops = dropeffect_to_drag_operation(effect, query->drag_drop.ops);
     return TRUE;
 }
 
@@ -287,29 +303,43 @@ static BOOL query_drag_operation(macdrv_query *query)
 BOOL query_ime_char_rect(macdrv_query* query)
 {
     HWND hwnd = macdrv_get_window_hwnd(query->window);
-    void *himc = query->ime_char_rect.data;
+    void *himc = query->ime_char_rect.himc;
     CFRange *range = &query->ime_char_rect.range;
-    CGRect *rect = &query->ime_char_rect.rect;
-    struct ime_query_char_rect_result result = { .location = 0 };
-    struct ime_query_char_rect_params params;
-    BOOL ret;
 
     TRACE_(imm)("win %p/%p himc %p range %ld-%ld\n", hwnd, query->window, himc, range->location,
                 range->length);
 
-    params.hwnd = HandleToUlong(hwnd);
-    params.data = (UINT_PTR)himc;
-    params.result = (UINT_PTR)&result;
-    params.location = range->location;
-    params.length = range->length;
-    ret = macdrv_client_func(client_func_ime_query_char_rect, &params, sizeof(params));
-    *range = CFRangeMake(result.location, result.length);
-    *rect = cgrect_from_rect(result.rect);
+    pthread_mutex_lock(&ime_mutex);
+    query->ime_char_rect.rect = cgrect_from_rect(ime_composition_rect);
+    pthread_mutex_unlock(&ime_mutex);
 
-    TRACE_(imm)(" -> %s range %ld-%ld rect %s\n", ret ? "TRUE" : "FALSE", range->location,
-                range->length, wine_dbgstr_cgrect(*rect));
+    TRACE_(imm)(" -> range %ld-%ld rect %s\n", range->location,
+                range->length, wine_dbgstr_cgrect(query->ime_char_rect.rect));
 
-    return ret;
+    return TRUE;
+}
+
+
+/***********************************************************************
+ *      SetIMECompositionRect (MACDRV.@)
+ */
+BOOL macdrv_SetIMECompositionRect(HWND hwnd, RECT rect)
+{
+    TRACE("hwnd %p, rect %s\n", hwnd, wine_dbgstr_rect(&rect));
+    pthread_mutex_lock(&ime_mutex);
+    ime_composition_rect = rect;
+    pthread_mutex_unlock(&ime_mutex);
+    return TRUE;
+}
+
+
+/***********************************************************************
+ *      NotifyIMEStatus (MACDRV.@)
+ */
+void macdrv_NotifyIMEStatus( HWND hwnd, UINT status )
+{
+    TRACE_(imm)( "hwnd %p, status %#x\n", hwnd, status );
+    if (!status) macdrv_clear_ime_text();
 }
 
 
@@ -325,17 +355,21 @@ static void macdrv_query_event(HWND hwnd, const macdrv_event *event)
 
     switch (query->type)
     {
-        case QUERY_DRAG_DROP:
-            TRACE("QUERY_DRAG_DROP\n");
-            success = query_drag_drop(query);
+        case QUERY_DRAG_DROP_ENTER:
+            TRACE("QUERY_DRAG_DROP_ENTER\n");
+            success = query_drag_drop_enter(query);
             break;
-        case QUERY_DRAG_EXITED:
-            TRACE("QUERY_DRAG_EXITED\n");
-            success = query_drag_exited(query);
+        case QUERY_DRAG_DROP_LEAVE:
+            TRACE("QUERY_DRAG_DROP_LEAVE\n");
+            success = query_drag_drop_leave(query);
             break;
-        case QUERY_DRAG_OPERATION:
-            TRACE("QUERY_DRAG_OPERATION\n");
-            success = query_drag_operation(query);
+        case QUERY_DRAG_DROP_DRAG:
+            TRACE("QUERY_DRAG_DROP_DRAG\n");
+            success = query_drag_drop_drag(query);
+            break;
+        case QUERY_DRAG_DROP_DROP:
+            TRACE("QUERY_DRAG_DROP_DROP\n");
+            success = query_drag_drop_drop(query);
             break;
         case QUERY_IME_CHAR_RECT:
             TRACE("QUERY_IME_CHAR_RECT\n");
@@ -510,24 +544,16 @@ static int process_events(macdrv_event_queue queue, macdrv_event_mask mask)
 
 
 /***********************************************************************
- *              MsgWaitForMultipleObjectsEx   (MACDRV.@)
+ *              ProcessEvents   (MACDRV.@)
  */
-NTSTATUS macdrv_MsgWaitForMultipleObjectsEx(DWORD count, const HANDLE *handles,
-                                            const LARGE_INTEGER *timeout, DWORD mask, DWORD flags)
+BOOL macdrv_ProcessEvents(DWORD mask)
 {
-    DWORD ret;
     struct macdrv_thread_data *data = macdrv_thread_data();
     macdrv_event_mask event_mask = get_event_mask(mask);
 
-    TRACE("count %d, handles %p, timeout %p, mask %x, flags %x\n", (unsigned int)count,
-          handles, timeout, (unsigned int)mask, (unsigned int)flags);
+    TRACE("mask %x\n", (unsigned int)mask);
 
-    if (!data)
-    {
-        if (!count && timeout && !timeout->QuadPart) return WAIT_TIMEOUT;
-        return NtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
-                                         !!(flags & MWMO_ALERTABLE), timeout );
-    }
+    if (!data) return FALSE;
 
     if (data->current_event && data->current_event->type != QUERY_EVENT &&
         data->current_event->type != QUERY_EVENT_NO_PREEMPT_WAIT &&
@@ -535,14 +561,5 @@ NTSTATUS macdrv_MsgWaitForMultipleObjectsEx(DWORD count, const HANDLE *handles,
         data->current_event->type != WINDOW_DRAG_BEGIN)
         event_mask = 0;  /* don't process nested events */
 
-    if (process_events(data->queue, event_mask)) ret = count - 1;
-    else if (count || !timeout || timeout->QuadPart)
-    {
-        ret = NtWaitForMultipleObjects( count, handles, !(flags & MWMO_WAITALL),
-                                        !!(flags & MWMO_ALERTABLE), timeout );
-        if (ret == count - 1) process_events(data->queue, event_mask);
-    }
-    else ret = WAIT_TIMEOUT;
-
-    return ret;
+    return process_events(data->queue, event_mask);
 }
