@@ -398,18 +398,39 @@ static const struct {
     { VK_VOLUME_UP | 0x100,     "Volume Up" },
 };
 
-static BOOL char_matches_string(WCHAR wchar, UniChar *string, BOOL ignore_diacritics)
+
+static const struct {
+    WCHAR       wchar;
+    const char *name;
+} dead_key_names[] = {
+    { '^',                      "CIRCUMFLEX ACCENT" },
+    { '`',                      "GRAVE ACCENT" },
+    { 0x00B4,                   "ACUTE ACCENT" },
+    { '~',                      "TILDE" },
+    { 0x00A8,                   "DIAERESIS" },
+    { 0x00B8,                   "CEDILLA" },
+    { 0x02D8,                   "BREVE" },
+    { 0x02D9,                   "DOT ABOVE" },
+    { 0x00AF,                   "MACRON" },
+    { 0x02DA,                   "RING ABOVE" },
+    { 0x02DB,                   "OGONEK" },
+    { 0x02DC,                   "SMALL TILDE" },
+    { 0x02DD,                   "DOUBLE ACUTE ACCENT" },
+};
+
+
+static Boolean char_matches_string(WCHAR wchar, UniChar *string, CollatorRef collatorRef)
 {
-    BOOL ret;
-    CFStringRef s1 = CFStringCreateWithCharactersNoCopy(NULL, (UniChar*)&wchar, 1, kCFAllocatorNull);
-    CFStringRef s2 = CFStringCreateWithCharactersNoCopy(NULL, string, wcslen(string), kCFAllocatorNull);
-    CFStringCompareFlags flags = kCFCompareCaseInsensitive | kCFCompareNonliteral | kCFCompareWidthInsensitive;
-    if (ignore_diacritics)
-        flags |= kCFCompareDiacriticInsensitive;
-    ret = (CFStringCompare(s1, s2, flags) == kCFCompareEqualTo);
-    CFRelease(s1);
-    CFRelease(s2);
-    return ret;
+    Boolean equivalent;
+    OSStatus status;
+
+    status = UCCompareText(collatorRef, (UniChar*)&wchar, 1, string, wcslen(string), &equivalent, NULL);
+    if (status != noErr)
+    {
+        WARN("Failed to compare %s to %s\n", debugstr_wn(&wchar, 1), debugstr_w(string));
+        return FALSE;
+    }
+    return equivalent;
 }
 
 
@@ -657,6 +678,9 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
     int keyc;
     WCHAR vkey;
     const UCKeyboardLayout *uchr;
+    LocaleRef localeRef;
+    CollatorRef collatorRef, caseInsensitiveCollatorRef, diacriticInsensitiveCollatorRef;
+    UCCollateOptions collateOptions = 0;
     const UInt32 modifier_combos[] = {
         0,
         shiftKey >> 8,
@@ -672,51 +696,72 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
     static const struct {
         WCHAR wchar;
         DWORD vkey;
+        /* Mac virtual key code that must match wchar under the current layout.
+         * A value of -1 means match-any.
+         * TODO: replace -1 with the actual mac virtual key codes for all mappings
+         * and their respective layouts to avoid false matches, when possible. */
+        int mac_keyc;
     } symbol_vkeys[] = {
-        { '-', VK_OEM_MINUS },
-        { '+', VK_OEM_PLUS },
-        { '_', VK_OEM_MINUS },
-        { ',', VK_OEM_COMMA },
-        { '.', VK_OEM_PERIOD },
-        { '=', VK_OEM_PLUS },
-        { '>', VK_OEM_PERIOD },
-        { '<', VK_OEM_COMMA },
-        { '|', VK_OEM_5 },
-        { '\\', VK_OEM_5 },
-        { '`', VK_OEM_3 },
-        { '[', VK_OEM_4 },
-        { '~', VK_OEM_3 },
-        { '?', VK_OEM_2 },
-        { ']', VK_OEM_6 },
-        { '/', VK_OEM_2 },
-        { ':', VK_OEM_1 },
-        { '}', VK_OEM_6 },
-        { '{', VK_OEM_4 },
-        { ';', VK_OEM_1 },
-        { '\'', VK_OEM_7 },
-        { ':', VK_OEM_PERIOD },
-        { ';', VK_OEM_COMMA },
-        { '"', VK_OEM_7 },
-        { 0x00B4, VK_OEM_4 }, /* 0x00B4 is ACUTE ACCENT */
-        { '\'', VK_OEM_2 },
-        { 0x00A7, VK_OEM_5 }, /* 0x00A7 is SECTION SIGN */
-        { '*', VK_OEM_PLUS },
-        { 0x00B4, VK_OEM_7 },
-        { '`', VK_OEM_4 },
-        { '[', VK_OEM_6 },
-        { '/', VK_OEM_5 },
-        { '^', VK_OEM_6 },
-        { '*', VK_OEM_2 },
-        { '{', VK_OEM_6 },
-        { '~', VK_OEM_1 },
-        { '?', VK_OEM_PLUS },
-        { '?', VK_OEM_4 },
-        { 0x00B4, VK_OEM_3 },
-        { '?', VK_OEM_COMMA },
-        { '~', VK_OEM_PLUS },
-        { ']', VK_OEM_4 },
-        { '\'', VK_OEM_3 },
-        { 0x00A7, VK_OEM_7 },
+        { '-', VK_OEM_PLUS, kVK_ANSI_Equal },
+        { '-', VK_OEM_MINUS, -1 },
+        { '+', VK_OEM_PLUS, -1 },
+        { '_', VK_OEM_MINUS, -1 },
+        { ',', VK_OEM_COMMA, -1 },
+        { '.', VK_OEM_PERIOD, -1 },
+        { '=', VK_OEM_8, kVK_ANSI_Slash },
+        { '=', VK_OEM_PLUS, -1 },
+        { '!', VK_OEM_8, kVK_ANSI_Slash },
+        { 0x00F9, VK_OEM_3, kVK_ANSI_Quote }, /* 0x00F9 is French U accent grave */
+        { '$', VK_OEM_1, kVK_ANSI_RightBracket },
+        { ':', VK_OEM_2, kVK_ANSI_Period },
+        { '*', VK_OEM_5, kVK_ANSI_Backslash },
+        { '`', VK_OEM_5, kVK_ANSI_Backslash },
+        { ';', VK_OEM_PERIOD, kVK_ANSI_Comma },
+        { ')', VK_OEM_4, kVK_ANSI_Minus },
+        { '>', VK_OEM_PERIOD, -1 },
+        { '<', VK_OEM_COMMA, -1 },
+        { '|', VK_OEM_5, -1 },
+        { '\\', VK_OEM_5, -1 },
+        { '`', VK_OEM_3, -1 },
+        { '[', VK_OEM_4, -1 },
+        { '~', VK_OEM_3, -1 },
+        { 0x00DF, VK_OEM_4, kVK_ANSI_Minus }, /* 0x00DF is ESZETT */
+        { 0x00FC, VK_OEM_1, kVK_ANSI_LeftBracket }, /* 0x00FC is German U Umlaut */
+        { 0x00F6, VK_OEM_3, kVK_ANSI_Semicolon }, /* 0x00F6 is German O Umlaut */
+        { 0x00E4, VK_OEM_7, kVK_ANSI_Quote }, /* 0x00B4 is German A Umlaut */
+        { '?', VK_OEM_2, -1 },
+        { ']', VK_OEM_6, -1 },
+        { '/', VK_OEM_2, -1 },
+        { ':', VK_OEM_1, -1 },
+        { '}', VK_OEM_6, -1 },
+        { '{', VK_OEM_4,  },
+        { ';', VK_OEM_1, -1 },
+        { '\'', VK_OEM_7, -1 },
+        { ':', VK_OEM_PERIOD, -1 },
+        { ';', VK_OEM_COMMA, -1 },
+        { '"', VK_OEM_7,  -1 },
+        { 0x00B4, VK_OEM_4, kVK_ANSI_Equal }, /* 0x00B4 is ACUTE ACCENT */
+        { '\'', VK_OEM_2, -1 },
+        { 0x00A7, VK_OEM_5, -1 }, /* 0x00A7 is SECTION SIGN */
+        { '*', VK_OEM_PLUS, -1 },
+        { 0x00B4, VK_OEM_7, -1 },
+        { '`', VK_OEM_4, -1 },
+        { '[', VK_OEM_6, -1 },
+        { '/', VK_OEM_5, -1 },
+        { '^', VK_OEM_6, -1 },
+        { '*', VK_OEM_2, -1 },
+        { '{', VK_OEM_6, -1 },
+        { 0x00B4, VK_OEM_6, -1 },
+        { '~', VK_OEM_1, -1 },
+        { '?', VK_OEM_PLUS, -1 },
+        { '?', VK_OEM_4, -1 },
+        { 0x00B4, VK_OEM_3, -1 },
+        { '?', VK_OEM_COMMA, -1 },
+        { '~', VK_OEM_PLUS, -1 },
+        { ']', VK_OEM_4, -1 },
+        { '\'', VK_OEM_3, -1 },
+        { 0x00A7, VK_OEM_7, -1 },
+        { '<', VK_OEM_102, -1 },
     };
     int i;
 
@@ -778,6 +823,13 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
 
     uchr = (const UCKeyboardLayout*)CFDataGetBytePtr(thread_data->keyboard_layout_uchr);
 
+    LocaleRefFromLocaleString("POSIX", &localeRef);
+    UCCreateCollator(localeRef, 0, collateOptions, &collatorRef);
+    collateOptions |= kUCCollateComposeInsensitiveMask | kUCCollateWidthInsensitiveMask | kUCCollateCaseInsensitiveMask;
+    UCCreateCollator(localeRef, 0, collateOptions, &caseInsensitiveCollatorRef);
+    collateOptions |= kUCCollateDiacritInsensitiveMask;
+    UCCreateCollator(localeRef, 0, collateOptions, &diacriticInsensitiveCollatorRef);
+
     /* Using the keyboard layout, build a map of key code + modifiers -> characters. */
     memset(map, 0, sizeof(map));
     for (keyc = 0; keyc < ARRAY_SIZE(map); keyc++)
@@ -824,7 +876,7 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
                     if (thread_data->keyc2vkey[keyc] || !map[keyc][combo][0])
                         continue;
 
-                    if (char_matches_string(vkey, map[keyc][combo], ignore_diacritics))
+                    if (char_matches_string(vkey, map[keyc][combo], ignore_diacritics ? diacriticInsensitiveCollatorRef : caseInsensitiveCollatorRef))
                     {
                         thread_data->keyc2vkey[keyc] = vkey;
                         vkey_used[vkey] = 1;
@@ -850,7 +902,7 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
                 if (thread_data->keyc2vkey[keyc] || !map[keyc][combo][0])
                     continue;
 
-                if (char_matches_string(vkey, map[keyc][combo], FALSE))
+                if (char_matches_string(vkey, map[keyc][combo], collatorRef))
                 {
                     thread_data->keyc2vkey[keyc] = vkey;
                     vkey_used[vkey] = 1;
@@ -864,22 +916,24 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
 
     /* Now try to match key codes for certain common punctuation characters to
        the most common OEM vkeys (e.g. '.' to VK_OEM_PERIOD). */
-    for (i = 0; i < ARRAY_SIZE(symbol_vkeys); i++)
+    for (combo = 0; combo < ARRAY_SIZE(modifier_combos); combo++)
     {
-        vkey = symbol_vkeys[i].vkey;
-
-        if (vkey_used[vkey])
-            continue;
-
-        for (combo = 0; combo < ARRAY_SIZE(modifier_combos); combo++)
+        for (i = 0; i < ARRAY_SIZE(symbol_vkeys); i++)
         {
+            vkey = symbol_vkeys[i].vkey;
+
+            if (vkey_used[vkey])
+                continue;
+
             for (keyc = 0; keyc < ARRAY_SIZE(map); keyc++)
             {
                 if (!thread_data->keyc2scan[keyc]) continue; /* not a known Mac key code */
                 if (thread_data->keyc2vkey[keyc] || !map[keyc][combo][0])
                     continue;
+                if (symbol_vkeys[i].mac_keyc != -1 && symbol_vkeys[i].mac_keyc != keyc)
+                    continue;
 
-                if (char_matches_string(symbol_vkeys[i].wchar, map[keyc][combo], FALSE))
+                if (char_matches_string(symbol_vkeys[i].wchar, map[keyc][combo], collatorRef))
                 {
                     thread_data->keyc2vkey[keyc] = vkey;
                     vkey_used[vkey] = 1;
@@ -888,9 +942,6 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
                     break;
                 }
             }
-
-            if (vkey_used[vkey])
-                break;
         }
     }
 
@@ -982,6 +1033,10 @@ void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_data)
         vkey_used[vkey] = 1;
         TRACE("keyc 0x%04x -> vkey 0x%04x (spare vkey)\n", keyc, vkey);
     }
+
+    UCDisposeCollator(&collatorRef);
+    UCDisposeCollator(&caseInsensitiveCollatorRef);
+    UCDisposeCollator(&diacriticInsensitiveCollatorRef);
 }
 
 
@@ -1001,26 +1056,7 @@ static void macdrv_send_keyboard_input(HWND hwnd, WORD vkey, WORD scan, unsigned
     input.ki.time           = time;
     input.ki.dwExtraInfo    = 0;
 
-    __wine_send_input(hwnd, &input, NULL);
-}
-
-
-/***********************************************************************
- *           get_async_key_state
- */
-static BOOL get_async_key_state(BYTE state[256])
-{
-    BOOL ret;
-
-    SERVER_START_REQ(get_key_state)
-    {
-        req->async = 1;
-        req->key = -1;
-        wine_server_set_reply(req, state, 256);
-        ret = !wine_server_call(req);
-    }
-    SERVER_END_REQ;
-    return ret;
+    NtUserSendHardwareInput(hwnd, 0, &input, 0);
 }
 
 
@@ -1152,7 +1188,7 @@ void macdrv_hotkey_press(const macdrv_event *event)
         BOOL got_keystate;
         DWORD flags;
 
-        if ((got_keystate = get_async_key_state(keystate)))
+        if ((got_keystate = NtUserGetAsyncKeyboardState(keystate)))
         {
             update_modifier_state(MOD_ALT, event->hotkey_press.mod_flags, keystate, VK_LMENU, VK_RMENU,
                                   0x38, 0x138, event->hotkey_press.time_ms, FALSE);
@@ -1310,7 +1346,6 @@ INT macdrv_GetKeyNameText(LONG lparam, LPWSTR buffer, INT size)
     {
         if (thread_data->keyc2scan[keyc] == scan)
         {
-            static const WCHAR dead[] = {' ','d','e','a','d',0};
             const UCKeyboardLayout *uchr;
             UInt32 deadKeyState = 0;
             UniCharCount len;
@@ -1372,10 +1407,21 @@ INT macdrv_GetKeyNameText(LONG lparam, LPWSTR buffer, INT size)
 
             if (status == noErr && deadKeyState)
             {
-                lstrcpynW(buffer + len, dead, size - len);
-                len = wcslen(buffer);
+                for (i = 0; i < ARRAY_SIZE(dead_key_names); i++)
+                {
+                    if (dead_key_names[i].wchar == buffer[0])
+                    {
+                        len = min(strlen(dead_key_names[i].name) + 1, size);
+                        ascii_to_unicode(buffer, dead_key_names[i].name, len);
+                        if (len) buffer[--len] = 0;
+                        break;
+                    }
+                }
             }
 
+            if (status == noErr && len == 1 && buffer[0] >= 'a' && buffer[0] <= 'z')
+                buffer[0] += 'A' - 'a';
+
             TRACE("lparam 0x%08x -> %s\n", (unsigned int)lparam, debugstr_w(buffer));
             return len;
         }
@@ -1622,7 +1668,7 @@ INT macdrv_ToUnicodeEx(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
     UInt32 modifierKeyState;
     OptionBits options;
     UInt32 deadKeyState, savedDeadKeyState;
-    UniCharCount len;
+    UniCharCount len = 0;
     BOOL dead = FALSE;
 
     TRACE_(key)("virtKey 0x%04x scanCode 0x%04x lpKeyState %p bufW %p bufW_size %d flags 0x%08x hkl %p\n",
@@ -1758,7 +1804,7 @@ INT macdrv_ToUnicodeEx(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
             &savedDeadKeyState, bufW_size, &len, bufW);
         if (status != noErr)
         {
-            ERR_(key)("Couldn't translate keycode 0x%04x, status %d\n", keyc, status);
+            ERR_(key)("Couldn't translate dead keycode 0x%04x, status %d\n", keyc, status);
             goto done;
         }
 
@@ -1768,8 +1814,7 @@ INT macdrv_ToUnicodeEx(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
     if (len > 0)
         len = strip_apple_private_chars(bufW, len);
 
-    if (dead && len > 0) ret = -1;
-    else ret = len;
+    ret = dead ? -len : len;
 
     /* Control-Return produces line feed instead of carriage return. */
     if (ret > 0 && (lpKeyState[VK_CONTROL] & 0x80) && virtKey == VK_RETURN)
@@ -1783,10 +1828,10 @@ INT macdrv_ToUnicodeEx(UINT virtKey, UINT scanCode, const BYTE *lpKeyState,
 done:
     /* Null-terminate the buffer, if there's room.  MSDN clearly states that the
        caller must not assume this is done, but some programs (e.g. Audiosurf) do. */
-    if (1 <= ret && ret < bufW_size)
-        bufW[ret] = 0;
+    if (len < bufW_size)
+        bufW[len] = 0;
 
-    TRACE_(key)("returning %d / %s\n", ret, debugstr_wn(bufW, abs(ret)));
+    TRACE_(key)("returning %d / %s\n", ret, debugstr_wn(bufW, len));
     return ret;
 }
 
