@@ -52,7 +52,6 @@ int right_option_is_alt = 0;
 int left_command_is_ctrl = 0;
 int right_command_is_ctrl = 0;
 BOOL allow_software_rendering = FALSE;
-BOOL disable_window_decorations = FALSE;
 int allow_immovable_windows = TRUE;
 int use_confinement_cursor_clipping = TRUE;
 int cursor_clipping_locks_windows = TRUE;
@@ -61,6 +60,9 @@ int gl_surface_mode = GL_SURFACE_IN_FRONT_OPAQUE;
 int retina_enabled = FALSE;
 int enable_app_nap = FALSE;
 
+UINT64 app_icon_callback = 0;
+UINT64 app_quit_request_callback = 0;
+
 CFDictionaryRef localized_strings;
 
 
@@ -131,13 +133,13 @@ HKEY open_hkcu_key(const char *name)
             return 0;
 
         sid = ((TOKEN_USER *)sid_data)->User.Sid;
-        len = sprintf(buffer, "\\Registry\\User\\S-%u-%u", sid->Revision,
+        len = snprintf(buffer, sizeof(buffer), "\\Registry\\User\\S-%u-%u", sid->Revision,
                       (unsigned int)MAKELONG(MAKEWORD(sid->IdentifierAuthority.Value[5],
                                                       sid->IdentifierAuthority.Value[4]),
                                              MAKEWORD(sid->IdentifierAuthority.Value[3],
                                                       sid->IdentifierAuthority.Value[2])));
         for (i = 0; i < sid->SubAuthorityCount; i++)
-            len += sprintf(buffer + len, "-%u", (unsigned int)sid->SubAuthority[i]);
+            len += snprintf(buffer + len, sizeof(buffer) - len, "-%u", (unsigned int)sid->SubAuthority[i]);
 
         ascii_to_unicode(bufferW, buffer, len);
         hkcu = reg_open_key(NULL, bufferW, len * sizeof(WCHAR));
@@ -347,10 +349,6 @@ static void setup_options(void)
     if (!get_config_key(hkey, appkey, "AllowSoftwareRendering", buffer, sizeof(buffer)))
         allow_software_rendering = IS_OPTION_TRUE(buffer[0]);
 
-    /* Value name chosen to match what's used in the X11 driver. */
-    if (!get_config_key(hkey, appkey, "Decorated", buffer, sizeof(buffer)))
-        disable_window_decorations = !IS_OPTION_TRUE(buffer[0]);
-
     if (!get_config_key(hkey, appkey, "AllowImmovableWindows", buffer, sizeof(buffer)))
         allow_immovable_windows = IS_OPTION_TRUE(buffer[0]);
 
@@ -383,6 +381,8 @@ static void setup_options(void)
     if (!get_config_key(hkey, NULL, "RetinaMode", buffer, sizeof(buffer)))
         retina_enabled = IS_OPTION_TRUE(buffer[0]);
 
+    retina_on = retina_enabled;
+
     if (appkey) NtClose(appkey);
     if (hkey) NtClose(hkey);
 }
@@ -433,6 +433,9 @@ static NTSTATUS macdrv_init(void *arg)
     SessionAttributeBits attributes;
     OSStatus status;
 
+    app_icon_callback = params->app_icon_callback;
+    app_quit_request_callback = params->app_quit_request_callback;
+
     status = SessionGetInfo(callerSecuritySession, NULL, &attributes);
     if (status != noErr || !(attributes & sessionHasGraphicAccess))
         return STATUS_UNSUCCESSFUL;
@@ -449,7 +452,6 @@ static NTSTATUS macdrv_init(void *arg)
     }
 
     init_user_driver();
-    macdrv_init_display_devices(FALSE);
     return STATUS_SUCCESS;
 }
 
@@ -597,27 +599,6 @@ BOOL macdrv_SystemParametersInfo( UINT action, UINT int_param, void *ptr_param,
 }
 
 
-NTSTATUS macdrv_client_func(enum macdrv_client_funcs id, const void *params, ULONG size)
-{
-    void *ret_ptr;
-    ULONG ret_len;
-    return KeUserModeCallback(id, params, size, &ret_ptr, &ret_len);
-}
-
-
-static NTSTATUS macdrv_ime_clear(void *arg)
-{
-    macdrv_clear_ime_text();
-    return 0;
-}
-
-
-static NTSTATUS macdrv_ime_using_input_method(void *arg)
-{
-    return macdrv_using_input_method();
-}
-
-
 static NTSTATUS macdrv_quit_result(void *arg)
 {
     struct quit_result_params *params = arg;
@@ -628,16 +609,7 @@ static NTSTATUS macdrv_quit_result(void *arg)
 
 const unixlib_entry_t __wine_unix_call_funcs[] =
 {
-    macdrv_dnd_get_data,
-    macdrv_dnd_get_formats,
-    macdrv_dnd_have_format,
-    macdrv_dnd_release,
-    macdrv_dnd_retain,
-    macdrv_ime_clear,
-    macdrv_ime_process_text_input,
-    macdrv_ime_using_input_method,
     macdrv_init,
-    macdrv_notify_icon,
     macdrv_quit_result,
 };
 
@@ -645,125 +617,25 @@ C_ASSERT( ARRAYSIZE(__wine_unix_call_funcs) == unix_funcs_count );
 
 #ifdef _WIN64
 
-static NTSTATUS wow64_dnd_get_data(void *arg)
-{
-    struct
-    {
-        UINT64 handle;
-        UINT format;
-        ULONG size;
-        ULONG data;
-    } *params32 = arg;
-    struct dnd_get_data_params params;
-
-    params.handle = params32->handle;
-    params.format = params32->format;
-    params.size = params32->size;
-    params.data = UlongToPtr(params32->data);
-    return macdrv_dnd_get_data(&params);
-}
-
-static NTSTATUS wow64_ime_process_text_input(void *arg)
-{
-    struct
-    {
-        UINT vkey;
-        UINT scan;
-        UINT repeat;
-        ULONG key_state;
-        ULONG himc;
-        ULONG done;
-    } *params32 = arg;
-    struct process_text_input_params params;
-
-    params.vkey = params32->vkey;
-    params.scan = params32->scan;
-    params.repeat = params32->repeat;
-    params.key_state = UlongToPtr(params32->key_state);
-    params.himc = UlongToPtr(params32->himc);
-    params.done = UlongToPtr(params32->done);
-    return macdrv_ime_process_text_input(&params);
-}
-
 static NTSTATUS wow64_init(void *arg)
 {
     struct
     {
         ULONG strings;
+        UINT64 app_icon_callback;
+        UINT64 app_quit_request_callback;
     } *params32 = arg;
     struct init_params params;
 
     params.strings = UlongToPtr(params32->strings);
+    params.app_icon_callback = params32->app_icon_callback;
+    params.app_quit_request_callback = params32->app_quit_request_callback;
     return macdrv_init(&params);
 }
 
-static NTSTATUS wow64_notify_icon(void *arg)
-{
-    struct
-    {
-        DWORD msg;
-        ULONG data;
-    } *params32 = arg;
-    struct
-    {
-        DWORD cbSize;
-        ULONG hWnd;
-        UINT uID;
-        UINT uFlags;
-        UINT uCallbackMessage;
-        ULONG hIcon;
-        WCHAR szTip[128];
-        DWORD dwState;
-        DWORD dwStateMask;
-        WCHAR szInfo[256];
-        UINT uTimeout;
-        WCHAR szInfoTitle[64];
-        DWORD dwInfoFlags;
-        GUID guidItem;
-        ULONG hBalloonIcon;
-    } *data32 = UlongToPtr(params32->data);
-
-    struct notify_icon_params params;
-    NOTIFYICONDATAW data;
-
-    params.msg = params32->msg;
-    params.data = &data;
-
-    data.cbSize = sizeof(data);
-    data.hWnd = UlongToHandle(data32->hWnd);
-    data.uID = data32->uID;
-    data.uFlags = data32->uFlags;
-    data.uCallbackMessage = data32->uCallbackMessage;
-    data.hIcon = UlongToHandle(data32->hIcon);
-    if (data.uFlags & NIF_TIP)
-        wcscpy(data.szTip, data32->szTip);
-    data.dwState = data32->dwState;
-    data.dwStateMask = data32->dwStateMask;
-    if (data.uFlags & NIF_INFO)
-    {
-        wcscpy(data.szInfoTitle, data32->szInfoTitle);
-        wcscpy(data.szInfo, data32->szInfo);
-        data.uTimeout = data32->uTimeout;
-        data.dwInfoFlags = data32->dwInfoFlags;
-    }
-    data.guidItem = data32->guidItem;
-    data.hBalloonIcon = UlongToHandle(data32->hBalloonIcon);
-
-    return macdrv_notify_icon(&params);
-}
-
 const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
 {
-    wow64_dnd_get_data,
-    macdrv_dnd_get_formats,
-    macdrv_dnd_have_format,
-    macdrv_dnd_release,
-    macdrv_dnd_retain,
-    macdrv_ime_clear,
-    wow64_ime_process_text_input,
-    macdrv_ime_using_input_method,
     wow64_init,
-    wow64_notify_icon,
     macdrv_quit_result,
 };
 
