@@ -551,7 +551,7 @@ static HRESULT setup_oss_device(AUDCLNT_SHAREMODE share, int fd,
     if(ret == S_FALSE && !out)
         ret = AUDCLNT_E_UNSUPPORTED_FORMAT;
 
-    if(ret == S_FALSE && out){
+    if(ret == S_FALSE){
         closest->Format.nBlockAlign =
             closest->Format.nChannels * closest->Format.wBitsPerSample / 8;
         closest->Format.nAvgBytesPerSec =
@@ -576,36 +576,6 @@ static NTSTATUS oss_create_stream(void *args)
 
     params->result = S_OK;
 
-    if (params->share == AUDCLNT_SHAREMODE_SHARED) {
-        params->period = def_period;
-        if (params->duration < 3 * params->period)
-            params->duration = 3 * params->period;
-    } else {
-        if (fmtex->Format.wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
-           (fmtex->dwChannelMask == 0 || fmtex->dwChannelMask & SPEAKER_RESERVED))
-            params->result = AUDCLNT_E_UNSUPPORTED_FORMAT;
-        else {
-            if (!params->period)
-                params->period = def_period;
-            if (params->period < min_period || params->period > 5000000)
-                params->result = AUDCLNT_E_INVALID_DEVICE_PERIOD;
-            else if (params->duration > 20000000) /* The smaller the period, the lower this limit. */
-                params->result = AUDCLNT_E_BUFFER_SIZE_ERROR;
-            else if (params->flags & AUDCLNT_STREAMFLAGS_EVENTCALLBACK) {
-                if (params->duration != params->period)
-                    params->result = AUDCLNT_E_BUFDURATION_PERIOD_NOT_EQUAL;
-
-                FIXME("EXCLUSIVE mode with EVENTCALLBACK\n");
-
-                params->result = AUDCLNT_E_DEVICE_IN_USE;
-            } else if (params->duration < 8 * params->period)
-                params->duration = 8 * params->period; /* May grow above 2s. */
-        }
-    }
-
-    if (FAILED(params->result))
-        return STATUS_SUCCESS;
-
     stream = calloc(1, sizeof(*stream));
     if(!stream){
         params->result = E_OUTOFMEMORY;
@@ -1721,6 +1691,7 @@ const unixlib_entry_t __wine_unix_call_funcs[] =
     oss_get_capture_buffer,
     oss_release_capture_buffer,
     oss_is_format_supported,
+    oss_not_implemented,
     oss_get_mix_format,
     oss_get_device_period,
     oss_get_buffer_size,
@@ -1731,6 +1702,7 @@ const unixlib_entry_t __wine_unix_call_funcs[] =
     oss_get_position,
     oss_set_volumes,
     oss_set_event_handle,
+    oss_not_implemented,
     oss_test_connect,
     oss_is_started,
     oss_get_prop_value,
@@ -2216,6 +2188,7 @@ const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
     oss_wow64_get_capture_buffer,
     oss_release_capture_buffer,
     oss_wow64_is_format_supported,
+    oss_not_implemented,
     oss_wow64_get_mix_format,
     oss_wow64_get_device_period,
     oss_wow64_get_buffer_size,
@@ -2226,6 +2199,7 @@ const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
     oss_wow64_get_position,
     oss_wow64_set_volumes,
     oss_wow64_set_event_handle,
+    oss_not_implemented,
     oss_wow64_test_connect,
     oss_is_started,
     oss_wow64_get_prop_value,
