@@ -358,7 +358,7 @@ static void add_xkb_layout(const char *xkb_layout, struct xkb_keymap *xkb_keymap
 
     unsigned int mod, keyc, len, names_len, min_keycode, max_keycode;
     struct xkb_state *xkb_state = xkb_state_new(xkb_keymap);
-    xkb_mod_mask_t shift_mask, control_mask, altgr_mask, capslock_mask;
+    xkb_mod_mask_t shift_mask, control_mask, altgr_mask, capslock_mask, numlock_mask;
     VSC_LPWSTR *names_entry, *names_ext_entry;
     VSC_VK *vsc2vk_e0_entry, *vsc2vk_e1_entry;
     VK_TO_WCHARS8 *vk2wchars_entry;
@@ -493,6 +493,7 @@ static void add_xkb_layout(const char *xkb_layout, struct xkb_keymap *xkb_keymap
     control_mask = 1 << xkb_keymap_mod_get_index(xkb_keymap, XKB_MOD_NAME_CTRL);
     capslock_mask = 1 << xkb_keymap_mod_get_index(xkb_keymap, XKB_MOD_NAME_CAPS);
     altgr_mask = 1 << xkb_keymap_mod_get_index(xkb_keymap, "Mod5");
+    numlock_mask = 1 << xkb_keymap_mod_get_index(xkb_keymap, XKB_MOD_NAME_NUM);
 
     for (keyc = min_keycode; keyc <= max_keycode; keyc++)
     {
@@ -502,6 +503,19 @@ static void add_xkb_layout(const char *xkb_layout, struct xkb_keymap *xkb_keymap
         uint32_t caps_ret, shift_ret;
         unsigned int mod;
 
+        if ((vkey & KBDNUMPAD) && (vkey & 0xff) == VK_DELETE)
+        {
+            VK_TO_WCHARS8 num_vkey2wch = {.VirtualKey = VK_DECIMAL};
+
+            xkb_state_update_mask(xkb_state, 0, 0, numlock_mask, 0, 0, xkb_group);
+            if (!(num_vkey2wch.wch[0] = xkb_state_key_get_utf32(xkb_state, keyc)))
+                num_vkey2wch.wch[0] = WCH_NONE;
+            for (mod = 1; mod < 8; ++mod) num_vkey2wch.wch[mod] = WCH_NONE;
+            num_vkey2wch.Attributes = 0;
+            TRACE("vkey %#06x -> %s\n", num_vkey2wch.VirtualKey, debugstr_wn(num_vkey2wch.wch, 8));
+            *vk2wchars_entry++ = num_vkey2wch;
+        }
+
         for (mod = 0; mod < 8; ++mod)
         {
             xkb_mod_mask_t mod_mask = 0;
@@ -603,29 +617,13 @@ static BOOL find_xkb_layout_variant(const char *name, const char **layout, const
     return FALSE;
 }
 
-static BOOL get_async_key_state(BYTE state[256])
-{
-    BOOL ret;
-
-    SERVER_START_REQ(get_key_state)
-    {
-        req->async = 1;
-        req->key = -1;
-        wine_server_set_reply(req, state, 256);
-        ret = !wine_server_call(req);
-    }
-    SERVER_END_REQ;
-
-    return ret;
-}
-
 static void release_all_keys(HWND hwnd)
 {
     BYTE state[256];
     int vkey;
     INPUT input = {.type = INPUT_KEYBOARD};
 
-    get_async_key_state(state);
+    NtUserGetAsyncKeyboardState(state);
 
     for (vkey = 1; vkey < 256; vkey++)
     {
@@ -642,7 +640,7 @@ static void release_all_keys(HWND hwnd)
             input.ki.wScan = scan & 0xff;
             input.ki.dwFlags = KEYEVENTF_KEYUP;
             if (scan & ~0xff) input.ki.dwFlags |= KEYEVENTF_EXTENDEDKEY;
-            __wine_send_input(hwnd, &input, NULL);
+            NtUserSendHardwareInput(hwnd, 0, &input, 0);
         }
     }
 }
@@ -738,12 +736,13 @@ static void keyboard_handle_keymap(void *data, struct wl_keyboard *wl_keyboard,
     xkb_keymap_unref(xkb_keymap);
 }
 
-static void keyboard_handle_enter(void *data, struct wl_keyboard *wl_keyboard,
+static void keyboard_handle_enter(void *private, struct wl_keyboard *wl_keyboard,
                                   uint32_t serial, struct wl_surface *wl_surface,
                                   struct wl_array *keys)
 {
     struct wayland_keyboard *keyboard = &process_wayland.keyboard;
     struct wayland_surface *surface;
+    struct wayland_win_data *data;
     HWND hwnd;
 
     if (!wl_surface) return;
@@ -760,7 +759,9 @@ static void keyboard_handle_enter(void *data, struct wl_keyboard *wl_keyboard,
     NtUserPostMessage(keyboard->focused_hwnd, WM_INPUTLANGCHANGEREQUEST, 0 /*FIXME*/,
                       (LPARAM)keyboard_hkl);
 
-    if ((surface = wayland_surface_lock_hwnd(hwnd)))
+    if (!(data = wayland_win_data_get(hwnd))) return;
+
+    if ((surface = data->wayland_surface))
     {
         /* TODO: Drop the internal message and call NtUserSetForegroundWindow
          * directly once it's updated to not explicitly deactivate the old
@@ -768,8 +769,9 @@ static void keyboard_handle_enter(void *data, struct wl_keyboard *wl_keyboard,
          * are in the same non-current thread. */
         if (surface->window.managed)
             NtUserPostMessage(hwnd, WM_WAYLAND_SET_FOREGROUND, 0, 0);
-        pthread_mutex_unlock(&surface->mutex);
     }
+
+    wayland_win_data_release(data);
 }
 
 static void keyboard_handle_leave(void *data, struct wl_keyboard *wl_keyboard,
@@ -801,11 +803,10 @@ static void send_right_control(HWND hwnd, uint32_t state)
 {
     INPUT input = {0};
     input.type = INPUT_KEYBOARD;
-    input.ki.wScan = key2scan(KEY_RIGHTCTRL);
-    input.ki.wVk = VK_RCONTROL;
-    input.ki.dwFlags |= KEYEVENTF_EXTENDEDKEY;
+    input.ki.wScan = 0xe000 | (key2scan(KEY_RIGHTCTRL) & 0xff);
+    input.ki.dwFlags = KEYEVENTF_SCANCODE | KEYEVENTF_EXTENDEDKEY;
     if (state == WL_KEYBOARD_KEY_STATE_RELEASED) input.ki.dwFlags |= KEYEVENTF_KEYUP;
-    __wine_send_input(hwnd, &input, NULL);
+    NtUserSendHardwareInput(hwnd, 0, &input, 0);
 }
 
 static void keyboard_handle_key(void *data, struct wl_keyboard *wl_keyboard,
@@ -824,12 +825,12 @@ static void keyboard_handle_key(void *data, struct wl_keyboard *wl_keyboard,
     if (key == KEY_RIGHTALT) send_right_control(hwnd, state);
 
     input.type = INPUT_KEYBOARD;
-    input.ki.wScan = scan & 0xff;
-    input.ki.wVk = NtUserMapVirtualKeyEx(scan, MAPVK_VSC_TO_VK_EX, keyboard_hkl);
+    input.ki.wScan = (scan & 0x300) ? scan + 0xdf00 : scan;
+    input.ki.dwFlags = KEYEVENTF_SCANCODE;
     if (scan & ~0xff) input.ki.dwFlags |= KEYEVENTF_EXTENDEDKEY;
 
     if (state == WL_KEYBOARD_KEY_STATE_RELEASED) input.ki.dwFlags |= KEYEVENTF_KEYUP;
-    __wine_send_input(hwnd, &input, NULL);
+    NtUserSendHardwareInput(hwnd, 0, &input, 0);
 }
 
 static void keyboard_handle_modifiers(void *data, struct wl_keyboard *wl_keyboard,
