@@ -1079,7 +1079,7 @@ static inline BOOL image_needs_byteswap( XImage *image, BOOL is_r8g8b8, int bit_
 }
 
 /* copy image bits with byte swapping and/or pixel mapping */
-static void copy_image_byteswap( BITMAPINFO *info, const unsigned char *src, unsigned char *dst,
+static void copy_image_byteswap( const BITMAPINFO *info, const unsigned char *src, unsigned char *dst,
                                  int src_stride, int dst_stride, int height, BOOL byteswap,
                                  const int *mapping, unsigned int zeropad_mask, unsigned int alpha_bits )
 {
@@ -1570,25 +1570,25 @@ DWORD get_pixmap_image( Pixmap pixmap, int width, int height, const XVisualInfo
     return ret;
 }
 
+#ifdef HAVE_X11_EXTENSIONS_XSHM_H
+typedef XShmSegmentInfo x11drv_xshm_info_t;
+#else
+typedef struct { int shmid; } x11drv_xshm_info_t;
+#endif
+
+struct x11drv_image
+{
+    XImage               *ximage;    /* XImage used for X11 drawing */
+    x11drv_xshm_info_t    shminfo;   /* XSHM extension info */
+};
 
 struct x11drv_window_surface
 {
     struct window_surface header;
     Window                window;
     GC                    gc;
-    XImage               *image;
-    RECT                  bounds;
+    struct x11drv_image  *image;
     BOOL                  byteswap;
-    BOOL                  is_argb;
-    DWORD                 alpha_bits;
-    COLORREF              color_key;
-    HRGN                  region;
-    void                 *bits;
-#ifdef HAVE_LIBXXSHM
-    XShmSegmentInfo       shminfo;
-#endif
-    pthread_mutex_t       mutex;
-    BITMAPINFO            info;   /* variable size, must be last */
 };
 
 static struct x11drv_window_surface *get_x11_surface( struct window_surface *surface )
@@ -1596,201 +1596,13 @@ static struct x11drv_window_surface *get_x11_surface( struct window_surface *sur
     return (struct x11drv_window_surface *)surface;
 }
 
-static inline UINT get_color_component( UINT color, UINT mask )
-{
-    int shift;
-    for (shift = 0; !(mask & 1); shift++) mask >>= 1;
-    return (color * mask / 255) << shift;
-}
-
-#ifdef HAVE_LIBXSHAPE
-static inline void flush_rgn_data( HRGN rgn, RGNDATA *data )
-{
-    HRGN tmp = NtGdiExtCreateRegion( NULL, data->rdh.dwSize + data->rdh.nRgnSize, data );
-    NtGdiCombineRgn( rgn, rgn, tmp, RGN_OR );
-    NtGdiDeleteObjectApp( tmp );
-    data->rdh.nCount = 0;
-}
-
-static inline void add_row( HRGN rgn, RGNDATA *data, int x, int y, int len )
-{
-    RECT *rect = (RECT *)data->Buffer + data->rdh.nCount;
-
-    if (len <= 0) return;
-    rect->left   = x;
-    rect->top    = y;
-    rect->right  = x + len;
-    rect->bottom = y + 1;
-    data->rdh.nCount++;
-    if (data->rdh.nCount * sizeof(RECT) > data->rdh.nRgnSize - sizeof(RECT))
-        flush_rgn_data( rgn, data );
-}
-#endif
-
-/***********************************************************************
- *           update_surface_region
- */
-static void update_surface_region( struct x11drv_window_surface *surface )
-{
-#ifdef HAVE_LIBXSHAPE
-    char buffer[4096];
-    RGNDATA *data = (RGNDATA *)buffer;
-    BITMAPINFO *info = &surface->info;
-    UINT *masks = (UINT *)info->bmiColors;
-    int x, y, start, width;
-    HRGN rgn;
-
-    if (!shape_layered_windows) return;
-
-    if (!surface->is_argb && surface->color_key == CLR_INVALID)
-    {
-        XShapeCombineMask( gdi_display, surface->window, ShapeBounding, 0, 0, None, ShapeSet );
-        return;
-    }
-
-    data->rdh.dwSize = sizeof(data->rdh);
-    data->rdh.iType  = RDH_RECTANGLES;
-    data->rdh.nCount = 0;
-    data->rdh.nRgnSize = sizeof(buffer) - sizeof(data->rdh);
-
-    rgn = NtGdiCreateRectRgn( 0, 0, 0, 0 );
-    width = surface->header.rect.right - surface->header.rect.left;
-
-    switch (info->bmiHeader.biBitCount)
-    {
-    case 16:
-    {
-        WORD *bits = surface->bits;
-        int stride = (width + 1) & ~1;
-        UINT mask = masks[0] | masks[1] | masks[2];
-
-        for (y = surface->header.rect.top; y < surface->header.rect.bottom; y++, bits += stride)
-        {
-            x = 0;
-            while (x < width)
-            {
-                while (x < width && (bits[x] & mask) == surface->color_key) x++;
-                start = x;
-                while (x < width && (bits[x] & mask) != surface->color_key) x++;
-                add_row( rgn, data, surface->header.rect.left + start, y, x - start );
-            }
-        }
-        break;
-    }
-    case 24:
-    {
-        BYTE *bits = surface->bits;
-        int stride = (width * 3 + 3) & ~3;
-
-        for (y = surface->header.rect.top; y < surface->header.rect.bottom; y++, bits += stride)
-        {
-            x = 0;
-            while (x < width)
-            {
-                while (x < width &&
-                       (bits[x * 3] == GetBValue(surface->color_key)) &&
-                       (bits[x * 3 + 1] == GetGValue(surface->color_key)) &&
-                       (bits[x * 3 + 2] == GetRValue(surface->color_key)))
-                    x++;
-                start = x;
-                while (x < width &&
-                       ((bits[x * 3] != GetBValue(surface->color_key)) ||
-                        (bits[x * 3 + 1] != GetGValue(surface->color_key)) ||
-                        (bits[x * 3 + 2] != GetRValue(surface->color_key))))
-                    x++;
-                add_row( rgn, data, surface->header.rect.left + start, y, x - start );
-            }
-        }
-        break;
-    }
-    case 32:
-    {
-        DWORD *bits = surface->bits;
-
-        if (info->bmiHeader.biCompression == BI_RGB)
-        {
-            for (y = surface->header.rect.top; y < surface->header.rect.bottom; y++, bits += width)
-            {
-                x = 0;
-                while (x < width)
-                {
-                    while (x < width &&
-                           ((bits[x] & 0xffffff) == surface->color_key ||
-                            (surface->is_argb && !(bits[x] & 0xff000000)))) x++;
-                    start = x;
-                    while (x < width &&
-                           !((bits[x] & 0xffffff) == surface->color_key ||
-                             (surface->is_argb && !(bits[x] & 0xff000000)))) x++;
-                    add_row( rgn, data, surface->header.rect.left + start, y, x - start );
-                }
-            }
-        }
-        else
-        {
-            UINT mask = masks[0] | masks[1] | masks[2];
-            for (y = surface->header.rect.top; y < surface->header.rect.bottom; y++, bits += width)
-            {
-                x = 0;
-                while (x < width)
-                {
-                    while (x < width && (bits[x] & mask) == surface->color_key) x++;
-                    start = x;
-                    while (x < width && (bits[x] & mask) != surface->color_key) x++;
-                    add_row( rgn, data, surface->header.rect.left + start, y, x - start );
-                }
-            }
-        }
-        break;
-    }
-    default:
-        assert(0);
-    }
-
-    if (data->rdh.nCount) flush_rgn_data( rgn, data );
-
-    if ((data = X11DRV_GetRegionData( rgn, 0 )))
-    {
-        XShapeCombineRectangles( gdi_display, surface->window, ShapeBounding, 0, 0,
-                                 (XRectangle *)data->Buffer, data->rdh.nCount, ShapeSet, YXBanded );
-        free( data );
-    }
-
-    NtGdiDeleteObjectApp( rgn );
-#endif
-}
-
-/***********************************************************************
- *           set_color_key
- */
-static void set_color_key( struct x11drv_window_surface *surface, COLORREF key )
-{
-    UINT *masks = (UINT *)surface->info.bmiColors;
-
-    if (key == CLR_INVALID)
-        surface->color_key = CLR_INVALID;
-    else if (surface->info.bmiHeader.biBitCount <= 8)
-        surface->color_key = CLR_INVALID;
-    else if (key & (1 << 24))  /* PALETTEINDEX */
-        surface->color_key = 0;
-    else if (key >> 16 == 0x10ff)  /* DIBINDEX */
-        surface->color_key = 0;
-    else if (surface->info.bmiHeader.biBitCount == 24)
-        surface->color_key = key;
-    else if (surface->info.bmiHeader.biCompression == BI_RGB)
-        surface->color_key = (GetRValue(key) << 16) | (GetGValue(key) << 8) | GetBValue(key);
-    else
-        surface->color_key = get_color_component( GetRValue(key), masks[0] ) |
-                             get_color_component( GetGValue(key), masks[1] ) |
-                             get_color_component( GetBValue(key), masks[2] );
-}
-
 #ifdef HAVE_LIBXXSHM
 static int xshm_error_handler( Display *display, XErrorEvent *event, void *arg )
 {
     return 1;  /* FIXME: should check event contents */
 }
 
-static XImage *create_shm_image( const XVisualInfo *vis, int width, int height, XShmSegmentInfo *shminfo )
+static XImage *create_shm_image( const XVisualInfo *vis, int width, int height, x11drv_xshm_info_t *shminfo )
 {
     XImage *image;
 
@@ -1826,145 +1638,224 @@ failed:
     XDestroyImage( image );
     return NULL;
 }
+
+static BOOL destroy_shm_image( XImage *image, x11drv_xshm_info_t *shminfo )
+{
+    if (shminfo->shmid == -1) return FALSE;
+
+    XShmDetach( gdi_display, shminfo );
+    shmdt( shminfo->shmaddr );
+
+    return TRUE;
+}
+
+static BOOL put_shm_image( XImage *image, x11drv_xshm_info_t *shminfo, Window window,
+                           GC gc, const RECT *rect, const RECT *dirty )
+{
+    if (shminfo->shmid == -1) return FALSE;
+
+    XShmPutImage( gdi_display, window, gc, image, dirty->left,
+                  dirty->top, rect->left + dirty->left, rect->top + dirty->top,
+                  dirty->right - dirty->left, dirty->bottom - dirty->top, False );
+
+    return TRUE;
+}
+
+#else /* HAVE_LIBXXSHM */
+
+static XImage *create_shm_image( const XVisualInfo *vis, int width, int height, x11drv_xshm_info_t *shminfo )
+{
+    shminfo->shmid = -1;
+    return NULL;
+}
+
+static BOOL destroy_shm_image( XImage *image, x11drv_xshm_info_t *shminfo )
+{
+    return FALSE;
+}
+
+static BOOL put_shm_image( XImage *image, x11drv_xshm_info_t *shminfo, Window window,
+                           GC gc, const RECT *rect, const RECT *dirty )
+{
+    return FALSE;
+}
+
 #endif /* HAVE_LIBXXSHM */
 
-/***********************************************************************
- *           x11drv_surface_lock
- */
-static void x11drv_surface_lock( struct window_surface *window_surface )
+static UINT get_dib_d3dddifmt( const BITMAPINFO *info )
 {
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-
-    pthread_mutex_lock( &surface->mutex );
-}
-
-/***********************************************************************
- *           x11drv_surface_unlock
- */
-static void x11drv_surface_unlock( struct window_surface *window_surface )
-{
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-
-    pthread_mutex_unlock( &surface->mutex );
-}
-
-/***********************************************************************
- *           x11drv_surface_get_bitmap_info
- */
-static void *x11drv_surface_get_bitmap_info( struct window_surface *window_surface, BITMAPINFO *info )
-{
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-
-    memcpy( info, &surface->info, get_dib_info_size( &surface->info, DIB_RGB_COLORS ));
-    return surface->bits;
-}
-
-/***********************************************************************
- *           x11drv_surface_get_bounds
- */
-static RECT *x11drv_surface_get_bounds( struct window_surface *window_surface )
-{
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-
-    return &surface->bounds;
-}
-
-/***********************************************************************
- *           x11drv_surface_set_region
- */
-static void x11drv_surface_set_region( struct window_surface *window_surface, HRGN region )
-{
-    RGNDATA *data;
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-
-    TRACE( "updating surface %p with %p\n", surface, region );
-
-    window_surface->funcs->lock( window_surface );
-    if (!region)
+    if (info->bmiHeader.biCompression == BI_RGB)
     {
-        if (surface->region) NtGdiDeleteObjectApp( surface->region );
-        surface->region = 0;
-        XSetClipMask( gdi_display, surface->gc, None );
+        if (info->bmiHeader.biBitCount == 8) return D3DDDIFMT_P8;
+        if (info->bmiHeader.biBitCount == 24) return D3DDDIFMT_R8G8B8;
+        if (info->bmiHeader.biBitCount == 32) return D3DDDIFMT_A8R8G8B8;
+        return D3DDDIFMT_UNKNOWN;
     }
-    else
+
+    if (info->bmiHeader.biCompression == BI_BITFIELDS)
     {
-        if (!surface->region) surface->region = NtGdiCreateRectRgn( 0, 0, 0, 0 );
-        NtGdiCombineRgn( surface->region, region, 0, RGN_COPY );
-        if ((data = X11DRV_GetRegionData( surface->region, 0 )))
+        DWORD *colors = (DWORD *)info->bmiColors;
+
+        if (info->bmiHeader.biBitCount == 16)
         {
-            XSetClipRectangles( gdi_display, surface->gc, 0, 0,
-                                (XRectangle *)data->Buffer, data->rdh.nCount, YXBanded );
-            free( data );
+            if (colors[0] == 0x0000f800 && colors[1] == 0x000007e0 && colors[2] == 0x0000001f) return D3DDDIFMT_R5G6B5;
+            if (colors[0] == 0x00007c00 && colors[1] == 0x000003e0 && colors[2] == 0x0000001f) return D3DDDIFMT_A1R5G5B5;
+            if (colors[0] == 0x00000f00 && colors[1] == 0x000000f0 && colors[2] == 0x0000000f) return D3DDDIFMT_A4R4G4B4;
         }
+        else if (info->bmiHeader.biBitCount == 24)
+        {
+            if (colors[0] == 0x00ff0000 && colors[1] == 0x0000ff00 && colors[2] == 0x000000ff) return D3DDDIFMT_R8G8B8;
+        }
+        else if (info->bmiHeader.biBitCount == 32)
+        {
+            if (colors[0] == 0x00ff0000 && colors[1] == 0x0000ff00 && colors[2] == 0x000000ff) return D3DDDIFMT_X8R8G8B8;
+        }
+
+        return D3DDDIFMT_UNKNOWN;
+    }
+
+    return D3DDDIFMT_UNKNOWN;
+}
+
+static void x11drv_image_destroy( struct x11drv_image *image )
+{
+    if (!destroy_shm_image( image->ximage, &image->shminfo ))
+        free( image->ximage->data );
+
+    image->ximage->data = NULL;
+    XDestroyImage( image->ximage );
+    free( image );
+}
+
+static struct x11drv_image *x11drv_image_create( const BITMAPINFO *info, const XVisualInfo *vis )
+{
+    UINT width = info->bmiHeader.biWidth, height = abs( info->bmiHeader.biHeight );
+    struct x11drv_image *image;
+
+    if (!(image = calloc( 1, sizeof(*image) ))) return NULL;
+
+    if (!(image->ximage = create_shm_image( vis, width, height, &image->shminfo )))
+    {
+        if (!(image->ximage = XCreateImage( gdi_display, vis->visual, vis->depth, ZPixmap,
+                                            0, NULL, width, height, 32, 0 ))) goto failed;
+        if (!(image->ximage->data = malloc( info->bmiHeader.biSizeImage ))) goto failed;
+    }
+
+    return image;
+
+failed:
+    x11drv_image_destroy( image );
+    return NULL;
+}
+
+static XRectangle *xrectangles_from_rects( const RECT *rects, UINT count )
+{
+    XRectangle *xrects;
+    if (!(xrects = malloc( count * sizeof(*xrects) ))) return NULL;
+    while (count--)
+    {
+        if (rects[count].left > SHRT_MAX) continue;
+        if (rects[count].top > SHRT_MAX) continue;
+        if (rects[count].right < SHRT_MIN) continue;
+        if (rects[count].bottom < SHRT_MIN) continue;
+        xrects[count].x      = max( min( rects[count].left, SHRT_MAX), SHRT_MIN);
+        xrects[count].y      = max( min( rects[count].top, SHRT_MAX), SHRT_MIN);
+        xrects[count].width  = max( min( rects[count].right, SHRT_MAX ) - xrects[count].x, 0);
+        xrects[count].height = max( min( rects[count].bottom, SHRT_MAX ) - xrects[count].y, 0);
+    }
+    return xrects;
+}
+
+/***********************************************************************
+ *           x11drv_surface_set_clip
+ */
+static void x11drv_surface_set_clip( struct window_surface *window_surface, const RECT *rects, UINT count )
+{
+    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
+    XRectangle *xrects;
+
+    TRACE( "surface %p, rects %p, count %u\n", surface, rects, count );
+
+    if (!count)
+        XSetClipMask( gdi_display, surface->gc, None );
+    else if ((xrects = xrectangles_from_rects( rects, count )))
+    {
+        XSetClipRectangles( gdi_display, surface->gc, 0, 0, xrects, count, YXBanded );
+        free( xrects );
     }
-    window_surface->funcs->unlock( window_surface );
 }
 
 /***********************************************************************
  *           x11drv_surface_flush
  */
-static void x11drv_surface_flush( struct window_surface *window_surface )
+static BOOL x11drv_surface_flush( struct window_surface *window_surface, const RECT *rect, const RECT *dirty,
+                                  const BITMAPINFO *color_info, const void *color_bits, BOOL shape_changed,
+                                  const BITMAPINFO *shape_info, const void *shape_bits )
 {
+    UINT alpha_mask = window_surface->alpha_mask, alpha_bits = window_surface->alpha_bits;
     struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-    unsigned char *src = surface->bits;
-    unsigned char *dst = (unsigned char *)surface->image->data;
-    struct bitblt_coords coords;
+    XImage *ximage = surface->image->ximage;
+    const unsigned char *src = color_bits;
+    unsigned char *dst = (unsigned char *)ximage->data;
 
-    window_surface->funcs->lock( window_surface );
-    coords.x = 0;
-    coords.y = 0;
-    coords.width  = surface->header.rect.right - surface->header.rect.left;
-    coords.height = surface->header.rect.bottom - surface->header.rect.top;
-    SetRect( &coords.visrect, 0, 0, coords.width, coords.height );
-    if (intersect_rect( &coords.visrect, &coords.visrect, &surface->bounds ))
+    if (alpha_bits == -1)
     {
-        TRACE( "flushing %p %dx%d bounds %s bits %p\n",
-               surface, coords.width, coords.height,
-               wine_dbgstr_rect( &surface->bounds ), surface->bits );
-
-        if (surface->is_argb || surface->color_key != CLR_INVALID) update_surface_region( surface );
-
-        if (src != dst)
-        {
-            int map[256], *mapping = get_window_surface_mapping( surface->image->bits_per_pixel, map );
-            int width_bytes = surface->image->bytes_per_line;
-
-            src += coords.visrect.top * width_bytes;
-            dst += coords.visrect.top * width_bytes;
-            copy_image_byteswap( &surface->info, src, dst, width_bytes, width_bytes,
-                                 coords.visrect.bottom - coords.visrect.top,
-                                 surface->byteswap, mapping, ~0u, surface->alpha_bits );
-        }
-        else if (surface->alpha_bits)
-        {
-            int x, y, stride = surface->image->bytes_per_line / sizeof(ULONG);
-            ULONG *ptr = (ULONG *)dst + coords.visrect.top * stride;
-
-            for (y = coords.visrect.top; y < coords.visrect.bottom; y++, ptr += stride)
-                for (x = coords.visrect.left; x < coords.visrect.right; x++)
-                    ptr[x] |= surface->alpha_bits;
-        }
-
-#ifdef HAVE_LIBXXSHM
-        if (surface->shminfo.shmid != -1)
-            XShmPutImage( gdi_display, surface->window, surface->gc, surface->image,
-                          coords.visrect.left, coords.visrect.top,
-                          surface->header.rect.left + coords.visrect.left,
-                          surface->header.rect.top + coords.visrect.top,
-                          coords.visrect.right - coords.visrect.left,
-                          coords.visrect.bottom - coords.visrect.top, False );
+        if (alpha_mask || color_info->bmiHeader.biBitCount != 32) alpha_bits = 0;
+        else if (color_info->bmiHeader.biCompression == BI_RGB) alpha_bits = 0xff000000;
         else
-#endif
-        XPutImage( gdi_display, surface->window, surface->gc, surface->image,
-                   coords.visrect.left, coords.visrect.top,
-                   surface->header.rect.left + coords.visrect.left,
-                   surface->header.rect.top + coords.visrect.top,
-                   coords.visrect.right - coords.visrect.left,
-                   coords.visrect.bottom - coords.visrect.top );
-        XFlush( gdi_display );
+        {
+            DWORD *colors = (DWORD *)color_info->bmiColors;
+            alpha_bits = ~(colors[0] | colors[1] | colors[2]);
+        }
     }
-    reset_bounds( &surface->bounds );
-    window_surface->funcs->unlock( window_surface );
+
+    if (src != dst)
+    {
+        int map[256], *mapping = get_window_surface_mapping( ximage->bits_per_pixel, map );
+        int width_bytes = ximage->bytes_per_line;
+
+        src += dirty->top * width_bytes;
+        dst += dirty->top * width_bytes;
+        copy_image_byteswap( color_info, src, dst, width_bytes, width_bytes, dirty->bottom - dirty->top,
+                             surface->byteswap, mapping, ~0u, alpha_bits );
+    }
+    else if (alpha_bits)
+    {
+        int x, y, stride = ximage->bytes_per_line / sizeof(ULONG);
+        ULONG *ptr = (ULONG *)dst + dirty->top * stride;
+
+        for (y = dirty->top; y < dirty->bottom; y++, ptr += stride)
+            for (x = dirty->left; x < dirty->right; x++)
+                ptr[x] |= alpha_bits;
+    }
+
+    if (!put_shm_image( ximage, &surface->image->shminfo, surface->window, surface->gc, rect, dirty ))
+        XPutImage( gdi_display, surface->window, surface->gc, ximage, dirty->left,
+                   dirty->top, rect->left + dirty->left, rect->top + dirty->top,
+                   dirty->right - dirty->left, dirty->bottom - dirty->top );
+
+    if (shape_changed)
+    {
+#ifdef HAVE_LIBXSHAPE
+        if (!shape_bits)
+            XShapeCombineMask( gdi_display, surface->window, ShapeBounding, 0, 0, None, ShapeSet );
+        else
+        {
+            struct gdi_image_bits bits = {.ptr = (void *)shape_bits};
+            XVisualInfo vis = default_visual;
+            Pixmap shape;
+
+            vis.depth = 1;
+            shape = create_pixmap_from_image( 0, &vis, shape_info, &bits, DIB_RGB_COLORS );
+            XShapeCombineMask( gdi_display, surface->window, ShapeBounding, 0, 0, shape, ShapeSet );
+            XFreePixmap( gdi_display, shape );
+        }
+#endif /* HAVE_LIBXSHAPE */
+    }
+
+    XFlush( gdi_display );
+
+    return TRUE;
 }
 
 /***********************************************************************
@@ -1974,34 +1865,14 @@ static void x11drv_surface_destroy( struct window_surface *window_surface )
 {
     struct x11drv_window_surface *surface = get_x11_surface( window_surface );
 
-    TRACE( "freeing %p bits %p\n", surface, surface->bits );
+    TRACE( "freeing %p\n", surface );
     if (surface->gc) XFreeGC( gdi_display, surface->gc );
-    if (surface->image)
-    {
-        if (surface->image->data != surface->bits) free( surface->bits );
-#ifdef HAVE_LIBXXSHM
-        if (surface->shminfo.shmid != -1)
-        {
-            XShmDetach( gdi_display, &surface->shminfo );
-            shmdt( surface->shminfo.shmaddr );
-        }
-        else
-#endif
-        free( surface->image->data );
-        surface->image->data = NULL;
-        XDestroyImage( surface->image );
-    }
-    if (surface->region) NtGdiDeleteObjectApp( surface->region );
-    free( surface );
+    if (surface->image) x11drv_image_destroy( surface->image );
 }
 
 static const struct window_surface_funcs x11drv_surface_funcs =
 {
-    x11drv_surface_lock,
-    x11drv_surface_unlock,
-    x11drv_surface_get_bitmap_info,
-    x11drv_surface_get_bounds,
-    x11drv_surface_set_region,
+    x11drv_surface_set_clip,
     x11drv_surface_flush,
     x11drv_surface_destroy
 };
@@ -2009,112 +1880,122 @@ static const struct window_surface_funcs x11drv_surface_funcs =
 /***********************************************************************
  *           create_surface
  */
-struct window_surface *create_surface( Window window, const XVisualInfo *vis, const RECT *rect,
-                                       COLORREF color_key, BOOL use_alpha )
+static struct window_surface *create_surface( HWND hwnd, Window window, const XVisualInfo *vis, const RECT *rect,
+                                              BOOL use_alpha )
 {
     const XPixmapFormatValues *format = pixmap_formats[vis->depth];
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *info = (BITMAPINFO *)buffer;
     struct x11drv_window_surface *surface;
     int width = rect->right - rect->left, height = rect->bottom - rect->top;
-    int colors = format->bits_per_pixel <= 8 ? 1 << format->bits_per_pixel : 3;
+    struct window_surface *window_surface;
+    struct x11drv_image *image;
+    D3DDDIFORMAT d3d_format;
+    HBITMAP bitmap = 0;
+    BOOL byteswap;
+    UINT status;
 
-    surface = calloc( 1, FIELD_OFFSET( struct x11drv_window_surface, info.bmiColors[colors] ));
-    if (!surface) return NULL;
-    surface->info.bmiHeader.biSize        = sizeof(surface->info.bmiHeader);
-    surface->info.bmiHeader.biWidth       = width;
-    surface->info.bmiHeader.biHeight      = -height; /* top-down */
-    surface->info.bmiHeader.biPlanes      = 1;
-    surface->info.bmiHeader.biBitCount    = format->bits_per_pixel;
-    surface->info.bmiHeader.biSizeImage   = get_dib_image_size( &surface->info );
-    if (format->bits_per_pixel > 8) set_color_info( vis, &surface->info, use_alpha );
+    memset( info, 0, sizeof(*info) );
+    info->bmiHeader.biSize        = sizeof(info->bmiHeader);
+    info->bmiHeader.biWidth       = width;
+    info->bmiHeader.biHeight      = -height; /* top-down */
+    info->bmiHeader.biPlanes      = 1;
+    info->bmiHeader.biBitCount    = format->bits_per_pixel;
+    info->bmiHeader.biSizeImage   = get_dib_image_size( info );
+    set_color_info( vis, info, use_alpha );
 
-    init_recursive_mutex( &surface->mutex );
+    if (!(image = x11drv_image_create( info, vis ))) return NULL;
 
-    surface->header.funcs = &x11drv_surface_funcs;
-    surface->header.rect  = *rect;
-    surface->header.ref   = 1;
-    surface->window = window;
-    surface->is_argb = (use_alpha && vis->depth == 32 && surface->info.bmiHeader.biCompression == BI_RGB);
-    set_color_key( surface, color_key );
-    reset_bounds( &surface->bounds );
-
-#ifdef HAVE_LIBXXSHM
-    surface->image = create_shm_image( vis, width, height, &surface->shminfo );
-    if (!surface->image)
-#endif
+    /* wrap the XImage data in a HBITMAP if we can write to the surface pixels directly */
+    if ((byteswap = image_needs_byteswap( image->ximage, is_r8g8b8( vis ), info->bmiHeader.biBitCount )) ||
+        info->bmiHeader.biBitCount <= 8 || !(d3d_format = get_dib_d3dddifmt( info )))
+        WARN( "Cannot use direct rendering, falling back to copies\n" );
+    else
     {
-        surface->image = XCreateImage( gdi_display, vis->visual, vis->depth, ZPixmap, 0, NULL,
-                                       width, height, 32, 0 );
-        if (!surface->image) goto failed;
-        surface->image->data = malloc( surface->info.bmiHeader.biSizeImage );
-        if (!surface->image->data) goto failed;
-    }
-
-    surface->gc = XCreateGC( gdi_display, window, 0, NULL );
-    XSetSubwindowMode( gdi_display, surface->gc, IncludeInferiors );
-    surface->byteswap = image_needs_byteswap( surface->image, is_r8g8b8(vis), format->bits_per_pixel );
-
-    if (vis->depth == 32 && !surface->is_argb)
-        surface->alpha_bits = ~(vis->red_mask | vis->green_mask | vis->blue_mask);
-
-    if (surface->byteswap || format->bits_per_pixel == 4 || format->bits_per_pixel == 8)
-    {
-        /* allocate separate surface bits if byte swapping or palette mapping is required */
-        if (!(surface->bits  = calloc( 1, surface->info.bmiHeader.biSizeImage )))
-            goto failed;
-    }
-    else surface->bits = surface->image->data;
-
-    TRACE( "created %p for %lx %s bits %p-%p image %p\n", surface, window, wine_dbgstr_rect(rect),
-           surface->bits, (char *)surface->bits + surface->info.bmiHeader.biSizeImage,
-           surface->image->data );
-
-    return &surface->header;
-
-failed:
-    x11drv_surface_destroy( &surface->header );
-    return NULL;
-}
-
-/***********************************************************************
- *           set_surface_color_key
- */
-void set_surface_color_key( struct window_surface *window_surface, COLORREF color_key )
-{
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-    COLORREF prev;
-
-    if (window_surface->funcs != &x11drv_surface_funcs) return;  /* we may get the null surface */
-
-    window_surface->funcs->lock( window_surface );
-    prev = surface->color_key;
-    set_color_key( surface, color_key );
-    if (surface->color_key != prev) update_surface_region( surface );
-    window_surface->funcs->unlock( window_surface );
-}
-
-/***********************************************************************
- *           expose_surface
- */
-HRGN expose_surface( struct window_surface *window_surface, const RECT *rect )
-{
-    struct x11drv_window_surface *surface = get_x11_surface( window_surface );
-    HRGN region = 0;
-    RECT rc = *rect;
-
-    if (window_surface->funcs != &x11drv_surface_funcs) return 0;  /* we may get the null surface */
-
-    window_surface->funcs->lock( window_surface );
-    OffsetRect( &rc, -window_surface->rect.left, -window_surface->rect.top );
-    add_bounds_rect( &surface->bounds, &rc );
-    if (surface->region)
-    {
-        region = NtGdiCreateRectRgn( rect->left, rect->top, rect->right, rect->bottom );
-        if (NtGdiCombineRgn( region, region, surface->region, RGN_DIFF ) <= NULLREGION)
+        D3DKMT_CREATEDCFROMMEMORY desc =
         {
-            NtGdiDeleteObjectApp( region );
-            region = 0;
+            .Width = info->bmiHeader.biWidth,
+            .Height = abs( info->bmiHeader.biHeight ),
+            .Pitch = info->bmiHeader.biSizeImage / abs( info->bmiHeader.biHeight ),
+            .Format = d3d_format,
+            .pMemory = image->ximage->data,
+            .hDeviceDc = NtUserGetDCEx( hwnd, 0, DCX_CACHE | DCX_WINDOW ),
+        };
+
+        if ((status = NtGdiDdDDICreateDCFromMemory( &desc )))
+            ERR( "Failed to create HBITMAP falling back to copies, status %#x\n", status );
+        else
+        {
+            bitmap = desc.hBitmap;
+            NtGdiDeleteObjectApp( desc.hDc );
         }
+        if (desc.hDeviceDc) NtUserReleaseDC( hwnd, desc.hDeviceDc );
     }
-    window_surface->funcs->unlock( window_surface );
-    return region;
+
+    if (!(window_surface = window_surface_create( sizeof(*surface), &x11drv_surface_funcs, hwnd, rect, info, bitmap )))
+    {
+        if (bitmap) NtGdiDeleteObjectApp( bitmap );
+        x11drv_image_destroy( image );
+    }
+    else
+    {
+        surface = get_x11_surface( window_surface );
+        surface->image = image;
+        surface->byteswap = byteswap;
+        surface->window = window;
+        surface->gc = XCreateGC( gdi_display, window, 0, NULL );
+        XSetSubwindowMode( gdi_display, surface->gc, IncludeInferiors );
+    }
+
+    return window_surface;
+}
+
+
+static BOOL enable_direct_drawing( struct x11drv_win_data *data, BOOL layered )
+{
+    if (layered) return FALSE;
+    if (data->embedded) return TRUE; /* draw directly to the window */
+    if (data->whole_window == root_window) return TRUE; /* draw directly to the window */
+    if (data->client_window) return TRUE; /* draw directly to the window */
+    if (!client_side_graphics) return TRUE; /* draw directly to the window */
+    return FALSE;
+}
+
+/***********************************************************************
+ *      CreateWindowSurface   (X11DRV.@)
+ */
+BOOL X11DRV_CreateWindowSurface( HWND hwnd, BOOL layered, const RECT *surface_rect, struct window_surface **surface )
+{
+    struct window_surface *previous;
+    struct x11drv_win_data *data;
+
+    TRACE( "hwnd %p, layered %u, surface_rect %s, surface %p\n", hwnd, layered, wine_dbgstr_rect( surface_rect ), surface );
+
+    if (!(data = get_win_data( hwnd ))) return TRUE; /* use default surface */
+    if ((previous = *surface) && previous->funcs == &x11drv_surface_funcs)
+    {
+        Window window = get_x11_surface(previous)->window;
+        if (data->whole_window == window && !enable_direct_drawing( data, layered )) goto done; /* use default surface */
+        /* re-create window surface is window has changed, which can happen when changing visual */
+        TRACE( "re-creating hwnd %p surface with new window %lx\n", data->hwnd, data->whole_window );
+    }
+    if (previous) window_surface_release( previous );
+
+    if (layered)
+    {
+        data->layered = TRUE;
+        if (!data->embedded && argb_visual.visualid) set_window_visual( data, &argb_visual, TRUE );
+    }
+    else if (enable_direct_drawing( data, layered ))
+    {
+        *surface = NULL;  /* indicate that we want to draw directly to the window */
+        goto done; /* draw directly to the window */
+    }
+
+    *surface = create_surface( data->hwnd, data->whole_window, &data->vis, surface_rect,
+                               layered ? data->use_alpha : FALSE );
+
+done:
+    release_win_data( data );
+    return TRUE;
 }
