@@ -44,19 +44,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(xim);
 #define XICProc XIMProc
 #endif
 
-struct ime_update
-{
-    struct list entry;
-    DWORD id;
-    DWORD cursor_pos;
-    WCHAR *comp_str;
-    WCHAR *result_str;
-    WCHAR buffer[];
-};
-
-static pthread_mutex_t ime_mutex = PTHREAD_MUTEX_INITIALIZER;
-static struct list ime_updates = LIST_INIT(ime_updates);
-static DWORD ime_update_count;
 static WCHAR *ime_comp_buf;
 
 static XIMStyle input_style = 0;
@@ -89,23 +76,8 @@ BOOL xim_in_compose_mode(void)
 
 static void post_ime_update( HWND hwnd, UINT cursor_pos, WCHAR *comp_str, WCHAR *result_str )
 {
-    UINT id, comp_len, result_len;
-    struct ime_update *update;
-
-    comp_len = comp_str ? wcslen( comp_str ) + 1 : 0;
-    result_len = result_str ? wcslen( result_str ) + 1 : 0;
-
-    if (!(update = malloc( offsetof(struct ime_update, buffer[comp_len + result_len]) ))) return;
-    update->cursor_pos = cursor_pos;
-    update->comp_str = comp_str ? memcpy( update->buffer, comp_str, comp_len * sizeof(WCHAR) ) : NULL;
-    update->result_str = result_str ? memcpy( update->buffer + comp_len, result_str, result_len * sizeof(WCHAR) ) : NULL;
-
-    pthread_mutex_lock( &ime_mutex );
-    id = update->id = ++ime_update_count;
-    list_add_tail( &ime_updates, &update->entry );
-    pthread_mutex_unlock( &ime_mutex );
-
-    NtUserPostMessage( hwnd, WM_IME_NOTIFY, IMN_WINE_SET_COMP_STRING, id );
+    NtUserMessageCall( hwnd, WINE_IME_POST_UPDATE, cursor_pos, (LPARAM)comp_str,
+                       result_str, NtUserImeDriverCall, FALSE );
 }
 
 static void xim_update_comp_string( UINT offset, UINT old_len, const WCHAR *text, UINT new_len )
@@ -156,10 +128,10 @@ static BOOL xic_preedit_state_notify( XIC xic, XPointer user, XPointer arg )
     switch (state)
     {
     case XIMPreeditEnable:
-        NtUserPostMessage( hwnd, WM_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, TRUE );
+        NtUserPostMessage( hwnd, WM_WINE_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, TRUE );
         break;
     case XIMPreeditDisable:
-        NtUserPostMessage( hwnd, WM_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, FALSE );
+        NtUserPostMessage( hwnd, WM_WINE_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, FALSE );
         break;
     }
 
@@ -175,7 +147,7 @@ static int xic_preedit_start( XIC xic, XPointer user, XPointer arg )
     if ((ime_comp_buf = realloc( ime_comp_buf, sizeof(WCHAR) ))) *ime_comp_buf = 0;
     else ERR( "Failed to allocate preedit buffer\n" );
 
-    NtUserPostMessage( hwnd, WM_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, TRUE );
+    NtUserPostMessage( hwnd, WM_WINE_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, TRUE );
     post_ime_update( hwnd, 0, ime_comp_buf, NULL );
 
     return -1;
@@ -191,7 +163,7 @@ static int xic_preedit_done( XIC xic, XPointer user, XPointer arg )
     ime_comp_buf = NULL;
 
     post_ime_update( hwnd, 0, NULL, NULL );
-    NtUserPostMessage( hwnd, WM_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, FALSE );
+    NtUserPostMessage( hwnd, WM_WINE_IME_NOTIFY, IMN_WINE_SET_OPEN_STATUS, FALSE );
 
     return 0;
 }
@@ -511,123 +483,39 @@ XIC X11DRV_get_ic( HWND hwnd )
 
 void xim_set_focus( HWND hwnd, BOOL focus )
 {
-    struct list updates = LIST_INIT(updates);
-    struct ime_update *update, *next;
     XIC xic;
-
     if (!(xic = X11DRV_get_ic( hwnd ))) return;
-
     if (focus) XSetICFocus( xic );
     else XUnsetICFocus( xic );
-
-    pthread_mutex_lock( &ime_mutex );
-    list_move_tail( &updates, &ime_updates );
-    pthread_mutex_unlock( &ime_mutex );
-
-    LIST_FOR_EACH_ENTRY_SAFE( update, next, &updates, struct ime_update, entry ) free( update );
-}
-
-static struct ime_update *find_ime_update( UINT id )
-{
-    struct ime_update *update;
-    LIST_FOR_EACH_ENTRY( update, &ime_updates, struct ime_update, entry )
-        if (update->id == id) return update;
-    return NULL;
 }
 
 /***********************************************************************
- *      ImeToAsciiEx (X11DRV.@)
- *
- * As XIM filters key events upfront, we don't use ImeProcessKey and ImeToAsciiEx is instead called
- * back from the IME UI window procedure when WM_IME_NOTIFY / IMN_WINE_SET_COMP_STRING messages are
- * sent to it, to retrieve composition string updates and generate WM_IME messages.
+ *      SetIMECompositionRect (X11DRV.@)
  */
-UINT X11DRV_ImeToAsciiEx( UINT vkey, UINT lparam, const BYTE *state, COMPOSITIONSTRING *compstr, HIMC himc )
+BOOL X11DRV_SetIMECompositionRect( HWND hwnd, RECT rect )
 {
-    UINT needed = sizeof(COMPOSITIONSTRING), comp_len, result_len;
-    struct ime_update *update;
-    void *dst;
+    struct x11drv_win_data *data = NULL;
+    XVaNestedList attr;
+    XPoint xpoint;
 
-    TRACE( "vkey %#x, lparam %#x, state %p, compstr %p, himc %p\n", vkey, lparam, state, compstr, himc );
+    if (!(input_style & XIMPreeditPosition))
+        return FALSE;
 
-    pthread_mutex_lock( &ime_mutex );
-
-    if (!(update = find_ime_update( lparam )))
+    if (!(data = get_win_data( hwnd )) || !data->xic)
     {
-        pthread_mutex_unlock( &ime_mutex );
-        return 0;
+        if (data) release_win_data( data );
+        return FALSE;
     }
 
-    if (!update->comp_str) comp_len = 0;
-    else
+    xpoint.x = rect.left - data->rects.visible.left;
+    xpoint.y = rect.top - data->rects.visible.top;
+    attr = XVaCreateNestedList( 0, XNSpotLocation, &xpoint, NULL );
+    if (attr)
     {
-        comp_len = wcslen( update->comp_str );
-        needed += comp_len * sizeof(WCHAR); /* GCS_COMPSTR */
-        needed += comp_len; /* GCS_COMPATTR */
-        needed += 2 * sizeof(DWORD); /* GCS_COMPCLAUSE */
+        XSetICValues( data->xic, XNPreeditAttributes, attr, NULL );
+        XFree( attr );
     }
 
-    if (!update->result_str) result_len = 0;
-    else
-    {
-        result_len = wcslen( update->result_str );
-        needed += result_len * sizeof(WCHAR); /* GCS_RESULTSTR */
-        needed += 2 * sizeof(DWORD); /* GCS_RESULTCLAUSE */
-    }
-
-    if (compstr->dwSize < needed)
-    {
-        compstr->dwSize = needed;
-        pthread_mutex_unlock( &ime_mutex );
-        return STATUS_BUFFER_TOO_SMALL;
-    }
-
-    list_remove( &update->entry );
-    pthread_mutex_unlock( &ime_mutex );
-
-    memset( compstr, 0, sizeof(*compstr) );
-    compstr->dwSize = sizeof(*compstr);
-
-    if (update->comp_str)
-    {
-        compstr->dwCursorPos = update->cursor_pos;
-
-        compstr->dwCompStrLen = comp_len;
-        compstr->dwCompStrOffset = compstr->dwSize;
-        dst = (BYTE *)compstr + compstr->dwCompStrOffset;
-        memcpy( dst, update->comp_str, compstr->dwCompStrLen * sizeof(WCHAR) );
-        compstr->dwSize += compstr->dwCompStrLen * sizeof(WCHAR);
-
-        compstr->dwCompClauseLen = 2 * sizeof(DWORD);
-        compstr->dwCompClauseOffset = compstr->dwSize;
-        dst = (BYTE *)compstr + compstr->dwCompClauseOffset;
-        *((DWORD *)dst + 0) = 0;
-        *((DWORD *)dst + 1) = compstr->dwCompStrLen;
-        compstr->dwSize += compstr->dwCompClauseLen;
-
-        compstr->dwCompAttrLen = compstr->dwCompStrLen;
-        compstr->dwCompAttrOffset = compstr->dwSize;
-        dst = (BYTE *)compstr + compstr->dwCompAttrOffset;
-        memset( dst, ATTR_INPUT, compstr->dwCompAttrLen );
-        compstr->dwSize += compstr->dwCompAttrLen;
-    }
-
-    if (update->result_str)
-    {
-        compstr->dwResultStrLen = result_len;
-        compstr->dwResultStrOffset = compstr->dwSize;
-        dst = (BYTE *)compstr + compstr->dwResultStrOffset;
-        memcpy( dst, update->result_str, compstr->dwResultStrLen * sizeof(WCHAR) );
-        compstr->dwSize += compstr->dwResultStrLen * sizeof(WCHAR);
-
-        compstr->dwResultClauseLen = 2 * sizeof(DWORD);
-        compstr->dwResultClauseOffset = compstr->dwSize;
-        dst = (BYTE *)compstr + compstr->dwResultClauseOffset;
-        *((DWORD *)dst + 0) = 0;
-        *((DWORD *)dst + 1) = compstr->dwResultStrLen;
-        compstr->dwSize += compstr->dwResultClauseLen;
-    }
-
-    free( update );
-    return 0;
+    release_win_data( data );
+    return TRUE;
 }
