@@ -61,6 +61,7 @@ static const WCHAR url18[] = L"http://%0D%1F%20%0A%7F%0D%0A";
 static const WCHAR url19[] = L"http://?text=\xfb00";
 static const WCHAR url20[] = L"http:///text=\xfb00";
 static const WCHAR url21[] = L"https://nba2k19-ws.2ksports.com:19133/nba/v4/Accounts/get_account?x=3789526775265663876";
+static const WCHAR url22[] = L"http://winehq.org:/";
 
 static const WCHAR url_k1[] = L"http://username:password@www.winehq.org/site/about";
 static const WCHAR url_k2[] = L"http://www.winehq.org";
@@ -104,31 +105,31 @@ static void WinHttpCreateUrl_test( void )
     SetLastError( 0xdeadbeef );
     ret = WinHttpCreateUrl( NULL, 0, NULL, &len );
     ok( !ret, "expected failure\n" );
-    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %u\n", GetLastError() );
-    ok( len == ~0u, "expected len ~0u got %u\n", len );
+    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %lu\n", GetLastError() );
+    ok( len == ~0u, "expected len ~0u got %lu\n", len );
 
     /* zero'ed components */
     memset( &uc, 0, sizeof(URL_COMPONENTS) );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCreateUrl( &uc, 0, NULL, &len );
     ok( !ret, "expected failure\n" );
-    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %u\n", GetLastError() );
-    ok( len == ~0u, "expected len ~0u got %u\n", len );
+    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %lu\n", GetLastError() );
+    ok( len == ~0u, "expected len ~0u got %lu\n", len );
 
     /* valid components, NULL url, NULL length */
     fill_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCreateUrl( &uc, 0, NULL, NULL );
     ok( !ret, "expected failure\n" );
-    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %u\n", GetLastError() );
+    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %lu\n", GetLastError() );
 
     /* valid components, NULL url, insufficient length */
     len = 0;
     SetLastError( 0xdeadbeef );
     ret = WinHttpCreateUrl( &uc, 0, NULL, &len );
     ok( !ret, "expected failure\n" );
-    ok( GetLastError() == ERROR_INSUFFICIENT_BUFFER, "expected ERROR_INSUFFICIENT_BUFFER got %u\n", GetLastError() );
-    ok( len == 57, "expected len 57 got %u\n", len );
+    ok( GetLastError() == ERROR_INSUFFICIENT_BUFFER, "expected ERROR_INSUFFICIENT_BUFFER got %lu\n", GetLastError() );
+    ok( len == 57, "expected len 57 got %lu\n", len );
 
     /* valid components, NULL url, sufficient length */
     SetLastError( 0xdeadbeef );
@@ -137,8 +138,8 @@ static void WinHttpCreateUrl_test( void )
     err = GetLastError();
     ok( !ret, "expected failure\n" );
     ok( err == ERROR_INVALID_PARAMETER || broken(err == ERROR_INSUFFICIENT_BUFFER) /* < win7 */,
-        "expected ERROR_INVALID_PARAMETER got %u\n", GetLastError() );
-    ok( len == 256 || broken(len == 57) /* < win7 */, "expected len 256 got %u\n", len );
+        "expected ERROR_INVALID_PARAMETER got %lu\n", GetLastError() );
+    ok( len == 256 || broken(len == 57) /* < win7 */, "expected len 256 got %lu\n", len );
 
     /* correct size, NULL url */
     fill_url_components( &uc );
@@ -147,8 +148,8 @@ static void WinHttpCreateUrl_test( void )
     err = GetLastError();
     ok( !ret, "expected failure\n" );
     ok( err == ERROR_INVALID_PARAMETER || broken(err == ERROR_INSUFFICIENT_BUFFER) /* < win7 */,
-        "expected ERROR_INVALID_PARAMETER got %u\n", GetLastError() );
-    ok( len == 256 || broken(len == 57) /* < win7 */, "expected len 256 got %u\n", len );
+        "expected ERROR_INVALID_PARAMETER got %lu\n", GetLastError() );
+    ok( len == 256 || broken(len == 57) /* < win7 */, "expected len 256 got %lu\n", len );
 
     /* valid components, allocated url, short length */
     SetLastError( 0xdeadbeef );
@@ -157,8 +158,8 @@ static void WinHttpCreateUrl_test( void )
     len = 2;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( !ret, "expected failure\n" );
-    ok( GetLastError() == ERROR_INSUFFICIENT_BUFFER, "expected ERROR_INSUFFICIENT_BUFFER got %u\n", GetLastError() );
-    ok( len == 57, "expected len 57 got %u\n", len );
+    ok( GetLastError() == ERROR_INSUFFICIENT_BUFFER, "expected ERROR_INSUFFICIENT_BUFFER got %lu\n", GetLastError() );
+    ok( len == 57, "expected len 57 got %lu\n", len );
 
     /* allocated url, NULL scheme */
     SetLastError( 0xdeadbeef );
@@ -168,8 +169,8 @@ static void WinHttpCreateUrl_test( void )
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
     ok( GetLastError() == ERROR_SUCCESS || broken(GetLastError() == 0xdeadbeef) /* < win7 */,
-        "expected ERROR_SUCCESS got %u\n", GetLastError() );
-    ok( len == 56, "expected len 56 got %u\n", len );
+        "expected ERROR_SUCCESS got %lu\n", GetLastError() );
+    ok( len == 56, "expected len 56 got %lu\n", len );
     ok( !lstrcmpW( url, url1 ), "url doesn't match\n" );
 
     /* allocated url, 0 scheme */
@@ -179,7 +180,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 56, "expected len 56 got %u\n", len );
+    ok( len == 56, "expected len 56 got %lu\n", len );
 
     /* valid components, allocated url */
     fill_url_components( &uc );
@@ -187,7 +188,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 56, "expected len 56 got %d\n", len );
+    ok( len == 56, "expected len 56 got %lu\n", len );
     ok( !lstrcmpW( url, url1 ), "url doesn't match\n" );
 
     /* valid username, NULL password */
@@ -205,7 +206,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 56, "expected len 56 got %u\n", len );
+    ok( len == 56, "expected len 56 got %lu\n", len );
     ok( !lstrcmpW( url, url2 ), "url doesn't match\n" );
 
     /* valid password, NULL username */
@@ -216,7 +217,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( !ret, "expected failure\n" );
-    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %u\n", GetLastError() );
+    ok( GetLastError() == ERROR_INVALID_PARAMETER, "expected ERROR_INVALID_PARAMETER got %lu\n", GetLastError() );
 
     /* valid password, empty username */
     fill_url_components( &uc );
@@ -234,7 +235,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 38, "expected len 38 got %u\n", len );
+    ok( len == 38, "expected len 38 got %lu\n", len );
     ok( !lstrcmpW( url, url3 ), "url doesn't match\n" );
 
     /* empty username, empty password */
@@ -245,7 +246,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 56, "expected len 56 got %u\n", len );
+    ok( len == 56, "expected len 56 got %lu\n", len );
     ok( !lstrcmpW( url, url4 ), "url doesn't match\n" );
 
     /* nScheme has lower precedence than lpszScheme */
@@ -256,7 +257,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == lstrlenW( url5 ), "expected len %d got %u\n", lstrlenW( url5 ) + 1, len );
+    ok( len == lstrlenW( url5 ), "expected len %d got %lu\n", lstrlenW( url5 ) + 1, len );
     ok( !lstrcmpW( url, url5 ), "url doesn't match\n" );
 
     /* non-standard port */
@@ -267,7 +268,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 59, "expected len 59 got %u\n", len );
+    ok( len == 59, "expected len 59 got %lu\n", len );
     ok( !lstrcmpW( url, url6 ), "url doesn't match\n" );
 
     /* escape extra info */
@@ -278,7 +279,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, ICU_ESCAPE, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 113, "expected len 113 got %u\n", len );
+    ok( len == 113, "expected len 113 got %lu\n", len );
     ok( !lstrcmpW( url, url7 ), "url doesn't match %s\n", wine_dbgstr_w(url) );
 
     /* escape extra info */
@@ -290,7 +291,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, ICU_ESCAPE, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == lstrlenW(url18), "expected len %u got %u\n", lstrlenW(url18), len );
+    ok( len == lstrlenW(url18), "expected len %u got %lu\n", lstrlenW(url18), len );
     ok( !lstrcmpW( url, url18 ), "url doesn't match\n" );
 
     /* extra info with Unicode characters */
@@ -303,8 +304,8 @@ static void WinHttpCreateUrl_test( void )
     SetLastError( 0xdeadbeef );
     ret = WinHttpCreateUrl( &uc, ICU_ESCAPE, url, &len );
     err = GetLastError();
-    ok( !ret, "expected failure\n" );
-    ok( err == ERROR_INVALID_PARAMETER, "got %u\n", err );
+    ok( !ret || GetACP() == CP_UTF8, "expected failure\n" );
+    ok( err == ERROR_INVALID_PARAMETER || (!err && GetACP() == CP_UTF8), "got %lu\n", err );
 
     /* extra info with Unicode characters, no ICU_ESCAPE */
     memset( &uc, 0, sizeof(uc) );
@@ -317,7 +318,7 @@ static void WinHttpCreateUrl_test( void )
     ok( ret || broken(!ret) /* < win7 */, "expected success\n" );
     if (ret)
     {
-        ok( len == lstrlenW(url19), "expected len %u got %u\n", lstrlenW(url19), len );
+        ok( len == lstrlenW(url19), "expected len %u got %lu\n", lstrlenW(url19), len );
         ok( !lstrcmpW( url, url19 ), "url doesn't match %s\n", wine_dbgstr_w(url) );
     }
 
@@ -330,7 +331,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, ICU_ESCAPE, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == lstrlenW(url18), "expected len %u got %u\n", lstrlenW(url18), len );
+    ok( len == lstrlenW(url18), "expected len %u got %lu\n", lstrlenW(url18), len );
     ok( !lstrcmpW( url, url18 ), "url doesn't match\n" );
 
     /* path with Unicode characters */
@@ -343,8 +344,8 @@ static void WinHttpCreateUrl_test( void )
     SetLastError( 0xdeadbeef );
     ret = WinHttpCreateUrl( &uc, ICU_ESCAPE, url, &len );
     err = GetLastError();
-    ok( !ret, "expected failure\n" );
-    ok( err == ERROR_INVALID_PARAMETER, "got %u\n", err );
+    ok( !ret || GetACP() == CP_UTF8, "expected failure\n" );
+    ok( err == ERROR_INVALID_PARAMETER || (!err && GetACP() == CP_UTF8), "got %lu\n", err );
 
     /* path with Unicode characters, no ICU_ESCAPE */
     memset( &uc, 0, sizeof(uc) );
@@ -357,7 +358,7 @@ static void WinHttpCreateUrl_test( void )
     ok( ret || broken(!ret) /* < win7 */, "expected success\n" );
     if (ret)
     {
-        ok( len == lstrlenW(url20), "expected len %u got %u\n", lstrlenW(url20), len );
+        ok( len == lstrlenW(url20), "expected len %u got %lu\n", lstrlenW(url20), len );
         ok( !lstrcmpW( url, url20 ), "url doesn't match %s\n", wine_dbgstr_w(url) );
     }
 
@@ -371,7 +372,7 @@ static void WinHttpCreateUrl_test( void )
     len = 256;
     ret = WinHttpCreateUrl( &uc, 0, url, &len );
     ok( ret, "expected success\n" );
-    ok( len == 58, "expected len 58 got %u\n", len );
+    ok( len == 58, "expected len 58 got %lu\n", len );
     ok( !lstrcmpW( url, url8 ), "url doesn't match\n" );
 
     HeapFree( GetProcessHeap(), 0, url );
@@ -417,21 +418,21 @@ static void WinHttpCrackUrl_test( void )
     uc.dwExtraInfoLength = 20;
 
     ret = WinHttpCrackUrl( url1, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nScheme == INTERNET_SCHEME_HTTP, "unexpected scheme: %u\n", uc.nScheme );
     ok( !memcmp( uc.lpszScheme, http, sizeof(http) ), "unexpected scheme: %s\n", wine_dbgstr_w(uc.lpszScheme) );
-    ok( uc.dwSchemeLength == 4, "unexpected scheme length: %u\n", uc.dwSchemeLength );
+    ok( uc.dwSchemeLength == 4, "unexpected scheme length: %lu\n", uc.dwSchemeLength );
     ok( !memcmp( uc.lpszUserName, username, sizeof(username) ), "unexpected username: %s\n", wine_dbgstr_w(uc.lpszUserName) );
-    ok( uc.dwUserNameLength == 8, "unexpected username length: %u\n", uc.dwUserNameLength );
+    ok( uc.dwUserNameLength == 8, "unexpected username length: %lu\n", uc.dwUserNameLength );
     ok( !memcmp( uc.lpszPassword, password, sizeof(password) ), "unexpected password: %s\n", wine_dbgstr_w(uc.lpszPassword) );
-    ok( uc.dwPasswordLength == 8, "unexpected password length: %u\n", uc.dwPasswordLength );
+    ok( uc.dwPasswordLength == 8, "unexpected password length: %lu\n", uc.dwPasswordLength );
     ok( !memcmp( uc.lpszHostName, winehq, sizeof(winehq) ), "unexpected hostname: %s\n", wine_dbgstr_w(uc.lpszHostName) );
-    ok( uc.dwHostNameLength == 14, "unexpected hostname length: %u\n", uc.dwHostNameLength );
+    ok( uc.dwHostNameLength == 14, "unexpected hostname length: %lu\n", uc.dwHostNameLength );
     ok( uc.nPort == 80, "unexpected port: %u\n", uc.nPort );
     ok( !memcmp( uc.lpszUrlPath, about, sizeof(about) ), "unexpected path: %s\n", wine_dbgstr_w(uc.lpszUrlPath) );
-    ok( uc.dwUrlPathLength == 11, "unexpected path length: %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 11, "unexpected path length: %lu\n", uc.dwUrlPathLength );
     ok( !memcmp( uc.lpszExtraInfo, query, sizeof(query) ), "unexpected extra info: %s\n", wine_dbgstr_w(uc.lpszExtraInfo) );
-    ok( uc.dwExtraInfoLength == 6, "unexpected extra info length: %u\n", uc.dwExtraInfoLength );
+    ok( uc.dwExtraInfoLength == 6, "unexpected extra info length: %lu\n", uc.dwExtraInfoLength );
 
     /* buffers of insufficient length */
     uc.dwSchemeLength   = 1;
@@ -441,19 +442,19 @@ static void WinHttpCrackUrl_test( void )
     ret = WinHttpCrackUrl( url1, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_INSUFFICIENT_BUFFER, "got %u, expected ERROR_INSUFFICIENT_BUFFER\n", error );
-    ok( uc.dwSchemeLength == 5, "unexpected scheme length: %u\n", uc.dwSchemeLength );
-    ok( uc.dwHostNameLength == 15, "unexpected hostname length: %u\n", uc.dwHostNameLength );
-    ok( uc.dwUrlPathLength == 11, "unexpected path length: %u\n", uc.dwUrlPathLength );
+    ok( error == ERROR_INSUFFICIENT_BUFFER, "got %lu, expected ERROR_INSUFFICIENT_BUFFER\n", error );
+    ok( uc.dwSchemeLength == 5, "unexpected scheme length: %lu\n", uc.dwSchemeLength );
+    ok( uc.dwHostNameLength == 15, "unexpected hostname length: %lu\n", uc.dwHostNameLength );
+    ok( uc.dwUrlPathLength == 11, "unexpected path length: %lu\n", uc.dwUrlPathLength );
 
     /* no buffers */
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url_k1, 0, 0, &uc);
     error = GetLastError();
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", error );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", error );
     ok( error == ERROR_SUCCESS || broken(error == ERROR_INVALID_PARAMETER) /* < win7 */,
-        "got %u, expected ERROR_SUCCESS\n", error );
+        "got %lu, expected ERROR_SUCCESS\n", error );
     ok( uc.nScheme == INTERNET_SCHEME_HTTP, "unexpected scheme\n" );
     ok( uc.lpszScheme == url_k1,"unexpected scheme\n" );
     ok( uc.dwSchemeLength == 4, "unexpected scheme length\n" );
@@ -473,7 +474,7 @@ static void WinHttpCrackUrl_test( void )
     uc.dwSchemeLength = uc.dwHostNameLength = uc.dwUserNameLength = 1;
     uc.dwPasswordLength = uc.dwUrlPathLength = uc.dwExtraInfoLength = 1;
     ret = WinHttpCrackUrl( url_k2, 0, 0,&uc);
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nScheme == INTERNET_SCHEME_HTTP, "unexpected scheme\n" );
     ok( uc.lpszScheme == url_k2, "unexpected scheme\n" );
     ok( uc.dwSchemeLength == 4, "unexpected scheme length\n" );
@@ -491,7 +492,7 @@ static void WinHttpCrackUrl_test( void )
 
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url_k3, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nScheme == INTERNET_SCHEME_HTTPS, "unexpected scheme\n" );
     ok( uc.lpszScheme == url_k3, "unexpected scheme\n" );
     ok( uc.dwSchemeLength == 5, "unexpected scheme length\n" );
@@ -513,80 +514,80 @@ static void WinHttpCrackUrl_test( void )
     ret = WinHttpCrackUrl( url_k4, 0, 0, &uc );
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
     error = GetLastError();
-    ok( error == ERROR_WINHTTP_INVALID_URL, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_INVALID_URL, "got %lu\n", error );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url_k5, 0, 0, &uc );
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
     error = GetLastError();
-    ok( error == ERROR_WINHTTP_INVALID_URL, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_INVALID_URL, "got %lu\n", error );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url_k6, 0, 0, &uc );
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
     error = GetLastError();
-    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %lu\n", error );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url_k7, 0, 0, &uc );
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
     error = GetLastError();
-    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %lu\n", error );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url_k8, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %lu\n", error );
 
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url_k9, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.lpszUrlPath == url_k9 + 14 || broken(uc.lpszUrlPath == url_k9 + 13) /* win8 */,
         "unexpected path: %s\n", wine_dbgstr_w(uc.lpszUrlPath) );
-    ok( uc.dwUrlPathLength == 0, "unexpected path length: %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 0, "unexpected path length: %lu\n", uc.dwUrlPathLength );
     ok( uc.lpszExtraInfo == url_k9 + 14 || broken(uc.lpszExtraInfo == url_k9 + 13) /* win8 */,
         "unexpected extra info: %s\n", wine_dbgstr_w(uc.lpszExtraInfo) );
     ok( uc.dwExtraInfoLength == 0 || broken(uc.dwExtraInfoLength == 1) /* win8 */,
-        "unexpected extra info length: %u\n", uc.dwExtraInfoLength );
+        "unexpected extra info length: %lu\n", uc.dwExtraInfoLength );
 
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url_k10, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.lpszUrlPath == url_k10 + 13, "unexpected path: %s\n", wine_dbgstr_w(uc.lpszUrlPath) );
-    ok( uc.dwUrlPathLength == 7, "unexpected path length: %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 7, "unexpected path length: %lu\n", uc.dwUrlPathLength );
     ok( uc.lpszExtraInfo == url_k10 + 20, "unexpected extra info: %s\n", wine_dbgstr_w(uc.lpszExtraInfo) );
-    ok( uc.dwExtraInfoLength == 0, "unexpected extra info length: %u\n", uc.dwExtraInfoLength );
+    ok( uc.dwExtraInfoLength == 0, "unexpected extra info length: %lu\n", uc.dwExtraInfoLength );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url4, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_WINHTTP_INVALID_URL, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_INVALID_URL, "got %lu\n", error );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( empty, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %lu\n", error );
 
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url1, 0, 0, NULL );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_INVALID_PARAMETER, "got %u\n", error );
+    ok( error == ERROR_INVALID_PARAMETER, "got %lu\n", error );
 
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( NULL, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_INVALID_PARAMETER, "got %u\n", error );
+    ok( error == ERROR_INVALID_PARAMETER, "got %lu\n", error );
 
     /* decoding without buffers */
     reset_url_components( &uc );
@@ -594,7 +595,7 @@ static void WinHttpCrackUrl_test( void )
     ret = WinHttpCrackUrl( url7, 0, ICU_DECODE, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_INVALID_PARAMETER, "got %u, expected ERROR_INVALID_PARAMETER\n", error );
+    ok( error == ERROR_INVALID_PARAMETER, "got %lu, expected ERROR_INVALID_PARAMETER\n", error );
 
     /* decoding with buffers */
     uc.lpszScheme = scheme;
@@ -613,11 +614,11 @@ static void WinHttpCrackUrl_test( void )
     path[0] = 0;
 
     ret = WinHttpCrackUrl( url7, 0, ICU_DECODE, &uc );
-    ok( ret, "WinHttpCrackUrl failed %u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed %lu\n", GetLastError() );
     ok( !memcmp( uc.lpszUrlPath + 11, escape, 21 * sizeof(WCHAR) ), "unexpected path\n" );
-    ok( uc.dwUrlPathLength == 32, "unexpected path length %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 32, "unexpected path length %lu\n", uc.dwUrlPathLength );
     ok( !memcmp( uc.lpszExtraInfo, escape + 21, 12 * sizeof(WCHAR) ), "unexpected extra info\n" );
-    ok( uc.dwExtraInfoLength == 12, "unexpected extra info length %u\n", uc.dwExtraInfoLength );
+    ok( uc.dwExtraInfoLength == 12, "unexpected extra info length %lu\n", uc.dwExtraInfoLength );
 
     /* Urls with specified port numbers */
     /* decoding with buffers */
@@ -637,25 +638,25 @@ static void WinHttpCrackUrl_test( void )
     path[0] = 0;
 
     ret = WinHttpCrackUrl( url6, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( !memcmp( uc.lpszHostName, winehq, sizeof(winehq) ), "unexpected host name: %s\n", wine_dbgstr_w(uc.lpszHostName) );
-    ok( uc.dwHostNameLength == 14, "unexpected host name length: %d\n", uc.dwHostNameLength );
+    ok( uc.dwHostNameLength == 14, "unexpected host name length: %lu\n", uc.dwHostNameLength );
     ok( uc.nPort == 42, "unexpected port: %u\n", uc.nPort );
 
     /* decoding without buffers */
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url8, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nPort == 0, "unexpected port: %u\n", uc.nPort );
 
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url9, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nPort == 80, "unexpected port: %u\n", uc.nPort );
 
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url10, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nPort == 443, "unexpected port: %u\n", uc.nPort );
 
     reset_url_components( &uc );
@@ -663,18 +664,18 @@ static void WinHttpCrackUrl_test( void )
     ret = WinHttpCrackUrl( empty, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %u, expected ERROR_WINHTTP_UNRECOGNIZED_SCHEME\n", error );
+    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %lu, expected ERROR_WINHTTP_UNRECOGNIZED_SCHEME\n", error );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( http, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %u, expected ERROR_WINHTTP_UNRECOGNIZED_SCHEME\n", error );
+    ok( error == ERROR_WINHTTP_UNRECOGNIZED_SCHEME, "got %lu, expected ERROR_WINHTTP_UNRECOGNIZED_SCHEME\n", error );
 
     reset_url_components( &uc );
     ret = WinHttpCrackUrl( url11, 0, 0, &uc);
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( uc.nScheme == INTERNET_SCHEME_HTTP, "unexpected scheme\n" );
     ok( uc.lpszScheme == url11,"unexpected scheme\n" );
     ok( uc.dwSchemeLength == 4, "unexpected scheme length\n" );
@@ -702,7 +703,7 @@ static void WinHttpCrackUrl_test( void )
     uc.dwExtraInfoLength = 0;
     uc.nPort = 0;
     ret = WinHttpCrackUrl( url12, 0, ICU_DECODE, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
 
     uc.lpszScheme = scheme;
     uc.dwSchemeLength = 20;
@@ -718,10 +719,10 @@ static void WinHttpCrackUrl_test( void )
     uc.dwExtraInfoLength = 0;
     uc.nPort = 0;
     ret = WinHttpCrackUrl( url13, 0, ICU_ESCAPE|ICU_DECODE, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( !lstrcmpW( uc.lpszHostName, L"winehq.o g" ), "unexpected host name\n" );
     ok( !lstrcmpW( uc.lpszUrlPath, L"/path%20with%20spaces" ), "unexpected path\n" );
-    ok( uc.dwUrlPathLength == lstrlenW(L"/path%20with%20spaces"), "got %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == lstrlenW(L"/path%20with%20spaces"), "got %lu\n", uc.dwUrlPathLength );
 
     uc.dwStructSize = sizeof(uc);
     uc.lpszScheme = NULL;
@@ -739,21 +740,21 @@ static void WinHttpCrackUrl_test( void )
     uc.lpszExtraInfo = NULL;
     uc.dwExtraInfoLength = ~0u;
     ret = WinHttpCrackUrl( url14, 0, 0, &uc );
-    ok( ret, "WinHttpCrackUrl failed le=%u\n", GetLastError() );
+    ok( ret, "WinHttpCrackUrl failed le = %lu\n", GetLastError() );
     ok( !uc.lpszScheme, "unexpected scheme %s\n", wine_dbgstr_w(uc.lpszScheme) );
-    ok( !uc.dwSchemeLength, "unexpected length %u\n", uc.dwSchemeLength );
+    ok( !uc.dwSchemeLength, "unexpected length %lu\n", uc.dwSchemeLength );
     ok( uc.nScheme == INTERNET_SCHEME_HTTP, "unexpected scheme %u\n", uc.nScheme );
     ok( !lstrcmpW( uc.lpszHostName, url14 + 7 ), "unexpected hostname %s\n", wine_dbgstr_w(uc.lpszHostName) );
-    ok( uc.dwHostNameLength == 14, "unexpected length %u\n", uc.dwHostNameLength );
+    ok( uc.dwHostNameLength == 14, "unexpected length %lu\n", uc.dwHostNameLength );
     ok( uc.nPort == 80, "unexpected port %u\n", uc.nPort );
     ok( !uc.lpszUserName, "unexpected username\n" );
-    ok( !uc.dwUserNameLength, "unexpected length %u\n", uc.dwUserNameLength );
+    ok( !uc.dwUserNameLength, "unexpected length %lu\n", uc.dwUserNameLength );
     ok( !uc.lpszPassword, "unexpected password\n" );
-    ok( !uc.dwPasswordLength, "unexpected length %u\n", uc.dwPasswordLength );
+    ok( !uc.dwPasswordLength, "unexpected length %lu\n", uc.dwPasswordLength );
     ok( !lstrcmpW( uc.lpszUrlPath, url14 + 21 ), "unexpected path %s\n", wine_dbgstr_w(uc.lpszUrlPath) );
-    ok( uc.dwUrlPathLength == 5, "unexpected length %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 5, "unexpected length %lu\n", uc.dwUrlPathLength );
     ok( !uc.lpszExtraInfo[0], "unexpected extra info %s\n", wine_dbgstr_w(uc.lpszExtraInfo) );
-    ok( uc.dwExtraInfoLength == 0, "unexpected length %u\n", uc.dwExtraInfoLength );
+    ok( uc.dwExtraInfoLength == 0, "unexpected length %lu\n", uc.dwExtraInfoLength );
 
     uc.dwStructSize = sizeof(uc);
     uc.lpszScheme = scheme;
@@ -774,40 +775,46 @@ static void WinHttpCrackUrl_test( void )
     ret = WinHttpCrackUrl( url14, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_INVALID_PARAMETER, "got %u\n", error );
+    ok( error == ERROR_INVALID_PARAMETER, "got %lu\n", error );
     ok( !lstrcmpW( uc.lpszScheme, http ), "unexpected scheme %s\n", wine_dbgstr_w(uc.lpszScheme) );
-    ok( !uc.dwSchemeLength, "unexpected length %u\n", uc.dwSchemeLength );
+    ok( !uc.dwSchemeLength, "unexpected length %lu\n", uc.dwSchemeLength );
     ok( uc.nScheme == 0, "unexpected scheme %u\n", uc.nScheme );
     ok( !uc.lpszHostName, "unexpected hostname %s\n", wine_dbgstr_w(uc.lpszHostName) );
-    ok( uc.dwHostNameLength == 0, "unexpected length %u\n", uc.dwHostNameLength );
+    ok( uc.dwHostNameLength == 0, "unexpected length %lu\n", uc.dwHostNameLength );
     ok( uc.nPort == 0, "unexpected port %u\n", uc.nPort );
     ok( !uc.lpszUserName, "unexpected username\n" );
-    ok( uc.dwUserNameLength == ~0u, "unexpected length %u\n", uc.dwUserNameLength );
+    ok( uc.dwUserNameLength == ~0u, "unexpected length %lu\n", uc.dwUserNameLength );
     ok( !uc.lpszPassword, "unexpected password\n" );
-    ok( uc.dwPasswordLength == ~0u, "unexpected length %u\n", uc.dwPasswordLength );
+    ok( uc.dwPasswordLength == ~0u, "unexpected length %lu\n", uc.dwPasswordLength );
     ok( !uc.lpszUrlPath, "unexpected path %s\n", wine_dbgstr_w(uc.lpszUrlPath) );
-    ok( uc.dwUrlPathLength == 0, "unexpected length %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 0, "unexpected length %lu\n", uc.dwUrlPathLength );
     ok( !uc.lpszExtraInfo, "unexpected extra info %s\n", wine_dbgstr_w(uc.lpszExtraInfo) );
-    ok( uc.dwExtraInfoLength == 0, "unexpected length %u\n", uc.dwExtraInfoLength );
+    ok( uc.dwExtraInfoLength == 0, "unexpected length %lu\n", uc.dwExtraInfoLength );
 
     reset_url_components( &uc );
     SetLastError( 0xdeadbeef );
     ret = WinHttpCrackUrl( url15, 0, 0, &uc );
     error = GetLastError();
     ok( !ret, "WinHttpCrackUrl succeeded\n" );
-    ok( error == ERROR_WINHTTP_INVALID_URL, "got %u\n", error );
+    ok( error == ERROR_WINHTTP_INVALID_URL, "got %lu\n", error );
 
     reset_url_components( &uc );
     uc.nPort = 1;
     ret = WinHttpCrackUrl( url16, 0, 0, &uc );
-    ok( ret, "got %u\n", GetLastError() );
+    ok( ret, "got %lu\n", GetLastError() );
     ok( !uc.nPort, "got %u\n", uc.nPort );
 
     reset_url_components( &uc );
     uc.nPort = 1;
     ret = WinHttpCrackUrl( url17, 0, 0, &uc );
-    ok( ret, "got %u\n", GetLastError() );
-    todo_wine ok( uc.nPort == 80, "got %u\n", uc.nPort );
+    ok( ret, "got %lu\n", GetLastError() );
+    ok( uc.nPort == 80, "got %u\n", uc.nPort );
+
+    reset_url_components( &uc );
+    uc.nPort = 1;
+    ret = WinHttpCrackUrl( url22, 0, 0, &uc );
+    ok( ret, "got %lu\n", GetLastError() );
+    ok( uc.nPort == 80, "got %u\n", uc.nPort );
 
     memset( &uc, 0, sizeof(uc) );
     uc.dwStructSize      = sizeof(uc);
@@ -818,9 +825,9 @@ static void WinHttpCrackUrl_test( void )
     uc.lpszUrlPath       = path;
     uc.dwUrlPathLength   = ARRAY_SIZE(path);
     ret = WinHttpCrackUrl( url21, 0, 0, &uc );
-    ok( ret, "got %u\n", GetLastError() );
+    ok( ret, "got %lu\n", GetLastError() );
     ok( !lstrcmpW( uc.lpszUrlPath, url21 + 37 ), "unexpected path %s\n", wine_dbgstr_w(uc.lpszUrlPath) );
-    ok( uc.dwUrlPathLength == 50, "unexpected length %u\n", uc.dwUrlPathLength );
+    ok( uc.dwUrlPathLength == 50, "unexpected length %lu\n", uc.dwUrlPathLength );
 }
 
 START_TEST(url)
