@@ -152,9 +152,25 @@ static BOOL WININET_GetSetPassword( HWND hdlg, LPCWSTR szServer,
     p = wcschr( szUserPass, ':' );
     if( p )
     {
+        struct WININET_ErrorDlgParams *params;
+        HWND hwnd;
+
+        params = (struct WININET_ErrorDlgParams*)
+                 GetWindowLongPtrW( hdlg, GWLP_USERDATA );
+
         *p = 0;
         SetWindowTextW( hUserItem, szUserPass );
-        SetWindowTextW( hPassItem, p+1 );
+
+        if (!params->req->clear_auth)
+        {
+            SetWindowTextW( hPassItem, p+1 );
+
+            hwnd = GetDlgItem( hdlg, IDC_SAVEPASSWORD );
+            if (hwnd)
+                SendMessageW(hwnd, BM_SETCHECK, BST_CHECKED, 0);
+        }
+        else
+            WININET_GetSetPassword( hdlg, szServer, szRealm, TRUE );
     }
 
     return TRUE;
@@ -169,14 +185,14 @@ static BOOL WININET_SetAuthorization( http_request_t *request, LPWSTR username,
     http_session_t *session = request->session;
     LPWSTR p, q;
 
-    p = heap_strdupW(username);
+    p = wcsdup(username);
     if( !p )
         return FALSE;
 
-    q = heap_strdupW(password);
+    q = wcsdup(password);
     if( !q )
     {
-        heap_free(p);
+        free(p);
         return FALSE;
     }
 
@@ -184,18 +200,18 @@ static BOOL WININET_SetAuthorization( http_request_t *request, LPWSTR username,
     {
         appinfo_t *hIC = session->appInfo;
 
-        heap_free(hIC->proxyUsername);
+        free(hIC->proxyUsername);
         hIC->proxyUsername = p;
 
-        heap_free(hIC->proxyPassword);
+        free(hIC->proxyPassword);
         hIC->proxyPassword = q;
     }
     else
     {
-        heap_free(session->userName);
+        free(session->userName);
         session->userName = p;
 
-        heap_free(session->password);
+        free(session->password);
         session->password = q;
     }
 
@@ -214,7 +230,7 @@ static INT_PTR WINAPI WININET_ProxyPasswordDialog(
 
     if( uMsg == WM_INITDIALOG )
     {
-        TRACE("WM_INITDIALOG (%08lx)\n", lParam);
+        TRACE("WM_INITDIALOG (%08Ix)\n", lParam);
 
         /* save the parameter list */
         params = (struct WININET_ErrorDlgParams*) lParam;
@@ -264,6 +280,8 @@ static INT_PTR WINAPI WININET_ProxyPasswordDialog(
                 WININET_GetSetPassword( hdlg, params->req->session->appInfo->proxy, szRealm, TRUE );
             WININET_SetAuthorization( params->req, username, password, TRUE );
 
+            params->req->clear_auth = TRUE;
+
             EndDialog( hdlg, ERROR_INTERNET_FORCE_RETRY );
             return TRUE;
         }
@@ -289,7 +307,7 @@ static INT_PTR WINAPI WININET_PasswordDialog(
 
     if( uMsg == WM_INITDIALOG )
     {
-        TRACE("WM_INITDIALOG (%08lx)\n", lParam);
+        TRACE("WM_INITDIALOG (%08Ix)\n", lParam);
 
         /* save the parameter list */
         params = (struct WININET_ErrorDlgParams*) lParam;
@@ -340,6 +358,7 @@ static INT_PTR WINAPI WININET_PasswordDialog(
                 WININET_GetSetPassword( hdlg, params->req->session->hostName, szRealm, TRUE );
             }
             WININET_SetAuthorization( params->req, username, password, FALSE );
+            params->req->clear_auth = TRUE;
 
             EndDialog( hdlg, ERROR_INTERNET_FORCE_RETRY );
             return TRUE;
@@ -366,7 +385,7 @@ static INT_PTR WINAPI WININET_InvalidCertificateDialog(
 
     if( uMsg == WM_INITDIALOG )
     {
-        TRACE("WM_INITDIALOG (%08lx)\n", lParam);
+        TRACE("WM_INITDIALOG (%08Ix)\n", lParam);
 
         /* save the parameter list */
         params = (struct WININET_ErrorDlgParams*) lParam;
@@ -390,7 +409,7 @@ static INT_PTR WINAPI WININET_InvalidCertificateDialog(
             LoadStringW( WININET_hModule, IDS_CERT_ERRORS, buf, 1024 );
             break;
         default:
-            FIXME( "No message for error %d\n", params->dwError );
+            FIXME( "No message for error %ld\n", params->dwError );
             buf[0] = '\0';
         }
 
@@ -472,7 +491,7 @@ DWORD WINAPI InternetErrorDlg(HWND hWnd, HINTERNET hRequest,
     http_request_t *req = NULL;
     DWORD res = ERROR_SUCCESS;
 
-    TRACE("%p %p %d %08x %p\n", hWnd, hRequest, dwError, dwFlags, lppvData);
+    TRACE("%p %p %ld %08lx %p\n", hWnd, hRequest, dwError, dwFlags, lppvData);
 
     if( !hWnd && !(dwFlags & FLAGS_ERROR_UI_FLAGS_NO_UI) )
         return ERROR_INVALID_HANDLE;
@@ -509,8 +528,11 @@ DWORD WINAPI InternetErrorDlg(HWND hWnd, HINTERNET hRequest,
             res = DialogBoxParamW( WININET_hModule, MAKEINTRESOURCEW( IDD_AUTHDLG ),
                                     hWnd, WININET_PasswordDialog, (LPARAM) &params );
             break;
+        case HTTP_STATUS_OK:
+            req->clear_auth = FALSE;
+            break;
         default:
-            WARN("unhandled status %u\n", req->status_code);
+            WARN("unhandled status %lu\n", req->status_code);
         }
         break;
     }
@@ -531,7 +553,7 @@ DWORD WINAPI InternetErrorDlg(HWND hWnd, HINTERNET hRequest,
         }
 
         if( dwFlags & ~FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS )
-            FIXME("%08x contains unsupported flags.\n", dwFlags);
+            FIXME("%08lx contains unsupported flags.\n", dwFlags);
 
         res = DialogBoxParamW( WININET_hModule, MAKEINTRESOURCEW( IDD_INVCERTDLG ),
                                hWnd, WININET_InvalidCertificateDialog, (LPARAM) &params );
@@ -542,7 +564,7 @@ DWORD WINAPI InternetErrorDlg(HWND hWnd, HINTERNET hRequest,
             res = ERROR_HTTP_COOKIE_DECLINED;
             break;
         }
-        FIXME("Need to display dialog for error %d\n", dwError);
+        FIXME("Need to display dialog for error %ld\n", dwError);
         res = ERROR_CANCELLED;
         break;
 
@@ -556,7 +578,7 @@ DWORD WINAPI InternetErrorDlg(HWND hWnd, HINTERNET hRequest,
     case ERROR_INTERNET_MIXED_SECURITY:
     case ERROR_INTERNET_HTTPS_HTTP_SUBMIT_REDIR:
         if(!(dwFlags & FLAGS_ERROR_UI_FLAGS_NO_UI))
-            FIXME("Need to display dialog for error %d\n", dwError);
+            FIXME("Need to display dialog for error %ld\n", dwError);
         res = ERROR_CANCELLED;
         break;
 
@@ -566,14 +588,14 @@ DWORD WINAPI InternetErrorDlg(HWND hWnd, HINTERNET hRequest,
             res = ERROR_SUCCESS;
             break;
         }
-        FIXME("Need to display dialog for error %d\n", dwError);
+        FIXME("Need to display dialog for error %ld\n", dwError);
         res = ERROR_CANCELLED;
         break;
 
     case ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR:
     case ERROR_INTERNET_POST_IS_NON_SECURE:
         if (!(dwFlags & FLAGS_ERROR_UI_FLAGS_NO_UI))
-            FIXME("Need to display dialog for error %d\n", dwError);
+            FIXME("Need to display dialog for error %ld\n", dwError);
         res = ERROR_SUCCESS;
         break;
 
@@ -611,7 +633,7 @@ BOOL WINAPI InternetShowSecurityInfoByURLW(LPCWSTR url, HWND window)
  */
 DWORD WINAPI ParseX509EncodedCertificateForListBoxEntry(LPBYTE cert, DWORD len, LPSTR szlistbox, LPDWORD listbox)
 {
-   FIXME("stub: %p %d %s %p\n", cert, len, debugstr_a(szlistbox), listbox);
+   FIXME("stub: %p %ld %s %p\n", cert, len, debugstr_a(szlistbox), listbox);
    return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
