@@ -282,13 +282,59 @@ static void test_InternetQueryOptionA(void)
 
   InternetCloseHandle(hinet);
 
+  /* Connect timeout */
   val = 12345;
   res = InternetSetOptionA(NULL, INTERNET_OPTION_CONNECT_TIMEOUT, &val, sizeof(val));
   ok(res, "InternetSetOptionA(INTERNET_OPTION_CONNECT_TIMEOUT) failed (%lu)\n", GetLastError());
 
   len = sizeof(val);
   res = InternetQueryOptionA(NULL, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
-  ok(res, "InternetQueryOptionA failed %ld)\n", GetLastError());
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_CONNECT_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 12345, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Receive Timeout */
+  val = 54321;
+  res = InternetSetOptionA(NULL, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  len = sizeof(val);
+  res = InternetQueryOptionA(NULL, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 54321, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Send Timeout */
+  val = 12345;
+  res = InternetSetOptionA(NULL, INTERNET_OPTION_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  len = sizeof(val);
+  res = InternetQueryOptionA(NULL, INTERNET_OPTION_SEND_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_SEND_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 12345, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Data Receive Timeout */
+  val = 54321;
+  res = InternetSetOptionA(NULL, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  len = sizeof(val);
+  res = InternetQueryOptionA(NULL, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 54321, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Data Send Timeout */
+  val = 12345;
+  res = InternetSetOptionA(NULL, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  val = 0xdeadbeef;
+  len = sizeof(val);
+  res = InternetQueryOptionA(NULL, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_DATA_SEND_TIMEOUT) failed %ld)\n", GetLastError());
   ok(val == 12345, "val = %ld\n", val);
   ok(len == sizeof(val), "len = %ld\n", len);
 
@@ -308,6 +354,7 @@ static void test_InternetQueryOptionA(void)
   ok(!res, "InternetQueryOptionA(INTERNET_OPTION_MAX_CONNS_PER_SERVER) succeeded\n");
   ok(GetLastError() == ERROR_INTERNET_INVALID_OPERATION, "GetLastError() = %lu\n", GetLastError());
 
+  /* Connect Timeout */
   val = 2;
   res = InternetSetOptionA(hinet, INTERNET_OPTION_MAX_CONNS_PER_SERVER, &val, sizeof(val));
   ok(!res, "InternetSetOptionA(INTERNET_OPTION_MAX_CONNS_PER_SERVER) succeeded\n");
@@ -315,7 +362,7 @@ static void test_InternetQueryOptionA(void)
 
   len = sizeof(val);
   res = InternetQueryOptionA(hinet, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
-  ok(res, "InternetQueryOptionA failed %ld)\n", GetLastError());
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_CONNECT_TIMEOUT) failed %ld)\n", GetLastError());
   ok(val == 12345, "val = %ld\n", val);
   ok(len == sizeof(val), "len = %ld\n", len);
 
@@ -325,16 +372,141 @@ static void test_InternetQueryOptionA(void)
 
   len = sizeof(val);
   res = InternetQueryOptionA(hinet, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
-  ok(res, "InternetQueryOptionA failed %ld)\n", GetLastError());
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_CONNECT_TIMEOUT) failed %ld)\n", GetLastError());
   ok(val == 1, "val = %ld\n", val);
   ok(len == sizeof(val), "len = %ld\n", len);
 
+  /* Receive Timeout */
+  val = 60;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
   len = sizeof(val);
-  res = InternetQueryOptionA(NULL, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
-  ok(res, "InternetQueryOptionA failed %ld)\n", GetLastError());
-  ok(val == 12345, "val = %ld\n", val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 60, "val = %ld\n", val);
   ok(len == sizeof(val), "len = %ld\n", len);
 
+  /* Send Timeout */
+  val = 120;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  len = sizeof(val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_SEND_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_SEND_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 120, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Data Receive Timeout */
+  val = 60;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  len = sizeof(val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_DATA_RECEIVE_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 60, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Data Send Timeout */
+  val = 120;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  len = sizeof(val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, &len);
+  ok(res, "InternetQueryOptionA(INTERNET_OPTION_DATA_SEND_TIMEOUT) failed %ld)\n", GetLastError());
+  ok(val == 120, "val = %ld\n", val);
+  ok(len == sizeof(val), "len = %ld\n", len);
+
+  /* Timeout inheritance */
+  val = 15000;
+  len = sizeof(val);
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_CONNECT_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_CONNECT_TIMEOUT) failed (%lu)\n", GetLastError());
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  hurl = InternetConnectA(hinet,"www.winehq.org",INTERNET_DEFAULT_HTTP_PORT,NULL,NULL,INTERNET_SERVICE_HTTP,0,0);
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
+  ok(val == 15000, "failed to inherit INTERNET_OPTION_CONNECT_TIMEOUT on child connection (found %ld) - Error: %ld)\n", val, GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_SEND_TIMEOUT, &val, &len);
+  ok(val == 15000, "failed to inherit INTERNET_OPTION_SEND_TIMEOUT on child connection (found %ld) - Error: %ld)\n", val, GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, &len);
+  ok(val == 15000, "failed to inherit INTERNET_OPTION_RECEIVE_TIMEOUT on child connection (found %ld) - Error: %ld)\n", val, GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, &len);
+  ok(val == 15000, "failed to inherit INTERNET_OPTION_DATA_SEND_TIMEOUTt on child connection (found %ld) - Error: %ld)\n", val, GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, &len);
+  ok(val == 15000, "failed to inherit INTERNET_OPTION_DATA_RECEIVE_TIMEOUT on child connection (found %ld) - Error: %ld)\n", val, GetLastError());
+
+  val = 12345;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_CONNECT_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_CONNECT_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
+  ok(val == 15000, "Connection handle inherited value (INTERNET_OPTION_CONNECT_TIMEOUT) as %ld\n", val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_CONNECT_TIMEOUT, &val, &len);
+  ok(val == 12345, "Parent handle set from inherited value (INTERNET_OPTION_CONNECT_TIMEOUT) as %ld\n", val);
+
+  val = 12345;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_SEND_TIMEOUT, &val, &len);
+  ok(val == 15000, "Connection handle inherited value (INTERNET_OPTION_SEND_TIMEOUT) as %ld\n", val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_SEND_TIMEOUT, &val, &len);
+  ok(val == 12345, "Parent handle set from inherited value (INTERNET_OPTION_SEND_TIMEOUT) as %ld\n", val);
+
+  val = 12345;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, &len);
+  ok(val == 15000, "Connection handle inherited value (INTERNET_OPTION_RECEIVE_TIMEOUT) as %ld\n", val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_RECEIVE_TIMEOUT, &val, &len);
+  ok(val == 12345, "Parent handle set from inherited value (INTERNET_OPTION_RECEIVE_TIMEOUT) as %ld\n", val);
+
+  val = 12345;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_SEND_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, &len);
+  ok(val == 15000, "Connection handle inherited value (INTERNET_OPTION_DATA_SEND_TIMEOUT) as %ld\n", val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_DATA_SEND_TIMEOUT, &val, &len);
+  ok(val == 12345, "Parent handle set from inherited value (INTERNET_OPTION_DATA_SEND_TIMEOUT) as %ld\n", val);
+
+  val = 12345;
+  res = InternetSetOptionA(hinet, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, sizeof(val));
+  ok(res, "InternetSetOptionA(INTERNET_OPTION_DATA_RECEIVE_TIMEOUT) failed (%lu)\n", GetLastError());
+
+  val = 0xdeadbeef;
+  res = InternetQueryOptionA(hurl, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, &len);
+  ok(val == 15000, "Connection handle inherited value (INTERNET_OPTION_DATA_RECEIVE_TIMEOUT) as %ld\n", val);
+  res = InternetQueryOptionA(hinet, INTERNET_OPTION_DATA_RECEIVE_TIMEOUT, &val, &len);
+  ok(val == 12345, "Parent handle set from inherited value (INTERNET_OPTION_DATA_RECEIVE_TIMEOUT) as %ld\n", val);
+
+  InternetCloseHandle(hurl);
   InternetCloseHandle(hinet);
 }
 
@@ -944,9 +1116,11 @@ static void test_InternetTimeToSystemTime(void)
     test_data[] =
     {
         { "Fri, 07 Jan 2005 12:06:35 GMT", &expect1, TRUE },
+        { "Fri, 07 Jan 2005 12:06:35 UTC", &expect1, TRUE },
         { " fri, 7 jan 2005 12 06 35",     &expect1, TRUE },
         { "Fri, 07-01-2005 12:06:35",      &expect1, TRUE },
         { "5, 07-01-2005 12:06:35 GMT",    &expect1, TRUE },
+        { "5, 07-01-2005 12:06:35 UTC",    &expect1, TRUE },
         { "5, 07-01-2005 12:06:35 GMT;",   &expect1, TRUE },
         { "5, 07-01-2005 12:06:35 GMT123", &expect1, TRUE },
         { "2, 11 01 2022 11 13 05",        &expect2, TRUE },
@@ -954,6 +1128,9 @@ static void test_InternetTimeToSystemTime(void)
         { "2, 11*01/2022 11+13=05",        &expect2, TRUE },
         { "2, 11-Jan-2022 11:13:05",       &expect2, TRUE },
         { "Fr",                            NULL,     FALSE },
+        { "Fri Jan 7 12:06:35 2005",       &expect1, TRUE, TRUE },
+        { "Fri Jan 7 12:06:35 2005 GMT",   &expect1, TRUE, TRUE },
+        { "Fri Jan 7 12:06:35 2005 UTC",   &expect1, TRUE, TRUE },
     };
 
     ret = pInternetTimeToSystemTimeA(NULL, NULL, 0);
@@ -1260,156 +1437,103 @@ static void test_end_browser_session(void)
     ok(!len, "len = %lu\n", len);
 }
 
-#define verifyProxyEnable(e) r_verifyProxyEnable(__LINE__, e)
-static void r_verifyProxyEnable(LONG l, DWORD exp)
-{
-    HKEY hkey;
-    DWORD type, val, size = sizeof(DWORD);
-    LONG ret;
-    static const CHAR szInternetSettings[] = "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings";
-    static const CHAR szProxyEnable[] = "ProxyEnable";
-
-    ret = RegOpenKeyA(HKEY_CURRENT_USER, szInternetSettings, &hkey);
-    ok_(__FILE__,l) (!ret, "RegOpenKeyA failed: 0x%08lx\n", ret);
-
-    ret = RegQueryValueExA(hkey, szProxyEnable, 0, &type, (BYTE*)&val, &size);
-    ok_(__FILE__,l) (!ret, "RegQueryValueExA failed: 0x%08lx\n", ret);
-    ok_(__FILE__,l) (type == REG_DWORD, "Expected regtype to be REG_DWORD, was: %ld\n", type);
-    ok_(__FILE__,l) (val == exp, "Expected ProxyEnabled to be %ld, got: %ld\n", exp, val);
-
-    ret = RegCloseKey(hkey);
-    ok_(__FILE__,l) (!ret, "RegCloseKey failed: 0x%08lx\n", ret);
-}
-
 static void test_Option_PerConnectionOption(void)
 {
     BOOL ret;
     DWORD size = sizeof(INTERNET_PER_CONN_OPTION_LISTW);
     INTERNET_PER_CONN_OPTION_LISTW list = {size};
-    INTERNET_PER_CONN_OPTIONW *orig_settings;
-    static WCHAR proxy_srvW[] = {'p','r','o','x','y','.','e','x','a','m','p','l','e',0};
+    INTERNET_PER_CONN_OPTIONW orig_settings[2], options[2];
+    static WCHAR proxy_srvW[] = L"proxy.example";
+    HINTERNET ses;
 
-    /* get the global IE proxy server info, to restore later */
+    ses = InternetOpenA(NULL, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
+    ok(ses != 0, "InternetOpen failed: 0x%08lx\n", GetLastError());
+
+    /* get the process IE proxy server info, to restore later */
     list.dwOptionCount = 2;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, 2 * sizeof(INTERNET_PER_CONN_OPTIONW));
+    list.pOptions = orig_settings;
+    orig_settings[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
+    orig_settings[1].dwOption = INTERNET_PER_CONN_FLAGS;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[1].dwOption = INTERNET_PER_CONN_FLAGS;
+    ret = InternetQueryOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
+    ok(ret, "InternetQueryOption should've succeeded\n");
 
-    ret = InternetQueryOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    orig_settings = list.pOptions;
+    /* set the process IE proxy server */
+    list.pOptions = options;
+    options[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
+    options[0].Value.pszValue = proxy_srvW;
+    options[1].dwOption = INTERNET_PER_CONN_FLAGS;
+    options[1].Value.dwValue = PROXY_TYPE_PROXY;
 
-    /* set the global IE proxy server */
-    list.dwOptionCount = 2;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, 2 * sizeof(INTERNET_PER_CONN_OPTIONW));
+    ret = InternetSetOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
+    ok(ret, "InternetSetOption should've succeeded\n");
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[0].Value.pszValue = proxy_srvW;
-    list.pOptions[1].dwOption = INTERNET_PER_CONN_FLAGS;
-    list.pOptions[1].Value.dwValue = PROXY_TYPE_PROXY;
+    /* get & verify the process IE proxy server */
+    options[0].Value.pszValue = NULL;
+    options[1].Value.dwValue = 0;
 
-    ret = InternetSetOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
-
-    /* get & verify the global IE proxy server */
-    list.dwOptionCount = 2;
-    list.dwOptionError = 0;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, 2 * sizeof(INTERNET_PER_CONN_OPTIONW));
-
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[1].dwOption = INTERNET_PER_CONN_FLAGS;
-
-    ret = InternetQueryOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    ok(!lstrcmpW(list.pOptions[0].Value.pszValue, proxy_srvW),
+    ret = InternetQueryOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
+    ok(ret, "InternetQueryOption should've succeeded\n");
+    ok(!lstrcmpW(options[0].Value.pszValue, proxy_srvW),
             "Retrieved proxy server should've been %s, was: %s\n",
-            wine_dbgstr_w(proxy_srvW), wine_dbgstr_w(list.pOptions[0].Value.pszValue));
-    ok(list.pOptions[1].Value.dwValue == PROXY_TYPE_PROXY,
+            wine_dbgstr_w(proxy_srvW), wine_dbgstr_w(options[0].Value.pszValue));
+    ok(options[1].Value.dwValue == PROXY_TYPE_PROXY,
             "Retrieved flags should've been PROXY_TYPE_PROXY, was: %ld\n",
-            list.pOptions[1].Value.dwValue);
-    verifyProxyEnable(1);
+            options[1].Value.dwValue);
 
-    ret = HeapValidate(GetProcessHeap(), 0, list.pOptions[0].Value.pszValue);
+    ret = HeapValidate(GetProcessHeap(), 0, options[0].Value.pszValue);
     ok(ret, "HeapValidate failed, last error %lu\n", GetLastError());
+    GlobalFree(options[0].Value.pszValue);
 
-    HeapFree(GetProcessHeap(), 0, list.pOptions[0].Value.pszValue);
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    /* verify that global proxy settings were not changed */
+    ret = InternetQueryOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
+    ok(ret, "InternetQueryOption should've succeeded\n");
+    ok(lstrcmpW(options[0].Value.pszValue, proxy_srvW),
+            "Retrieved proxy server should've been %s, was: %s\n",
+            wine_dbgstr_w(proxy_srvW), wine_dbgstr_w(options[0].Value.pszValue));
 
     /* disable the proxy server */
     list.dwOptionCount = 1;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, sizeof(INTERNET_PER_CONN_OPTIONW));
+    options[0].dwOption = INTERNET_PER_CONN_FLAGS;
+    options[0].Value.dwValue = PROXY_TYPE_DIRECT;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_FLAGS;
-    list.pOptions[0].Value.dwValue = PROXY_TYPE_DIRECT;
-
-    ret = InternetSetOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    ret = InternetSetOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
+    ok(ret, "InternetSetOption should've succeeded\n");
 
     /* verify that the proxy is disabled */
-    list.dwOptionCount = 1;
-    list.dwOptionError = 0;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, sizeof(INTERNET_PER_CONN_OPTIONW));
+    options[0].Value.dwValue = 0;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_FLAGS;
-
-    ret = InternetQueryOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    ok(list.pOptions[0].Value.dwValue == PROXY_TYPE_DIRECT,
+    ret = InternetQueryOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
+    ok(ret, "InternetQueryOption should've succeeded\n");
+    ok(options[0].Value.dwValue == PROXY_TYPE_DIRECT,
             "Retrieved flags should've been PROXY_TYPE_DIRECT, was: %ld\n",
-            list.pOptions[0].Value.dwValue);
-    verifyProxyEnable(0);
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+            options[0].Value.dwValue);
 
     /* set the proxy flags to 'invalid' value */
-    list.dwOptionCount = 1;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, sizeof(INTERNET_PER_CONN_OPTIONW));
+    options[0].dwOption = INTERNET_PER_CONN_FLAGS;
+    options[0].Value.dwValue = PROXY_TYPE_PROXY | PROXY_TYPE_DIRECT;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_FLAGS;
-    list.pOptions[0].Value.dwValue = PROXY_TYPE_PROXY | PROXY_TYPE_DIRECT;
-
-    ret = InternetSetOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    ret = InternetSetOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
+    ok(ret, "InternetSetOption should've succeeded\n");
 
     /* verify that the proxy is enabled */
-    list.dwOptionCount = 1;
-    list.dwOptionError = 0;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, sizeof(INTERNET_PER_CONN_OPTIONW));
+    options[0].Value.dwValue = 0;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_FLAGS;
-
-    ret = InternetQueryOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    todo_wine ok(list.pOptions[0].Value.dwValue == (PROXY_TYPE_PROXY | PROXY_TYPE_DIRECT),
-            "Retrieved flags should've been PROXY_TYPE_PROXY | PROXY_TYPE_DIRECT, was: %ld\n",
-            list.pOptions[0].Value.dwValue);
-    verifyProxyEnable(1);
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    ret = InternetQueryOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
+    ok(ret, "InternetQueryOption should've succeeded\n");
+    todo_wine ok(options[0].Value.dwValue == PROXY_TYPE_DIRECT,
+            "Retrieved flags should've been PROXY_TYPE_DIRECT, was: %ld\n",
+            options[0].Value.dwValue);
 
     /* restore original settings */
     list.dwOptionCount = 2;
     list.pOptions = orig_settings;
 
-    ret = InternetSetOptionW(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
+    ret = InternetSetOptionW(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
+    ok(ret, "InternetSetOption should've succeeded\n");
 
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    GlobalFree(orig_settings[0].Value.pszValue);
+    InternetCloseHandle(ses);
 }
 
 static void test_Option_PerConnectionOptionA(void)
@@ -1417,95 +1541,76 @@ static void test_Option_PerConnectionOptionA(void)
     BOOL ret;
     DWORD size = sizeof(INTERNET_PER_CONN_OPTION_LISTA);
     INTERNET_PER_CONN_OPTION_LISTA list = {size};
-    INTERNET_PER_CONN_OPTIONA *orig_settings;
+    INTERNET_PER_CONN_OPTIONA orig_settings[2], options[2];
     char proxy_srv[] = "proxy.example";
+    HINTERNET ses;
 
-    /* get the global IE proxy server info, to restore later */
+    ses = InternetOpenA(NULL, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
+    ok(ses != 0, "InternetOpen failed: 0x%08lx\n", GetLastError());
+
+    /* get the process IE proxy server info, to restore later */
     list.dwOptionCount = 2;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, 2 * sizeof(INTERNET_PER_CONN_OPTIONA));
+    list.pOptions = orig_settings;
+    orig_settings[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
+    orig_settings[1].dwOption = INTERNET_PER_CONN_FLAGS;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[1].dwOption = INTERNET_PER_CONN_FLAGS;
-
-    ret = InternetQueryOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
+    ret = InternetQueryOptionA(ses, INTERNET_OPTION_PER_CONNECTION_OPTION,
             &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    orig_settings = list.pOptions;
+    ok(ret, "InternetQueryOption should've succeeded\n");
 
-    /* set the global IE proxy server */
-    list.dwOptionCount = 2;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, 2 * sizeof(INTERNET_PER_CONN_OPTIONA));
+    /* set the process IE proxy server */
+    list.pOptions = options;
+    options[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
+    options[0].Value.pszValue = proxy_srv;
+    options[1].dwOption = INTERNET_PER_CONN_FLAGS;
+    options[1].Value.dwValue = PROXY_TYPE_PROXY;
 
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[0].Value.pszValue = proxy_srv;
-    list.pOptions[1].dwOption = INTERNET_PER_CONN_FLAGS;
-    list.pOptions[1].Value.dwValue = PROXY_TYPE_PROXY;
-
-    ret = InternetSetOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
+    ret = InternetSetOptionA(ses, INTERNET_OPTION_PER_CONNECTION_OPTION,
             &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
+    ok(ret, "InternetSetOption should've succeeded\n");
 
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    /* get & verify the process IE proxy server */
+    options[0].Value.pszValue = NULL;
+    options[1].Value.dwValue = 0;
 
-    /* get & verify the global IE proxy server */
-    list.dwOptionCount = 2;
-    list.dwOptionError = 0;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, 2 * sizeof(INTERNET_PER_CONN_OPTIONA));
-
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[1].dwOption = INTERNET_PER_CONN_FLAGS;
-
-    ret = InternetQueryOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
+    ret = InternetQueryOptionA(ses, INTERNET_OPTION_PER_CONNECTION_OPTION,
             &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    ok(!lstrcmpA(list.pOptions[0].Value.pszValue, "proxy.example"),
+    ok(ret, "InternetQueryOption should've succeeded\n");
+    ok(!lstrcmpA(options[0].Value.pszValue, "proxy.example"),
             "Retrieved proxy server should've been \"%s\", was: \"%s\"\n",
-            proxy_srv, list.pOptions[0].Value.pszValue);
-    ok(list.pOptions[1].Value.dwValue == PROXY_TYPE_PROXY,
+            proxy_srv, options[0].Value.pszValue);
+    ok(options[1].Value.dwValue == PROXY_TYPE_PROXY,
             "Retrieved flags should've been PROXY_TYPE_PROXY, was: %ld\n",
-            list.pOptions[1].Value.dwValue);
+            options[1].Value.dwValue);
 
-    HeapFree(GetProcessHeap(), 0, list.pOptions[0].Value.pszValue);
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    GlobalFree(options[0].Value.pszValue);
 
     /* test with NULL as proxy server */
     list.dwOptionCount = 1;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, sizeof(INTERNET_PER_CONN_OPTIONA));
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
-    list.pOptions[0].Value.pszValue = NULL;
+    options[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
+    options[0].Value.pszValue = NULL;
 
-    ret = InternetSetOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
-
-    ret = InternetSetOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    ret = InternetSetOptionA(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
+    ok(ret, "InternetSetOption should've succeeded\n");
 
     /* get & verify the proxy server */
-    list.dwOptionCount = 1;
-    list.dwOptionError = 0;
-    list.pOptions = HeapAlloc(GetProcessHeap(), 0, sizeof(INTERNET_PER_CONN_OPTIONA));
-    list.pOptions[0].dwOption = INTERNET_PER_CONN_PROXY_SERVER;
+    options[0].Value.dwValue = 0xdeadbeef;
 
-    ret = InternetQueryOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
-    ok(ret == TRUE, "InternetQueryOption should've succeeded\n");
-    ok(!list.pOptions[0].Value.pszValue,
+    ret = InternetQueryOptionA(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, &size);
+    ok(ret, "InternetQueryOption should've succeeded\n");
+    ok(!options[0].Value.pszValue,
             "Retrieved proxy server should've been NULL, was: \"%s\"\n",
-            list.pOptions[0].Value.pszValue);
-
-    HeapFree(GetProcessHeap(), 0, list.pOptions[0].Value.pszValue);
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+            options[0].Value.pszValue);
 
     /* restore original settings */
     list.dwOptionCount = 2;
     list.pOptions = orig_settings;
 
-    ret = InternetSetOptionA(NULL, INTERNET_OPTION_PER_CONNECTION_OPTION,
-            &list, size);
-    ok(ret == TRUE, "InternetSetOption should've succeeded\n");
+    ret = InternetSetOptionA(ses, INTERNET_OPTION_PER_CONNECTION_OPTION, &list, size);
+    ok(ret, "InternetSetOption should've succeeded\n");
 
-    HeapFree(GetProcessHeap(), 0, list.pOptions);
+    GlobalFree(orig_settings[0].Value.pszValue);
+    InternetCloseHandle(ses);
 }
 
 #define FLAG_NEEDREQ  0x1
