@@ -176,7 +176,7 @@ static void wave_in_test_deviceIn(int device, WAVEFORMATEX *pwfx, DWORD format,
        pwfx->nChannels, nSamplesPerSec, wBitsPerSample, nChannels);
 
     /* waveInGetDevCaps allows an open handle instead of a device id */
-    rc=waveInGetDevCapsW(HandleToUlong(win),&capsW,sizeof(capsW));
+    rc=waveInGetDevCapsW((UINT_PTR)win,&capsW,sizeof(capsW));
     ok(rc==MMSYSERR_NOERROR,
        "waveInGetDevCapsW(%s): MMSYSERR_NOERROR "
        "expected, got %s\n",dev_name(device),wave_in_error(rc));
@@ -192,7 +192,7 @@ static void wave_in_test_deviceIn(int device, WAVEFORMATEX *pwfx, DWORD format,
     /* Check that the position is 0 at start */
     check_position(device, win, 0, pwfx);
 
-    frag.lpData=HeapAlloc(GetProcessHeap(), 0, pwfx->nAvgBytesPerSec);
+    frag.lpData=malloc(pwfx->nAvgBytesPerSec);
     frag.dwBufferLength=pwfx->nAvgBytesPerSec;
     frag.dwBytesRecorded=0;
     frag.dwUser=0;
@@ -298,7 +298,7 @@ static void wave_in_test_deviceIn(int device, WAVEFORMATEX *pwfx, DWORD format,
             trace("Unable to play back the recorded sound\n");
     }
 
-    HeapFree(GetProcessHeap(), 0, frag.lpData);
+    free(frag.lpData);
     CloseHandle(hevent);
 }
 
@@ -379,7 +379,7 @@ static void wave_in_test_device(UINT_PTR device)
        "waveInMessage(%s): failed to get interface size: rc=%s\n",
        dev_name(device),wave_in_error(rc));
     if (rc==MMSYSERR_NOERROR) {
-        nameW = HeapAlloc(GetProcessHeap(), 0, size);
+        nameW = malloc(size);
         rc=waveInMessage((HWAVEIN)device, DRV_QUERYDEVICEINTERFACE,
                          (DWORD_PTR)nameW, size);
         ok(rc==MMSYSERR_NOERROR,"waveInMessage(%s): failed to get interface "
@@ -387,14 +387,13 @@ static void wave_in_test_device(UINT_PTR device)
         ok(lstrlenW(nameW)+1==size/sizeof(WCHAR),
            "got an incorrect size %ld\n", size);
         if (rc==MMSYSERR_NOERROR) {
-            nameA = HeapAlloc(GetProcessHeap(), 0, size/sizeof(WCHAR));
+            nameA = malloc(size/sizeof(WCHAR));
             WideCharToMultiByte(CP_ACP, 0, nameW, size/sizeof(WCHAR),
                                 nameA, size/sizeof(WCHAR), NULL, NULL);
         }
-        HeapFree(GetProcessHeap(), 0, nameW);
+        free(nameW);
     } else if (rc==MMSYSERR_NOTSUPPORTED) {
-        nameA=HeapAlloc(GetProcessHeap(), 0, sizeof("not supported"));
-        strcpy(nameA, "not supported");
+        nameA=strdup("not supported");
     }
 
     trace("  %s: \"%s\" (%s) %d.%d (%d:%d)\n",dev_name(device),capsA.szPname,
@@ -403,7 +402,7 @@ static void wave_in_test_device(UINT_PTR device)
     trace("     channels=%d formats=%05lx\n",
           capsA.wChannels,capsA.dwFormats);
 
-    HeapFree(GetProcessHeap(), 0, nameA);
+    free(nameA);
 
     for (f = 0; f < ARRAY_SIZE(win_formats); f++) {
         format.wFormatTag=WAVE_FORMAT_PCM;
