@@ -58,6 +58,9 @@ struct catadmin
     DWORD magic;
     WCHAR path[MAX_PATH];
     HANDLE find;
+    ALG_ID alg;
+    const WCHAR *providerName;
+    DWORD providerType;
 };
 
 struct catinfo
@@ -97,6 +100,29 @@ static HCATINFO create_catinfo(const WCHAR *filename)
  */
 BOOL WINAPI CryptCATAdminAcquireContext(HCATADMIN *catAdmin,
                                         const GUID *sys, DWORD dwFlags)
+{
+    TRACE("%p %s %lx\n", catAdmin, debugstr_guid(sys), dwFlags);
+    return CryptCATAdminAcquireContext2(catAdmin, sys, NULL, NULL, dwFlags);
+}
+
+/***********************************************************************
+ *             CryptCATAdminAcquireContext2 (WINTRUST.@)
+ * Get a catalog administrator context handle.
+ *
+ * PARAMS
+ *   catAdmin  [O] Pointer to the context handle.
+ *   sys       [I] Pointer to a GUID for the needed subsystem.
+ *   algorithm [I] String of hashing algorithm to use for catalog (SHA1/SHA256).
+ *   policy    [I] Pointer to policy structure for checking strong signatures.
+ *   dwFlags   [I] Reserved.
+ *
+ * RETURNS
+ *   Success: TRUE. catAdmin contains the context handle.
+ *   Failure: FALSE.
+ *
+ */
+BOOL WINAPI CryptCATAdminAcquireContext2(HCATADMIN *catAdmin, const GUID *sys, const WCHAR *algorithm,
+                                         const CERT_STRONG_SIGN_PARA *policy, DWORD dwFlags)
 {
     static const WCHAR catroot[] =
         {'\\','c','a','t','r','o','o','t',0};
@@ -110,20 +136,49 @@ BOOL WINAPI CryptCATAdminAcquireContext(HCATADMIN *catAdmin,
 
     WCHAR catroot_dir[MAX_PATH];
     struct catadmin *ca;
+    ALG_ID alg;
+    const WCHAR *providerName;
+    DWORD providerType;
 
-    TRACE("%p %s %lx\n", catAdmin, debugstr_guid(sys), dwFlags);
+    TRACE("%p %s %s %p %lx\n", catAdmin, debugstr_guid(sys), debugstr_w(algorithm), policy, dwFlags);
 
     if (!catAdmin || dwFlags)
     {
         SetLastError(ERROR_INVALID_PARAMETER);
         return FALSE;
     }
+
+    if (policy != NULL)
+        FIXME("strong policy parameter is unimplemented\n");
+
+    if (algorithm == NULL || wcscmp(algorithm, BCRYPT_SHA1_ALGORITHM) == 0)
+    {
+        alg = CALG_SHA1;
+        providerName = MS_DEF_PROV_W;
+        providerType = PROV_RSA_FULL;
+    }
+    else if (wcscmp(algorithm, BCRYPT_SHA256_ALGORITHM) == 0)
+    {
+        alg = CALG_SHA_256;
+        providerName = MS_ENH_RSA_AES_PROV_W;
+        providerType = PROV_RSA_AES;
+    }
+    else
+    {
+        SetLastError(NTE_BAD_ALGID);
+        return FALSE;
+    }
+
     if (!(ca = malloc(sizeof(*ca))))
     {
         SetLastError(ERROR_OUTOFMEMORY);
         return FALSE;
     }
 
+    ca->alg = alg;
+    ca->providerName = providerName;
+    ca->providerType = providerType;
+
     GetSystemDirectoryW(catroot_dir, MAX_PATH);
     lstrcatW(catroot_dir, catroot);
 
@@ -146,17 +201,6 @@ BOOL WINAPI CryptCATAdminAcquireContext(HCATADMIN *catAdmin,
     return TRUE;
 }
 
-/***********************************************************************
- *             CryptCATAdminAcquireContext2 (WINTRUST.@)
- */
-BOOL WINAPI CryptCATAdminAcquireContext2(HCATADMIN *catAdmin, const GUID *sys, const WCHAR *algorithm,
-                                         const CERT_STRONG_SIGN_PARA *policy, DWORD flags)
-{
-    FIXME("%p %s %s %p %lx stub\n", catAdmin, debugstr_guid(sys), debugstr_w(algorithm), policy, flags);
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-    return FALSE;
-}
-
 /***********************************************************************
  *             CryptCATAdminAddCatalog (WINTRUST.@)
  */
@@ -214,29 +258,124 @@ HCATINFO WINAPI CryptCATAdminAddCatalog(HCATADMIN catAdmin, PWSTR catalogFile,
     return ci;
 }
 
-/***********************************************************************
- *             CryptCATAdminCalcHashFromFileHandle (WINTRUST.@)
- */
-BOOL WINAPI CryptCATAdminCalcHashFromFileHandle(HANDLE hFile, DWORD* pcbHash,
-                                                BYTE* pbHash, DWORD dwFlags )
+static BOOL pe_image_hash( HANDLE file, HCRYPTHASH hash )
 {
+    UINT32 size, offset, file_size, sig_pos;
+    HANDLE mapping;
+    BYTE *view;
+    IMAGE_NT_HEADERS *nt;
     BOOL ret = FALSE;
 
-    TRACE("%p %p %p %lx\n", hFile, pcbHash, pbHash, dwFlags);
+    if ((file_size = GetFileSize( file, NULL )) == INVALID_FILE_SIZE) return FALSE;
+
+    if ((mapping = CreateFileMappingW( file, NULL, PAGE_READONLY, 0, 0, NULL )) == INVALID_HANDLE_VALUE)
+        return FALSE;
+
+    if (!(view = MapViewOfFile( mapping, FILE_MAP_READ, 0, 0, 0 )) || !(nt = ImageNtHeader( view ))) goto done;
+
+    if (nt->OptionalHeader.Magic == IMAGE_NT_OPTIONAL_HDR64_MAGIC)
+    {
+        const IMAGE_NT_HEADERS64 *nt64 = (const IMAGE_NT_HEADERS64 *)nt;
+
+        /* offset from start of file to checksum */
+        offset = (BYTE *)&nt64->OptionalHeader.CheckSum - view;
+
+        /* area between checksum and security directory entry */
+        size = FIELD_OFFSET( IMAGE_OPTIONAL_HEADER64, DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY] ) -
+               FIELD_OFFSET( IMAGE_OPTIONAL_HEADER64, Subsystem );
+
+        if (nt64->OptionalHeader.NumberOfRvaAndSizes < IMAGE_FILE_SECURITY_DIRECTORY + 1) goto done;
+        sig_pos = nt64->OptionalHeader.DataDirectory[IMAGE_FILE_SECURITY_DIRECTORY].VirtualAddress;
+    }
+    else if (nt->OptionalHeader.Magic == IMAGE_NT_OPTIONAL_HDR32_MAGIC)
+    {
+        const IMAGE_NT_HEADERS32 *nt32 = (const IMAGE_NT_HEADERS32 *)nt;
+
+        /* offset from start of file to checksum */
+        offset = (BYTE *)&nt32->OptionalHeader.CheckSum - view;
+
+        /* area between checksum and security directory entry */
+        size = FIELD_OFFSET( IMAGE_OPTIONAL_HEADER32, DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY] ) -
+               FIELD_OFFSET( IMAGE_OPTIONAL_HEADER32, Subsystem );
+
+        if (nt32->OptionalHeader.NumberOfRvaAndSizes < IMAGE_FILE_SECURITY_DIRECTORY + 1) goto done;
+        sig_pos = nt32->OptionalHeader.DataDirectory[IMAGE_FILE_SECURITY_DIRECTORY].VirtualAddress;
+    }
+    else goto done;
+
+    if (!CryptHashData( hash, view, offset, 0 )) goto done;
+    offset += sizeof(DWORD); /* skip checksum */
+    if (!CryptHashData( hash, view + offset, size, 0 )) goto done;
+
+    offset += size + sizeof(IMAGE_DATA_DIRECTORY); /* skip security entry */
+    if (offset > file_size) goto done;
+    if (sig_pos)
+    {
+        if (sig_pos < offset) goto done;
+        if (sig_pos > file_size) goto done;
+        size = sig_pos - offset; /* exclude signature */
+    }
+    else size = file_size - offset;
+
+    if (!CryptHashData( hash, view + offset, size, 0 )) goto done;
+    ret = TRUE;
+
+    if (!sig_pos && (size = file_size % 8))
+    {
+        static const BYTE pad[7];
+        ret = CryptHashData( hash, pad, 8 - size, 0 );
+    }
+
+done:
+    UnmapViewOfFile( view );
+    CloseHandle( mapping );
+    return ret;
+}
+
+static BOOL catadmin_calc_hash_from_filehandle(HCATADMIN catAdmin, HANDLE hFile, DWORD *pcbHash,
+                                               BYTE *pbHash, DWORD dwFlags)
+{
+    BOOL ret = FALSE;
+    struct catadmin *ca = catAdmin;
+    ALG_ID alg = CALG_SHA1;
+    const WCHAR *providerName = MS_DEF_PROV_W;
+    DWORD providerType = PROV_RSA_FULL;
+    DWORD hashLength;
 
     if (!hFile || !pcbHash || dwFlags)
     {
         SetLastError(ERROR_INVALID_PARAMETER);
         return FALSE;
     }
-    if (*pcbHash < 20)
+
+    if (ca)
     {
-        *pcbHash = 20;
+        alg = ca->alg;
+        providerName = ca->providerName;
+        providerType = ca->providerType;
+    }
+
+    switch (alg)
+    {
+        case CALG_SHA1:
+            hashLength = 20;
+            break;
+        case CALG_SHA_256:
+            hashLength = 32;
+            break;
+        default:
+            FIXME("unsupported algorithm %x\n", alg);
+            return FALSE;
+    }
+
+    if (*pcbHash < hashLength)
+    {
+        *pcbHash = hashLength;
         SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return TRUE;
     }
 
-    *pcbHash = 20;
+    *pcbHash = hashLength;
     if (pbHash)
     {
         HCRYPTPROV prov;
@@ -249,22 +388,26 @@ BOOL WINAPI CryptCATAdminCalcHashFromFileHandle(HANDLE hFile, DWORD* pcbHash,
             SetLastError(ERROR_OUTOFMEMORY);
             return FALSE;
         }
-        ret = CryptAcquireContextW(&prov, NULL, MS_DEF_PROV_W, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
+        ret = CryptAcquireContextW(&prov, NULL, providerName, providerType, CRYPT_VERIFYCONTEXT);
         if (!ret)
         {
             free(buffer);
             return FALSE;
         }
-        ret = CryptCreateHash(prov, CALG_SHA1, 0, 0, &hash);
+        ret = CryptCreateHash(prov, alg, 0, 0, &hash);
         if (!ret)
         {
             free(buffer);
             CryptReleaseContext(prov, 0);
             return FALSE;
         }
-        while ((ret = ReadFile(hFile, buffer, 4096, &bytes_read, NULL)) && bytes_read)
+
+        if (!(ret = pe_image_hash(hFile, hash)))
         {
-            CryptHashData(hash, buffer, bytes_read, 0);
+            while ((ret = ReadFile(hFile, buffer, 4096, &bytes_read, NULL)) && bytes_read)
+            {
+                CryptHashData(hash, buffer, bytes_read, 0);
+            }
         }
         if (ret) ret = CryptGetHashParam(hash, HP_HASHVAL, pbHash, pcbHash, 0);
 
@@ -275,6 +418,48 @@ BOOL WINAPI CryptCATAdminCalcHashFromFileHandle(HANDLE hFile, DWORD* pcbHash,
     return ret;
 }
 
+/***********************************************************************
+ *             CryptCATAdminCalcHashFromFileHandle (WINTRUST.@)
+ */
+BOOL WINAPI CryptCATAdminCalcHashFromFileHandle(HANDLE hFile, DWORD *pcbHash, BYTE *pbHash, DWORD dwFlags)
+{
+    TRACE("%p %p %p %lx\n", hFile, pcbHash, pbHash, dwFlags);
+    return catadmin_calc_hash_from_filehandle(NULL, hFile, pcbHash, pbHash, dwFlags);
+}
+
+/***********************************************************************
+ *    CryptCATAdminCalcHashFromFileHandle2 (WINTRUST.@)
+ *
+ * Calculate hash for a specific file using a catalog administrator context.
+ *
+ * PARAMS
+ *   catAdmin  [I] Catalog administrator context handle.
+ *   hFile     [I] Handle for the file to hash.
+ *   pcbHash   [I] Pointer to the length of the hash.
+ *   pbHash    [O] Pointer to the buffer that will store that hash
+ *   dwFlags   [I] Reserved.
+ *
+ * RETURNS
+ *   Success: TRUE. If pcbHash is too small, LastError will be set to ERROR_INSUFFICIENT_BUFFER.
+ *                  pbHash contains the computed hash, if supplied.
+ *   Failure: FALSE.
+ *
+ */
+BOOL WINAPI CryptCATAdminCalcHashFromFileHandle2(HCATADMIN catAdmin, HANDLE hFile,  DWORD *pcbHash,
+                                                 BYTE *pbHash, DWORD dwFlags)
+{
+    TRACE("%p %p %p %p %lx\n", catAdmin, hFile, pcbHash, pbHash, dwFlags);
+
+    if (!catAdmin || ((struct catadmin *)catAdmin)->magic != CATADMIN_MAGIC)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
+
+    return catadmin_calc_hash_from_filehandle(catAdmin, hFile, pcbHash, pbHash, dwFlags);
+}
+
+
 /***********************************************************************
  *             CryptCATAdminEnumCatalogFromHash (WINTRUST.@)
  */
