@@ -25,6 +25,12 @@
 extern "C" {
 #endif
 
+#ifdef _WINMM_
+#define WINMMAPI
+#else
+#define WINMMAPI DECLSPEC_IMPORT
+#endif
+
 typedef LPSTR		HPSTR;          /* a huge version of LPSTR */
 typedef LPCSTR		HPCSTR;         /* a huge version of LPCSTR */
 typedef UINT*           LPUINT;
@@ -54,12 +60,6 @@ typedef HWAVEOUT *LPHWAVEOUT;
 
 typedef LRESULT (CALLBACK *DRIVERPROC)(DWORD_PTR,HDRVR,UINT,LPARAM,LPARAM);
 
-#define MAXWAVEDRIVERS	10
-#define MAXMIDIDRIVERS	10
-#define MAXAUXDRIVERS	10
-#define MAXMCIDRIVERS	32
-#define MAXMIXERDRIVERS	10
-
 #define MAXPNAMELEN      32     /* max product name length (including NULL) */
 #define MAXERRORLENGTH   256    /* max error text length (including NULL) */
 #define MAX_JOYSTICKOEMVXDNAME	260
@@ -232,20 +232,16 @@ typedef struct tagDRVCONFIGINFOEX {
     DWORD			dnDevNode;
 } DRVCONFIGINFOEX, *LPDRVCONFIGINFOEX;
 
-LRESULT WINAPI DefDriverProc(DWORD_PTR,HDRVR,UINT,LPARAM,LPARAM);
+WINMMAPI LRESULT WINAPI DefDriverProc(DWORD_PTR,HDRVR,UINT,LPARAM,LPARAM);
 /* this sounds odd, but it's the way it is. OpenDriverA even disappeared
  * from latest SDK
  */
-HDRVR 	WINAPI OpenDriverA(LPCSTR szDriverName, LPCSTR szSectionName,
-			   LPARAM lParam2);
-HDRVR 	WINAPI OpenDriver(LPCWSTR szDriverName, LPCWSTR szSectionName,
-                          LPARAM lParam2);
-LRESULT WINAPI CloseDriver(HDRVR hDriver, LPARAM lParam1, LPARAM lParam2);
-LRESULT WINAPI SendDriverMessage(HDRVR hDriver, UINT message,
-				 LPARAM lParam1, LPARAM lParam2);
-HMODULE WINAPI GetDriverModuleHandle(HDRVR hDriver);
-
-DWORD	WINAPI GetDriverFlags(HDRVR hDriver);
+WINMMAPI HDRVR   WINAPI OpenDriverA(LPCSTR szDriverName, LPCSTR szSectionName, LPARAM lParam2);
+WINMMAPI HDRVR   WINAPI OpenDriver(LPCWSTR szDriverName, LPCWSTR szSectionName, LPARAM lParam2);
+WINMMAPI LRESULT WINAPI CloseDriver(HDRVR hDriver, LPARAM lParam1, LPARAM lParam2);
+WINMMAPI LRESULT WINAPI SendDriverMessage(HDRVR hDriver, UINT message, LPARAM lParam1, LPARAM lParam2);
+WINMMAPI HMODULE WINAPI GetDriverModuleHandle(HDRVR hDriver);
+WINMMAPI DWORD   WINAPI GetDriverFlags(HDRVR hDriver);
 
 typedef void (CALLBACK *LPDRVCALLBACK)(HDRVR,UINT,DWORD_PTR,DWORD_PTR,DWORD_PTR);
 
@@ -271,13 +267,13 @@ typedef void (CALLBACK *LPDRVCALLBACK)(HDRVR,UINT,DWORD_PTR,DWORD_PTR,DWORD_PTR)
 
 #define MM_CREATIVE_SBP16_WAVEOUT   104 
 
-UINT 		WINAPI 	mmsystemGetVersion(void);
-BOOL 		WINAPI	sndPlaySoundA(LPCSTR lpszSound, UINT fuSound);
-BOOL 		WINAPI	sndPlaySoundW(LPCWSTR lpszSound, UINT fuSound);
-#define 		sndPlaySound WINELIB_NAME_AW(sndPlaySound)
-BOOL 		WINAPI 	PlaySoundA(LPCSTR pszSound, HMODULE hmod, DWORD fdwSound);
-BOOL 		WINAPI 	PlaySoundW(LPCWSTR pszSound, HMODULE hmod, DWORD fdwSound);
-#define 		PlaySound WINELIB_NAME_AW(PlaySound)
+WINMMAPI UINT WINAPI mmsystemGetVersion(void);
+WINMMAPI BOOL WINAPI sndPlaySoundA(LPCSTR lpszSound, UINT fuSound);
+WINMMAPI BOOL WINAPI sndPlaySoundW(LPCWSTR lpszSound, UINT fuSound);
+#define              sndPlaySound WINELIB_NAME_AW(sndPlaySound)
+WINMMAPI BOOL WINAPI PlaySoundA(LPCSTR pszSound, HMODULE hmod, DWORD fdwSound);
+WINMMAPI BOOL WINAPI PlaySoundW(LPCWSTR pszSound, HMODULE hmod, DWORD fdwSound);
+#define              PlaySound WINELIB_NAME_AW(PlaySound)
 
 #define SND_SYNC            	0x0000  /* play synchronously (default) */
 #define SND_ASYNC           	0x0001  /* play asynchronously */
@@ -513,49 +509,49 @@ typedef struct tWAVEFORMATEX {
 #endif
 typedef const WAVEFORMATEX *LPCWAVEFORMATEX;
 
-UINT 		WINAPI 	waveOutGetNumDevs(void);
-UINT           WINAPI  waveOutGetDevCapsA(UINT_PTR,LPWAVEOUTCAPSA,UINT);
-UINT           WINAPI  waveOutGetDevCapsW(UINT_PTR,LPWAVEOUTCAPSW,UINT);
-#define 		waveOutGetDevCaps WINELIB_NAME_AW(waveOutGetDevCaps)
-UINT 		WINAPI	waveOutGetVolume(HWAVEOUT,DWORD*);
-UINT 		WINAPI 	waveOutSetVolume(HWAVEOUT,DWORD);
-UINT 		WINAPI 	waveOutGetErrorTextA(UINT,LPSTR,UINT);
-UINT 		WINAPI 	waveOutGetErrorTextW(UINT,LPWSTR,UINT);
-#define 	    	waveOutGetErrorText WINELIB_NAME_AW(waveOutGetErrorText)
-MMRESULT        WINAPI  waveOutOpen(HWAVEOUT*,UINT,LPCWAVEFORMATEX,DWORD_PTR,DWORD_PTR,DWORD);
-UINT 		WINAPI 	waveOutClose(HWAVEOUT);
-UINT 		WINAPI 	waveOutPrepareHeader(HWAVEOUT,WAVEHDR*,UINT);
-UINT 		WINAPI 	waveOutUnprepareHeader(HWAVEOUT,WAVEHDR*,UINT);
-UINT 		WINAPI 	waveOutWrite(HWAVEOUT,WAVEHDR*,UINT);
-UINT 		WINAPI 	waveOutPause(HWAVEOUT);
-UINT 		WINAPI 	waveOutRestart(HWAVEOUT);
-UINT 		WINAPI 	waveOutReset(HWAVEOUT);
-UINT 		WINAPI 	waveOutBreakLoop(HWAVEOUT);
-UINT 		WINAPI 	waveOutGetPosition(HWAVEOUT,LPMMTIME,UINT);
-UINT 		WINAPI 	waveOutGetPitch(HWAVEOUT,DWORD*);
-UINT 		WINAPI 	waveOutSetPitch(HWAVEOUT,DWORD);
-UINT 		WINAPI 	waveOutGetPlaybackRate(HWAVEOUT,DWORD*);
-UINT 		WINAPI 	waveOutSetPlaybackRate(HWAVEOUT,DWORD);
-UINT 		WINAPI 	waveOutGetID(HWAVEOUT,UINT*);
-UINT           WINAPI  waveOutMessage(HWAVEOUT,UINT,DWORD_PTR,DWORD_PTR);
-UINT 		WINAPI 	waveInGetNumDevs(void);
-UINT           WINAPI  waveInGetDevCapsA(UINT_PTR,LPWAVEINCAPSA,UINT);
-UINT           WINAPI  waveInGetDevCapsW(UINT_PTR,LPWAVEINCAPSW,UINT);
-#define 		waveInGetDevCaps WINELIB_NAME_AW(waveInGetDevCaps)
-UINT 		WINAPI 	waveInGetErrorTextA(UINT,LPSTR,UINT);
-UINT 		WINAPI 	waveInGetErrorTextW(UINT,LPWSTR,UINT);
-#define 		waveInGetErrorText WINELIB_NAME_AW(waveInGetErrorText)
-MMRESULT        WINAPI  waveInOpen(HWAVEIN*,UINT,LPCWAVEFORMATEX,DWORD_PTR,DWORD_PTR,DWORD);
-UINT 		WINAPI  waveInClose(HWAVEIN);
-UINT 		WINAPI  waveInPrepareHeader(HWAVEIN,WAVEHDR*,UINT);
-UINT 		WINAPI  waveInUnprepareHeader(HWAVEIN,WAVEHDR*,UINT);
-UINT 		WINAPI  waveInAddBuffer(HWAVEIN,WAVEHDR*,UINT);
-UINT 		WINAPI  waveInStart(HWAVEIN);
-UINT 		WINAPI  waveInStop(HWAVEIN);
-UINT 		WINAPI  waveInReset(HWAVEIN);
-UINT 		WINAPI  waveInGetPosition(HWAVEIN,LPMMTIME,UINT);
-UINT 		WINAPI  waveInGetID(HWAVEIN,UINT*);
-UINT           WINAPI  waveInMessage(HWAVEIN,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI UINT     WINAPI waveOutGetNumDevs(void);
+WINMMAPI UINT     WINAPI waveOutGetDevCapsA(UINT_PTR,LPWAVEOUTCAPSA,UINT);
+WINMMAPI UINT     WINAPI waveOutGetDevCapsW(UINT_PTR,LPWAVEOUTCAPSW,UINT);
+#define                  waveOutGetDevCaps WINELIB_NAME_AW(waveOutGetDevCaps)
+WINMMAPI UINT     WINAPI waveOutGetVolume(HWAVEOUT,DWORD*);
+WINMMAPI UINT     WINAPI waveOutSetVolume(HWAVEOUT,DWORD);
+WINMMAPI UINT     WINAPI waveOutGetErrorTextA(UINT,LPSTR,UINT);
+WINMMAPI UINT     WINAPI waveOutGetErrorTextW(UINT,LPWSTR,UINT);
+#define                  waveOutGetErrorText WINELIB_NAME_AW(waveOutGetErrorText)
+WINMMAPI MMRESULT WINAPI waveOutOpen(HWAVEOUT*,UINT,LPCWAVEFORMATEX,DWORD_PTR,DWORD_PTR,DWORD);
+WINMMAPI UINT     WINAPI waveOutClose(HWAVEOUT);
+WINMMAPI UINT     WINAPI waveOutPrepareHeader(HWAVEOUT,WAVEHDR*,UINT);
+WINMMAPI UINT     WINAPI waveOutUnprepareHeader(HWAVEOUT,WAVEHDR*,UINT);
+WINMMAPI UINT     WINAPI waveOutWrite(HWAVEOUT,WAVEHDR*,UINT);
+WINMMAPI UINT     WINAPI waveOutPause(HWAVEOUT);
+WINMMAPI UINT     WINAPI waveOutRestart(HWAVEOUT);
+WINMMAPI UINT     WINAPI waveOutReset(HWAVEOUT);
+WINMMAPI UINT     WINAPI waveOutBreakLoop(HWAVEOUT);
+WINMMAPI UINT     WINAPI waveOutGetPosition(HWAVEOUT,LPMMTIME,UINT);
+WINMMAPI UINT     WINAPI waveOutGetPitch(HWAVEOUT,DWORD*);
+WINMMAPI UINT     WINAPI waveOutSetPitch(HWAVEOUT,DWORD);
+WINMMAPI UINT     WINAPI waveOutGetPlaybackRate(HWAVEOUT,DWORD*);
+WINMMAPI UINT     WINAPI waveOutSetPlaybackRate(HWAVEOUT,DWORD);
+WINMMAPI UINT     WINAPI waveOutGetID(HWAVEOUT,UINT*);
+WINMMAPI UINT     WINAPI waveOutMessage(HWAVEOUT,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI UINT     WINAPI waveInGetNumDevs(void);
+WINMMAPI UINT     WINAPI waveInGetDevCapsA(UINT_PTR,LPWAVEINCAPSA,UINT);
+WINMMAPI UINT     WINAPI waveInGetDevCapsW(UINT_PTR,LPWAVEINCAPSW,UINT);
+#define                  waveInGetDevCaps WINELIB_NAME_AW(waveInGetDevCaps)
+WINMMAPI UINT     WINAPI waveInGetErrorTextA(UINT,LPSTR,UINT);
+WINMMAPI UINT     WINAPI waveInGetErrorTextW(UINT,LPWSTR,UINT);
+#define                  waveInGetErrorText WINELIB_NAME_AW(waveInGetErrorText)
+WINMMAPI MMRESULT WINAPI waveInOpen(HWAVEIN*,UINT,LPCWAVEFORMATEX,DWORD_PTR,DWORD_PTR,DWORD);
+WINMMAPI UINT     WINAPI waveInClose(HWAVEIN);
+WINMMAPI UINT     WINAPI waveInPrepareHeader(HWAVEIN,WAVEHDR*,UINT);
+WINMMAPI UINT     WINAPI waveInUnprepareHeader(HWAVEIN,WAVEHDR*,UINT);
+WINMMAPI UINT     WINAPI waveInAddBuffer(HWAVEIN,WAVEHDR*,UINT);
+WINMMAPI UINT     WINAPI waveInStart(HWAVEIN);
+WINMMAPI UINT     WINAPI waveInStop(HWAVEIN);
+WINMMAPI UINT     WINAPI waveInReset(HWAVEIN);
+WINMMAPI UINT     WINAPI waveInGetPosition(HWAVEIN,LPMMTIME,UINT);
+WINMMAPI UINT     WINAPI waveInGetID(HWAVEIN,UINT*);
+WINMMAPI UINT     WINAPI waveInMessage(HWAVEIN,UINT,DWORD_PTR,DWORD_PTR);
 
 #define MIDIERR_UNPREPARED    (MIDIERR_BASE + 0)
 #define MIDIERR_STILLPLAYING  (MIDIERR_BASE + 1)
@@ -777,52 +773,51 @@ typedef struct midievent_tag {
 #define	MEVT_TEMPO	((BYTE)0x01)
 #define	MEVT_VERSION	((BYTE)0x84)
 
-UINT		WINAPI	midiOutGetNumDevs(void);
-UINT           WINAPI  midiOutGetDevCapsA(UINT_PTR,LPMIDIOUTCAPSA,UINT);
-UINT           WINAPI  midiOutGetDevCapsW(UINT_PTR,LPMIDIOUTCAPSW,UINT);
-#define 		midiOutGetDevCaps WINELIB_NAME_AW(midiOutGetDevCaps)
-UINT		WINAPI	midiOutGetVolume(HMIDIOUT,DWORD*);
-UINT		WINAPI	midiOutSetVolume(HMIDIOUT,DWORD);
-UINT		WINAPI	midiOutGetErrorTextA(UINT,LPSTR,UINT);
-UINT		WINAPI	midiOutGetErrorTextW(UINT,LPWSTR,UINT);
-#define 		midiOutGetErrorText WINELIB_NAME_AW(midiOutGetErrorText)
-MMRESULT        WINAPI  midiOutOpen(LPHMIDIOUT,UINT,DWORD_PTR,DWORD_PTR,DWORD);
-UINT		WINAPI	midiOutClose(HMIDIOUT);
-UINT		WINAPI	midiOutPrepareHeader(HMIDIOUT,MIDIHDR*,UINT);
-UINT		WINAPI	midiOutUnprepareHeader(HMIDIOUT,MIDIHDR*,UINT);
-UINT		WINAPI	midiOutShortMsg(HMIDIOUT,DWORD);
-UINT		WINAPI	midiOutLongMsg(HMIDIOUT,MIDIHDR*,UINT);
-UINT		WINAPI	midiOutReset(HMIDIOUT);
-UINT		WINAPI	midiOutCachePatches(HMIDIOUT,UINT,WORD*,UINT);
-UINT		WINAPI	midiOutCacheDrumPatches(HMIDIOUT,UINT,WORD*,UINT);
-UINT		WINAPI	midiOutGetID(HMIDIOUT,UINT*);
-UINT           WINAPI  midiOutMessage(HMIDIOUT,UINT,DWORD_PTR,DWORD_PTR);
-
-UINT		WINAPI	midiInGetNumDevs(void);
-UINT           WINAPI  midiInGetDevCapsA(UINT_PTR,LPMIDIINCAPSA,UINT);
-UINT           WINAPI  midiInGetDevCapsW(UINT_PTR,LPMIDIINCAPSW,UINT);
-#define 		midiInGetDevCaps WINELIB_NAME_AW(midiInGetDevCaps)
-UINT		WINAPI	midiInGetErrorTextA(UINT,LPSTR,UINT);
-UINT		WINAPI	midiInGetErrorTextW(UINT,LPWSTR,UINT);
-#define 		midiInGetErrorText WINELIB_NAME_AW(midiInGetErrorText)
-MMRESULT        WINAPI  midiInOpen(LPHMIDIIN,UINT,DWORD_PTR,DWORD_PTR,DWORD);
-UINT		WINAPI	midiInClose(HMIDIIN);
-UINT		WINAPI	midiInPrepareHeader(HMIDIIN,MIDIHDR*,UINT);
-UINT		WINAPI	midiInUnprepareHeader(HMIDIIN,MIDIHDR*,UINT);
-UINT		WINAPI	midiInAddBuffer(HMIDIIN,MIDIHDR*,UINT);
-UINT		WINAPI	midiInStart(HMIDIIN);
-UINT		WINAPI	midiInStop(HMIDIIN);
-UINT		WINAPI	midiInReset(HMIDIIN);
-UINT		WINAPI	midiInGetID(HMIDIIN,UINT*);
-UINT           WINAPI  midiInMessage(HMIDIIN,UINT,DWORD_PTR,DWORD_PTR);
-MMRESULT	WINAPI	midiStreamClose(HMIDISTRM hms);
-MMRESULT       WINAPI  midiStreamOpen(LPHMIDISTRM,LPUINT,DWORD,DWORD_PTR,DWORD_PTR,DWORD);
-MMRESULT	WINAPI	midiStreamOut(HMIDISTRM hms, LPMIDIHDR lpMidiHdr, UINT cbMidiHdr);
-MMRESULT	WINAPI	midiStreamPause(HMIDISTRM hms);
-MMRESULT	WINAPI	midiStreamPosition(HMIDISTRM hms, LPMMTIME lpmmt, UINT cbmmt);
-MMRESULT	WINAPI	midiStreamProperty(HMIDISTRM hms, LPBYTE lpPropData, DWORD dwProperty);
-MMRESULT	WINAPI	midiStreamRestart(HMIDISTRM hms);
-MMRESULT	WINAPI	midiStreamStop(HMIDISTRM hms);
+WINMMAPI UINT     WINAPI midiOutGetNumDevs(void);
+WINMMAPI UINT     WINAPI midiOutGetDevCapsA(UINT_PTR,LPMIDIOUTCAPSA,UINT);
+WINMMAPI UINT     WINAPI midiOutGetDevCapsW(UINT_PTR,LPMIDIOUTCAPSW,UINT);
+#define                  midiOutGetDevCaps WINELIB_NAME_AW(midiOutGetDevCaps)
+WINMMAPI UINT     WINAPI midiOutGetVolume(HMIDIOUT,DWORD*);
+WINMMAPI UINT     WINAPI midiOutSetVolume(HMIDIOUT,DWORD);
+WINMMAPI UINT     WINAPI midiOutGetErrorTextA(UINT,LPSTR,UINT);
+WINMMAPI UINT     WINAPI midiOutGetErrorTextW(UINT,LPWSTR,UINT);
+#define                  midiOutGetErrorText WINELIB_NAME_AW(midiOutGetErrorText)
+WINMMAPI MMRESULT WINAPI midiOutOpen(LPHMIDIOUT,UINT,DWORD_PTR,DWORD_PTR,DWORD);
+WINMMAPI UINT     WINAPI midiOutClose(HMIDIOUT);
+WINMMAPI UINT     WINAPI midiOutPrepareHeader(HMIDIOUT,MIDIHDR*,UINT);
+WINMMAPI UINT     WINAPI midiOutUnprepareHeader(HMIDIOUT,MIDIHDR*,UINT);
+WINMMAPI UINT     WINAPI midiOutShortMsg(HMIDIOUT,DWORD);
+WINMMAPI UINT     WINAPI midiOutLongMsg(HMIDIOUT,MIDIHDR*,UINT);
+WINMMAPI UINT     WINAPI midiOutReset(HMIDIOUT);
+WINMMAPI UINT     WINAPI midiOutCachePatches(HMIDIOUT,UINT,WORD*,UINT);
+WINMMAPI UINT     WINAPI midiOutCacheDrumPatches(HMIDIOUT,UINT,WORD*,UINT);
+WINMMAPI UINT     WINAPI midiOutGetID(HMIDIOUT,UINT*);
+WINMMAPI UINT     WINAPI midiOutMessage(HMIDIOUT,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI UINT     WINAPI midiInGetNumDevs(void);
+WINMMAPI UINT     WINAPI midiInGetDevCapsA(UINT_PTR,LPMIDIINCAPSA,UINT);
+WINMMAPI UINT     WINAPI midiInGetDevCapsW(UINT_PTR,LPMIDIINCAPSW,UINT);
+#define                  midiInGetDevCaps WINELIB_NAME_AW(midiInGetDevCaps)
+WINMMAPI UINT     WINAPI midiInGetErrorTextA(UINT,LPSTR,UINT);
+WINMMAPI UINT     WINAPI midiInGetErrorTextW(UINT,LPWSTR,UINT);
+#define                  midiInGetErrorText WINELIB_NAME_AW(midiInGetErrorText)
+WINMMAPI MMRESULT WINAPI midiInOpen(LPHMIDIIN,UINT,DWORD_PTR,DWORD_PTR,DWORD);
+WINMMAPI UINT     WINAPI midiInClose(HMIDIIN);
+WINMMAPI UINT     WINAPI midiInPrepareHeader(HMIDIIN,MIDIHDR*,UINT);
+WINMMAPI UINT     WINAPI midiInUnprepareHeader(HMIDIIN,MIDIHDR*,UINT);
+WINMMAPI UINT     WINAPI midiInAddBuffer(HMIDIIN,MIDIHDR*,UINT);
+WINMMAPI UINT     WINAPI midiInStart(HMIDIIN);
+WINMMAPI UINT     WINAPI midiInStop(HMIDIIN);
+WINMMAPI UINT     WINAPI midiInReset(HMIDIIN);
+WINMMAPI UINT     WINAPI midiInGetID(HMIDIIN,UINT*);
+WINMMAPI UINT     WINAPI midiInMessage(HMIDIIN,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI MMRESULT WINAPI midiStreamClose(HMIDISTRM hms);
+WINMMAPI MMRESULT WINAPI midiStreamOpen(LPHMIDISTRM,LPUINT,DWORD,DWORD_PTR,DWORD_PTR,DWORD);
+WINMMAPI MMRESULT WINAPI midiStreamOut(HMIDISTRM hms, LPMIDIHDR lpMidiHdr, UINT cbMidiHdr);
+WINMMAPI MMRESULT WINAPI midiStreamPause(HMIDISTRM hms);
+WINMMAPI MMRESULT WINAPI midiStreamPosition(HMIDISTRM hms, LPMMTIME lpmmt, UINT cbmmt);
+WINMMAPI MMRESULT WINAPI midiStreamProperty(HMIDISTRM hms, LPBYTE lpPropData, DWORD dwProperty);
+WINMMAPI MMRESULT WINAPI midiStreamRestart(HMIDISTRM hms);
+WINMMAPI MMRESULT WINAPI midiStreamStop(HMIDISTRM hms);
 
 #define AUX_MAPPER     (-1)
 
@@ -884,13 +879,13 @@ DECL_WINELIB_TYPE_AW(LPAUXCAPS2)
 #define AUXCAPS_VOLUME          0x0001  /* supports volume control */
 #define AUXCAPS_LRVOLUME        0x0002  /* separate left-right volume control */
 
-UINT		WINAPI	auxGetNumDevs(void);
-UINT           WINAPI  auxGetDevCapsA(UINT_PTR,LPAUXCAPSA,UINT);
-UINT           WINAPI  auxGetDevCapsW(UINT_PTR,LPAUXCAPSW,UINT);
-#define 		auxGetDevCaps WINELIB_NAME_AW(auxGetDevCaps)
-UINT		WINAPI	auxSetVolume(UINT,DWORD);
-UINT		WINAPI	auxGetVolume(UINT,LPDWORD);
-UINT           WINAPI  auxOutMessage(UINT,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI UINT WINAPI auxGetNumDevs(void);
+WINMMAPI UINT WINAPI auxGetDevCapsA(UINT_PTR,LPAUXCAPSA,UINT);
+WINMMAPI UINT WINAPI auxGetDevCapsW(UINT_PTR,LPAUXCAPSW,UINT);
+#define              auxGetDevCaps WINELIB_NAME_AW(auxGetDevCaps)
+WINMMAPI UINT WINAPI auxSetVolume(UINT,DWORD);
+WINMMAPI UINT WINAPI auxGetVolume(UINT,LPDWORD);
+WINMMAPI UINT WINAPI auxOutMessage(UINT,UINT,DWORD_PTR,DWORD_PTR);
 
 #define TIMERR_NOERROR        (0)                  /* no error */
 #define TIMERR_NOCANDO        (TIMERR_BASE+1)      /* request not completed */
@@ -911,13 +906,13 @@ typedef struct timecaps_tag {
 } TIMECAPS, *LPTIMECAPS;
 
 
-MMRESULT	WINAPI	timeGetSystemTime(LPMMTIME,UINT);
-DWORD		WINAPI	timeGetTime(void);	/* same for win32/win16 */
-MMRESULT       WINAPI  timeSetEvent(UINT,UINT,LPTIMECALLBACK,DWORD_PTR,UINT);
-MMRESULT	WINAPI	timeKillEvent(UINT);
-MMRESULT	WINAPI	timeGetDevCaps(LPTIMECAPS,UINT);
-MMRESULT	WINAPI	timeBeginPeriod(UINT);
-MMRESULT	WINAPI	timeEndPeriod(UINT);
+WINMMAPI MMRESULT WINAPI timeGetSystemTime(LPMMTIME,UINT);
+WINMMAPI DWORD    WINAPI timeGetTime(void);
+WINMMAPI MMRESULT WINAPI timeSetEvent(UINT,UINT,LPTIMECALLBACK,DWORD_PTR,UINT);
+WINMMAPI MMRESULT WINAPI timeKillEvent(UINT);
+WINMMAPI MMRESULT WINAPI timeGetDevCaps(LPTIMECAPS,UINT);
+WINMMAPI MMRESULT WINAPI timeBeginPeriod(UINT);
+WINMMAPI MMRESULT WINAPI timeEndPeriod(UINT);
 
 #define JOYERR_NOERROR        (0)                  /* no error */
 #define JOYERR_PARMS          (JOYERR_BASE+5)      /* bad parameters */
@@ -1123,16 +1118,16 @@ typedef struct joyinfoex_tag {
 } JOYINFOEX,*LPJOYINFOEX;
 
 
-MMRESULT       WINAPI  joyGetDevCapsA(UINT_PTR,LPJOYCAPSA,UINT);
-MMRESULT       WINAPI  joyGetDevCapsW(UINT_PTR,LPJOYCAPSW,UINT);
-#define joyGetDevCaps WINELIB_NAME_AW(joyGetDevCaps)
-UINT		WINAPI	joyGetNumDevs(void);
-MMRESULT	WINAPI	joyGetPos(UINT,LPJOYINFO);
-MMRESULT	WINAPI	joyGetPosEx(UINT,LPJOYINFOEX);
-MMRESULT	WINAPI	joyGetThreshold(UINT,UINT*);
-MMRESULT	WINAPI	joyReleaseCapture(UINT);
-MMRESULT	WINAPI	joySetCapture(HWND,UINT,UINT,BOOL);
-MMRESULT	WINAPI	joySetThreshold(UINT,UINT);
+WINMMAPI MMRESULT WINAPI joyGetDevCapsA(UINT_PTR,LPJOYCAPSA,UINT);
+WINMMAPI MMRESULT WINAPI joyGetDevCapsW(UINT_PTR,LPJOYCAPSW,UINT);
+#define                  joyGetDevCaps WINELIB_NAME_AW(joyGetDevCaps)
+WINMMAPI UINT     WINAPI joyGetNumDevs(void);
+WINMMAPI MMRESULT WINAPI joyGetPos(UINT,LPJOYINFO);
+WINMMAPI MMRESULT WINAPI joyGetPosEx(UINT,LPJOYINFOEX);
+WINMMAPI MMRESULT WINAPI joyGetThreshold(UINT,UINT*);
+WINMMAPI MMRESULT WINAPI joyReleaseCapture(UINT);
+WINMMAPI MMRESULT WINAPI joySetCapture(HWND,UINT,UINT,BOOL);
+WINMMAPI MMRESULT WINAPI joySetThreshold(UINT,UINT);
 
 #define	MIXERR_BASE		1024
 #define	MIXERR_INVALLINE	(MIXERR_BASE + 0)
@@ -1512,24 +1507,24 @@ typedef struct {
 #define MIXER_SETCONTROLDETAILSF_CUSTOM         __MSABI_LONG(0x00000001)
 #define MIXER_SETCONTROLDETAILSF_QUERYMASK      __MSABI_LONG(0x0000000F)
 
-UINT		WINAPI	mixerGetNumDevs(void);
-UINT           WINAPI  mixerOpen(LPHMIXER,UINT,DWORD_PTR,DWORD_PTR,DWORD);
-UINT		WINAPI	mixerClose(HMIXER);
-DWORD           WINAPI  mixerMessage(HMIXER,UINT,DWORD_PTR,DWORD_PTR);
-UINT           WINAPI  mixerGetDevCapsA(UINT_PTR,LPMIXERCAPSA,UINT);
-UINT           WINAPI  mixerGetDevCapsW(UINT_PTR,LPMIXERCAPSW,UINT);
-#define 		mixerGetDevCaps WINELIB_NAME_AW(mixerGetDevCaps)
-UINT		WINAPI	mixerGetLineInfoA(HMIXEROBJ,LPMIXERLINEA,DWORD);
-UINT		WINAPI	mixerGetLineInfoW(HMIXEROBJ,LPMIXERLINEW,DWORD);
-#define 		mixerGetLineInfo WINELIB_NAME_AW(mixerGetLineInfo)
-UINT		WINAPI	mixerGetID(HMIXEROBJ,LPUINT,DWORD);
-UINT		WINAPI	mixerGetLineControlsA(HMIXEROBJ,LPMIXERLINECONTROLSA,DWORD);
-UINT		WINAPI	mixerGetLineControlsW(HMIXEROBJ,LPMIXERLINECONTROLSW,DWORD);
-#define 		mixerGetLineControls WINELIB_NAME_AW(mixerGetLineControls)
-UINT		WINAPI	mixerGetControlDetailsA(HMIXEROBJ,LPMIXERCONTROLDETAILS,DWORD);
-UINT		WINAPI	mixerGetControlDetailsW(HMIXEROBJ,LPMIXERCONTROLDETAILS,DWORD);
-#define 		mixerGetControlDetails WINELIB_NAME_AW(mixerGetControlDetails)
-UINT		WINAPI	mixerSetControlDetails(HMIXEROBJ,LPMIXERCONTROLDETAILS,DWORD);
+WINMMAPI UINT  WINAPI mixerGetNumDevs(void);
+WINMMAPI UINT  WINAPI mixerOpen(LPHMIXER,UINT,DWORD_PTR,DWORD_PTR,DWORD);
+WINMMAPI UINT  WINAPI mixerClose(HMIXER);
+WINMMAPI DWORD WINAPI mixerMessage(HMIXER,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI UINT  WINAPI mixerGetDevCapsA(UINT_PTR,LPMIXERCAPSA,UINT);
+WINMMAPI UINT  WINAPI mixerGetDevCapsW(UINT_PTR,LPMIXERCAPSW,UINT);
+#define               mixerGetDevCaps WINELIB_NAME_AW(mixerGetDevCaps)
+WINMMAPI UINT  WINAPI mixerGetLineInfoA(HMIXEROBJ,LPMIXERLINEA,DWORD);
+WINMMAPI UINT  WINAPI mixerGetLineInfoW(HMIXEROBJ,LPMIXERLINEW,DWORD);
+#define               mixerGetLineInfo WINELIB_NAME_AW(mixerGetLineInfo)
+WINMMAPI UINT  WINAPI mixerGetID(HMIXEROBJ,LPUINT,DWORD);
+WINMMAPI UINT  WINAPI mixerGetLineControlsA(HMIXEROBJ,LPMIXERLINECONTROLSA,DWORD);
+WINMMAPI UINT  WINAPI mixerGetLineControlsW(HMIXEROBJ,LPMIXERLINECONTROLSW,DWORD);
+#define               mixerGetLineControls WINELIB_NAME_AW(mixerGetLineControls)
+WINMMAPI UINT  WINAPI mixerGetControlDetailsA(HMIXEROBJ,LPMIXERCONTROLDETAILS,DWORD);
+WINMMAPI UINT  WINAPI mixerGetControlDetailsW(HMIXEROBJ,LPMIXERCONTROLDETAILS,DWORD);
+#define               mixerGetControlDetails WINELIB_NAME_AW(mixerGetControlDetails)
+WINMMAPI UINT  WINAPI mixerSetControlDetails(HMIXEROBJ,LPMIXERCONTROLDETAILS,DWORD);
 
 #define MMIOERR_BASE            256
 #define MMIOERR_FILENOTFOUND    (MMIOERR_BASE + 1)  /* file not found */
@@ -1651,58 +1646,53 @@ typedef struct _MMCKINFO
                 ( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |    \
                 ( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )
 
-LPMMIOPROC 	WINAPI 	mmioInstallIOProcA(FOURCC,LPMMIOPROC,DWORD);
-LPMMIOPROC 	WINAPI 	mmioInstallIOProcW(FOURCC,LPMMIOPROC,DWORD);
-#define      		mmioInstallIOProc WINELIB_NAME_AW(mmioInstallIOProc)
-
-FOURCC 		WINAPI	mmioStringToFOURCCA(LPCSTR,UINT);
-FOURCC 		WINAPI	mmioStringToFOURCCW(LPCWSTR,UINT);
-#define 		mmioStringToFOURCC WINELIB_NAME_AW(mmioStringToFOURCC)
-HMMIO		WINAPI	mmioOpenA(LPSTR,MMIOINFO*,DWORD);
-HMMIO		WINAPI	mmioOpenW(LPWSTR,MMIOINFO*,DWORD);
-#define			mmioOpen WINELIB_NAME_AW(mmioOpen)
-
-MMRESULT	WINAPI	mmioRenameA(LPCSTR szFileName, LPCSTR szNewFileName,
-				    const MMIOINFO * lpmmioinfo, DWORD dwRenameFlags);
-MMRESULT	WINAPI	mmioRenameW(LPCWSTR szFileName, LPCWSTR szNewFileName,
-				    const MMIOINFO * lpmmioinfo, DWORD dwRenameFlags);
-#define 		mmioRename WINELIB_NAME_AW(mmioRename)
-
-MMRESULT 	WINAPI	mmioClose(HMMIO,UINT);
-LONG 		WINAPI	mmioRead(HMMIO,HPSTR,LONG);
-LONG 		WINAPI	mmioWrite(HMMIO,HPCSTR,LONG);
-LONG 		WINAPI	mmioSeek(HMMIO,LONG,INT);
-MMRESULT 	WINAPI	mmioGetInfo(HMMIO,MMIOINFO*,UINT);
-MMRESULT 	WINAPI	mmioSetInfo(HMMIO,const MMIOINFO*,UINT);
-MMRESULT	WINAPI	mmioSetBuffer(HMMIO,LPSTR,LONG,UINT);
-MMRESULT	WINAPI	mmioFlush(HMMIO,UINT);
-MMRESULT	WINAPI	mmioAdvance(HMMIO,MMIOINFO*,UINT);
-LRESULT		WINAPI	mmioSendMessage(HMMIO,UINT,LPARAM,LPARAM);
-MMRESULT	WINAPI	mmioDescend(HMMIO,MMCKINFO*,const MMCKINFO*,UINT);
-MMRESULT	WINAPI	mmioAscend(HMMIO,MMCKINFO*,UINT);
-MMRESULT	WINAPI	mmioCreateChunk(HMMIO,MMCKINFO*,UINT);
+WINMMAPI LPMMIOPROC  WINAPI mmioInstallIOProcA(FOURCC,LPMMIOPROC,DWORD);
+WINMMAPI LPMMIOPROC  WINAPI mmioInstallIOProcW(FOURCC,LPMMIOPROC,DWORD);
+#define                     mmioInstallIOProc WINELIB_NAME_AW(mmioInstallIOProc)
+WINMMAPI FOURCC      WINAPI mmioStringToFOURCCA(LPCSTR,UINT);
+WINMMAPI FOURCC      WINAPI mmioStringToFOURCCW(LPCWSTR,UINT);
+#define                     mmioStringToFOURCC WINELIB_NAME_AW(mmioStringToFOURCC)
+WINMMAPI HMMIO       WINAPI mmioOpenA(LPSTR,MMIOINFO*,DWORD);
+WINMMAPI HMMIO       WINAPI mmioOpenW(LPWSTR,MMIOINFO*,DWORD);
+#define                     mmioOpen WINELIB_NAME_AW(mmioOpen)
+WINMMAPI MMRESULT    WINAPI mmioRenameA(LPCSTR szFileName, LPCSTR szNewFileName, const MMIOINFO * lpmmioinfo, DWORD dwRenameFlags);
+WINMMAPI MMRESULT    WINAPI mmioRenameW(LPCWSTR szFileName, LPCWSTR szNewFileName, const MMIOINFO * lpmmioinfo, DWORD dwRenameFlags);
+#define                     mmioRename WINELIB_NAME_AW(mmioRename)
+WINMMAPI MMRESULT    WINAPI mmioClose(HMMIO,UINT);
+WINMMAPI LONG        WINAPI mmioRead(HMMIO,HPSTR,LONG);
+WINMMAPI LONG        WINAPI mmioWrite(HMMIO,HPCSTR,LONG);
+WINMMAPI LONG        WINAPI mmioSeek(HMMIO,LONG,INT);
+WINMMAPI MMRESULT    WINAPI mmioGetInfo(HMMIO,MMIOINFO*,UINT);
+WINMMAPI MMRESULT    WINAPI mmioSetInfo(HMMIO,const MMIOINFO*,UINT);
+WINMMAPI MMRESULT    WINAPI mmioSetBuffer(HMMIO,LPSTR,LONG,UINT);
+WINMMAPI MMRESULT    WINAPI mmioFlush(HMMIO,UINT);
+WINMMAPI MMRESULT    WINAPI mmioAdvance(HMMIO,MMIOINFO*,UINT);
+WINMMAPI LRESULT     WINAPI mmioSendMessage(HMMIO,UINT,LPARAM,LPARAM);
+WINMMAPI MMRESULT    WINAPI mmioDescend(HMMIO,MMCKINFO*,const MMCKINFO*,UINT);
+WINMMAPI MMRESULT    WINAPI mmioAscend(HMMIO,MMCKINFO*,UINT);
+WINMMAPI MMRESULT    WINAPI mmioCreateChunk(HMMIO,MMCKINFO*,UINT);
 
 typedef UINT (CALLBACK *YIELDPROC)(MCIDEVICEID,DWORD);
 
-BOOL            WINAPI  mciExecute(LPCSTR);
-DWORD           WINAPI  mciSendCommandA(MCIDEVICEID,UINT,DWORD_PTR,DWORD_PTR);
-DWORD           WINAPI  mciSendCommandW(MCIDEVICEID,UINT,DWORD_PTR,DWORD_PTR);
-#define 		mciSendCommand WINELIB_NAME_AW(mciSendCommand)
-DWORD		WINAPI	mciSendStringA(LPCSTR,LPSTR,UINT,HWND);
-DWORD		WINAPI	mciSendStringW(LPCWSTR,LPWSTR,UINT,HWND);
-#define 		mciSendString WINELIB_NAME_AW(mciSendString)
-MCIDEVICEID	WINAPI	mciGetDeviceIDA(LPCSTR);
-MCIDEVICEID	WINAPI	mciGetDeviceIDW(LPCWSTR);
-#define 		mciGetDeviceID WINELIB_NAME_AW(mciGetDeviceID)
-MCIDEVICEID     WINAPI  mciGetDeviceIDFromElementIDA(DWORD,LPCSTR);
-MCIDEVICEID     WINAPI  mciGetDeviceIDFromElementIDW(DWORD,LPCWSTR);
-#define                 mciGetDeviceIDFromElementID WINELIB_NAME_AW(mciGetDeviceIDFromElementID)
-BOOL            WINAPI  mciGetErrorStringA(MCIERROR,LPSTR,UINT);
-BOOL            WINAPI  mciGetErrorStringW(MCIERROR,LPWSTR,UINT);
-#define 		mciGetErrorString WINELIB_NAME_AW(mciGetErrorString)
-BOOL            WINAPI  mciSetYieldProc(MCIDEVICEID,YIELDPROC,DWORD);
-HTASK           WINAPI  mciGetCreatorTask(MCIDEVICEID);
-YIELDPROC       WINAPI  mciGetYieldProc(MCIDEVICEID,DWORD*);
+WINMMAPI BOOL        WINAPI mciExecute(LPCSTR);
+WINMMAPI DWORD       WINAPI mciSendCommandA(MCIDEVICEID,UINT,DWORD_PTR,DWORD_PTR);
+WINMMAPI DWORD       WINAPI mciSendCommandW(MCIDEVICEID,UINT,DWORD_PTR,DWORD_PTR);
+#define                     mciSendCommand WINELIB_NAME_AW(mciSendCommand)
+WINMMAPI DWORD       WINAPI mciSendStringA(LPCSTR,LPSTR,UINT,HWND);
+WINMMAPI DWORD       WINAPI mciSendStringW(LPCWSTR,LPWSTR,UINT,HWND);
+#define                     mciSendString WINELIB_NAME_AW(mciSendString)
+WINMMAPI MCIDEVICEID WINAPI mciGetDeviceIDA(LPCSTR);
+WINMMAPI MCIDEVICEID WINAPI mciGetDeviceIDW(LPCWSTR);
+#define                     mciGetDeviceID WINELIB_NAME_AW(mciGetDeviceID)
+WINMMAPI MCIDEVICEID WINAPI mciGetDeviceIDFromElementIDA(DWORD,LPCSTR);
+WINMMAPI MCIDEVICEID WINAPI mciGetDeviceIDFromElementIDW(DWORD,LPCWSTR);
+#define                     mciGetDeviceIDFromElementID WINELIB_NAME_AW(mciGetDeviceIDFromElementID)
+WINMMAPI BOOL        WINAPI mciGetErrorStringA(MCIERROR,LPSTR,UINT);
+WINMMAPI BOOL        WINAPI mciGetErrorStringW(MCIERROR,LPWSTR,UINT);
+#define                     mciGetErrorString WINELIB_NAME_AW(mciGetErrorString)
+WINMMAPI BOOL        WINAPI mciSetYieldProc(MCIDEVICEID,YIELDPROC,DWORD);
+WINMMAPI HTASK       WINAPI mciGetCreatorTask(MCIDEVICEID);
+WINMMAPI YIELDPROC   WINAPI mciGetYieldProc(MCIDEVICEID,DWORD*);
 
 #define MCIERR_INVALID_DEVICE_ID        (MCIERR_BASE + 1)
 #define MCIERR_UNRECOGNIZED_KEYWORD     (MCIERR_BASE + 3)
