@@ -11,6 +11,7 @@
 #include <corecrt_wstdio.h>
 
 /* file._flag flags */
+#ifndef _UCRT
 #define _IOREAD          0x0001
 #define _IOWRT           0x0002
 #define _IOMYBUF         0x0008
@@ -18,6 +19,7 @@
 #define _IOERR           0x0020
 #define _IOSTRG          0x0040
 #define _IORW            0x0080
+#endif
 
 #define STDIN_FILENO  0
 #define STDOUT_FILENO 1
@@ -185,8 +187,8 @@ static inline int __cdecl _vsnprintf_s(char *buffer, size_t size, size_t count,
     return ret < 0 ? -1 : ret;
 }
 
-static inline int WINAPIV _snprintf_s(char *buffer, size_t size, size_t count, const char *format, ...) __WINE_CRT_PRINTF_ATTR(4, 5);
-static inline int WINAPIV _snprintf_s(char *buffer, size_t size, size_t count, const char *format, ...)
+static inline int __cdecl _snprintf_s(char *buffer, size_t size, size_t count, const char *format, ...) __WINE_CRT_PRINTF_ATTR(4, 5);
+static inline int __cdecl _snprintf_s(char *buffer, size_t size, size_t count, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -205,8 +207,8 @@ static inline int __cdecl _vscprintf(const char *format, va_list args)
     return ret < 0 ? -1 : ret;
 }
 
-static inline int WINAPIV _scprintf(const char *format, ...) __WINE_CRT_PRINTF_ATTR(1, 2);
-static inline int WINAPIV _scprintf(const char *format, ...)
+static inline int __cdecl _scprintf(const char *format, ...) __WINE_CRT_PRINTF_ATTR(1, 2);
+static inline int __cdecl _scprintf(const char *format, ...)
 {
     int ret;
     va_list args;
@@ -233,8 +235,8 @@ static inline int __cdecl vsprintf_s(char *buffer, size_t size, const char *form
     return ret < 0 ? -1 : ret;
 }
 
-static inline int WINAPIV sprintf_s(char *buffer, size_t size, const char *format, ...) __WINE_CRT_PRINTF_ATTR(3, 4);
-static inline int WINAPIV sprintf_s(char *buffer, size_t size, const char *format, ...)
+static inline int __cdecl sprintf_s(char *buffer, size_t size, const char *format, ...) __WINE_CRT_PRINTF_ATTR(3, 4);
+static inline int __cdecl sprintf_s(char *buffer, size_t size, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -258,8 +260,8 @@ static inline int __cdecl vfprintf(FILE *file, const char *format, va_list args)
     return __stdio_common_vfprintf(_CRT_INTERNAL_LOCAL_PRINTF_OPTIONS, file, format, NULL, args);
 }
 
-static inline int WINAPIV fprintf(FILE *file, const char *format, ...) __WINE_CRT_PRINTF_ATTR(2, 3);
-static inline int WINAPIV fprintf(FILE *file, const char *format, ...)
+static inline int __cdecl fprintf(FILE *file, const char *format, ...) __WINE_CRT_PRINTF_ATTR(2, 3);
+static inline int __cdecl fprintf(FILE *file, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -276,8 +278,8 @@ static inline int __cdecl vfprintf_s(FILE *file, const char *format, va_list arg
     return __stdio_common_vfprintf_s(_CRT_INTERNAL_LOCAL_PRINTF_OPTIONS, file, format, NULL, args);
 }
 
-static inline int WINAPIV fprintf_s(FILE *file, const char *format, ...) __WINE_CRT_PRINTF_ATTR(2, 3);
-static inline int WINAPIV fprintf_s(FILE *file, const char *format, ...)
+static inline int __cdecl fprintf_s(FILE *file, const char *format, ...) __WINE_CRT_PRINTF_ATTR(2, 3);
+static inline int __cdecl fprintf_s(FILE *file, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -294,8 +296,8 @@ static inline int vprintf(const char *format, va_list args)
     return __stdio_common_vfprintf(_CRT_INTERNAL_LOCAL_PRINTF_OPTIONS, stdout, format, NULL, args);
 }
 
-static inline int WINAPIV printf(const char *format, ...) __WINE_CRT_PRINTF_ATTR(1, 2);
-static inline int WINAPIV printf(const char *format, ...)
+static inline int __cdecl printf(const char *format, ...) __WINE_CRT_PRINTF_ATTR(1, 2);
+static inline int __cdecl printf(const char *format, ...)
 {
     int ret;
     va_list args;
@@ -312,8 +314,8 @@ static inline int vprintf_s(const char *format, va_list args)
     return __stdio_common_vfprintf_s(_CRT_INTERNAL_LOCAL_PRINTF_OPTIONS, stdout, format, NULL, args);
 }
 
-static inline int WINAPIV printf_s(const char *format, ...) __WINE_CRT_PRINTF_ATTR(1, 2);
-static inline int WINAPIV printf_s(const char *format, ...)
+static inline int __cdecl printf_s(const char *format, ...) __WINE_CRT_PRINTF_ATTR(1, 2);
+static inline int __cdecl printf_s(const char *format, ...)
 {
     int ret;
     va_list args;
@@ -324,8 +326,21 @@ static inline int WINAPIV printf_s(const char *format, ...)
     return ret;
 }
 
-static inline int WINAPIV sscanf(const char *buffer, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
-static inline int WINAPIV sscanf(const char *buffer, const char *format, ...)
+static inline int __cdecl _sprintf_l(char *buffer, const char *format, _locale_t locale, ...) __WINE_CRT_PRINTF_ATTR(2, 4);
+static inline int __cdecl _sprintf_l(char *buffer, const char *format, _locale_t locale, ...)
+{
+    int ret;
+    va_list args;
+
+    va_start(args, locale);
+    ret = __stdio_common_vsprintf(_CRT_INTERNAL_LOCAL_PRINTF_OPTIONS | _CRT_INTERNAL_PRINTF_LEGACY_VSPRINTF_NULL_TERMINATION,
+                                  buffer, -1, format, locale, args);
+    va_end(args);
+    return ret < 0 ? -1 : ret;
+}
+
+static inline int __cdecl sscanf(const char *buffer, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
+static inline int __cdecl sscanf(const char *buffer, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -336,8 +351,8 @@ static inline int WINAPIV sscanf(const char *buffer, const char *format, ...)
     return ret;
 }
 
-static inline int WINAPIV sscanf_s(const char *buffer, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
-static inline int WINAPIV sscanf_s(const char *buffer, const char *format, ...)
+static inline int __cdecl sscanf_s(const char *buffer, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
+static inline int __cdecl sscanf_s(const char *buffer, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -348,8 +363,8 @@ static inline int WINAPIV sscanf_s(const char *buffer, const char *format, ...)
     return ret;
 }
 
-static inline int WINAPIV _snscanf_l(const char *buffer, size_t size, const char *format, _locale_t locale, ...) __WINE_CRT_SCANF_ATTR(3, 5);
-static inline int WINAPIV _snscanf_l(const char *buffer, size_t size, const char *format, _locale_t locale, ...)
+static inline int __cdecl _snscanf_l(const char *buffer, size_t size, const char *format, _locale_t locale, ...) __WINE_CRT_SCANF_ATTR(3, 5);
+static inline int __cdecl _snscanf_l(const char *buffer, size_t size, const char *format, _locale_t locale, ...)
 {
     int ret;
     va_list args;
@@ -360,8 +375,8 @@ static inline int WINAPIV _snscanf_l(const char *buffer, size_t size, const char
     return ret;
 }
 
-static inline int WINAPIV _sscanf_l(const char *buffer, const char *format, _locale_t locale, ...) __WINE_CRT_SCANF_ATTR(2, 4);
-static inline int WINAPIV _sscanf_l(const char *buffer, const char *format, _locale_t locale, ...)
+static inline int __cdecl _sscanf_l(const char *buffer, const char *format, _locale_t locale, ...) __WINE_CRT_SCANF_ATTR(2, 4);
+static inline int __cdecl _sscanf_l(const char *buffer, const char *format, _locale_t locale, ...)
 {
     int ret;
     va_list args;
@@ -372,8 +387,8 @@ static inline int WINAPIV _sscanf_l(const char *buffer, const char *format, _loc
     return ret;
 }
 
-static inline int WINAPIV fscanf(FILE *file, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
-static inline int WINAPIV fscanf(FILE *file, const char *format, ...)
+static inline int __cdecl fscanf(FILE *file, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
+static inline int __cdecl fscanf(FILE *file, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -384,8 +399,8 @@ static inline int WINAPIV fscanf(FILE *file, const char *format, ...)
     return ret;
 }
 
-static inline int WINAPIV fscanf_s(FILE *file, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
-static inline int WINAPIV fscanf_s(FILE *file, const char *format, ...)
+static inline int __cdecl fscanf_s(FILE *file, const char *format, ...) __WINE_CRT_SCANF_ATTR(2, 3);
+static inline int __cdecl fscanf_s(FILE *file, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -396,8 +411,8 @@ static inline int WINAPIV fscanf_s(FILE *file, const char *format, ...)
     return ret;
 }
 
-static inline int WINAPIV scanf(const char *format, ...) __WINE_CRT_SCANF_ATTR(1, 2);
-static inline int WINAPIV scanf(const char *format, ...)
+static inline int __cdecl scanf(const char *format, ...) __WINE_CRT_SCANF_ATTR(1, 2);
+static inline int __cdecl scanf(const char *format, ...)
 {
     int ret;
     va_list args;
@@ -408,8 +423,8 @@ static inline int WINAPIV scanf(const char *format, ...)
     return ret;
 }
 
-static inline int WINAPIV scanf_s(const char *format, ...) __WINE_CRT_SCANF_ATTR(1, 2);
-static inline int WINAPIV scanf_s(const char *format, ...)
+static inline int __cdecl scanf_s(const char *format, ...) __WINE_CRT_SCANF_ATTR(1, 2);
+static inline int __cdecl scanf_s(const char *format, ...)
 {
     int ret;
     va_list args;
@@ -422,16 +437,16 @@ static inline int WINAPIV scanf_s(const char *format, ...)
 
 #else /* _UCRT && !_NO_CRT_STDIO_INLINE */
 
-_ACRTIMP int WINAPIV _scprintf(const char *,...) __WINE_CRT_PRINTF_ATTR(1, 2);
-_ACRTIMP int WINAPIV _snprintf_s(char*,size_t,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(4, 5);
+_ACRTIMP int __cdecl _scprintf(const char *,...) __WINE_CRT_PRINTF_ATTR(1, 2);
+_ACRTIMP int __cdecl _snprintf_s(char*,size_t,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(4, 5);
 _ACRTIMP int __cdecl _vscprintf(const char*,va_list) __WINE_CRT_PRINTF_ATTR(1, 0);
 _ACRTIMP int __cdecl _vsnprintf_s(char*,size_t,size_t,const char*,va_list) __WINE_CRT_PRINTF_ATTR(4, 0);
 _ACRTIMP int __cdecl _vsprintf_p_l(char*,size_t,const char*,_locale_t,va_list) __WINE_CRT_PRINTF_ATTR(3, 0);
-_ACRTIMP int WINAPIV fprintf(FILE*,const char*,...) __WINE_CRT_PRINTF_ATTR(2, 3);
-_ACRTIMP int WINAPIV fprintf_s(FILE*,const char*,...) __WINE_CRT_PRINTF_ATTR(2, 3);
-_ACRTIMP int WINAPIV printf(const char*,...) __WINE_CRT_PRINTF_ATTR(1, 2);
-_ACRTIMP int WINAPIV printf_s(const char*,...) __WINE_CRT_PRINTF_ATTR(1, 2);
-_ACRTIMP int WINAPIV sprintf_s(char*,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(3, 4);
+_ACRTIMP int __cdecl fprintf(FILE*,const char*,...) __WINE_CRT_PRINTF_ATTR(2, 3);
+_ACRTIMP int __cdecl fprintf_s(FILE*,const char*,...) __WINE_CRT_PRINTF_ATTR(2, 3);
+_ACRTIMP int __cdecl printf(const char*,...) __WINE_CRT_PRINTF_ATTR(1, 2);
+_ACRTIMP int __cdecl printf_s(const char*,...) __WINE_CRT_PRINTF_ATTR(1, 2);
+_ACRTIMP int __cdecl sprintf_s(char*,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(3, 4);
 _ACRTIMP int __cdecl vfprintf(FILE*,const char*,va_list) __WINE_CRT_PRINTF_ATTR(2, 0);
 _ACRTIMP int __cdecl vfprintf_s(FILE*,const char*,va_list) __WINE_CRT_PRINTF_ATTR(2, 0);
 _ACRTIMP int __cdecl vprintf(const char*,va_list) __WINE_CRT_PRINTF_ATTR(1, 0);
@@ -444,14 +459,14 @@ static inline int vsnprintf(char *buffer, size_t size, const char *format, va_li
 static inline int vsnprintf(char *buffer, size_t size, const char *format, va_list args)
 { return _vsnprintf(buffer,size,format,args); }
 
-_ACRTIMP int WINAPIV _snscanf_l(const char*,size_t,const char*,_locale_t,...) __WINE_CRT_SCANF_ATTR(3, 5);
-_ACRTIMP int WINAPIV _sscanf_l(const char *,const char*,_locale_t,...) __WINE_CRT_SCANF_ATTR(2, 4);
-_ACRTIMP int WINAPIV fscanf(FILE*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
-_ACRTIMP int WINAPIV fscanf_s(FILE*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
-_ACRTIMP int WINAPIV scanf(const char*,...) __WINE_CRT_SCANF_ATTR(1, 2);
-_ACRTIMP int WINAPIV scanf_s(const char*,...) __WINE_CRT_SCANF_ATTR(1, 2);
-_ACRTIMP int WINAPIV sscanf(const char*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
-_ACRTIMP int WINAPIV sscanf_s(const char*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
+_ACRTIMP int __cdecl _snscanf_l(const char*,size_t,const char*,_locale_t,...) __WINE_CRT_SCANF_ATTR(3, 5);
+_ACRTIMP int __cdecl _sscanf_l(const char *,const char*,_locale_t,...) __WINE_CRT_SCANF_ATTR(2, 4);
+_ACRTIMP int __cdecl fscanf(FILE*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
+_ACRTIMP int __cdecl fscanf_s(FILE*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
+_ACRTIMP int __cdecl scanf(const char*,...) __WINE_CRT_SCANF_ATTR(1, 2);
+_ACRTIMP int __cdecl scanf_s(const char*,...) __WINE_CRT_SCANF_ATTR(1, 2);
+_ACRTIMP int __cdecl sscanf(const char*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
+_ACRTIMP int __cdecl sscanf_s(const char*,const char*,...) __WINE_CRT_SCANF_ATTR(2, 3);
 
 #endif /* _UCRT && !_NO_CRT_STDIO_INLINE */
 
@@ -476,8 +491,8 @@ static inline int unlink(const char* path) { return _unlink(path); }
 
 #if !defined(_NO_CRT_STDIO_INLINE)
 
-static inline int WINAPIV snprintf(char *buffer, size_t size, const char *format, ...) __WINE_CRT_PRINTF_ATTR(3, 4);
-static inline int WINAPIV snprintf(char *buffer, size_t size, const char *format, ...)
+static inline int __cdecl snprintf(char *buffer, size_t size, const char *format, ...) __WINE_CRT_PRINTF_ATTR(3, 4);
+static inline int __cdecl snprintf(char *buffer, size_t size, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -488,8 +503,8 @@ static inline int WINAPIV snprintf(char *buffer, size_t size, const char *format
     return ret;
 }
 
-static inline int WINAPIV _snprintf(char *buffer, size_t size, const char *format, ...) __WINE_CRT_PRINTF_ATTR(3, 4);
-static inline int WINAPIV _snprintf(char *buffer, size_t size, const char *format, ...)
+static inline int __cdecl _snprintf(char *buffer, size_t size, const char *format, ...) __WINE_CRT_PRINTF_ATTR(3, 4);
+static inline int __cdecl _snprintf(char *buffer, size_t size, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -500,8 +515,8 @@ static inline int WINAPIV _snprintf(char *buffer, size_t size, const char *forma
     return ret;
 }
 
-static inline int WINAPIV sprintf(char *buffer, const char *format, ...) __WINE_CRT_PRINTF_ATTR(2, 3);
-static inline int WINAPIV sprintf(char *buffer, const char *format, ...)
+static inline int __cdecl sprintf(char *buffer, const char *format, ...) __WINE_CRT_PRINTF_ATTR(2, 3);
+static inline int __cdecl sprintf(char *buffer, const char *format, ...)
 {
     int ret;
     va_list args;
@@ -514,9 +529,10 @@ static inline int WINAPIV sprintf(char *buffer, const char *format, ...)
 
 #else /* !_NO_CRT_STDIO_INLINE */
 
-_ACRTIMP int WINAPIV snprintf(char*,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(3, 4);
-_ACRTIMP int WINAPIV _snprintf(char*,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(3, 4);
-_ACRTIMP int WINAPIV sprintf(char*,const char*,...) __WINE_CRT_PRINTF_ATTR(2, 3);
+_ACRTIMP int __cdecl snprintf(char*,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(3, 4);
+_ACRTIMP int __cdecl _snprintf(char*,size_t,const char*,...) __WINE_CRT_PRINTF_ATTR(3, 4);
+_ACRTIMP int __cdecl sprintf(char*,const char*,...) __WINE_CRT_PRINTF_ATTR(2, 3);
+_ACRTIMP int __cdecl _sprintf_l(char*,const char*,_locale_t,...) __WINE_CRT_PRINTF_ATTR(2, 4);
 
 #endif /* !_NO_CRT_STDIO_INLINE */
 
