@@ -84,6 +84,12 @@ typedef struct _RTL_BALANCED_NODE
 
 #define RTL_BALANCED_NODE_RESERVED_PARENT_MASK 3
 
+typedef struct _RTL_RB_TREE
+{
+    RTL_BALANCED_NODE *root;
+    RTL_BALANCED_NODE *min;
+} RTL_RB_TREE, *PRTL_RB_TREE;
+
 #define RTL_CONSTANT_STRING(s) { sizeof(s) - sizeof(s[0]), sizeof(s), (void*)s }
 
 #endif /* _NTDEF_ */
