@@ -760,6 +760,27 @@ typedef struct _DNS_CACHE_ENTRY
     ULONG Flags;
 } DNS_CACHE_ENTRY, *PDNS_CACHE_ENTRY;
 
+typedef void WINAPI DNS_SERVICE_BROWSE_CALLBACK(DWORD, void *, PDNS_RECORD);
+typedef DNS_SERVICE_BROWSE_CALLBACK *PDNS_SERVICE_BROWSE_CALLBACK;
+
+typedef struct _DNS_SERVICE_BROWSE_REQUEST
+{
+    ULONG Version;
+    ULONG InterfaceIndex;
+    const WCHAR *QueryName;
+    union
+    {
+        PDNS_SERVICE_BROWSE_CALLBACK pBrowseCallback;
+        DNS_QUERY_COMPLETION_ROUTINE *pBrowseCallbackV2;
+    };
+    void *pQueryContext;
+} DNS_SERVICE_BROWSE_REQUEST, *PDNS_SERVICE_BROWSE_REQUEST;
+
+typedef struct _DNS_SERVICE_CANCEL
+{
+    void *reserved;
+} DNS_SERVICE_CANCEL, *PDNS_SERVICE_CANCEL;
+
 DNS_STATUS WINAPI DnsAcquireContextHandle_A(DWORD,PVOID,PHANDLE);
 DNS_STATUS WINAPI DnsAcquireContextHandle_W(DWORD,PVOID,PHANDLE);
 #define DnsAcquireContextHandle WINELIB_NAME_AW(DnsAcquireContextHandle_)
@@ -791,6 +812,7 @@ DNS_STATUS WINAPI DnsReplaceRecordSetA(PDNS_RECORDA,DWORD,HANDLE,PVOID,PVOID);
 DNS_STATUS WINAPI DnsReplaceRecordSetW(PDNS_RECORDW,DWORD,HANDLE,PVOID,PVOID);
 DNS_STATUS WINAPI DnsReplaceRecordSetUTF8(PDNS_RECORDA,DWORD,HANDLE,PVOID,PVOID);
 #define DnsReplaceRecordSet WINELIB_NAME_AW(DnsReplaceRecordSet)
+DNS_STATUS WINAPI DnsServiceBrowse(PDNS_SERVICE_BROWSE_REQUEST, PDNS_SERVICE_CANCEL);
 DNS_STATUS WINAPI DnsValidateName_A(PCSTR,DNS_NAME_FORMAT);
 DNS_STATUS WINAPI DnsValidateName_W(PCWSTR, DNS_NAME_FORMAT);
 DNS_STATUS WINAPI DnsValidateName_UTF8(PCSTR,DNS_NAME_FORMAT);
