@@ -192,7 +192,8 @@ typedef int INTERNET_SCHEME, *LPINTERNET_SCHEME;
 #define WINHTTP_DECOMPRESSION_FLAG_ALL ( WINHTTP_DECOMPRESSION_FLAG_GZIP | WINHTTP_DECOMPRESSION_FLAG_DEFLATE )
 
 #define WINHTTP_PROTOCOL_FLAG_HTTP2 0x1
-#define WINHTTP_PROTOCOL_MASK WINHTTP_PROTOCOL_FLAG_HTTP2
+#define WINHTTP_PROTOCOL_FLAG_HTTP3 0x2
+#define WINHTTP_PROTOCOL_MASK (WINHTTP_PROTOCOL_FLAG_HTTP2 | WINHTTP_PROTOCOL_FLAG_HTTP3)
 
 #define WINHTTP_AUTOLOGON_SECURITY_LEVEL_MEDIUM   0
 #define WINHTTP_AUTOLOGON_SECURITY_LEVEL_LOW      1
