@@ -23,10 +23,12 @@
 extern "C" {
 #endif /* defined(__cplusplus) */
 
+#ifndef WINADVAPI
 #ifdef _ADVAPI32_
 #define WINADVAPI
 #else
-#define WINADVAPI DECLSPEC_HIDDEN
+#define WINADVAPI DECLSPEC_IMPORT
+#endif
 #endif
 
 /* Service database names */
@@ -197,8 +199,6 @@ typedef struct _SERVICE_NOTIFY_2W {
     WCHAR *pszServiceNames;
 } SERVICE_NOTIFY_2W, SERVICE_NOTIFYW;
 
-DWORD WINAPI NotifyServiceStatusChangeW(SC_HANDLE,DWORD,SERVICE_NOTIFYW*);
-
 typedef enum _SC_STATUS_TYPE {
   SC_STATUS_PROCESS_INFO      = 0
 } SC_STATUS_TYPE;
@@ -415,6 +415,7 @@ WINADVAPI BOOL        WINAPI GetServiceKeyNameW(SC_HANDLE,LPCWSTR,LPWSTR,LPDWORD
 #define                      GetServiceKeyName WINELIB_NAME_AW(GetServiceKeyName)
 WINADVAPI SC_LOCK     WINAPI LockServiceDatabase(SC_HANDLE);
 WINADVAPI BOOL        WINAPI NotifyBootConfigStatus(BOOL);
+WINADVAPI DWORD       WINAPI NotifyServiceStatusChangeW(SC_HANDLE,DWORD,SERVICE_NOTIFYW*);
 WINADVAPI SC_HANDLE   WINAPI OpenSCManagerA(LPCSTR,LPCSTR,DWORD);
 WINADVAPI SC_HANDLE   WINAPI OpenSCManagerW(LPCWSTR,LPCWSTR,DWORD);
 #define                      OpenSCManager WINELIB_NAME_AW(OpenSCManager)
