@@ -354,23 +354,6 @@ typedef struct XACT_NOTIFICATION_CUE
 } XACT_NOTIFICATION_CUE, *LPXACT_NOTIFICATION_CUE;
 typedef const XACT_NOTIFICATION_CUE *LPCXACT_NOTIFICATION_CUE;
 
-typedef struct XACT_WAVE_PROPERTIES
-{
-    char friendlyName[WAVEBANK_ENTRYNAME_LENGTH];
-    WAVEBANKMINIWAVEFORMAT format;
-    DWORD durationInSamples;
-    WAVEBANKSAMPLEREGION loopRegion;
-    BOOL streaming;
-} XACT_WAVE_PROPERTIES, *LPXACT_WAVE_PROPERTIES;
-typedef const XACT_WAVE_PROPERTIES *LPCXACT_WAVE_PROPERTIES;
-
-typedef struct XACT_WAVE_INSTANCE_PROPERTIES
-{
-    XACT_WAVE_PROPERTIES properties;
-    BOOL backgroundMusic;
-} XACT_WAVE_INSTANCE_PROPERTIES, *LPXACT_WAVE_INSTANCE_PROPERTIES;
-typedef const XACT_WAVE_INSTANCE_PROPERTIES *LPCXACT_WAVE_INSTANCE_PROPERTIES;
-
 typedef struct XACT_NOTIFICATION_MARKER
 {
     XACTINDEX cueIndex;
@@ -434,7 +417,7 @@ typedef struct XACT_NOTIFICATION
         XACT_NOTIFICATION_VARIABLE variable;
         XACT_NOTIFICATION_GUI gui;
         XACT_NOTIFICATION_WAVE wave;
-    } DUMMYUNIONNAME;
+    };
 } XACT_NOTIFICATION, *LPXACT_NOTIFICATION;
 typedef const XACT_NOTIFICATION *LPCXACT_NOTIFICATION;
 
@@ -442,6 +425,25 @@ typedef void (__stdcall *XACT_NOTIFICATION_CALLBACK)(
     const XACT_NOTIFICATION *pNotification
 );
 
+#include <poppack.h>
+
+typedef struct XACT_WAVE_PROPERTIES
+{
+    char friendlyName[WAVEBANK_ENTRYNAME_LENGTH];
+    WAVEBANKMINIWAVEFORMAT format;
+    DWORD durationInSamples;
+    WAVEBANKSAMPLEREGION loopRegion;
+    BOOL streaming;
+} XACT_WAVE_PROPERTIES, *LPXACT_WAVE_PROPERTIES;
+typedef const XACT_WAVE_PROPERTIES *LPCXACT_WAVE_PROPERTIES;
+
+typedef struct XACT_WAVE_INSTANCE_PROPERTIES
+{
+    XACT_WAVE_PROPERTIES properties;
+    BOOL backgroundMusic;
+} XACT_WAVE_INSTANCE_PROPERTIES, *LPXACT_WAVE_INSTANCE_PROPERTIES;
+typedef const XACT_WAVE_INSTANCE_PROPERTIES *LPCXACT_WAVE_INSTANCE_PROPERTIES;
+
 typedef struct XACT_RUNTIME_PARAMETERS
 {
     DWORD lookAheadTime;
@@ -457,8 +459,6 @@ typedef struct XACT_RUNTIME_PARAMETERS
 } XACT_RUNTIME_PARAMETERS, *LPXACT_RUNTIME_PARAMETERS;
 typedef const XACT_RUNTIME_PARAMETERS *LPCXACT_RUNTIME_PARAMETERS;
 
-#include <poppack.h>
-
 #define XACT_FLAG_CUE_STOP_RELEASE   XACT_FLAG_STOP_RELEASE
 #define XACT_FLAG_CUE_STOP_IMMEDIATE XACT_FLAG_STOP_IMMEDIATE
 
@@ -487,8 +487,10 @@ DECLARE_INTERFACE(IXACT3Cue)
     STDMETHOD(GetVariable)(THIS_ XACTVARIABLEINDEX nIndex, XACTVARIABLEVALUE *nValue) PURE;
     STDMETHOD(Pause)(THIS_ BOOL fPause) PURE;
     STDMETHOD(GetProperties)(THIS_ LPXACT_CUE_INSTANCE_PROPERTIES *ppProperties) PURE;
+#if XACT3_VER >= 0x0305
     STDMETHOD(SetOutputVoices)(THIS_ const XAUDIO2_VOICE_SENDS *pSendList) PURE;
     STDMETHOD(SetOutputVoiceMatrix)(THIS_ IXAudio2Voice *pDestinationVoice, UINT32 SourceChannels, UINT32 DestinationChannels, const float *pLevelMatrix) PURE;
+#endif
 };
 #undef INTERFACE
 
@@ -662,7 +664,7 @@ DECLARE_INTERFACE(IXACT3WaveBank)
 DECLARE_INTERFACE_(IXACT3Engine,IUnknown)
 {
     /*** IUnknown methods ***/
-    STDMETHOD_(HRESULT,QueryInterface)(THIS_ REFIID riid, void** ppvObject) PURE;
+    STDMETHOD(QueryInterface)(THIS_ REFIID riid, void** ppvObject) PURE;
     STDMETHOD_(ULONG,AddRef)(THIS) PURE;
     STDMETHOD_(ULONG,Release)(THIS) PURE;
     /*** IXACT3Engine methods ***/
