@@ -22,8 +22,8 @@
  */
 
 #define WIN32_LEAN_AND_MEAN
-#define NONAMELESSUNION
 
+#include <locale.h>
 #include <windows.h>
 #include <commctrl.h>
 #include <objbase.h>
@@ -74,8 +74,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_APPCFG);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_APPCFG);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = AppDlgProc;
     psp[pg].pszTitle = load_string (IDS_TAB_APPLICATIONS);
     psp[pg].lParam = 0;
@@ -88,8 +88,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_DLLCFG);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_DLLCFG);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = LibrariesDlgProc;
     psp[pg].pszTitle = load_string (IDS_TAB_DLLS);
     psp[pg].lParam = 0;
@@ -102,8 +102,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_GRAPHCFG);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_GRAPHCFG);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = GraphDlgProc;
     psp[pg].pszTitle =  load_string (IDS_TAB_GRAPHICS);
     psp[pg].lParam = 0;
@@ -112,8 +112,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_DESKTOP_INTEGRATION);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_DESKTOP_INTEGRATION);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = ThemeDlgProc;
     psp[pg].pszTitle =  load_string (IDS_TAB_DESKTOP_INTEGRATION);
     psp[pg].lParam = 0;
@@ -122,8 +122,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_DRIVECFG);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_DRIVECFG);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = DriveDlgProc;
     psp[pg].pszTitle =  load_string (IDS_TAB_DRIVES);
     psp[pg].lParam = 0;
@@ -132,8 +132,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_AUDIOCFG);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_AUDIOCFG);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = AudioDlgProc;
     psp[pg].pszTitle =  load_string (IDS_TAB_AUDIO);
     psp[pg].lParam = 0;
@@ -146,8 +146,8 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psp[pg].dwSize = sizeof (PROPSHEETPAGEW);
     psp[pg].dwFlags = PSP_USETITLE;
     psp[pg].hInstance = hInstance;
-    psp[pg].u.pszTemplate = MAKEINTRESOURCEW (IDD_ABOUTCFG);
-    psp[pg].u2.pszIcon = NULL;
+    psp[pg].pszTemplate = MAKEINTRESOURCEW (IDD_ABOUTCFG);
+    psp[pg].pszIcon = NULL;
     psp[pg].pfnDlgProc = AboutDlgProc;
     psp[pg].pszTitle =  load_string (IDS_TAB_ABOUT);
     psp[pg].lParam = 0;
@@ -160,12 +160,12 @@ doPropertySheet (HINSTANCE hInstance, HWND hOwner)
     psh.dwFlags = PSH_PROPSHEETPAGE | PSH_USEICONID | PSH_USECALLBACK;
     psh.hwndParent = hOwner;
     psh.hInstance = hInstance;
-    psh.u.pszIcon = MAKEINTRESOURCEW (IDI_WINECFG);
+    psh.pszIcon = MAKEINTRESOURCEW (IDI_WINECFG);
     psh.pszCaption =  load_string (IDS_WINECFG_TITLE);
     psh.nPages = NUM_PROPERTY_PAGES;
-    psh.u3.ppsp = psp;
+    psh.ppsp = psp;
     psh.pfnCallback = PropSheetCallback;
-    psh.u2.nStartPage = 0;
+    psh.nStartPage = 0;
 
     /*
      * Display the modal property sheet
@@ -187,6 +187,9 @@ ProcessCmdLine(LPWSTR lpCmdLine)
     {
         return -1;
     }
+
+    setlocale(LC_ALL, "en-US");
+
     if (lpCmdLine[1] == 'V' || lpCmdLine[1] == 'v')
     {
         if (wcslen(lpCmdLine) > 4)
@@ -195,22 +198,19 @@ ProcessCmdLine(LPWSTR lpCmdLine)
         print_current_winver();
         return 0;
     }
+    if (lpCmdLine[1] != '?')
+        MESSAGE("Unsupported option '%ls'\n", lpCmdLine);
 
-    if (lpCmdLine[1] == '?')
-    {
-        printf("Usage: winecfg [options]\n\n");
-        printf("Options:\n");
-        printf("  [no option] Launch the graphical version of this program.\n");
-        printf("  /v          Display the current global Windows version.\n");
-        printf("  /v version  Set global Windows version to 'version'.\n");
-        printf("  /?          Display this information and exit.\n\n");
-        printf("Valid versions for 'version':\n\n");
-        print_windows_versions();
+    MESSAGE("Usage: winecfg [options]\n\n");
+    MESSAGE("Options:\n");
+    MESSAGE("  [no option] Launch the graphical version of this program.\n");
+    MESSAGE("  /v          Display the current global Windows version.\n");
+    MESSAGE("  /v version  Set global Windows version to 'version'.\n");
+    MESSAGE("  /?          Display this information and exit.\n\n");
+    MESSAGE("Valid versions for 'version':\n\n");
+    print_windows_versions();
 
-        return 0;
-    }
-
-    return -1;
+    return 0;
 }
 
 /*****************************************************************************
