@@ -340,8 +340,17 @@ void break_add_break_from_lineno(const char *filename, int lineno, BOOL swbp)
     if (bkln.addr.Offset)
         break_add_break(&bkln.addr, TRUE, swbp);
     else
-        dbg_printf("Unknown line number\n"
-                   "(either out of file, or no code at given line number)\n");
+    {
+        /* winedbg's lexer can't tell in 'break foo : 3' whether foo shall be interpreted
+         * as a debuggee symbol or as a debuggee source file.
+         * Try now symbol since source file failed.
+         */
+        if (filename)
+            break_add_break_from_id(filename, lineno, swbp);
+        else
+            dbg_printf("Unknown line number\n"
+                       "(either out of file, or no code at given line number)\n");
+    }
 }
 
 /***********************************************************************
@@ -386,10 +395,10 @@ void break_check_delayed_bp(void)
  *
  * Add a watchpoint.
  */
-static void break_add_watch(const struct dbg_lvalue* lvalue, BOOL is_write)
+void break_add_watch(const struct dbg_lvalue* lvalue, BOOL is_write)
 {
     int         num;
-    DWORD64     l = 4;
+    DWORD       l = ADDRSIZE;
 
     if (!lvalue->in_debuggee)
     {
@@ -400,21 +409,30 @@ static void break_add_watch(const struct dbg_lvalue* lvalue, BOOL is_write)
                       &lvalue->addr);
     if (num == -1) return;
 
+    /* FIXME: this validation part should be moved to CPU backends (likely in a new method) */
     if (lvalue->type.id != dbg_itype_none)
     {
-        if (types_get_info(&lvalue->type, TI_GET_LENGTH, &l))
+        DWORD64 l64;
+        if (types_get_info(&lvalue->type, TI_GET_LENGTH, &l64))
         {
-            switch (l)
+            /* Only allow size of a power of 2, smaller than CPU's pointer size.
+             * (Validation is done by CPU backend with insert_Xpoint method)
+             */
+            if (!(l64 & (l64 - 1)) && l64 <= l)
+                l = l64;
+            else
+                dbg_printf("Unsupported length (%I64x) for watch-points, defaulting to %lu\n", l64, l);
+            /* most CPUs request watch address to be aligned on length */
+            if (lvalue->addr.Offset & (l - 1))
             {
-            case 4: case 2: case 1: break;
-            default:
-                dbg_printf("Unsupported length (%I64x) for watch-points, defaulting to 4\n", l);
-                break;
+                dbg_printf("Watchpoint on unaligned address is not supported\n");
+                dbg_curr_process->bp[num].refcount = 0;
+                return;
             }
         }
-        else dbg_printf("Cannot get watch size, defaulting to 4\n");
+        else dbg_printf("Cannot get watch size, defaulting to %lu\n", l);
     }
-    dbg_curr_process->bp[num].w.len = (DWORD)l - 1;
+    dbg_curr_process->bp[num].w.len = l - 1;
 
     if (!get_watched_value(num, &dbg_curr_process->bp[num].w.oldval))
     {
@@ -427,43 +445,6 @@ static void break_add_watch(const struct dbg_lvalue* lvalue, BOOL is_write)
     dbg_printf("\n");
 }
 
-/******************************************************************
- *		break_add_watch_from_lvalue
- *
- * Adds a watch point from an address (stored in a lvalue)
- */
-void break_add_watch_from_lvalue(const struct dbg_lvalue* lvalue,BOOL is_write)
-{
-    struct dbg_lvalue   lval;
-
-    types_extract_as_address(lvalue, &lval.addr);
-    lval.type.id = dbg_itype_none;
-
-    break_add_watch(&lval, is_write);
-}
-
-/***********************************************************************
- *           break_add_watch_from_id
- *
- * Add a watchpoint from a symbol name
- */
-void	break_add_watch_from_id(const char *name, BOOL is_write)
-{
-    struct dbg_lvalue    lvalue;
-
-    switch (symbol_get_lvalue(name, -1, &lvalue, TRUE))
-    {
-    case sglv_found:
-        break_add_watch(&lvalue, is_write);
-        break;
-    case sglv_unknown:
-        dbg_printf("Unable to add watchpoint\n");
-        break;
-    case sglv_aborted: /* user aborted symbol lookup */
-        break;
-    }
-}
-
 /***********************************************************************
  *           break_delete_xpoint
  *
