@@ -25,19 +25,20 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include "wine/debug.h"
+
+#define xprintf( fmt, ... ) output( logfile, fmt, ## __VA_ARGS__ )
+extern void output( HANDLE file, const char *fmt, ... ) __WINE_PRINTF_ATTR(2,3);
+extern char *strmake( const char *fmt, ... ) __WINE_PRINTF_ATTR(1,2);
 
 void fatal (const char* msg);
 void warning (const char* msg);
-void WINAPIV xprintf (const char *fmt, ...);
-char *vstrmake (size_t *lenp, va_list ap);
-char * WINAPIV strmake (size_t *lenp, ...);
+char *vstrmake (va_list ap);
 int goodtagchar (char c);
 const char *findbadtagchar (const char *tag);
 
 int send_file (const char *url, const char *name);
 
-extern HANDLE logfile;
-
 /* GUI definitions */
 
 #include <windows.h>
