@@ -647,7 +647,7 @@ static inline BOOL is_whitespace(WCHAR c)
     return c == ' ' || c == '\t';
 }
 
-static WCHAR *skip_whitespace(WCHAR *p)
+static const WCHAR *skip_whitespace(const WCHAR *p)
 {
     for (; *p && is_whitespace(*p); p++);
     return p;
@@ -662,92 +662,88 @@ static inline BOOL is_digit(WCHAR c)
    that lacks the escaped-quote logic of build_argv(), because
    literal double quotes are illegal in any of its arguments.
    Example: 'XCOPY "c:\DIR A" "c:DIR B\"' is OK. */
-static int find_end_of_word(const WCHAR *word, WCHAR **end)
+static int get_arg(const WCHAR **cmdline, WCHAR **arg)
 {
-    BOOL in_quotes = FALSE;
-    const WCHAR *ptr = word;
-    for (;;) {
-        for (; *ptr != '\0' && *ptr != '"' &&
-                 (in_quotes || !is_whitespace(*ptr)); ptr++);
+    const WCHAR *ptr = *cmdline;
+    int len;
+
+    if (*ptr == '/') ptr++;
+    while (*ptr && !is_whitespace(*ptr) && *ptr != '/') {
         if (*ptr == '"') {
-            in_quotes = !in_quotes;
-            ptr++;
+            while (*ptr && *ptr != '"') ptr++;
+            /* Odd number of double quotes is illegal for XCOPY */
+            if (!*ptr) return RC_INITERROR;
         }
-        /* Odd number of double quotes is illegal for XCOPY */
-        if (in_quotes && *ptr == '\0')
-            return RC_INITERROR;
-        if (*ptr == '\0' || (!in_quotes && is_whitespace(*ptr)))
-            break;
+        ptr++;
     }
-    *end = (WCHAR*)ptr;
+
+    len = ptr - *cmdline;
+    *arg = malloc((len + 1) * sizeof(WCHAR));
+    if (!*arg)
+        return RC_INITERROR;
+    memcpy(*arg, *cmdline, len * sizeof(WCHAR));
+    (*arg)[len] = 0;
+
+    *cmdline = skip_whitespace(ptr);
     return RC_OK;
 }
 
 /* Remove all double quotes from a word */
-static void strip_quotes(WCHAR *word, WCHAR **end)
+static void strip_quotes(WCHAR *word)
 {
-    WCHAR *rp, *wp;
-    for (rp = word, wp = word; *rp != '\0'; rp++) {
-        if (*rp == '"')
+    WCHAR *wp;
+    for (wp = word; *word != '\0'; word++) {
+        if (*word == '"')
             continue;
-        if (wp < rp)
-            *wp = *rp;
+        if (wp < word)
+            *wp = *word;
         wp++;
     }
     *wp = '\0';
-    *end = wp;
 }
 
 static int XCOPY_ParseCommandLine(WCHAR *suppliedsource,
                                   WCHAR *supplieddestination, DWORD *pflags)
 {
     DWORD flags = *pflags;
-    WCHAR *cmdline, *word, *end, *next;
-    int rc = RC_INITERROR;
-
-    cmdline = _wcsdup(GetCommandLineW());
-    if (cmdline == NULL)
-        return rc;
+    const WCHAR *cmdline;
+    WCHAR *word;
+    int rc;
 
+    cmdline = GetCommandLineW();
     /* Skip first arg, which is the program name */
-    if ((rc = find_end_of_word(cmdline, &word)) != RC_OK)
-        goto out;
-    word = skip_whitespace(word);
+    if ((rc = get_arg(&cmdline, &word)) != RC_OK)
+        exit(rc);
+    free(word);
 
-    while (*word)
+    while (*cmdline)
     {
-        WCHAR first;
-        if ((rc = find_end_of_word(word, &end)) != RC_OK)
-            goto out;
-
-        next = skip_whitespace(end);
-        first = word[0];
-        *end = '\0';
-        strip_quotes(word, &end);
+        if ((rc = get_arg(&cmdline, &word)) != RC_OK)
+            exit(rc);
         WINE_TRACE("Processing Arg: '%s'\n", wine_dbgstr_w(word));
 
         /* First non-switch parameter is source, second is destination */
-        if (first != '/') {
+        if (word[0] != '/') {
+            strip_quotes(word);
             if (suppliedsource[0] == 0x00) {
                 lstrcpyW(suppliedsource, word);
             } else if (supplieddestination[0] == 0x00) {
                 lstrcpyW(supplieddestination, word);
             } else {
                 XCOPY_wprintf(XCOPY_LoadMessage(STRING_INVPARMS));
-                goto out;
+                exit(RC_INITERROR);
             }
         } else {
             /* Process all the switch options
                  Note: Windows docs say /P prompts when dest is created
                        but tests show it is done for each src file
                        regardless of the destination                   */
-            int skip=0;
-            WCHAR *rest;
+            WCHAR *p = word + 1, *rest;
 
-            while (word[0]) {
-                rest = NULL;
+            while (*p) {
+                rest = p + 1;
 
-                switch (toupper(word[1])) {
+                switch (toupper(*p)) {
                 case 'I': flags |= OPT_ASSUMEDIR;     break;
                 case 'S': flags |= OPT_RECURSIVE;     break;
                 case 'Q': flags |= OPT_QUIET;         break;
@@ -769,15 +765,13 @@ static int XCOPY_ParseCommandLine(WCHAR *suppliedsource,
 
                 /* E can be /E or /EXCLUDE */
                 case 'E': if (CompareStringW(LOCALE_USER_DEFAULT, NORM_IGNORECASE | SORT_STRINGSORT,
-                                             &word[1], 8, L"EXCLUDE:", -1) == CSTR_EQUAL) {
-                            if (XCOPY_ProcessExcludeList(&word[9])) {
+                                             p, 8, L"EXCLUDE:", -1) == CSTR_EQUAL) {
+                            if (XCOPY_ProcessExcludeList(&p[8])) {
                               XCOPY_FailMessage(ERROR_INVALID_PARAMETER);
-                              goto out;
+                              exit(RC_INITERROR);
                             } else {
                               flags |= OPT_EXCLUDELIST;
-
-                              /* Do not support concatenated switches onto exclude lists yet */
-                              rest = end;
+                              rest = p + wcslen(p);
                             }
                           } else {
                               flags |= OPT_EMPTYDIR | OPT_RECURSIVE;
@@ -785,9 +779,9 @@ static int XCOPY_ParseCommandLine(WCHAR *suppliedsource,
                           break;
 
                 /* D can be /D or /D: */
-                case 'D': if (word[2]==':' && is_digit(word[3])) {
+                case 'D': if (p[1]==':' && is_digit(p[2])) {
                               SYSTEMTIME st;
-                              WCHAR     *pos = &word[3];
+                              WCHAR     *pos = &p[2];
                               BOOL       isError = FALSE;
                               memset(&st, 0x00, sizeof(st));
 
@@ -831,45 +825,33 @@ static int XCOPY_ParseCommandLine(WCHAR *suppliedsource,
                                              wine_dbgstr_w(datestring), wine_dbgstr_w(timestring));
                               } else {
                                   XCOPY_FailMessage(ERROR_INVALID_PARAMETER);
-                                  goto out;
+                                  exit(RC_INITERROR);
                               }
                           } else {
                               flags |= OPT_DATENEWER;
                           }
                           break;
 
-                case '-': if (toupper(word[2])=='Y') {
+                case '-': if (toupper(p[1])=='Y') {
                               flags &= ~OPT_NOPROMPT;
-                              rest = &word[3];  /* Skip over 3 characters */
+                              rest = &p[2];  /* Skip over 2 characters */
                           }
                           break;
                 case '?': XCOPY_wprintf(XCOPY_LoadMessage(STRING_HELP));
-                          rc = RC_HELP;
-                          goto out;
+                          exit(RC_OK);
                 case 'V':
                     WINE_FIXME("ignoring /V\n");
                     break;
                 default:
-                    WINE_TRACE("Unhandled parameter '%s'\n", wine_dbgstr_w(word));
-                    XCOPY_wprintf(XCOPY_LoadMessage(STRING_INVPARM), word);
-                    goto out;
+                    WINE_TRACE("Unhandled parameter '%s'\n", wine_dbgstr_w(p));
+                    XCOPY_wprintf(XCOPY_LoadMessage(STRING_INVPARM), p);
+                    exit(RC_INITERROR);
                 }
 
-                /* Unless overridden above, skip over the '/' and the first character */
-                if (rest == NULL) rest = &word[2];
-
-                /* By now, rest should point either to the null after the
-                   switch, or the beginning of the next switch if there
-                   was no whitespace between them                          */
-                if (!skip && *rest && *rest != '/') {
-                    WINE_FIXME("Unexpected characters found and ignored '%s'\n", wine_dbgstr_w(rest));
-                    skip=1;
-                } else {
-                    word = rest;
-                }
+                p = rest;
             }
         }
-        word = next;
+        free(word);
     }
 
     /* Default the destination if not supplied */
@@ -877,11 +859,7 @@ static int XCOPY_ParseCommandLine(WCHAR *suppliedsource,
         lstrcpyW(supplieddestination, L".");
 
     *pflags = flags;
-    rc = RC_OK;
-
- out:
-    free(cmdline);
-    return rc;
+    return RC_OK;
 }
 
 
@@ -1098,12 +1076,8 @@ int __cdecl wmain (int argc, WCHAR *argvW[])
      * Parse the command line
      */
     if ((rc = XCOPY_ParseCommandLine(suppliedsource, supplieddestination,
-                                     &flags)) != RC_OK) {
-        if (rc == RC_HELP)
-            return RC_OK;
-        else
-            return rc;
-    }
+                                     &flags)) != RC_OK)
+        return rc;
 
     /* Trace out the supplied information */
     WINE_TRACE("Supplied parameters:\n");
