@@ -694,14 +694,16 @@ static void propagate_console_signal( struct console *console,
         set_error( STATUS_INVALID_PARAMETER );
         return;
     }
-    /* FIXME: should support the other events (like CTRL_BREAK) */
-    if (sig != CTRL_C_EVENT)
+    switch (sig)
     {
+    case CTRL_C_EVENT:     csi.signal = SIGINT; break;
+    case CTRL_BREAK_EVENT: csi.signal = SIGQUIT; break;
+    default:
+        /* FIXME: should support the other events */
         set_error( STATUS_NOT_IMPLEMENTED );
         return;
     }
     csi.console = console;
-    csi.signal  = SIGINT;
     csi.group   = group_id;
 
     enum_processes(propagate_console_signal_cb, &csi);
@@ -794,6 +796,12 @@ static struct object *console_lookup_name( struct object *obj, struct unicode_st
     static const WCHAR connectionW[]    = {'C','o','n','n','e','c','t','i','o','n'};
     assert( obj->ops == &console_ops );
 
+    if (!name)
+    {
+        set_error( STATUS_NOT_FOUND );
+        return NULL;
+    }
+
     if (name->len == sizeof(connectionW) && !memcmp( name->str, connectionW, name->len ))
     {
         name->len = 0;
@@ -879,6 +887,12 @@ static struct object *console_server_lookup_name( struct object *obj, struct uni
     static const WCHAR referenceW[] = {'R','e','f','e','r','e','n','c','e'};
     assert( obj->ops == &console_server_ops );
 
+    if (!name)
+    {
+        set_error( STATUS_NOT_FOUND );
+        return NULL;
+    }
+
     if (name->len == sizeof(referenceW) && !memcmp( name->str, referenceW, name->len ))
     {
         struct screen_buffer *screen_buffer;
@@ -1181,7 +1195,7 @@ static void console_server_ioctl( struct fd *fd, ioctl_code_t code, struct async
                 return;
             }
             term = server->termios;
-            term.c_lflag &= ~(ECHO | ECHONL | ICANON | IEXTEN);
+            term.c_lflag &= ~(ECHO | ECHONL | ICANON | IEXTEN | ISIG);
             term.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
             term.c_cflag &= ~(CSIZE | PARENB);
             term.c_cflag |= CS8;
