@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <signal.h>
@@ -46,18 +45,32 @@ struct debug_event
     struct object          obj;       /* object header */
     struct list            entry;     /* entry in event queue */
     struct thread         *sender;    /* thread which sent this event */
-    struct thread         *debugger;  /* debugger thread receiving the event */
+    struct file           *file;      /* file object for events that need one */
     enum debug_event_state state;     /* event state */
     int                    status;    /* continuation status */
-    debug_event_t          data;      /* event data */
+    union debug_event_data data;      /* event data */
 };
 
-/* debug context */
-struct debug_ctx
+static const WCHAR debug_obj_name[] = {'D','e','b','u','g','O','b','j','e','c','t'};
+
+struct type_descr debug_obj_type =
+{
+    { debug_obj_name, sizeof(debug_obj_name) },   /* name */
+    DEBUG_ALL_ACCESS | SYNCHRONIZE,               /* valid_access */
+    {                                             /* mapping */
+        STANDARD_RIGHTS_READ | DEBUG_READ_EVENT,
+        STANDARD_RIGHTS_WRITE | DEBUG_PROCESS_ASSIGN,
+        STANDARD_RIGHTS_EXECUTE | SYNCHRONIZE,
+        DEBUG_ALL_ACCESS
+    },
+};
+
+/* debug object */
+struct debug_obj
 {
     struct object        obj;         /* object header */
     struct list          event_queue; /* pending events queue */
-    int                  kill_on_exit;/* kill debuggees on debugger exit ? */
+    unsigned int         flags;       /* debug flags */
 };
 
 
@@ -68,15 +81,15 @@ static void debug_event_destroy( struct object *obj );
 static const struct object_ops debug_event_ops =
 {
     sizeof(struct debug_event),    /* size */
+    &no_type,                      /* type */
     debug_event_dump,              /* dump */
-    no_get_type,                   /* get_type */
     add_queue,                     /* add_queue */
     remove_queue,                  /* remove_queue */
     debug_event_signaled,          /* signaled */
     no_satisfied,                  /* satisfied */
     no_signal,                     /* signal */
     no_get_fd,                     /* get_fd */
-    no_map_access,                 /* map_access */
+    default_map_access,            /* map_access */
     default_get_sd,                /* get_sd */
     default_set_sd,                /* set_sd */
     no_get_full_name,              /* get_full_name */
@@ -89,160 +102,141 @@ static const struct object_ops debug_event_ops =
     debug_event_destroy            /* destroy */
 };
 
-static void debug_ctx_dump( struct object *obj, int verbose );
-static int debug_ctx_signaled( struct object *obj, struct wait_queue_entry *entry );
-static void debug_ctx_destroy( struct object *obj );
+static void debug_obj_dump( struct object *obj, int verbose );
+static int debug_obj_signaled( struct object *obj, struct wait_queue_entry *entry );
+static void debug_obj_destroy( struct object *obj );
 
-static const struct object_ops debug_ctx_ops =
+static const struct object_ops debug_obj_ops =
 {
-    sizeof(struct debug_ctx),      /* size */
-    debug_ctx_dump,                /* dump */
-    no_get_type,                   /* get_type */
+    sizeof(struct debug_obj),      /* size */
+    &debug_obj_type,               /* type */
+    debug_obj_dump,                /* dump */
     add_queue,                     /* add_queue */
     remove_queue,                  /* remove_queue */
-    debug_ctx_signaled,            /* signaled */
+    debug_obj_signaled,            /* signaled */
     no_satisfied,                  /* satisfied */
     no_signal,                     /* signal */
     no_get_fd,                     /* get_fd */
-    no_map_access,                 /* map_access */
+    default_map_access,            /* map_access */
     default_get_sd,                /* get_sd */
     default_set_sd,                /* set_sd */
-    no_get_full_name,              /* get_full_name */
+    default_get_full_name,         /* get_full_name */
     no_lookup_name,                /* lookup_name */
-    no_link_name,                  /* link_name */
-    NULL,                          /* unlink_name */
+    directory_link_name,           /* link_name */
+    default_unlink_name,           /* unlink_name */
     no_open_file,                  /* open_file */
     no_kernel_obj_list,            /* get_kernel_obj_list */
     no_close_handle,               /* close_handle */
-    debug_ctx_destroy              /* destroy */
+    debug_obj_destroy              /* destroy */
 };
 
+/* get a pointer to TEB->ArbitraryUserPointer in the client address space */
+static client_ptr_t get_teb_user_ptr( struct thread *thread )
+{
+    unsigned int ptr_size = is_machine_64bit( native_machine ) ? 8 : 4;
+    return thread->teb + 5 * ptr_size;
+}
+
 
 /* routines to build an event according to its type */
 
-static int fill_exception_event( struct debug_event *event, const void *arg )
+static void fill_exception_event( struct debug_event *event, const void *arg )
 {
-    const debug_event_t *data = arg;
+    const union debug_event_data *data = arg;
     event->data.exception = data->exception;
     event->data.exception.nb_params = min( event->data.exception.nb_params, EXCEPTION_MAXIMUM_PARAMETERS );
-    return 1;
 }
 
-static int fill_create_thread_event( struct debug_event *event, const void *arg )
+static void fill_create_thread_event( struct debug_event *event, const void *arg )
 {
-    struct process *debugger = event->debugger->process;
-    struct thread *thread = event->sender;
     const client_ptr_t *entry = arg;
-    obj_handle_t handle;
 
-    /* documented: THREAD_GET_CONTEXT | THREAD_SET_CONTEXT | THREAD_SUSPEND_RESUME */
-    if (!(handle = alloc_handle( debugger, thread, THREAD_ALL_ACCESS, 0 ))) return 0;
-    event->data.create_thread.handle = handle;
-    event->data.create_thread.teb    = thread->teb;
     if (entry) event->data.create_thread.start = *entry;
-    return 1;
 }
 
-static int fill_create_process_event( struct debug_event *event, const void *arg )
+static void fill_create_process_event( struct debug_event *event, const void *arg )
 {
-    struct process *debugger = event->debugger->process;
-    struct thread *thread = event->sender;
-    struct process *process = thread->process;
-    struct process_dll *exe_module = get_process_exe_module( process );
-    const client_ptr_t *entry = arg;
-    struct file *file;
-    obj_handle_t handle;
-
-    /* documented: PROCESS_VM_READ | PROCESS_VM_WRITE */
-    if (!(handle = alloc_handle( debugger, process, PROCESS_ALL_ACCESS, 0 ))) return 0;
-    event->data.create_process.process = handle;
-
-    /* documented: THREAD_GET_CONTEXT | THREAD_SET_CONTEXT | THREAD_SUSPEND_RESUME */
-    if (!(handle = alloc_handle( debugger, thread, THREAD_ALL_ACCESS, 0 )))
-    {
-        close_handle( debugger, event->data.create_process.process );
-        return 0;
-    }
-    event->data.create_process.thread     = handle;
-    event->data.create_process.file       = 0;
-    event->data.create_process.teb        = thread->teb;
-    event->data.create_process.base       = exe_module->base;
-    event->data.create_process.start      = *entry;
-    event->data.create_process.dbg_offset = exe_module->dbg_offset;
-    event->data.create_process.dbg_size   = exe_module->dbg_size;
-    event->data.create_process.name       = exe_module->name;
-    event->data.create_process.unicode    = 1;
+    const struct memory_view *view = arg;
+    const struct pe_image_info *image_info = get_view_image_info( view, &event->data.create_process.base );
 
+    event->data.create_process.start      = event->sender->entry_point;
+    event->data.create_process.dbg_offset = image_info->dbg_offset;
+    event->data.create_process.dbg_size   = image_info->dbg_size;
     /* the doc says write access too, but this doesn't seem a good idea */
-    if ((file = get_mapping_file( process, exe_module->base, GENERIC_READ,
-                                  FILE_SHARE_READ | FILE_SHARE_WRITE )))
-    {
-        event->data.create_process.file = alloc_handle( debugger, file, GENERIC_READ, 0 );
-        release_object( file );
-    }
-    return 1;
+    event->file = get_view_file( view, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE );
 }
 
-static int fill_exit_thread_event( struct debug_event *event, const void *arg )
+static void fill_exit_thread_event( struct debug_event *event, const void *arg )
 {
     const struct thread *thread = arg;
     event->data.exit.exit_code = thread->exit_code;
-    return 1;
 }
 
-static int fill_exit_process_event( struct debug_event *event, const void *arg )
+static void fill_exit_process_event( struct debug_event *event, const void *arg )
 {
     const struct process *process = arg;
     event->data.exit.exit_code = process->exit_code;
-    return 1;
 }
 
-static int fill_load_dll_event( struct debug_event *event, const void *arg )
+static void fill_load_dll_event( struct debug_event *event, const void *arg )
 {
-    struct process *process = event->sender->process;
-    struct process *debugger = event->debugger->process;
-    const struct process_dll *dll = arg;
-    struct file *file;
+    const struct memory_view *view = arg;
+    const struct pe_image_info *image_info = get_view_image_info( view, &event->data.load_dll.base );
 
-    event->data.load_dll.handle     = 0;
-    event->data.load_dll.base       = dll->base;
-    event->data.load_dll.dbg_offset = dll->dbg_offset;
-    event->data.load_dll.dbg_size   = dll->dbg_size;
-    event->data.load_dll.name       = dll->name;
-    event->data.load_dll.unicode    = 1;
-    if ((file = get_mapping_file( process, dll->base, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE )))
-    {
-        event->data.load_dll.handle = alloc_handle( debugger, file, GENERIC_READ, 0 );
-        release_object( file );
-    }
-    return 1;
+    event->data.load_dll.dbg_offset = image_info->dbg_offset;
+    event->data.load_dll.dbg_size   = image_info->dbg_size;
+    event->data.load_dll.name       = get_teb_user_ptr( event->sender );
+    event->file = get_view_file( view, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE );
 }
 
-static int fill_unload_dll_event( struct debug_event *event, const void *arg )
+static void fill_unload_dll_event( struct debug_event *event, const void *arg )
 {
-    const mod_handle_t *base = arg;
-    event->data.unload_dll.base = *base;
-    return 1;
+    const struct memory_view *view = arg;
+    get_view_image_info( view, &event->data.unload_dll.base );
 }
 
-typedef int (*fill_event_func)( struct debug_event *event, const void *arg );
+typedef void (*fill_event_func)( struct debug_event *event, const void *arg );
 
-#define NB_DEBUG_EVENTS UNLOAD_DLL_DEBUG_EVENT
-
-static const fill_event_func fill_debug_event[NB_DEBUG_EVENTS] =
+static const fill_event_func fill_debug_event[] =
 {
-    fill_exception_event,            /* EXCEPTION_DEBUG_EVENT */
-    fill_create_thread_event,        /* CREATE_THREAD_DEBUG_EVENT */
-    fill_create_process_event,       /* CREATE_PROCESS_DEBUG_EVENT */
-    fill_exit_thread_event,          /* EXIT_THREAD_DEBUG_EVENT */
-    fill_exit_process_event,         /* EXIT_PROCESS_DEBUG_EVENT */
-    fill_load_dll_event,             /* LOAD_DLL_DEBUG_EVENT */
-    fill_unload_dll_event            /* UNLOAD_DLL_DEBUG_EVENT */
+    fill_create_thread_event,  /* DbgCreateThreadStateChange */
+    fill_create_process_event, /* DbgCreateProcessStateChange */
+    fill_exit_thread_event,    /* DbgExitThreadStateChange */
+    fill_exit_process_event,   /* DbgExitProcessStateChange */
+    fill_exception_event,      /* DbgExceptionStateChange */
+    fill_exception_event,      /* DbgBreakpointStateChange */
+    fill_exception_event,      /* DbgSingleStepStateChange */
+    fill_load_dll_event,       /* DbgLoadDllStateChange */
+    fill_unload_dll_event      /* DbgUnloadDllStateChange */
 };
 
+/* allocate the necessary handles in the event data */
+static void alloc_event_handles( struct debug_event *event, struct process *process )
+{
+    switch (event->data.code)
+    {
+    case DbgCreateThreadStateChange:
+        /* documented: THREAD_GET_CONTEXT | THREAD_SET_CONTEXT | THREAD_SUSPEND_RESUME */
+        event->data.create_thread.handle = alloc_handle( process, event->sender, THREAD_ALL_ACCESS, 0 );
+        break;
+    case DbgCreateProcessStateChange:
+        event->data.create_process.thread = alloc_handle( process, event->sender, THREAD_ALL_ACCESS, 0 );
+        /* documented: PROCESS_VM_READ | PROCESS_VM_WRITE */
+        event->data.create_process.process = alloc_handle( process, event->sender->process,
+                                                           PROCESS_ALL_ACCESS, 0 );
+        if (event->file)
+            event->data.create_process.file = alloc_handle( process, event->file, GENERIC_READ, 0 );
+        break;
+    case DbgLoadDllStateChange:
+        if (event->file)
+            event->data.load_dll.handle = alloc_handle( process, event->file, GENERIC_READ, 0 );
+        break;
+    }
+    clear_error();  /* ignore errors, simply set handles to 0 */
+}
 
 /* unlink the first event from the queue */
-static void unlink_event( struct debug_ctx *debug_ctx, struct debug_event *event )
+static void unlink_event( struct debug_obj *debug_obj, struct debug_event *event )
 {
     list_remove( &event->entry );
     if (event->sender->process->debug_event == event) event->sender->process->debug_event = NULL;
@@ -250,53 +244,44 @@ static void unlink_event( struct debug_ctx *debug_ctx, struct debug_event *event
 }
 
 /* link an event at the end of the queue */
-static void link_event( struct debug_event *event )
+static void link_event( struct debug_obj *debug_obj, struct debug_event *event )
 {
-    struct debug_ctx *debug_ctx = event->debugger->debug_ctx;
-
-    assert( debug_ctx );
     grab_object( event );
-    list_add_tail( &debug_ctx->event_queue, &event->entry );
+    list_add_tail( &debug_obj->event_queue, &event->entry );
     if (!event->sender->process->debug_event)
     {
         /* grab reference since debugger could be killed while trying to wake up */
-        grab_object( debug_ctx );
-        wake_up( &debug_ctx->obj, 0 );
-        release_object( debug_ctx );
+        grab_object( debug_obj );
+        wake_up( &debug_obj->obj, 0 );
+        release_object( debug_obj );
     }
 }
 
 /* resume a delayed debug event already in the queue */
-static void resume_event( struct debug_event *event )
+static void resume_event( struct debug_obj *debug_obj, struct debug_event *event )
 {
-    struct debug_ctx *debug_ctx = event->debugger->debug_ctx;
-
-    assert( debug_ctx );
     event->state = EVENT_QUEUED;
     if (!event->sender->process->debug_event)
     {
-        grab_object( debug_ctx );
-        wake_up( &debug_ctx->obj, 0 );
-        release_object( debug_ctx );
+        grab_object( debug_obj );
+        wake_up( &debug_obj->obj, 0 );
+        release_object( debug_obj );
     }
 }
 
 /* delay a debug event already in the queue to be replayed when thread wakes up */
-static void delay_event( struct debug_event *event )
+static void delay_event( struct debug_obj *debug_obj, struct debug_event *event )
 {
-    struct debug_ctx *debug_ctx = event->debugger->debug_ctx;
-
-    assert( debug_ctx );
     event->state = EVENT_DELAYED;
     if (event->sender->process->debug_event == event) event->sender->process->debug_event = NULL;
 }
 
 /* find the next event that we can send to the debugger */
-static struct debug_event *find_event_to_send( struct debug_ctx *debug_ctx )
+static struct debug_event *find_event_to_send( struct debug_obj *debug_obj )
 {
     struct debug_event *event;
 
-    LIST_FOR_EACH_ENTRY( event, &debug_ctx->event_queue, struct debug_event, entry )
+    LIST_FOR_EACH_ENTRY( event, &debug_obj->event_queue, struct debug_event, entry )
     {
         if (event->state == EVENT_SENT) continue;  /* already sent */
         if (event->state == EVENT_DELAYED) continue;  /* delayed until thread resumes */
@@ -326,64 +311,66 @@ static void debug_event_destroy( struct object *obj )
     struct debug_event *event = (struct debug_event *)obj;
     assert( obj->ops == &debug_event_ops );
 
-    /* If the event has been sent already, the handles are now under the */
-    /* responsibility of the debugger process, so we don't touch them    */
-    if (event->state == EVENT_QUEUED)
-    {
-        struct process *debugger = event->debugger->process;
-        switch(event->data.code)
-        {
-        case CREATE_THREAD_DEBUG_EVENT:
-            close_handle( debugger, event->data.create_thread.handle );
-            break;
-        case CREATE_PROCESS_DEBUG_EVENT:
-            if (event->data.create_process.file)
-                close_handle( debugger, event->data.create_process.file );
-            close_handle( debugger, event->data.create_process.thread );
-            close_handle( debugger, event->data.create_process.process );
-            break;
-        case LOAD_DLL_DEBUG_EVENT:
-            if (event->data.load_dll.handle)
-                close_handle( debugger, event->data.load_dll.handle );
-            break;
-        }
-    }
+    if (event->file) release_object( event->file );
     release_object( event->sender );
-    release_object( event->debugger );
 }
 
-static void debug_ctx_dump( struct object *obj, int verbose )
+static void debug_obj_dump( struct object *obj, int verbose )
 {
-    struct debug_ctx *debug_ctx = (struct debug_ctx *)obj;
-    assert( obj->ops == &debug_ctx_ops );
+    struct debug_obj *debug_obj = (struct debug_obj *)obj;
+    assert( obj->ops == &debug_obj_ops );
     fprintf( stderr, "Debug context head=%p tail=%p\n",
-             debug_ctx->event_queue.next, debug_ctx->event_queue.prev );
+             debug_obj->event_queue.next, debug_obj->event_queue.prev );
 }
 
-static int debug_ctx_signaled( struct object *obj, struct wait_queue_entry *entry )
+static int debug_obj_signaled( struct object *obj, struct wait_queue_entry *entry )
 {
-    struct debug_ctx *debug_ctx = (struct debug_ctx *)obj;
-    assert( obj->ops == &debug_ctx_ops );
-    return find_event_to_send( debug_ctx ) != NULL;
+    struct debug_obj *debug_obj = (struct debug_obj *)obj;
+    assert( obj->ops == &debug_obj_ops );
+    return find_event_to_send( debug_obj ) != NULL;
 }
 
-static void debug_ctx_destroy( struct object *obj )
+static void debug_obj_destroy( struct object *obj )
 {
     struct list *ptr;
-    struct debug_ctx *debug_ctx = (struct debug_ctx *)obj;
-    assert( obj->ops == &debug_ctx_ops );
+    struct debug_obj *debug_obj = (struct debug_obj *)obj;
+    assert( obj->ops == &debug_obj_ops );
+
+    detach_debugged_processes( debug_obj,
+                               (debug_obj->flags & DEBUG_KILL_ON_CLOSE) ? STATUS_DEBUGGER_INACTIVE : 0 );
 
     /* free all pending events */
-    while ((ptr = list_head( &debug_ctx->event_queue )))
-        unlink_event( debug_ctx, LIST_ENTRY( ptr, struct debug_event, entry ));
+    while ((ptr = list_head( &debug_obj->event_queue )))
+        unlink_event( debug_obj, LIST_ENTRY( ptr, struct debug_event, entry ));
+}
+
+struct debug_obj *get_debug_obj( struct process *process, obj_handle_t handle, unsigned int access )
+{
+    return (struct debug_obj *)get_handle_obj( process, handle, access, &debug_obj_ops );
+}
+
+static struct debug_obj *create_debug_obj( struct object *root, const struct unicode_str *name,
+                                           unsigned int attr, unsigned int flags,
+                                           const struct security_descriptor *sd )
+{
+    struct debug_obj *debug_obj;
+
+    if ((debug_obj = create_named_object( root, &debug_obj_ops, name, attr, sd )))
+    {
+        if (get_error() != STATUS_OBJECT_NAME_EXISTS)
+        {
+            debug_obj->flags = flags;
+            list_init( &debug_obj->event_queue );
+        }
+    }
+    return debug_obj;
 }
 
 /* continue a debug event */
-static int continue_debug_event( struct process *process, struct thread *thread, int status )
+static int continue_debug_event( struct debug_obj *debug_obj, struct process *process,
+                                 struct thread *thread, int status )
 {
-    struct debug_ctx *debug_ctx = current->debug_ctx;
-
-    if (debug_ctx && process->debugger == current && thread->process == process)
+    if (process->debug_obj == debug_obj && thread->process == process)
     {
         struct debug_event *event;
 
@@ -391,19 +378,19 @@ static int continue_debug_event( struct process *process, struct thread *thread,
         {
             /* if thread is suspended, delay all its events and resume process
              * if not, reset the event for immediate replay */
-            LIST_FOR_EACH_ENTRY( event, &debug_ctx->event_queue, struct debug_event, entry )
+            LIST_FOR_EACH_ENTRY( event, &debug_obj->event_queue, struct debug_event, entry )
             {
                 if (event->sender != thread) continue;
                 if (thread->suspend)
                 {
-                    delay_event( event );
+                    delay_event( debug_obj, event );
                     resume_process( process );
                 }
                 else if (event->state == EVENT_SENT)
                 {
                     assert( event->sender->process->debug_event == event );
                     event->sender->process->debug_event = NULL;
-                    resume_event( event );
+                    resume_event( debug_obj, event );
                     return 1;
                 }
             }
@@ -411,7 +398,7 @@ static int continue_debug_event( struct process *process, struct thread *thread,
         }
 
         /* find the event in the queue */
-        LIST_FOR_EACH_ENTRY( event, &debug_ctx->event_queue, struct debug_event, entry )
+        LIST_FOR_EACH_ENTRY( event, &debug_obj->event_queue, struct debug_event, entry )
         {
             if (event->state != EVENT_SENT) continue;
             if (event->sender == thread)
@@ -420,7 +407,7 @@ static int continue_debug_event( struct process *process, struct thread *thread,
                 event->status = status;
                 event->state  = EVENT_CONTINUED;
                 wake_up( &event->obj, 0 );
-                unlink_event( debug_ctx, event );
+                unlink_event( debug_obj, event );
                 resume_process( process );
                 return 1;
             }
@@ -434,26 +421,17 @@ static int continue_debug_event( struct process *process, struct thread *thread,
 /* alloc a debug event for a debugger */
 static struct debug_event *alloc_debug_event( struct thread *thread, int code, const void *arg )
 {
-    struct thread *debugger = thread->process->debugger;
     struct debug_event *event;
 
-    assert( code > 0 && code <= NB_DEBUG_EVENTS );
-    /* cannot queue a debug event for myself */
-    assert( debugger->process != thread->process );
+    assert( code >= DbgCreateThreadStateChange && code <= DbgUnloadDllStateChange );
 
     /* build the event */
     if (!(event = alloc_object( &debug_event_ops ))) return NULL;
     event->state     = EVENT_QUEUED;
     event->sender    = (struct thread *)grab_object( thread );
-    event->debugger  = (struct thread *)grab_object( debugger );
+    event->file      = NULL;
     memset( &event->data, 0, sizeof(event->data) );
-
-    if (!fill_debug_event[code-1]( event, arg ))
-    {
-        event->data.code = -1;  /* make sure we don't attempt to close handles */
-        release_object( event );
-        return NULL;
-    }
+    fill_debug_event[code - DbgCreateThreadStateChange]( event, arg );
     event->data.code = code;
     return event;
 }
@@ -461,12 +439,14 @@ static struct debug_event *alloc_debug_event( struct thread *thread, int code, c
 /* generate a debug event from inside the server and queue it */
 void generate_debug_event( struct thread *thread, int code, const void *arg )
 {
-    if (thread->process->debugger)
+    struct debug_obj *debug_obj = thread->process->debug_obj;
+
+    if (debug_obj)
     {
         struct debug_event *event = alloc_debug_event( thread, code, arg );
         if (event)
         {
-            link_event( event );
+            link_event( debug_obj, event );
             suspend_process( thread->process );
             release_object( event );
         }
@@ -476,51 +456,45 @@ void generate_debug_event( struct thread *thread, int code, const void *arg )
 
 void resume_delayed_debug_events( struct thread *thread )
 {
-    struct thread *debugger = thread->process->debugger;
+    struct debug_obj *debug_obj = thread->process->debug_obj;
     struct debug_event *event;
 
-    if (debugger)
+    if (debug_obj)
     {
-        assert( debugger->debug_ctx );
-        LIST_FOR_EACH_ENTRY( event, &debugger->debug_ctx->event_queue, struct debug_event, entry )
+        LIST_FOR_EACH_ENTRY( event, &debug_obj->event_queue, struct debug_event, entry )
         {
             if (event->sender != thread) continue;
             if (event->state != EVENT_DELAYED) continue;
-            resume_event( event );
+            resume_event( debug_obj, event );
             suspend_process( thread->process );
         }
     }
 }
 
 /* attach a process to a debugger thread and suspend it */
-static int debugger_attach( struct process *process, struct thread *debugger )
+static int debugger_attach( struct process *process, struct thread *debugger, struct debug_obj *debug_obj )
 {
-    if (process->debugger) goto error;  /* already being debugged */
     if (debugger->process == process) goto error;
     if (!is_process_init_done( process )) goto error;  /* still starting up */
     if (list_empty( &process->thread_list )) goto error;  /* no thread running in the process */
-
     /* don't let a debugger debug its console... won't work */
     if (debugger->process->console)
     {
         struct thread *renderer = console_get_renderer(debugger->process->console);
-        if (renderer && renderer->process == process)
-            goto error;
+        if (renderer && renderer->process == process) goto error;
     }
 
     suspend_process( process );
-    if (!set_process_debugger( process, debugger ))
-    {
-        resume_process( process );
-        return 0;
-    }
+
     if (!set_process_debug_flag( process, 1 ))
     {
-        process->debugger = NULL;
         resume_process( process );
         return 0;
     }
+    process->debug_obj = debug_obj;
     process->debug_children = 0;
+    generate_startup_debug_events( process );
+    resume_process( process );
     return 1;
 
  error:
@@ -529,24 +503,15 @@ static int debugger_attach( struct process *process, struct thread *debugger )
 }
 
 
-/* detach a process from a debugger thread (and resume it ?) */
-int debugger_detach( struct process *process, struct thread *debugger )
+/* detach a process from a debugger thread (and resume it) */
+void debugger_detach( struct process *process, struct debug_obj *debug_obj )
 {
     struct debug_event *event, *next;
-    struct debug_ctx *debug_ctx;
-
-    if (!process->debugger || process->debugger != debugger)
-        goto error;  /* not currently debugged, or debugged by another debugger */
-    if (!debugger->debug_ctx ) goto error; /* should be a debugger */
-    /* init should be done, otherwise wouldn't be attached */
-    assert(is_process_init_done(process));
 
     suspend_process( process );
-    /* send continue indication for all events */
-    debug_ctx = debugger->debug_ctx;
 
-    /* free all events from this process */
-    LIST_FOR_EACH_ENTRY_SAFE( event, next, &debug_ctx->event_queue, struct debug_event, entry )
+    /* send continue indication for all events */
+    LIST_FOR_EACH_ENTRY_SAFE( event, next, &debug_obj->event_queue, struct debug_event, entry )
     {
         if (event->sender->process != process) continue;
 
@@ -554,126 +519,69 @@ int debugger_detach( struct process *process, struct thread *debugger )
         event->status = DBG_CONTINUE;
         event->state  = EVENT_CONTINUED;
         wake_up( &event->obj, 0 );
-        unlink_event( debug_ctx, event );
+        unlink_event( debug_obj, event );
         /* from queued debug event */
         resume_process( process );
     }
 
     /* remove relationships between process and its debugger */
-    process->debugger = NULL;
+    process->debug_obj = NULL;
     if (!set_process_debug_flag( process, 0 )) clear_error();  /* ignore error */
 
-    /* from this function */
     resume_process( process );
-    return 0;
-
- error:
-    set_error( STATUS_ACCESS_DENIED );
-    return 0;
 }
 
-/* generate all startup events of a given process */
-void generate_startup_debug_events( struct process *process, client_ptr_t entry )
+/* create a debug object */
+DECL_HANDLER(create_debug_obj)
 {
-    struct list *ptr;
-    struct thread *thread, *first_thread = get_process_first_thread( process );
+    struct debug_obj *debug_obj;
+    struct unicode_str name;
+    struct object *root;
+    const struct security_descriptor *sd;
+    const struct object_attributes *objattr = get_req_object_attributes( &sd, &name, &root );
 
-    generate_debug_event( first_thread, CREATE_PROCESS_DEBUG_EVENT, &entry );
-    ptr = list_head( &process->dlls ); /* skip main module reported in create process event */
-
-    /* generate ntdll.dll load event */
-    if (ptr && (ptr = list_next( &process->dlls, ptr )))
+    if (!objattr) return;
+    if ((debug_obj = create_debug_obj( root, &name, objattr->attributes, req->flags, sd )))
     {
-        struct process_dll *dll = LIST_ENTRY( ptr, struct process_dll, entry );
-        generate_debug_event( first_thread, LOAD_DLL_DEBUG_EVENT, dll );
-    }
-
-    /* generate creation events */
-    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
-    {
-        if (thread != first_thread)
-            generate_debug_event( thread, CREATE_THREAD_DEBUG_EVENT, NULL );
-    }
-
-    /* generate dll events (in loading order) */
-    while (ptr && (ptr = list_next( &process->dlls, ptr )))
-    {
-        struct process_dll *dll = LIST_ENTRY( ptr, struct process_dll, entry );
-        generate_debug_event( first_thread, LOAD_DLL_DEBUG_EVENT, dll );
-    }
-}
-
-/* set the debugger of a given process */
-int set_process_debugger( struct process *process, struct thread *debugger )
-{
-    struct debug_ctx *debug_ctx;
-
-    assert( !process->debugger );
-
-    if (!debugger->debug_ctx)  /* need to allocate a context */
-    {
-        if (!(debug_ctx = alloc_object( &debug_ctx_ops ))) return 0;
-        debug_ctx->kill_on_exit = 1;
-        list_init( &debug_ctx->event_queue );
-        debugger->debug_ctx = debug_ctx;
-    }
-    process->debugger = debugger;
-    return 1;
-}
-
-/* a thread is exiting */
-void debug_exit_thread( struct thread *thread )
-{
-    if (thread->debug_ctx)  /* this thread is a debugger */
-    {
-        if (thread->debug_ctx->kill_on_exit)
-        {
-            /* kill all debugged processes */
-            kill_debugged_processes( thread, STATUS_DEBUGGER_INACTIVE );
-        }
+        if (get_error() == STATUS_OBJECT_NAME_EXISTS)
+            reply->handle = alloc_handle( current->process, debug_obj, req->access, objattr->attributes );
         else
-        {
-            detach_debugged_processes( thread );
-        }
-        release_object( thread->debug_ctx );
-        thread->debug_ctx = NULL;
+            reply->handle = alloc_handle_no_access_check( current->process, debug_obj,
+                                                          req->access, objattr->attributes );
+        release_object( debug_obj );
     }
+    if (root) release_object( root );
 }
 
 /* Wait for a debug event */
 DECL_HANDLER(wait_debug_event)
 {
-    struct debug_ctx *debug_ctx = current->debug_ctx;
+    struct debug_obj *debug_obj;
     struct debug_event *event;
 
-    if (!debug_ctx)  /* current thread is not a debugger */
+    if (!(debug_obj = get_debug_obj( current->process, req->debug, DEBUG_READ_EVENT ))) return;
+
+    if ((event = find_event_to_send( debug_obj )))
     {
-        set_error( STATUS_INVALID_HANDLE );
-        return;
-    }
-    reply->wait = 0;
-    if ((event = find_event_to_send( debug_ctx )))
-    {
-        data_size_t size = get_reply_max_size();
         event->state = EVENT_SENT;
         event->sender->process->debug_event = event;
         reply->pid = get_process_id( event->sender->process );
         reply->tid = get_thread_id( event->sender );
-        if (size > sizeof(debug_event_t)) size = sizeof(debug_event_t);
-        set_reply_data( &event->data, size );
+        alloc_event_handles( event, current->process );
+        set_reply_data( &event->data, min( get_reply_max_size(), sizeof(event->data) ));
     }
-    else  /* no event ready */
+    else
     {
-        reply->pid  = 0;
-        reply->tid  = 0;
-        if (req->get_handle)
-            reply->wait = alloc_handle( current->process, debug_ctx, SYNCHRONIZE, 0 );
+        int state = list_empty( &debug_obj->event_queue ) ? DbgIdle : DbgReplyPending;
+        set_reply_data( &state, min( get_reply_max_size(), sizeof(state) ));
     }
+    release_object( debug_obj );
 }
 
 /* Continue a debug event */
 DECL_HANDLER(continue_debug_event)
 {
+    struct debug_obj *debug_obj;
     struct process *process;
 
     if (req->status != DBG_EXCEPTION_NOT_HANDLED &&
@@ -685,32 +593,42 @@ DECL_HANDLER(continue_debug_event)
         return;
     }
 
+    if (!(debug_obj = get_debug_obj( current->process, req->debug, DEBUG_READ_EVENT ))) return;
+
     if ((process = get_process_from_id( req->pid )))
     {
         struct thread *thread = get_thread_from_id( req->tid );
         if (thread)
         {
-            continue_debug_event( process, thread, req->status );
+            continue_debug_event( debug_obj, process, thread, req->status );
             release_object( thread );
         }
         release_object( process );
     }
+    release_object( debug_obj );
 }
 
-/* Start debugging an existing process */
+/* start or stop debugging an existing process */
 DECL_HANDLER(debug_process)
 {
-    struct process *process = get_process_from_id( req->pid );
-    if (!process) return;
+    struct debug_obj *debug_obj;
+    struct process *process = get_process_from_handle( req->handle,
+                                            PROCESS_VM_READ | PROCESS_VM_WRITE | PROCESS_SUSPEND_RESUME );
 
-    if (!req->attach)
+    if (!process) return;
+    if ((debug_obj = get_debug_obj( current->process, req->debug, DEBUG_PROCESS_ASSIGN )))
     {
-        debugger_detach( process, current );
-    }
-    else if (debugger_attach( process, current ))
-    {
-        generate_startup_debug_events( process, 0 );
-        resume_process( process );
+        if (req->attach)
+        {
+            if (!process->debug_obj) debugger_attach( process, current, debug_obj );
+            else set_error( STATUS_ACCESS_DENIED );
+        }
+        else
+        {
+            if (process->debug_obj == debug_obj) debugger_detach( process, debug_obj );
+            else set_error( STATUS_ACCESS_DENIED );
+        }
+        release_object( debug_obj );
     }
     release_object( process );
 }
@@ -718,10 +636,12 @@ DECL_HANDLER(debug_process)
 /* queue an exception event */
 DECL_HANDLER(queue_exception_event)
 {
+    struct debug_obj *debug_obj = current->process->debug_obj;
+
     reply->handle = 0;
-    if (current->process->debugger)
+    if (debug_obj)
     {
-        debug_event_t data;
+        union debug_event_data data;
         struct debug_event *event;
         struct thread *thread = current;
 
@@ -741,11 +661,11 @@ DECL_HANDLER(queue_exception_event)
         data.exception.nb_params = req->len / sizeof(client_ptr_t);
         memcpy( data.exception.params, get_req_data(), req->len );
 
-        if ((event = alloc_debug_event( thread, EXCEPTION_DEBUG_EVENT, &data )))
+        if ((event = alloc_debug_event( thread, DbgExceptionStateChange, &data )))
         {
             if ((reply->handle = alloc_handle( thread->process, event, SYNCHRONIZE, 0 )))
             {
-                link_event( event );
+                link_event( debug_obj, event );
                 suspend_process( thread->process );
             }
             release_object( event );
@@ -767,13 +687,12 @@ DECL_HANDLER(get_exception_status)
     }
 }
 
-/* set debugger kill on exit flag */
-DECL_HANDLER(set_debugger_kill_on_exit)
+/* set debug object information */
+DECL_HANDLER(set_debug_obj_info)
 {
-    if (!current->debug_ctx)
-    {
-        set_error( STATUS_ACCESS_DENIED );
-        return;
-    }
-    current->debug_ctx->kill_on_exit = req->kill_on_exit;
+    struct debug_obj *debug_obj;
+
+    if (!(debug_obj = get_debug_obj( current->process, req->debug, DEBUG_SET_INFORMATION ))) return;
+    debug_obj->flags = req->flags;
+    release_object( debug_obj );
 }
