@@ -49,7 +49,7 @@ struct irp_call
     struct device_file    *file;          /* file containing this irp */
     struct thread         *thread;        /* thread that queued the irp */
     struct async          *async;         /* pending async op */
-    irp_params_t           params;        /* irp parameters */
+    union irp_params       params;        /* irp parameters */
     struct iosb           *iosb;          /* I/O status block */
     int                    canceled;      /* the call was canceled */
     client_ptr_t           user_ptr;      /* client side pointer */
@@ -348,7 +348,8 @@ static void irp_call_destroy( struct object *obj )
     if (irp->thread) release_object( irp->thread );
 }
 
-static struct irp_call *create_irp( struct device_file *file, const irp_params_t *params, struct async *async )
+static struct irp_call *create_irp( struct device_file *file, const union irp_params *params,
+                                    struct async *async )
 {
     struct irp_call *irp;
 
@@ -455,7 +456,7 @@ static struct object *device_open_file( struct object *obj, unsigned int access,
     if (device->manager)
     {
         struct irp_call *irp;
-        irp_params_t params;
+        union irp_params params;
 
         memset( &params, 0, sizeof(params) );
         params.create.type    = IRP_CALL_CREATE;
@@ -512,7 +513,7 @@ static int device_file_close_handle( struct object *obj, struct process *process
     if (!file->closed && file->device->manager && obj->handle_count == 1)  /* last handle */
     {
         struct irp_call *irp;
-        irp_params_t params;
+        union irp_params params;
 
         file->closed = 1;
         memset( &params, 0, sizeof(params) );
@@ -542,7 +543,7 @@ static void device_file_destroy( struct object *obj )
     release_object( file->device );
 }
 
-static int fill_irp_params( struct device_manager *manager, struct irp_call *irp, irp_params_t *params )
+static int fill_irp_params( struct device_manager *manager, struct irp_call *irp, union irp_params *params )
 {
     switch (irp->params.type)
     {
@@ -595,7 +596,7 @@ static void free_irp_params( struct irp_call *irp )
 }
 
 /* queue an irp to the device */
-static void queue_irp( struct device_file *file, const irp_params_t *params, struct async *async )
+static void queue_irp( struct device_file *file, const union irp_params *params, struct async *async )
 {
     struct irp_call *irp = create_irp( file, params, async );
     if (!irp) return;
@@ -615,7 +616,7 @@ static enum server_fd_type device_file_get_fd_type( struct fd *fd )
 static void device_file_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class )
 {
     struct device_file *file = get_fd_user( fd );
-    irp_params_t params;
+    union irp_params params;
 
     memset( &params, 0, sizeof(params) );
     params.volume.type = IRP_CALL_VOLUME;
@@ -626,7 +627,7 @@ static void device_file_get_volume_info( struct fd *fd, struct async *async, uns
 static void device_file_read( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct device_file *file = get_fd_user( fd );
-    irp_params_t params;
+    union irp_params params;
 
     memset( &params, 0, sizeof(params) );
     params.read.type = IRP_CALL_READ;
@@ -638,7 +639,7 @@ static void device_file_read( struct fd *fd, struct async *async, file_pos_t pos
 static void device_file_write( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct device_file *file = get_fd_user( fd );
-    irp_params_t params;
+    union irp_params params;
 
     memset( &params, 0, sizeof(params) );
     params.write.type = IRP_CALL_WRITE;
@@ -650,7 +651,7 @@ static void device_file_write( struct fd *fd, struct async *async, file_pos_t po
 static void device_file_flush( struct fd *fd, struct async *async )
 {
     struct device_file *file = get_fd_user( fd );
-    irp_params_t params;
+    union irp_params params;
 
     memset( &params, 0, sizeof(params) );
     params.flush.type = IRP_CALL_FLUSH;
@@ -660,7 +661,7 @@ static void device_file_flush( struct fd *fd, struct async *async )
 static void device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
 {
     struct device_file *file = get_fd_user( fd );
-    irp_params_t params;
+    union irp_params params;
 
     memset( &params, 0, sizeof(params) );
     params.ioctl.type = IRP_CALL_IOCTL;
@@ -671,7 +672,7 @@ static void device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *a
 static void cancel_irp_call( struct irp_call *irp )
 {
     struct irp_call *cancel_irp;
-    irp_params_t params;
+    union irp_params params;
 
     irp->canceled = 1;
     if (!irp->user_ptr || !irp->file || !irp->file->device->manager) return;
@@ -842,7 +843,7 @@ void free_kernel_objects( struct object *obj )
     {
         struct kernel_object *kernel_object = LIST_ENTRY( ptr, struct kernel_object, list_entry );
         struct irp_call *irp;
-        irp_params_t params;
+        union irp_params params;
 
         assert( !kernel_object->owned );
 
