@@ -137,7 +137,7 @@ static struct key_value *find_value( const struct key *key, const struct unicode
 struct save_branch_info
 {
     struct key  *key;
-    const char  *path;
+    const char  *filename;
 };
 
 #define MAX_SAVE_BRANCH_INFO 3
@@ -446,7 +446,7 @@ static struct security_descriptor *key_get_sd( struct object *obj )
         key_default_sd->dacl_len  = dacl_len;
         sid = (struct sid *)(key_default_sd + 1);
         sid = copy_sid( sid, &builtin_admins_sid );
-        sid = copy_sid( sid, &builtin_admins_sid );
+        copy_sid( sid, &builtin_admins_sid );
 
         dacl = (struct acl *)((char *)(key_default_sd + 1) + 2 * admins_sid_len);
         dacl->revision = ACL_REVISION;
@@ -979,8 +979,8 @@ static void enum_key( struct key *key, int index, int info_class, struct enum_ke
         if (len > namelen)
         {
             reply->namelen = namelen;
-            memcpy( data, key->obj.name->name, namelen );
-            memcpy( data + namelen, key->class, len - namelen );
+            data = mem_append( data, key->obj.name->name, namelen );
+            memcpy( data, key->class, len - namelen );
         }
         else if (info_class == KeyNameInformation)
         {
@@ -1001,7 +1001,7 @@ static void enum_key( struct key *key, int index, int info_class, struct enum_ke
 static void rename_key( struct key *key, const struct unicode_str *new_name )
 {
     struct object_name *new_name_ptr;
-    struct key *subkey, *parent = get_parent( key );
+    struct key *parent = get_parent( key );
     data_size_t len;
     int i, index, cur_index;
 
@@ -1014,7 +1014,7 @@ static void rename_key( struct key *key, const struct unicode_str *new_name )
     }
 
     /* check for existing subkey with the same name */
-    if (!parent || (subkey = find_subkey( parent, new_name, &index )))
+    if (!parent || find_subkey( parent, new_name, &index ))
     {
         set_error( STATUS_CANNOT_DELETE );
         return;
@@ -1292,8 +1292,8 @@ static void enum_value( struct key *key, int i, int info_class, struct enum_key_
             if (maxlen > namelen)
             {
                 reply->namelen = namelen;
-                memcpy( data, value->name, namelen );
-                memcpy( (char *)data + namelen, value->data, maxlen - namelen );
+                data = mem_append( data, value->name, namelen );
+                memcpy( data, value->data, maxlen - namelen );
             }
             else
             {
@@ -1635,7 +1635,7 @@ static int load_value( struct key *key, const char *buffer, struct file_load_inf
     case REG_SZ:
         if (!get_file_tmp_space( info, strlen(buffer) * sizeof(WCHAR) )) return 0;
         len = info->tmplen;
-        if ((res = parse_strW( info->tmp, &len, buffer, '\"' )) == -1) goto error;
+        if (parse_strW( info->tmp, &len, buffer, '\"' ) == -1) goto error;
         ptr = info->tmp;
         break;
     case REG_DWORD:
@@ -1696,7 +1696,7 @@ static int get_prefix_len( struct key *key, const char *name, struct file_load_i
     if (!get_file_tmp_space( info, strlen(name) * sizeof(WCHAR) )) return 0;
 
     len = info->tmplen;
-    if ((res = parse_strW( info->tmp, &len, name, ']' )) == -1)
+    if (parse_strW( info->tmp, &len, name, ']' ) == -1)
     {
         file_read_error( "Malformed key", info );
         return 0;
@@ -1823,7 +1823,7 @@ static int load_init_registry_from_file( const char *filename, struct key *key )
 
     assert( save_branch_count < MAX_SAVE_BRANCH_INFO );
 
-    save_branch_info[save_branch_count].path = filename;
+    save_branch_info[save_branch_count].filename = filename;
     save_branch_info[save_branch_count++].key = (struct key *)grab_object( key );
     make_object_permanent( &key->obj );
     return (f != NULL);
@@ -1831,15 +1831,16 @@ static int load_init_registry_from_file( const char *filename, struct key *key )
 
 static WCHAR *format_user_registry_path( const struct sid *sid, struct unicode_str *path )
 {
-    char buffer[7 + 11 + 11 + 11 * ARRAY_SIZE(sid->sub_auth)], *p = buffer;
+    char buffer[7 + 11 + 11 + 11 * ARRAY_SIZE(sid->sub_auth)];
     unsigned int i;
+    int len;
 
-    p += sprintf( p, "User\\S-%u-%u", sid->revision,
-                  ((unsigned int)sid->id_auth[2] << 24) |
-                  ((unsigned int)sid->id_auth[3] << 16) |
-                  ((unsigned int)sid->id_auth[4] << 8) |
-                  ((unsigned int)sid->id_auth[5]) );
-    for (i = 0; i < sid->sub_count; i++) p += sprintf( p, "-%u", sid->sub_auth[i] );
+    len = snprintf( buffer, sizeof(buffer), "User\\S-%u-%u", sid->revision,
+                    ((unsigned int)sid->id_auth[2] << 24) |
+                    ((unsigned int)sid->id_auth[3] << 16) |
+                    ((unsigned int)sid->id_auth[4] << 8) |
+                    ((unsigned int)sid->id_auth[5]) );
+    for (i = 0; i < sid->sub_count; i++) len += snprintf( buffer + len, sizeof(buffer) - len, "-%u", sid->sub_auth[i] );
     return ascii_to_unicode_str( buffer, path );
 }
 
@@ -1857,10 +1858,9 @@ static void init_supported_machines(void)
     if (prefix_type == PREFIX_64BIT)
     {
         supported_machines[count++] = IMAGE_FILE_MACHINE_ARM64;
-        supported_machines[count++] = IMAGE_FILE_MACHINE_AMD64;
         supported_machines[count++] = IMAGE_FILE_MACHINE_I386;
+        /* supported_machines[count++] = IMAGE_FILE_MACHINE_ARMNT;  not supported yet */
     }
-    supported_machines[count++] = IMAGE_FILE_MACHINE_ARMNT;
 #else
 #error Unsupported machine
 #endif
@@ -1877,25 +1877,26 @@ void init_registry(void)
     static const WCHAR classes_i386[] = {'S','o','f','t','w','a','r','e','\\',
                                          'C','l','a','s','s','e','s','\\',
                                          'W','o','w','6','4','3','2','N','o','d','e'};
-    static const WCHAR classes_amd64[] = {'S','o','f','t','w','a','r','e','\\',
-                                          'C','l','a','s','s','e','s','\\',
-                                          'W','o','w','6','4','6','4','N','o','d','e'};
     static const WCHAR classes_arm[] = {'S','o','f','t','w','a','r','e','\\',
                                         'C','l','a','s','s','e','s','\\',
                                         'W','o','w','A','A','3','2','N','o','d','e'};
-    static const WCHAR classes_arm64[] = {'S','o','f','t','w','a','r','e','\\',
-                                          'C','l','a','s','s','e','s','\\',
-                                          'W','o','w','A','A','6','4','N','o','d','e'};
     static const WCHAR perflib[] = {'S','o','f','t','w','a','r','e','\\',
                                     'M','i','c','r','o','s','o','f','t','\\',
                                     'W','i','n','d','o','w','s',' ','N','T','\\',
                                     'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
                                     'P','e','r','f','l','i','b','\\',
                                     '0','0','9'};
+    static const WCHAR controlset[] = {'S','y','s','t','e','m','\\',
+                                       'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t'};
+    static const WCHAR controlset001_path[] = {'\\','R','e','g','i','s','t','r','y','\\',
+                                               'M','a','c','h','i','n','e','\\',
+                                               'S','y','s','t','e','m','\\',
+                                               'C','o','n','t','r','o','l','S','e','t','0','0','1'};
     static const struct unicode_str root_name = { REGISTRY, sizeof(REGISTRY) };
     static const struct unicode_str HKLM_name = { HKLM, sizeof(HKLM) };
     static const struct unicode_str HKU_name = { HKU_default, sizeof(HKU_default) };
     static const struct unicode_str perflib_name = { perflib, sizeof(perflib) };
+    static const struct unicode_str controlset_name = { controlset, sizeof(controlset) };
 
     WCHAR *current_user_path;
     struct unicode_str current_user_str;
@@ -1923,6 +1924,13 @@ void init_registry(void)
             prefix_type = PREFIX_32BIT;
         else
             prefix_type = sizeof(void *) > sizeof(int) ? PREFIX_64BIT : PREFIX_32BIT;
+
+        if ((key = create_key_recursive( hklm, &controlset_name, current_time )))
+        {
+            key->flags |= KEY_SYMLINK;
+            set_value( key, &symlink_str, REG_LINK, controlset001_path, sizeof(controlset001_path) );
+            release_object( key );
+        }
     }
     else if (prefix_type == PREFIX_UNKNOWN)
         prefix_type = PREFIX_32BIT;
@@ -1956,8 +1964,7 @@ void init_registry(void)
         {
         case IMAGE_FILE_MACHINE_I386:  name.str = classes_i386;  name.len = sizeof(classes_i386);  break;
         case IMAGE_FILE_MACHINE_ARMNT: name.str = classes_arm;   name.len = sizeof(classes_arm);   break;
-        case IMAGE_FILE_MACHINE_AMD64: name.str = classes_amd64; name.len = sizeof(classes_amd64); break;
-        case IMAGE_FILE_MACHINE_ARM64: name.str = classes_arm64; name.len = sizeof(classes_arm64); break;
+        default: continue;
         }
         if ((key = create_key_recursive( hklm, &name, current_time )))
         {
@@ -2045,10 +2052,10 @@ static void save_registry( struct key *key, obj_handle_t handle )
 }
 
 /* save a registry branch to a file */
-static int save_branch( struct key *key, const char *path )
+static int save_branch( struct key *key, const char *filename )
 {
     struct stat st;
-    char *p, *tmp = NULL;
+    char tmp[32];
     int fd, count = 0, ret = 0;
     FILE *f;
 
@@ -2057,14 +2064,15 @@ static int save_branch( struct key *key, const char *path )
         if (debug_level > 1) dump_operation( key, NULL, "Not saving clean" );
         return 1;
     }
+    tmp[0] = 0;
 
     /* test the file type */
 
-    if ((fd = open( path, O_WRONLY )) != -1)
+    if ((fd = open( filename, O_WRONLY )) != -1)
     {
         /* if file is not a regular file or has multiple links or is accessed
          * via symbolic links, write directly into it; otherwise use a temp file */
-        if (!lstat( path, &st ) && (!S_ISREG(st.st_mode) || st.st_nlink > 1))
+        if (!lstat( filename, &st ) && (!S_ISREG(st.st_mode) || st.st_nlink > 1))
         {
             ftruncate( fd, 0 );
             goto save;
@@ -2072,15 +2080,11 @@ static int save_branch( struct key *key, const char *path )
         close( fd );
     }
 
-    /* create a temp file in the same directory */
+    /* create a temp file */
 
-    if (!(tmp = malloc( strlen(path) + 20 ))) goto done;
-    strcpy( tmp, path );
-    if ((p = strrchr( tmp, '/' ))) p++;
-    else p = tmp;
     for (;;)
     {
-        sprintf( p, "reg%lx%04x.tmp", (long) getpid(), count++ );
+        snprintf( tmp, sizeof(tmp), "reg%lx%04x.tmp", (long) getpid(), count++ );
         if ((fd = open( tmp, O_CREAT | O_EXCL | O_WRONLY, 0666 )) != -1) break;
         if (errno != EEXIST) goto done;
         close( fd );
@@ -2091,29 +2095,28 @@ static int save_branch( struct key *key, const char *path )
  save:
     if (!(f = fdopen( fd, "w" )))
     {
-        if (tmp) unlink( tmp );
+        if (tmp[0]) unlink( tmp );
         close( fd );
         goto done;
     }
 
     if (debug_level > 1)
     {
-        fprintf( stderr, "%s: ", path );
+        fprintf( stderr, "%s: ", filename );
         dump_operation( key, NULL, "saving" );
     }
 
     save_all_subkeys( key, f );
     ret = !fclose(f);
 
-    if (tmp)
+    if (tmp[0])
     {
         /* if successfully written, rename to final name */
-        if (ret) ret = !rename( tmp, path );
+        if (ret) ret = !rename( tmp, filename );
         if (!ret) unlink( tmp );
     }
 
 done:
-    free( tmp );
     if (ret) make_clean( key );
     return ret;
 }
@@ -2126,7 +2129,7 @@ static void periodic_save( void *arg )
     if (fchdir( config_dir_fd ) == -1) return;
     save_timeout_user = NULL;
     for (i = 0; i < save_branch_count; i++)
-        save_branch( save_branch_info[i].key, save_branch_info[i].path );
+        save_branch( save_branch_info[i].key, save_branch_info[i].filename );
     if (fchdir( server_dir_fd ) == -1) fatal_error( "chdir to server dir: %s\n", strerror( errno ));
     set_periodic_save_timer();
 }
@@ -2146,10 +2149,10 @@ void flush_registry(void)
     if (fchdir( config_dir_fd ) == -1) return;
     for (i = 0; i < save_branch_count; i++)
     {
-        if (!save_branch( save_branch_info[i].key, save_branch_info[i].path ))
+        if (!save_branch( save_branch_info[i].key, save_branch_info[i].filename ))
         {
             fprintf( stderr, "wineserver: could not save registry branch to %s",
-                     save_branch_info[i].path );
+                     save_branch_info[i].filename );
             perror( " " );
         }
     }
@@ -2189,7 +2192,11 @@ DECL_HANDLER(create_key)
             key->classlen = (key->classlen / sizeof(WCHAR)) * sizeof(WCHAR);
             if (!(key->class = memdup( class, key->classlen ))) key->classlen = 0;
         }
-        reply->hkey = alloc_handle( current->process, key, access, objattr->attributes );
+        if (get_error() == STATUS_OBJECT_NAME_EXISTS)
+            reply->hkey = alloc_handle( current->process, key, access, objattr->attributes );
+        else
+            reply->hkey = alloc_handle_no_access_check( current->process, key,
+                                                        access, objattr->attributes );
         release_object( key );
     }
     if (parent) release_object( parent );
@@ -2357,6 +2364,8 @@ DECL_HANDLER(unload_registry)
     {
         if (key->obj.handle_count)
             set_error( STATUS_CANNOT_DELETE );
+        else if (key->obj.is_permanent)
+            set_error( STATUS_ACCESS_DENIED );
         else
             delete_key( key, 1 );     /* FIXME */
         release_object( key );
