@@ -56,6 +56,8 @@
 #  define strncasecmp _strnicmp
 #  define strcasecmp _stricmp
 # endif
+# include <windef.h>
+# include <winbase.h>
 #else
 extern char **environ;
 # include <spawn.h>
@@ -326,6 +328,43 @@ static inline char *replace_extension( const char *name, const char *old_ext, co
     return strmake( "%.*s%s", name_len, name, new_ext );
 }
 
+/* build a path with the relative dir from 'from' to 'dest' appended to base */
+static inline char *build_relative_path( const char *base, const char *from, const char *dest )
+{
+    const char *start;
+    char *ret;
+    unsigned int dotdots = 0;
+
+    for (;;)
+    {
+        while (*from == '/') from++;
+        while (*dest == '/') dest++;
+        start = dest;  /* save start of next path element */
+        if (!*from) break;
+
+        while (*from && *from != '/' && *from == *dest) { from++; dest++; }
+        if ((!*from || *from == '/') && (!*dest || *dest == '/')) continue;
+
+        do  /* count remaining elements in 'from' */
+        {
+            dotdots++;
+            while (*from && *from != '/') from++;
+            while (*from == '/') from++;
+        }
+        while (*from);
+        break;
+    }
+
+    ret = xmalloc( strlen(base) + 3 * dotdots + strlen(start) + 2 );
+    strcpy( ret, base );
+    while (dotdots--) strcat( ret, "/.." );
+
+    if (!start[0]) return ret;
+    strcat( ret, "/" );
+    strcat( ret, start );
+    return ret;
+}
+
 /* temp files management */
 
 extern const char *temp_dir;
@@ -573,7 +612,7 @@ static inline const char *get_arch_dir( struct target target )
         [CPU_x86_64]  = "x86_64",
         [CPU_ARM]     = "arm",
         [CPU_ARM64]   = "aarch64",
-        [CPU_ARM64EC] = "arm64ec",
+        [CPU_ARM64EC] = "aarch64",
     };
 
     if (!cpu_names[target.cpu]) return "";
@@ -651,12 +690,13 @@ static inline struct target init_argv0_target( const char *argv0 )
 }
 
 
-static inline char *get_argv0_dir( const char *argv0 )
+static inline char *get_bindir( const char *argv0 )
 {
 #ifndef _WIN32
     char *dir = NULL;
 
-#if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__)
+#if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__) \
+        || defined(__CYGWIN__) || defined(__MSYS__)
     dir = realpath( "/proc/self/exe", NULL );
 #elif defined (__FreeBSD__) || defined(__DragonFly__)
     static int pathname[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
@@ -669,10 +709,56 @@ static inline char *get_argv0_dir( const char *argv0 )
     if (!dir && !(dir = realpath( argv0, NULL ))) return NULL;
     return get_dirname( dir );
 #else
-    return get_dirname( argv0 );
+    char path[MAX_PATH], *p;
+    GetModuleFileNameA( NULL, path, ARRAYSIZE(path) );
+    for (p = path; *p; p++) if (*p == '\\') *p = '/';
+    return get_dirname( path );
 #endif
 }
 
+#ifdef LIBDIR
+static inline const char *get_libdir( const char *bindir )
+{
+#ifdef BINDIR
+    if (bindir) return build_relative_path( bindir, BINDIR, LIBDIR );
+#endif
+    return LIBDIR;
+}
+#endif
+
+#ifdef DATADIR
+static inline const char *get_datadir( const char *bindir )
+{
+#ifdef BINDIR
+    if (bindir) return build_relative_path( bindir, BINDIR, DATADIR );
+#endif
+    return DATADIR;
+}
+#endif
+
+#ifdef INCLUDEDIR
+static inline const char *get_includedir( const char *bindir )
+{
+#ifdef BINDIR
+    if (bindir) return build_relative_path( bindir, BINDIR, INCLUDEDIR );
+#endif
+    return INCLUDEDIR;
+}
+#endif
+
+static inline const char *get_nlsdir( const char *bindir, const char *srcdir )
+{
+    if (bindir && strendswith( bindir, srcdir )) return strmake( "%s/../../nls", bindir );
+#ifdef DATADIR
+    else
+    {
+        const char *datadir = get_datadir( bindir );
+        if (datadir) return strmake( "%s/wine/nls", datadir );
+    }
+#endif
+    return NULL;
+}
+
 
 /* output buffer management */
 
