@@ -80,7 +80,7 @@ file_type get_file_type(const char* filename)
     if (strendswith(filename, ".res")) return file_res;
     if (strendswith(filename, ".so")) return file_so;
     if (strendswith(filename, ".dylib")) return file_so;
-    if (strendswith(filename, ".def")) return file_def;
+    if (strendswith(filename, ".def")) return file_spec;
     if (strendswith(filename, ".spec")) return file_spec;
     if (strendswith(filename, ".rc")) return file_rc;
     if (cnt >= sizeof(elf_sig) && !memcmp(buf, elf_sig, sizeof(elf_sig))) return file_so;  /* ELF lib */
@@ -133,10 +133,6 @@ static file_type guess_lib_type(struct target target, const char* dir,
         /* Mach-O (Darwin/Mac OS X) Dynamic Library behaves mostly like .so */
         if ((*file = try_lib_path(dir, "", prefix, library, ".dylib", file_so)))
             return file_so;
-
-        /* Windows DLL */
-        if ((*file = try_lib_path(dir, "", prefix, library, ".def", file_def)))
-            return file_dll;
     }
     else
     {
@@ -199,13 +195,14 @@ const char *find_binary( struct strarray prefix, const char *name )
 int spawn(struct strarray prefix, struct strarray args, int ignore_errors)
 {
     int status;
+    const char *cmd;
 
-    args.str[0] = find_binary( prefix, args.str[0] );
+    cmd = args.str[0] = find_binary( prefix, args.str[0] );
     if (verbose) strarray_trace( args );
 
     if ((status = strarray_spawn( args )) && !ignore_errors)
     {
-	if (status > 0) error("%s failed\n", args.str[0]);
+	if (status > 0) error("%s failed\n", cmd);
 	else perror("winegcc");
 	exit(3);
     }
