@@ -91,16 +91,12 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <signal.h>
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
 #include <limits.h>
 #include <sys/types.h>
-#ifdef HAVE_SYS_SYSCTL_H
-# include <sys/sysctl.h>
-#endif
 
 #include "utils.h"
 
@@ -148,10 +144,7 @@ static const char *output_debug_file;
 static const char *output_implib;
 static int keep_generated = 0;
 const char *temp_dir = NULL;
-static struct strarray tmp_files;
-#ifdef HAVE_SIGSET_T
-static sigset_t signal_mask;
-#endif
+struct strarray temp_files = { 0 };
 
 static const char *bindir;
 static const char *libdir;
@@ -184,6 +177,8 @@ struct options
     int unwind_tables;
     int strip;
     int pic;
+    int no_default_config;
+    int build_id;
     const char* wine_objdir;
     const char* winebuild;
     const char* output_name;
@@ -195,9 +190,9 @@ struct options
     const char* lib_suffix;
     const char* subsystem;
     const char* entry_point;
-    const char* prelink;
     const char* debug_file;
     const char* out_implib;
+    const char* native_arch;
     struct strarray prefix;
     struct strarray lib_dirs;
     struct strarray args;
@@ -217,13 +212,7 @@ static void cleanup_output_files(void)
 
 static void clean_temp_files(void)
 {
-    unsigned int i;
-
-    if (keep_generated) return;
-
-    for (i = 0; i < tmp_files.count; i++)
-	unlink(tmp_files.str[i]);
-    if (temp_dir) rmdir( temp_dir );
+    if (!keep_generated) remove_temp_files();
 }
 
 /* clean things up when aborting on a signal */
@@ -232,25 +221,6 @@ static void exit_on_signal( int sig )
     exit(1);  /* this will call the atexit functions */
 }
 
-static char* get_temp_file(const char* prefix, const char* suffix)
-{
-    int fd;
-    char *tmp;
-
-#ifdef HAVE_SIGPROCMASK
-    sigset_t old_set;
-    /* block signals while manipulating the temp files list */
-    sigprocmask( SIG_BLOCK, &signal_mask, &old_set );
-#endif
-    fd = make_temp_file( prefix, suffix, &tmp );
-    close( fd );
-    strarray_add(&tmp_files, tmp);
-#ifdef HAVE_SIGPROCMASK
-    sigprocmask( SIG_SETMASK, &old_set, NULL );
-#endif
-    return tmp;
-}
-
 static int is_pe_target( const struct options *opts )
 {
     switch (opts->target.platform)
@@ -287,7 +257,7 @@ static const struct
     { "objcopy", "llvm-objcopy" },
 };
 
-static struct strarray build_tool_name( struct options *opts, enum tool tool )
+static struct strarray build_tool_name( struct options *opts, const char *target, enum tool tool )
 {
     const char *base = tool_names[tool].base;
     const char *llvm_base = tool_names[tool].llvm_base;
@@ -296,13 +266,13 @@ static struct strarray build_tool_name( struct options *opts, enum tool tool )
     struct strarray ret = empty_strarray;
     char* str;
 
-    if (opts->target_alias && opts->version)
+    if (target && opts->version)
     {
-        str = strmake("%s-%s-%s", opts->target_alias, base, opts->version);
+        str = strmake( "%s-%s-%s", target, base, opts->version );
     }
-    else if (opts->target_alias)
+    else if (target)
     {
-        str = strmake("%s-%s", opts->target_alias, base);
+        str = strmake( "%s-%s", target, base );
     }
     else if (opts->version)
     {
@@ -325,44 +295,39 @@ static struct strarray build_tool_name( struct options *opts, enum tool tool )
     ret = strarray_fromstring( path, " " );
     if (!strncmp( llvm_base, "clang", 5 ))
     {
-        if (opts->target_alias)
+        if (target)
         {
             strarray_add( &ret, "-target" );
-            strarray_add( &ret, opts->target_alias );
+            strarray_add( &ret, target );
         }
         strarray_add( &ret, "-Wno-unused-command-line-argument" );
         strarray_add( &ret, "-fuse-ld=lld" );
+        if (opts->no_default_config) strarray_add( &ret, "--no-default-config" );
     }
     return ret;
 }
 
 static struct strarray get_translator(struct options *opts)
 {
-    enum tool tool;
-
     switch(opts->processor)
     {
     case proc_cpp:
-        tool = TOOL_CPP;
-        break;
+        return build_tool_name( opts, opts->target_alias, TOOL_CPP );
     case proc_cc:
     case proc_as:
-        tool = TOOL_CC;
-        break;
+        return build_tool_name( opts, opts->target_alias, TOOL_CC );
     case proc_cxx:
-        tool = TOOL_CXX;
-        break;
-    default:
-        assert(0);
+        return build_tool_name( opts, opts->target_alias, TOOL_CXX );
     }
-    return build_tool_name( opts, tool );
+    assert(0);
+    return empty_strarray;
 }
 
 static int try_link( struct strarray prefix, struct strarray link_tool, const char *cflags )
 {
-    const char *in = get_temp_file( "try_link", ".c" );
-    const char *out = get_temp_file( "try_link", ".out" );
-    const char *err = get_temp_file( "try_link", ".err" );
+    const char *in = make_temp_file( "try_link", ".c" );
+    const char *out = make_temp_file( "try_link", ".out" );
+    const char *err = make_temp_file( "try_link", ".err" );
     struct strarray link = empty_strarray;
     int sout = -1, serr = -1;
     int ret;
@@ -419,12 +384,11 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
 
     case PLATFORM_SOLARIS:
         {
-            char *mapfile = get_temp_file( output_name, ".map" );
+            char *mapfile = make_temp_file( output_name, ".map" );
             const char *align = opts->section_align ? opts->section_align : "0x1000";
 
             create_file( mapfile, 0644, "text = A%s;\ndata = A%s;\n", align, align );
             strarray_add( &flags, strmake("-Wl,-M,%s", mapfile) );
-            strarray_add( &tmp_files, mapfile );
         }
         break;
 
@@ -447,7 +411,10 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
         if (opts->nostartfiles || opts->use_msvcrt) strarray_add( &flags, "-nostartfiles" );
         if (opts->subsystem) strarray_add( &flags, strmake("-Wl,--subsystem,%s", opts->subsystem ));
 
+        strarray_add( &flags, "-Wl,--exclude-all-symbols" );
         strarray_add( &flags, "-Wl,--nxcompat" );
+        strarray_add( &flags, "-Wl,--dynamicbase" );
+        strarray_add( &flags, "-Wl,--disable-auto-image-base" );
 
         if (opts->image_base) strarray_add( &flags, strmake("-Wl,--image-base,%s", opts->image_base ));
 
@@ -461,6 +428,9 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
         if (opts->debug_file && strendswith(opts->debug_file, ".pdb"))
             strarray_add(&link_args, strmake("-Wl,--pdb=%s", opts->debug_file));
 
+        if (opts->build_id)
+            strarray_add( &link_args, "-Wl,--build-id");
+
         if (opts->out_implib)
             strarray_add(&link_args, strmake("-Wl,--out-implib,%s", opts->out_implib));
 
@@ -500,8 +470,13 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
         else if (!opts->strip)
             strarray_add(&link_args, "-Wl,-debug:dwarf");
 
+        if (opts->build_id)
+            strarray_add( &link_args, "-Wl,-build-id");
+
         if (opts->out_implib)
             strarray_add(&link_args, strmake("-Wl,-implib:%s", opts->out_implib));
+        else
+            strarray_add(&link_args, strmake("-Wl,-implib:%s", make_temp_file( output_name, ".lib" )));
 
         strarray_add( &link_args, strmake( "-Wl,-filealign:%s", opts->file_align ? opts->file_align : "0x1000" ));
 
@@ -513,14 +488,15 @@ static struct strarray get_link_args( struct options *opts, const char *output_n
         {
             if (!try_link( opts->prefix, link_args, strmake("-Wl,-Ttext-segment=%s", opts->image_base)) )
                 strarray_add( &flags, strmake("-Wl,-Ttext-segment=%s", opts->image_base) );
-            else
-                opts->prelink = PRELINK;
         }
         if (!try_link( opts->prefix, link_args, "-Wl,-z,max-page-size=0x1000"))
             strarray_add( &flags, "-Wl,-z,max-page-size=0x1000");
         break;
     }
 
+    if (opts->build_id)
+        strarray_add( &link_args, "-Wl,--build-id");
+
     /* generic Unix shared library flags */
 
     strarray_add( &link_args, "-shared" );
@@ -542,8 +518,9 @@ static const char *get_multiarch_dir( struct target target )
    case CPU_x86_64:  return "/x86_64-linux-gnu";
    case CPU_ARM:     return "/arm-linux-gnueabi";
    case CPU_ARM64:   return "/aarch64-linux-gnu";
+   default:
+       assert(0);
    }
-   assert(0);
    return NULL;
 }
 
@@ -644,28 +621,6 @@ static char *get_lib_dir( struct options *opts )
     return strmake( "%s%s", root, LIBDIR );
 }
 
-static void init_argv0_dir( const char *argv0 )
-{
-#ifndef _WIN32
-    char *dir = NULL;
-
-#if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__)
-    dir = realpath( "/proc/self/exe", NULL );
-#elif defined (__FreeBSD__) || defined(__DragonFly__)
-    static int pathname[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
-    size_t path_size = PATH_MAX;
-    char *path = xmalloc( path_size );
-    if (!sysctl( pathname, ARRAY_SIZE(pathname), path, &path_size, NULL, 0 ))
-        dir = realpath( path, NULL );
-    free( path );
-#endif
-    if (!dir && !(dir = realpath( argv0, NULL ))) return;
-    bindir = get_dirname( dir );
-    includedir = strmake( "%s/%s", bindir, BIN_TO_INCLUDEDIR );
-    libdir = strmake( "%s/%s", bindir, BIN_TO_LIBDIR );
-#endif
-}
-
 static void compile(struct options* opts, const char* lang)
 {
     struct strarray comp_args = get_translator(opts);
@@ -684,8 +639,8 @@ static void compile(struct options* opts, const char* lang)
 	/* mixing different C and C++ compilers isn't supported in configure anyway */
 	case proc_cc:
 	case proc_cxx:
-            gcc = build_tool_name(opts, TOOL_CC);
-            gpp = build_tool_name(opts, TOOL_CXX);
+            gcc = build_tool_name( opts, opts->target_alias, TOOL_CC );
+            gpp = build_tool_name( opts, opts->target_alias, TOOL_CXX );
             for ( j = 0; !gcc_defs && j < comp_args.count; j++ )
             {
                 const char *cc = comp_args.str[j];
@@ -752,6 +707,8 @@ static void compile(struct options* opts, const char* lang)
             strarray_add(&comp_args, "-D__cdecl=__stdcall");
             strarray_add(&comp_args, "-D__fastcall=__stdcall");
             break;
+        case CPU_ARM64EC:
+            break;
         }
         strarray_add(&comp_args, "-D_stdcall=__stdcall");
         strarray_add(&comp_args, "-D_cdecl=__cdecl");
@@ -848,7 +805,7 @@ static const char* compile_to_object(struct options* opts, const char* file, con
     /* make a copy so we don't change any of the initial stuff */
     /* a shallow copy is exactly what we want in this case */
     copts = *opts;
-    copts.output_name = get_temp_file(get_basename_noext(file), ".o");
+    copts.output_name = make_temp_file(get_basename_noext(file), ".o");
     copts.compile_only = 1;
     copts.files = empty_strarray;
     strarray_add(&copts.files, file);
@@ -857,7 +814,7 @@ static const char* compile_to_object(struct options* opts, const char* file, con
 }
 
 /* return the initial set of options needed to run winebuild */
-static struct strarray get_winebuild_args(struct options *opts)
+static struct strarray get_winebuild_args( struct options *opts, const char *target )
 {
     const char* winebuild = getenv("WINEBUILD");
     const char *binary = NULL;
@@ -878,24 +835,22 @@ static struct strarray get_winebuild_args(struct options *opts)
     strarray_add( &spec_args, binary );
     if (verbose) strarray_add( &spec_args, "-v" );
     if (keep_generated) strarray_add( &spec_args, "--save-temps" );
-    if (opts->target_alias)
+    if (target)
     {
         strarray_add( &spec_args, "--target" );
-        strarray_add( &spec_args, opts->target_alias );
+        strarray_add( &spec_args, target );
     }
     if (opts->force_pointer_size)
         strarray_add(&spec_args, strmake("-m%u", 8 * opts->force_pointer_size ));
     for (i = 0; i < opts->prefix.count; i++)
         strarray_add( &spec_args, strmake( "-B%s", opts->prefix.str[i] ));
     strarray_addall( &spec_args, opts->winebuild_args );
-    if (opts->unwind_tables) strarray_add( &spec_args, "-fasynchronous-unwind-tables" );
-    else strarray_add( &spec_args, "-fno-asynchronous-unwind-tables" );
     return spec_args;
 }
 
 static void fixup_constructors( struct options *opts, const char *file )
 {
-    struct strarray args = get_winebuild_args( opts );
+    struct strarray args = get_winebuild_args( opts, opts->target_alias );
 
     strarray_add( &args, "--fixup-ctors" );
     strarray_add( &args, file );
@@ -904,7 +859,7 @@ static void fixup_constructors( struct options *opts, const char *file )
 
 static void make_wine_builtin( struct options *opts, const char *file )
 {
-    struct strarray args = get_winebuild_args( opts );
+    struct strarray args = get_winebuild_args( opts, opts->target_alias );
 
     strarray_add( &args, "--builtin" );
     strarray_add( &args, file );
@@ -922,16 +877,19 @@ static char *find_static_lib( const char *dll )
 
 static const char *find_libgcc(struct strarray prefix, struct strarray link_tool)
 {
-    const char *out = get_temp_file( "find_libgcc", ".out" );
-    const char *err = get_temp_file( "find_libgcc", ".err" );
+    const char *out = make_temp_file( "find_libgcc", ".out" );
+    const char *err = make_temp_file( "find_libgcc", ".err" );
     struct strarray link = empty_strarray;
-    int sout = -1, serr = -1;
+    int sout = -1, serr = -1, i;
     char *libgcc, *p;
     struct stat st;
     size_t cnt;
     int ret;
 
-    strarray_addall( &link, link_tool );
+    for (i = 0; i < link_tool.count; i++)
+	if (strcmp(link_tool.str[i], "--no-default-config" ))
+            strarray_add( &link, link_tool.str[i] );
+
     strarray_add( &link, "-print-libgcc-file-name" );
 
     sout = dup( fileno(stdout) );
@@ -992,31 +950,33 @@ static void add_library( struct options *opts, struct strarray lib_dirs,
 }
 
 /* run winebuild to generate the .spec.o file */
-static const char *build_spec_obj( struct options *opts, const char *spec_file, const char *output_file,
-                                   struct strarray files, struct strarray lib_dirs, const char *entry_point )
+static void build_spec_obj( struct options *opts, const char *spec_file, const char *output_file,
+                            const char *target, struct strarray files, struct strarray resources,
+                            const char *entry_point, struct strarray *spec_objs )
 {
     unsigned int i;
     int is_pe = is_pe_target( opts );
-    struct strarray spec_args = get_winebuild_args( opts );
+    struct strarray spec_args = get_winebuild_args( opts, target );
     struct strarray tool;
     const char *spec_o_name, *output_name;
 
     /* get the filename from the path */
     output_name = get_basename( output_file );
 
-    tool = build_tool_name( opts, TOOL_CC );
+    tool = build_tool_name( opts, target, TOOL_CC );
     strarray_add( &spec_args, strmake( "--cc-cmd=%s", strarray_tostring( tool, " " )));
     if (!is_pe)
     {
-        tool = build_tool_name( opts, TOOL_LD );
+        tool = build_tool_name( opts, target, TOOL_LD );
         strarray_add( &spec_args, strmake( "--ld-cmd=%s", strarray_tostring( tool, " " )));
     }
 
-    spec_o_name = get_temp_file(output_name, ".spec.o");
+    spec_o_name = make_temp_file(output_name, ".spec.o");
     if (!is_pe)
     {
         if (opts->pic) strarray_add(&spec_args, "-fPIC");
         if (opts->use_msvcrt) strarray_add(&spec_args, "-mno-cygwin");
+        if (opts->unwind_tables) strarray_add( &spec_args, "-fasynchronous-unwind-tables" );
     }
     strarray_add(&spec_args, opts->shared ? "--dll" : "--exe");
     if (opts->fake_module)
@@ -1045,7 +1005,8 @@ static const char *build_spec_obj( struct options *opts, const char *spec_file,
         if (opts->large_address_aware) strarray_add( &spec_args, "--large-address-aware" );
     }
 
-    if (opts->target.platform == PLATFORM_WINDOWS) strarray_add(&spec_args, "--safeseh");
+    if (opts->target.platform == PLATFORM_WINDOWS && opts->target.cpu == CPU_i386)
+        strarray_add(&spec_args, "--safeseh");
 
     if (entry_point)
     {
@@ -1059,18 +1020,13 @@ static const char *build_spec_obj( struct options *opts, const char *spec_file,
         strarray_add(&spec_args, opts->subsystem);
     }
 
-    for (i = 0; i < lib_dirs.count; i++)
-	strarray_add(&spec_args, strmake("-L%s", lib_dirs.str[i]));
-
     if (!is_pe)
     {
         for (i = 0; i < opts->delayimports.count; i++)
             strarray_add(&spec_args, strmake("-d%s", opts->delayimports.str[i]));
     }
 
-    /* add resource files */
-    for (i = 0; i < files.count; i++)
-	if (files.str[i][1] == 'r') strarray_add(&spec_args, files.str[i]);
+    strarray_addall( &spec_args, resources );
 
     /* add other files */
     strarray_add(&spec_args, "--");
@@ -1087,14 +1043,14 @@ static const char *build_spec_obj( struct options *opts, const char *spec_file,
     }
 
     spawn(opts->prefix, spec_args, 0);
-    return spec_o_name;
+    strarray_add( spec_objs, spec_o_name );
 }
 
 /* run winebuild to generate a data-only library */
 static void build_data_lib( struct options *opts, const char *spec_file, const char *output_file, struct strarray files )
 {
     unsigned int i;
-    struct strarray spec_args = get_winebuild_args( opts );
+    struct strarray spec_args = get_winebuild_args( opts, opts->target_alias );
 
     strarray_add(&spec_args, opts->shared ? "--dll" : "--exe");
     strarray_add(&spec_args, "-o");
@@ -1114,16 +1070,18 @@ static void build_data_lib( struct options *opts, const char *spec_file, const c
 
 static void build(struct options* opts)
 {
+    struct strarray resources = empty_strarray;
+    struct strarray spec_objs = empty_strarray;
     struct strarray lib_dirs = empty_strarray;
     struct strarray files = empty_strarray;
     struct strarray link_args;
     char *output_file, *output_path;
-    const char *spec_o_name = NULL, *libgcc = NULL;
     const char *output_name, *spec_file, *lang;
+    const char *libgcc = NULL;
     int generate_app_loader = 1;
     const char *crt_lib = NULL, *entry_point = NULL;
     int is_pe = is_pe_target( opts );
-    unsigned int j;
+    unsigned int i, j;
 
     /* NOTE: for the files array we'll use the following convention:
      *    -axxx:  xxx is an archive (.a)
@@ -1183,7 +1141,6 @@ static void build(struct options* opts)
 	{
 	    switch(get_file_type(file))
 	    {
-		case file_def:
 		case file_spec:
 		    if (spec_file)
 			error("Only one spec file can be specified\n");
@@ -1272,7 +1229,19 @@ static void build(struct options* opts)
         build_data_lib( opts, spec_file, output_file, files );
         return;
     }
-    spec_o_name = build_spec_obj( opts, spec_file, output_file, files, lib_dirs, entry_point );
+
+    for (i = 0; i < files.count; i++)
+	if (files.str[i][1] == 'r') strarray_add( &resources, files.str[i] );
+
+    build_spec_obj( opts, spec_file, output_file, opts->target_alias, files, resources,
+                    entry_point, &spec_objs );
+    if (opts->native_arch)
+    {
+        const char *suffix = strchr( opts->target_alias, '-' );
+        if (!suffix) suffix = "";
+        build_spec_obj( opts, spec_file, output_file, strmake( "%s%s", opts->native_arch, suffix ),
+                        files, empty_strarray, entry_point, &spec_objs );
+    }
 
     if (opts->fake_module) return;  /* nothing else to do */
 
@@ -1312,7 +1281,7 @@ static void build(struct options* opts)
                                             entry_point));
     }
 
-    if (spec_o_name) strarray_add(&link_args, spec_o_name);
+    strarray_addall( &link_args, spec_objs );
 
     if (is_pe)
     {
@@ -1388,7 +1357,7 @@ static void build(struct options* opts)
 
     if (opts->debug_file && !strendswith(opts->debug_file, ".pdb"))
     {
-        struct strarray tool, objcopy = build_tool_name(opts, TOOL_OBJCOPY);
+        struct strarray tool, objcopy = build_tool_name(opts, opts->target_alias, TOOL_OBJCOPY);
 
         tool = empty_strarray;
         strarray_addall( &tool, objcopy );
@@ -1418,10 +1387,10 @@ static void build(struct options* opts)
         if (!spec_file)
             error("--out-implib requires a .spec or .def file\n");
 
-        implib_args = get_winebuild_args( opts );
-        tool = build_tool_name( opts, TOOL_CC );
+        implib_args = get_winebuild_args( opts, opts->target_alias );
+        tool = build_tool_name( opts, opts->target_alias, TOOL_CC );
         strarray_add( &implib_args, strmake( "--cc-cmd=%s", strarray_tostring( tool, " " )));
-        tool = build_tool_name( opts, TOOL_LD );
+        tool = build_tool_name( opts, opts->target_alias, TOOL_LD );
         strarray_add( &implib_args, strmake( "--ld-cmd=%s", strarray_tostring( tool, " " )));
 
         strarray_add(&implib_args, "--implib");
@@ -1433,20 +1402,6 @@ static void build(struct options* opts)
         spawn(opts->prefix, implib_args, 0);
     }
 
-    /* set the base address with prelink if linker support is not present */
-    if (opts->prelink && !opts->target_alias)
-    {
-        if (opts->prelink[0] && strcmp(opts->prelink,"false"))
-        {
-            struct strarray prelink_args = empty_strarray;
-            strarray_add(&prelink_args, opts->prelink);
-            strarray_add(&prelink_args, "--reloc-only");
-            strarray_add(&prelink_args, opts->image_base);
-            strarray_add(&prelink_args, output_path);
-            spawn(opts->prefix, prelink_args, 1);
-        }
-    }
-
     if (!is_pe) fixup_constructors( opts, output_path );
     else if (opts->wine_builtin) make_wine_builtin( opts, output_path );
 
@@ -1535,18 +1490,10 @@ int main(int argc, char **argv)
     char* lang = 0;
     char* str;
 
-#ifdef SIGHUP
-    signal( SIGHUP, exit_on_signal );
-#endif
-    signal( SIGTERM, exit_on_signal );
-    signal( SIGINT, exit_on_signal );
-#ifdef HAVE_SIGADDSET
-    sigemptyset( &signal_mask );
-    sigaddset( &signal_mask, SIGHUP );
-    sigaddset( &signal_mask, SIGTERM );
-    sigaddset( &signal_mask, SIGINT );
-#endif
-    init_argv0_dir( argv[0] );
+    init_signals( exit_on_signal );
+    bindir = get_bindir( argv[0] );
+    libdir = get_libdir( bindir );
+    includedir = get_includedir( bindir );
 
     /* setup tmp file removal at exit */
     atexit(clean_temp_files);
@@ -1700,6 +1647,8 @@ int main(int argc, char **argv)
                         opts.unwind_tables = 1;
 		    else if (!strcmp("-fno-asynchronous-unwind-tables", opts.args.str[i]))
                         opts.unwind_tables = 0;
+		    else if (!strcmp("-fms-hotpatch", opts.args.str[i]))
+                        raw_linker_arg = 1;
                     else if (!strcmp("-fPIC", opts.args.str[i]) || !strcmp("-fpic", opts.args.str[i]))
                         opts.pic = 1;
                     else if (!strcmp("-fno-PIC", opts.args.str[i]) || !strcmp("-fno-pic", opts.args.str[i]))
@@ -1771,12 +1720,15 @@ int main(int argc, char **argv)
                     else if (!strcmp("-marm", opts.args.str[i] ) || !strcmp("-mthumb", opts.args.str[i] ))
                     {
 			raw_linker_arg = 1;
-                        raw_winebuild_arg = 1;
+                    }
+                    else if (!strcmp("-marm64x", opts.args.str[i] ))
+                    {
+                        raw_linker_arg = 1;
+                        opts.native_arch = "aarch64";
                     }
                     else if (!strncmp("-mcpu=", opts.args.str[i], 6) ||
                              !strncmp("-mfpu=", opts.args.str[i], 6) ||
-                             !strncmp("-march=", opts.args.str[i], 7) ||
-                             !strncmp("-mfloat-abi=", opts.args.str[i], 12))
+                             !strncmp("-march=", opts.args.str[i], 7))
                         raw_winebuild_arg = 1;
 		    break;
                 case 'n':
@@ -1897,6 +1849,11 @@ int main(int argc, char **argv)
                                 opts.out_implib = xstrdup( Wl.str[++j] );
                                 continue;
                             }
+                            if (!strcmp( Wl.str[j], "--build-id" ))
+                            {
+                                opts.build_id = 1;
+                                continue;
+                            }
                             if (!strcmp(Wl.str[j], "-static")) linking = -1;
                             strarray_add(&opts.linker_args, strmake("-Wl,%s",Wl.str[j]));
                         }
@@ -1924,6 +1881,11 @@ int main(int argc, char **argv)
                 case '-':
                     if (strcmp("-static", opts.args.str[i]+1) == 0)
                         linking = -1;
+                    else if (!strcmp( "-no-default-config", opts.args.str[i] + 1 ))
+                    {
+                        opts.no_default_config = 1;
+                        raw_compiler_arg = raw_linker_arg = 1;
+                    }
                     else if (is_option( &opts, i, "--sysroot", &option_arg ))
                     {
                         opts.sysroot = option_arg;
